//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// April 27, 2018		Design Documents                     CSI_COLOR.c 
// 
//       Color palettes and color routines
//           for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
///
// History
//  04/27/2018  moved from color.h
//  06/07/2019  revised to use RGB_color type
//
//==============================================================================
// NAME OF ROUTINE                      Function
//
// Status written, tested, working
//   set_pal_color               Sets a current color to a palette color
//   get_pal_color               Gets the palette color of a current color
//   show_palette                Displays a palette color bar
//   show_pal_selector_frame     Displays the selector box around a palette item
//   show_pal_selector           Displays the pallette and selector box
//   Show_brightness_control     Displays slider bars for day/night brightness
//   display_color_block         Displays a color block of the current color
//   display_color_selector      Displays a color adjusment box with RGB values
//   RGB_copy                    Copies a RGB color to another RGB variable
//   save_colorset               Stores a color set to the EEPROM
//   erase_colorset              Erases color set data in EEPROM
//   load_colorset               Reads a colorset from the EEPROM
//   restore_default_colors      Loads default colors into curent color settings
//   load_palette                Reads a palette from the EEPROM
//   color_check                 Checks color settings for bad color choices
//   set_font_colors             Set foreground and background colors
//   invert_font_colors          Swaps foreground and background colors
//
// Status written, not used
//   erase_colorset              Stores 0xFF to a color set in the EEPROM
//   setup_color                 Sets a foreground and background pair of colors 
//
//==============================================================================
#include "CSI_COLOR.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "CSI_SSD1963.h"
#include "CSI_GRAPHICS.h"
#include "CSI_i2c.h"
#include "CSI_eeprom.h"
#include "CSI_ERROR_LOG.h"

RGB_color current_palette[16];

unsigned char cur_foreground;
unsigned char cur_background;

RGB_color cg_font;
RGB_color cg_background;
RGB_color highlight;
RGB_color border;
RGB_color greyout;
RGB_color shadow;

// =============================================================================
//
//   Color settings 
//
// 
struct color_data
{
  int pal_number;   // from current palette
  char *color_name; 
}color_setting[10] =
{
  { 3, "Text Font"},            //0
  {13, "Text Background"},      //1
  { 9, "Border"},               //2
  { 8, "Highlight"},            //3
  {11, "Shadow"},               //4
  { 2, "Greyout"},              //5
  {10, "TouchPoint"},           //6
  { 7, "Progress Bar"},         //7
  { 4, "Active"},               //8
  { 6, "Alert"}                 //9
};

int default_colorset[10] = { 3, 13, 9, 8, 11, 2, 10, 7, 4, 6}; 

//  short routines to keep all direct use of color_setting in the color.c module 
// =============================================================================
//
//  sets a color setting to a palette color
//
void set_pal_color( uint8_t color_num, int pal_num )
{
  color_setting[color_num].pal_number = pal_num;
}
// =============================================================================
//
//  gets the palette color of a setting
//
int get_pal_color( uint8_t color_num )
{
  return color_setting[color_num].pal_number;
}


// =============================================================================
//
//   Show palette (any palette)
//
//  to the show current palette use:
//     show_palette( startX, startY, current_palette )
//
// History 09/27/2018 written

void show_palette( uint16_t startX, uint16_t startY, RGB_color *palette)
{
  int i;
  int n;
  char colornum[3];
  
  // prevent a null pointer error
  if( palette == 0 ) 
  {
    error_log( 3003 );
    return;
  }
  
  for (i = 0; i < 16; i ++)
  {
    // print color number
    n = sprintf((char*) colornum, "%2d ", i);
    strncpy( (char*) cg_line_buffer, (char const*) colornum, 3);
    ssd1963_cg_write_line( n, 4, startX + 5  + 40 * i , startY, 2); 
    // draw a color block
    draw_rectangle( startX + 40 * i, startY + 25, 40, 40, palette[i] );
  }
}
// =============================================================================
//
//  Just the frame, to be used during color selection
//
void show_pal_selector_frame( int16_t startX, uint16_t startY,
                       RGB_color *palette, uint8_t pal_num)
{
  // prevent a null pointer error
  if( palette == 0 ) 
  {
    error_log( 3003 );
    return;
  }  
  draw_frame(startX + 40 * pal_num, startY + 25, 40, 40, 3,
             palette[color_setting[3].pal_number] );   
}

// ============================================================================= 
//
// show palette with selector box around the color contained in the table
//  
//

void show_pal_selector( uint16_t startX, uint16_t startY,
                       RGB_color *palette, uint8_t colr_num)
{
  // prevent a null pointer error
  if( palette == 0 ) 
  {
    error_log( 3003 );
    return;
  }   
  show_palette( startX, startY, palette );
  // draw selector frame
  show_pal_selector_frame(startX, startY, palette,
                          color_setting[colr_num].pal_number);
}
// =============================================================================
//
//  show slider bars for day and night brightness control
//
// default start values
uint8_t day_bright_val = 128;       
uint8_t night_bright_val = 50;

// controller accepts 0 to 255, above 200 looks washed out
// values are limited to 1 to 200

// 0 for day 1 for night

void show_brightness_control( uint8_t dayn )
{
  uint16_t x_loc = 480;
  uint16_t y_loc;
  int slider_val;
  
  y_loc = 100;
  
  // limit brightness to 200, prevent a bad slider display 
  if( day_bright_val > 200 ) day_bright_val = 200;
  if( night_bright_val > 200 ) night_bright_val = 200;
  
  if( dayn  == 0 )   // day
  {
    // reduce to 0-100 scale
    slider_val = day_bright_val >> 1;
  }
  else // night
  {
    // reduce to 0-100 scale
    slider_val = night_bright_val >> 1; 
    y_loc += 50;    
  }
  // show slider
  draw_progress_bar( slider_val, 0, x_loc, y_loc, 200, 30,
               get_pal_color( 2 ), get_pal_color( 7 ), 1);  
  
}


// =============================================================================
//
//   draw a colored rectangle using current color settings
//
void display_color_block( uint16_t startX, uint16_t startY, int num,
                         uint16_t width, uint16_t height )
{
  draw_rectangle( startX, startY, width, height,
                  current_palette[color_setting[num-1].pal_number]);   
}

// ================================================================
//
// draw color adjuster box for RGB
//

void display_color_selector( uint16_t startX, uint16_t startY, int num) 
{
//  int n;
  // draw color adjuster box for defined color items
  strncpy((char*) cg_line_buffer, (char*) color_setting[num].color_name, 16);  
  ssd1963_cg_write_line( 16, 4, startX, startY, 2);
  
  // red
  strncpy( (char*) cg_line_buffer, "red", 3);
  ssd1963_cg_write_line( 3, 4, startX, startY, 2);
  // red number
  sprintf((char*) cg_line_buffer, "%3u ",
              current_palette[color_setting[num].pal_number].red);
  ssd1963_cg_write_line( 3, 4, startX, startY, 2);
  // green
  strncpy( (char*) cg_line_buffer, "green", 5);  
  ssd1963_cg_write_line( 5, 4, startX + 30 , startY, 2); 
  // green number
  sprintf((char*) cg_line_buffer, "%3u ",
              current_palette[color_setting[num].pal_number].green); 
  ssd1963_cg_write_line( 3, 4, startX, startY, 2);
  // blue
  strncpy( (char*) cg_line_buffer, "blue", 4);  
  ssd1963_cg_write_line( 4, 4, startX + 60, startY, 2);  
  // blue number
  sprintf((char*) cg_line_buffer, "%3u ",
              current_palette[color_setting[num].pal_number].blue);
  ssd1963_cg_write_line( 3, 4, startX, startY, 2);
  
  // draw palette
  
  // display adjustment touch points (pick a number from the palette)
  
  // display the color
  display_color_block ( startX, startY, num, 30, 30);
}
// =============================================================================
//  06/07/2019
//
//   Copy RGB_color source to RGB_color destination
//
//   The parameters are pointers to a RGB_color.
//   Using the RGB color type,
//   simple assignment a_color = b_color will be a pointer assignment.
//   Use this to copy, store RGB_colors
//
//   Example:   RGB_copy( &rgb_temp, &cg_font);  // copies cg_font to rgb_temp
//
void RGB_copy( RGB_color *dest, RGB_color *source )
{
  // prevent a null pointer error
  if( dest == 0 || source == 0) 
  {
    error_log( 3003 );
    return;
  }   
  dest->red   = source->red;
  dest->green = source->green;
  dest->blue  = source->blue;
}

// =============================================================================
//
//   Save user defined color settings to EEPROM
//     current palette
//     defined colors
//
// History 10/08/2018 written
//         11/1/2019 added brightness control data

// eeprom address for color settings
// 0x2000 - 0x203B
#define COLORSET_HI  0x20
#define COLORSET_LO  0x00

void save_colorset( void )
{
  int i;
  
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout(  COLORSET_HI );  
  I2C_put_dataout(  COLORSET_LO );
  // palette
  for( i = 0; i < 16; i++)
  {
    I2C_put_dataout( current_palette[i].red );   
    I2C_put_dataout( current_palette[i].green );
    I2C_put_dataout( current_palette[i].blue );    
  }
  // color settings
  for( i = 0; i < 10; i++)
  {
    I2C_put_dataout( color_setting[i].pal_number );   
  }
  // brightness values
  I2C_put_dataout( day_bright_val );
  I2C_put_dataout( night_bright_val );

  // write to EEPROM  3+48+10+2  63 bytes
  EEPROM_MultipleWrite( 0, 63);
  
  EEPROM_waitfor_I2C();
  // while(!I2C_2_txrx_complete);    // debug: hangs here  
  // EEPROM_busypoll(); 
   
}
// =============================================================================
//
//  Fill color table 
//   for debugging to fill color data with FF to simulate a blank chip

void erase_colorset( void )
{
  EEPROM_Mem_fill( 0, COLORSET_HI, COLORSET_LO, 128, 0xFF );
}

// =============================================================================
//
//   Load user color settings from eeprom
//     current palette
//     defined colors
//
// History 10/08/2018 written
//         10/24/2018 debug, removed double load of address to tx buffer
//         11/1/2019 added brightness control data

void load_colorset( void )
{
  int i;
  
  // read colorset
  EEPROM_MemRead( 0, COLORSET_HI,COLORSET_LO, 60);
  // wait for I2C to finish
  EEPROM_waitfor_I2C();  
  
  // store palette
  for( i = 0; i < 16; i++)
  {
    current_palette[i].red   = I2C_get_datain();   
    current_palette[i].green = I2C_get_datain();
    current_palette[i].blue  = I2C_get_datain();
  }
  // store color settings
  for( i = 0; i < 10; i++)
  {
    color_setting[i].pal_number  = I2C_get_datain();  
  }
  day_bright_val = I2C_get_datain();
  night_bright_val = I2C_get_datain();
  // store defaults if FF is read from Eprom 
  if( day_bright_val == 0xFF || night_bright_val == 0xFF )
  {
    // set defaults
    day_bright_val = 128;       
    night_bright_val = 50;
    // store back to eeprom
    save_colorset();
  }
}

// =============================================================================
// 
//   restore default colorset
//

void restore_default_colors( void )
{
  int i;
  for( i = 0; i < 10; i++)
  {
    color_setting[i].pal_number = default_colorset[i];  
  }
  day_bright_val = 128;       
  night_bright_val = 50;  
}


// =============================================================================
//
//  Load colors to current palette
//
void load_palette( RGB_color *palette )
{
  int i;
  
  RGB_color *palset;
  palset = palette;
  // prevent a null pointer error
  if( palette == 0 ) 
  {
    error_log( 3003 );
    palset = default_palette;
  }   
  for(i= 0; i < 16; i++)
  {
    current_palette[i].red   = palset[i].red;
    current_palette[i].green = palset[i].green;
    current_palette[i].blue  = palset[i].blue;
  }
}
// =============================================================================
//
//  color selection check
//   checks color selection to prevent text and background
//   from having the same color making the text invisible.
//    
//   passed *palettte is a pointer to the palette being checked 
//
uint8_t color_check( RGB_color *palette )
{
  int i;
  // prevent a null pointer error
  if( palette == 0 ) 
  {
    error_log( 3003 );
    return 0;
  }    
  // test color selection
  // if color number for text = color number for background
  if(color_setting[0].pal_number == color_setting[1].pal_number) return 0;
  
  // test the palette
  // if RGB for chosen text color = RGB for chosen background color  
  if(palette[color_setting[0].pal_number].red   == palette[color_setting[1].pal_number].red   &&
     palette[color_setting[0].pal_number].green == palette[color_setting[1].pal_number].green &&
     palette[color_setting[0].pal_number].blue  == palette[color_setting[1].pal_number].blue) return 0;
  
  // check for all 0xFF or all 0
  int all_FF = 1;
  int all_zero = 1;
  
  for( i = 0; i < 10; i++ )
  {
  if(palette[color_setting[i].pal_number].red   != 0xFF  ||
     palette[color_setting[i].pal_number].green != 0xFF  ||
     palette[color_setting[i].pal_number].blue  != 0xFF ) all_FF = 0;
  if(palette[color_setting[i].pal_number].red   != 0  ||
     palette[color_setting[i].pal_number].green != 0  ||
     palette[color_setting[i].pal_number].blue  != 0 ) all_zero = 0;
  }
  if( all_FF == 1 || all_zero == 1) return 0;     
     
  return 1;  // color choices are OK
}

// =============================================================================
//
//  Set foreground color and background to a current palette color number
//   foreground color, and background colors are used by the character generator
//
void set_font_colors( unsigned char fore_color, unsigned char back_color )
{
  // set foreground
  RGB_copy( &cg_font, &current_palette[fore_color]);
  // set background
  RGB_copy( &cg_background, &current_palette[back_color]);
  
  cur_foreground = fore_color;
  cur_background = back_color;  
}
// =============================================================================
//
//  Set foreground color and background to a current palette color number
//   foreground color, and background colors are used by the character generator
//
void invert_font_colors( void )
{
  RGB_color rgb_temp;
  unsigned int b_temp;
  
  // store temp
  RGB_copy( &rgb_temp, &cg_font);
  // set foreground
  RGB_copy( &cg_font, &cg_background);
  // set background
  RGB_copy( &cg_background, &rgb_temp );
  
  // set palette colors
  b_temp = cur_background;
  
  cur_background = cur_foreground;
  cur_foreground = b_temp;
}



//
// *****************************************************************************
//
// March 28, 2018		Design Documents                     setup_color
//
// The display programs require both a foreground color and a background color
// whenever something is being written to the display. If the bit of the 
// character is a 1 then it uses a foreground color and if the bit is 0 it uses
// a background color. Because the colors are represented by 24 bits, the number
// of combinations are very large. This program allows the programmer to define
// up to 16 different colors.
//
//  This is unused now. It was written without any colors defined.
//  It might be used to define color pairs that look good for text on the sceen.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  foreground colors are setup
//  bacground colors are setup
//
// External functions called
//  
//
// External variables used
//  cg_red is the red foreground byte
//  cg_green is the green foreground byte
//  cg_blue is the blue foreground byte
//
//  cg_background_red is the red background byte
//  cg_background_green is the green background byte
//  cg_background_blue is the blue background byte
//
// Internal variables used
//
// History
//    08/01/2018 moved from Process_packets, added color data
//
// Status
//  written, adding color data, not being used
//
uint8_t color_table[16][6] = {
  //  foreground         background
  //Red  Green  Blue   Red  Green  Blue
    255, 255, 255,   0,   0,   0,   // Color 0  white on black
      0,   0,   0, 255, 255, 255,   // Color 1  black on white
      0, 200,   0,   0,   0,   0,   // Color 2  green on black
      0,   0,   0, 170, 190, 220,   // Color 3  black on light blue
    255, 192,   0,   0,   0,   0,   // Color 4  orange on black
      0,   0, 170, 255, 255, 255,   // Color 5  dark blue on white
    255, 255, 255,  48,  84, 150,   // Color 6  white on dark blue
      0,   0,   0, 241, 236, 203,   // Color 7  black on tan
    0, 0, 0,  0, 0, 0,   // Color 8
    0, 0, 0,  0, 0, 0,   // Color 9
    0, 0, 0,  0, 0, 0,   // Color 10
    0, 0, 0,  0, 0, 0,   // Color 11
    0, 0, 0,  0, 0, 0,   // Color 12
    0, 0, 0,  0, 0, 0,   // Color 13
    0, 0, 0,  0, 0, 0,   // Color 14
    0, 0, 0,  0, 0, 0};  // Color 15

void setup_color(unsigned int color)
{
  RGB_copy( &cg_font, (RGB_color *) &color_table[color][0] );
  RGB_copy( &cg_background, (RGB_color *) &color_table[color][3] );
}


// VGA standard palette, HTML standard 4.01 
RGB_color standard_palette[16] = {
//red green blue  
  {255, 255, 255},   // White
  {192, 192, 192},   // Silver
  {128, 128, 128},   // Gray
  {  0,   0,   0},   // Black
  {255,	  0,   0},   // Red
  {128,	  0,   0},   // Maroon
  {255, 255,   0},   // Yellow
  {128, 128,   0},   // Olive
  {  0, 255,   0},   // Lime
  {  0, 128,   0},   // Green
  {  0, 255, 255},   // Acqua
  {  0, 128, 128},   // Teal
  {  0,   0, 255},   // Blue
  {  0,	  0, 128},   // Navy
  {255,	  0, 255},   // Fuchsia
  {128,	  0, 128}    // Purple
};
// Rainbow palette
RGB_color rainbow_palette[16] = {
//red green blue  
  {255, 255, 255},   // White
  {128, 128, 128},   // Gray
  {  0,   0,   0},   // Black
  {128,   0,   0},   // Maroon 
  {255,   0,   0},   // Red 
  {237, 125,  49},   // Orange
  {255, 192,   0},   // Bright Orange
  {255, 255,   0},   // Yellow
  {  0, 255,   0},   // Lime
  {  0, 192,   0},   // Bright Green
  { 51, 153,  51},   // Sea Green
  {  0, 153, 204},   // French Blue
  {  0,   0, 255},   // Blue
  { 90,   0, 207},   // Light Purple
  {128,   0, 128},   // Purple
  { 88,   0,  88}      // Violet
};

// Greyscale palette
RGB_color greyscale_palette[16] = {
//red green blue    
  {255, 255, 255},   // White
  {238, 238, 238},
  {221, 221, 221},
  {204, 204, 204},
  {187, 187, 187},
  {170, 170, 170},
  {153, 153, 153},
  {136, 136, 136},
  {119, 119, 119},
  {102, 102, 102},
  {85,  85,  85},
  {68,  68,  68},
  {51,  51,  51},
  {34,  34,  34},
  {17,  17,  17},
  {0,	0,   0}  // Black
};

// Default palette, more useful and less garish than VGA
// using RGB_color type

RGB_color default_palette[16] = {
//red green blue  
  {255, 255, 255},   // 0 White
  {192, 192, 192},   // 1 Silver - inside of progress bar
  {160, 160, 160},   // 2 Gray  - greyout color
  {  0,   0,   0},   // 3 Black - text
  {255,	  0,   0},   // 4 Red   - active
  {255, 192,   0},   // 5 Bright Orange
  {255, 255,   0},   // 6 Yellow - alert
  {  0, 200,   0},   // 7 Spring Green //was   51, 153,  51,   // 7 Sea Green, prog bar
  {  0, 255, 255},   // 8 Acqua  - highlight
  { 30, 130, 200},   // 9 Blue  // was 0,0,255, is now border color
 //128,   0, 128,   // 10 Purple // 15, 70, 120 tried Dark Blue 
  {160,  40, 160},   // 10 Light Purple - touch points
  { 15,  70, 120},   // 11 Dark blue, shadow color  
  {212, 188, 122},   // 12 Tan
  {170, 190, 220},   // 13 light Blue  - background color
  {198, 224, 180},   // 14 Pale Green
  {255, 153, 204}    // 15 Pink
};  

// Designer palette, fancy!
RGB_color designer_palette[16] = {
//red green blue    
  {255, 212,  75},   // Gold
  {255, 246, 202},   // Champagne
  {250, 163,  46},   // Tangerine
  {255, 132, 128},   // Salmon
  {175,   5,   5},   // Rust
  {189, 225, 192},   // Sea Foam
  {  0, 200,   0},   // Spring Green
  {  0,  96,   0},   // Hunter Green
  {137, 217, 255},   // Sky Blue
  {  0, 153, 204},   // French Blue
  { 48,  84, 150},   // Indigo
  {100,  60,  30},   // Chocolate
  {231, 212, 175},   // Beige
  {130,	0,  74},   // Wine
  {158,	0, 158},   // Plum
  {201, 155, 175}    // Mauve
};  