//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// November 1, 2019         Design Documents               CSI_CS800_TGMonitor.c
//
//           routines for the talk group monitor (TGM) advanced feature
//                     large panel display project
//
// Copyright
//  Connect Systems Inc.
//
// Programmer
//  Dean S. Graham
//  
// History
//     11/01/2019 started
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written,
//   TGM_clear_record                   Clear a single record
//   TGM_add_record                     Add a record
//   TGM_update_record                  Update record
//   TGM_get_record                     Returns table index for a talkgroup
//   TGM_clear_table                    Clears all the data in the database
//   TGM_new_call                       Starts processing a new call
//   TGM_end_call                       Ends processing a new call
//   TGM_load_test_data                 Loads test data for debugging
//   TGM_display_new_graph              Display data in Graphical form
//   TGM_copy                           Copy TGM records
//   TGM_swap                           Swap TGM records
//   TGM_sort                           Sort TGM records
//   TGM_display_text                   Display data in textual form
//   TGM_display_data                   Displays graphical or textual data
//   process_EMon_mode                  Processed enhanced monitor screen   
// =============================================================================
//
//    DMR talkgroup monitoring
//
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "CSI_CS800_TGMonitor.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_SSD1963.h"
#include "CSI_GRAPHICS.h"
#include "CSI_Menus.h"
#include "CSI_SWITCHES.h"
#include "CSI_TouchPanel.h"

//  database for info
//  100 records
TGM_record TGM_table[100];

// variables
uint32_t volatile TGM_call_seconds = 0;  // current call seconds counter
char TGM_group_id[ 17 ];        // current call group name string

uint8_t TGM_monitor_on = 0;     // flag set when montior feature is runnig
uint8_t TGM_call_active = 0;    // flag set while receiving a call
uint8_t TGM_current_rec = 0;    // record number of current call
uint8_t TGM_display_type = 0;   // display graph 0 or text 1
uint8_t TGM_update = 0;         // flag for repaint

// =============================================================================
//
//   clear a single record
//

void TGM_clear_record( uint8_t rec_num )
{
  TGM_table[rec_num].record_ID = 0;
  strncpy( TGM_table[rec_num].talkgroup, "", 17);
  TGM_table[rec_num].seconds = 0;
  TGM_table[rec_num].contacts = 0;
}
// =============================================================================
//
//   add a record
//    the new table record number is obtained with get_record and passed here
//    the current data is stored in the new record location
//

uint32_t ID_hash = 0;

void TGM_add_record( uint8_t rec_num )
{
  int i;
  ID_hash = 0;
  uint8_t  ID_seq = 0;
  // new table record number is supplied
  
  // make sub get_unique ID?
  
  // make hash record ID using 3 bytes
  ID_hash =  TGM_group_id[ 0 ] << 24;
  ID_hash += TGM_group_id[ 1 ] << 16;
  ID_hash += TGM_group_id[ 2 ] <<  8; 
  if( ID_hash == 0 ) return; 
  
  // check to see if this new hash exists
  // scan through all records, if hash high 3 bytes match,
  // increment sequential id
  for( i = 0 ; i < 100 ; i++ )
  {
    if( TGM_table[ i ].record_ID & 0xFFFFFF00 == ID_hash )
    {
      ID_seq++;
    }
  }
  // add sequential Id to hash, store
  TGM_table[rec_num].record_ID = ID_hash + ID_seq;
  // copy string
  strncpy( TGM_table[rec_num].talkgroup, (char const * ) TGM_group_id, 17);
  // store this call seconds
  TGM_table[rec_num].seconds = TGM_call_seconds;                                                     
  // first contact,  1
  TGM_table[rec_num].contacts = 1;                                                    
}
// =============================================================================
//
//  Update record
//    get the record number with TGM_get_record
//    run at the end of a call

void TGM_update_record( uint8_t rec_num )
{
    TGM_table[rec_num].seconds += TGM_call_seconds;
    TGM_table[rec_num].contacts++;
}
// =============================================================================
//
//  returns table index for a talkgroup
//   or the next available for no match
//   if the table is full it returns the index to the smallest seconds
//
//   could be split into get existing and get new, does both now


uint8_t TGM_get_record( char* talkgroup )
{
  int i;
  uint32_t ID_hash = 0;
  uint8_t rec_num = 0;
  uint8_t found = 0;   // rec_num 0 is a valid record, need another flag
  
  // make hash for the talkgroup, 3 bytes
    // make hash record ID using 3 bytes
  ID_hash =  TGM_group_id[ 0 ] << 24;
  ID_hash += TGM_group_id[ 1 ] << 16;
  ID_hash += TGM_group_id[ 2 ] <<  8; 
  
  // scan through table to find the talkgroup
  // if found set the record number
  i = 0;
  while( found == 0  && i < 100 )
  {
    if( ( TGM_table[ i ].record_ID & 0xFFFFFF00 ) == ID_hash )
    {
      // hash matched, check the whole string
      if ( strncmp( TGM_table[ i ].talkgroup, talkgroup, 17 ) == 0 )
      {
        rec_num = i;  // the string also matched
        found = 1;
      }
    }
    i++;
  }
  // if not found get next empty record, set the record number
  i = 0;
  while( found == 0  && i < 100 )
  {
    if( TGM_table[ i ].record_ID == 0 )
    {
      rec_num = i;
      found = 1;
      // store current data into the empty location
      TGM_add_record( rec_num );      
    }
    i++;
  }
  // !!! maybe could sort and use the last rec
  
  // if table is full return index to the smallest second count
  uint32_t min_secs = 0xFFFFFFFF;
  if( found == 0 )
  {
    for( i = 0 ; i < 100 ; i++ )
    {   
      if( TGM_table[ i ].seconds < min_secs )
      {
        min_secs = TGM_table[ i ].seconds;
        rec_num = i;
      }
    }
    // store current data into the location with smallest time
    TGM_add_record( rec_num );
  }  
  return rec_num;
}


// =============================================================================
//
//   Clear all the data in the table
//
//  clear the database  - to be done on channel change, and at the start


void TGM_clear_table( void )
{
  int i;
  for(i = 0; i < 100; i++ )
  {
    TGM_clear_record( i );
  }
  TGM_call_active = 0;
  TGM_current_rec = 0;
}
// =============================================================================
//
//   new call
//
void TGM_new_call( void )
{
  // only log DMR calls
  if( channel_type == 0 )
  {
    strncpy( TGM_group_id, (char * ) cs_rx_group_id_alias, 17);
    if( TGM_group_id[ 0 ] == 0 ) return; 
    //  get record number, returns an existing talkgroup, or a new one
    TGM_current_rec = TGM_get_record( (char * ) TGM_group_id );
    // set flag to count
    TGM_call_active = 1;
    // restart seconds counter
    TGM_call_seconds = 0;
  }
}
// =============================================================================
//
//  end call
//

void TGM_end_call( void )
{
  // only log DMR calls
  if( channel_type == 0 )
  {
    TGM_call_active = 0;
    // don't count one second bursts as calls
    if( TGM_call_seconds > 1 ) 
    {  
      // update record with elapsed seconds
      TGM_update_record( TGM_current_rec );
      // display the graph or page
      TGM_update = 1;   
      // TGM_display_graph();    
    }
  }
}

// =============================================================================
//
//  load tests data for debugging
// 
uint8_t TGM_dataload = 0;

void TGM_load_test_data( void )
{
  uint8_t rec_num = 0;
 
  if( TGM_dataload == 0 )
  {
    strncpy( TGM_group_id, (char const * ) "CA 3106", 17);
    rec_num = TGM_get_record( (char * ) TGM_group_id );
    // first rec
    TGM_table[ rec_num ].seconds = 3915;  //  1:05:15
    strncpy( TGM_table[ rec_num ].talkgroup, "CA 3106", 7);
    
    strncpy( TGM_group_id, (char const * ) "SoCal", 17 );
    rec_num = TGM_get_record( (char * ) TGM_group_id );    
    // second rec
    TGM_table[ rec_num ].seconds = 1814;  //  0:30:14
    strncpy( TGM_table[ rec_num ].talkgroup, "SoCal", 7);
    
    strncpy( TGM_group_id, (char const * ) "WORLD", 17 );
    rec_num = TGM_get_record( (char * ) TGM_group_id );    
    // second rec
    TGM_table[ rec_num ].seconds = 1014;  //  0:16:54
    strncpy( TGM_table[ rec_num ].talkgroup, "WORLD", 7);
    
    TGM_dataload = 1;
  }
  
}


// =============================================================================
//
//  display graph
//
//  24 32 wide
// 

void TGM_display_graph( void )
{
  int i;
  int j;
  uint16_t x_pos = 10;
  uint16_t y_pos = 0;
  uint32_t max_seconds = 0;
  uint16_t graph_height = 400;
  uint16_t graph_scale = 100;
  uint16_t bar_top = 0;
  uint16_t bar_size = 0;
  uint8_t num_chars = 0;
  uint16_t name_height = 0;
  uint16_t text_top = 0;
  
  // for debugging
  //TGM_load_test_data( );
  
  // font colors 
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
  // set shadow color
  RGB_copy( &highlight, &current_palette[get_pal_color( 4 )]); 
  // set border color
  RGB_copy( &border, &current_palette[get_pal_color( 2 )]);  

  // clear
  ssd1963_cg_clear_display();
  
  //active_screen_num = 10;
  // draw frame
  // border shaded frame
  draw_shaded_frame( 0, 0, 800, 410, 5, border, highlight);
  // sort data
  TGM_sort();
  max_seconds = TGM_table[ 0 ].seconds;
  
  while( max_seconds > graph_scale )
  {
    // next factor of 10
    graph_scale *= 10;
  }

  // draw bars
  for( i = 0; i < 19; i++)
  {
    // table is sorted so no need to draw any past a 0 bar
    if( TGM_table[ i ].seconds == 0 ) break;
    {
      // size of bar
      bar_size = graph_height * TGM_table[ i ].seconds / graph_scale;
      // calc top of bar
      bar_top = y_pos + 400 - bar_size;
      // display red
      draw_rectangle( x_pos + 21 + i * 40, bar_top, 18, bar_size,
                     current_palette[get_pal_color( 8 )] ); 

      // write talkgroup name
      // get top of text
      num_chars = strlen( TGM_table[ i ].talkgroup );
      name_height =  num_chars * 21;
      text_top = y_pos + 400 - name_height - 8;
      for( j = 0; j < num_chars; j++ )
      {
        ssd1963_cg_output_character( TGM_table[ i ].talkgroup[ j ], 11,
                           x_pos + i * 40, text_top + 21 * j, 0 );
      }
    }
  }
  // display clear button
  // draw button 80 wide 40 high
  draw_shaded_frame( 40, 420, 80, 40, 4, highlight, border );
  // write text
  strncpy( (char*) cg_line_buffer, "Clear", 5);
  ssd1963_cg_write_line( 5, 4, 55, 430, 2);
  // activate touch point  
  CS800D_touch_point[65].is_active = 1; // clear btn
  // use icons for buttons
  active_button_mode = 0;      
  draw_navbuttons( 1 );
    
}
    

// =============================================================================
//
//  copy TGM records
//
void TGM_copy( TGM_record * dest, TGM_record * src )
{
  dest->record_ID = src->record_ID;
  strncpy( dest->talkgroup, src->talkgroup, 17 );
  dest->seconds = src->seconds;
  dest->contacts = src->contacts; 
}

// =============================================================================
//
//  swap records
//
void TGM_swap( uint8_t rec_a, uint8_t rec_b )
{
  TGM_record temp_rec;
  temp_rec.record_ID = 0;  // prevents complier warning

  // copy record a to temp
  TGM_copy( &temp_rec, &TGM_table[ rec_a ] );
  // copy record b to a  
  TGM_copy( &TGM_table[ rec_a ], &TGM_table[ rec_b ] );  
  // copy record temp to b
  TGM_copy( &TGM_table[ rec_b ], &temp_rec );   
}

// =============================================================================
//
//  sort database, based on the number of seconds
//
//  bubble sort, just one temporary record is needed in swap

void TGM_sort( void )
{
  int i = 0;
  
  while( i < 99 )
  {
     if( TGM_table[ i ].seconds < TGM_table[ i + 1 ].seconds)
     {
       // swap records
       TGM_swap( i, i + 1 );
       if( i > 0 ) i--;
     }
     else i++;
  }
}
// =============================================================================
//
//  display TGM text 
//
//  two column, 16 lines
//
void TGM_display_text( uint8_t page_num )
{
  int n;
  int i = 0;
  int j;
  uint8_t page_start = 0;
  uint16_t row = 36;
  uint16_t column = 10;
  int hours = 0;
  uint8_t minutes = 0;
  
  // font colors 
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
  // set shadow color
  RGB_copy( &highlight, &current_palette[get_pal_color( 4 )]); 
  // set border color
  RGB_copy( &border, &current_palette[get_pal_color( 2 )]);  

  // clear
  ssd1963_cg_clear_display();
  CS800D_touch_point[65].is_active = 0; // turn off clear btn
  
  // draw frame
  // border shaded frame
  draw_shaded_frame( 0, 0, 800, 410, 5, border, highlight);
  // sort data
  TGM_sort();  
  
  // display text titles
  // talkgroup, time, contacts
   strncpy( (char*) cg_line_buffer, "Talkgroup             Time    Contacts", 38);
   ssd1963_cg_write_line( 38, 4, 10, 10, 2);
   // repeat for next column
   ssd1963_cg_write_line( 38, 4, 410, 10, 2);
  
  // calc page start
  page_start = page_num * 32;
  
  for( i = 0; i < 32; i++ )
  {
    j = i + page_start;
    if( TGM_table[ j ].seconds != 0 )
    {
      minutes = (TGM_table[ j ].seconds / 60 ) % 60;
      hours = TGM_table[ j ].seconds / 3600;
      
      strncpy( (char*) cg_line_buffer, TGM_table[ j ].talkgroup, 16);
      ssd1963_cg_write_line( 16, 4, column , row, 2);
      // time
      n = sprintf((char*) cg_line_buffer, "%2i:%02i:%02i", hours, minutes, TGM_table[ j ].seconds % 60);
      ssd1963_cg_write_line(n, 4, column + 200 , row, 2);
      // contacts
      n = sprintf((char*) cg_line_buffer, "%i", TGM_table[ j ].contacts);
      ssd1963_cg_write_line(n, 4, column + 340 , row, 2);      
      
      row += 26;
      if( i == 15 )
      {
        row = 36;
        column = 410;
      }
    }
  }
  // use icons for buttons
  active_button_mode = 0;      
  draw_navbuttons( 1 );
}
// =============================================================================
//
//  display TGM_info
//     displays graph or text page number
//    
void TGM_display_data( uint8_t page_num )
{
  // display the chosen page
  switch( page_num )
  {
  case 0:
    {
      // display the graph
      TGM_display_graph();              
      break;
    }
  case 1:
    {
      // display the page
      TGM_display_text( 0 );
      break;
    }
  case 2:
    {
      // display the page
      TGM_display_text( 1 );
      break;
    }            
  case 3:
    {
      // display the page
      TGM_display_text( 2 );              
      break;
    }                        
  default: break;
  }
  
}

// =============================================================================
//
//  Enhanced monitor screen
//
// History: 11/07/2019
//
// Status:  writing

void process_EMon_mode( uint8_t nav_key )
{
  static uint8_t e_mon_state = 0;
  static uint8_t enh_mon_page = 0; 

  switch( e_mon_state )
  {
  case 0:   // display graph
    {
      all_touches_off();
      // turn on only the ones for this screen
      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4         
      TGM_display_graph();
      TGM_monitor_on = 1;
      e_mon_state++;
      push_menu_stack( ENHMON );      
      break;
    }
  case 1: // handle enhanced monitor mode
    {
      // navigate
      nav_key = get_nav_key();
      last_key_press = 0;      
      switch( nav_key )
      {
      case UP:  //left
        {
          // change active page
          if ( enh_mon_page == 0 )enh_mon_page = 3;
          else enh_mon_page--;
          // display the new page
          TGM_display_data( enh_mon_page );          
          // stay in this state
          break;
        }
      case DOWN:
        {
          // change active page
          if ( enh_mon_page == 3 )enh_mon_page = 0;
          else enh_mon_page++;
          // display the new page
          TGM_display_data( enh_mon_page );          
          // stay in this state
          break;
        }
      case SELECT:
        {
          // nothing for now
          
          // debug, force redraqw
          TGM_update = 1;
          
          if( TGM_update )
          {
            TGM_update = 0;
            TGM_display_data( enh_mon_page );
          }          
          break;
        }
      case CANCEL:
        {
          e_mon_state = 0;  // set for restart
          // pop_menu_stack();
          CS800D_touch_point[65].is_active = 0; // clear btn          
          menu_exit = 1;   // force return to main screen          
          break;
        }
      default: 
        {
          // no nav key, redisplay current page if there is an update
          if( TGM_update )
          {
            TGM_update = 0;
            TGM_display_data( enh_mon_page );
          }
          break;
        }
      }
      break;
    }
    default: break;
  }
}
