//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// May 29, 2018             Design Documents                      CSI_ExtFlash.c
//
//            Routines for Winbond W25Q64 serial flash memory
//                     For large panel project
//
//
// Mfr ID  0xEF16
// JDECID  0xEF4017
//
// History
//  05/29/2018 moved from CSI_spi.c
//
// Status
//  writing
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  flash_reset                      software command reset of flash chip
//  flash_mfrid                      reads mfr id of flash chip
//  flash_jdecid                     reads jdec id of flash chip
//  flash_uniqueid                   reads 64 bit unique id of flash chip
//  flash_read_stat1                 reads status register 1 of the flash chip
//  flash_read_stat2                 reads status register 2 of the flash chip
//  flash_busy_wait                  wait for flash chip while it is busy
//  flash_read                       read data from the flash chip
//  flash_command                    send a command to the flash chip
//  flash_write                      write data to the flash chip
//  flash_erase                      erase various sized sections of data
//
// Status: empty routine
// initialize_flash_memory           Does nothing

//==============================================================================
#include <stdint.h>
#include <assert.h>
#include "CSI_spi.h"
#include "CSI_ExtFlash.h"
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_interrupt.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_TIMERS.h"

extern uint32_t *RCC_APB2ENR; 

// =============================================================================
//
//   Initialize flash memory
//
//  Status: unwritten 

void initialize_flash_memory(void)
{
}


// =============================================================================
//
//  Flash reset
//
// History
//    05/24/2018
//    06/05/2018 rewritten 
// Status written, tested, working


void flash_reset(void)
{
  flash_command( ENABLE_RESET ); 
  flash_command( RESET );
}

// =============================================================================
//                                                                    
// read MFR ID / DEVICE ID
//
//   Using TX buffer, RX buffer and interrupt ISR routine
//
// 05/22/2018 transferred from cs7000 QSPI revised for ST32F405
// 
//
// History
//    05/24/2018 
// Status written, tested, working

int flash_mfrid(void)
{
  int chip_id;
  uint8_t hi,lo;
    
  // load tx buffer interrupt routine will send 
  SPI1_tx_buffer_add( MFR_DEVICE_ID );
  // set number of bytes to read
  spi1_num_bytes_to_rx = 6;
  spi1_num_bytes_to_tx = 1;
  
  spi_repeat_tx = 1;  // will send 5 0's
  
  spi1_write_bytes();
  
  // wait till done, read data from buffer
  while(spi1_num_bytes_to_rx > 0);
  
  rx_bufr_tail += 4; // skip over command data
  
  // read
  hi = spi_receive_buffer[rx_bufr_tail++];
  lo = spi_receive_buffer[rx_bufr_tail++];
  chip_id = (hi << 8) + lo;
  
  msec_delay(5);
  return chip_id;
}
// =============================================================================
//                                                                    
// read JDEC ID 
//
//   Using TX buffer, RX buffer and interrupt ISR routine
//
//  JDECID  for flash chip should be 0xEF4017
//
// History
//    05/31/2018 
// Status written, tested, working

uint32_t flash_jdecid(void)
{
  uint32_t jdecid;
  
  // load tx buffer, interrupt routine will send 
  SPI1_tx_buffer_add( JEDEC_ID );
  // set number of bytes 
  spi1_num_bytes_to_tx = 1;
  spi1_num_bytes_to_rx = 4;
  
  spi_repeat_tx = 1;
  
  spi1_write_bytes();  
  
  // wait till done, read data
  while(SPI1_SR->bsy);
  // skip the command
  rx_bufr_tail++;
  
  jdecid = spi_receive_buffer[rx_bufr_tail++];
  jdecid <<= 8;
  jdecid |= spi_receive_buffer[rx_bufr_tail++];
  jdecid <<= 8;
  jdecid |= spi_receive_buffer[rx_bufr_tail++];
 
  return jdecid;
}
// =============================================================================
//                                                                    
// Read 64 bit unique chip ID
//
//   Using TX buffer, RX buffer and interrupt ISR routine
//
//  for debugging prototype  D165 8824 6760 3633
//                proto 2    1994 F08D 9D3D 8BEA
//                           14F0 8D9D 3D8B EA38
//
// History
//    06/04/2018 
// Status written, tested, working

uint64_t flash_uniqueid(void)
{
  uint64_t unique_id = 0;
  int i;
  
  // load tx buffer, interrupt routine will send 
  SPI1_tx_buffer_add( READ_UNIQUE_ID );

  // transmit is 1 data and 12 dummies
  // receive is 5 skipped and 8 data read
  
  // set number of bytes
  spi1_num_bytes_to_tx = 1;
  spi1_num_bytes_to_rx = 13;
  spi_repeat_tx = 1;   
  
  spi1_write_bytes();
  
  // wait till done, read data from buffer
  while(spi1_num_bytes_to_rx > 0);
  rx_bufr_tail += 5; // skip over command data
  // read
  for ( i = 0; i < 8; i++)
  {
    unique_id <<= 8;
    unique_id |= spi_receive_buffer[rx_bufr_tail++];
  }
  
  return unique_id;
}
// =============================================================================
//                                                                    
// Read Flash Status register 1
//
//   Using TX buffer, RX buffer and interrupt ISR routine
//    This is a one time read, CSN will be brought up at the end
//
//
// History
//    06/04/2018 
// Status written, tested, working 

uint8_t flash_read_stat1(void)
{
  uint8_t status1 = 0;
  // be sure rx buffer is up to date
  rx_bufr_tail = rx_bufr_head;
  
  // load tx buffer
  SPI1_tx_buffer_add( READ_STATUS_REG1 );
  SPI1_tx_buffer_add( 0 );  // for read
  
  // set number of bytes
  spi1_num_bytes_to_rx = 2;
  spi1_num_bytes_to_tx = 2;
  spi1_write_bytes();

  while(spi1_num_bytes_to_rx > 0);  
  // wait till done, read data from buffer
  while(SPI1_SR->bsy);
  rx_bufr_tail++; // skip over command
  // read
  status1 = spi_receive_buffer[rx_bufr_tail++];

  return status1;  
}
// =============================================================================
//                                                                    
// Read Flash Status register 2
//
//   Using TX buffer, RX buffer and interrupt ISR routine
//
//
// History
//    06/04/2018 
// Status written, tested, working 

uint8_t flash_read_stat2(void)
{
  uint8_t status2 = 0;
  // be sure rx buffer is up to date
  rx_bufr_tail = rx_bufr_head;  
  // load tx buffer
  SPI1_tx_buffer_add( READ_STATUS_REG2 );
  SPI1_tx_buffer_add( 0 );  // for read
  
  // set number of bytes
  spi1_num_bytes_to_rx = 2;
  spi1_num_bytes_to_tx = 2;
  spi1_write_bytes();
  
  while(spi1_num_bytes_to_rx > 0);    
  // wait till done, read data from buffer
  while(SPI1_SR->bsy);
  rx_bufr_tail++; // skip over command
  // read
  status2 = spi_receive_buffer[rx_bufr_tail++];

  return status2;  
}
// =============================================================================
//                                                                    
// Wait for Flash not busy
//
//
//   This is the Flash Chip busy wait, different than the SPI busy
//   status can be continuously read, and CSN left low
//
// History
//    06/04/2018 
//    06/26/2018 working 
//
uint8_t status1 = 0;

void flash_busy_wait(void)
{
  uint8_t status1 = 0;
  // be sure rx buffer is up to date
  rx_bufr_tail = rx_bufr_head;
  
  // load tx buffer
  SPI1_tx_buffer_add( READ_STATUS_REG1 );
  
  // set number of bytes
  spi1_num_bytes_to_rx = 1000;
  spi1_num_bytes_to_tx = 1;  
  spi_repeat_tx = 1;   
  
  spi1_write_bytes();
  
  rx_bufr_tail++; // skip over command  
  rx_bufr_head++; // skip over command  
  
  // after the read command, status is sent continuously 
  do
  {
    // wait for data, a new status reg
    while( rx_bufr_tail == rx_bufr_head );
    // read from buffer
    status1 = spi_receive_buffer[rx_bufr_tail++];
    if (spi1_num_bytes_to_rx == 0)
    {
      error_log( 2303 );  // timeout
      break;  
    }
  }  
  while( status1 & 1);  // while Flash is busy
  // stop reading
  spi1_num_bytes_to_rx = 0;
  spi1_num_bytes_to_tx = 0;
  spi_repeat_tx = 0;
  
  while(SPI1_SR->bsy); // wait for SPI not busy 
  
  spi1_csn( 1 );
  SPI1_CR2->txeie = 0;  // turn off TX interrupt 
  spi1_enable( 0 );  // turn off SPI1

  tx_bufr_tail = tx_bufr_head;  
  rx_bufr_tail = rx_bufr_head;

}  
// =============================================================================
//
// Read flash memory
//
//  a number of bytes is read starting at the address given
//  data is read into the Rx buffer
//  buffer is 256 but 4 bytes are used by the read command and address
//  max 252 bytes at a time with this. The buffer will wrap around so it
//  must be read before it does.

// History
//    06/04/2018 
// Status written, tested, working 

void flash_read( uint32_t address, uint32_t numbytes )
{
  // if address is out of range for the flash chip
  if( (address + numbytes) >  0x007FFFFF) error_log( 2301 );
  
  // load tx buffer, interrupt routine will send 
  SPI1_tx_buffer_add( READ_DATA );
  SPI1_tx_buffer_add((address & 0x00FF0000) >> 16 );  // MSB address
  SPI1_tx_buffer_add((address & 0x0000FF00) >> 8 );  // address
  SPI1_tx_buffer_add( address & 0x000000FF);      // LSB address
  
  // set number of bytes to read
  spi1_num_bytes_to_rx = 4 + numbytes;
  spi1_num_bytes_to_tx = 4;
  spi_repeat_tx = 1;  
  
  spi1_write_bytes();
  
  while(spi1_num_bytes_to_rx > 0);
  while(SPI1_SR->bsy);
  
  // rx buffer has data
  // skip command and address in the buffer so that reading
  // from the buffer tail is the data
  rx_bufr_tail += 4;
  
}

// =============================================================================
//
//   FLASH Command - send a single byte command 
//     many Flash commands are a single byte with no receiving needed
//     use this for:
//       WRITE_ENABLE
//       VOL_SR_WRITE_ENABLE
//       WRITE_DISABLE
//       CHIP_ERASE
//       CHIP_ERASE_ALT 
//       ERASE_SUSPEND
//       ERASE_RESUME
//       ENABLE_QPI
//       ENABLE_RESET
//       RESET
//
//     This is Not for commands with addresses or that expect a response.
//
// History
//    06/22/2018 
// Status written, testing, revising

void flash_command( uint8_t command )
{

  // load tx buffer, interrupt routine will send 
  SPI1_tx_buffer_add( command );

   // set number of bytes to read, just one to match tx
  spi1_num_bytes_to_rx = 1;
  spi1_num_bytes_to_tx = 1;
  spi_repeat_tx = 0;  
  
  spi1_write_bytes();
  
  while(spi1_num_bytes_to_rx > 0);
  while(SPI1_SR->bsy);
  
  //msec_delay(3);
  // skip command in buffer
  rx_bufr_tail++;

}
// =============================================================================
//
//   Write to external flash memory (page program)
//
// maximum = 252 bytes, 256 bytes - 4 bytes for command and address 
// If address is in the middle of the page, bytes written past the end
// will wrap around to the start of the page, not go to the next page.
// The two MSB of the address are the page number, the LSB is the position
// within the page
//
// In full duplex SPI must read for each write, so the rx buffer will fill
// up with useless data. Adjust the buffer pointer at the end to skip garbage.

// History
//    06/05/2018 
// Status written, tested, working

// todo: modify for ability to write a full page

void flash_write ( uint8_t *data, uint32_t address, uint8_t numbytes )
{
  int i;
  uint8_t status1;   
  
  // if address is out of range for the flash chip
  if( (address + numbytes) >  0x007FFFFF) error_log( 2301 );
  // if write will wrap around to beginning of the page
  if( ((address & 0x0000FF) + numbytes) > 256) error_log( 3031 );
  
  // send write enable
  flash_command( WRITE_ENABLE );
  // check WEL bit   
  status1 = flash_read_stat1();
  if(!(status1 & 0x02) ) error_log( 2302 );
  // write command
  SPI1_tx_buffer_add( PAGE_PROGRAM ); 
  // 24 bit address 
  SPI1_tx_buffer_add((address & 0x00FF0000) >> 16 );  // MSB address
  SPI1_tx_buffer_add((address & 0x0000FF00) >> 8 );  // address
  SPI1_tx_buffer_add( address & 0x000000FF);      // LSB address  
  // load transmit buffer with data
  for( i=0; i< numbytes; i++)
  {
    SPI1_tx_buffer_add( data[i] );  
  }
   // set number of bytes
  spi1_num_bytes_to_tx = 4 + numbytes;  
  spi1_num_bytes_to_rx = 4 + numbytes;

  // send data
  spi1_write_bytes();

  // wait for last byte to be received
  while(spi1_num_bytes_to_rx > 0);
  // wait till SPI is done 
  while(SPI1_SR->bsy);  
  // wait till flash is done
  flash_busy_wait();
  
}
  
// =============================================================================
//
//   Flash erase
//
//   Erasing the flash chip writes all ones to the flash memory.
//   Writing data will change the ones to zeros depending on the data.
//   Writing will not change a zero back to a one. The chip must be erased.
//   
// all the erase commands (except chip erase) have the same procedure,
// just the size erased and the command are different 
// erase flash memory sector  4K
// erase flash memory block  32K
// erase flash memory block  64K
// erase flash memory chip   all
  // cs low
  // send write enable 
  // send erase command
  // 24 bit address
  // cs high at end of address
  // wait for erase to complete

// History
//    06/05/2018 
// Status written, tested sector erase working, others not tested

void flash_erase( uint32_t address, uint8_t size )
{
  // send write enable
  flash_command( WRITE_ENABLE );
  // check WEL bit   
  status1 = flash_read_stat1();
  if(!(status1 & 0x02) ) error_log( 2302 );
  
  switch( size )
  {
  case 0:  // sector erase
    {
      // load tx buffer, 
      SPI1_tx_buffer_add( SECTOR_ERASE );
      // sector address (last three bytes should be 0) 0x007FF000
      break;
    }
   case 1:  // block erase 32K
    {
      // load tx buffer,
      SPI1_tx_buffer_add( BLOCK_ERASE_32K );
      // block address (last three bytes should be 0
      // and low three bits of 4th byte) 0x007F8000
    }
    case 2:  // block erase 64K
    {
      // load tx buffer,
      SPI1_tx_buffer_add( BLOCK_ERASE_64K );
      // block address (last four bytes should be 0) 0x007F0000
      break;
    }
    case 3:  // chip erase
    {
      flash_command( CHIP_ERASE ); 
      break;    
    }
    default: error_log( 3001); break;
  }
  // all run through the ISR, except chip erase
  if(size < 3)
  {
      SPI1_tx_buffer_add((address & 0x00FF0000) >> 16 );  // MSB address
      SPI1_tx_buffer_add((address & 0x0000FF00) >> 8 );  // address
      SPI1_tx_buffer_add( address & 0x000000FF);      // LSB address
       // set number of bytes
      spi1_num_bytes_to_tx = 4;  
      spi1_num_bytes_to_rx = 4;
      // send 
      spi1_write_bytes();
  }

  rx_bufr_tail += 4;   // treat command and address as read
  
  // wait for ISR to finish
  while(spi1_num_bytes_to_rx > 0);
  // wait till SPI is done 
  while(SPI1_SR->bsy);  
  // wait till flash is done
  flash_busy_wait();
} 





