//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// December 11, 2018         Design Documents                     CSI_FEATURES.c
//
//		      routines for enhanced features
//                     large panel display project
//
// Copyright
//  Connect Systems Inc.
//
// Programmer
//  Dean S. Graham
//  
// History
//     12/11/2018 started
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working

//  get_number_input                 reads digits from mic keypad
//  display_fav_zones                display favorite zone screen
//  display_fav_channels             display favorite channel screen
//  read_favorites                   read zone or channel favorites from EEPROM      
//  save_all_favorites               save zone or channel favorites to EEPROM
//  save_favorite                    save one zone or channel favorite to EEPROM
//  clear_all_favorites              clear zone or channel favorites
//  process_zone_favs                handle zone favorites screen
//  process_channel_favs             handle channel favorites screen
//  display_chan_menuitem            display a menu item from the chanMenu table
//  display_item_selector            draw a selctor box around a menu item
//  display_float_item               displays a float value
//  display_integer_item             displays an integer value
//  display_freq_units               display frequency units MHz, KHz, Hz
//  display_chan_freq_units          display a float frequency with units
//  display_text_item                display a text item
//  check_freq_inrange               check if a frequency is in range limits
//  process_vfo_mode                 handle VFO mde screen
//  set_chanmenu                     sets menu sequence depending on settings
//  inc_inrange                      increments a value and keeps within a range
//  dec_inrange                      decrements a value and keeps within a range
//  process_channel_screen           handle entry of channel settings
//  display_frequency                display an integer frequency (radio /10)
//  display_contact_id               displays a DMR contact id
//  display_squelch_type             displays the channel squelch type 
//  process_e_chan_mode              handle enhanced channel entry screen
//  display_spectrum_graph           displays the spectrum graph
//  get_spectrum_data                gets rssi data from radio for spectrum
//  process_spectrum_view            handle spectrum view screen
//  process_enhscan_mode             handle enhanced scan screen

// Status: written, not used
 
//  DirectZoneChannelSelect          enter and change to a zone and channel
//  AF_DZCS_State_00
//  AF_DZCS_State_01
//  AF_DZCS_State_02
//  AF_DZCS_State_03
//  AF_DZCS_State_04

//==============================================================================
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "CSI_FEATURES.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_SSD1963.h"
#include "CSI_MC59.h"
#include "CSI_REGISTERS.h"
#include "CSI_SCREENS.h"
#include "CSI_SWITCHES.h"
#include "CSI_Menus.h"
#include "CSI_GRAPHICS.h"
#include "CSI_COLOR.h"
#include "CSI_ICONS.h"
#include "CSI_eeprom.h"
#include "CSI_TouchPanel.h"
#include "CSI_i2c.h"
#include "CSI_ERROR_LOG.h"

uint32_t AF_DZCS_State = 0;
uint32_t nav_key;

// structures for zone and channel favorites
fav_info favorite_zone[16];
fav_info favorite_channel[16];

// move to appropriate header
void get_number_input( uint16_t pos_x, uint16_t pos_y,
                      uint8_t font, uint8_t num_digits);

// for vfo, enhanced scan 
uint8_t dial_rotation = 0;
uint32_t rx_freq;
uint32_t tx_freq;
// for enhanced scan
uint32_t scan_start_freq = 400000000;  // 400 Mhz default
uint32_t scan_end_freq =   470000000;  // 470 Mhz
uint32_t scanning_freq =   400000000;  //
uint8_t  is_scanning = 0;

uint32_t scan_step_delay = 200;  // 1/5 second delay between scan steps
uint32_t scan_rx_delay = 10;  // 10 second delay if receiving

unsigned long scan_step_timer;
unsigned long scan_rx_timer;

uint32_t freq_step = 100;  // default, 1 Khz
                           // step and offset are / 10 for radio
//int txrx_offset =  -500000; // for quick test 5 Mhz
int txrx_offset =  0; // direct mode

uint32_t ctcss_encode = 100;
uint32_t ctcss_decode = 100;

uint32_t dcs_encode = 23;
uint32_t dcs_decode = 23;


char user_input_buffer[17];
char temp_text[17]; // temp for string building 
/*
// =============================================================================
// 
//  Direct zone channel select
//
// Programmer
//  Jerry Wenger
//
// History
//  12/11/2018 started
//
// Status
//  written, not in use
//  state machine with while(1) needs to be written to allow being called
//   by process_panel_menu. No trapped while loops

void DirectZoneChannelSelect(void)
{
  while(1)
  {
    switch(AF_DZCS_State)
    {
      // Setup screen
      case 0:
        AF_DZCS_State_00();
        break;
       // Enter Zone Number       
      case 1:
        AF_DZCS_State_01();
        break; 
      case 2:
        AF_DZCS_State_02();
        break;
      // Enter Channel Number        
      case 3:
        AF_DZCS_State_03();
        break;  
      case 4:
        AF_DZCS_State_04();
        break;
      // Send packet to radio
      case 5:
        set_zone_channel( zone_number, channel_number );
        AF_DZCS_State = 0;
        // generate a cancel
        CS800_buttons[ 3 ].press_flag = 1;
        // set timer to release, short press only      
        short_release_flag = &CS800_buttons[ 3 ].release_flag;
        short_press_timeout = 200;       
        break;

      default:
        error_log( 3100 );
        AF_DZCS_State = 0;
        break;
     }
  }
}
//
// *****************************************************************************
// Clear and display the title and messages
// *****************************************************************************
//
void AF_DZCS_State_00(void)
{
  // title
  ssd1963_cg_clear_display();
  strncpy( (char*) cg_line_buffer, "Select  Zone  and  Channel", 26);
  ssd1963_cg_write_line(26, 2, 100 ,50, 4);
  // zone 
  strncpy( (char*) cg_line_buffer, "Zone Number      ", 17);
  ssd1963_cg_write_line(17, 3, 50 , 150, 2);
  // channel
  strncpy( (char*) cg_line_buffer, "Channel Number    ", 19);
  ssd1963_cg_write_line(19, 3, 50 , 250, 2);
  
  draw_navbuttons(1);
  last_key_press = 0;
  dtmf_code_flag = 0;
  
  AF_DZCS_State++;
}
//
// *****************************************************************************
// Enter Zone Number: Initialize data entry buffer
// *****************************************************************************
//
void AF_DZCS_State_01(void)
{
  cg_line_buffer[0] = ' ';
  cg_line_buffer[1] = ' ';
  cg_line_buffer[2] = ' ';
  cg_line_buffer[3] = 0;
  AF_DZCS_State++;
}
//
// *****************************************************************************
// Enter Zone Number: Enter data
// *****************************************************************************
//

void AF_DZCS_State_02(void)
{
  //
  // Process numeric characters
  //
  cursor_blink( 275 + 2 * cg_max_hori_size + 14, 150 + 32, 3 );   
  
  if (dtmf_code_flag)
  {
    dtmf_code_flag = 0;
    if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) return;
    
    get_number_input( 325, 150, 3, 3); 
    return;
  }
  //
  // If NONE, UP, or DOWN, ignore
  //
  nav_key = get_nav_key();
  if (nav_key == NONE) return;
  last_key_press = 0;
  if ((nav_key == UP) || (nav_key == DOWN)) return;
  //
  // if a CANCEL key pressed, clear buffer and output blanks
  //
  if (nav_key == CANCEL)
  {
    clear_rectangle(275,150, 3 * cg_max_hori_size, cg_max_vert_size);
    memcpy( cg_line_buffer, "   ", 3);
    // ssd1963_cg_write_line(5, 3, 275 , 150, 6);
    return;
  }
//
// If a SELECT key pressed, convert buffer to binary to check if in range.
// if out of range, clear buffer and display. If in range, store data and 
// advance to next state.
//
  if (nav_key == SELECT)
  {
    zone_number = (100*(cg_line_buffer[0] & 0x0F)) +
                  ( 10*(cg_line_buffer[1] & 0x0F)) +
                  (  1*(cg_line_buffer[2] & 0x0F));
    if ((zone_number == 0) || (zone_number > 250))
    {
      memcpy( user_input_buffer, "   ", 3);
      clear_rectangle(275,150, 3 * cg_max_hori_size, cg_max_vert_size);
      //ssd1963_cg_write_line(5, 3, 275 , 150, 0);
      return;      
    }
    AF_DZCS_State++;
  }
}
//
// *****************************************************************************
// Enter Channel Number: Initialize data entry buffer
// *****************************************************************************
//
void AF_DZCS_State_03(void)
{
  // memcpy( cg_line_buffer, "     ", 5);
  user_input_buffer[0] = ' ';
  user_input_buffer[1] = ' ';
  user_input_buffer[2] = 0;  
  AF_DZCS_State++;
}
//
// *****************************************************************************
// Enter Channel Number: Enter data
// *****************************************************************************
//
void AF_DZCS_State_04(void)
{
  // clear zone cursor, may have left in the on state
  display_cursor( 0, 275 + 2 * cg_max_hori_size + 14, 150 + 32, 3 );
  //
  // Process numeric characters
  //
  cursor_blink( 325 + cg_max_hori_size + 14 , 250 + 32, 3 );
  
  if (dtmf_code_flag)
  {
    dtmf_code_flag = 0;
    if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) return;
   
    get_number_input( 375, 250, 3, 2);    
    return;
  }
  //
  // If NONE, UP, or DOWN, ignore
  //
  nav_key = get_nav_key();
  if (nav_key == NONE) return;
  last_key_press = 0;
  if ((nav_key == UP) || (nav_key == DOWN)) return;
  //
  // if a CANCEL key pressed, clear buffer and output blanks
  //
  if (nav_key == CANCEL)
  {
    user_input_buffer[0] = ' ';
    user_input_buffer[1] = ' ';
    clear_rectangle(375,200, 2 * cg_max_hori_size, cg_max_vert_size); 
    return;
  }
  //
  // If a SELECT key pressed, convert buffer to binary to check if in range.
  // if out of range, clear buffer and display. If in range, store data and 
  // advance to next state.
  //
  if (nav_key == SELECT)
  {
    channel_number = (10*(cg_line_buffer[0] & 0x0F)) +
                     ( 1*(cg_line_buffer[1] & 0x0F));
    if ((channel_number == 0) || (channel_number > 16))
    {
      user_input_buffer[0] = ' ';
      user_input_buffer[1] = ' ';
      clear_rectangle(375,200, 2 * cg_max_hori_size, cg_max_vert_size);
      return;      
    }
    // clear channel cursor, may have left in the on state
    display_cursor( 0, 325 + cg_max_hori_size + 14, 250 + 32, 3 );
    
    AF_DZCS_State++;
  }
}

*/
// =============================================================================
//
//  Enter numberic data with the mic keypad 
//
//  reads digits from mic keys, displays on the screen
//  The buffer is shifted to the left, then data is entered in smallest digit
//  the result is in the text buffer
//  when pixel_line_width is called, cg sizes are set
//
// Programmer Dean S. Graham
//
// History: 12/17/2018 written
//          01/04/2019 modified to use a dedicated buffer
//
// Status: tested, working
//
int pix_width;

static void get_number_input( uint16_t pos_x, uint16_t pos_y,
                      uint8_t font, uint8_t num_digits)
{
  int i = 0;
  // adjust width for max digit size + spacing (not max_width)
  // should be constants stored in font file, and read with font info
  int digit_width = 0;
  switch(font)
  {
  case 0: digit_width = 48; break;
  case 1: digit_width = 34; break;
  case 2: digit_width = 27; break;
  case 3: digit_width = 18; break;
  case 4: digit_width = 16; break;
  }
  // be sure buffer has a null at the end of the digits
  user_input_buffer[num_digits] = '\0';
  
  while(i < num_digits - 1)
  {
    user_input_buffer[i] = user_input_buffer[i+1];
    i++;
  }
  user_input_buffer[num_digits - 1] = dtmf_code_char;
  
  pix_width = pixel_line_width( user_input_buffer, font, 5 );
  
  // clear 
  clear_rectangle(pos_x, pos_y, num_digits * digit_width, cg_max_vert_size);
  // copy to text output buffer
  strncpy( (char *) cg_line_buffer, user_input_buffer, 9); 
  // right justify
  // start at x_pos + box width - pixel width
  ssd1963_cg_write_line(num_digits, font,
           pos_x + ( num_digits * digit_width) - pix_width, pos_y, 5);    
}

// =============================================================================
//
//  display favorite zones
//
//  full screen, 2 columns of 8
//
// Programmer Dean S. Graham
//
// History: 12/17/2018 written
//
// Status: tested, working

static void display_fav_zones( uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int i;
  int column = 0;
  int row = 0;
  
  for(i = 0; i < 16; i++ )
  {
    strncpy( (char *) cg_line_buffer, favorite_zone[i].name, 17);
    ssd1963_cg_write_line(17, font, pos_x + column , pos_y + row * 40, 3);
    row++;
    if( i == 7)
    {
      row = 0;
      column = 400;
    }
  }
}
// =============================================================================
//
//  display favorite channels
//
//  full screen, 2 columns of 8
//
// Programmer Dean S. Graham
//
// History: 12/17/2018 written, working
//
static void display_fav_channels( uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int i;
  int column = 0;
  int row = 0;
  
  for(i = 0; i < 16; i++ )
  {
    strncpy( (char *) cg_line_buffer, favorite_channel[i].name, 17);
    ssd1963_cg_write_line(17, font, pos_x + column , pos_y + row * 40, 3);
    row++;
    if( i == 7)
    {
      row = 0;
      column = 400;
    }
  }
}
// =============================================================================
//
//  read zone or channel favorites from eeprom
// 
//  reads eeprom and stores favorites into the structure
//  favorites array has 16 records of structure:
//  name    - 17 bytes
//  zone    - 1 byte 1-250
//  channel - 2 byte 1-16  (0-999)
//  20 * 16 = 320  I2C buffer is 130 bytes, 128 for one page +2 address
// 
//  read from three pages separately 
//
// Programmer Dean S. Graham
//
// History: 12/17/2018 written
//
// Status: tested, working

// example use:
// read_favorites( favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO )
// read_favorites( favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO )

void read_favorites( fav_info* favorite, uint8_t Loc_HI, uint8_t Loc_LO )
{
  int i;
  int j;
  int h_byte;
  
  // read  favorites
  EEPROM_MemRead( 0, Loc_HI, Loc_LO, 120);
  // wait for I2C to finish
  EEPROM_waitfor_I2C();  
 
  // read lower fav's from buffer 6 * 20 = 120
  // 6 0x00 to 0x78, 0 - 5
  for( i = 0; i < 6; i++)
  {
    // read text
    for( j = 0; j < 17; j++)
    {
      favorite[i].name[j] = I2C_get_datain();  
    }
    // read zone and channel nums
    favorite[i].zone_num = I2C_get_datain();
    h_byte = I2C_get_datain();
    favorite[i].channel_num = I2C_get_datain() + ( h_byte << 8);
  }
  // read more
  EEPROM_MemRead( 0, Loc_HI, Loc_LO + 0x80, 120);
  EEPROM_waitfor_I2C();  
 
  // 6 more 0x80 to 0xF8, 6 - 11
  for( i = 6; i < 12; i++)
  {
    for( j = 0; j < 17; j++)
    {
      favorite[i].name[j] = I2C_get_datain(); 
    }
    favorite[i].zone_num = I2C_get_datain();
    h_byte = I2C_get_datain();
    favorite[i].channel_num = I2C_get_datain() + ( h_byte << 8);
  }
  // read more
  EEPROM_MemRead( 0, Loc_HI + 1, Loc_LO, 80);
  EEPROM_waitfor_I2C();  
 
  // 4 more 0x100 to 0x178, 12 - 15
  for( i = 12; i < 16; i++)
  {
    for( j = 0; j < 17; j++)
    {
      favorite[i].name[j] = I2C_get_datain();  
    }
    favorite[i].zone_num = I2C_get_datain();
    h_byte = I2C_get_datain();
    favorite[i].channel_num = I2C_get_datain() + ( h_byte << 8);
  } 
}
// =============================================================================
//
//   Save favorites array to EEPROM
//
//  stores favorites in the structure to the EEPROM
//  store to three pages separately 6,6,4  16 records
//
// Programmer Dean S. Graham
//
// History 12/18/2018 written

// example use:
// save_favorites( favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO )
// save_favorites( favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO )

extern void msec_delay( unsigned int delay );

void save_all_favorites( fav_info* favorite, uint8_t Loc_HI, uint8_t Loc_LO )
{
  int i;
  int j;
  
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout( Loc_HI );  
  I2C_put_dataout( Loc_LO );
  // write first 6
  for( i = 0; i < 6; i++)
  {
    for( j = 0; j < 17; j++)
    {
      I2C_put_dataout( favorite[i].name[j] );
    }
    I2C_put_dataout( favorite[i].zone_num); 
    I2C_put_dataout( (favorite[i].channel_num & 0xFF00) >> 8 );
    I2C_put_dataout( favorite[i].channel_num & 0x00FF );                    
  }
  // write to EEPROM  3+120  123 bytes
  EEPROM_MultipleWrite( 0, 123);
  EEPROM_waitfor_I2C();
  msec_delay(5);
  
  // put address in output buffer
  I2C_put_dataout( Loc_HI );  
  I2C_put_dataout( Loc_LO + 0x80 );  
  // write second 6
  for( i = 6; i < 12; i++)
  {
    for( j = 0; j < 17; j++)
    {
      I2C_put_dataout( favorite[i].name[j] );
    }
    I2C_put_dataout( favorite[i].zone_num); 
    I2C_put_dataout( (favorite[i].channel_num & 0xFF00) >> 8 );
    I2C_put_dataout( favorite[i].channel_num & 0x00FF );                    
  }
  // write to EEPROM  3+120  123 bytes
  EEPROM_MultipleWrite( 0, 123);
  EEPROM_waitfor_I2C();
  msec_delay(5);
  
  // put address in output buffer
  I2C_put_dataout( Loc_HI + 1 );  
  I2C_put_dataout( Loc_LO );  
  // write last 4
  for( i = 12; i < 16; i++)
  {
    for( j = 0; j < 17; j++)
    {
      I2C_put_dataout( favorite[i].name[j] );
    }
    I2C_put_dataout( favorite[i].zone_num); 
    I2C_put_dataout( (favorite[i].channel_num & 0xFF00) >> 8 );
    I2C_put_dataout( favorite[i].channel_num & 0x00FF );                    
  }
  // write to EEPROM  3+80  83 bytes
  EEPROM_MultipleWrite( 0, 83);
  EEPROM_waitfor_I2C(); 
  msec_delay(5);
   
}
// =============================================================================
//
//   Save a single favorite to EEPROM
//
//   favorite numbers are 0-15
//
// Programmer Dean S. Graham
//
// History 12/18/2018 written
//
// Status: tested, working

// example use:
// save_favorite( 3, favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO )
// save_favorite( 3, favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO )

void save_favorite( uint8_t fav_num, fav_info* favorite, uint8_t Loc_HI, uint8_t Loc_LO )
{
  int i;
  uint8_t fav_LO;
  uint8_t fav_HI;
  uint8_t page = 0;   // an error if not set  
  
  // calculate address for this fav number
  if( fav_num < 6 ) page = 1;                   // 0-5
  if( fav_num > 5  && fav_num < 12 ) page = 2;  // 6-11
  if( fav_num > 11 && fav_num < 16 ) page = 3;  // 12-15
  
  switch( page )
  {
  case 1: 
    {
      fav_HI = Loc_HI;
      fav_LO = Loc_LO + fav_num * 20;
      break;
    }
  case 2:
    {
      fav_HI = Loc_HI;
      fav_LO = Loc_LO + 0x80 + (fav_num - 6) * 20;
      break;      
    }
  case 3: 
    {
      fav_HI = Loc_HI + 1;
      fav_LO = Loc_LO + (fav_num - 12) * 20;
      break;      
    }
  default:
    {
      error_log( 3100 );
      return;     // return with no write
      // break;
    }
  }
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout( fav_HI );  
  I2C_put_dataout( fav_LO );
  // load favorite data
  for( i = 0; i < 17; i++)
  {
    I2C_put_dataout( favorite[fav_num].name[i] );
  }
  I2C_put_dataout( favorite[fav_num].zone_num); 
  I2C_put_dataout( (favorite[fav_num].channel_num & 0xFF00) >> 8 );
  I2C_put_dataout( favorite[fav_num].channel_num & 0x00FF );
  
  // write to EEPROM  3+20  23 bytes
  EEPROM_MultipleWrite( 0, 23);
  EEPROM_waitfor_I2C();
  
}
// =============================================================================
//
//   clears favorites 
//
//  erases all favorites, zone or channel 
//
// Programmer Dean S. Graham
//
// History 12/19/2018 written, working 

void clear_all_favorites( fav_info* favorite  )
{
  int i;
  for(i = 0; i< 16; i++)
  {
    strncpy( (char *) favorite[i].name, "                ", 16);
    favorite[i].name[16] = '\0';
    favorite[i].zone_num = 0;
    favorite[i].channel_num = 0;
  }
}
// =============================================================================
//
//  process zone select page
//
// Programmer Dean S. Graham
//
// History 12/18/2018 written
//
// Status: working, needs revision. while(1) loop needs to go

uint8_t zf_state = 0; // zone favorite state
int zf_num = 0;   // favorite number

uint16_t zf_x_pos = 30;  // starting corner of list
uint16_t zf_y_pos = 80;

uint8_t zone_fav_changed = 0;

void process_zone_favs( uint8_t nav_key )
{
  int i;
  static int row;
  static int column;
  uint32_t zone_number;  
   
  uint8_t prev_dtmf = 0;
  
  switch( zf_state )
  {
  case 0:   // display page
    {
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0;   // force redraw

      // display title
      strncpy( (char*) cg_line_buffer, "Favorite  Zones", 15);
      ssd1963_cg_write_line(15, 2, 225 ,20, 4);
      // display zone favorites
      display_fav_zones( zf_x_pos, zf_y_pos, 3 );
      column = 0;
      row = 0;
      zf_num = 0;
      select_box_width = 325;        
      // draw selector box at first
      draw_frame(zf_x_pos + column - 10, zf_y_pos + ( row * 40) - 5,
                 select_box_width, 45, 3, highlight );      
  
      //draw buttons (up down)
      draw_navbuttons( 2 );
      last_key_press = 0;
      dtmf_code_flag = 0;

      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4
      
      // activate favorite touch points
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 1;           
    
      push_menu_stack( ZONEFAV );
      zf_state++;
      break;
    }
  case 1:  // navigate list
    {
      // if a number key is pressed, go to zone number entry
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) break;
        prev_dtmf = dtmf_code_char; 
        zf_state = 2; 
        break;
      }
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;        
      switch( nav_key )
      {
      case UP:
        {
          // erase previous selector
          draw_frame(zf_x_pos + column - 10, zf_y_pos + ( row * 40) - 5,
                         select_box_width, 45, 3, cg_background ); 
          zf_num--;
          if( zf_num < 0 )zf_num = 15;
          if( zf_num < 8 )
          {
            column = 0;
            row = zf_num;
          }
          else
          {
            column = 400;
            row = zf_num - 8;            
          }
          // draw new selector box
          draw_frame(zf_x_pos + column - 10, zf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight );
          
          break;
        }
      case DOWN:
        {
          // erase previous selector
          draw_frame(zf_x_pos + column - 10, zf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, cg_background );
          
          zf_num++;
          if( zf_num > 15 )zf_num = 0;
          if( zf_num < 8 )
          {
            column = 0;
            row = zf_num;
          }
          else
          {
            column = 400;
            row = zf_num - 8;            
          }          
          // draw new selector box
          draw_frame(zf_x_pos + column - 10, zf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight );
          break;
        }      
      case SELECT:
        {
          // if zone is 0 or > 250 favorite is not valid
          // go to get input state 
          if( favorite_zone[zf_num].zone_num == 0 ||
              favorite_zone[zf_num].zone_num > 250 )
          {
            zf_state++;
            break;
          }
          // there is a favorite, goto send command to radio state
          zf_state = 4;
          break;
        }
      case CANCEL:
        {
          zf_state = 5;  // skip data entry, jump to exit
          break;
        }        
      default: break;
      }
      break;
    }
  case 2:  // show text to enter new zone num
    {
      // clear current name
      clear_rectangle( zf_x_pos + column, zf_y_pos + ( row * 40),
                      select_box_width-20, 36);
      
      // display zone =
      strncpy( (char *) cg_line_buffer, "zone = ", 7);
      ssd1963_cg_write_line( 7, 3, zf_x_pos + column, zf_y_pos + row * 40, 3);
      // clear buffer 
      memcpy( user_input_buffer, "   ", 3);
      user_input_buffer[3] = '\0';      
      zf_state++;
      break;
    }
  case 3:
    {
      cursor_blink( zf_x_pos + column + 88 + 2 * cg_max_hori_size,
                    zf_y_pos + ( row * 40) + 32, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        //if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) break;
        if( isdigit(dtmf_code_char)) 
        {
          get_number_input( zf_x_pos + column + 120, zf_y_pos + row * 40, 3, 3);
          break;
        }
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // if cancel on a blank space, exit 
        if(strncmp( (char *) cg_line_buffer, "   ", 3) == 0)
        {
          // clear zone entry
          clear_rectangle( zf_x_pos + column, zf_y_pos + ( row * 40),
                           select_box_width-20, 36);            
          // put back the zone name
          strncpy( (char *) cg_line_buffer, favorite_zone[zf_num].name, 17);
          ssd1963_cg_write_line(17, 3, zf_x_pos + column , zf_y_pos + row * 40, 3);
          
          zf_state = 5;  // jump to exit
          break; 
        }                   
        clear_rectangle(zf_x_pos + column + 110 , zf_y_pos + row * 40,
                        3 * cg_max_hori_size, cg_max_vert_size);
        memcpy( user_input_buffer, "   ", 3);
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. If in range, store data and 
      // advance to next state.

      if (nav_key == SELECT)
      {
        // zone number is in the text buffer
        zone_number = atoi( (char *) user_input_buffer );

        if ( zone_number > 250 )
        {
          display_message(" Zone out of range", 20, 420, 3);             
          memcpy( user_input_buffer, "   ", 3);
          user_input_buffer[3] = '\0';
          clear_rectangle(zf_x_pos + column + 110 , zf_y_pos + row * 40,
                          3 * cg_max_hori_size, cg_max_vert_size);
          break;      
        }
        // erase a zone
        if ( zone_number == 0 ) 
        {
          favorite_zone[zf_num].zone_num = 0;
          favorite_zone[zf_num].channel_num = 0;
          favorite_zone[zf_num].name[0] = '\0';
          // write to eeprom
          save_favorite( zf_num, favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO );          
          log_event( "Erase fav zone" );
          // clear the page
          clear_rectangle( 0,0, 800, 410);
          // redraw previous sub menu, with zone selected
          display_sub_menu( 0, 4, 460, 3 );
          // reset box width for this menu
          select_box_width = 460;              
          pop_menu_stack();
          zf_state = 0;   // go back
          break;
        }
        
        // store zone
        favorite_zone[zf_num].zone_num = zone_number;
        // set channel to 1 for all zone favorites
        favorite_zone[zf_num].channel_num = 1;        
        zf_state++;
        break;
      }
    }
  case 4:  // send radio command, set variables 
    {
      // clear channel info is not shown while radio updates
      // zone alias
      strncpy( (char *) cs_channel_zone_alias, "                ", 16); 
      // channel alias
      strncpy( (char *) cs_channel_alias, "                ",16 );
      // set new channel_number
      cs_channel_number_in_zone = 1;
      // set new zone number
      cs_channel_zone_number = zone_number;
      
      // send zone and channel for the selected favorite
      set_zone_channel( favorite_zone[zf_num].zone_num,
                        favorite_zone[zf_num].channel_num );
      zf_state++;
      log_event( "Zone favorite" );
      menu_exit = 1;
      // set variable for packet handler to update the name field
      zone_fav_changed = 1;
      //  break;  // fall through
    }
  case 5:   // return to previous menu
    {
      // turn off favorite touch points
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 0;  
      // clear the page
      clear_rectangle( 0,0, 800, 410);
      if( !menu_exit )
      {
        // redraw previous sub menu, with zone selected
        display_sub_menu( 0, 4, 460, 3 );
        // reset box width for this menu
        select_box_width = 460;              
      }
      pop_menu_stack();
      zf_state = 0;
      return;
    }
  default:
    {
      error_log( 3001 );
      break;
    }
  } // end of state switch
}
// =============================================================================
//
//  process channel select page
//
// Programmer Dean S. Graham
//
// History 12/19/2018 written
//
// Status: working.


uint8_t cf_state = 0; // channel favorite state
int cf_num = 0;       // channel favorite number

uint16_t cf_x_pos = 30;  // starting corner of list
uint16_t cf_y_pos = 80;

uint8_t channel_fav_changed = 0;

void process_channel_favs( uint8_t nav_key )
{
  int i;
  static int column;
  static int row;
  static uint32_t zone_number;
  static uint32_t channel_number;  
  uint8_t prev_dtmf = 0;
    
  switch( cf_state )
  {
  case 0:   // display page
    {
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0;   // force redraw
      // display title
      strncpy( (char*) cg_line_buffer, "Favorite  Channels", 18);
      ssd1963_cg_write_line(18, 2, 175 ,20, 4);
      // display channel favorites
      display_fav_channels( cf_x_pos, cf_y_pos, 3 );
      column = 0;
      row = 0;
      cf_num = 0;  
      select_box_width = 375;        
      // draw selector box at first
      draw_frame(cf_x_pos + column - 10, cf_y_pos + ( row * 40) - 5,
                 select_box_width, 45, 3, highlight);
      //draw buttons (up down)
      draw_navbuttons( 2 );
      last_key_press = 0;
      dtmf_code_flag = 0;
      push_menu_stack( CHANFAV );

      // turn on the four buttons as touch points
      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4  
      // activate favorite touch points
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 1;           
    
      cf_state++;
      break;
    }
  case 1:  // navigate list
    {
      // if a number key is pressed, go to zone and channel number entry
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) break;
        prev_dtmf = dtmf_code_char; 
        cf_state = 2; 
        break;
      }
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;        
      switch( nav_key )
      {
      case UP:
        {
          // erase previous selector
          draw_frame(cf_x_pos + column - 10, cf_y_pos + ( row * 40) - 5,
                         select_box_width, 45, 3,
                         cg_background);
          cf_num--;
          if( cf_num < 0 )cf_num = 15;
          if( cf_num < 8 )
          {
            column = 0;
            row = cf_num;
          }
          else
          {
            column = 400;
            row = cf_num - 8;            
          }
          // draw new selector box
          draw_frame(cf_x_pos + column - 10, cf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight);          
          
          break;
        }
      case DOWN:
        {
          // erase previous selector
          draw_frame(cf_x_pos + column - 10, cf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3,
                     cg_background);
          
          cf_num++;
          if( cf_num > 15 )cf_num = 0;
          if( cf_num < 8 )
          {
            column = 0;
            row = cf_num;
          }
          else
          {
            column = 400;
            row = cf_num - 8;            
          }          
          // draw new selector box
          draw_frame(cf_x_pos + column - 10, cf_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight);          
          break;
        }      
      case SELECT:
        {
          // if channel is 0 or > 999 favorite is not valid
          // go to get input state 
          if( favorite_channel[cf_num].channel_num == 0 ||
              favorite_channel[cf_num].channel_num > 999 )
          {
            cf_state++;
            break;
          }
          // there is a favorite, goto send command to radio state
          cf_state = 5;
          break;
        }
      case CANCEL:
        {
          cf_state = 6;  // skip data entry, jump to exit
          break;
        }        
      default: break;
      }
      break;
    }
  case 2:  // show text to enter new zone num, chan num
    {
      // clear current name
      clear_rectangle( cf_x_pos + column, cf_y_pos + ( row * 40),
                      select_box_width-20, 36);
      
      // display zone =
      strncpy( (char *) cg_line_buffer, "zone = ", 7);
      ssd1963_cg_write_line( 7, 3, cf_x_pos + column, cf_y_pos + row * 40, 3);
      // display channel =
      strncpy( (char *) cg_line_buffer, "chan = ", 7);
      ssd1963_cg_write_line( 7, 3, cf_x_pos + column + 190, cf_y_pos + row * 40, 3);
      
      // clear buffer, space for zone number 
      memcpy( user_input_buffer, "   ", 3);
      user_input_buffer[3] = '\0';      
      cf_state++;
      break;
    }
  case 3:  // get zone number
    {
      cursor_blink( cf_x_pos + column + 84 + 2 * cg_max_hori_size,
                    cf_y_pos + ( row * 40) + 32, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) break;
        
        get_number_input( cf_x_pos + column + 120, cf_y_pos + row * 40, 3, 3); 
        break;
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "   ", 3) == 0)
        {
          // clear zone and channel entry
          clear_rectangle( cf_x_pos + column, cf_y_pos + ( row * 40),
                           select_box_width-20, 36);            
          // put back the channel name
          strncpy( (char *) cg_line_buffer, favorite_channel[cf_num].name, 17);
          ssd1963_cg_write_line(17, 3, cf_x_pos + column , cf_y_pos + row * 40, 3);
          
          cf_state = 6;  // jump to exit
          break; 
        }              
        clear_rectangle(cf_x_pos + column + 95 , cf_y_pos + row * 40,
                        3 * cg_max_hori_size-25, cg_max_vert_size);
        memcpy( user_input_buffer, "   ", 3);
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. If in range, store data and 
      // advance to next state.

      if (nav_key == SELECT)
      {
        // zone number is in the text buffer
        zone_number = atoi((char *) user_input_buffer ); 
        if ( zone_number > 250)
        {
          display_message(" Zone out of range", 20, 420, 3);   
            
          memcpy( user_input_buffer, "   ", 3);
          user_input_buffer[3] = '\0';  
          clear_rectangle(cf_x_pos + column + 75 , cf_y_pos + row * 40,
                          3 * cg_max_hori_size-25, cg_max_vert_size);
          break;      
        }
        // erase
        if ( zone_number == 0 )    
        {
          favorite_channel[cf_num].zone_num = 0;
          favorite_channel[cf_num].channel_num = 0;
          favorite_channel[cf_num].name[0] = '\0';
          // write to eeprom
          save_favorite( cf_num, favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO );          
          cf_state++;
          log_event( "Erase fav channel" );
          // clear the page
          clear_rectangle( 0,0, 800, 410);
          // redraw previous sub menu, with channel selected
          display_sub_menu( 0, 5, 460, 3 );
          // reset box width for this menu
          select_box_width = 460;              
          pop_menu_stack();          
          cf_state = 0;  // zone 0, don't send to radio, go back 
          break;
        }          
          
      }
        
        // clear buffer, space for channel number 
        memcpy( user_input_buffer, "  ", 2);
        user_input_buffer[2] = '\0'; 
        
        // clear zone cursor
        display_cursor( 0, cf_x_pos + column + 84 + 2 * cg_max_hori_size,
                           cf_y_pos + ( row * 40) + 32, 3 );
        cf_state++;
        break;
    }
  case 4:  // get channel number 
    {
      cursor_blink( cf_x_pos + column + 260 + 2 * cg_max_hori_size,
                    cf_y_pos + ( row * 40) + 32, 3 );   
      
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if ((dtmf_code_char < 0x30) || (dtmf_code_char > 0x39)) break;
        
        get_number_input( cf_x_pos + column + 300, cf_y_pos + row * 40, 3, 2); 
        break;
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "  ", 2) == 0)
        {
          // clear zone and channel entry
          clear_rectangle( cf_x_pos + column, cf_y_pos + ( row * 40),
                           select_box_width-20, 36);            
          // put back the channel name
          strncpy( (char *) cg_line_buffer, favorite_channel[cf_num].name, 17);
          ssd1963_cg_write_line(17, 3, cf_x_pos + column , cf_y_pos + row * 40, 3);
          
          cf_state = 6;  // jump to exit
          break; 
        }
        clear_rectangle(cf_x_pos + column + 280 , cf_y_pos + row * 40,
                        2 * cg_max_hori_size -5, cg_max_vert_size);
        memcpy( user_input_buffer, "  ", 2);
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. If in range, store data and 
      // advance to next state.

      if (nav_key == SELECT)
      {
        // channel number is in the text buffer
        channel_number = atoi( (char *) cg_line_buffer );

        if ((channel_number == 0) || (channel_number > 999))
        {
          display_message(" Channel out of range", 20, 420, 3); 
          
          memcpy( user_input_buffer, "  ", 2);
          user_input_buffer[2] = '\0';              
          clear_rectangle(cf_x_pos + column + 285 , cf_y_pos + row * 40,
                          2 * cg_max_hori_size, cg_max_vert_size);
          break;      
        }
        // store zone in channel favorite
        favorite_channel[cf_num].zone_num = zone_number;               
        // store channel in channel favorite
        favorite_channel[cf_num].channel_num = channel_number;
            
        cf_state++;
        break;
      }
      break;
    }
  case 5:  // send radio command, set variables 
    {
      // clear channel info is not shown while radio updates
      // zone alias
      strncpy( (char *) cs_channel_zone_alias, "                ", 16); 
      // channel alias
      strncpy( (char *) cs_channel_alias, "                ",16 );
      // set new channel_number
      cs_channel_number_in_zone = channel_number;
      // set new zone number
      cs_channel_zone_number = zone_number;
      
      // send zone and channel for the selected favorite to the radio
      set_zone_channel( favorite_channel[cf_num].zone_num,
                        favorite_channel[cf_num].channel_num );
      
      log_event( "Channel favorite" ); 
      cf_state++;
      menu_exit = 1;
       // set variable for packet handler to update the name field
      channel_fav_changed = 1;
      // break;   // no break, fall through to exit
    }
  case 6: // return to previous menu - separate state for cancel bail out
    {
      // turn off favorite touch points
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 0;  
      // clear the page
      clear_rectangle( 0,0, 800, 410);
      if( !menu_exit )
      {
        // redraw previous sub menu, with channel favs selected
        display_sub_menu( 0, 5, 460, 3 );
        // reset box width for this menu
        select_box_width = 460;
      }
      cf_state = 0;
      pop_menu_stack();
      break;
    }
  default:
    {
      error_log( 3001 );
      break;
    }
  } // end of state switch
}
// =============================================================================
//
//  process VFO mode
//
// Programmer: Dean S. Graham
//
// History: 12/21/2018
//          01/17/2019 revised to use channel screen process
//
// Status: writing, copied from channel favorite, rewriting 

// CTCSS codes table (codes are 10 x frequency) 
uint16_t ctcss_code[41] = {
  670, 693, 719, 744, 770, 797, 825, 854, 885, 915, 948, 974,
  1000, 1035, 1072, 1109, 1148, 1188, 1230, 1273, 1318, 1365,
  1416, 1462, 1500, 1514, 1567, 1622, 1679, 1738, 1799, 1862,
  1928, 2035, 2107, 2181, 2257, 2336, 2418, 2503, 0  
};
uint8_t ctcss_encode_indx = 12;  // default 100.0 hz
uint8_t ctcss_decode_indx = 12;

// DCS code table 
uint16_t dcs_code[85] = {
 23,  25,  26,  31,  32,  43,  47,  51,  54,  65,
 71,  72,  73,  74, 114, 115, 116, 125, 131, 132,
134, 143, 152, 155, 156, 162, 165, 172, 174, 205,
223, 226, 243, 244, 245, 251, 261, 263, 265, 266,
271, 306, 311, 315, 331, 343, 346, 351, 364, 365,
371, 411, 412, 413, 423, 431, 432, 445, 464, 465,
466, 503, 506, 516, 532, 546, 565, 606, 612, 624,
627, 631, 632, 654, 662, 664, 703, 712, 723, 731,
732, 734, 743, 754, 0 
};
/* Alternate 104 code table
uint16_t dcs_code[104] = {
 23,  25,  26,  31,  32,  36,  43,  47,  51,  53,
 54,  65,  71,  72,  73,  74, 114, 115, 116, 122,
125, 131, 132, 134, 143, 145, 152, 155, 156, 162,
165, 172, 174, 205, 212, 223, 225, 226, 243, 244,
245, 246, 251, 252, 255, 261, 263, 265, 266, 271,
274, 306, 311, 315, 325, 331, 332, 343, 346, 351,
356, 364, 365, 371, 411, 412, 413, 423, 431, 432,
445, 446, 452, 454, 455, 462, 464, 465, 466, 503,
506, 516, 523, 526, 532, 546, 565, 606, 612, 624,
627, 631, 632, 654, 662, 664, 703, 712, 723, 731,
732, 734, 743, 754, 0 
};
*/
uint8_t dcs_encode_indx = 1; 
uint8_t dcs_decode_indx = 1;

// menu page info
// item number, text, x, y, box_x, box_y, 
typedef struct { 
  int item_num;
  int pos_x;
  int pos_y;
  char *text;
  int font;
  int box_x;
  int box_y;
  int box_width;
  int box_height;
} subMenu_item;

subMenu_item chanMenu[17] = {
  { 0, 20, 100, "Rx frequency = ", 2, 390, 95, 360, 75 },
  { 1, 20, 170, "Step",         3, 190, 165, 190, 50 },
  { 2, 20, 210, "Offset",       3, 190, 205, 190, 50 },
  { 3, 20, 250, "Mode",         3, 190, 245, 190, 50 },
  { 4, 20, 300, "Call type",    3, 190, 295, 190, 50 },  
  { 5, 20, 340, "Color",        3, 190, 335, 190, 50 },
  { 6, 20, 380, "Slot",         3, 190, 375, 190, 50 },
  { 7, 20, 420, "Contact id",   3, 190, 415, 190, 50 },
  { 8, 20, 300, "Squelch type", 3, 270, 295, 190, 50 },
  { 9, 20, 345, "CTCSS encode", 3, 270, 340, 190, 50 },
  {10, 20, 385, "CTCSS decode", 3, 270, 380, 190, 50 },
  {11, 20, 345, "DCS encode",   3, 270, 340, 190, 50 },
  {12, 20, 385, "DCS decode",   3, 270, 380, 190, 50 },
  {13, 20,  90, "Start freq",   3, 190, 85,  300, 50 },
  {14, 20, 130, "End freq",     3, 190, 125, 300, 50 },
  {15, 490, 220, "Scan",        3, 490, 255, 300, 50 },
  {16, 500, 300, "Activate",    3, 490, 295, 250, 50 }, 
};
// number of menu items, ordered list of the menu 
typedef struct { 
  uint8_t num;
  uint8_t seq[16];
} menu_sequence;


// sequences use for the different menus
menu_sequence  chanMenuSequence[12] = {
  {  8, {0, 1, 2, 3, 4, 5, 6, 7} },          // vfo DMR
  {  5, {0, 1, 2, 3, 8} },                   // vfo analog no squelch
  {  7, {0, 1, 2, 3, 8, 9, 10} },            // vfo analog CTCSS
  {  7, {0, 1, 2, 3, 8, 11, 12} },           // vfo analog DCS -DCS
  { 10, {13, 14, 1, 2, 3, 4, 5, 6, 7, 15} }, // EHscan DMR
  {  7, {13, 14, 1, 2, 3, 8, 15} },          // EHscan analog no squelch
  {  9, {13, 14, 1, 2, 3, 8, 9, 10, 15} },   // EHscan analog CTCSS
  {  9, {13, 14, 1, 2, 3, 8, 11, 12, 15} },  // EHscan analog DCS -DCS
  {  9, {0, 1, 2, 3, 4, 5, 6, 7, 16} },      // EHchan DMR
  {  6, {0, 1, 2, 3, 8, 16} },               // EHchan analog no squelch
  {  8, {0, 1, 2, 3, 8, 9, 10, 16} },        // EHchan analog CTCSS
  {  8, {0, 1, 2, 3, 8, 11, 12, 16} },       // EHchan analog DCS -DCS  
};

menu_sequence *current_menu_sequence;    // pointer to the current menu
uint8_t current_item_index = 1;          // index to the current item in the menu

// enum item names to make table index clear / modifiable
enum chanMenuItems { RXFREQ, STEP, OFFSET, MODE, CALLTP, COLOR, SLOT, TXID,
                      SQUEL, CTCEN, CTCDE, DCSEN, DCSDE, STFQ, ENDFQ, SCAN,
                      ACTIV };
// enum squelch types
enum vfoSquelch { NOSQUEL, CTCSS, DCS, INVDCS };  // original

// step sizes for frequency adjustment
// frequencies are in hz.
uint32_t step_size_table[16] = {
10, 25, 50, 100, 250, 500, 1000, 5000, 10000, 25000, 50000,
100000, 250000, 500000, 1000000, 5000000  };

// test strings for call type (8 bytes each)
char* dmr_call_type_name[3] = { "Private ", "Group   ", "All call" };

// standard offsets, 0,  +_ 600 Khz, +- 5 Mhz
// offset frequencies are in hz. / 10
int offset_table[5] = { 500000, 60000, 0, -60000, -500000 };


// =============================================================================
//
//  display a menu item from the table

void display_chan_menuitem( int item_num )
{
  int n = strlen(chanMenu[item_num].text);
  strncpy( (char*) cg_line_buffer, chanMenu[item_num].text, n);
  ssd1963_cg_write_line(n, chanMenu[item_num].font,
                        chanMenu[item_num].pos_x, chanMenu[item_num].pos_y, 3);
  
}
// =============================================================================
//
//  draw or clear a selector box
//  onoff
//
// History: 01/04/2019
//          01/10/2019 added color change for "selected" frame

// color is 0 for background, 1 for a highlight box,
//   any other palette color can be used for the selected box

static void display_item_selector( int item_num, int color )
{
  switch( color ) 
  {
  case 0:  // to background color
    {
      draw_frame( chanMenu[item_num].box_x, chanMenu[item_num].box_y,
                chanMenu[item_num].box_width, chanMenu[item_num].box_height, 3,    
                cg_background);       
      break;
    }
  case 1:  // to highlight color
    {
      draw_frame( chanMenu[item_num].box_x, chanMenu[item_num].box_y,
                chanMenu[item_num].box_width, chanMenu[item_num].box_height, 3,
                highlight);       
      break;
    }
  default: 
    {
      if(color > 15 ) return;  // a bad color number
      draw_frame( chanMenu[item_num].box_x, chanMenu[item_num].box_y,
                chanMenu[item_num].box_width, chanMenu[item_num].box_height, 3,
                current_palette[ color ] );
      break;
    }
  }
}
// =============================================================================
//
// clear a menu item box and display a float number in the box with 4.1 format
//  the number is displayed with "= " in front of it 
//  for the vfo munu screen only, uses the chanMenu table 
//
//
// History: 01/15/2019 written to reduce the clutter in the screen process
//
void display_float_item( int menu_item, float value )
{
  int n;
  // clear previous data in the box
  clear_rectangle( chanMenu[menu_item].box_x + 5, chanMenu[menu_item].box_y + 5,
                  chanMenu[menu_item].box_width - 10, chanMenu[menu_item].box_height - 10);
  // display new value
  n = sprintf((char*) cg_line_buffer, "= %4.1f", value );
  ssd1963_cg_write_line( n+1, chanMenu[menu_item].font, chanMenu[menu_item].box_x + 10,
                              chanMenu[menu_item].box_y + 5, 3); 
}
// =============================================================================
//
// clear a menu item box and display an integer number in the box
//  the number is displayed with "= " in front of it
//  for the vfo munu screen only, uses the chanMenu table 
//
// History: 01/15/2019 written to reduce the clutter in the screen process
//
void display_integer_item( int menu_item, int value )
{
  int n;
  // clear previous data in the box
  clear_rectangle( chanMenu[menu_item].box_x + 5, chanMenu[menu_item].box_y + 5,
                  chanMenu[menu_item].box_width - 10, chanMenu[menu_item].box_height - 10);
  // display new value
  n = sprintf((char*) cg_line_buffer, "= %i", value );
  ssd1963_cg_write_line( n+1, chanMenu[menu_item].font, chanMenu[menu_item].box_x + 10,
                              chanMenu[menu_item].box_y + 5, 3); 
}
// =============================================================================
//
//  display a frequency with engineering units text
//
//  hz, Khz, Mhz 
//
//  At an x,y location with font, and x,y for units 
//

void display_freq_units( int value,
                    uint16_t x_pos,  uint16_t y_pos,
                    uint16_t x_upos, uint16_t y_upos,
                    uint8_t font )
{
  int n;
  float eng_val;
  uint8_t units_exp = 0;
  // convert for engineering units
  eng_val = value;
  units_exp = 0;
  if( eng_val > 999  || eng_val < -999 )
  {
    // Khz
    eng_val = eng_val / 1000;
    units_exp = 3;
  }          
  // first time it will be reduced /1000
  // do again for Mhz
  if( eng_val > 999  || eng_val < -999 )
  {
    // Mhz
    eng_val = eng_val / 1000;
    units_exp = 6;            
  }
  // show Units txt
  switch ( units_exp )
  {
  case 0: // hz
    {
      strncpy( (char*) cg_line_buffer, "hz ", 3);
      break;
    }
  case 3: // Khz
    {
      strncpy( (char*) cg_line_buffer, "Khz", 3);
      break;
    }    
  case 6: // Mhz
    {
      strncpy( (char*) cg_line_buffer, "Mhz", 3);
      break;
    }
  default: break;
  }

  // clear previous units
  clear_rectangle( x_upos, y_upos, 70, 40);
  // display units
  ssd1963_cg_write_line(3, font, x_upos, y_upos, 3);
  
  // clear previous data in the box
  clear_rectangle( x_pos, y_pos, x_upos - x_pos, 40);
  // display new value
  n = sprintf((char*) cg_line_buffer, "= %3.1f", eng_val );
  ssd1963_cg_write_line( n+1, font, x_pos, y_pos, 3);   
  
}

// =============================================================================
//
//  display a frequency with engineering units text
//
//  hz, Khz, Mhz 
//
//  On a channel menu item
//

void display_chan_freq_units( int menu_item, int value )
{
  int n;
  float eng_val;
  uint8_t units_exp = 0;
  // convert for engineering units
  eng_val = value;
  units_exp = 0;
  if( eng_val > 999  || eng_val < -999 )
  {
    // Khz
    eng_val = eng_val / 1000;
    units_exp = 3;
  }          
  // first time it will be reduced /1000
  // do again for Mhz
  if( eng_val > 999  || eng_val < -999 )
  {
    // Mhz
    eng_val = eng_val / 1000;
    units_exp = 6;            
  }
  // show Units txt
  switch ( units_exp )
  {
  case 0: // hz
    {
      strncpy( (char*) cg_line_buffer, "hz ", 3);
      break;
    }
  case 3: // Khz
    {
      strncpy( (char*) cg_line_buffer, "Khz", 3);
      break;
    }    
  case 6: // Mhz
    {
      strncpy( (char*) cg_line_buffer, "Mhz", 3);
      break;
    }
  default: break;
  }

  // clear previous units
  clear_rectangle( chanMenu[menu_item].box_x + chanMenu[menu_item].box_width + 10,
                  chanMenu[menu_item].box_y + 5, 80,
                  chanMenu[menu_item].box_height - 10);
  // display units
  ssd1963_cg_write_line(3, chanMenu[menu_item].font,
             chanMenu[menu_item].box_x + chanMenu[menu_item].box_width + 10,
             chanMenu[menu_item].box_y + 5, 3);
  
  // clear previous data in the box
  clear_rectangle( chanMenu[menu_item].box_x + 5, chanMenu[menu_item].box_y + 5,
                  chanMenu[menu_item].box_width - 10, chanMenu[menu_item].box_height - 10);
  // display new value
  n = sprintf((char*) cg_line_buffer, "= %3.1f", eng_val );
  ssd1963_cg_write_line( n+1, chanMenu[menu_item].font, chanMenu[menu_item].box_x + 10,
                              chanMenu[menu_item].box_y + 5, 3);   
  
}

// =============================================================================
//
// clear a menu item box and display a text string in the box
//  for the vfo munu screen only, uses the chanMenu table 
//
// History: 01/15/2019 written to reduce the clutter in the screen process
//
void display_text_item( int menu_item, char* text, int num_chars )
{
  // clear previous
  clear_rectangle( chanMenu[menu_item].box_x + 5, chanMenu[menu_item].box_y + 5,
                   chanMenu[menu_item].box_width - 10, chanMenu[menu_item].box_height - 10);    
            
  strncpy( (char*) cg_line_buffer, text, num_chars);
  ssd1963_cg_write_line(num_chars, chanMenu[menu_item].font, chanMenu[menu_item].box_x + 10,
                        chanMenu[menu_item].box_y + 5, 3);
}
// =============================================================================
//
//  Frequency in range check
// 
//  checks keypad entry of frequency
//  pops up message that frequency is out of range if it is
//  returns a 1 if in range, 0 if not

// need to move these to radio info header, make dual band ranges and check
uint32_t radio_max_freq = 500000000;
uint32_t radio_min_freq = 100000000;

uint8_t check_freq_inrange( uint32_t freq )
{
  uint8_t in_range = 0;
  if ((freq < radio_min_freq) || (freq > radio_max_freq))
  {
    display_message(" Frequency out of range", 20, 390, 3);   
    memcpy( user_input_buffer, "         ", 10);
    user_input_buffer[10] = '\0';
    in_range = 0;
  }
  else in_range = 1;
  return in_range;
}
// =============================================================================
//
//  VFO mode screen
//
// History: 01/18/2019

uint8_t vfo_state = 0; // vfo state

float ctcss_freq = 0; // temp variable for true freq (table value / 10)

void process_vfo_mode( uint8_t nav_key )
{
  switch( vfo_state )
  {
  case 0:   // starting 
    {
      push_menu_stack( VFOMODE ); 
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0; 
      // display title
      strncpy( (char*) cg_line_buffer, "VFO mode", 8);
      ssd1963_cg_write_line(8, 2, 300 ,20, 3);
      log_event( "VFO mode" );         
      vfo_state++;
      break;
    }
  case 1: // pass through to screen handler
    {
      process_channel_screen( nav_key );
      // if it returns with flag to send
      if( chansc_flag == 1 ) vfo_state = 2;
      // if it returns with flag to exit
      if( chansc_flag == 2 ) vfo_state = 3;
      break;      
    }
  case 2: // send command 
    {
      tx_freq = rx_freq + txrx_offset; 
      // send frequency command to radio
      if( chansc_channel_type )
      {
        // set digital monitor off
        cs_channel_monitor = 0;
        set_temp_ACH( tx_freq, rx_freq ); // analog
      }
      else
      {
        // set digital monitor on
        cs_channel_monitor = 1;
        set_temp_DCH( tx_freq, rx_freq ); // DMR
      }

       RxFreq = rx_freq;
       TxFreq = tx_freq;
        
      // back to screen handler
      vfo_state = 1;
      break;
    }
  case 3: // exit  
    {
      // clear the page
      clear_rectangle( 0,0, 800, 415);
      // clear extra space at the bottom      
      clear_rectangle( chanMenu[TXID].pos_x, chanMenu[TXID].pos_y - 5, 360, 80);
      // redraw previous sub menu, with vfo menu selected
      display_sub_menu( 0, 3, 460, 3 );
      // reset box width for this menu
      select_box_width = 460; 
      vfo_state = 0;  // set for restart
      pop_menu_stack();
      return;    
      break;
    }

  default: break;
  }
}
// =============================================================================
//
//   Set menu sequence for current settings
//     use this to set the meun sequence for the correct channel page
//      EnhancedChannel, vfo, EnhancedScan
//    and whenever a setting changes that requires a different menu
//
//    There are three levels of settings that change the menus:   
//    page, mode, squelch type
//
// History: 04/02/2019 written

uint8_t chan_squelch_type = 0;

//
void set_chanmenu( void )
{
  //  page
  if( get_menu_stack() == ENHSCAN )
  {
    // mode 
    if( chansc_channel_type ) // Analog mode ==1
    {
      // squelch type
      switch( chan_squelch_type )
      {
      case NOSQUEL:
        {
        current_menu_sequence = &chanMenuSequence[5];         
        break;    
        }
      case CTCSS:
        {
        current_menu_sequence = &chanMenuSequence[6]; 
        break;             
        }
      case DCS:
      case INVDCS:
        {
        current_menu_sequence = &chanMenuSequence[7];      
        break;             
        }          
      default: break;
      }
    }
    else  // DMR == 0
    {
      current_menu_sequence = &chanMenuSequence[4];        
    }
  }
  if( get_menu_stack() == VFOMODE )
  {
    // mode 
    if( chansc_channel_type ) // Analog mode ==1
    {
      // squelch type
      switch( chan_squelch_type )
      {
      case NOSQUEL:
        {
        current_menu_sequence = &chanMenuSequence[1];         
        break;    
        }
      case CTCSS:
        {
        current_menu_sequence = &chanMenuSequence[2]; 
        break;             
        }
      case DCS:
      case INVDCS:
        {
        current_menu_sequence = &chanMenuSequence[3];      
        break;             
        }          
      default: break;
      }
    }
    else  // DMR == 0
    {
      current_menu_sequence = &chanMenuSequence[0];        
    }    
  }
  if( get_menu_stack() == ECHAN )
  {
    // mode 
    if( chansc_channel_type ) // Analog mode ==1
    {
      // squelch type
      switch( chan_squelch_type )
      {
      case NOSQUEL:
        {
        current_menu_sequence = &chanMenuSequence[9];         
        break;    
        }
      case CTCSS:
        {
        current_menu_sequence = &chanMenuSequence[10]; 
        break;             
        }
      case DCS:
      case INVDCS:
        {
        current_menu_sequence = &chanMenuSequence[11];      
        break;             
        }          
      default: break;
      }
    }
    else  // DMR == 0
    {
      current_menu_sequence = &chanMenuSequence[8];        
    }    
  }
  
}
// =============================================================================
//
//   Increment a value and keep within the range supplied
//
//   wraps around to min if increment exceeds max 
//
// History: 04/02/2019 written

int inc_inrange( int val, int min, int max )
{
  if( val < max ) val++;
  else val = min;
  
  return val;
}
// =============================================================================
//
//   Deccrement a value and keep within the range supplied
//
//   wraps around to max if decrement is less than min 
//
// History: 04/02/2019 written

int dec_inrange( int val, int min, int max )
{
  if( val > min ) val--;
  else val = max;
  
  return val;
}

// =============================================================================
//
//  Channel info entry screen for:
//    VFO mode
//    Enhanced channel entry mode
 //   Enhanced scan mode
// 
// History: 01/17/2019 revised from process_vfo_screen to also be used for the
//                     enhanced channel screen.

uint8_t e_chan_state = 0;  

uint8_t chansc_state = 0; // channel screen state
uint8_t chansc_flag = 0;  // screen return flag
uint8_t prev_dtmf = 0;
uint8_t chan_active_item = 0;
uint8_t chan_prev_item = 0;
uint8_t chan_step_size = 6;   // table index start at 1 Khz
uint8_t chan_offset = 0;      // table index defaults at 0

uint8_t chansc_channel_type = 0;

void process_channel_screen( uint8_t nav_key )
{
  int n;
  int i;
  
  switch( chansc_state )
  {
  case 0:   // display page
    {
      // store channel type so that new packets don't change state
      // while in the menu 
      chansc_channel_type = cs_channel_type;
      chansc_flag = 0;

      // set menu sequence based on current settings
      set_chanmenu(); 
      // start at the first item of the list
      current_item_index = 0;
      
      if( get_menu_stack() == ENHSCAN )
      {
        display_chan_menuitem( STFQ );
        // show some = signs
        strncpy( ( char * ) cg_line_buffer, "= ", 2);
        ssd1963_cg_write_line( 2, chanMenu[STFQ].font, chanMenu[STFQ].box_x + 10,
                             chanMenu[STFQ].box_y + 8, 3);          
        ssd1963_cg_write_line( 2, chanMenu[ENDFQ].font, chanMenu[ENDFQ].box_x + 10,
                             chanMenu[ENDFQ].box_y + 8, 3); 
        // display start frequency
        display_frequency( scan_start_freq, chanMenu[ STFQ ].box_x + 60,
                          chanMenu[ STFQ ].box_y + 8, chanMenu[ STFQ ].font);
        display_chan_menuitem( ENDFQ );
        // display end frequency
        display_frequency( scan_end_freq, chanMenu[ ENDFQ ].box_x + 60,
                          chanMenu[ ENDFQ ].box_y + 8, chanMenu[ ENDFQ ].font);
        chan_active_item = STFQ;
        chan_prev_item = STFQ;
        
        scanning_freq = scan_start_freq;

        display_chan_menuitem( SCAN );
        clear_rectangle( chanMenu[ SCAN ].box_x + 60, chanMenu[ SCAN ].box_y + 8, 235, 36 );          
        display_frequency( scanning_freq, chanMenu[ SCAN ].box_x + 60,
                       chanMenu[ SCAN ].box_y + 8, chanMenu[ SCAN ].font);        
       
        // draw selector box
        display_item_selector( STFQ, 1 );
      }
      else   // vfo mode and enhanced channel
      {
        display_chan_menuitem( RXFREQ );
        if( cs_channel_rx_frequency > 0 ) rx_freq = cs_channel_rx_frequency;   
        else rx_freq = 10000000;
        if( rx_freq < 10000000 ) rx_freq *= 10;
        // display frequency of current channel RX
        display_frequency( rx_freq,  chanMenu[RXFREQ].box_x + 10,
                            chanMenu[RXFREQ].box_y + 5, chanMenu[RXFREQ].font ); 
        chan_active_item = 0;
        chan_prev_item = 0;
        // draw selector box
        display_item_selector( RXFREQ, 1 );
      } 
      // Step
      display_chan_menuitem( STEP );
      display_chan_freq_units( STEP, freq_step * 10 );   
      
      //n = sprintf((char*) cg_line_buffer, "= %i", freq_step * 10);
      //ssd1963_cg_write_line( n+1, chanMenu[STEP].font, chanMenu[STEP].box_x + 10,
      //                       chanMenu[STEP].box_y + 5, 3);     
      // Offset
      display_chan_menuitem( OFFSET );
      // calc offset from current channel 
      txrx_offset = cs_channel_tx_frequency - cs_channel_rx_frequency;
      chan_offset = 0;
      // get table index
      for( i = 0; i < 5; i++)
      {
        if( offset_table[i] == txrx_offset ) chan_offset = i; 
      }
      // if none matched, it's not standard, but will show the tx-rx difference
      display_chan_freq_units( OFFSET, txrx_offset * 10 );  
 
      //n = sprintf((char*) cg_line_buffer, "= %i", txrx_offset * 10);
      //ssd1963_cg_write_line( n+1, chanMenu[OFFSET].font, chanMenu[OFFSET].box_x + 10,
      //                       chanMenu[OFFSET].box_y + 5, 3);   
      
      // Mode
      display_chan_menuitem( MODE );         
      
      if( chansc_channel_type ) // Analog mode ==1
      {
        display_text_item( MODE, "= Analog", 8);
        // Squelch type
        display_chan_menuitem( SQUEL );
        
        display_squelch_type( chan_squelch_type, chanMenu[SQUEL].box_x + 10,
                              chanMenu[SQUEL].box_y + 5, chanMenu[SQUEL].font );
      }
      else    // DMR mode ==0
      {
        current_menu_sequence = &chanMenuSequence[0];   
        
        display_text_item( MODE, "= DMR", 5);
        // Call type
        display_chan_menuitem( CALLTP );
        strncpy( temp_text, "= ", 2);
        strncpy( temp_text + 2, dmr_call_type_name[cs_call_type], 8);
        display_text_item( CALLTP, temp_text, 10 );        
        // Color
        display_chan_menuitem( COLOR );
        display_integer_item( COLOR, cs_rx_color );        
        // Slot
        display_chan_menuitem( SLOT );
        display_integer_item( SLOT, dmr_slot + 1 );        
        // Contact ID
        display_chan_menuitem( TXID );
        display_integer_item( TXID, tx_contact_num );
      }
      // other options, Step size, offset, mode  
      select_box_width = 330;

      // display activate button in echan mode
      if( get_menu_stack() == ECHAN )
      {
        // put a frame around it
        draw_shaded_frame( 485, 290, 260, 60, 5, border, highlight);        
        display_chan_menuitem( ACTIV );        
      }
      
      // draw nav buttons
      draw_navbuttons( 2 );

      if( get_menu_stack() == VFOMODE )
      {
        // draw knob
        draw_icon_mirrored( ICON_Big_Knob, 500, 200, 1 );
        draw_dial_spin( 530, 230,  dial_rotation );      
      }
      last_key_press = 0;
      dtmf_code_flag = 0;
     
      chansc_state++;
      break;
    }
  case 1:  // select item
    {
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;
      
      // set menu sequence based on current settings
      set_chanmenu();       
      
      switch( nav_key )
      {
      case UP:
        {
          // change active item -
          if( current_item_index == 0 ) 
          {
            current_item_index = current_menu_sequence->num-1;
          }
          else current_item_index--;
          
          chan_active_item = current_menu_sequence->seq[ current_item_index ];
          
          // erase previous highlight box
          display_item_selector( chan_prev_item, 0 ); 
          // draw new highlight box
          display_item_selector( chan_active_item, 1 );
          chan_prev_item = chan_active_item;
          // stay in this state
          break;
        }
      case DOWN:
        {
          // change active item +
          if( current_item_index == current_menu_sequence->num-1 ) 
          {
            current_item_index = 0;
          }
          else current_item_index++;
          
          chan_active_item = current_menu_sequence->seq[ current_item_index ];
          
          // erase previous highlight box
          display_item_selector( chan_prev_item, 0 );           
          // draw new highlight box
          display_item_selector( chan_active_item, 1 );
          chan_prev_item = chan_active_item;          
          // stay in this state          
          break;
        }
      case SELECT:
        {
          // show selected item frame
          display_item_selector( chan_active_item, 4 );   
          // goto state for the selected item
          switch( chan_active_item )
          {
          case 0: // frequency
            {
              chansc_state = 2;
              // activate touch points for the knob
              for(i = 35; i < 59; i++) CS800D_touch_point[i].is_active = 1; 
              break;
            }
          case 1:  chansc_state = 8;  break;  // step
          case 2:  chansc_state = 9;  break;  // offset
          case 3:  chansc_state = 10; break;  // mode
          case 4:  chansc_state = 11; break;  // call type
          case 5:  chansc_state = 12; break;  // color
          case 6:  chansc_state = 13; break;  // slot
          case 7:  chansc_state = 14; break;  // contact id
          case 8:  chansc_state = 15; break;  // squelch type
          case 9:  chansc_state = 16; break;  // ctcss encode
          case 10: chansc_state = 17; break;  // ctcss decode
          case 11: chansc_state = 20; break;  // dcs encode
          case 12: chansc_state = 21; break;  // dcs decode
          case 13: chansc_state = 22; break;  // scan start freq
          case 14: chansc_state = 24; break;  // scan end freq
          case 15: chansc_state = 26; break;  // scan start/stop 
          case 16: chansc_state = 27; break;  // activate enhanced channel 
          default: break;
          }
          break;
        }
      case CANCEL:
        {
          // deactivate touch points for the knob
          for(i=35; i<59; i++) CS800D_touch_point[i].is_active = 0;
          // clear selector box
          display_item_selector( chan_active_item, 0 ); 
          // set exit flag for return
          if( get_menu_stack() == ECHAN ) chansc_flag = 3;
          else chansc_flag = 2;
          chansc_state = 0;  // set for restart
          break;
        }
      default: break;
      }
      break;
    }
  case 2:  // frequency is selected 
    {
     // if a number key is pressed, go to frequency entry
      if (dtmf_code_flag)
      {
        // get numeric input 
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          prev_dtmf = dtmf_code_char; 
          chansc_state = 3; 
          break;
        }
      }
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;

      switch( nav_key )
      {
      case DOWN: // - rotation for down
        {
          // decrease frequency by step
          rx_freq -= freq_step;
          tx_freq = rx_freq + txrx_offset;          
          // redisplay freq
          // clear old data
          clear_rectangle( 400, 100, 292, 65 );
          display_frequency( rx_freq, 400, 100, 2 );
          if( get_menu_stack() == VFOMODE )
          {
            if( dial_rotation > 0) dial_rotation--;
            else dial_rotation = 23;            
            draw_dial_spin( 530, 230,  dial_rotation );
            // send frequency command to radio
            if( chansc_channel_type )
            {
              // set digital monitor off
              cs_channel_monitor = 0;              
              set_temp_ACH( tx_freq, rx_freq ); // analog
            }
            else
            {
              // set digital monitor on
              cs_channel_monitor = 1;              
              set_temp_DCH( tx_freq, rx_freq ); // DMR 
            }
          }
          // get rssi test
          //request_RSSI_value();
          
          // stay in this state
          break;
        }
      case UP: // + rotation for up
        {
          // increase frequency by step
          rx_freq += freq_step;
          tx_freq = rx_freq + txrx_offset;
          // redisplay freq
          // clear old data
          clear_rectangle( 400, 100, 292, 65 );
          display_frequency( rx_freq, 400, 100, 2 );
          if( get_menu_stack() == VFOMODE )
          {
            dial_rotation = inc_inrange( dial_rotation, 0, 23 );
            draw_dial_spin( 530, 230,  dial_rotation );            
            // send frequency command to radio
            if( chansc_channel_type )
            {
              // set digital monitor off 
              cs_channel_monitor = 0;
              set_temp_ACH( tx_freq, rx_freq ); // analog
            }
            else
            {
              // set digital monitor on
              cs_channel_monitor = 1;
              set_temp_DCH( tx_freq, rx_freq ); // DMR
            }
          }          
          // get rssi test
          // request_RSSI_value();
          
          // stay in this state
          break;
        }      
      case SELECT:
        {
          // check frequency
          if( check_freq_inrange( rx_freq*10 ) == 0)
          {
            // not in range,  abort
            chansc_state = 0;
            break;                 
          }
          else
          {
            // clear buffer, space for frequency
            memcpy( user_input_buffer, "          ", 10);
            user_input_buffer[10] = '\0'; 
            chansc_state = 6;
          }
          break;
        }
      case CANCEL:
        {
          // remove selected highlight
          display_item_selector( chan_active_item, 1 ); 
          chansc_state = 1;  // back to previous state
          break;
        }        
      default: break;
      }
      break;
    }
  case 3:  // prepare to enter frequency 
    {
      // clear current 
      clear_rectangle( 400, 100, 310, 65);
   
      // clear buffer, space for frequency
      memcpy( user_input_buffer, "          ", 10);
      user_input_buffer[10] = '\0';      
      chansc_state++;
      break;
    }
  case 4:  // get mic numeric frequency
    {
      cursor_blink( 640, 100 + 62, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          get_number_input( 440, 100, 2, 9); 
          break;
        }
        // otherwise it could be a valid dtmf navigation key
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // clear frequency
        clear_rectangle( 400, 100, 310, 65 );
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "         ", 9) == 0)
        {
          display_frequency( rx_freq, 400, 100, 2 ); 
          chansc_state = 7;  // jump to exit
          break; 
        }              
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] ='\0';
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. If in range, store data and 
      // advance to next state.

      if (nav_key == SELECT)
      {
        // remove leading spaces
        n = 0;
        while( user_input_buffer[n] == ' ') n++;
        // n is location of first digit
        strncpy( user_input_buffer, user_input_buffer + n, 16); 
        
        // frequency is in the user input buffer
        // for user input frequency is all 9 digits
        rx_freq = atoi( user_input_buffer );
        if( rx_freq > 0)   // can hang if zero
        {
          // multiply frequency to add zeros to end
          while(rx_freq < 100000000) rx_freq *= 10;
        }
        
        // check frequency
        if( check_freq_inrange( rx_freq ) == 0) break;

        // frequency needs to be reduced before sending to the radio
        rx_freq /= 10;        
        // redisplay frequency
        clear_rectangle( 400, 100, 300, 65 );
        display_frequency( rx_freq, 400, 100, 2 );          

        // clear buffer, space for frequency
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] = '\0'; 
        chansc_state++;
        break;
      }
      break;
    }
  case 5:  // get frequency 
    {
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        chansc_state = 7;  // jump to exit without sending command
        break;
      }
      // If a SELECT key 
      if (nav_key == SELECT)
      {
        // clear frequency
        clear_rectangle( 400, 100, 300, 65 );
        display_frequency( rx_freq, 400, 100, 2 );         
        chansc_state++;
        break;
      }
      break;
    }
  case 6:   // done entering,  send frequency to radio moved to activate button
    {
      if( get_menu_stack() == ECHAN )
      {
        // remove selected highlight
        display_item_selector( chan_active_item, 1 ); 
        chansc_state = 1;  // back to menu state
        break;
      }
      //else
      tx_freq = rx_freq + txrx_offset;      
      // send frequency command to radio
      if( chansc_channel_type )
      {
        // set digital monitor off
        cs_channel_monitor = 0;        
        set_temp_ACH( tx_freq, rx_freq ); // analog
      }
      else
      {
        // set digital monitor on
        cs_channel_monitor = 1;        
        set_temp_DCH( tx_freq, rx_freq ); // DMR 
      }

      chansc_flag = 0;      
      chansc_state = 0;
      
      vfo_state = 0;
      e_chan_state = 0;
      pop_menu_stack();
      menu_exit = 1;   // force return to main screen
      break;
      //return;
    }
  case 7: // return to previous menu - separate state for cancel bail out
    {
      chansc_flag = 2;
      chansc_state = 0;
      break;
    }
  case 8: // change step size
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          // increase step
          chan_step_size = inc_inrange( chan_step_size, 0, 15 );          
          freq_step = step_size_table[ chan_step_size ] / 10;
          // convert for engineering units
          // clear and display
          display_chan_freq_units( STEP, step_size_table[ chan_step_size ] );
          break;
        }
      case DOWN:
        {
          // decrease step
          chan_step_size = dec_inrange( chan_step_size, 0, 15 );            
          freq_step = step_size_table[ chan_step_size ] / 10;
          // convert for engineering units
          // clear and display
          display_chan_freq_units( STEP, step_size_table[ chan_step_size ] );
          
          break;
        }
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;            
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
      break;
    }
 case 9: // change offset
    {
      // get numeric input for other offsets
      
      // todo: ...
      
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          // increase offset
          chan_offset = inc_inrange( chan_offset, 0, 4 );          
          txrx_offset = offset_table[ chan_offset ];
          // convert for engineering units
          // clear and display
          display_chan_freq_units( OFFSET, offset_table[ chan_offset ] *10);          
          // clear and display
          // display_integer_item( OFFSET, txrx_offset * 10 );
          break;
        }
      case DOWN:
        {
          // decrease offset
          chan_offset = dec_inrange( chan_offset, 0, 4 );          
          txrx_offset = offset_table[ chan_offset ];
          // convert for engineering units
          // clear and display
          display_chan_freq_units( OFFSET, offset_table[ chan_offset ] * 10 );           
          // clear and display
          //display_integer_item( OFFSET, txrx_offset * 10 );
          break;
        }
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
      break;
    }
 case 10: // change mode
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
      case DOWN:  
        {
          if(  !chansc_channel_type ) // in DMR mode ==0
          {
            //current_menu_sequence = &chanMenuSequence[3];              
            
            // change to analog mode
            channel_type = 1;
            chansc_channel_type = 1;
            
            // clear and display
            display_text_item( MODE, "= Analog", 8 );           
            // clear previous info
            clear_rectangle( chanMenu[CALLTP].pos_x,  chanMenu[CALLTP].pos_y, 400, 120);
            // clear extra space at the bottom       
            clear_rectangle( chanMenu[TXID].pos_x, chanMenu[TXID].pos_y - 5,360, 80);
            // Squelch type
            display_chan_menuitem( SQUEL );
            display_squelch_type( chan_squelch_type, chanMenu[SQUEL].box_x + 10,
                              chanMenu[SQUEL].box_y + 5, chanMenu[SQUEL].font );
          }
          else    // in Analog mode ==1
          {
            // change to dmr mode
            channel_type = 0;
            chansc_channel_type = 0;

            // clear and display
            display_text_item( MODE, "= DMR", 5 );   
            // clear previous analog info
            clear_rectangle( chanMenu[ CALLTP ].pos_x,  chanMenu[ CALLTP ].pos_y, 400, 120); 

            // Call type
            display_chan_menuitem( CALLTP );
            strncpy( temp_text, "= ", 2);
            strncpy( temp_text + 2, dmr_call_type_name[cs_call_type], 8);
            display_text_item( CALLTP, temp_text, 10 );
            // Color
            display_chan_menuitem( COLOR );
            display_integer_item( COLOR, cs_rx_color );
            // Slot
            display_chan_menuitem( SLOT );
            display_integer_item( SLOT, dmr_slot + 1 );
            // Contact ID
            display_chan_menuitem( TXID );
            display_integer_item( TXID, tx_contact_num );
          }
          break;
        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }
 case 11: // change call type
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        { // increase call_type
          cs_call_type = inc_inrange( cs_call_type, 0, 2 );
          // display new call type
          strncpy( temp_text, "= ", 2);
          strncpy( temp_text + 2, dmr_call_type_name[cs_call_type], 8);
          display_text_item( CALLTP, temp_text, 10 );
          break;
        }
      case DOWN:
        { // decrease call_type
          cs_call_type = dec_inrange( cs_call_type, 0, 2 );          
          // display new call type
          strncpy( temp_text, "= ", 2);
          strncpy( temp_text + 2, dmr_call_type_name[cs_call_type], 8);
          display_text_item( CALLTP, temp_text, 10 );
          break;
        }
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    }    
 case 12: // change color
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          // increase color
          dmr_color = inc_inrange( dmr_color, 0, 15 );          
          display_integer_item( COLOR, dmr_color );              
          break;
        }
      case DOWN:  
        {
          // decrease color
          dmr_color = dec_inrange( dmr_color, 0, 15 );           
          display_integer_item( COLOR, dmr_color );    
          break;
        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        {
          // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }    
 case 13: // slot
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
      case DOWN:  
        {
          if( dmr_slot ) dmr_slot = 0;
          else dmr_slot = 1;
          // clear and display
          display_integer_item( SLOT, dmr_slot + 1 );    
          break;
        }
      case SELECT:
        {
          // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        {
          // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }
 case 14: // Contact Id
    {
      // when a number key is pressed, get contact id number
      if (dtmf_code_flag)
      {
        // get numeric input 
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          prev_dtmf = dtmf_code_char; 
          chansc_state = 18; 
          break;
        }
      }
      
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
      case DOWN:
        {
          break;
        }

      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    } 
 case 15: // Squelch type
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP: 
        {
          chan_squelch_type = dec_inrange( chan_squelch_type, 0, 3 );   
          display_squelch_type( chan_squelch_type, chanMenu[SQUEL].box_x + 10,
                              chanMenu[SQUEL].box_y + 5, chanMenu[SQUEL].font );
          break;
        }
      case DOWN:  
        {
          chan_squelch_type = inc_inrange( chan_squelch_type, 0, 3 );            
          display_squelch_type( chan_squelch_type, chanMenu[SQUEL].box_x + 10,
                              chanMenu[SQUEL].box_y + 5, chanMenu[SQUEL].font );
          break;
        }
      case SELECT:
        { 
          // the same for both tx & rx (for now)
          ctcss_decode_type = chan_squelch_type;
          ctcss_encode_type = chan_squelch_type;
          
          if( chan_squelch_type == 0) rx_sqlmode = 3;
          else rx_sqlmode = 2;
          
          // remove selected box
          display_item_selector( chan_active_item, 1 );

          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );          
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }    
 case 16: // CTCSS encode
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          ctcss_encode_indx = inc_inrange( ctcss_encode_indx, 0, 39 );             
          ctcss_encode = ctcss_code[ctcss_encode_indx];
          ctcss_freq = ctcss_encode / 10.0;
          // clear and display
          display_float_item( CTCEN, ctcss_freq ); 
          break;
        }
      case DOWN:  
        {
          ctcss_encode_indx = dec_inrange( ctcss_encode_indx, 0, 39 );                       
          ctcss_encode = ctcss_code[ctcss_encode_indx];
          ctcss_freq = ctcss_encode / 10.0;
          // clear and display
          display_float_item( CTCEN, ctcss_freq ); 
          break;        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }      
 case 17: // CTCSS decode
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          ctcss_decode_indx = inc_inrange( ctcss_decode_indx, 0, 40 );                       
          ctcss_decode = ctcss_code[ctcss_decode_indx];
          ctcss_freq = ctcss_decode / 10.0;
          // clear and display
          if( ctcss_decode == 0 )
          {
            // clear previous data in the box
            clear_rectangle( chanMenu[CTCDE].box_x + 5,
                             chanMenu[CTCDE].box_y + 5,
                             chanMenu[CTCDE].box_width - 10,
                             chanMenu[CTCDE].box_height - 10);
            
            strncpy( (char*) cg_line_buffer, "= none", 6);
            ssd1963_cg_write_line( 6, chanMenu[CTCDE].font,
                                      chanMenu[CTCDE].box_x + 10,
                                      chanMenu[CTCDE].box_y + 5, 3); 
          }
          else display_float_item( CTCDE, ctcss_freq );
          
          break;
        }
      case DOWN:  
        {
          ctcss_decode_indx = dec_inrange( ctcss_decode_indx, 0, 40 );            
          ctcss_decode = ctcss_code[ctcss_decode_indx];
          ctcss_freq = ctcss_decode / 10.0;
          // clear and display
          if( ctcss_decode == 0 )
          {
            // clear previous data in the box
            clear_rectangle( chanMenu[CTCDE].box_x + 5,
                             chanMenu[CTCDE].box_y + 5,
                             chanMenu[CTCDE].box_width - 10,
                             chanMenu[CTCDE].box_height - 10);
            
            strncpy( (char*) cg_line_buffer, "= none", 6);
            ssd1963_cg_write_line( 6, chanMenu[CTCDE].font,
                                      chanMenu[CTCDE].box_x + 10,
                                      chanMenu[CTCDE].box_y + 5, 3); 
          }
          else display_float_item( CTCDE, ctcss_freq );
          break;
        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }
 case 18: // enter contact ID
    { // clear current 
      clear_rectangle( chanMenu[TXID].box_x + 5, chanMenu[TXID].box_y + 5,
                       chanMenu[TXID].box_width - 10, chanMenu[TXID].box_height - 10);
   
      // clear buffer, space for contact id
      memcpy( user_input_buffer, "       ", 8);
      user_input_buffer[8] = '\0';      
      chansc_state++;
      break;
    }
 case 19:  // get mic tx contact
    {
      cursor_blink( chanMenu[TXID].pos_x + 330, chanMenu[TXID].pos_y + 35, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          get_number_input( chanMenu[TXID].box_x + 52, chanMenu[TXID].pos_y, 3, 7); 
          break;
        }
        // otherwise it could be a valid dtmf navigation key
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // clear contact id
        clear_rectangle( chanMenu[TXID].box_x + 5, chanMenu[TXID].box_y + 5,
                       chanMenu[TXID].box_width - 10, chanMenu[TXID].box_height - 10);
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "       ", 7) == 0)
        {
          n = sprintf((char*) cg_line_buffer, "= %i", tx_contact_num );        
          ssd1963_cg_write_line( n+1, chanMenu[TXID].font, chanMenu[TXID].box_x + 10,
                             chanMenu[TXID].box_y + 5, 3); 
          display_contact_id( tx_contact_num, chanMenu[TXID].pos_x,
                             chanMenu[TXID].pos_x, chanMenu[TXID].font ); 
          // clear cursor
          display_cursor( 0, chanMenu[TXID].pos_x + 330, chanMenu[TXID].pos_y + 35, 3 );
          // restore highlight box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;                
          break; 
        }              
        memcpy( user_input_buffer, "          ", 8);
        user_input_buffer[8] ='\0';
        break;
      }
      // If a SELECT key
      if (nav_key == SELECT)
      {
        // remove leading spaces
        n = 0;
        while( user_input_buffer[n] == ' ') n++;
        // n is location of first digit
        strncpy( user_input_buffer, user_input_buffer + n, 16); 
        
        // contact is in the user input buffer
        tx_contact_num = atoi( user_input_buffer );
        
        // remove selected box
        display_item_selector( chan_active_item, 1 );
        // clear cursor
        display_cursor( 0, chanMenu[TXID].pos_x + 330, chanMenu[TXID].pos_y + 35, 3 );
        
        // clear buffer, space for next
        memcpy( user_input_buffer, "          ", 8);
        user_input_buffer[8] = '\0'; 
        // go back to channel menu
        chansc_state = 1;
        
        break;
      }
      break;
    }
 case 20: // DCS encode
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          dcs_encode_indx = inc_inrange( dcs_encode_indx, 0, 83 );            
          dcs_encode = dcs_code[dcs_encode_indx];
          // clear and display
          if( chan_squelch_type == 3 ) display_integer_item( DCSEN, -dcs_encode );
          else display_integer_item( DCSEN, dcs_encode );
          break;
        }
      case DOWN:  
        {
          dcs_encode_indx = dec_inrange( dcs_encode_indx, 0, 83 );          
          dcs_encode = dcs_code[dcs_encode_indx];

          // clear and display
          if( chan_squelch_type == 3 ) display_integer_item( DCSEN, -dcs_encode );
          else display_integer_item( DCSEN, dcs_encode );
          break;
        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }
 case 21: // DCS decode
    {
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      switch( nav_key )
      {
      case UP:
        {
          dcs_decode_indx = inc_inrange( dcs_decode_indx, 0, 84 );          
          dcs_decode = dcs_code[dcs_decode_indx];
          // clear and display
          if(dcs_decode == 0 )
          {
            // clear previous data in the box
            clear_rectangle( chanMenu[DCSDE].box_x + 5,
                             chanMenu[DCSDE].box_y + 5,
                             chanMenu[DCSDE].box_width - 10,
                             chanMenu[DCSDE].box_height - 10);
            
            strncpy( (char*) cg_line_buffer, "= none", 6);
            ssd1963_cg_write_line( 6, chanMenu[DCSDE].font,
                                      chanMenu[DCSDE].box_x + 10,
                                      chanMenu[DCSDE].box_y + 5, 3);             
          }
          else
          {           
            if( chan_squelch_type == 3 ) display_integer_item( DCSDE, -dcs_decode );
            else display_integer_item( DCSDE, dcs_decode );      
          }       
          break;
        }
      case DOWN:  
        {
          dcs_decode_indx = dec_inrange( dcs_decode_indx, 0, 84 );   
          dcs_decode = dcs_code[dcs_decode_indx];
          // clear and display
          if(dcs_decode == 0 )
          {
            // clear previous data in the box
            clear_rectangle( chanMenu[DCSDE].box_x + 5,
                             chanMenu[DCSDE].box_y + 5,
                             chanMenu[DCSDE].box_width - 10,
                             chanMenu[DCSDE].box_height - 10);
            
            strncpy( (char*) cg_line_buffer, "= none", 6);
            ssd1963_cg_write_line( 6, chanMenu[DCSDE].font,
                                      chanMenu[DCSDE].box_x + 10,
                                      chanMenu[DCSDE].box_y + 5, 3);             
          }
          else
          {           
            if( chan_squelch_type == 3 ) display_integer_item( DCSDE, -dcs_decode );
            else display_integer_item( DCSDE, dcs_decode );      
          }
          break;
        }          
      case SELECT:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;          
          break;
        }
      case CANCEL:
        { // remove selected box
          display_item_selector( chan_active_item, 1 );
          // go back to channel menu
          chansc_state = 1;
          break;
        }
      default: break;
      }
    break;
    }
  case 22:   // start frequency for scanning
    {
     // if a number key is pressed, go to frequency entry
      if (dtmf_code_flag)
      {
        // get numeric input 
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          prev_dtmf = dtmf_code_char;
          // clear buffer, space for frequency
          // the user buffer must have spaces in the front so it will print
          memcpy( user_input_buffer, "          ", 10);
          user_input_buffer[10] = '\0';    // must have a null at the end        
          // clear frequency
          clear_rectangle( chanMenu[ STFQ ].box_x + 60,
                          chanMenu[ STFQ ].box_y + 8, 235, 36 );          
          chansc_state = 23; 
          break;
        }
      }
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;

      switch( nav_key )
      {
      case DOWN: // - rotation for down
        {
          // decrease frequency by step
          scan_start_freq -= freq_step*10;
          scanning_freq = scan_start_freq; 
          // redisplay freq
          clear_rectangle( chanMenu[ STFQ ].box_x + 60, chanMenu[ STFQ ].box_y + 8, 235, 36 );
          // redisplay start frequency
          display_frequency( scan_start_freq, chanMenu[ STFQ ].box_x + 60,
                       chanMenu[ STFQ ].box_y + 8, chanMenu[ STFQ ].font); 
          // redisplay scanning frequency
          clear_rectangle( chanMenu[ SCAN ].box_x + 60, chanMenu[ SCAN ].box_y + 8, 235, 36 );          
          display_frequency( scanning_freq, chanMenu[ SCAN ].box_x + 60,
                       chanMenu[ SCAN ].box_y + 8, chanMenu[ SCAN ].font);
          // stay in this state
          break;
        }
      case UP: // + rotation for up
        {
          // increase frequency by step
          scan_start_freq += freq_step*10;
          scanning_freq = scan_start_freq; 
          // redisplay freq
          // clear old data
          clear_rectangle( chanMenu[ STFQ ].box_x + 60, chanMenu[ STFQ ].box_y + 8, 235, 36 );
          // re display start frequency
          display_frequency( scan_start_freq, chanMenu[ STFQ ].box_x + 60,
                       chanMenu[ STFQ ].box_y + 8, chanMenu[ STFQ ].font);
          // redisplay scanning frequency
          clear_rectangle( chanMenu[ SCAN ].box_x + 60, chanMenu[ SCAN ].box_y + 8, 235, 36 );          
          display_frequency( scanning_freq, chanMenu[ SCAN ].box_x + 60,
                       chanMenu[ SCAN ].box_y + 8, chanMenu[ SCAN ].font);          
          // stay in this state
          break;
        }      
      case SELECT:
        {
          // check frequency, display error box
          if( check_freq_inrange( scan_start_freq ) == 0)
          {
            chansc_state = 0;
            break;  
          }
          // clear buffer, space for frequency
          memcpy( user_input_buffer, "          ", 10);
          user_input_buffer[10] = '\0'; 
          chansc_state = 1;   // back to menu
          break;
        }
      case CANCEL:
        {
          // remove selected highlight
          display_item_selector( chan_active_item, 1 ); 
          chansc_state = 1;  // back to menu
          break;
        }        
      default: break;
      }
      break;
    }
  case 23:   // get start freq from keypad
    {
      cursor_blink( 370, 127, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          get_number_input( 300, 90, 3, 9); 
          break;
        }
        // otherwise it could be a valid dtmf navigation key
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // clear frequency
        clear_rectangle( chanMenu[ STFQ ].box_x + 60, chanMenu[ STFQ ].box_y + 8, 235, 36 );
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "         ", 9) == 0)
        {
          display_frequency( scan_start_freq, chanMenu[ STFQ ].box_x + 60,
                     chanMenu[ STFQ ].box_y + 8, chanMenu[ STFQ ].font);
          // clear cursor
          display_cursor( 0, 370, 127, 3 );          
          chansc_state = 1;  // back to menu
          break; 
        }              
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] ='\0';
        // remove selected highlight
        display_item_selector( chan_active_item, 1 );         
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. 
      if (nav_key == SELECT)
      {
        // remove leading spaces
        n = 0;
        while( user_input_buffer[n] == ' ') n++;
        // n is location of first digit
        strncpy( user_input_buffer, user_input_buffer + n, 16); 
        
        // frequency is in the user input buffer
        // for user input frequency is all 9 digits
        scan_start_freq = atoi( user_input_buffer );

        if( scan_start_freq > 0)   // can hang if zero
        {
          // multiply frequency to add zeros to end
          while(scan_start_freq < 100000000) scan_start_freq *= 10;
        }
        // check frequency
        if( check_freq_inrange( scan_start_freq ) == 0) break;

        // redisplay freq
        clear_rectangle( chanMenu[ STFQ ].box_x + 60, chanMenu[ STFQ ].box_y + 8, 235, 36 );
        // re display start frequency
        display_frequency( scan_start_freq, chanMenu[ STFQ ].box_x + 60,
                     chanMenu[ STFQ ].box_y + 8, chanMenu[ STFQ ].font);
        
        scanning_freq = scan_start_freq;
        // redisplay scanning frequency
        clear_rectangle( chanMenu[ SCAN ].box_x + 60, chanMenu[ SCAN ].box_y + 8, 235, 36 );          
        display_frequency( scanning_freq, chanMenu[ SCAN ].box_x + 60,
                     chanMenu[ SCAN ].box_y + 8, chanMenu[ SCAN ].font);

        // clear buffer, space for frequency
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] = '\0'; 
        
        // remove selected highlight
        display_item_selector( chan_active_item, 1 );
        // clear cursor
        display_cursor( 0, 370, 127, 3 );
        chansc_state = 1; // back to menu
        break;
      }
      break;
    }
  case 24:   // end frequency for scanning
    {
     // if a number key is pressed, go to frequency entry
      if (dtmf_code_flag)
      {
        // get numeric input 
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          prev_dtmf = dtmf_code_char;
          // clear buffer, space for frequency
          // the user buffer must have spaces in the front so it will print
          memcpy( user_input_buffer, "          ", 10);
          user_input_buffer[10] = '\0';    // must have a null at the end        
          // clear frequency
          clear_rectangle( chanMenu[ ENDFQ ].box_x + 60,
                          chanMenu[ ENDFQ ].box_y + 8, 235, 36 );          
          chansc_state = 25; 
          break;
        }
      }
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;

      switch( nav_key )
      {
      case DOWN: // - rotation for down
        {
          // decrease frequency by step
          scan_end_freq -= freq_step*10;
          // redisplay freq
          clear_rectangle( chanMenu[ ENDFQ ].box_x + 60, chanMenu[ ENDFQ ].box_y + 8, 235, 36 );
          // redisplay end frequency
          display_frequency( scan_end_freq, chanMenu[ ENDFQ ].box_x + 60,
                       chanMenu[ ENDFQ ].box_y + 8, chanMenu[ ENDFQ ].font);          
          // stay in this state
          break;
        }
      case UP: // + rotation for up
        {
          // increase frequency by step
          scan_end_freq += freq_step*10;
          // redisplay freq
          // clear old data
          clear_rectangle( chanMenu[ ENDFQ ].box_x + 60, chanMenu[ ENDFQ ].box_y + 8, 235, 36 );
          // re display end frequency
          display_frequency( scan_end_freq, chanMenu[ ENDFQ ].box_x + 60,
                       chanMenu[ ENDFQ ].box_y + 8, chanMenu[ ENDFQ ].font);  
          // stay in this state
          break;
        }      
      case SELECT:
        {
          // check frequency, display error box
          if( check_freq_inrange( scan_end_freq ) == 0)
          {
            chansc_state = 0;
            break;  
          }
          // clear buffer, space for frequency
          memcpy( user_input_buffer, "          ", 10);
          user_input_buffer[10] = '\0'; 
          chansc_state = 1;   // back to menu
          break;
        }
      case CANCEL:
        {
          // remove selected highlight
          display_item_selector( chan_active_item, 1 ); 
          chansc_state = 1;  // back to menu
          break;
        }        
      default: break;
      }
      break;
    }
  case 25:   // get end freq from keypad
    {
      cursor_blink( 370, 167, 3 );   
      
      // if a number press has already occured, set flag to use the number 
      if( prev_dtmf )
      {
        dtmf_code_flag = 1;
        dtmf_code_char = prev_dtmf;
        prev_dtmf = 0;
      }
      if (dtmf_code_flag)
      {
        dtmf_code_flag = 0;
        if( isdigit( dtmf_code_char ))
        {
          get_number_input( 300, 130, 3, 9); 
          break;
        }
        // otherwise it could be a valid dtmf navigation key
      }      
      // If NONE, UP, or DOWN, ignore
      nav_key = get_nav_key();
      if (nav_key == NONE) break;
      last_key_press = 0;
      if ((nav_key == UP) || (nav_key == DOWN)) break;
      // if a CANCEL key pressed, clear buffer and output blanks
      if (nav_key == CANCEL)
      {
        // clear frequency
        clear_rectangle( chanMenu[ ENDFQ ].box_x + 60, chanMenu[ ENDFQ ].box_y + 8, 235, 36 );
        // if cancel on a blank space, exit 
        if(strncmp( (char *) user_input_buffer, "         ", 9) == 0)
        {
          display_frequency( scan_end_freq, chanMenu[ ENDFQ ].box_x + 60,
                     chanMenu[ ENDFQ ].box_y + 8, chanMenu[ ENDFQ ].font);
          // clear cursor
          display_cursor( 0, 370, 167, 3 );          
          chansc_state = 1;  // back to menu
          break; 
        }              
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] ='\0';
        // remove selected highlight
        display_item_selector( chan_active_item, 1 );         
        break;
      }
      // If a SELECT key pressed, convert buffer to binary to check if in range.
      // if out of range, clear buffer and display. 
      if (nav_key == SELECT)
      {
        // remove leading spaces
        n = 0;
        while( user_input_buffer[n] == ' ') n++;
        // n is location of first digit
        strncpy( user_input_buffer, user_input_buffer + n, 16); 
        
        // frequency is in the user input buffer
        // for user input frequency is all 9 digits
        scan_end_freq = atoi( user_input_buffer );

        if( scan_end_freq > 0)   // can hang if zero
        {
          // multiply frequency to add zeros to end
          while(scan_end_freq < 100000000) scan_end_freq *= 10;
        }
        // check frequency
        if( check_freq_inrange( scan_end_freq ) == 0) break;

        // redisplay freq
        clear_rectangle( chanMenu[ ENDFQ ].box_x + 60, chanMenu[ ENDFQ ].box_y + 8, 235, 36 );
        // re display start frequency
        display_frequency( scan_end_freq, chanMenu[ ENDFQ ].box_x + 60,
                     chanMenu[ ENDFQ ].box_y + 8, chanMenu[ ENDFQ ].font);  

        // clear buffer, space for frequency
        memcpy( user_input_buffer, "          ", 10);
        user_input_buffer[10] = '\0'; 
        
        // remove selected highlight
        display_item_selector( chan_active_item, 1 );
        // clear cursor
        display_cursor( 0, 370, 167, 3 );
        chansc_state = 1; // back to menu
        break;
      }
      break;
    }
  case 26: // scanning, start, stop
    {
      nav_key = get_nav_key();
      last_key_press = 0;

      switch( nav_key )
      {
      //case UP:
      //case DOWN:  
      case SELECT:
        {
          // toggle scanning
          is_scanning ^= 1;
          if( is_scanning )
          {
            // clear timers
            scan_step_timer = 0;
            scan_rx_timer = 0;
          }
          break;
        }
      case CANCEL:
        {
          // stop and leave menu
          is_scanning = 0;
          chansc_state = 0;          
          chansc_flag = 2;  // set to exit
          break;
        }        
      default: break;
      }
      // no keys, scan
      if( is_scanning )
      {
        cursor_blink( 490, 310, 3 );          
        // if both timers are done
        if( scan_step_timer==0 && scan_rx_timer==0 )
        {
          // increase frequency by step
          scanning_freq += freq_step*10;
          // check for limit
          if( scanning_freq > scan_end_freq ) scanning_freq = scan_start_freq;
          
          // redisplay scanning frequency
          clear_rectangle( chanMenu[ SCAN ].box_x + 60, chanMenu[ SCAN ].box_y + 8, 235, 36 );          
          display_frequency( scanning_freq, chanMenu[ SCAN ].box_x + 60,
                       chanMenu[ SCAN ].box_y + 8, chanMenu[ SCAN ].font);
          // tell scan process to send a new frequency command
          chansc_flag = 1;
        }
        // 
        else if( radio_receiving && scan_rx_timer == 0 )
        {
          // if receiving, set pause delay
          scan_rx_timer = scan_rx_delay;
        }
      }
      break;
    }
  case 27:   // send frequency to radio in enhanced screen
    {
      chansc_flag = 1;  // flag to tell process routine to send     
      chansc_state = 0; // aet for restart 
      
      vfo_state = 0;
      e_chan_state = 0;
      pop_menu_stack();      
      break;
    }    
// =============    
  default:
    {
      error_log( 3001 );
      break;
    }
  } // end of state switch
}

// =============================================================================
//
//  Display a frequency number
//
//  converts a 32bit unsigned integer to ascii digits in the print buffer
//  adds a decimal point and prints to location x,y in font
// if the frequency is divided by 10 to suit the radio, adds a zero to the end
// to always show 9 digits 

void display_frequency( uint32_t freq, uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int n;
  int i;
  n = sprintf((char*) cg_line_buffer, "%i", freq );
  
  // make space for decimal point
  for (i = 20; i > 2 ;i--)                      
  {
    cg_line_buffer[i+1] = cg_line_buffer[i];
  }
  cg_line_buffer[3] = '.';  // add decimal point  
  if(n == 8 )
  {
  cg_line_buffer[n+1] = '0'; // add extra 0
  cg_line_buffer[n+2] = '\0'; // null
  ssd1963_cg_write_line( n+3, font, pos_x, pos_y, 5);    
  }
  else
  {
  cg_line_buffer[n+1] = '\0'; // null
  ssd1963_cg_write_line( n+2, font, pos_x, pos_y, 5);
  }
}
// =============================================================================
//
//
void display_contact_id( uint32_t contact, uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int n;
  
  if( contact == 0 )
  { // display "all"
    strncpy( (char*) cg_line_buffer, "all", 3);
    ssd1963_cg_write_line( 3, font, pos_x, pos_y, 5);    
  }
  else
  { // show the contact number
    n = sprintf((char*) cg_line_buffer, "= %i", contact );
    ssd1963_cg_write_line( n+3, font, pos_x, pos_y, 5);       
  }
}
// =============================================================================
//
//
void display_squelch_type( uint8_t stype, uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int n;
  
  // clear the box
  clear_rectangle( chanMenu[SQUEL].box_x + 5, chanMenu[SQUEL].box_y + 5,
                   chanMenu[SQUEL].box_width - 10, chanMenu[SQUEL].box_height - 10);  
  
  switch( stype )
  {
  case NOSQUEL:
    {
      strncpy( (char*) cg_line_buffer, "= None", 6);
      n = 6;
      break;
    }    
  case CTCSS:
    {
      strncpy( (char*) cg_line_buffer, "= CTCSS", 7);
      n = 7;
      break;
    }
  case DCS:
    {
      strncpy( (char*) cg_line_buffer, "= DCS", 5);
      n = 5;
      break;
    }
  case INVDCS:
    {
      strncpy( (char*) cg_line_buffer, "= DCSinv", 8);
      n = 8;
      break;
    }

  default: break;          
  }
  cg_line_buffer[n] = '\0';  
  ssd1963_cg_write_line( n, font, pos_x, pos_y, 3);

  // clear ctcss and dcs
  clear_rectangle( chanMenu[CTCEN].pos_x, chanMenu[CTCEN].pos_y, 400, 75); 
  
  // sub menu
  switch( stype )
  {
  case CTCSS:
    {
    // CTCSS
    display_chan_menuitem( CTCEN );         
    ctcss_encode = ctcss_code[ctcss_encode_indx];
    ctcss_freq = ctcss_encode / 10.0;
    // clear and display
    display_float_item( CTCEN, ctcss_freq );

    display_chan_menuitem( CTCDE );
    ctcss_decode = ctcss_code[ctcss_decode_indx];
    ctcss_freq = ctcss_decode / 10.0;        
    // clear and display
    display_float_item( CTCDE, ctcss_freq );
    break;
    }
  case DCS:
    {
    display_chan_menuitem( DCSEN );
    dcs_encode = dcs_code[ dcs_encode_indx ];
    // clear and display
    display_integer_item( DCSEN, dcs_encode );
    
    display_chan_menuitem( DCSDE );
    dcs_decode = dcs_code[ dcs_decode_indx ];
    // clear and display
    display_integer_item( DCSDE, dcs_decode );    

    break;
    }
  case INVDCS:  
    {
    display_chan_menuitem( DCSEN );
    dcs_encode = dcs_code[ dcs_encode_indx ];
    // clear and display
    display_integer_item( DCSEN, -dcs_encode );
    
    display_chan_menuitem( DCSDE );
    dcs_decode = dcs_code[ dcs_decode_indx ];
    // clear and display
    display_integer_item( DCSDE, -dcs_decode );    

    break;
    }
  default: break;  //        
  }
}
// =============================================================================
//
//  Enhanced channel entry screen
//
// History: 01/18/2019
//          04/17/2019  modified for corrected rxsqlmode types
//          04/18/2019  modified for proper exit 
// Status:  written, working 

void process_e_chan_mode( uint8_t nav_key )
{
  //in_e_chan_mode = 1;
  //if( get_menu_stack != ECHAN ) push_menu_stack( ECHAN );  

  switch( e_chan_state )
  {
  case 0:   // display title
    {
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0; 
      // display title
      strncpy( (char*) cg_line_buffer, "Enhanced Channel", 16 );
      ssd1963_cg_write_line( 16, 2, 200 ,20, 3);
     
      e_chan_state++;
      push_menu_stack( ECHAN );      
      break;
    }
  case 1: // pass through to channel page
    {
      process_channel_screen( nav_key );
      // if it returns with flag to send
      if( chansc_flag == 1 ) e_chan_state = 2;
      // if it returns with flag to exit
      if( chansc_flag == 2 ) e_chan_state = 3;
      // if it returns with a flag to previous
      if( chansc_flag == 3 ) e_chan_state = 4; 
      break;
    }
  case 2: // send command
    {
      tx_freq = rx_freq + txrx_offset;
      // send frequency command to radio
      if( chansc_channel_type )
      {
        cs_channel_monitor = 0;
        set_temp_ACH( tx_freq, rx_freq ); // analog
      }
      else
      {
        cs_channel_monitor = 0; 
        set_temp_DCH( tx_freq, rx_freq ); // DMR 
      }
    
      // question_box( 450, 100, 300, 100, "Save channel?", 9, 10 );
      channel_type = chansc_channel_type; 
      // store_temp_CH( tx_freq, rx_freq );
      // done
      e_chan_state = 3;
      // break;  // no break, fall through to exit      
    }
  case 3: // full exit from menu
    {
      e_chan_state = 0;  // set for restart
      pop_menu_stack();       
      chansc_flag = 0;
      chan_nums_active = 0;  // turn off channel numbers
      menu_exit = 1;   // force return to main screen
      break;
    }
  case 4: // exit sub page, return to menu
    {
      // clear the page
      clear_rectangle( 0,0, 800, 415);
      // clear extra space at bottom 
      clear_rectangle( 20, 420, 360, 80);
      pop_menu_stack();      
      // redraw previous sub menu, enh channel menu selected
      display_sub_menu( 0, 1, 460, 3 );
      // reset box width for this menu
      select_box_width = 460;       
      e_chan_state = 0;  // set for restart
      chansc_flag = 0;
      break;
    }    
    default: break;
  }
}
// =============================================================================
//
//  display spectrum graph
//
//
// line, bar
//

// uint32_t freq_step = 10;
uint32_t center_freq;
uint16_t spect_xpos = 150;
uint16_t spect_ypos = 70;
uint16_t spect_height = 300;

//uint8_t spectrum_rssi[251]; // 2 pixels per bar center pixel + 125 on each side
                            // scale rssi to 0-100 before storing

// fewer bars to run faster
uint8_t spectrum_rssi[63]; // 8 pixels per bar center pixel + 31 on each side
uint8_t spectrum_peak[63];

void display_spectrum_graph( uint16_t pos_x, uint16_t pos_y )
{
  int i;
  uint16_t bar_size;
  uint16_t peak_size;

  // display frame
  draw_frame( spect_xpos - 5, spect_ypos - 5, 504 + 10,
              spect_height + 10, 5, highlight);
  // display tick marks
   draw_rectangle( spect_xpos, spect_ypos + spect_height + 5, 4, 6,
                  highlight);
   draw_rectangle( spect_xpos + 249, spect_ypos + spect_height + 5, 4, 6,
                  highlight);
   draw_rectangle( spect_xpos + 502, spect_ypos + spect_height + 5, 4, 6,
                  highlight);

   set_font_colors( 3, 14 );   //   
   
   // display scale
   strncpy( (char*) cg_line_buffer, "uV", 2);
   ssd1963_cg_write_line(2, 4, spect_xpos - 50, spect_ypos - 40, 3);
   strncpy( (char*) cg_line_buffer, "180", 3);
   ssd1963_cg_write_line(3, 4, spect_xpos - 50, spect_ypos - 10, 3);
   strncpy( (char*) cg_line_buffer, "100", 3);
   ssd1963_cg_write_line(3, 4, spect_xpos - 50, spect_ypos + 20, 3);
   strncpy( (char*) cg_line_buffer, "60", 2);
   ssd1963_cg_write_line(2, 4, spect_xpos - 40, spect_ypos + 50, 3);
   strncpy( (char*) cg_line_buffer, "40", 2);
   ssd1963_cg_write_line(2, 4, spect_xpos - 40, spect_ypos + 80, 3);
   strncpy( (char*) cg_line_buffer, "20", 2);
   ssd1963_cg_write_line(2, 4, spect_xpos - 42, spect_ypos + 110, 3);
   strncpy( (char*) cg_line_buffer, "12", 2);
   ssd1963_cg_write_line(2, 4, spect_xpos - 40, spect_ypos + 140, 3);
   strncpy( (char*) cg_line_buffer, "7", 1);
   ssd1963_cg_write_line(1, 4, spect_xpos - 30, spect_ypos + 170, 3);
   strncpy( (char*) cg_line_buffer, "3", 1);
   ssd1963_cg_write_line(1, 4, spect_xpos - 30, spect_ypos + 200, 3);
   strncpy( (char*) cg_line_buffer, "2", 1);
   ssd1963_cg_write_line(1, 4, spect_xpos - 32, spect_ypos + 230, 3);
   strncpy( (char*) cg_line_buffer, "1", 1);
   ssd1963_cg_write_line(1, 4, spect_xpos - 29, spect_ypos + 260, 3);

  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );     

   // display bars
  for(i = 0; i < 63; i++)
  {
    // adjust bar size
    bar_size = spectrum_rssi[i] * spect_height/100 ; 
    peak_size = spectrum_peak[i] * spect_height/100 ;
    if(peak_size < bar_size) peak_size = bar_size;
    
    // clear top
    clear_rectangle( spect_xpos + i * 8, spect_ypos, 8, spect_height - peak_size);
  
    if( i == current_bin )
    {
      // display current bar pointer
      draw_rectangle( spect_xpos + i * 8 + 5, spect_ypos + 4, 1, spect_height - 4,
                     current_palette[get_pal_color( 0 )] );
    }
    RGB_color peak_colr;
    peak_colr.red = (current_palette[get_pal_color( 8 )].red +
                    current_palette[get_pal_color( 5 )].red) >> 1;   
    peak_colr.green  = (current_palette[get_pal_color( 8 )].green
                    + current_palette[get_pal_color( 5 )].green) >> 1; 
    peak_colr.blue = (current_palette[get_pal_color( 8 )].blue
                    + current_palette[get_pal_color( 5 )].blue) >> 1; 
    
    // display peak 
    draw_rectangle( spect_xpos + i * 8, spect_ypos + spect_height - peak_size, 8,
                   peak_size - bar_size, peak_colr);
    // display current , red
    draw_rectangle( spect_xpos + i * 8, spect_ypos + spect_height - bar_size, 8,
                   bar_size, current_palette[get_pal_color( 8 )] );    
  }  
}

// =============================================================================
//
//  Get data for spectrum graph
//
//  state machine gets called in the process menu routine
//

//uint8_t in_spectrum_data = 0;
uint8_t current_bin;

uint32_t center_freq = 0;          // is / 10
uint8_t gsd_state = 0;             // needs to be avilable to be restarted
uint8_t bin;
uint8_t sv_step_size = 7;          // table index start at 2 Khz

void get_spectrum_data( uint8_t nav_key )
{
  static uint32_t start_freq; 
  static uint32_t tx_freq; 
  static uint32_t rx_freq;

  uint32_t rssi_val;
  static uint8_t noack_counter;
  static uint8_t noreply_counter;

  // add nav key activity for up and down
  if( nav_key == CANCEL ) 
  {
    gsd_state = 0;
    pop_menu_stack(); 
    return;
  }
  if(get_menu_stack() != SPECTV) push_menu_stack( SPECTV ); 
  freq_step = step_size_table[ sv_step_size ] / 10;
  switch( gsd_state )
  {
  case 0: // start
    {
      bin = 0;
      // only set center at the start, prevent changing mid scan
      center_freq = cs_channel_rx_frequency;
      // set digital monitor off for spectrum
      cs_channel_monitor = 0;       
      // noack_counter = 0;
      noreply_counter = 0;      
      gsd_state++; 
      break;
    }
  case 1: // set frequencies
    {
      //start_freq = center_freq - freq_step * 125;  // 256 wide graph
      start_freq = center_freq - freq_step * 31;  // 63 wide graph      
      rx_freq = start_freq + bin * freq_step;
      tx_freq = rx_freq;
      // send frequency change
      set_temp_ACH( tx_freq, rx_freq );
      noack_counter = 0;
      gsd_state++;

      //break;
    }
  case 2: // wait for ack of temp ACH
    {
      if( tempACH_packet_ack == 0 )
      {
        if(noack_counter++ > 250 )
        {
          // go back and send command again
          if( noreply_counter++ > 5)
          {
            // allow channel control
            activate_keypress( 9 );              
            channel_updown( 1 );
            msec_delay(500);            
            channel_updown( 0 );
            deactivate_keypress( 9 );
            noreply_counter = 0;
            last_key_press = 0;            
          }
          gsd_state = 1;
        }
      }
      else // have an ack
      {
        gsd_state++;
      }
      break;
    }
  case 3: // send get rssi command
    {
      // send get rssi packet
      request_RSSI_value();
      noack_counter = 0;      
      gsd_state++;
      break;
    }
  case 4: // wait for data for RSSI, store data
    {
      // wait for data
      if( rssi_packet_ack == 0 )
      {
        if(noack_counter++ > 250 )
        {
          // go back and send command again
          if( noreply_counter++ > 5)
          {
            activate_keypress( 9 );
            channel_updown( 1 );
            msec_delay(400);
            channel_updown( 0 );
            noreply_counter = 0;
            deactivate_keypress( 9 );
            last_key_press = 0;
          }
          // go back and send commands again
          gsd_state = 3;
        }
      }
      else // have an ack 
      {
        // store data RSSI_value is from the packet
        //rssi_val =  RSSI_value * RSSI_value * RSSI_value  / 30000 ;
        rssi_val =  RSSI_value * RSSI_value  / 200 ;
        // prevent graph overflow
        if( rssi_val > 100 ) rssi_val = 100;
        // store in array
        //spectrum_rssi[bin] = bin;
        spectrum_rssi[bin] = (uint8_t) (rssi_val & 0xFF);
        if( spectrum_rssi[bin] > spectrum_peak[bin] )
        {
          spectrum_peak[bin] = spectrum_rssi[bin];
        }
        current_bin = bin;
        // check if done
        if( ++bin > 62 ) bin = 0;
        gsd_state = 1;
      }
      break;
    }
  default: break;  
  }
}
// =============================================================================
//
//  process spectrum view screen
//
//

uint8_t sv_state = 0;
uint8_t sv_item = 0;

void process_spectrum_view( uint8_t nav_key )
{
  static uint32_t disp_counter = 0;
  int i;
  
  // every time it's called
  get_spectrum_data( 0 ); // get data 
  
  switch( sv_state )
   {
   case 0:  // setup
     {
       // center_freq = cs_channel_rx_frequency;
       // standard nav buttons
       CS800D_touch_point[7].is_active = 1;  // P1
       CS800D_touch_point[8].is_active = 1;  // P2
       CS800D_touch_point[9].is_active = 1;  // P3
       CS800D_touch_point[10].is_active = 1; // P4         
       log_event( "Spectrum" );
       // turn off ptt to radio
       CS800_buttons[0].send_to_radio = 0; 
       sv_state++;
       break;
     }
   case 1:  // display screen
     {
      // clear
      ssd1963_cg_clear_display();
      // erase previous data
      for( i = 62; i >= 0; i--)
      {
          spectrum_rssi[i] = 0;
          spectrum_peak[i] = 0;
      }
      // display title
      strncpy( (char*) cg_line_buffer, "Spectrum view", 13);
      ssd1963_cg_write_line(13, 2, 230, 5, 3);

      // diplay center frequency
      display_frequency( center_freq, 340, 383, 4 ) ;    
      // display start frequency
      display_frequency( center_freq - (freq_step / 10) * 31, 110, 383, 4 );
      // display end frequency
      display_frequency( center_freq + (freq_step / 10) * 31, 590, 383, 4 ) ;

      // display step
      strncpy( (char*) cg_line_buffer, "Step", 4);
      ssd1963_cg_write_line(4, 3, 40, 410, 3);      
      
      display_freq_units( freq_step, 120, 410, 250, 410, 3 ); 


      active_button_mode = 0;       
      draw_navbuttons( 2 );

      // draw selection box around center frequency
      draw_frame(300, 380, 220, 30, 3, current_palette[ 4 ] );
      sv_item = 0;
      if( get_menu_stack() != SPECTV ) push_menu_stack( SPECTV );
      sv_state++;
      break;
    }
   case 2:
    {
      // use counter and diplay less often
      if(disp_counter++ > 100 )
      {
        display_spectrum_graph( 100, 50 );  // display graph       
        disp_counter = 0;
      }
      nav_key = get_nav_key();
      if(nav_key == NONE) return;
      last_key_press = 0;
      
      // process frequency change
      if(sv_item == 0 )
      {
      switch(nav_key)
       {
       case CANCEL:
         {
          sv_state = 0;
          gsd_state = 0;
          pop_menu_stack(); 
          // reset color
          set_font_colors( 3, 14 ); 
          // clear the page
          ssd1963_cg_clear_display();          
          active_button_mode = 0;
          // turn ptt to radio back on
          CS800_buttons[0].send_to_radio = 1; 
          draw_navbuttons( 2 );                        
          // redraw previous sub menu, with spectrum selected
          display_sub_menu( 0, 6, 460, 3 );
          // reset box width for this menu
          select_box_width = 460;            
          break;
          }
       case UP:
         {
           // increase center frequency by step
           center_freq += freq_step / 10;
           // scroll graph left 
           for( i = 0; i < 62; i++)
           {
             spectrum_rssi[i] = spectrum_rssi[i+1];
             spectrum_peak[i] = spectrum_peak[i+1];
           }
           spectrum_rssi[62] = 0;
           spectrum_peak[62] = 0;
           // clear space
           set_font_colors( 3, 14 );   // 
           clear_rectangle(110, 383, 650, 25);
           // redisplay frequencies
           // diplay center frequency
           display_frequency( center_freq, 340, 383, 4 ) ;    
           // display start frequency
           display_frequency( center_freq - (freq_step / 10) * 31, 110, 383, 4 );
           // display end frequency
           display_frequency( center_freq + (freq_step / 10) * 31, 590, 383, 4 ) ;
           
           // draw selection box around center frequency
           draw_frame(300, 380, 220, 30, 3, current_palette[ 4 ] );
           
           set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
           // restart spectrum data 
           gsd_state = 1;              
           break;
         }
       case DOWN:
         {
           // decrease center frequency by step
           center_freq -= freq_step / 10;
           // scroll graph right 
           for( i = 62; i > 0; i--)
           {
             spectrum_rssi[i] = spectrum_rssi[i-1];
             spectrum_peak[i] = spectrum_peak[i-1];
           }
           spectrum_rssi[0] = 0;
           spectrum_peak[0] = 0;
           // clear space
           set_font_colors( 3, 14 );   // 
           clear_rectangle(110, 383, 650, 25);
           // redisplay frequencies
           // diplay center frequency
           display_frequency( center_freq, 340, 383, 4 ) ;    
           // display start frequency
           display_frequency( center_freq - (freq_step / 10) * 31, 110, 383, 4 );
           // display end frequency
           display_frequency( center_freq + (freq_step / 10) * 31, 590, 383, 4 );
           // draw selection box around center frequency
           draw_frame(300, 380, 220, 30, 3, current_palette[ 4 ] );           
           set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) ); 
           // restart spectrum data 
           gsd_state = 1;  
           break;
         }
       case SELECT:
         {
           sv_item = 1;
          // clear selection box around center frequency
          draw_frame(300, 380, 220, 30, 3, current_palette[ 14 ] );
          // draw selection box around step
          draw_frame(20, 405, 350, 50, 3, current_palette[ 4 ] );          
           break;
         }
       default: break;
       }
      break;
      }
      else //  sv_item == 1 set step
      {
      switch(nav_key)
       {
       case CANCEL:
         {
          sv_state = 0;
          gsd_state = 0;
          pop_menu_stack();          
          // reset color
          set_font_colors( 3, 14 ); 
          // clear the page
          ssd1963_cg_clear_display();          
          active_button_mode = 0;
          // turn ptt to radio back on
          CS800_buttons[0].send_to_radio = 1;           
          draw_navbuttons( 2 );                        
          // redraw previous sub menu, with spectrum selected
          display_sub_menu( 0, 6, 460, 3 );
          // reset box width for this menu
          select_box_width = 460;            
          break;
          }         
       case UP:
         {
           // increase step (skipping 1Mhz step, too big for spectrum)
           sv_step_size = inc_inrange( sv_step_size, 0, 14 );              
           freq_step = step_size_table[ sv_step_size ] / 10;
           
           set_font_colors( 3, 14 );      
           clear_rectangle( 120, 410, 240, 40 );
           // re display step
           display_freq_units( freq_step, 120, 410, 240, 410, 3 );            
           
           set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );            
           break;
         }
       case DOWN:
         {
           // decrease step (skipping 1Mhz step, too big for spectrum)
           sv_step_size = dec_inrange( sv_step_size, 0, 14 );            
           freq_step = step_size_table[ sv_step_size ] / 10;

           set_font_colors( 3, 14 );  
           clear_rectangle( 120, 410, 240, 40 );
           // re display step
           display_freq_units( freq_step, 120, 410, 240, 410, 3 ); 

           set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) ); 
           
           break;
         }
       case SELECT:
         {
           sv_item = 0;
           // restart spectrum data 
           sv_state = 1;
           // also reset getting data state
           gsd_state = 1;
           bin = 0;
           set_font_colors( 3, 14 );   // 
           break;           
         }
       default: break;
       }
      }
    }
   default: break;
   }
}
// =============================================================================
//
//  Enhanced scan  mode screen
//
// History: 03/22/2019
//
// Status: copied from vfo mode, writing

uint8_t enhscan_state = 0; // enhanced scan state

void process_enhscan_mode( uint8_t nav_key )
{
  switch( enhscan_state )
  {
  case 0:   // starting 
    {
      push_menu_stack( ENHSCAN );
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0; 
      // display title
      strncpy( (char*) cg_line_buffer, "Enhanced Scan", 13);
      ssd1963_cg_write_line(13, 2, 200 ,20, 3);
      
      enhscan_state++;
      break;
    }
  case 1: // pass through to screen handler
    {
      process_channel_screen( nav_key );
      // if it returns with flag to send
      if( chansc_flag == 1 ) enhscan_state = 2;
      // if it returns with flag to exit
      if( chansc_flag == 2 ) enhscan_state = 3;
      break;      
    }
  case 2: // send command 
    {
      rx_freq = scanning_freq/10;
      tx_freq = rx_freq;
      // send frequency command to radio
      if( chansc_channel_type )
      {
        // set digital monitor off
        cs_channel_monitor = 0; 
        set_temp_ACH( tx_freq, rx_freq ); // analog
      }
      else
      {
        // set digital monitor on
        cs_channel_monitor = 1; 
        set_temp_DCH( tx_freq, rx_freq ); // DMR
      }

      scan_step_timer = scan_step_delay;  
      // back to screen handler
      enhscan_state = 1;
      chansc_flag = 0;
      break;
    }
  case 3: // exit  
    {
      // clear the page
      clear_rectangle( 0,0, 800, 415);
      // clear extra space at the bottom      
      clear_rectangle( chanMenu[TXID].pos_x, chanMenu[TXID].pos_y - 5, 360, 80);
      // redraw previous sub menu, with enhanced scan menu selected
      display_sub_menu( 0, 2, 460, 3 );
      // reset box width for this menu
      select_box_width = 460; 
      enhscan_state = 0;  // set for restart
      pop_menu_stack();
      return;    
      break;
    }

  default: break;
  }
}

