//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// February 18, 2019         Design Documents                       CSI_FT857D.c
//
//			routines for FT857D
//                   large panel display project
//
//
// Programmer
//  Jerry Wenger, Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//  
// History: 05/10/2019 made header, moved some routines here.
//          05/17/2019 added error log 
//        
// Status
//  written, working
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  calculate_address_of_column          calculates pixel location of a column
//  FT857D_output_character              displays a character
//  FT857D_output_mirror                 displays text in mirror array 
//  process_FT857D_packets               handle packets from radio
//  process_FT857D_packet_type_0x40      handle packet 0x40
//  process_FT857D_packet_type_0x41      handle packet 0x41
//  FT857_display_squelch_graph          display the squelch / RFgain graph
//  FT857_clear_squelch_graph            clear the squelch / RFgain graph
//  process_FT857D_packet_type_0x43      handle packet 0x43
//  process_FT857D_packet_type_0x45      handle packet 0x45
//  clear_spectrum                       clear the spectrum area
//  process_FT857D_packet_type_0x47      handle packet 0x47
//  process_FT857D_packet_type_0x48      handle packet 0x48
//  process_FT857D_packet_type_0x4A      handle packet 0x4A
//  process_FT857D_packet_type_0x4B      handle packet 0x4B
//  process_FT857D_packet_type_0x4C      handle packet 0x4C
//  process_FT857D_packet_type_0x4D      handle packet 0x4D
//  process_FT857D_packet_type_0x4E      handle packet 0x4E
//  process_FT857D_packet_unknown        handle an unknown packet
//  process_FT857D_display_array_unknown handle packet with unknown information
//  get_rx_uart2_fifo_data               get data from uart2 buffer  
//  FT857D_clear_mirror_line             fill line with spaces, store to mirror
//  FT857D_clear_mirror_line_4_partial   clear line 4, special case
//  FT857D_clear_all_mirror_lines        clear all lines with spaces, store
//  FT857D_display_menu_number           display a menu number
//  process_blinking_cursor              blink a cursor
//  FT857D_turn_on_cursor                activate a cursor
//  FT857D_turn_off_cursor               deactivate a cursor
//  clear_radio_screen_and_rewrite_lines  clear and rewrite all
//  clear_spectrum_hold                  clears spectrum hold data
//  clear_spectrum_last                  clears spectrum last data
//  clear_spectrum_hold_and_last         clears both hold and last data
//  paint_spectrum_rectangle             dispaly a rectange of the spectrum
//  save_FT857_info                      write FT857 data to EEPROM
//  load_FT857_info                      read FT857 data from EEPROM
//  process_release_packet               Send a key release to the radio
//  FT857D_set_volume                    Send a volume packet to the radio
//  FT857D_set_squelch                   Send a squelch/ rfgain packet
//  absval                               absolute value of an integer
//  FT857D_mechanical_switches           handle touch buttons   

//==============================================================================
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "CSI_FT857D.h"
#include "CSI_GPIO.h"
#include "CSI_SSD1963.h"
#include "CSI_FT857_SCREENS.h"
#include "CSI_GRAPHICS.h"
#include "CSI_color.h"
#include "CSI_UART.h"
#include "CSI_i2c.h"
#include "CSI_SWITCHES.h"
#include "CSI_eeprom.h"
#include "CSI_error_log.h"

void process_FT857D_packet_type_0x40(void);
void process_FT857D_packet_type_0x41(void);
void process_FT857D_packet_type_0x43(void);
void process_FT857D_packet_type_0x45(void);
void process_FT857D_packet_type_0x47(void);
void process_FT857D_packet_type_0x48(void);
void process_FT857D_packet_type_0x4A(void);
void process_FT857D_packet_type_0x4B(void);
void process_FT857D_packet_type_0x4C(void);
void process_FT857D_packet_type_0x4D(void);
void process_FT857D_packet_type_0x4E(void);
void process_FT857D_packet_unknown(void);

void FT857D_clear_mirror_line_4_partial(void);

void FT857D_turn_on_cursor(void);
void FT857D_turn_off_cursor(void);

void clear_spectrum_hold(void);
void clear_spectrum_last(void);
void clear_spectrum_hold_and_last(void);
void paint_spectrum_rectangle(void);

uint16_t calculate_address_of_column(uint8_t line, uint8_t column);
uint16_t FT857D_output_character(uint8_t character, uint8_t line, uint8_t column);
void FT857D_output_mirror(uint8_t character, uint8_t line, uint8_t column);

void FT857D_clear_all_mirror_lines(void);
void FT857D_display_active_mirror_messages(void);
void FT857D_display_menu_number(void);

void process_FT857D_display_array_unknown(void);
void clear_radio_screen_and_rewrite_lines( void );

uint8_t get_rx_uart2_fifo_data(void);

uint8_t FT857_squelch_value;
uint8_t FT857_volume_value;

uint8_t mirror_line[5][22];

uint8_t spectrum_last[127];
uint8_t spectrum_hold[127];

uint32_t FT857D_packet_count;
uint32_t FT857D_packet_type;
uint32_t FT857D_parm_d1;
uint32_t FT857D_parm_d2;
// uint32_t FT857D_parm_d3;

uint8_t unknown_array_array[256][4];
uint8_t unknown_array_pointer;

uint8_t FT857D_Type_43_status[101];

uint32_t FT857D_cursor_state;
uint32_t FT857D_cursor_position;
uint32_t FT857D_cursor_pointer;

uint32_t spectrum_mode;
uint32_t spectrum_hold_flag; 
uint32_t spectrum_flag;

uint32_t FT857D_cursor_timer;
uint32_t Knob_Timer_Delay = 100;  // msec packet timing 

RGB_color FT857D_font;
RGB_color FT857D_background;

uint32_t special_encoder_flag;
uint8_t FT857D_rfsql_mode = 0;  //  0 if not rec'd the info packet yet
                                //  rf gain = 1, squelch = 2


const char Type_40_05[][10] =    // (all memnus)
{                                // Length 10, number if items 1
  "ADJUST No-"                   // column 1, line 0
};
const char Type_40_06[][12] =    // (all menus)
{                                // Length 12, number of items 1
  "MENU MODE No"                 // Column 1  line 0
};
const char Type_40_07[][17]  =   // Main Selection of menus (all menus)
{                                // Length 17, number of items 91
  "EXT MENU         ",           // Column 1  line 1
  "144 MHz ARS      ",
  "430MHz ARS       ",
  "AM&FM DIAL       ",
  "AM MIC GAIN      ",
  "AM STEP          ",
  "APO TIME         ",
  "ARTS BEEP        ", 
  "ARTS ID          ",
  "ARTS IDW         ",
  "BEACON TEXT 1    ",
  "BEACON TIME      ",
  "BEEP TONE        ",
  "BEEP VOL         ",
  "CAR LSB R        ",
  "CAR LSB T        ",

  "CAR USB R        ",
  "CAR USB T        ",
  "CAR RATE         ",
  "CAT/LIN/TUN      ",
  "CLAR DIAL SEL    ",
  "CW AUTO MODE     ",
  "CW BFO           ",
  "CW DELAY         ",
  "CW KEY REV       ",
  "CW PADDLE        ",
  "CW PITCH         ",
  "CW QSK           ",
  "CW SIDE TONE     ",
  "CW SPEED         ",
  "CW TRAINING      ",
  "CW WEIGHT        ",

  "DCS CODE         ",
  "DCS INV          ",
  "DIAL STEP        ",
  "DIG DISP         ",
  "DIG GAIN         ",
  "DIG MODE         ",
  "DIG SHIFT        ",
  "DIG VOX          ",
  "DISP COLOR       ",
  "DISP CONTRAST    ",
  "DISP INTENSITY   ",
  "DISP MODE        ",
  "DSP BPF WIDTH    ",
  "DSP HPF CUTOFF   ",
  "DSP LPF CUTOFF   ",
  "DSP MIC EQ       ",
    
  "DSP NR LEVEL     ",
  "EMERGENCY        ",
  "FM MIC GAIN      ",
  "FM STEP          ",
  "HOME ->VFO       ",
  "LOCK MODE        ",
  "MEM GROUP        ",
  "MEM TAG          ",
  "M/V/SEL DIAL MODE",
  "MIC SCAN         ",
  "MIC SEL          ",
  "MTR ARX SEL      ",
  "MTR ATX SEL      ",
  "MTR PEAK HOLD    ",
  "NB  LEVEL        ",
  "OP FILTER 1      ",

  "PG A             ",
  "PG B             ",
  "PG C             ",
  "PG ACC           ",
  "PG P1            ",
  "PG P2            ",
  "PKT 1200         ",
  "PKT 9600         ",
  "PKT RATE         ",
  "PROC LEVEL       ",
  "RF POWER SET     ",
  "RPT SHIFT        ",
  "SCAN MODE        ",
  "SCAN RESUME      ",
  "SPLIT TONE       ",
  "SQL/RF GAIN      ",
    
  "SSB MIC GAIN     ",
  "SSB STEP         ",
  "TONE FREQ        ",
  "TOT TIME         ",
  "TUNER/ATAS       ",
  "TX IF FILTER     ",
  "VOX DELAY        ",
  "VOX GAIN         ",
  "XVTR A FREQ      ",
  "XVTR B FREQ      ",
  "XVTR SEL         ",

};

const char  Type_40_09[][4] =        // CLAR DIAL SEL (021)
{                                    // Length 4, number of items 3
  "MAIN",                            // Column 7, line 2
  " SEL", 
  "CLAR",
};
const char  Type_40_0A[][3] =        // EXT MENU (001)/(002)/(003)/(009)/(022)
{                                    // Length 3, number of items 2 
  "OFF",                             // Column 8, line 2
  " ON",
};
const char  Type_40_0B[][7] =        // AM&FM DIAL (004)
{                                    // Length 7, number of items 2 
  " ENABLE",                         // Column 4, line 2 
  "DISABLE",
};
const char  Type_40_0C[][7] =        // AM STEP (006)
{                                    // Length 7, number of items 6
  " 2.5KHz",                         // Column 4, line 2
  "   5KHz",
  "   9KHz",
  "  10KHz",  
  "12.5KHz",  
  "  25KHz",  
};
const char  Type_40_0D[][3] =        // APO TIME (007)
{                                    // Length 3, number of items 7
  "OFF",                             // Column 8, line 2
  " 1h",
  " 2h",
  " 3h",  
  " 4h",  
  " 5h", 
  " 6h", 
};
const char  Type_40_0E[][5] =        // ARTS BEEP (008)
{                                    // Length 5, number of items 3
  "  OFF",                           // Column 6, line 2 
  "RANGE",
  "  ALL",
};
const char  Type_40_0F[][10] =       // (all menus)
{                                    // Length 10, number of items 4
  "       PWR",                      // Column 1, line 2 
  "       ALC",
  "       SWR",
  "       MOD",
};

const char  Type_40_10[][6] =        // BEEP TONE (013)
{                                    // Length 6, number of items 3 
  " 440Hz",                          // Column 5, line 2 
  " 880Hz",
  "1760Hz",
};
const char  Type_40_11[][6] =        // (all menus)
{                                    // Length 6, number of items 3 
  " 100Hz",                          // Column 5, line 2 
  " 160Hz",
  " 220Hz",
};
const char  Type_40_12[][4] =        // (all menus)
{                                    // Length 4, number of items 3 
  "MAIN",                            // Column 7, line 2 
  " M/V",
  "CLAR",
};
const char  Type_40_13[][3] =        //
{                                    // Length 3,  number of items 2
  "OFF",                             // Column 8, line 2 
  " ON",
};

const char  Type_40_14[][4] =        // CW BFO (023)
{                                    // Length  4, number of items 3 
  " USB",                            // Column 7, line 2 
  " LSB",
  "AUTO",
};
const char  Type_40_15[][7] =        // CW KEY REV (025)
{                                    // Length  7, number of items 2
  " NORMAL",                         // Column  4, line 2 
  "REVERSE", 
};
const char  Type_40_16[][6] =        // CW PITCH (027)
{                                    // Length 6, number of items 5
  " 400Hz",                          // Column 5, line 2
  " 500Hz",
  " 600Hz",
  " 700Hz",
  " 800Hz",  
};
const char  Type_40_17[][4] =        // CW QSK (028)
{                                    // Length 4,  number of items 7
  "10ms",                            // Column 7, line 2
  "15ms",
  "20ms",
  "25ms",
  "30ms",
  "    ", 
  "    ",
};
const char  Type_40_18[][10] =       // (all menus)
{                                    // Length 10, number of items 4
  "       PWR",                      // Column 1, line 2 
  "       ALC",
  "       SWR",
  "       MOD",
};
const char  Type_40_19[][8] =        // CW TRAINING (031)
{                                    // Length 8,  number of items 3
  " N:     ",                        // Column 3, line 2
  " A:     ",
  "AN:     ",
};
const char  Type_40_1A[][5] =        // CW WEIGHT (032)
{                                    // Length 5,  number of items 21 
  "1:2.5",                           // Column 6, line 2
  "1:2.6",
  "1:2.7",
  "1:2.8",
  "1:2.9",
  "1:3.0",
  "1:3.1",
  "1:3.2",
  "1:3.3",
  "1:3.4",
  "1:3.5",
  "1:3.6",
  "1:3.7",
  "1:3.8",
  "1:3.9",
  "1:4.0",
  "1:4.1",
  "1:4.2",
  "1:4.3",
  "1:4.4",  
  "1:4.5",
};
const char  Type_40_1B[][7] =        // DCS INV (034)
{                                    // Length 7, number of items 4
  "  Tn-Rn",                         // Column 4, line 2
  " Tn-RiV",
  " Tiv-Rn",
  "Tiv-Riv",
};
const char  Type_40_1C[][6] =        // DIAL STEP (035)
{                                    // Length 6, number of items 2
  "  FINE",                          // Column 5, line 2
  "COARSE",
};
const char  Type_40_1D[][7] =        // DIG MODE (038)
{                                    // Length 7, number of items 6
  " RTTY-L",                         // Column 4, line 2
  " RTTY-U",
  "PSK31-L",  
  "PSK31-U",   
  " USER-L",    
  " USER-U",    
};
const char  Type_40_1E[][1] =        // (all menus)
{                                    // Length 1, number of items 5
  "0",                               // Column 10, line 2
  "1",
  "2", 
  "3",
  " ",
};
const char  Type_40_1F[][10] =       // For changing color of various functions
{                                    // Length 10, number of items 7
  "  ARTS:   ",                      // Column 1, line 2
  "  BAND:   ",
  "   FIX:   ",
  "MEMGRP:   ",
  "  MODE:   ",
  "   MTR:   ",
  "   VFO:   ",
};
const char  Type_40_20[][5] =        // DISP MODE (044)
{                                    // Length 5, number of items 4 
  "  OFF",                           // Column 6, line 2
  "AUTO1",
  "AUTO2", 
  "   ON", 
};
const char  Type_40_21[][4] =        // DSP MIC EQ (048)
{                                    // Length 4,  number of items 4
  " OFF",                            // Column 7, line 2
  " LPF",
  " HPF", 
  "BOTH",
};
const char  Type_40_22[][5] =        // DSP BPF WIDTH (045)
{                                    // Length 5, number of items 3
  " 60Hz",                           // Column 6, line 2
  "120Hz",
  "240Hz", 
};
const char  Type_40_23[][6] =        // DSP LPF CUTOFF (047)
{                                    // Length 6, number of items 32
  "1000Hz",                          // Column 5, line 2
  "1160Hz",  
  "1320Hz",
  "1480Hz",
  "1650Hz",
  "1800Hz",
  "1970Hz",
  "2130Hz",
  "2290Hz",
  "2450Hz",
  "2610Hz",
  "2770Hz",
  "2940Hz",
  "3100Hz",
  "3260Hz",
  "3420Hz",
  "3580Hz",
  "3740Hz",
  "3900Hz",
  "4060Hz",
  "4230Hz",
  "4390Hz",
  "4550Hz",
  "4710Hz",
  "4870Hz",
  "5030Hz",
  "5190Hz",
  "5390Hz",
  "5520Hz",
  "5680Hz",
  "5840Hz", 
  "6000Hz",
};
const char  Type_40_24[][6] =        // DSP HPF CUTOFF (045)
{                                    // Length 6, number of items 19
  " 100Hz",                          // Column 5, line 2
  " 160Hz",
  " 220Hz", 
  " 280Hz",
  " 340Hz",
  " 400Hz",   
  " 460Hz",
  " 520Hz", 
  " 580Hz",
  " 640Hz",
  " 700Hz",
  " 760Hz",
  " 820Hz",
  " 880Hz",
  " 940Hz",
  "1000Hz",
  " 100Hz",
  " 160Hz",
  " 220Hz",
};
const char  Type_40_25[][7] =        // FM STEP (052)
{                                    // Length 7, number of items 8
  "   5KHz",                         // Column 4, line 2
  "6.25KHz",
  "  10KHz",
  "12.5KHz",
  "  15KHz",
  "  20KHz",
  "  25KHz",
  "  50KHz",
};
const char  Type_40_26[][5] =        // LOCK MODE (054)
{                                    // Length 5, number of items 4
  " DIAL",                           // Column 6, line 2
  "FREQ",
  "PANEL",
  "  ALL",
};  
const char  Type_40_27[][3] =        // MTR ATX SEL (061)
{                                    // Length 3,  number of items 7
  "PWR",                             // Column 8, line 2
  "ALC",
  "MOD",
  "SWR",
  "VLT",
  "N/A",
  "OFF",
};   
const char  Type_40_28[][6] =        // CAT/LIN/TUNER (020)
{                                    // Length 6, number of items 3
  "   CAT",                          // Column 5, line 2
  "LINEAR",
  " TUNER",
};
const char  Type_40_29[][6] =        // (all menus)
{                                    // Length 6, number of items 8
  "1: non",                          // Column 5, line 2
  "2: 2.3",
  "3: 300",
  "4:OPF1",
  "5:OPF1",
  "6:OPF2",
  "7:OPF3",
  "8:OPF4",
};
const char  Type_40_2A[][4] =        // SCAN MODE (077)
{                                    // Length 4,  number of items 3
  "TIME",                            // Column 7, line 2
  "BUSY",
  "STOP",
};
const char  Type_40_2B[][11] =       // (no menu)
{                                    // Length 11,  number of items 2
  "CW SIDETONE",                     // Column 1, line 2 (actually col 0)
  "   CW SPEED",
};
const char  Type_40_2C[][8] =        // CAT RATE (019)
{                                    // Length 8, number of items 3
  " 4800bps",                        // Column 3, line 2
  " 9600bps",
  "38400bps",
};
const char  Type_40_2D[][6] =        // SSB STEP (082)
{                                    // Length 6, number of items 4
  "  1KHz",                          // Column 5, line 2
  "2.5KHz",
  "  5KHz",
  "      ",
};
const char  Type_40_2E[][11] =        // (no menu)
{                                    // Length 11, number of items 5
  "        OFF",                     // Column 1, line 2 (actually col 0)
  "   ATAS(HF)",
  "ATAS(HF&50)",
  "  ATAS(ALL)",
  "      TUNER",
  "      ",
};
const char  Type_40_2F[][4] =        // (any menu)
{                                    // Length 4, number of items 4
  "CFIL",                            // Column 7, line 2
  "FIL1",
  "FIL2",
  "    ",
};
const char  Type_40_30[][6] =        // VOX DELAY (087)
{                                    // Length 6, number of items 31
  "      ",                          // Column 5, line 2
  " 100ms",
  " 200ms",
  " 300ms",
  " 400ms",
  " 500ms",
  " 600ms",
  " 700ms",
  " 800ms",
  " 900ms",
  "1000ms",
  "1100ms",
  "1200ms",
  "1300ms",
  "1400ms",
  "1500ms",
  "1600ms",
  "1700ms",
  "1800ms",
  "1900ms",
  "2000ms",
  "2100ms",
  "2200ms",
  "2300ms",
  "2400ms",
  "2500ms",
  "2600ms",
  "2700ms",
  "2800ms",
  "2900ms",
  "3000MS",
};
const char  Type_40_31[][7] =        // XVTR SEL (091)
{                                    // Length 7, number of items 3
  "    OFF",                         // Column 4, line 2
  " XVTR A",
  " XVTR B",
};
const char  Type_40_32[][7] =        // PKT RATE (073)
{                                    // Length 7, number of items 2
  "1200bps",                         // Column 4, line 2
  "9600bps",
};
const char  Type_40_33[][7] =        // SQL/RF GAIN (080)
{                                    // Length 7, number of items 2
  "RF-GAIN",                         // Column 4, line 2
  "    SQL",
};
const char  Type_40_34[][6] =        // CW PADDLE (026)
{                                    // Length 6, number of items 3
  "ELEKEY",                          // Column 5, line 2
  "MICKEY",
  "      ",
};
const char  Type_40_35[][11] =       // PG A OR PG B (065)/(066)/(067)/068/069
{                                    // Length 11, number of items 120
  "        A/B",                     // Column 1,  line 1 (actuall col 0)
  "        A=B",
  "        SPL",
  "         MW",
  "  SKIP/MCLR",
  "        TAG",
  "        STO",
  "        RCL",
  "       PROC",
  "        RPT",
  "        REV",
  "        VOX",
  "    TON/ENC",
  "       /DEC",
  "       TDCH",
  "       ARTS",   // 0F

  "       SRCH",
  "        PMS",
  "        SCN",
  "        PRI",
  "         DW",
  "       SCOP",   
  "        WID",
  "       STEP",
  "        MTR",
  "        PWR",
  "       DISP",
  "       SPOT",
  "         BK",
  "        KYR",
  "       TUNE",  
  "       DOWN",   //1F
  
  "         UP",
  "         NB",
  "        AGC",
  "    AGC SEL",
  "        IPO",
  "        ATT",
  "        NAR",
  "       CFIL",
  "       FIL1",
  "       FIL2",  
  "       PLY1",
  "       PLY2",    
  "       PLY3",  
  "        DNR",
  "        DNF",
  "        DBF",    //2F  
  
  "01:EXT MENU",    //30  1-16
  "02:144MHz A",
  "03:430MHz A",
  "04:AM&FM DI",
  "05:AM MIC G",
  "06:AM STEP ",
  "07:APO TIME",
  "08:ARTS BEE",
  "09:ARTS ID ",
  "10:ARTS IDW",  
  "11:BEACON T",
  "12:BEACON T",    
  "13:BEEP TON",  
  "14:BEEP VOL",
  "15:CAR LSB ",
  "16:CAR LSB ",    // 3F
  
  "17:CAR USB ",    //40  17-32
  "18:CAR USB ",
  "19:CAT RATE",
  "20:CAT/LIN/",
  "21:CLAR DIA",
  "22:CW AUTO ",
  "23:CW BFO  ",
  "24:CW DELAY",
  "25:CW KEY R",
  "26:CW PADDL",  
  "27:CW PITCH",
  "28:CW QSK  ",    
  "29:CW SIDE ",  
  "30:CW SPEED",
  "31:CW TRAIN",
  "32:CW WEIGH",   //4F
  
  "33:DCS CODE",    //50  33-48
  "34:DCS INV ",
  "35:DIAL STE",
  "36:DIG DISP",
  "37:DIG GAIN",
  "38:DIG MODE",
  "39:DIG SHIF",
  "40:DIG VOX ",
  "41:DISP COL",
  "42:DISP CON",  
  "43:DISP INT",
  "44:DISP MOD",    
  "45:DSP BPF ",  
  "46:DSP HPF ",
  "47:DSP LPF ",
  "48:DSP MIC ",  //5F
  
  "49:DSP NR L",    //60  49-64
  "50:EMERGENC",
  "51:FM MIC G",
  "52:FM STEP ",
  "53:HOME>VFO",
  "54:LOCK MOD",
  "55:MEM GROU",
  "56:MEM TAG ",
  "57:MEM/VFO ",
  "58:MIC SCAN",  
  "59:MIC SEL ",
  "60:MTR ARX ",    
  "61:MTR ATX ",  
  "62:MTR PEAK",
  "63:NB LEVEL",
  "64:OP FILTE",   //6F 

  "71:PKT 1200",    //70  71-86
  "72:PKT 9600",
  "73:PKT RATE",
  "74:PROC LEV",
  "75:RF POWER",
  "76:RPT SHIF",
  "77:SCAN MOD",
  "78:SCAN RES",
  "79:SPLIT TO",
  "80:SQL/RF G",  
  "81:SSB MIC ",
  "82:SSB STEP",    
  "83:TONE FRE",  
  "84:TOT TIME",
  "85:TUNER/AT",
  "86:TX IF FI",  //7F

  "87:VOX DELA",    //80  87-91, MISC
  "88:VOX GAIN",
  "89:XVTR A F",
  "90:XVTR B F",
  "91:XVTR SEL",
  "       MONI",
  "      Q.SPL",
  "      TCALL",
  "        ATC",
  "       USER", 
};

const char  Type_40_36[][3] =        // MIC SEL (059)
{                                    // Length 3,  number of items 3 
  "NOR",                             // Column 8, line 2
  "RMT",
  "CAT",
};
const char  Type_40_37[][6] =        // (any menu)
{                                    // Length 6, number of items 2
  " STRT ",                          // Column 9, line 4
  "      ",
};


const char  Type_40_39[][5] =        // MTR ARX SEL (060)
{                                    // Length 5,  number of items 8
  "  SIG",                           // Column 6, line 2
  "  CTR",
  "  VLT",
  "  N/A",
  "  FS ",
  "  OFF",
  " SKIP",
  "     ",
};
const char  Type_40_3A[][11] =       // PG A OR PG B (065)/(066)/(067)/068/069
{                                    // Length 11, number of items 120
  "        A/B",                     // Column 1,  line 1
  "        A=B",
  "        SPL",
  "         MW",
  "  SKIP/MCLR",
  "        TAG",
  "        STO",
  "        RCL",
  "       PROC",
  "        RPT",
  "        REV",
  "        VOX",
  "    TON/ENC",
  "       /DEC",
  "       TDCH",
  "       ARTS",   // 0F

  "       SRCH",
  "        PMS",
  "        SCN",
  "        PRI",
  "         DW",
  "       SCOP",   
  "        WID",
  "       STEP",
  "        MTR",
  "        PWR",
  "       DISP",
  "       SPOT",
  "         BK",
  "        KYR",
  "       TUNE",  
  "       DOWN",   //1F
  
  "         UP",
  "         NB",
  "        AGC",
  "    AGC SEL",
  "        IPO",
  "        ATT",
  "        NAR",
  "       CFIL",
  "       FIL1",
  "       FIL2",  
  "       PLY1",
  "       PLY2",    
  "       PLY3",  
  "        DNR",
  "        DNF",
  "        DBF",    //2F  
  
  "01:EXT MENU",    //30  1-16
  "02:144MHz A",
  "03:430MHz A",
  "04:AM&FM DI",
  "05:AM MIC G",
  "06:AM STEP ",
  "07:APO TIME",
  "08:ARTS BEE",
  "09:ARTS ID ",
  "10:ARTS IDW",  
  "11:BEACON T",
  "12:BEACON T",    
  "13:BEEP TON",  
  "14:BEEP VOL",
  "15:CAR LSB ",
  "16:CAR LSB ",    // 3F
  
  "17:CAR USB ",    //40  17-32
  "18:CAR USB ",
  "19:CAT RATE",
  "20:CAT/LIN/",
  "21:CLAR DIA",
  "22:CW AUTO ",
  "23:CW BFO  ",
  "24:CW DELAY",
  "25:CW KEY R",
  "26:CW PADDL",  
  "27:CW PITCH",
  "28:CW QSK  ",    
  "29:CW SIDE ",  
  "30:CW SPEED",
  "31:CW TRAIN",
  "32:CW WEIGH",   //4F
  
  "33:DCS CODE",    //50  33-48
  "34:DCS INV ",
  "35:DIAL STE",
  "36:DIG DISP",
  "37:DIG GAIN",
  "38:DIG MODE",
  "39:DIG SHIF",
  "40:DIG VOX ",
  "41:DISP COL",
  "42:DISP CON",  
  "43:DISP INT",
  "44:DISP MOD",    
  "45:DSP BPF ",  
  "46:DSP HPF ",
  "47:DSP LPF ",
  "48:DSP MIC ",  //5F
  
  "49:DSP NR L",    //60  49-64
  "50:EMERGENC",
  "51:FM MIC G",
  "52:FM STEP ",
  "53:HOME>VFO",
  "54:LOCK MOD",
  "55:MEM GROU",
  "56:MEM TAG ",
  "57:MEM/VFO ",
  "58:MIC SCAN",  
  "59:MIC SEL ",
  "60:MTR ARX ",    
  "61:MTR ATX ",  
  "62:MTR PEAK",
  "63:NB LEVEL",
  "64:OP FILTE",   //6F 

  "71:PKT 1200",    //70  71-86
  "72:PKT 9600",
  "73:PKT RATE",
  "74:PROC LEV",
  "75:RF POWER",
  "76:RPT SHIF",
  "77:SCAN MOD",
  "78:SCAN RES",
  "79:SPLIT TO",
  "80:SQL/RF G",  
  "81:SSB MIC ",
  "82:SSB STEP",    
  "83:TONE FRE",  
  "84:TOT TIME",
  "85:TUNER/AT",
  "86:TX IF FI",  //7F

  "87:VOX DELA",    //80  87-91, MISC
  "88:VOX GAIN",
  "89:XVTR A F",
  "90:XVTR B F",
  "91:XVTR SEL",
  "       MONI",
  "      Q.SPL",
  "      TCALL",
  "        ATC",
  "       USER", 
};
const char  Type_40_3B[][11] =       // M/V/SEL DIAL MODE (057)
{                                    // Length 11, number of items 7  
  "CW SIDETONE",                     // Column 1,  line 2
  "   CW SPEED",
  "MHz/MEM GRP",
  "   MIC GAIN",
  "   NB LEVEL",
  "   RF POWER",
  "       STEP",
};
const char  Type_40_3C[][6] =        // Special type
{                                    // Length 6, number of items 2
  " SKIP ",                          // Column 2, line 4
  "      ",
};
const char  Type_40_3D[][4] =        // (any menu)
{                                    // Length 4, number of items 8
  "AUTO",                            // Column 7, line 2
  "FAST",
  "SLOW",
  "TUNE",
  "HSWR",
  "PWRA",
  "LCSW",
  "RMOD",
};

const char  Type_40_3E[][11] =       // ATAS(HF) (085)
{                                    // Length 11, number of items 5  
  "        OFF",                     // Column 1,  line 2
  "   ATAS(HF)",
  "ATAS(HF&50)",
  "  ATAS(ALL)",
  "      TUNER",
};
const char  Type_40_40[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_41[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_42[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_43[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_44[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_45[][12] =       // (any menu)
{                                    // Length 12, number of items 1
  "GROUP NO ERR",                    // Column 1, Line 2
};
const char  Type_40_46[][4] =        // (any menu)
{                                    // Length 4, number of items 5
  "TUNE",                            // Column 17, line 1
  "HSWR",
  "PWRA",
  "LCSW",
  "RMOD",
};
const char  Type_40_47[][3] =        // (any menu)
{                                    // Length 3, number of items 4
  "PWR",                             // Column 1, line 3
  "ALC",
  "SWR",
  "MOD",
};
const char  Type_40_48[][3] =        // (any menu)
{                                    // Length 3, number of items 12
  "S  ",                             // Column 1, line 3
  "S1 ",
  "S2 ",
  "S3 ",
  "S4 ",
  "S5 ",
  "S6 ",
  "S7 ",
  "S8 ",
  "S9 ",
  "S9+",
  'S','9',0x83,0x00,                 // Special double plus character  
};

const char  Type_40_49[][3] =        // (any menu)
{                                    // Length 3, number of items 4
  "TEN",                             // Column 5, line 0
  "TSQ",
  "DEN",
  "DCS",
};
const char  Type_40_4A[][4] =        // (any menu)
{                                    // Length 4, number of items 5
  "VFOa",                            // Column 1, line 1
  " VFO",
};
const char Type_40_4B[][1] =         // (any menu)
{                                    // Length 1, number of items 11
  "a",                               // Column 2, line 1
  "b",
  "c",
  "d",
  "e",
  "f",
  "g",
  "h",
  "i",
  "j",
  "k",
};
const char  Type_40_4C[][3] =        // DCS CODE (033)
{                                    // Length 3,  number of items 104
  "023",                             // Column 8, line 2
  "025",
  "026",
  "031",
  "032",
  "036",
  "043",
  "047",
  "051",
  "053",
  
  "054",
  "065",
  "071",
  "072",
  "073",
  "074",
  "114",
  "115",
  "116",
  "122",
  
  "125",
  "131",
  "132",
  "134",
  "143",
  "145",
  "152",
  "155",
  "156",
  "162",
  
  "166",
  "172",
  "174",
  "205",
  "212",
  "223",
  "225",
  "226",
  "243",
  "244",
  
  "245",
  "246",
  "251",
  "252",
  "255",
  "261",
  "263",
  "265",
  "266",
  "271",
  
  "274",
  "306",
  "311",
  "315",
  "325",
  "331",
  "332",
  "343",
  "346",
  "351",
  
  "356",
  "364",
  "365",
  "371",
  "411",
  "412",
  "413",
  "423",
  "431",
  "432",
  
  "445",
  "446",
  "452",
  "454",
  "455",
  "462",
  "464",
  "465",
  "466",
  "503",
  
  "506",
  "516",
  "523",
  "526",
  "532",
  "546",
  "565",
  "606",
  "612",
  "624",

  "627",
  "631",
  "632",
  "654",
  "662",
  "664",
  "703",
  "712",
  "723",
  "731",
  
  "732",
  "734",
  "743",
  "754",
};
const char  Type_40_4D[][4] =         // (any menu)
{                                     // Length 2, number of items 2
  "T:",                               // column 6, line 2
  "R:",
};
const char  Type_40_4E[][3] =         //S Meter characters
{                                     // Length 3, number of items 11
  "S  ",                              // Column 1, line 3
  "S1 ",
  "S2 ",
  "S3 ",
  "S4 ",
  "S5 ",
  "S6 ",
  "S7 ",
  "S8 ",
  "S9 ",
  "S9+",
  'S','9',0x83,0x00,                 // Special double plus character  
};
const char  Type_40_4F[][2] =        // DCS CODE (033)
{                                    // Length 2, number of items 2
  "T:",                              // Column 4, line 2
  "R:",
};
const char  Type_40_50[][5] =        // TONE FREQ (083)
{                                    // Length 5, number of items 50
  " 67.0",                           // Column 6, line 2
  " 69.3",
  " 71.9",
  " 74.4",
  " 77.0",
  " 79.7",
  " 82.5",
  " 85.4",
  
  " 88.5",
  " 91.5",
  " 94.8",
  " 97.4",  
  "100.0",
  "103.5",
  "107.2",
  "110.9",

  "114.8",
  "118.8",
  "123.0",  
  "127.3",
  "131.8",
  "136.5",
  "141.3",
  "146.2",

  "151.4",
  "156.7",
  "159.8",  
  "162.2",
  "165.5",
  "167.9",
  "171.3",
  "173.8",  
  
  "177.3",
  "179.9",
  "183.5",  
  "186.2",
  "189.9",
  "192.8",
  "196.6",
  "199.5", 
  
  "203.5",
  "206.5",
  "210.7",  
  "218.1",
  "225.7",
  "229.1",
  "233.6",
  "241.8",

  "250.3",
  "254.1",  
};
const char  Type_40_51[][3] =        // (any menu)
{                                    // Length 3,  number of items 17
  "MFa",                             // Column 1, line 3
  "MFb",
  "MFc",
  "MFd",
  "MFe",
  "MFf",
  "MFg",
  "MFh",
  "MFi",
  "MFj",
  "MFk",
  "MFl",
  "MFm",
  "MFn",
  "MFo",
  "MFp",
  "MFq",
};
const char  Type_40_52[][3] =        // (any menu)
{                                    // Length 3,  number of items 4
  "-+ ",                             // Column 1, line 0
  " + ",
  " - ",
  "SPL",
};
const char  Type_40_53[][19] =       // (any menu)
{                                    // The numbers go to line 0 col 11 to end
  "HF1-RXG         001",             // Other info go to line 1 col 1 - col 17
  "HF2-RXG         002",             // might have to add 1-2 spaces
  "HF3-RXG         003",
  "50M-RXG         004",
  "VHF-RXG         005",
  "UHF-RXG         006",
  "SSB-S9          007",
  "SSB-FS          008",
  "FM-S1           009",
  "FM-FS           010",
  "DISC-L          011",
  "DISC-H          012",
  "FM-TH1          013",
  "FM-TH2          014",
  "FM-TI1          015",
  "FM-TI2          016",
  "VCC             017",
  "HF1-IC          018",
  "HF2-IC          019",
  "HF3-IC          020",
  "50M-IC          021",
  "VHF-IC          022",
  "UHF-IC          023",
  "HF1-PO-MAX      024",
  "HF1-PO-MID2     025",
  "HF1-PO-MOD1     026",
  "HF1-PO-MIN      027",  
  "HF2-PO-MAX      028",
  "HF2-PO-MID2     029",
  "HF2-PO-MOD1     030",
  "HF2-PO-MIN      031", 
  "HF3-PO-MAX      032",
  "HF3-PO-MID2     033",
  "HF3-PO-MOD1     034",
  "HF3-PO-MIN      035",   
  "50M-PO-MAX      036",
  "50M-PO-MID2     037",
  "50M-PO-MOD1     038",
  "50M-PO-MIN      039",    
  "VHF-PO-MAX      040",
  "VHF-PO-MID      041",
  "VHF-PO-MIN      042",
  "UHF-PO-MAX      043", 
  "UHF-PO-MIN      044",
  "HF1-TXG         045",
  "HF2-TXG         046",
  "HF3-TXG         047",
  "50M-TXG         048",
  "VHF-TXG         049",
  "UHF-TXG         050",
  "ALC1-M          051",
  "ALC-M           052",
  "HF1-REV-ALC     053",
  "HF2-REV-ALC     054",
  "HF3-REV-ALC     055",  
  "50M-REV-ALC     056",
  "VHF-REV-ALC     057",
  "UHF-REV-ALC     058",
  "CW-CAR-LEVEL    059", 
  "AM-CAR-LEVEL    060",
  "DEV-W           061",
  "DEV N           062",
  "MOD-MTR         063",
  "DTMF-DEV        064",
  "CTCSS-DEV       065",
  "DCS-DEV         066",
  "LSB-CAR-POINT   067",
  "USB-CAR-POINT   068",
  "VSWR2 at 10W    069",
  "VSWR3 at 10W    070",
  "ATAS-TEST       071",
  "AMTR-TEST       072",
  "HTEMP-THRESHOLD 073",
  "FTEMP-THRESHOLD 074",
  "                   ", 
};
  //
// Each array has a certain number of characters to be displayed. The following
// table defines how many characters need to be displayed. If the value is zero,
// then the item is not used.
//
const uint8_t FT857D_define_array_length[0x53] =
{
  0,  //Type 00
  0,  //Type 01
  0,  //Type 02
  0,  //Type 03
  0,  //Type 04
  10, //Type 05
  12, //Type 06
  17, //Type 07
  0,  //Type 08
  4,  //Type 09
  3,  //Type 0A
  7,  //Type 0B
  7,  //Type 0C
  3,  //Type 0D
  5,  //Type 0E
  10, //Type 0F  
  
  6,  //Type 10
  6,  //Type 11
  4,  //Type 12
  3,  //Type 13
  4,  //Type 14
  7,  //Type 15
  6,  //Type 16
  4,  //Type 17
  10, //Type 18
  8,  //Type 19
  5,  //Type 1A
  7,  //Type 1B
  6,  //Type 1C
  7,  //Type 1D
  1,  //Type 1E
  10, //Type 1F   
  
  5,  //Type 20
  4,  //Type 21
  5,  //Type 22
  6,  //Type 23
  6,  //Type 24
  7,  //Type 25
  5,  //Type 26
  3,  //Type 27
  6,  //Type 28
  6,  //Type 29
  4,  //Type 2A
  11, //Type 2B
  8,  //Type 2C
  6,  //Type 2D
  11, //Type 2E
  2,  //Type 2F

  6,  //Type 30
  7,  //Type 31
  7,  //Type 32
  7,  //Type 33
  6,  //Type 34
  11, //Type 35
  3,  //Type 36
  6,  //Type 37
  0,  //Type 38
  5,  //Type 39
  11, //Type 3A
  11, //Type 3B
  6,  //Type 3C
  4,  //Type 3D
  11, //Type 3E
  0,  //Type 3F

  12, //Type 40
  12, //Type 41
  12, //Type 42
  12, //Type 43
  12, //Type 44
  12, //Type 45
  4,  //Type 46
  3,  //Type 47
  3,  //Type 48
  3,  //Type 49
  4,  //Type 4A
  1,  //Type 4B
  3,  //Type 4C
  2,  //Type 4D
  3,  //Type 4E
  2,  //Type 4F  

  5,  //Type 50
  3,  //Type 51
  3,  //Type 52
};

// Each array has a starting column and line. The following table defines the 
// starting column and line for each array. If the column number is zero, then
// that item is not used

const uint8_t FT857D_define_array_location[0x53][2] =
{ 
  0,  0,   //Type 00
  0,  0,   //Type 01
  0,  0,   //Type 02
  0,  0,   //Type 03
  0,  0,   //Type 04
  1,  0,   //Type 05
  1,  0,   //Type 06
  1,  1,   //Type 07
  0,  0,   //Type 08
  7,  2,   //Type 09
  8,  2,   //Type 0A
  4,  2,   //Type 0B
  4,  2,   //Type 0C
  8,  2,   //Type 0D
  6,  2,   //Type 0E
  1,  2,   //Type 0F  
  
  5,  2,   //Type 10
  5,  2,   //Type 11
  7,  2,   //Type 12
  8,  2,   //Type 13
  7,  2,   //Type 14
  4,  2,   //Type 15
  5,  2,   //Type 16
  7,  2,   //Type 17
  1,  2,   //Type 18
  3,  2,   //Type 19
  6,  2,   //Type 1A
  4,  2,   //Type 1B
  5,  2,   //Type 1C
  4,  2,   //Type 1D
  10, 2,   //Type 1E
  1,  2,   //Type 1F   
  
  6,  2,   //Type 20
  7,  2,   //Type 21
  6 , 2,   //Type 22
  5,  2,   //Type 23
  5,  2,   //Type 24
  4,  2,   //Type 25 *
  6,  2,   //Type 26
  8,  2,   //Type 27
  5,  2,   //Type 28
  5,  2,   //Type 29
  7,  2,   //Type 2A
  1,  2,   //Type 2B
  3,  2,   //Type 2C
  5,  2,   //Type 2D
  1,  2,   //Type 2E
  7,  2,   //Type 2F

  5,  2,   //Type 30
  4,  2,   //Type 31
  4,  2,   //Type 32
  4,  2,   //Type 33
  5,  2,   //Type 34
  1,  2,   //Type 35
  8,  2,   //Type 36
  9,  4,   //Type 37
  0,  0,   //Type 38
  6,  2,   //Type 39
  1,  2,   //Type 3A
  1,  2,   //Type 3B
  2,  4,   //Type 3C
  7,  2,   //Type 3D
  1,  2,   //Type 3E
  0,  0,   //Type 3F

  1,  2,   //Type 40
  1,  2,   //Type 41
  1,  2,   //Type 42
  1,  2,   //Type 43
  1,  2,   //Type 44
  1,  2,   //Type 45
  17, 1,   //Type 46
  1,  3,   //Type 47
  1,  3,   //Type 48
  5,  0,   //Type 49
  1,  1,   //Type 4A
  2,  1,   //Type 4B
  8,  2,   //Type 4C
  6,  2,   //Type 4D
  1,  3,   //Type 4E
  4,  2,   //Type 4F  

  6,  2,   //Type 50
  1,  3,   //Type 51
  1,  0,   //Type 52
};

// This is the table to generate the plot for the S meter. This parameters
// meaning is a follows:
// First parameter:  Vertical address of first line to output
// Second parameter: Vertical address of second line to output
// Third parameter:  Length of line

const uint8_t FT857D_Type_43[101][3] =
{
  250, 250,   0,     // parameter value 0    00 -  20 
  249, 249,   9,     // parameter value 1    00 -  20
  248, 248,   9,     // parameter value 2    00 -  20
  247, 247,  10,     // parameter value 3    00 -  20
  246, 246,  10,     // parameter value 4    00 -  20
  245, 245,  10,     // parameter value 5    00 -  20
  244, 244,  10,     // parameter value 6    00 -  20
  243, 243,  10,     // parameter value 7    00 -  20
  242, 242,  10,     // parameter value 8    00 -  20
  241, 241,  11,     // parameter value 9    00 -  20
  240, 240,  11,     // parameter value 10   00 -  20
  239, 239,  11,     // parameter value 11   00 -  20
  238, 238,  11,     // parameter value 12   00 -  20
  237, 237,  11,     // parameter value 13   00 -  20
  236, 236,  12,     // parameter value 14   00 -  20  
  235, 235,  12,     // parameter value 15   21 -  30   
  234, 234,  12,     // parameter value 16   21 -  30 
  233, 233,  12,     // parameter value 17   21 -  30 
  232, 232,  13,     // parameter value 18   21 -  30 
  231, 231,  13,     // parameter value 19   21 -  30  
  230, 230,  13,     // parameter value 20   21 -  30 
  229, 229,  13,     // parameter value 21   21 -  30 
  228, 228,  14,     // parameter value 22   21 -  30 
  227, 228,  14,     // parameter value 23   21 -  30    
  225, 226,  14,     // parameter value 24   21 -  30   
  223, 224,  14,     // parameter value 25   31 -  30   
  221, 222,  15,     // parameter value 26   31 -  40
  219, 220,  15,     // parameter value 27   31 -  40
  217, 218,  15,     // parameter value 28   31 -  40
  215, 216,  15,     // parameter value 29   31 -  40
  213, 214,  16,     // parameter value 30   31 -  40
  211, 212,  16,     // parameter value 31   31 -  40
  209, 210,  16,     // parameter value 32   31 -  40
  207, 208,  17,     // parameter value 33   31 -  40
  205, 206,  17,     // parameter value 34   31 -  40
  203, 204,  17,     // parameter value 35   31 -  40
  201, 202,  18,     // parameter value 36   31 -  40
  199, 200,  18,     // parameter value 37   31 -  40  
  197, 198,  18,     // parameter value 38   31 -  40
  195, 196,  19,     // parameter value 39   31 -  40
  193, 194,  19,     // parameter value 40   31 -  40  
  191, 192,  19,     // parameter value 41   41 -  50
  189, 190,  20,     // parameter value 42   41 -  50
  187, 188,  20,     // parameter value 43   41 -  50
  185, 186,  20,     // parameter value 44   41 -  50
  183, 184,  21,     // parameter value 45   41 -  50
  181, 182,  21,     // parameter value 46   41 -  50
  179, 180,  21,     // parameter value 47   41 -  50
  177, 178,  22,     // parameter value 48   41 -  50
  175, 176,  22,     // parameter value 49   41 -  50
  173, 174,  22,     // parameter value 50   41 -  50  
  171, 172,  23,     // parameter value 51   41 -  50
  169, 170,  23,     // parameter value 52   41 -  50
  167, 168,  24,     // parameter value 53   41 -  50
  165, 166,  24,     // parameter value 54   41 -  50
  163, 164,  25,     // parameter value 55   41 -  50  
  161, 162,  25,     // parameter value 56   51 -  60
  159, 160,  26,     // parameter value 57   51 -  60
  157, 158,  26,     // parameter value 58   51 -  60
  155, 156,  27,     // parameter value 59   51 -  60
  153, 154,  27,     // parameter value 60   51 -  60
  151, 152,  28,     // parameter value 61   51 -  60
  149, 150,  28,     // parameter value 62   51 -  60
  147, 148,  29,     // parameter value 63   51 -  60
  145, 146,  29,     // parameter value 64   51 -  60
  143, 144,  30,     // parameter value 65   51 -  60
  141, 142,  30,     // parameter value 66   61 -  70  
  139, 140,  31,     // parameter value 67   61 -  70
  137, 138,  31,     // parameter value 68   61 -  70
  135, 136,  32,     // parameter value 69   61 -  70
  133, 134,  33,     // parameter value 70   61 -  70
  131, 132,  34,     // parameter value 71   71 -  80
  129, 130,  35,     // parameter value 72   71 -  80
  127, 128,  36,     // parameter value 73   71 -  80
  125, 126,  37,     // parameter value 74   71 -  80
  123, 124,  38,     // parameter value 75   71 -  80  
  121, 122,  39,     // parameter value 76   81 -  90
  119, 120,  40,     // parameter value 77   81 -  90
  117, 118,  41,     // parameter value 78   81 -  90
  115, 116,  42,     // parameter value 79   81 -  90
  113, 114,  43,     // parameter value 80   81 -  90
  111, 112,  44,     // parameter value 81   91 - 100
  109, 110,  45,     // parameter value 82   91 - 100  
  107, 108,  46,     // parameter value 83   91 - 100
  105, 106,  47,     // parameter value 84   91 - 100
  103, 104,  48,     // parameter value 85   91 - 100
  101, 102,  49,     // parameter value 86  101 - 120
   99, 100,  50,     // parameter value 87  101 - 120
   97,  98,  52,     // parameter value 88  101 - 120
   95,  96,  54,     // parameter value 89  101 - 120
   93,  94,  56,     // parameter value 90  101 - 120  
   91,  92,  58,     // parameter value 91  121 - 140   
   89,  90,  60,     // parameter value 92  121 - 140
   87,  88,  62,     // parameter value 93  121 - 140
   85,  86,  64,     // parameter value 94  121 - 140
   83,  84,  66,     // parameter value 95  121 - 140
   81,  82,  68,     // parameter value 96  141 - 150
   79,  80,  70,     // parameter value 97  141 - 150
   77,  78,  72,     // parameter value 98  141 - 150
   75,  76,  74,     // parameter value 99  141 - 150
   73,  74,  76,     // parameter value 100 141 - 150 
};
//
// This table is for defining the 32 different color combinations available in
// the FT857D. The order is red, green, blue.
//
const uint8_t FT857D_colors[16][2] =
{
//  foreground    background
       0xF0, 0x00,
       0xE0, 0x10,
       0xD0, 0x20,
       0xC0, 0x30,
       0xB0, 0x40,
       0xA0, 0x50,
       0x90, 0x60,
       0x80, 0x70,
       0x70, 0x80,
       0x60, 0x90,
       0x50, 0xA0,
       0x40, 0xB0,
       0x30, 0xC0,
       0x20, 0xD0,
       0x10, 0xE0,
       0x00, 0xF0,
};
//
// *****************************************************************************
//                         Calculation of Font Size
//
// Line #  Start  End  Difference  Char/Line  Bits/Char  Font  Index
//  0      270    776  506         19         26         28    7
//  1      250    776  526         20         26         28    7
//  2      290    776  486         12         40         48    5
//  3      190    288   98         3          32         28    7
//  4      140    776  636         22         28         28    7
//
// *****************************************************************************
//
// This program calculates the starting horizontal address of the column number
// based on the line number supplied. It is assumed the first column number is
// 1 except for line 2 where to 0 is a good position.
//
uint16_t calculate_address_of_column(uint8_t line, uint8_t column)
{
  uint16_t address;
  
  switch (line) {
    case 0:
      address = FT857_Message[line].x_pos + (column-1)*26;
      break;
    case 1:
      address = FT857_Message[line].x_pos + (column-1)*26;
      break;
    case 2:
      address = FT857_Message[line].x_pos + (column-1)*40;   
      break;
    case 3:
      address = FT857_Message[line].x_pos + (column-1)*26;
      break;
    case 4:
      address = FT857_Message[line].x_pos + (column-1)*26;
      break;
    default:
      address = FT857_Message[0].x_pos + (column-1)*26;
      break;
  }
  return address;
}

// =============================================================================
// This program outputs a character to the display. This program requires the
// following:
//   column number to put the character
//   line number to put the character
//   character to be outputed
//
// We have to match up with the following routine:
// uint16_t ssd1963_cg_output_character(uint8_t cg_character,uint8_t character_set,
//                           uint16_t cg_hori_location, uint16_t cg_vert_location, 
//                          uint16_t cg_spacing_type)
//
//
uint16_t FT857D_output_character(uint8_t character, uint8_t line, uint8_t column)
{
uint16_t cg_hori_location;
uint16_t cg_vert_location;
uint8_t  character_set;

  // calculate horizonetal location
  cg_hori_location = calculate_address_of_column(line, column);
  // Calculate vertical location
  cg_vert_location = FT857_Message[ line ].y_pos;
  // Calculate character set
  character_set = FT857_Message[ line ].font; 
  // Output character to display
  return ssd1963_cg_output_character(character,character_set,
                           cg_hori_location, cg_vert_location, 0);
}
// =============================================================================
//
// Outputs data to the mirror
//
void FT857D_output_mirror(uint8_t character, uint8_t line, uint8_t column)
{
  column--;
  switch (line) {
    case 0:
      mirror_line[0][column] = character;
      break;
    case 1:
      mirror_line[1][column] = character;
      break;
    case 2:
      mirror_line[2][column] = character;
      break;
    case 3:
      mirror_line[3][column] = character;
      break;
    case 4:
      mirror_line[4][column] = character;
      break;
  }  
}
// ****************************************************************************
//
// Parses the different packet types
//
void process_FT857D_packets(void)
{
  // If there are no packets to process return immediately
   if(FT857D_rx_fifo_input_pointer == FT857D_rx_fifo_output_pointer) return;

  // Skip the A5 character and get the number of characters in the packet
  get_rx_uart2_fifo_data();
  FT857D_packet_count = get_rx_uart2_fifo_data();

  // Get the packet type 
  FT857D_packet_type = get_rx_uart2_fifo_data(); 

  // Select which packet to process
  switch(FT857D_packet_type) {
    case 0x40:
      process_FT857D_packet_type_0x40();
      break;
    case 0x41:
      process_FT857D_packet_type_0x41();
      break;
    case 0x43:
      process_FT857D_packet_type_0x43();
      break;
    case 0x45:
      process_FT857D_packet_type_0x45();
      break;
    case 0x47:
      process_FT857D_packet_type_0x47();
      break;
    case 0x48:
      process_FT857D_packet_type_0x48();
      break;
    case 0x4A:
      process_FT857D_packet_type_0x4A();
      break;
    case 0x4B:
      process_FT857D_packet_type_0x4B();
      break;
    case 0x4C:
      process_FT857D_packet_type_0x4C();
      break;
    case 0x4D:
      process_FT857D_packet_type_0x4D();
      break;
    case 0x4E:
      process_FT857D_packet_type_0x4E();
      break;
  default:
      process_FT857D_packet_unknown();
  }
}
//
// *****************************************************************************
// Process type 40 which is the table look-up. The parameters passed is
//   FT857D_parm_d1   Array Number
//   FT857D_parm_d2   Element in the array
//
// Based on the array number, we get the following information
//   character size
//   starting horizontal location
//   starting vertical location
//   number of characters to output
// 
// *****************************************************************************
//
void process_FT857D_packet_type_0x40(void)
{ 
  uint16_t j;
  uint8_t  FT857D_line;
  uint8_t  FT857D_column;
  uint8_t  FT857D_length;  
  
  const char *array_type;

  // Get the array number and the element within the array
  FT857D_parm_d1    = get_rx_uart2_fifo_data();
  FT857D_parm_d2    = get_rx_uart2_fifo_data();

  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();

  // Calculate some parameters so you can display what is in the array.
  FT857D_line       = FT857D_define_array_location[FT857D_parm_d1][1];
  FT857D_column     = FT857D_define_array_location[FT857D_parm_d1][0];
  FT857D_length     = FT857D_define_array_length[FT857D_parm_d1];


  switch(FT857D_parm_d1) {
    case 0x00:
      FT857D_clear_all_mirror_lines();
      return;
    case 0x01:
      FT857D_clear_mirror_line(0);
      return;
    case 0x02:
      FT857D_clear_mirror_line(1);
      return;
    case 0x03:
      FT857D_clear_mirror_line(2);
      return;
    case 0x04:
      FT857D_clear_mirror_line(3);
      FT857D_clear_mirror_line_4_partial();
      return;
    case 0x05:
      array_type = &Type_40_05[FT857D_parm_d2][0];
      break;
    case 0x06:
      array_type = &Type_40_06[FT857D_parm_d2][0];
      break;
    case 0x07:
      array_type = &Type_40_07[FT857D_parm_d2][0];
      FT857D_display_menu_number();
      break;
    case 0x08:
      FT857D_clear_mirror_line(2);
      return;
    case 0x09:
      array_type = &Type_40_09[FT857D_parm_d2][0];
      break;
    case 0x0A:
      array_type = &Type_40_0A[FT857D_parm_d2][0];
      break;
    case 0x0B:
      array_type = &Type_40_0B[FT857D_parm_d2][0];
      break;
    case 0x0C:
      array_type = &Type_40_0C[FT857D_parm_d2][0];
      break;
    case 0x0D:
      array_type = &Type_40_0D[FT857D_parm_d2][0];
      break;
    case 0x0E:
      array_type = &Type_40_0E[FT857D_parm_d2][0];
      break;
    case 0x0F:
      array_type = &Type_40_0F[FT857D_parm_d2][0];
      break;
    case 0x10:
      array_type = &Type_40_10[FT857D_parm_d2][0];
      break;       
    case 0x11:
      array_type = &Type_40_11[FT857D_parm_d2][0];
      break;     
    case 0x12:
      array_type = &Type_40_12[FT857D_parm_d2][0];
      break;    
    case 0x13:
      array_type = &Type_40_13[FT857D_parm_d2][0];
      break;
    case 0x14:
      array_type = &Type_40_14[FT857D_parm_d2][0];
      break;
    case 0x15:
      array_type = &Type_40_15[FT857D_parm_d2][0];
      break;
    case 0x16:
      array_type = &Type_40_16[FT857D_parm_d2][0];
      break;
    case 0x17:
      array_type = &Type_40_17[FT857D_parm_d2][0];
      break;
    case 0x18:
      array_type = &Type_40_18[FT857D_parm_d2][0];
      break;
    case 0x19:
      array_type = &Type_40_19[FT857D_parm_d2][0];
      break;
    case 0x1A:
      array_type = &Type_40_1A[FT857D_parm_d2][0];
      break;
    case 0x1B:
      array_type = &Type_40_1B[FT857D_parm_d2][0];
      break;
    case 0x1C:
      array_type = &Type_40_1C[FT857D_parm_d2][0];
      break;
    case 0x1D:
      array_type = &Type_40_1D[FT857D_parm_d2][0];
      break;
    case 0x1E:
      array_type = &Type_40_1E[FT857D_parm_d2][0];
      break;
    case 0x20:
      array_type = &Type_40_20[FT857D_parm_d2][0];
      break;
    case 0x21:
      array_type = &Type_40_21[FT857D_parm_d2][0];
      break;      
    case 0x22:
      array_type = &Type_40_22[FT857D_parm_d2][0];
      break;
    case 0x23:
      array_type = &Type_40_23[FT857D_parm_d2][0];
      break;
    case 0x24:
      array_type = &Type_40_24[FT857D_parm_d2][0];
      break;
    case 0x25:
      array_type = &Type_40_25[FT857D_parm_d2][0];
      break;
    case 0x26:
      array_type = &Type_40_26[FT857D_parm_d2][0];
      break;
    case 0x27:
      array_type = &Type_40_27[FT857D_parm_d2][0];
      break;
    case 0x28:
      array_type = &Type_40_28[FT857D_parm_d2][0];
      break;  
    case 0x29:
      array_type = &Type_40_29[FT857D_parm_d2][0];
      break;  
    case 0x2A:
      array_type = &Type_40_2A[FT857D_parm_d2][0];
      break;
    case 0x2B:
      array_type = &Type_40_2B[FT857D_parm_d2][0];
      break;
    case 0x2C:
      array_type = &Type_40_2C[FT857D_parm_d2][0];
      break;
    case 0x2D:
      array_type = &Type_40_2D[FT857D_parm_d2][0];
      break;
    case 0x2E:
      array_type = &Type_40_2E[FT857D_parm_d2][0];
      break;
    case 0x2F:
      array_type = &Type_40_2F[FT857D_parm_d2][0];
      break;
    case 0x30:
      array_type = &Type_40_30[FT857D_parm_d2][0];
      break;      
    case 0x31:
      array_type = &Type_40_31[FT857D_parm_d2][0];
      break;
    case 0x32:
      array_type = &Type_40_32[FT857D_parm_d2][0];
      break;
    case 0x33:
      array_type = &Type_40_33[FT857D_parm_d2][0];
      if( FT857D_parm_d2 == 0) FT857D_rfsql_mode = 1;
      if( FT857D_parm_d2 == 1) FT857D_rfsql_mode = 2;
      break;
    case 0x34:
      array_type = &Type_40_34[FT857D_parm_d2][0];
      break;
    case 0x35:
      array_type = &Type_40_35[FT857D_parm_d2][0];
      break;
    case 0x36:
      array_type = &Type_40_36[FT857D_parm_d2][0];
      break;
    case 0x37:
      array_type = &Type_40_37[FT857D_parm_d2][0];
      break;
    case 0x39:
      array_type = &Type_40_39[FT857D_parm_d2][0];
      break;
    case 0x3A:
      array_type = &Type_40_3A[FT857D_parm_d2][0];
      break;
    case 0x3B:
      array_type = &Type_40_3B[FT857D_parm_d2][0];
      break;
    case 0x3C:
      array_type = &Type_40_3C[FT857D_parm_d2][0];
      break;
    case 0x3D:
      array_type = &Type_40_3D[FT857D_parm_d2][0];
      break;
    case 0x3E:
      array_type = &Type_40_3E[FT857D_parm_d2][0];
      break;       
    case 0x3F:
      FT857D_clear_mirror_line(2);
      return;
    case 0x40:
      array_type = &Type_40_40[FT857D_parm_d2][0];
      break; 
    case 0x41:
      array_type = &Type_40_41[FT857D_parm_d2][0];
      break; 
    case 0X42:
      array_type = &Type_40_42[FT857D_parm_d2][0];
      break; 
    case 0X43:
      array_type = &Type_40_43[FT857D_parm_d2][0];
      break; 
    case 0X44:
      array_type = &Type_40_44[FT857D_parm_d2][0];
      break; 
    case 0X45:            
      array_type = &Type_40_45[FT857D_parm_d2][0];
      break; 
    case 0x46:
      array_type = &Type_40_46[FT857D_parm_d2][0];
      break;   
    case 0x47:
      array_type = &Type_40_47[FT857D_parm_d2][0];
      break;  
    case 0x48:
      array_type = &Type_40_48[FT857D_parm_d2][0];
      break;  
    case 0x49:
      array_type = &Type_40_49[FT857D_parm_d2][0];
      break;  
    case 0x4A:
      array_type = &Type_40_4A[FT857D_parm_d2][0];
      break;  
    case 0x4B:
      array_type = &Type_40_4B[FT857D_parm_d2][0];
      break;  
    case 0x4C:
      array_type = &Type_40_4C[FT857D_parm_d2][0];
      break;
    case 0x4D:
      array_type = &Type_40_4D[FT857D_parm_d2][0];
      break;
    case 0x4E:
      array_type = &Type_40_4E[FT857D_parm_d2][0];
      break;  
    case 0x4F:
      array_type = &Type_40_4F[FT857D_parm_d2][0];
      break;  
    case 0x50:
      array_type = &Type_40_50[FT857D_parm_d2][0];
      break;    
    case 0x51:
      array_type = &Type_40_51[FT857D_parm_d2][0];
      break;  
    case 0x52:
      array_type = &Type_40_52[FT857D_parm_d2][0];
      break;
    case 0x53:   // special case where string has to be broken up into two different lines.
      return;
    default:
      process_FT857D_display_array_unknown();
      return;
  }

  // Output data to mirror and display.
  for (j = 0; j < FT857D_length; j++)
  {
    // display only if on the main screen
    if( active_screen_num == 0)
    {      
      FT857D_output_character(*array_type, FT857D_line, FT857D_column); 
    }
    // always store to mirror
    FT857D_output_mirror(*array_type++, FT857D_line, FT857D_column++);
  }
}  

// *****************************************************************************
// Process type 41. Output ASCII characters to the display
//   FT857D_parm_d1       Line Number
//   FT857D_parm_d2       Column Number
//   FT857D_packet_count  Characters (+ four extra characters)
// 
// *****************************************************************************

void process_FT857D_packet_type_0x41(void)
{
  uint8_t i;
  uint8_t j;
  uint8_t  FT857D_line;
  uint8_t  FT857D_column;  

  // Get the line number and the column number
  FT857D_line    = get_rx_uart2_fifo_data();  // Line Number
  FT857D_column  = get_rx_uart2_fifo_data();  // column number
  FT857D_column++;                            // Normalize to start at 1

  // Output data to mirror and display
  for (i = 0; i <  FT857D_packet_count - 4; i++) 
  {
    j = get_rx_uart2_fifo_data();
    if( active_screen_num == 0 )
    {
      FT857D_output_character(j, FT857D_line, FT857D_column); 
    }
    FT857D_output_mirror(j, FT857D_line, FT857D_column++);    
  }
  // Point to next packet in fifo
  get_rx_uart2_fifo_data();
}
// =============================================================================
//
//   Display squelch graph
//
//  Called on a full screen repaint
//  Packet 43 sets line status and does a partial
//    repaint of only graph bars needed
//
//
void FT857_display_squelch_graph( void )
{
  uint16_t starty_1;
  uint16_t starty_2; 
  uint16_t length;
  uint32_t i;
  
  RGB_color s_color;
  s_color.red   = 0;     // s meter start w/ black
  s_color.green = 0;
  s_color.blue  = 0;
  // light colored backgrounds use black  
  // if background is dark..
  if( (cg_background.red + cg_background.green + cg_background.blue) < 255 )
  {
    // use red unless lots of red in the color
    if( cg_background.red < 128 ) s_color.red = 255;
    else s_color.green = 255; // else green
  }
  for (i = 0; i < 101; i++)  
  {
      starty_1 = FT857D_Type_43[i][0];
      starty_2 = FT857D_Type_43[i][1];
      length   = FT857D_Type_43[i][2];    
    // erase if not active
    if (FT857D_Type_43_status[i] == 0)
    {
      draw_line(0, 130, starty_1, length, 1, cg_background );                  
      draw_line(0, 130, starty_2, length, 1, cg_background );   
    }
    // else draw lines
    else
    {
      draw_line(0, 130, starty_1, length, 1, s_color );
      draw_line(0, 130, starty_2, length, 1, s_color ); 
    }
  }
}
// =============================================================================
//
//  Set all graph lines to off
//    use at radio startup
//  
void FT857_clear_squelch_graph( void )
{
  int i;
  for (i = 0; i < 101; i++)  
  {
    FT857D_Type_43_status[i] = 0;
  }
}

//
// *****************************************************************************
// Process type 43. Output graph for the S meter. 
// FT857D_Type_43[][] contains the information for the shape of the graph on
// the screen. FT857D_Type_43_status[] contains the status for each line on 
// the screen. If the status for that line is a "1", then the line should be
// omn the screen.
// *****************************************************************************

void process_FT857D_packet_type_0x43(void)
{
  uint16_t starty_1;
  uint16_t starty_2; 
  uint16_t length;
  uint32_t i;
  
  RGB_color s_color;
  s_color.red   = 0;     // s meter start w/ black
  s_color.green = 0;
  s_color.blue  = 0;
  // Get graph value and point to next packet. Keep max. value to 100.
  FT857D_parm_d1 = get_rx_uart2_fifo_data();
  if (FT857D_parm_d1 > 100) FT857D_parm_d1 = 100;
  get_rx_uart2_fifo_data();
  // Set Status to indicate graph as line active
  FT857D_Type_43_status[FT857D_parm_d1] = 1;

  // light colored backgrounds use black  
  // if background is dark..
  if( (cg_background.red + cg_background.green + cg_background.blue) < 255 )
  {
    // use red unless lots of red in the color
    if( cg_background.red < 128 ) s_color.red = 255;
    else s_color.green = 255; // else green
  }

  // display only if on the main screen
  if( active_screen_num == 0)
  { 
    // Output two lines specified
    starty_1 = FT857D_Type_43[FT857D_parm_d1][0];
    starty_2 = FT857D_Type_43[FT857D_parm_d1][1];
    length   = FT857D_Type_43[FT857D_parm_d1][2];
    draw_line(0, 130, starty_1, length, 1, s_color);
    draw_line(0, 130, starty_2, length, 1, s_color); 

    // Erase lines above it if lines are active
    for (i = FT857D_parm_d1+1; i < 101; i++)
    {
      if (FT857D_Type_43_status[i] == 1)
      {
        FT857D_Type_43_status[i] = 0;
        starty_1 = FT857D_Type_43[i][0];
        starty_2 = FT857D_Type_43[i][1];
        length   = FT857D_Type_43[i][2];
        draw_line(0, 130, starty_1, length, 1, cg_background );                  
        draw_line(0, 130, starty_2, length, 1, cg_background );   
      }
    }
    // Insert lines below if not active
    if (FT857D_parm_d1 == 0) return;

    for (i = 0; i < FT857D_parm_d1; i++)
    {
      if (FT857D_Type_43_status[i] == 0)
      {
        FT857D_Type_43_status[i] = 1;
        starty_1 = FT857D_Type_43[i][0];
        starty_2 = FT857D_Type_43[i][1];
        length   = FT857D_Type_43[i][2];
        draw_line(0, 130, starty_1, length, 1, s_color );
        draw_line(0, 130, starty_2, length, 1, s_color ); 
      }
    }
  }
}
// *****************************************************************************
// Process type 45. Output underline for entering data.
//
// The packet for processing underline is A5 04 45 LL PP CS
//
// LL is line number and should always be a 2 and PP is position and should be
// between 1 and 12.
// *****************************************************************************
// 
void process_FT857D_packet_type_0x45(void)
{
  // Get the line number and the position for the cursor.
  // Set the flag to allow the mainline program to process it.
  FT857D_parm_d1          = get_rx_uart2_fifo_data();
  FT857D_cursor_position  = get_rx_uart2_fifo_data() - 1;

  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();   

  // If cursor pointer is zero, then its a new cursor and we just turn it on.
  if (FT857D_cursor_pointer == 0)
  {
      FT857D_cursor_pointer = FT857D_cursor_position;
      FT857D_turn_on_cursor();
      FT857D_cursor_timer = 50;    
  }
  else
  {
      FT857D_turn_off_cursor();
      FT857D_cursor_pointer = FT857D_cursor_position;
      FT857D_turn_on_cursor();
      FT857D_cursor_timer = 50;    
  }
}
// =============================================================================
//
//  clears spectrum area
//
void clear_spectrum( void )
{
  clear_rectangle(128,73,648,284);
}

// *****************************************************************************
// Turn off blinking cursor and set spectrum analyzer parameters
// *****************************************************************************
void process_FT857D_packet_type_0x47(void)
{
  // The packet A5 04 47 03 42 8C shuts off the cursor.

  // Get the line number and the position for the cursor.
  // Set the flag to allow the mainline program to process it.
  FT857D_parm_d1  = get_rx_uart2_fifo_data();
  FT857D_parm_d2  = get_rx_uart2_fifo_data();
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();   
  // Check if its a terminate cursor command
  spectrum_flag = 0;
  
  if (FT857D_parm_d1 == 0x03)
  {
    switch(FT857D_parm_d2) {
      case 0x42:                          // turn off cursor
        FT857D_turn_off_cursor();
        FT857D_cursor_pointer = 0; 
        break;
      case 0xC2:                         // Define Fine Screen
        clear_spectrum();        
        spectrum_hold_flag = 0;
        spectrum_mode = 1;
        break;
      case 0xCA:                         // Define Medium Screen
        clear_spectrum(); 
        spectrum_hold_flag = 0;
        spectrum_mode = 2;
        break;
      case 0xD2:                         // Define Large Screen
        clear_spectrum();         
        spectrum_hold_flag = 0;
        spectrum_mode = 3;
        break;
      case 0x62:                         // Exit Fine Screen
        clear_spectrum(); 
        clear_spectrum_hold_and_last();
        spectrum_mode = 0;
        break;
      case 0x4A:                         // Exit Fine Screen
        clear_spectrum(); 
        clear_spectrum_hold_and_last();
        spectrum_mode = 0;
        spectrum_hold_flag = 0;
        break;
      case 0x52:                         // Exit Large Screen
        clear_spectrum(); 
        clear_spectrum_hold_and_last();
        spectrum_mode = 0;
        spectrum_hold_flag = 0;
        break;
      case 0x6A:                         // Exit Medium Screen with hold
        clear_spectrum(); 
        clear_spectrum_hold_and_last();
        spectrum_mode = 0;
        spectrum_hold_flag = 0;
        break;        
      case 0x72:                         // Exit Large Screen with hold
        clear_spectrum(); 
        clear_spectrum_hold_and_last();
        spectrum_mode = 0;
        spectrum_hold_flag = 0;
        break;                     
      case 0xF2:                         // Peak Hold large Screen
        clear_spectrum(); 
        spectrum_hold_flag = 1;
        spectrum_mode = 3;
        break;
      case 0xEA:                         // Peak Hold Medium Screen
        clear_spectrum();         
        spectrum_hold_flag = 1;
        spectrum_mode = 2;
        break;
      case 0xE2:                         // Peak Hold Fine Screen
        clear_spectrum();         
        spectrum_hold_flag = 1;
        spectrum_mode = 1;
        break; 
      default:
        spectrum_mode = 0;
        break;
    }
    return;
  } 
  if (FT857D_parm_d1 == 0 )
  {  
    // 47 00 00 or 47 00 02  at startup. Don't know what it means
    spectrum_mode = 0;                  
    return;    
  }
  // If we got to this point, there was an unknown packet being sent.
  // Store it for future analysis
  error_log( 3051 );
}
// *****************************************************************************
// Process spectrum analyzer packets.
//
// *****************************************************************************
void process_FT857D_packet_type_0x48(void)
{
  // Get the position and the value for this packet.
  FT857D_parm_d1  = get_rx_uart2_fifo_data();
  FT857D_parm_d2  = get_rx_uart2_fifo_data();
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();  
  // update the hold parameter
  if (FT857D_parm_d2 > spectrum_hold[FT857D_parm_d1])
  {
    spectrum_hold[FT857D_parm_d1] = FT857D_parm_d2;
  } 
  // Outpput data to the screen
  if( active_screen_num == 0 )
  {
    paint_spectrum_rectangle();
  }
}
// *****************************************************************************
// This packet changes the color of the display
// This packet is used for setting up the color. There are 32 colors. D1 and D2 
// define the 32 possible colors.  This packet controls the RGB LED backlight 
// of the LCD display and there are four bits per color as follows:
// Bit   7        6        5        4        3        2        1        0
// D1  green    green    green    green    blue      blue     blue     blue
// D2                                      red       red      red      red
//
// *****************************************************************************
//
void process_FT857D_packet_type_0x4A(void)
{
  // The packet A5 04 4A D1 D2 CS selects the color
  // Get the D1 and D2. Then erase the display and repaint the 5 lines of data
  FT857D_parm_d1  = get_rx_uart2_fifo_data();
  FT857D_parm_d2  = get_rx_uart2_fifo_data();
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();
  
  cg_font.blue  = FT857D_colors[FT857D_parm_d1 & 0X0F][0];
  cg_font.green = FT857D_colors[FT857D_parm_d1 >> 4][0];
  cg_font.red   = FT857D_colors[FT857D_parm_d2 & 0X0F][0];
  
  cg_background.blue =  FT857D_colors[FT857D_parm_d1 & 0X0F][1];
  cg_background.green = FT857D_colors[FT857D_parm_d1 >> 4][1];
  cg_background.red =   FT857D_colors[FT857D_parm_d2 & 0X0F][1]; 
  
  // store radio screen colors for repaint
  RGB_copy( &FT857D_font, &cg_font);
  RGB_copy( &FT857D_background, &cg_background);
  
  if( active_screen_num == 0 )
  {
    clear_radio_screen_and_rewrite_lines();
  }
}
// *****************************************************************************
// This program turns on and off the Red and Green LED. Note: For this packet,
// There is a parameter for turning on the blue led. Their is no blue led in
// this hardware configuration. There is also a parameter for turning on and off
// the switch backlights. There is no on/off switch for the switch backlights.
// Bit 0 controls the Red LED. 0 turns it on and 1 turns it off.
// Bit 1 controls the Green LED. 0 turns it on and 1 turns it off.
// *****************************************************************************
//
void process_FT857D_packet_type_0x4B(void)
{
  // Get the parameter and turn on and off the leds as appropriate
  FT857D_parm_d1  = get_rx_uart2_fifo_data();
  // turn on and off the REDS led according to the parameter
  if (FT857D_parm_d1 & 0x01)
  {
    tx_led_off();
  }
  else
  {
    tx_led_on();
  }
  // turn on and off the REDS led according to the parameter
  if (FT857D_parm_d1 & 0x02)
  {
    rx_led_off();
  }
  else
  {
    rx_led_on();
  }
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();
}
// *****************************************************************************
// This packet is used for an external analog meter. We do not support that
// feature in this version of the hardware. This program will accept the packet
// and then return without doing anything.
// *****************************************************************************
void process_FT857D_packet_type_0x4C(void)
{
  // Get the parameter associated with this packet
  get_rx_uart2_fifo_data();
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();
}
// *****************************************************************************
// Process contrast
// *****************************************************************************
void process_FT857D_packet_type_0x4D(void)
{
  uint8_t parm1;
  uint8_t parm2;
  uint8_t parm3;
  uint8_t parm4;
  uint32_t get_post_process;

  // Get the D1 and D2. Then erase the display and repaint the 5 lines of data
  FT857D_parm_d1  = get_rx_uart2_fifo_data();
  // Adjust fifo output pointer so it is now pointing to the next packet
  get_rx_uart2_fifo_data();
  // Change contrast
  FT857D_parm_d1 = FT857D_parm_d1 * 5;  //multiplier is by experiment
  get_post_process = ssd1963_get_post_proc();
  parm1 = (FT857D_parm_d1        ) & 0x000000FF;
  parm2 = (get_post_process >>  8) & 0x000000FF;
  parm3 = (get_post_process >> 16) & 0x000000FF;
  parm4 = 1;
  ssd1963_set_post_proc(parm1, parm2, parm3, parm4);  
}
// =============================================================================
//
//  Packet type 4E.

// These are sent by the radio at startup, in this order
//  4E 99
//  4E 9A
//  4E 91
//  4E 97
//  4E 96 or 4E 95
// 
void process_FT857D_packet_type_0x4E(void)
{
  uint8_t parm1;
  // Get the parameter associated with this packet
  parm1 = get_rx_uart2_fifo_data();
  // Adjust fifo output pointer to the next packet
  get_rx_uart2_fifo_data();
  
  switch( parm1 )
  {
  case 0x95: // squelch mode 
    {
      FT857D_auto_init_trigger = 1;
      FT857D_rfsql_mode = 2;  //squelch
      break;
    }
  case 0x96: // RF gain mode
    {
      FT857D_auto_init_trigger = 1;
      FT857D_rfsql_mode = 1;  //rf gain
      break;
    }
    // don't know what the rest mean but these are seen
    // assume 92,93,94 and 98 exist
  case 0x91:
  case 0x97:
  case 0x99:
  case 0x9A:
    break;
  default:
    error_log( 3052 );
    break;
  }
}
// =============================================================================
//
//
void process_FT857D_packet_unknown(void)
{
uint8_t i; 
  FT857D_parm_d1 = get_rx_uart2_fifo_data();
  for (i = 0; i < FT857D_packet_count - 3; i++)
  {
    get_rx_uart2_fifo_data();
  }
  FT857D_parm_d2 = get_rx_uart2_fifo_data();
  unknown_array_array[unknown_array_pointer  ][0] = FT857D_packet_count;
  unknown_array_array[unknown_array_pointer  ][1] = FT857D_packet_type;
  unknown_array_array[unknown_array_pointer  ][2] = FT857D_parm_d1;
  unknown_array_array[unknown_array_pointer++][3] = FT857D_parm_d2;
  // log unknown packet err
  error_log( 3051 );  
}
// *****************************************************************************
// process error condition of unknown display
// *****************************************************************************
//
void process_FT857D_display_array_unknown(void)
{
  unknown_array_array[unknown_array_pointer  ][0] = FT857D_packet_count;
  unknown_array_array[unknown_array_pointer  ][1] = FT857D_packet_type;
  unknown_array_array[unknown_array_pointer  ][2] = FT857D_parm_d1;
  unknown_array_array[unknown_array_pointer++][3] = FT857D_parm_d2;  
  // log packet info err
  error_log( 3052 );    
}
// ****************************************************************************
// Read data from fifo
// ****************************************************************************
uint8_t get_rx_uart2_fifo_data(void)
{
uint8_t rxdata;
  rxdata = FT857D_rx_fifo[(FT857D_rx_fifo_output_pointer)];
  FT857D_rx_fifo_output_pointer++;
  if (FT857D_rx_fifo_output_pointer > 1023) FT857D_rx_fifo_output_pointer = 0;
  return rxdata;
}

// =============================================================================
// 
//   Fill a screen line with blank spaces
//     and store spaces in mirror line array    
//
//  History: 
//   06/07/2019 revised to use mirror_lines in an array,
//              and pass line number to use one function for all lines
//
// mirror line lengths
uint8_t mirr_length[5] = { 19, 20, 12, 3, 22 };

void FT857D_clear_mirror_line( uint8_t line_num )
{
  int i;
  
  for (i = 0; i < mirr_length[line_num]; i++) 
  {
    mirror_line[line_num][i] = 0x20;
    FT857D_output_character(0x20, line_num, i+1); 
  }     
}
// =============================================================================

// *****************************************************************************
// Clear mirror line 4 columns 1 - 19
//

void FT857D_clear_mirror_line_4_partial(void)
{
  int i;
  for (i = 0; i < 19; i++) 
  {
    mirror_line[4][i] = 0x20;
    FT857D_output_character(0x20, 4, i+1); 
  }
}
// ****************************************************************************
// Clear all mirror lines
// ****************************************************************************
//
//  History: 
//   06/07/2019 revised to use mirror_lines in an array and run in a loop
//
void FT857D_clear_all_mirror_lines(void)
{
  int i;
  for( i = 0; i < 5; i++)
  {
    FT857D_clear_mirror_line(i);
  } 
}
//
// *****************************************************************************
// Display menu number in columns 14-16
// *****************************************************************************
//
void FT857D_display_menu_number(void)
{
uint8_t i;
  sprintf((char *) cg_line_buffer, "%03u",FT857D_parm_d2+1);
  for (i = 0; i <  3; i++) 
  {
    if( active_screen_num == 0 )
    {
      FT857D_output_character(cg_line_buffer[i], 0, 14+i); 
    }
    FT857D_output_mirror(cg_line_buffer[i], 0, 14+i);    
  }
  
}
// *****************************************************************************
// Process blinking cursor
// 
// The packet 45 processing program passes FT857D_cursor_position to indicate 
// position of the blinking cursor. If this variable is non zero it indicates
// there is a new position to process and if zero it indicates the posistion 
// has been processed.
//
// When FT857D_cursor_terminate is set to a 1 from packet 47, it indicates the 
// blinking cursor is terminated.
//
// Being that this is a state program, FT857D_cursor_state is the current state.
//
// FT857D_cursor_pointer is the current position of the blinking cursor. Its 
// valus should be between 1 and 12 when active.
//
// When FT857D_cursor_timer is zero, its time to process the blink function.
//
void process_blinking_cursor(void)
{
  // If FT857D_cursor_pointer is zero, then there are no cursors to process.
  if (FT857D_cursor_pointer == 0) return;
  
  // If the timer is zero, we turn off the cursor, and go to the next state
  if (FT857D_cursor_state == 0)
  {
    if (FT857D_cursor_timer == 0)
    {
      FT857D_turn_off_cursor();
      FT857D_cursor_timer = 50;
      FT857D_cursor_state++;  
    }
    return;
  }

  // If the timer is zero, we turn on the cursor, and go to the previous state
  if (FT857D_cursor_state == 1)
  {
    if (FT857D_cursor_timer == 0)
    {
      FT857D_turn_on_cursor();
      FT857D_cursor_timer = 50;
      FT857D_cursor_state--;  
    }
    return;
  } 

  // We should never get here but if we do we clear the state counter
  //and turn off the cursor.
  FT857D_turn_off_cursor();
  FT857D_cursor_state = 0;
  error_log( 3002 );
}
// =============================================================================
//    Turn on
//
// This program turns on the cursor on line 2 at position FT857D_cursor_pointer.
// FT857D_cursor_pointer can have a value between 1 and 12.
//
void FT857D_turn_on_cursor(void)
{
  uint16_t x_position;
  uint16_t y_position;

  if (FT857D_cursor_pointer == 0) return;
  if (FT857D_cursor_pointer > 12) return;

  // calculate the position of the cursor
  //x_position = calculate_address_of_column(2, FT857D_cursor_pointer+1);
  x_position = calculate_address_of_column(2, FT857D_cursor_pointer+2);
  // y_position = calculate_address_of_line(2) + 69 + 5;
  y_position = FT857_Message[ 2 ].y_pos + 69 + 5;
  draw_rectangle( x_position, y_position, 40, 2, cg_font );
}
// =============================================================================
//    Turn off
//
//
void FT857D_turn_off_cursor(void)
{
  uint16_t x_position;
  uint16_t y_position;

  if (FT857D_cursor_pointer == 0) return;
  if (FT857D_cursor_pointer > 12) return;

  // calculate the position of the cursor
  //x_position = calculate_address_of_column(2, FT857D_cursor_pointer+1);
  x_position = calculate_address_of_column(2, FT857D_cursor_pointer+2);
  //y_position = calculate_address_of_line(2) + 69 + 5;
  y_position = FT857_Message[ 2 ].y_pos + 69 + 5;  
  draw_rectangle( x_position, y_position, 40, 2, cg_background );
}
//
// *****************************************************************************
//  clear_radio_screen_and_rewrite_lines
//
//  used to redraw the screen after a color change and for screen redisplay
//
// *****************************************************************************
void clear_radio_screen_and_rewrite_lines( void )
{
  int i;
  int line_num;
  
  // restore radio screen colors before clearing
  RGB_copy( &cg_font, &FT857D_font );
  RGB_copy( &cg_background, &FT857D_background );
  
  clear_rectangle( 128, 73, 648, 284 );

  for( line_num = 0; line_num < 5; line_num++ )
  {
    for (i = 0; i < mirr_length[line_num]; i++)
    {
      FT857D_output_character(mirror_line[line_num][i], line_num, i+1); 
    }
  }
  
  // Redraw squelch graph
  FT857_display_squelch_graph();

  // redraw squelch and volume bars
  if (special_encoder_flag)
  {
    FT857_display_squelch_bar(FT857_squelch_value); 
    FT857_display_volume_bar(FT857_volume_value);
  }
  else
  {
    FT857_erase_squelch_bar();
    FT857_erase_volume_bar();
  }
}
// =============================================================================
//
//   Spectrum display routines
//
void clear_spectrum_hold(void)
{
  uint32_t i;
  for (i = 0; i < 127; i++) spectrum_hold[i] = 0;  
}
void clear_spectrum_last(void)
{
  uint32_t i;
  for (i = 0; i < 127; i++) spectrum_last[i] = 0;  
}
void clear_spectrum_hold_and_last(void)
{
  clear_spectrum_hold();
  clear_spectrum_last();
}
void paint_spectrum_rectangle(void)
{
  uint16_t start_x;
  uint16_t start_y;
  uint16_t height;

  // FT857D_parm_d1 is position and can have a value of 0 - 126
  // FT857D_parm_d2 is value and can have a value between 0 and 100
  // spectrum_mode = 1 fine screen
  // spectrum_mode = 2 medium screen
  // spectrum_mode = 3 large screen
  // spectrum_hold_flag = 0 for disabled hold screen and 1 for enabled hold screen
 
  if (spectrum_mode == 1)                                // Fine
  {
    height = 2 * FT857D_parm_d2 ;
    start_x = 135 + (FT857D_parm_d1 * 5);
    start_y = 273- (FT857D_parm_d2 * 2);
    clear_rectangle(start_x, 73, 3, 200);
    color_rectangle(start_x, start_y, 3, height);
    
    // If hold active, put bigest value on screen
    if (spectrum_hold_flag)
    {
      start_y = 273- (spectrum_hold[FT857D_parm_d1] * 2);
      color_rectangle(start_x, start_y, 3, 1);      
    }
    return;
  }
  if (spectrum_mode == 2)                                // Medium
  {
    height = 2 * FT857D_parm_d2 ;
    start_x = 158 + (FT857D_parm_d1 * 19);
    start_y = 273- (FT857D_parm_d2 * 2);
    clear_rectangle(start_x, 73, 11, 200);
    color_rectangle(start_x, start_y, 11, height);

    // If hold active, put bigest value on screen
    if (spectrum_hold_flag)
    {
      start_y = 273- (spectrum_hold[FT857D_parm_d1] * 2);
      color_rectangle(start_x, start_y, 11, 1);      
    }
    return;
  }
  if (spectrum_mode == 3)                                // Large
  {
    height = 2 * FT857D_parm_d2 ;
    start_x = 158 + (FT857D_parm_d1 * 28);
    start_y = 273- (FT857D_parm_d2 * 2);
    clear_rectangle(start_x, 73, 20, 200);
    color_rectangle(start_x, start_y, 20, height);

    // If hold active, put bigest value on screen
    if (spectrum_hold_flag)
    {
      start_y = 273- (spectrum_hold[FT857D_parm_d1] * 2);
      color_rectangle(start_x, start_y, 20, 1);      
    }
    return;
  }
}
// =============================================================================
//
//   Save FT857 radio info to eeprom
//
//  Saves: type of squelch/RF gain for the knob control
//         squelch knob position
//         volume knob position
//
//   more radio info will probably be stored here
//
// History 05/09/2019 added variables for FT857

// eeprom address for radio type
// 0x2500
#define FT857_INFO_HI  0x27
#define FT857_INFO_LO  0x00

// values that are stored on eprom, no need to reread eprom
uint8_t FT857_savd_sq_type = 0;
uint8_t FT857_savd_squelch_value = 0;
uint8_t FT857_savd_volume_value = 0;

void save_FT857_info( uint8_t sq_type, uint8_t sq_knob, uint8_t vol_knob  )
{
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout( FT857_INFO_HI );  
  I2C_put_dataout( FT857_INFO_LO );

  I2C_put_dataout( sq_type );   
  I2C_put_dataout( sq_knob );
  I2C_put_dataout( vol_knob ); 
  
  // write to EEPROM  3+3 6 bytes
  EEPROM_MultipleWrite( 0, 6);

  FT857_savd_sq_type = sq_type;
  FT857_savd_squelch_value = sq_knob;
  FT857_savd_volume_value = vol_knob;
  
  // EEPROM_waitfor_I2C();  // this hung, can't wait here, it's used in the isr
}
// =============================================================================
//
//   Load FT857 radio info from eeprom
//
//  loads: type of squelch/RF gain for the knob control
//         squelch knob position
//         volume knob position
//
// History 05/09/2019 added variables for FT857

// this needs to be tied to radio packets info
uint8_t FT857_sq_type = 0;

void load_FT857_info( void )
{
  // read 
  EEPROM_MemRead( 0, FT857_INFO_HI,FT857_INFO_LO, 3);
  // wait for I2C to finish
  EEPROM_waitfor_I2C();  
  FT857_sq_type = I2C_get_datain();
  FT857_squelch_value = I2C_get_datain();
  FT857_volume_value = I2C_get_datain();
  
  FT857_savd_sq_type = FT857_sq_type;
  FT857_savd_squelch_value = FT857_squelch_value;
  FT857_savd_volume_value = FT857_volume_value;
  // default. first load, or bad data ( type should be 0 or 1 )
  // the first load from the eeprom will be 0xFF
  if( FT857_sq_type > 1 )
  {
    FT857_sq_type = 0;
    FT857_squelch_value = 0;
    FT857_volume_value = 0;  
  }
}
// =============================================================================
//
// Setup buffer for release packet and then start the transmission by sending 
// the first byte of the packet which is 0xA5  
//
//  History: 05/10/2019 moved here from touch_panel.c 
//
void process_release_packet(void)
{
  // setup buffer for the Key release packet
  FT857D_tx_buffer[0] = 0x06;     
  FT857D_tx_buffer[1] = 0x91;
  FT857D_tx_buffer[2] = 0xFF;
  FT857D_tx_buffer[3] = 0x8F;
  FT857D_tx_buffer[4] = 0x18;
  FT857D_tx_buffer[5] = 0x00;
  FT857D_tx_buffer[6] = 0x37;
  uart_FT857D_send_0xA5();
}
// =============================================================================
//
// set volume
//
void FT857D_set_volume( uint8_t volume )
{
  FT857D_tx_buffer[0] = 0x03; 
  FT857D_tx_buffer[1] = 0x97;
  FT857D_tx_buffer[2] = volume;
  FT857D_tx_buffer[3] = (0x97 + volume) & 0xFF;
  uart_FT857D_send_0xA5(); 
  RKnob_delay_timer = Knob_Timer_Delay;  
}
// =============================================================================
//
// set squelch
//
void FT857D_set_squelch( uint8_t squelch )
{
  FT857D_tx_buffer[0] = 0x03; 
  FT857D_tx_buffer[1] = 0x95;
  FT857D_tx_buffer[2] = squelch;
  FT857D_tx_buffer[3] = (0x95 + squelch) & 0xFF;
  uart_FT857D_send_0xA5(); 
  LKnob_delay_timer = Knob_Timer_Delay;  
}
// =============================================================================
//
//       absolute value of int
//
int absval( int val )
{
  int abs;

  if (val < 0 ) abs = -val;
  else abs = val;
  return abs;
}

// =============================================================================
//
//   FT857D_mechanical_switches
//
//  History: 05/10/2019 moved here from touch_panel.c
//           07/09/2019 added timers and counters to knobs to handle fast turing
//
// This program handles the mechanical switches. The definition of the switches
// are as follows:
//
// FT857D_left_encoder_switch      Encoder left switch (select push key)
// FT857D_key_released             Encoder left switch (select push key)
//
// FT857D_left_encoder_switch      Encoder right switch
// FT857D_key_released             Encoder right switch
//
// FT857D_p1_switch                P1.
// FT857D_key_released             P1
//
// FT857D_p2_switch                P2
// FT857D_key_released             P2
//
// FT857D_p3_switch                P3
// FT857D_key_released             P3
//
// FT857D_p4_switch                P4
// FT857D_key_released             P4
//
// FT857D_encoder_rt_cw            Encoder right clockwise (Channel/Volume)
// FT857D_encoder_rt_ccw           Encoder right counter clockwise (Channel/volume)
//
// FT857D_encoder_lft_cw           Encoder left clockwise (select/squelch)
// FT857D_encoder_lft_ccw          Encoder left counter clockwise (select/squelch)
//

uint32_t RKnob_delay_timer = 0;
uint32_t LKnob_delay_timer = 0;
int  RKnob_counter = 0;   // signed, cw goes positve, ccw goes negative
int  LKnob_counter = 0;
int  step_count = 0;     // count to send

void FT857D_mechanical_switches(void)
{
  // P1 is used to select between normal encoder use and special encoder use.
  // When in special encoder use, the right encoder is used for volume control
  // and the left encoder is used for squelch adjustment.
  if (FT857D_p1_switch )
  {
    FT857D_p1_switch = 0;
    special_encoder_flag ^= 1;  // toggle flag
    if (special_encoder_flag)
    {
      FT857_display_squelch_bar(FT857_squelch_value); 
      FT857_display_volume_bar(FT857_volume_value);
    }
    else
    {
      FT857_erase_squelch_bar();
      FT857_erase_volume_bar();
    }
  }

  // If you are not ready to transmit, stop right here.
  if ( !FT857D_uart1_isready() ) return;

  // process release key
  if (FT857D_key_released)
  {
    FT857D_key_released = 0;
    process_release_packet();
    return;
  }  

  // process select push key
  if (FT857D_left_encoder_switch )
  { 
    FT857D_left_encoder_switch = 0;
    FT857D_tx_buffer[0] = 0x06;     // Setup Select push key packet
    FT857D_tx_buffer[1] = 0x91;
    FT857D_tx_buffer[2] = 0xFF;
    FT857D_tx_buffer[3] = 0x8F;
    FT857D_tx_buffer[4] = 0x08;
    FT857D_tx_buffer[5] = 0x00;
    FT857D_tx_buffer[6] = 0x27;
    uart_FT857D_send_0xA5(); 
    return;
  }

  // Process select rotary key clockwise
  if (FT857D_encoder_lft_cw && (!special_encoder_flag))
  {
    FT857D_encoder_lft_cw = 0; 
    // increment step counter, limit to 6
    if( LKnob_counter < 6 ) LKnob_counter++;
    // absolute value of counter
    step_count = absval( LKnob_counter );

    //log_event( "Lknob cw click" );
    // send if enough time has passed since last
    if( LKnob_delay_timer == 0 && step_count != 0)
    {    
      LKnob_counter = 0;

      FT857D_tx_buffer[0] = 0x04; 
      FT857D_tx_buffer[1] = 0x93;
      FT857D_tx_buffer[2] = 0x00;
      FT857D_tx_buffer[3] = step_count;  // steps since last tranmission
      FT857D_tx_buffer[4] = (FT857D_tx_buffer[1] + FT857D_tx_buffer[2] +
                             FT857D_tx_buffer[3] ) & 0xFF;  // checksum
      uart_FT857D_send_0xA5();
      LKnob_delay_timer = Knob_Timer_Delay;
    }
    log_event( "select knob cw" ); 
    return;
  }

  // Process select rotary key counter clockwise
  if (FT857D_encoder_lft_ccw && (!special_encoder_flag))
  {
      
    FT857D_encoder_lft_ccw = 0;    
    // decrement step counter, limit to 6
    if( LKnob_counter > -6 ) LKnob_counter--;
    // absolute value of counter
    step_count = absval( LKnob_counter );    

    //log_event( "Lknob ccw click" );    
    // send if enough time has passed since last
    if( LKnob_delay_timer == 0 && step_count != 0)
    {    
      LKnob_counter = 0;

      FT857D_tx_buffer[0] = 0x04; 
      FT857D_tx_buffer[1] = 0x93;
      FT857D_tx_buffer[2] = 0x80;
      FT857D_tx_buffer[3] = step_count;  // steps since last tranmission
      FT857D_tx_buffer[4] = (FT857D_tx_buffer[1] + FT857D_tx_buffer[2] +
                             FT857D_tx_buffer[3] ) & 0xFF;  // checksum
      uart_FT857D_send_0xA5(); 
      LKnob_delay_timer = Knob_Timer_Delay;
      log_event( "select knob ccw" );       
    }
    return;
  }
  // Process squelch key clockwise
  if (FT857D_encoder_lft_cw && special_encoder_flag)
  {
    FT857D_encoder_lft_cw = 0;
    // increment step counter, limit to 6
    if( LKnob_counter < 6 ) LKnob_counter++;
    // absolute value of counter
    step_count = absval( LKnob_counter );    
    
    // send if enough time has passed since last
    if( LKnob_delay_timer == 0 && step_count != 0)
    {    
      LKnob_counter = 0;    

      if (FT857_squelch_value < 0xFF - step_count)
      {
        FT857_squelch_value += step_count;
      }
      else FT857_squelch_value = 0xFF;
          
      FT857D_set_squelch( FT857_squelch_value );
      FT857_display_squelch_bar(FT857_squelch_value);
      if( FT857D_rfsql_mode == 1 ) log_event( "rf gain +" );
      if( FT857D_rfsql_mode == 2 ) log_event( "squelch +" );
    }
    return;
  }

  // Process squelch key counter clockwise
  if (FT857D_encoder_lft_ccw && special_encoder_flag)
  {
    FT857D_encoder_lft_ccw = 0;
    // decrement step counter, limit to 6
    if( LKnob_counter > -6 ) LKnob_counter--;
    // absolute value of counter
    step_count = absval( LKnob_counter );    
    
    // send if enough time has passed since last
    if( LKnob_delay_timer == 0 && step_count != 0)
    {    
      LKnob_counter = 0; 
      if (FT857_squelch_value > step_count)
      {
        FT857_squelch_value -= step_count;
      }
      else FT857_squelch_value = 0;      
      
      FT857D_set_squelch( FT857_squelch_value );
      FT857_display_squelch_bar(FT857_squelch_value);
      if( FT857D_rfsql_mode == 1 ) log_event( "rf gain -" );
      if( FT857D_rfsql_mode == 2 ) log_event( "squelch -" );     
    }
    return;
  }  

  // Process channel rotary key clockwise
  if (FT857D_encoder_rt_cw && (!special_encoder_flag))
  {
    FT857D_encoder_rt_cw = 0;
    // increment step counter, limit to 6
    if( RKnob_counter < 6 ) RKnob_counter++;
    // absolute value of counter
    step_count = absval( RKnob_counter );    

    // send if enough time has passed since last
    if( RKnob_delay_timer == 0 && step_count != 0)
    {    
      RKnob_counter = 0;    
      FT857D_tx_buffer[0] = 0x04; 
      FT857D_tx_buffer[1] = 0x92;
      FT857D_tx_buffer[2] = 0x00;
      FT857D_tx_buffer[3] = step_count;  // steps since last tranmission
      FT857D_tx_buffer[4] = (FT857D_tx_buffer[1] + FT857D_tx_buffer[2] +
                             FT857D_tx_buffer[3] ) & 0xFF;  // checksum
      uart_FT857D_send_0xA5();
      RKnob_delay_timer = Knob_Timer_Delay;       
    }
    return;
  }

  // Process channel rotary key counter clockwise
  if (FT857D_encoder_rt_ccw && (!special_encoder_flag))
  {
    FT857D_encoder_rt_ccw = 0;
    // decrement step counter, limit to 6
    if( RKnob_counter > -6 ) RKnob_counter--;
    // absolute value of counter
    step_count = absval( RKnob_counter );    

    // send if enough time has passed since last
    if( RKnob_delay_timer == 0 && step_count != 0)
    {    
      RKnob_counter = 0;    
      FT857D_tx_buffer[0] = 0x04; 
      FT857D_tx_buffer[1] = 0x92;
      FT857D_tx_buffer[2] = 0x80;
      FT857D_tx_buffer[3] = step_count;  // steps since last tranmission
      FT857D_tx_buffer[4] = (FT857D_tx_buffer[1] + FT857D_tx_buffer[2] +
                             FT857D_tx_buffer[3] ) & 0xFF;  // checksum
      uart_FT857D_send_0xA5(); 
      uart_FT857D_send_0xA5();
      RKnob_delay_timer = Knob_Timer_Delay; 
    }      
    return;
  }

  // Process volume rotary clockwise
  if (FT857D_encoder_rt_cw && special_encoder_flag)
  {
    FT857D_encoder_rt_cw = 0;
    // increment step counter, limit to 6
    if( RKnob_counter < 6 ) RKnob_counter++;
     // absolute value of counter
    step_count = absval( RKnob_counter );   
    
    // send if enough time has passed since last
    if( RKnob_delay_timer == 0 && step_count != 0)
    {    
      RKnob_counter = 0;     

      if (FT857_volume_value < 0xFF - step_count)
      {
        FT857_volume_value += step_count;
      }
      else FT857_volume_value = 0xFF;      

      FT857D_set_volume( FT857_volume_value );
      FT857_display_volume_bar(FT857_volume_value);
    }
    return;
  }

  // Process volume rotary counter clockwise
  if (FT857D_encoder_rt_ccw && special_encoder_flag)
  {
    FT857D_encoder_rt_ccw = 0;
    // decrement step counter, limit to 6
    if( RKnob_counter > -6 ) RKnob_counter--;
    // absolute value of counter
    step_count = absval( RKnob_counter );    
    
    // send if enough time has passed since last
    if( RKnob_delay_timer == 0 && step_count != 0)
    {    
      RKnob_counter = 0;     

      if (FT857_volume_value > step_count)
      {
        FT857_volume_value -= step_count;
      }
      else FT857_volume_value = 0;      

      FT857D_set_volume( FT857_volume_value );
      FT857_display_volume_bar(FT857_volume_value);
    }
    return;
  }  
}
