//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
///
// Feb 19, 2018			Design Documents                      CSI_GPIO.c 
// 
//                 GPIO routines for the large display project
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Status
//  written, tested, working
//
// History
//   04/17/2018 merged, removed read_ptt(), was duplicated with read_mike_ptt() 
//   03/11/2019 made global register pointers and CSI_GPIO.h
//
// =============================================================================
#include "CSI_GPIO.h"
#include <stdint.h>
#include "CSI_REGISTERS.h"

// *****************************************************************************
// LCD DATA: PG0 - PG7

// read_switch_s1                  PB0  Encoder left
// read_switch_s2                  PB1  Encoder right
// read_switch_s3                  PE11 Left soft key  - P1
// read_switch_s4                  PE10 Next to left   - P2
// read_switch_s5                  PE9  Next to right  - P3
// read_switch_s6                  PE8  Right soft key - P4         
// read_rotary_encoder_1_0         PE14 Encoder right
// read_rotary_encoder_1_1         PE15 Encoder right
// read_rotary_encoder_2_0         PE12 Encoder left
// read_rotary_encoder_2_1         PE13 Encoder left
//
// rx_led_on                       PD14
// rx_led_off                      PD14
// tx_led_on                       PD15
// tx_led_off                      PD15
// key_backlight_on                PC8
// key_backlight_off               PC8
// lcd_backlight_on                PF7
// lcd_backlight_off               PF7
//
// lcd_cs_high                     PB2
// lcd_cs_low                      PB2
// lcd_d_c_high                    PF11
// lcd_d_c_low                     PF11
// lcd_rd_high                     PF13
// lcd_rd_low                      PF13
// lcd_wr_high                     PF12
// lcd_wr_low                      PF12
// lcd_reset_high                  PF14
// lcd_reset_low                   PF14
// read_lcd_tear                   PF15
// read_lcd_port                   PG
// write_lcd_port                  PG

// read_mike_hook                  PD10
// read_mike_ptt                   PD9
// read_mike_dm                    PD12
// read_mike_mbl                   PD11

// mike_dm_high                    PD12
// mike_dm_low                     PD12
// mike_mbl_high                   PD11
// mike_mbl_low                    PD11
// *****************************************************************************
// GPIO global register pointers
uint32_t volatile *GPIO_A_MODER = (uint32_t *) GPIOA_MODER_ADDRESS;
uint32_t volatile *GPIO_A_ALTHI = (uint32_t *) GPIOA_AFRH_ADDRESS;
uint32_t volatile *GPIO_A_IDR   = (uint32_t *) GPIOA_IDR_ADDRESS;
uint32_t volatile *GPIO_A_BSRR  = (uint32_t *) GPIOA_BSRR_ADDRESS;

uint32_t volatile *GPIO_B_MODER = (uint32_t *) GPIOB_MODER_ADDRESS; 
uint32_t volatile *GPIO_B_ALTLO = (uint32_t *) GPIOB_AFRL_ADDRESS;
uint32_t volatile *GPIO_B_IDR   = (uint32_t *) GPIOB_IDR_ADDRESS;
uint32_t volatile *GPIO_B_BSRR  = (uint32_t *) GPIOB_BSRR_ADDRESS; 

uint32_t volatile *GPIO_C_MODER = (uint32_t *) GPIOC_MODER_ADDRESS; 
uint32_t volatile *GPIO_C_ALTLO = (uint32_t *) GPIOC_AFRL_ADDRESS;
uint32_t volatile *GPIO_C_IDR   = (uint32_t *) GPIOC_IDR_ADDRESS;
uint32_t volatile *GPIO_C_BSRR  = (uint32_t *) GPIOC_BSRR_ADDRESS; 

uint32_t volatile *GPIO_D_MODER = (uint32_t *) GPIOD_MODER_ADDRESS; 
uint32_t volatile *GPIO_D_ALTLO = (uint32_t *) GPIOD_AFRL_ADDRESS;
uint32_t volatile *GPIO_D_IDR   = (uint32_t *) GPIOD_IDR_ADDRESS;
uint32_t volatile *GPIO_D_BSRR  = (uint32_t *) GPIOD_BSRR_ADDRESS; 

uint32_t volatile *GPIO_E_MODER = (uint32_t *) GPIOE_MODER_ADDRESS; 
uint32_t volatile *GPIO_E_ALTLO = (uint32_t *) GPIOE_AFRL_ADDRESS;
uint32_t volatile *GPIO_E_IDR   = (uint32_t *) GPIOE_IDR_ADDRESS;
uint32_t volatile *GPIO_E_BSRR  = (uint32_t *) GPIOE_BSRR_ADDRESS; 

uint32_t volatile *GPIO_F_MODER = (uint32_t *) GPIOF_MODER_ADDRESS; 
uint32_t volatile *GPIO_F_ALTLO = (uint32_t *) GPIOF_AFRL_ADDRESS;
uint32_t volatile *GPIO_F_IDR   = (uint32_t *) GPIOF_IDR_ADDRESS;
uint32_t volatile *GPIO_F_BSRR  = (uint32_t *) GPIOF_BSRR_ADDRESS; 

uint32_t volatile *GPIO_G_MODER = (uint32_t *) GPIOG_MODER_ADDRESS; 
uint32_t volatile *GPIO_G_ALTLO = (uint32_t *) GPIOG_AFRL_ADDRESS;
uint32_t volatile *GPIO_G_IDR   = (uint32_t *) GPIOG_IDR_ADDRESS;
uint32_t volatile *GPIO_G_BSRR  = (uint32_t *) GPIOG_BSRR_ADDRESS; 
uint16_t volatile *GPIO_G_ODR   = (uint16_t *) GPIOG_ODR_ADDRESS;

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s1
//
// Read switch on rotary encoder 1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 or 0
//
// External variables used
//  *GPIO_B_IDR
//
// History
//  04/17/2018 merged, Jerry changed BIT 1 to Bit 0
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted 

uint8_t read_switch_s1(void)
{
  return ~(*GPIO_B_IDR) & 1 ;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s2
//
// Read switch on rotary encoder 2.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 or 0
//
// External variables used
//  *GPIO_B_IDR
//
// History
//  04/17/2018 merged Jerry changed Bit 0 to Bit 1
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted 

uint8_t read_switch_s2(void)
{
  return ~((*GPIO_B_IDR) >> 1) & 1 ;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s3
//
// Read switch s3. Left button P1
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 if pressed, 0 if not
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  04/17/2018 merged, Jerry changed Bit 11 to Bit 08
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted  

uint8_t read_switch_s3(void)
{
  return ~((*GPIO_E_IDR) >> 8) & 1 ;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s4
//
// Read switch s4. Center left button P2
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 if pressed, 0 if not
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  04/17/2018 merged, Jerry changed Bit 10 to Bit 09
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted  

uint8_t read_switch_s4(void)
{
  return ~((*GPIO_E_IDR) >> 9) & 1 ;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s5
//
// Read switch s5. Center right button P3
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 if pressed, 0 if not
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  04/17/2018 merged, Jerry changed Bit 09 to Bit 10
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted  

uint8_t read_switch_s5(void)
{
  return ~((*GPIO_E_IDR) >> 10) & 1 ;  
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s6
//
// Read switch s6. Right button P4
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a 1 if pressed, 0 if not
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  04/17/2018 merged, Jerry changed Bit 08 to Bit 11
//  01/18/2019 modified to return 8 bit, 0 nopress 1 press, inverted 

uint8_t read_switch_s6(void)
{
  return ~((*GPIO_E_IDR) >> 11) & 1 ;  
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_1_0
//
// Read rotary encoder signal 1_0.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a value of 0 or 1
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  01/21/2019 modified to return 8 bit, 0 or 1, NOT inverted 

uint8_t read_rotary_encoder_1_0(void)
{
  return ((*GPIO_E_IDR) >> 14) & 1 ; 
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_1_1
//
// Read rotary encoder signal 1_1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a value of 0 or 2
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  01/21/2019 modified to return 8 bit, 0 or 2, NOT inverted 

uint8_t read_rotary_encoder_1_1(void)
{
  // this tests in bit 2 and returns in bit 2  (shifted 14 not 15) 
  return ((*GPIO_E_IDR) >> 14) & 2 ;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_2_0
//
// Read rotary encoder signal 2_0.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a value of 0 or 1
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  01/21/2019 modified to return 8 bit, 0 or 1, NOT inverted 

uint8_t read_rotary_encoder_2_0(void)
{
  return ((*GPIO_E_IDR) >> 12) & 1 ; 
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_2_1
//
// Read rotary encoder signal 2_1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns a value of 0 or 2
//
// External variables used
//  *GPIO_E_IDR
//
// History
//  01/21/2019 modified to return 8 bit, 0 or 2, NOT inverted 

uint8_t read_rotary_encoder_2_1(void)
{
  // this tests in bit 2 and returns in bit 2  (shifted 12 not 13)   
  return ((*GPIO_E_IDR) >> 12) & 2 ; 
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		       rx_led_on
//
// Turns on RX LED.  RX LED is connected to Port D bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The RX LED is turned on
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT14_ON

void rx_led_on(void)
{
  // Turn on RX LED by setting bit 14 on port D
  *GPIO_D_BSRR = BIT14_ON;
}

//******************************************************************************
//
// Feb 19, 2016			Design Documents      		      rx_led_off
//
// Turns off RX LED.  RX LED is connected to Port D bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The RX LED is turned off
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT14_OFF

void rx_led_off(void)
{
  // Turn off RX LED by clearing bit 14 on port D
  *GPIO_D_BSRR = BIT14_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		       tx_led_on
//
// Turns on TX LED.  TX LED is connected to Port D bit 15 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The TX LED is turned on
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT15_ON

void tx_led_on(void)
{
  // Turn on TX LED by setting bit 14 on port D
  *GPIO_D_BSRR = BIT15_ON;
}

//******************************************************************************
//
// Feb 19, 2016			Design Documents      		      tx_led_off
//
// Turns off TX LED.  TX LED is connected to Port D bit 15 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The TX LED is turned off
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT15_OFF

void tx_led_off(void)
{
  // Turn off TX LED by clearing bit 15 on port D
  *GPIO_D_BSRR = BIT15_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		key_backlight_on
//
// Turns on the key backlight.  KLED is connected to Port C bit 8 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The Key backlight is turned on
// 
// External variables used
//  *GPIO_C_BSRR
//
// Constants
//    BIT08_ON

void key_backlight_on(void)
{
  // Turn on key backlight by setting bit 8 on port C
  *GPIO_C_BSRR = BIT08_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	       key_backlight_off
//
// Turns off the key backlight.  KLED is connected to Port C bit 8 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The Key backlight is turned on
// 
// External variables used
//  *GPIO_C_BSRR
//
// Constants
//    BIT08_OFF

void key_backlight_off(void)
{
  // Turn off key backlight by clearing bit 8 on port C
  *GPIO_C_BSRR = BIT08_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		lcd_backlight_on
//
// Turns on the lcd backlight.  LCD_BACKLIGHT is connected to Port F bit 7 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD backlight is turned on
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT07_ON

void lcd_backlight_on(void)
{
  // Turn on lcd backlight by setting bit 7 on port F
  *GPIO_F_BSRR = BIT07_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	       lcd_backlight_off
//
// Turns off the lcd backlight.  LCD_BACKLIGHT is connected to Port F bit 7 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The lcd backlight is turned on
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT07_OFF

void lcd_backlight_off(void)
{
  // Turn off lcd backlight by clearing bit 7 on port F
  *GPIO_F_BSRR = BIT07_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		     lcd_cs_high
//
// Set the LCD CS line high.  LCD_CS/ is connected to Port B bit 2 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD CS line is set high
// 
// External variables used
//  *GPIO_B_BSRR
//
// Constants
//    BIT02_ON

void lcd_cs_high(void)
{
  // Set the CS line high by setting bit 2 on port B
  *GPIO_B_BSRR = BIT02_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_cs_low
//
// Set the LCD CS line low.  LCD_CS/ is connected to Port B bit 2 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD CS line is set low
// 
// External variables used
//  *GPIO_B_BSRR
//
// Constants
//    BIT02_OFF
//
// History
//  04/17/2018 merged, Jerry changed Bit 07 to Bit 02

void lcd_cs_low(void)
{
  // Set the LCD CS line low by clearing bit 2 on port B
  *GPIO_B_BSRR = BIT02_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_d_c_high
//
// Set the LCD D/C line high.  LCD_D/C is connected to Port F bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD D/C line is set high
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT11_ON

void lcd_d_c_high(void)
{
  // Set the LCD D_C line high by setting bit 11 on port F
  *GPIO_F_BSRR = BIT11_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	             lcd_d_c_low
//
// Set the LCD D/C line low.  LCD_D/C is connected to Port F bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD D/C line is set low
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT11_OFF

void lcd_d_c_low(void)
{
  // Set the LCD D/C line low by clearing bit 11 on port F
  *GPIO_F_BSRR = BIT11_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_rd_high
//
// Set the LCD RD/ line high.  LCD_RD/ is connected to Port F bit 13
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD RD/ line is set high
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT13_ON
//
// History
//  04/13/2018 swapped pins  WR and RD to match hardware

void lcd_rd_high(void)
{
  // Set the LCD RD/ line high by setting bit 13 on port F
  *GPIO_F_BSRR = BIT13_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_rd_low
//
// Set the LCD RD/ line low.  LCD_RD is connected to Port F bit 13
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD RD/ line is set low
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT13_OFF
//
// History
//  04/13/2018 swapped pins  WR and RD to match hardware

void lcd_rd_low(void)
{
  // Set the LCD RD/ line low by clearing bit 13 on port F
  *GPIO_F_BSRR = BIT13_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_wr_high
//
// Set the LCD WR/ line high.  LCD_WR/ is connected to Port F bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD WR/ line is set high
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT12_ON
//
// History
//  04/13/2018 swapped pins  WR and RD to match hardware

void lcd_wr_high(void)
{
  // Set the LCD WR/ line high by setting bit 12 on port F
  *GPIO_F_BSRR = BIT12_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_wr_low
//
// Set the LCD WR/ line low.  LCD_WR/ is connected to Port F bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD WR/ line is set low
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT12_OFF
//
// History
//  04/13/2018 swapped pins  WR and RD to match hardware

void lcd_wr_low(void)
{
  // Set the LCD WR/ line low by clearing bit 12 on port F
  *GPIO_F_BSRR = BIT12_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		  lcd_reset_high
//
// Set the LCD RESET line high.  LCD_RESET is connected to Port F bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD RESET line is set high
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT14_ON

void lcd_reset_high(void)
{
  // Set the LCD RESET line high by setting bit 14 on port F
  *GPIO_F_BSRR = BIT14_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	           lcd_reset_low
//
// Set the LCD RESET line low.  LCD_RESET is connected to Port F bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The LCD RESET line is set low
// 
// External variables used
//  *GPIO_F_BSRR
//
// Constants
//    BIT14_OFF

void lcd_reset_low(void)
{
  // Set the LCD RESET line low by clearing bit 14 on port F
  *GPIO_F_BSRR = BIT14_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_lcd_tear 
//
// Read LCD TEAR.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns lcd_tear bit with other bits in the register masked out 
//
// External variables used
//  *GPIO_F_IDR
//
// Constants
//    BIT15_MASK 

uint32_t read_lcd_tear(void)
{
  return (*GPIO_F_IDR) & BIT15_MASK;   
}

//******************************************************************************
//
// Feb 20, 2018			Design Documents                   read_lcd_port 
//
// This data reads the microprocessor port attached to the LCD 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns microprocessor port attached to LCD display with unused bits set to
//  zero. The LCD is on the LSB of port G.
//
// External variables used
//    uint32_t volatile *GPIO_G_IDR
//    uint32_t volatile *GPIO_G_MODER
//
// History
// 04/17/2018 merge Jerry rewrote, added nop pause

uint8_t read_lcd_port(void)
{
  *GPIO_G_MODER = 0x00000000;          //set port to input
  asm("NOP");                          // Settle down before read
  asm("NOP"); 
  return (uint8_t)(*GPIO_G_IDR & 0x00FF);       //Read the port
}

//******************************************************************************
//
// Feb 20, 2018			Design Documents                  write_lcd_port 
//
// This data writes to the microprocessor port attached to the LCD 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry Conditions
//  Data supplied by calling program
//
// Exit Conditions
//  Output the data passed to this program to Port G. The MSB are set to zero at
//  this time so the other pins on this port cannot currently be used.
//
// External variables used
//    uint32_t volatile *GPIO_G_ODR
//    uint32_t volatile *GPIO_G_MODER
//
// History
//   04/17/2018 merge Jerry rewrote, added nops pause

void write_lcd_port(uint8_t lcd_data)
{
  *GPIO_G_MODER = 0x00005555;          //Set port to output
  asm("NOP");        // let port change settle
  asm("NOP");                            
  *GPIO_G_ODR = lcd_data & 0x00FF;     //Write to port
  asm("NOP");                          // settle down before you pass it on
  asm("NOP"); 
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_mike_hook
//
// Read MIKE HOOK.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns mike hook bit with other bits in the register masked out 
//
// External variables used
//  *GPIO_D_IDR
//
// Constants
//    BIT10_MASK 

uint32_t read_mike_hook(void)
{
  return (*GPIO_D_IDR) & BIT10_MASK;   
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_mike_ptt
//
// Read MIKE PTT.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns mike ptt bit with other bits in the register masked out 
//
// External variables used
//  *GPIO_D_IDR
//
// Constants
//    BIT09_MASK 

uint32_t read_mike_ptt(void)
{
  return (*GPIO_D_IDR) & BIT09_MASK;   
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                    read_mike_dm
//
// Read MIKE DM.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns mike dm bit with other bits in the register masked out 
//
// External variables used
//  *GPIO_D_MODER
//  *GPIO_D_IDR
//
// Functions used
//    wait_36_nanoseconds
//
// Constants
//    BIT12_MASK 

extern void wait_36_nanoseconds(void);

uint32_t read_mike_dm(void)
{
  *GPIO_D_MODER &= 0xFEFFFFFF;   //Set port to input clear bit 24
  wait_36_nanoseconds();
  return (*GPIO_D_IDR) & BIT12_MASK;   
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_mike_mbl
//
// Read MIKE MBL.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  Returns mike mbl bit with other bits in the register masked out 
//
// External variables used
//  *GPIO_D_IDR
//
// Constants
//    BIT11_MASK 

uint32_t read_mike_mbl(void)
{
  return (*GPIO_D_IDR) & BIT11_MASK;   
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    mike_dm_high
//
// Set the MIKE DM line high.  MIKE_DM is connected to Port D bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The MIKE_DM line is set high
// 
// External variables used
//  *GPIO_D_MODER
//  *GPIO_D_BSRR
//
// Constants
//    BIT12_ON

void mike_dm_high(void)
{
  *GPIO_D_MODER |= 0x01000000;   //Set port to ouput set bit 24
  // Set the MIKE DM line high by setting bit 12 on port D
  *GPIO_D_BSRR = BIT12_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	             mike_dm_low
//
// Set the MIKE DM line low.  MIKE_DM is connected to Port D bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The MIKE_DM line is set low
// 
// External variables used
//  *GPIO_D_MODER
//  *GPIO_D_BSRR
//
// Constants
//    BIT12_OFF

void mike_dm_low(void)
{
  *GPIO_D_MODER |= 0x01000000;   //Set port to ouput set bit 24
  // Set the MIKE DM line low by clearing bit 12 on port D
  *GPIO_D_BSRR = BIT12_OFF;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      		   mike_mbl_high
//
// Set the MIKE MBL line high.  MIKE_MBL is connected to Port D bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The MIKE MBL line is set high
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT11_ON

void mike_mbl_high(void)
{
  // Set the MIKE MBL line high by setting bit 11 on port D
  *GPIO_D_BSRR = BIT11_ON;
}

//******************************************************************************
//
// Feb 19, 2018			Design Documents      	            mike_mbl_low
//
// Set the MIKE MBL line low.  MIKE_MBL is connected to Port D bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  The MIKE MBL line is set low
// 
// External variables used
//  *GPIO_D_BSRR
//
// Constants
//    BIT11_OFF

void mike_mbl_low(void)
{
  // Set the LCD WR/ line low by clearing bit 13 on port F
  *GPIO_D_BSRR = BIT11_OFF;
}

//******************************************************************************


