//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// April 26, 2018		Design Documents                  CSI_graphics.c 
// 
//       Graphics routines for LCD display
//           for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  04/26/2018  started
//  06/07/2019  refactored to use RGB_colors
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  WR_cycle            Cycle write line of LCD
//  draw_pixel          Write RGB for one pixel location
//  draw_rectangle      Draws a rectangle of the color given on the LCD 
//  clear_rectangle     Clears a rectangle to the current background color
//  color_rectangle     Draws a rectangle of the current font color
//  draw_line           Draws a horizontal or vertical line in a color
//  draw_frame          Draws a rectangular frame in a color
//  draw_shaded_frame   Draws a rectangular frame with a shadowed border
//  draw_icon           Draws an icon in the two colors given
//  draw_icon_bkg       Draws an icon in the current font and background colors
//  draw_icon_rle       Draws a multi colored icon, data is run length encoded
//  draw_icon_mirrored  Draws a mirrored RLE icon
//  draw_progress_bar   Draws a progess bar
//  display_cursor      Activates a cursor 
//  cursor_blink        blinks a cursor
//  get_xy_dial         gets screen xy coordinates for vfo knob spinner
//  draw_dial_spin      Draws spinner for VFO knob


// Status: written, tested, not used
//  draw_pixel       Draw a single pixel on the sceren
//
//==============================================================================
#include <stdint.h>
#include "CSI_GRAPHICS.h"
#include "CSI_SSD1963.h"
#include "CSI_GPIO.h"
#include "CSI_REGISTERS.h"
#include "CSI_COLOR.h"

// module variable for use by all graphics routines
uint16_t volatile *LCD_DATA = (uint16_t *) GPIOG_ODR_ADDRESS;

//  Primitives 

// draw_pixel  - done
// draw_line   
//     horizontal  - done
//     vertical    - done
//     angle
// draw_rectangle  - done
// draw_frame      - done
// draw icon       - done, need to make larger icons, multi color
// draw_circle
//     open
//     filled

// Composite objects

// Progress bar    - done, can be improved
// Slider
// Spinner
// Button
//   Type 
//     Radio button
//     One shot
//   Shape
//     Round
//     Rectangle
// 

// Actions
//  Blink
//  Move
//  Fade
//  Rotate

// =============================================================================
// 
//  Cycle WR line
//     cycles WR line for writing data to LCD port
//
// History
//  02/13/2019 removed 12nanosecond waits, subroutine call is enough time. 
// 
// Status
//    Written, tested

void WR_cycle( void )
{
  lcd_wr_low();
  lcd_wr_high();
}


// =============================================================================
//
// Draw Pixel
//   write RGB for one pixel location ( X, Y )
//   RGB is any color, specify R, G and B
//
// Status
//    Written, tested, not used

void draw_pixel(uint16_t Xpos, uint16_t Ypos, RGB_color colr )
{
  uint16_t volatile *LCD_D = LCD_DATA;
  
  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( Xpos, Xpos ); 
  ssd1963_set_row_address( Ypos, Ypos );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();

  *LCD_D = colr.red;
  WR_cycle();
  *LCD_D = colr.green;
  WR_cycle();
  *LCD_D = colr.blue;
  WR_cycle();

  lcd_cs_high(); 
}
// =============================================================================
//
// Draw Rectangle
//  starting at X and Y location, upper left corner
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top

// Status
//    Written, tested
//
// 02/11/2019 removed *3 in write loop, Data was being sent 3 x
// 08/21/2019 made a local pointer for the output port, decrementing counter
//            reduced assembly from 23 to 16 instruction in the loop
//            should be 30% faster

void draw_rectangle( uint16_t startX, uint16_t startY, uint16_t width,
               uint16_t height, RGB_color colr )
{
  int i;
  uint16_t volatile *LCD_D = LCD_DATA;

  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( startX, startX + width - 1 ); 
  ssd1963_set_row_address( startY, startY + height -1 );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();

  for(i=(width * height) ; i != 0; i--)
  {
    *LCD_D = colr.red;
    WR_cycle();
    *LCD_D = colr.green;
    WR_cycle();
    *LCD_D = colr.blue;
    WR_cycle();
  }  
  lcd_cs_high();  
}
// =============================================================================
//
// Clear Rectangle
//
//  Uses draw rectangle with background colors to clear
//
//  starting at X and Y location, upper left corner
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top

// Status
//    Written, tested
//
void clear_rectangle( uint16_t startX, uint16_t startY,
                     uint16_t width, uint16_t height)
{
  // clear
  draw_rectangle( startX, startY, width, height, cg_background ); 
}
// =============================================================================
//
// Color Rectangle
//
//  Uses draw rectangle with foreground colors to clear
//
//  starting at X and Y location, upper left corner
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top

// Status
//    Written, tested
//

void color_rectangle(uint16_t startX, uint16_t startY,
                     uint16_t width, uint16_t height)
{
  draw_rectangle( startX, startY, width, height, cg_font);                   
}
// =============================================================================
//
// Draw Line
//    uses draw rectangle 
//  ltype is  0 for horizontal, 1 for vertical 
//  starting at X and Y location, upper left corner
//  length is horizontal length to the right for a horizontal line
//            vertical length down for a vertical line
//            
// Status
//    Written, tested
//
void draw_line( int ltype, uint16_t startX, uint16_t startY, uint16_t length,
               uint16_t width, RGB_color colr )
{
  // length and with are swapped for a vertical line
  if(ltype == 1)  // vertical
  {
    draw_rectangle( startX, startY, width, length, colr);
  }
  else  // horizontal
  {
    draw_rectangle( startX, startY, length, width, colr );
  }
}
// =============================================================================
//
// Draw Frame
//    draws a bordered rectangle
//  starting at X and Y location, upper left corner, the outside of the frame
//  length is horizontal pixels to the right 
//  height is vertcal pixels down
//  border_size is the number of pixels in the border
//  RGB is for the border color
//  The interior of the frame is not changed
//            
// Status
//    Written, tested
//
void draw_frame( uint16_t startX, uint16_t startY, uint16_t length,
               uint16_t height, uint16_t border_size, RGB_color colr )
{
  // Top line
  draw_rectangle( startX, startY, length, border_size, colr);
  // Bottom line
  draw_rectangle( startX, startY + height - border_size,
                  length, border_size, colr );
  // Left side
  draw_rectangle( startX, startY + border_size, border_size,
                 height - (2* border_size), colr ); 
  // Right side
  draw_rectangle( startX + length - border_size, startY + border_size, border_size,
                 height - (2* border_size), colr ); 
}
// =============================================================================
//
// Draw Shaded Frame
//    draws a bordered rectangle with shading
//  starting at X and Y location, upper left corner, the outside of the frame
//  length is horizontal pixels to the right 
//  height is vertcal pixels down
//  border_size is the number of pixels in the border
//  RGB are for the border color and shadow color
//  The interior of the frame is not changed
//            
// Status
//    Written, tested
//
void draw_shaded_frame( uint16_t startX, uint16_t startY, uint16_t length,
               uint16_t height, uint16_t border_size, 
               RGB_color b_colr, RGB_color s_colr )
{
  uint16_t shade;
  shade = border_size >> 1;
  // draw shadow first, offset right and down
  draw_frame( startX + shade, startY + shade, length - shade, height - shade ,
             border_size, s_colr);
  // draw frame over top
  draw_frame( startX, startY, length - shade, height - shade ,
             border_size, b_colr);
}
// =============================================================================
//
// Draw Icon
//
//  The icon is drawn in the given forground and background color  
//  starting at X and Y location, upper left corner
//  width and height are contained in the Icon data first two bytes
//  
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top
// 

// Status
//    Written, tested
//
void draw_icon( uint8_t *icondata, uint16_t startX, uint16_t startY,
               uint8_t foregr_color, uint8_t backgr_color)
{
  int i;
  uint16_t volatile *LCD_D = LCD_DATA;  
  uint8_t width;
  uint8_t height;
  uint16_t dindx;
  uint8_t dbyte;
  
  // make local color variables for speed
  uint8_t f_red = current_palette[foregr_color].red;
  uint8_t f_green = current_palette[foregr_color].green;
  uint8_t f_blue = current_palette[foregr_color].blue;
  
  uint8_t b_red = current_palette[backgr_color].red;
  uint8_t b_green = current_palette[backgr_color].green;
  uint8_t b_blue = current_palette[backgr_color].blue;
    
  width = icondata[0];
  height = icondata[1];

  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( startX, startX + width - 1 ); 
  ssd1963_set_row_address( startY, startY + height -1 );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();
  
  dindx = 2;  // first data is after reading width and height
  dbyte =  icondata[dindx++]; 
  
  int bitcount = 8;
  
  for(i=(width * height);i != 0 ;i--)
  {
    // test high bit
    if(dbyte & 0x80)
    {
      // if its a 1 send foreground color
      *LCD_D = f_red ;
      WR_cycle();
      *LCD_D = f_green;
      WR_cycle();
      *LCD_D = f_blue;
      WR_cycle();      
    }
    else
    {
      // send background color
      *LCD_D = b_red;
      WR_cycle();
      *LCD_D = b_green;
      WR_cycle();
      *LCD_D = b_blue;
      WR_cycle();
    } 
    // next bit   
    bitcount--;
    if( bitcount == 0)
    {
      // read next byte 
      dbyte =  icondata[dindx++];     
      bitcount = 8;
    }
    else  dbyte <<= 1; // shift for next bit
  }
  
  lcd_cs_high();  
  
}
// =============================================================================
//
// Draw Icon, simpler version uses foreground colors rgb
//            and background colors rgb instead of the current palette
//
//  The icon is drawn in the current text font and background color  
//  starting at X and Y location, upper left corner
//  width and height are contained in the Icon data first two bytes
//  
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top
//
// History
//      02/27/2019 added to allow custom colors for an icon

// Status
//    Written, tested
//
//
void draw_icon_bkg( uint8_t *icondata, uint16_t startX, uint16_t startY )

{
  int i;
  uint16_t volatile *LCD_D = LCD_DATA;  
  uint8_t width;
  uint8_t height;
  uint16_t dindx;
  uint8_t dbyte;
  
  width = icondata[0];
  height = icondata[1];

  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( startX, startX + width - 1 ); 
  ssd1963_set_row_address( startY, startY + height -1 );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();
  
  dindx = 2;  // first data is after reading width and height
  dbyte =  icondata[dindx++]; 
  
  int bitcount = 8;
  
  for(i=(width * height);i != 0 ;i--)
  {
    // test high bit
    if(dbyte & 0x80)
    {
      // if its a 1 send foreground color
      *LCD_D = cg_font.red ;
      WR_cycle();
      *LCD_D = cg_font.green;
      WR_cycle();
      *LCD_D = cg_font.blue;
      WR_cycle();      
    }
    else
    {
      // send background color
      *LCD_D = cg_background.red;
      WR_cycle();
      *LCD_D = cg_background.green;
      WR_cycle();
      *LCD_D = cg_background.blue;
      WR_cycle();
    } 
    // next bit   
    bitcount--;
    if( bitcount == 0)
    {
      // read next byte 
      dbyte =  icondata[dindx++];     
      bitcount = 8;
    }
    else  dbyte <<= 1; // shift for next bit
  }
  
  lcd_cs_high();  
  
}

// =============================================================================
//
// Draw Multi color Icon
//
//  starting at X and Y location, upper left corner
//
//  Icon data format:
//  width and height are in the first two bytes
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top
//
//  number of colors, max 16, can increase if wanted 
//  RGB of color 1
//  RGB of color 2...
//  colors are for default icon, can be overridden for different colors
//  color 0 is background. It should be overridden to match screen background

//  Use simple RLE for the pixel data.
//  Data to have: number of pixels, color number
//  maximum icon in use is 180 x 200 bits
// 
// Status
//    Wrtten, working
//

void draw_icon_rle( uint8_t *icondata, uint16_t startX, uint16_t startY )
{
  uint16_t volatile *LCD_D = LCD_DATA;  
  uint8_t color_table[16][3];
  unsigned int i_index = 0;
  
  uint16_t width = 0;
  uint16_t height = 0;
  int i;
  unsigned int numpixels = 0;
  
  uint8_t rcount = 0;
  uint8_t rdata = 0;
  
  uint8_t num_colors;

  static unsigned int pix_count = 0;
  
  // read width and height 
  width = icondata[0];
  height = icondata[1];
  numpixels = width * height;
  // read number of colors
  num_colors = icondata[2];
  
  // load color rgb data into color array
  for(i = 0; i < num_colors; i++)
  {
    color_table[i][0] = icondata[3 + i*3];   // red 
    color_table[i][1] = icondata[4 + i*3];   // green
    color_table[i][2] = icondata[5 + i*3];   // blue
  }
  // override (this could be controlled by an external switch)
  // set background to current
  color_table[0][0] = cg_background.red;   // red 
  color_table[0][1] = cg_background.green; // green
  color_table[0][2] = cg_background.blue;  // blue
  
    // set font to current
  color_table[1][0] = cg_font.red;   // red 
  color_table[1][1] = cg_font.green; // green
  color_table[1][2] = cg_font.blue;  // blue
  
  // begin transmission
  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( startX, startX + width - 1 ); 
  ssd1963_set_row_address( startY, startY + height -1 );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();  
  
  // icondata index to beginning of rle 
  i_index = 3 + 3 * num_colors;    // w,h, numcolrs, r,g,b ...
  
  pix_count = 0;
  while( pix_count < numpixels)
  {
    // read count and data
    rcount = icondata[i_index];
    rdata  = icondata[i_index + 1];
    // send pixel R G B data
    for(i = rcount; i != 0 ; i--)
    {
      *LCD_D = color_table[rdata][0];   // red
      WR_cycle();
      *LCD_D = color_table[rdata][1];   // green
      WR_cycle();
      *LCD_D = color_table[rdata][2];   // blue
      WR_cycle();
      pix_count++;  // one pixel
    }
    // next rle
    i_index+= 2;    
  }
  // end transmission
  lcd_cs_high();  

}
// =============================================================================
//
// Draw Multi color Icon, variaton for mirroring logo radio waves
//
//  starting at X and Y location, upper left corner
//
//  Icon data format:
//  width and height are in the first two bytes
//  width is number of pixels horizontal right from start 
//  height is number of pixels vertical down from top
//
//  number of colors, max 16, can increase if wanted 
//  RGB of color 1
//  RGB of color 2...
//  colors are for default icon, can be overridden for different colors
//  color 0 is background. It should be overridden to match screen background

//  Use simple RLE for the pixel data.
//  Data to have: number of pixels, color number
//  In order to mirror, icon data needs to be in rows of "width" bits
// 
// History
//     02/27/2019 added use background switch to allow color 0 in 
//                the icon file to be used or curren background

// Status
//    Written, working
//

void draw_icon_mirrored( uint8_t *icondata, uint16_t startX, uint16_t startY, uint8_t usebkg )
{
  uint16_t volatile *LCD_D = LCD_DATA;  
  uint8_t color_table[16][3];
  int i_index = 0;
  
  uint8_t row_data[26];
  int row_index = 0;
  int row_pix_count = 0;
  
  uint16_t width = 0;
  uint16_t height = 0;
  int i;
  unsigned int numpixels = 0;
  
  uint8_t rcount = 0;
  uint8_t rdata = 0;
  
  uint8_t num_colors;

  static unsigned int pix_count = 0;
  
  // read width and height 
  width = icondata[0];
  height = icondata[1];
  numpixels = width * height;
  // read number of colors
  num_colors = icondata[2];
  
  // load color rgb data into color array
  for(i = 0; i < num_colors; i++)
  {
    color_table[i][0] = icondata[3 + i*3];   // red 
    color_table[i][1] = icondata[4 + i*3];   // green
    color_table[i][2] = icondata[5 + i*3];   // blue
  }
  // override (this could be controlled by an external switch)
  if(usebkg)
  {
    // set background to current
    color_table[0][0] = cg_background.red;   // red 
    color_table[0][1] = cg_background.green; // green
    color_table[0][2] = cg_background.blue;  // blue
  }
  // begin transmission
  ssd1963_set_port_direction( 1 );
  
  ssd1963_set_column_address( startX, startX + width - 1 ); 
  // twice as high
  ssd1963_set_row_address( startY, startY + 2 * height - 1 );
  
  ssd1963_write_command(0x2C);  // write memory start
  
  lcd_cs_low();
  lcd_d_c_high();  
  
  // icondata index to beginning of rle 
  i_index = 3 + 3 * num_colors;    // w,h, numcolrs, r,g,b ...
  
  pix_count = 0;
  while( pix_count < numpixels)
  {
    // read count and data
    rcount = icondata[i_index];
    rdata  = icondata[i_index + 1];
    // send pixel R G B data
    for(i = rcount; i != 0; i--)
    {
      *LCD_D = color_table[rdata][0];   // red
      WR_cycle();
      *LCD_D = color_table[rdata][1];   // green
      WR_cycle();
      *LCD_D = color_table[rdata][2];   // blue
      WR_cycle();
      pix_count++;  // one pixel
    }
    // next rle
    i_index += 2;    
  }
  // end of top half
  // i_index has been incremented 2 past the end
  // reset to end
  i_index -= 2;  
  
  // walk back in reverse through the icon array  
  while( i_index >= 3 + 3 * num_colors)
  {
    row_index = 0;
    while(row_pix_count < width)
    {
      // load row_data with "width" bits
      // read count and data
      rcount = icondata[i_index];
      rdata  = icondata[i_index + 1];
      row_pix_count += rcount;      
      // store in the row array
      row_data[row_index] = rcount;
      row_data[row_index + 1] = rdata;
      row_index += 2;
      i_index -= 2;   // going backwards
    }
    // send a row
    // row_index was incremented past the last, readjust to last
    row_index -= 2;
    
    while( row_index >= 0 )
    {
      rcount = row_data[row_index];
      rdata  = row_data[row_index + 1];    
      // send pixel R G B data
      for(i = rcount; i != 0; i--)
      {
        *LCD_D = color_table[rdata][0];   // red
        WR_cycle();
        *LCD_D = color_table[rdata][1];   // green
        WR_cycle();
        *LCD_D = color_table[rdata][2];   // blue
        WR_cycle();
        row_pix_count--;  // one pixel
      }
      row_index -= 2;  // in reverse
    }
    
  }
   
  // end transmission
  lcd_cs_high();  

}  
 
// =============================================================================
//
// Draw Progress bar
//
//  percent fill is an integer 0 to 100
//  ltype is  0 for horizontal, 1 for vertical 
//  starting at X and Y location, upper left corner
//   use colors from current palette

// border is fixed at 4, 
// bar gap is fixed at 2, make variable later
// so bar with frame is 12 pixels wider than just the fill 
//
//
void draw_progress_bar( uint8_t percent_fill, int bar_type,
                    uint16_t startX, uint16_t startY,
                    uint16_t width, uint16_t height,
                    uint8_t frame_color, uint8_t bar_color, uint8_t backg_color)
{
  uint8_t border_size = 4; // pixels
  uint8_t bar_gap = 2;  // pixels
  uint16_t fill_start = 0;  // beginning of fill bar
  uint16_t fill_space = 0;  // amount of space inside to fill at 100%
  uint16_t fill_size = 0;   // amount to be filled by bar 
  
  // frame doesn't care which direction type
  draw_frame( startX, startY, width, height, border_size,
              current_palette[frame_color] ); 
  // vertical bar
  if(bar_type == 1)

  {
    // draw bar to percent filled
    // for vertical fill goes up from bottom
    fill_space = height - (2* (border_size + bar_gap));
    fill_start = ((100 - percent_fill) * fill_space / 100);
    fill_size = fill_space - fill_start;
    // fill background
    draw_rectangle( startX + border_size + bar_gap,
                    startY + border_size + bar_gap,
                    width - (2* (border_size + bar_gap)),
                    fill_start, current_palette[backg_color]);
    // fill bar
    draw_rectangle( startX + border_size + bar_gap,
                    startY + border_size + bar_gap + fill_start,
                    width - (2* (border_size + bar_gap)),
                    fill_size, current_palette[bar_color]);
  }
  else  // horizontal bar  
  {
    // draw bar to percent filled
    // for horizontal fill goes to the right
    fill_space = width - (2* (border_size + bar_gap));
    fill_start = startX + border_size + bar_gap;
    fill_size = percent_fill * fill_space / 100;  // multply uints first
    
    // fill background
    draw_rectangle( fill_start + fill_size,
                    startY + border_size + bar_gap,
                    fill_space - fill_size,
                    height - (2* (border_size + bar_gap)),
                    current_palette[backg_color] );
    // fill bar
    draw_rectangle( fill_start,
                    startY + border_size + bar_gap,
                    fill_size,
                    height - (2* (border_size + bar_gap)),
                    current_palette[bar_color] );
  }
}

// =============================================================================
// 12/13/2018
//
//  draw cursor, on or off  
//
extern uint16_t cg_max_hori_size;

void display_cursor( uint8_t onoff, uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int curs_width;
  
  ssd1963_cg_read_char_size( font );
  curs_width = cg_max_hori_size - 14;
  
  if(onoff) draw_rectangle( pos_x, pos_y, curs_width, 4, cg_font);
  else clear_rectangle( pos_x, pos_y, curs_width, 4 );
  
}
// =============================================================================
// 12/13/2018
//
//  cursor blink
//
// uses seconds counter run by systick
extern unsigned long volatile seconds;

void cursor_blink( uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  static uint8_t cur_toggle = 0;
  static unsigned long last_sec;
  
  if(seconds != last_sec )
  {
    if(cur_toggle) display_cursor( 1, pos_x, pos_y, font );
    else display_cursor( 0, pos_x, pos_y, font );
    cur_toggle ^= 1;
    last_sec = seconds;
  }
}
// =============================================================================
//             
// big dial spinner data
// index location, x, y
// x,y is the center of the icon for the dial indicator
// index going + is clockwise

// the knob is 140 pixels wide, 70 r
uint8_t knob_angle[72] = {

 0,  70,   0, // 12:00
 1,  88,   2,
 2, 105,   9,
 3, 119,  21,
 4, 131,  35,
 5, 138,  52,
 6, 140,  70, // 3:00
 7, 138,  88,
 8, 131, 105,
 9, 119, 119,
10, 105, 131,
11,  88, 138, 
12,  70, 140,// 6:00 
13,  52, 138,
14,  35, 131,
15,  21, 119,
16,   9, 105,
17,   2,  88,
18,   0,  70, // 9:00
19,   2,  52,
20,   9,  35,
21,  21,  21,
22,  35,   9,
23,  52,   2, 
};

// the knob is 200 pixels wide, 100 r
/*  200 pixel wide 
 0, 100,   0,  // 12:00
 1, 126,   3,
 2, 150,  13,
 3, 171,  29,
 4, 187,  50,
 5, 197,  74,
 6, 200, 100,  // 3:00
 7, 197, 126,
 8, 187, 150,
 9, 171, 171,
10, 150, 187,
11, 126, 197,
12, 100, 200,  // 6:00
13,  74, 197,
14,  50, 187,
15,  29, 171,
16,  13, 150,
17,   3, 126,
18,   0, 100,  // 9:00
19,   3,  74,
20,  13,  50,
21,  29,  29,
22,  50,  13,
23,  74,   3 
};
   */
// =============================================================================
//
//  gets index to the x and xy for a rotation position
//

uint8_t get_xy_dial( uint8_t rotation )
{
  uint8_t i = 0;
  while( rotation != knob_angle[i*3]) i++;
  return i;
}

// =============================================================================
//
//  draw dial spinner at rotation position
//  The surrounding knob is not redrawn, only the location of the spinner
//
//
// 12/21/2018  writing, use correct colors

extern unsigned char ICON_DIAL_SPIN[242];

uint8_t  prev_rotation = 23;  // because starting rotation is 0

// for debug test only
// #include "CSI_TouchPanel.h"

void draw_dial_spin( uint16_t pos_x, uint16_t pos_y,  uint8_t rotation )
{
  int dial_x;
  int dial_y;
  uint8_t r_idx;
  
  // get xy for old
  //r_idx = get_xy_dial( prev_rotation );
  r_idx = prev_rotation * 3;
  dial_x = knob_angle[r_idx + 1];
  dial_y = knob_angle[r_idx + 2];

  // custom background color same as in the icon file grey
  cg_background.red   = 140;
  cg_background.green = 140;
  cg_background.blue  = 140;
  
  // clear old   
  clear_rectangle( pos_x + dial_x - 20,  pos_y + dial_y - 20, 40, 44);

  // get xy for new 
  // r_idx = get_xy_dial( rotation );
  r_idx = rotation * 3;
  dial_x = knob_angle[r_idx + 1];
  dial_y = knob_angle[r_idx + 2];
  prev_rotation = rotation;
  
  // draw new  
  draw_icon_bkg( ICON_DIAL_SPIN, pos_x + dial_x - 20,  pos_y + dial_y - 20 );

/*  
  // debug touch points
  // draw a frame around the touch point (x and y reversed)
 draw_frame( CS800D_touch_point[rotation+35].ymin, CS800D_touch_point[rotation+35].xmin,
             CS800D_touch_point[rotation+35].ymax - CS800D_touch_point[rotation+35].ymin,
             CS800D_touch_point[rotation+35].xmax - CS800D_touch_point[rotation+35].xmin,
             2, cg_font.red, cg_font.green, cg_font.blue);
*/
  
  // reset colors for the screen
  set_font_colors( 3, 14 );  
}








