//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//
// Janaury 24, 2016           Design Document               initialize_registers
//
// This program initializes all the registers in the ARM processor
//
// Algorithm
//
//  It should be noted that the Cortex M7 internal and external peripherals have 
//  reserved addresses of:
//    400X XXXX 
//    500X XXXX 
//    A00X XXXX 
//    E00X XXXX 
//
//  We will use the Following for special purposes as described below:
//    FFFF FFFF  as end of file
//    1000 0000  as used as a timer with register value the time
//    8XXX XXXX  as write 16 bit to register instead of 32 bit
//
//    When 16 bit writes are done, the 8XXX XXXX  has to be converted
//    to 4XXX XXXX.
//
//  Most of the memory locations are written with a zero. If you make those as
//  special cases then it might be possible to save memory space by getting rid
//  of half the table.
//
//  It is also possible to set this table into external flash memory to save
//  even more space. However if that is done some provisions for reading the 
//  external flash memory has to be made.
//
// Programmer
//  Jerry Wanger
//
// Entry Conditions
//  All definitions of what to do is passed through the init_array. The first
//  element of the array is the address of the register and the second element
//  is the value to set the register to.
//
//  There are two special cases for this array as follows: 
//  (1) If the first element of the array is 0xFFFFFFFF, then this is end of
//      program indication
//  (2) If the first elementof the array is 0x8XXXXXXX, then this is defined as
//      a sixteen bit data transfer to the address. The address is changed to
//      0x4XXXXXXX.
//
// Exit Conditions
//  The registers defined by the table is initialized.
//
// External Variables Used
//  None
//
// Internal Variables Used
//  unsigned int init_array[][2]       // Contains the register definitions
//  unsigned int register_address
//
// History:
// 01/24/2016  Original by Jerry for STM32F7 in CS7000 proto 
// 01/26/2016  Modified by Dean Graham
// 03/20/2018  Rewritten for STM32F405 for Large Display project
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  initialize_registers              loads CPU registers with starting values
//  initialize_registers_fast         loads CPU registers, skips zeros
// 
//******************************************************************************
#include <stdint.h>
#include "CSI_INITIALIZE_REGISTERS.h"


const unsigned int init_array[][2] =
  {
//
// Power Controller Registers
//
     PWR_CR_ADDRESS,         0x00004000, // PWR Power control register
     PWR_CSR_ADDRESS,        0x00000000, // PWR power control/status register
//  there is only one power control unit in the STM32F405
//     PWR_CR2_ADDRESS,         0x00000000, // PWR Power control registers
//     PWR_CSR2_ADDRESS,        0x00000000, // PWR power control/status register
//
// RESET and Clock Control. See ST Manual DM00031020, page 115 for details
//
// The source of the clock for the microprocessor is a 8 MHz crystal.
// Need to set the computer to its rate of 168 MHz
//  Clock startup is done in void startup_clocks(void)     
//   The clocks should be started correctly before running this,
//   not try to do it in the middle of running.
/*
// RCC_CR
//    PLLSAIRDY = 0 Read only bit
//    PLLSAION = 0 Disable timer because SAI PLL not used
//    PLLI2SRDY = 0 Read only bit
//    PLLI2SON = 0 Disable timer because I2S PLL not used
//    PLLRDY = 0 Read only bit
//    PLLON = 1 Ensable timer because main PLL is used
//    CSSON = 0 Disable clock security. Enable later to improve reliability
//    HSEBYP = 0 for XTAL. If want external clock use a value of 1
//    HSERDY  = 0 Read only bit
//    HSEON = 1 to enable XTAL or External Clock
//    HSICAL = 0 Internal clock parameter not used
//    HSITRIM = 0 Internal clock parameter not used
//    HSIRDY = 0 Internal clock parameter not used
//    HSION = 0 Internal clock parameter not used
// RCC_PLLCFGR    
//    PLLQ = 9 to give 48 MHz clock for USB    
//    PLLSRC = 1 To select XTAL clock as source
//    PLLP = 2 Divides by 2 to generate 216 MHz master frequency
//    PLLN = 216 Multiplies PLL to generate a 432 PLL frequency
//    PLLM = 4 Divides the 8 MHz XTAL to 2 MHz for optimal PLL jitter
// RCC_CFGR
//    MCO2 = 00 For clock output 2. Not used.
//    MCO2PRE = 000 For clock output 2. Not used
//    MCO1PRE = 000 For clock output 1. Not used
//    I2SSRC = 0 I2S clock selection. Not used
//    MCO1 = 0 For clock output 1. Not used
//    RTCPRE = 01000 for divide by 8 to generate 1 MHz RTC clock
//    PPRE2 = 4 Set APB2 to 54 MHz. Limited to 90 MHz
//    PPRE1 = 9 Set APB1 to 24 MHz. Limited to 45 MHz
//    HPRE = 11 Divide by 16 gives clock of 13.5 MHz. Need min 25 if Ethernet
//    SWS = 00 Read only
//    SW = 10 Set PLL as system clock
*/
// RCC_CIR
//    Not processing any clock failures or clock ready's as interrupts so can
//    set all bits to zero.
// RCC Reset Registers
//    Set to all zeros as we are not using this function at this time
// RCC_AHB1ENR 
//    GPIOGEN = 1 Enables clock for GPIO Port G
//    GPIOFEN = 1 Enables clock for GPIO Port F
//    GPIOEEN = 1 Enables clock for GPIO Port E
//    GPIODEN = 1 Enables clock for GPIO Port D
//    GPIOCEN = 1 Enables clock for GPIO Port C
//    GPIOBEN = 1 Enables clock for GPIO Port B
//    GPIOAEN = 1 Enables clock for GPIO Port A
//    CCMDATARAMEN = 1 Core Coupled Memory RAM, must be on for debugger
//    All other bits set to zero at this time
// RCC_AHB2ENR
//    All bits set to zero at this time
// RCC_AHB3ENR
//    All bits set to zero at this time
// RCC_APB1ENR
//    - not used - DACEN = 1 Enables clock for DAC 1 and DAC2
//    PWREN = 1 Enables clock for Power Controller
//    - not used - SPI2EN = 1 Enables clock for SPI2
//    I2C2EN = 1 Enables clock for I2C2
//    All other bits set to zero at this time
// RCC_APB2ENR
//    SYSCFGEN = 1 Enables clock for System Configuration Generator
//    USART1EN = 1 Enables clock for UART 1
//    SPI1EN = 1 Enables clock SPI1
//    - not used - ADC3EN = 1 Enables clock for ADC 3
//    - not used - ADC2EN = 1 Enables clock for ADC 2
//    - not used - ADC1EN = 1 Enables clock for ADC 1
//    All other bits set to zero at this time 
// DCKCFGR2
//    SDMMCSEL = 1 Clock Source is system clock
//    CK48MSEL = 0 Clock Source is PLL
//    CECSEL = 0 Clock Source is LSE
//    All the reset clock source is System Clock
//
//***********************************************************************************************

/* using  void startup_clocks(void) instead      
     RCC_CFGR_ADDRESS,        0x00000000, //set the default value which uses the internal
     RCC_CR_ADDRESS,          0x00018983, // turn on HSE
     RCC_PLLCFGR_ADDRESS,     0x28403604, // configure PLL reset is 0x24003010
//     RCC_CR_ADDRESS,          0x00018983, // turn on HSE
     WaitTimer,               Delay1,     // timer, replaces wait for enabled
     RCC_CR_ADDRESS,          0x01018983, // turn on PLL
     WaitTimer,               Delay1,  
     RCC_CFGR_ADDRESS,        0x0008B8A2, // use PLL for sys clock
     WaitTimer,               Delay1,
*/     
     RCC_AHB1RSTR_ADDRESS,    0x00000000, // RCC AHB1 peripheral reset register
     RCC_AHB2RSTR_ADDRESS,    0x00000000, // RCC AHB2 peripheral reset register
     RCC_AHB3RSTR_ADDRESS,    0x00000000, // RCC AHB3 peripheral reset register
     RCC_APB1RSTR_ADDRESS,    0x00000000, // RCC APB1 peripheral reset register
     RCC_APB2RSTR_ADDRESS,    0x00000000, // RCC APB2 peripheral reset register
     RCC_AHB1ENR_ADDRESS,     0x0010007F, // RCC AHB1 peripheral clock enable register
                                          // CCMDATA RAM, GPIO A-G on
                                          // bit 20 must be set, it's the clock for debugging RAM
     RCC_AHB2ENR_ADDRESS,     0x00000000, // RCC AHB2 peripheral clock enable register
     RCC_AHB3ENR_ADDRESS,     0x00000000, // RCC AHB3 peripheral clock enable register
     RCC_APB1ENR_ADDRESS,     0x10420000, // RCC APB1 peripheral clock enable register
                                                 //PWR bit 28 I2C2 bit 22
     RCC_APB2ENR_ADDRESS,     0x00005010, // RCC APB2 peripheral clock enable register
     RCC_AHB1LPENR_ADDRESS,   0x0003007F, // RCC AHB1 peripheral clock enable in low power
     RCC_AHB2LPENR_ADDRESS,   0x00000000, // RCC AHB2 peripheral clock enable in low power
     RCC_AHB3LPENR_ADDRESS,   0x00000000, // RCC AHB3 peripheral clock enable in low power
     RCC_APB1LPENR_ADDRESS,   0x104008FF, // RCC APB1 peripheral clock enable in low power mode register
     RCC_APB2LPENR_ADDRESS,   0x00075F13, // RCC APB2 peripheral clock enable in low power mode register
     RCC_BDCR_ADDRESS,        0x00000000, // RCC Backup domain control register
     RCC_CSR_ADDRESS,         0x00000000, // RCC clock control and status register
     RCC_SSCGR_ADDRESS,       0x00000000, // RCC spread spectrum clock generation register
     RCC_PLLI2SCFGR_ADDRESS,  0x20003000, // RCC PLLI2S configuration register
//     RCC_PLLSAICFGR_ADDRESS,  0x54036000, // RCC PLL configuration register
//     RCC_DCKCFGR1_ADDRESS,    0x00020000, // RCC dedicated clocks configuration register
//     RCC_DCKCFGR2_ADDRESS,    0x08000000, // RCC dedicated clocks configuration register
//
//     use void init_systick(void) instead of setting here    
// SysTick System Timer. See ST Manual PM0253, page 212 for details of timer
//
// SYST_CSR
//    CLKSOURCE = 1 to use the processor clock
//    TICKINT = 1 to generate an interrupt on zero count
//    ENABLE = 1 to enable the counter
// SYST_RVR
//    This is the 24 bit countdown reload value register. This takes the
//    processor clock and decrements to zero at which time an interrupt is 
//    generated and the timer is reloaded again. If we start with a 216 MHz
//    clock and we want to generate a 1 mS interrupt, then we need a divide by
//    216,000 - 1 = 215999
// SYST_CALIB
//    NOREF = 1 for no reference clock provided
//    SKEW 1 1 for TENMS value no given
//
//     SYST_CSR_ADDRESS,        0x00000007, // SysTick Control and Status Register
//     SYST_RSR_ADDRESS,        26999,      // SysTick Reload Value Register
//     SYST_CVR_ADDRESS,        0x00000000, // SysTick Current Value Register
//     SYST_CALIB_ADDRESS,      0x4000493E, // SysTick Calibration Value Register     
//
// GPIOA Addresses
//    GPIO A (13,14) are being used for debug, must be correct  
//
     // port config order is important.
     // set alt functions correctly before turning on with the mode setting
     // set output data last  
     GPIOA_AFRL_ADDRESS,      0x55557000, // Alternate Function Low Register
     GPIOA_AFRH_ADDRESS,      0x000AA000, // Alternate Function High Register
                                          // was 0x000AA000, turned off uart1
     GPIOA_MODER_ADDRESS,     0xAA84A980, // Port Mode Register 
                                          // was 0xAAA4A980 uartrx 
     GPIOA_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOA_OSPEEDR_ADDRESS,   0xFEB0FFC0, // Output Speed Register
     GPIOA_PUPDR_ADDRESS,     0x6400AA00, // Push Up-Pull Down Register
     GPIOA_ODR_ADDRESS,       0x00000000, // Output Data Register
//     GPIOA_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOA_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOB Addresses
//
     GPIOB_AFRL_ADDRESS,      0x00000000, // Alternate Function Low Register
                                          //  0x07000000 turns on uart1
     GPIOB_AFRH_ADDRESS,      0xAA000000, // Alternate Function High Register
     
     GPIOB_ODR_ADDRESS,       0x00000040, // Output Data Register
                                          // set before changing to output 
     GPIOB_MODER_ADDRESS,     0xA0001010, // Port Mode Register
                                          // 0xA0001010 made UART1 an output
     GPIOB_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOB_OSPEEDR_ADDRESS,   0xA00013C0, // Output Speed Register
     GPIOB_PUPDR_ADDRESS,     0x00000105, // Push Up-Pull Down Register

//     GPIOB_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOB_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOC Addresses
//
     GPIOC_AFRL_ADDRESS,      0x00000000, // Alternate Function Low Register
     GPIOC_AFRH_ADDRESS,      0x00000000, // Alternate Function High Register
     
     GPIOC_MODER_ADDRESS,     0x00010100, // Port Mode Register
     GPIOC_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOC_OSPEEDR_ADDRESS,   0x00000000, // Output Speed Register
     GPIOC_PUPDR_ADDRESS,     0x00000000, // Push Up-Pull Down Register
     GPIOC_ODR_ADDRESS,       0x00000110, // Output Data Register
//     GPIOC_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOC_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOD Addresses
//
     GPIOD_AFRL_ADDRESS,      0x00700000, // Alternate Function Low Register
     GPIOD_AFRH_ADDRESS,      0x00000000, // Alternate Function High Register
     
     GPIOD_MODER_ADDRESS,     0x51000800, // Port Mode Register
     GPIOD_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOD_OSPEEDR_ADDRESS,   0x03000400, // Output Speed Register
     GPIOD_PUPDR_ADDRESS,     0x02540000, // Push Up-Pull Down Register
     GPIOD_ODR_ADDRESS,       0x00000000, // Output Data Register
//     GPIOD_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOD_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOE Addresses
//
     GPIOE_AFRL_ADDRESS,      0x00000000, // Alternate Function Low Register
     GPIOE_AFRH_ADDRESS,      0x00000000, // Alternate Function High Register
     
     GPIOE_MODER_ADDRESS,     0x00000000, // Port Mode Register
     GPIOE_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOE_OSPEEDR_ADDRESS,   0x00000000, // Output Speed Register
     GPIOE_PUPDR_ADDRESS,     0x55550000, // Push Up-Pull Down Register
     GPIOE_ODR_ADDRESS,       0x00000000, // Output Data Register
//     GPIOE_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOE_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOF Addresses
//
     GPIOF_AFRL_ADDRESS,      0x00000044, // Alternate Function Low Register
     GPIOF_AFRH_ADDRESS,      0x00000000, // Alternate Function High Register

     GPIOF_MODER_ADDRESS,     0x5540501A, // Port Mode Register // takes I2C2 SCL line low if not
                                                                // set to alt function first 
     GPIOF_OTYPER_ADDRESS,    0x00000001, // Output Type Register  // important for I2C (pin 0 SDA) open drain
     GPIOF_OSPEEDR_ADDRESS,   0x5540201F, // Output Speed Register
     GPIOF_PUPDR_ADDRESS,     0x00008028, // Push Up-Pull Down Register
     GPIOF_ODR_ADDRESS,       0x00000040, // Output Data Register , Touch Screen INT set high
//     GPIOF_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOF_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// GPIOG Addresses
//
     GPIOG_AFRL_ADDRESS,      0x00000000, // Alternate Function Low Register
     GPIOG_AFRH_ADDRESS,      0x00000000, // Alternate Function High Register     
     
     GPIOG_MODER_ADDRESS,     0x00005555, // Port Mode Register
     GPIOG_OTYPER_ADDRESS,    0x00000000, // Output Type Register
     GPIOG_OSPEEDR_ADDRESS,   0x0000FFFF, // Output Speed Register
     GPIOG_PUPDR_ADDRESS,     0x00000000, // Push Up-Pull Down Register
     GPIOG_ODR_ADDRESS,       0x00000000, // Output Data Register
//     GPIOE_BSRR_ADDRESS,      0x00000000, // bit set/reset Register
//     GPIOE_LCKR_ADDRESS,      0x00000000, // Configuration Lock Register

//
// System Configuration Controller
//
     SYSCFG_MEMRMP_ADDRESS,   0x00000000, // SYSCFG memory remap register
     SYSCFG_PMC_ADDRESS,      0x00000000, // SYSCFG peripheral mode config register
     SYSCFG_EXTICR1_ADDRESS,  0x00000000, // SYSCFG external interrupt configuration register 1
     SYSCFG_EXTICR2_ADDRESS,  0x00000500, // SYSCFG external interrupt configuration register 2
                                          // set gpio pin F6 to be an interrupt for Touch Panel
     SYSCFG_EXTICR3_ADDRESS,  0x00000000, // SYSCFG external interrupt configuration register 3
     SYSCFG_EXTICR4_ADDRESS,  0x00000000, // SYSCFG external interrupt configuration register 4
     SYSCFG_CMPCR_ADDRESS,    0x00000000, // SYSCFG compensation cell control register  
//
// DMA 1 Registers
//
     DMA1_LISR_ADDRESS,       0x00000000, // DMA1 low interrupt status register
     DMA1_HISR_ADDRESS,       0x00000000, // DMA1 high interrupt status register
     DMA1_LIFCR_ADDRESS,      0x00000000, // DMA1 low interrupt flag clear register
     DMA1_HIFCR_ADDRESS,      0x00000000, // DMA1 high interrupt flag clear register

     DMA1_S0CR_ADDRESS,       0x00000000, // DMA1 stream 0 configuration register
     DMA1_S0NDTR_ADDRESS,     0x00000000, // DMA1 stream 0 number of data register
     DMA1_S0PAR_ADDRESS,      0x00000000, // DMA1 stream 0 peripheral address register
     DMA1_S0M0AR_ADDRESS,     0x00000000, // DMA1 stream 0 memory 0 address register
     DMA1_S0M1AR_ADDRESS,     0x00000000, // DMA1 stream 0 memory 1 address register
     DMA1_S0FCR_ADDRESS,      0x00000000, // DMA1 stream 0 FIFO control register

     DMA1_S1CR_ADDRESS,       0x00000000, // DMA1 stream 1 configuration register
     DMA1_S1NDTR_ADDRESS,     0x00000000, // DMA1 stream 1 number of data register
     DMA1_S1PAR_ADDRESS,      0x00000000, // DMA1 stream 1 peripheral address register
     DMA1_S1M0AR_ADDRESS,     0x00000000, // DMA1 stream 1 memory 0 address register
     DMA1_S1M1AR_ADDRESS,     0x00000000, // DMA1 stream 1 memory 1 address register
     DMA1_S1FCR_ADDRESS,      0x00000000, // DMA1 stream 1 FIFO control register

     DMA1_S2CR_ADDRESS,       0x00000000, // DMA1 stream 2 configuration register
     DMA1_S2NDTR_ADDRESS,     0x00000000, // DMA1 stream 2 number of data register
     DMA1_S2PAR_ADDRESS,      0x00000000, // DMA1 stream 2 peripheral address register
     DMA1_S2M0AR_ADDRESS,     0x00000000, // DMA1 stream 2 memory 0 address register
     DMA1_S2M1AR_ADDRESS,     0x00000000, // DMA1 stream 2 memory 1 address register
     DMA1_S2FCR_ADDRESS,      0x00000000, // DMA1 stream 2 FIFO control register

     DMA1_S3CR_ADDRESS,       0x00000000, // DMA1 stream 3 configuration register
     DMA1_S3NDTR_ADDRESS,     0x00000000, // DMA1 stream 3 number of data register
     DMA1_S3PAR_ADDRESS,      0x00000000, // DMA1 stream 3 peripheral address register
     DMA1_S3M0A_ADDRESS,      0x00000000, // DMA1 stream 3 memory 0 address register
     DMA1_S3M1AR_ADDRESS,     0x00000000, // DMA1 stream 3 memory 1 address register
     DMA1_S3FCR_ADDRESS,      0x00000000, // DMA1 stream 3 FIFO control register

     DMA1_S4CR_ADDRESS,       0x00000000, // DMA1 stream 4 configuration register
     DMA1_S4NDTR_ADDRESS,     0x00000000, // DMA1 stream 4 number of data register
     DMA1_S4PAR_ADDRESS,      0x00000000, // DMA1 stream 4 peripheral address register
     DMA1_S4M0AR_ADDRESS,     0x00000000, // DMA1 stream 4 memory 0 address register
     DMA1_S4M1AR_ADDRESS,     0x00000000, // DMA1 stream 4 memory 1 address register
     DMA1_S4FCR_ADDRESS,      0x00000000, // DMA1 stream 4 FIFO control register

     DMA1_S5CR_ADDRESS,       0x00000000, // DMA1 stream 5 configuration register
     DMA1_S5NDTR_ADDRESS,     0x00000000, // DMA1 stream 5 number of data register
     DMA1_S5PAR_ADDRESS,      0x00000000, // DMA1 stream 5 peripheral address register
     DMA1_S5M0AR_ADDRESS,     0x00000000, // DMA1 stream 5 memory 0 address register
     DMA1_S5M1AR_ADDRESS,     0x00000000, // DMA1 stream 5 memory 1 address register
     DMA1_S5FCR_ADDRESS,      0x00000000, // DMA1 stream 5 FIFO control register

     DMA1_S6CR_ADDRESS,       0x00000000, // DMA1 stream 6 configuration register
     DMA1_S6NDTR_ADDRESS,     0x00000000, // DMA1 stream 6 number of data register
     DMA1_S6PAR_ADDRESS,      0x00000000, // DMA1 stream 6 peripheral address register
     DMA1_S6M0AR_ADDRESS,     0x00000000, // DMA1 stream 6 memory 0 address register
     DMA1_S6M1AR_ADDRESS,     0x00000000, // DMA1 stream 6 memory 1 address register
     DMA1_S6FCR_ADDRESS,      0x00000000, // DMA1 stream 6 FIFO control register

     DMA1_S7CR_ADDRESS,       0x00000000, // DMA1 stream 7 configuration register
     DMA1_S7NDTR_ADDRESS,     0x00000000, // DMA1 stream 7 number of data register
     DMA1_S7PAR_ADDRESS,      0x00000000, // DMA1 stream 7 peripheral address register
     DMA1_S7M0AR_ADDRESS,     0x00000000, // DMA1 stream 7 memory 0 address register
     DMA1_S7M1AR_ADDRESS,     0x00000000, // DMA1 stream 7 memory 1 address register
     DMA1_S7FCR_ADDRESS,      0x00000000, // DMA1 stream 7 FIFO control register
//
// DMA 2 Registers
//
     DMA2_LISR_ADDRESS,       0x00000000, // DMA2 low interrupt status register
     DMA2_HISR_ADDRESS,       0x00000000, // DMA2 high interrupt status register
     DMA2_LIFCR_ADDRESS,      0x00000000, // DMA2 low interrupt flag clear register
     DMA2_HIFCR_ADDRESS,      0x00000000, // DMA2 high interrupt flag clear register

     DMA2_S0CR_ADDRESS,       0x00000000, // DMA2 stream 0 configuration register
     DMA2_S0NDTR_ADDRESS,     0x00000000, // DMA2 stream 0 number of data register
     DMA2_S0PAR_ADDRESS,      0x00000000, // DMA2 stream 0 peripheral address register
     DMA2_S0M0AR_ADDRESS,     0x00000000, // DMA2 stream 0 memory 0 address register
     DMA2_S0M1AR_ADDRESS,     0x00000000, // DMA2 stream 0 memory 1 address register
     DMA2_S0FCR_ADDRESS,      0x00000000, // DMA2 stream 0 FIFO control register

     DMA2_S1CR_ADDRESS,       0x00000000, // DMA2 stream 1 configuration register
     DMA2_S1NDTR_ADDRESS,     0x00000000, // DMA2 stream 1 number of data register
     DMA2_S1PAR_ADDRESS,      0x00000000, // DMA2 stream 1 peripheral address register
     DMA2_S1M0AR_ADDRESS,     0x00000000, // DMA2 stream 1 memory 0 address register
     DMA2_S1M1AR_ADDRESS,     0x00000000, // DMA2 stream 1 memory 1 address register
     DMA2_S1FCR_ADDRESS,      0x00000000, // DMA2 stream 1 FIFO control register

     DMA2_S2CR_ADDRESS,       0x00000000, // DMA2 stream 2 configuration register
     DMA2_S2NDTR_ADDRESS,     0x00000000, // DMA2 stream 2 number of data register
     DMA2_S2PAR_ADDRESS,      0x00000000, // DMA2 stream 2 peripheral address register
     DMA2_S2M0AR_ADDRESS,     0x00000000, // DMA2 stream 2 memory 0 address register
     DMA2_S2M1AR_ADDRESS,     0x00000000, // DMA2 stream 2 memory 1 address register
     DMA2_S2FCR_ADDRESS,      0x00000000, // DMA2 stream 2 FIFO control register

     DMA2_S3CR_ADDRESS,       0x00000000, // DMA2 stream 3 configuration register
     DMA2_S3NDTR_ADDRESS,     0x00000000, // DMA2 stream 3 number of data register
     DMA2_S3PAR_ADDRESS,      0x00000000, // DMA2 stream 3 peripheral address register
     DMA2_S3M0A_ADDRESS,      0x00000000, // DMA2 stream 3 memory 0 address register
     DMA2_S3M1AR_ADDRESS,     0x00000000, // DMA2 stream 3 memory 1 address register
     DMA2_S3FCR_ADDRESS,      0x00000000, // DMA2 stream 3 FIFO control register

     DMA2_S4CR_ADDRESS,       0x00000000, // DMA2 stream 4 configuration register
     DMA2_S4NDTR_ADDRESS,     0x00000000, // DMA2 stream 4 number of data register
     DMA2_S4PAR_ADDRESS,      0x00000000, // DMA2 stream 4 peripheral address register
     DMA2_S4M0AR_ADDRESS,     0x00000000, // DMA2 stream 4 memory 0 address register
     DMA2_S4M1AR_ADDRESS,     0x00000000, // DMA2 stream 4 memory 1 address register
     DMA2_S4FCR_ADDRESS,      0x00000000, // DMA2 stream 4 FIFO control register

     DMA2_S5CR_ADDRESS,       0x00000000, // DMA2 stream 5 configuration register
     DMA2_S5NDTR_ADDRESS,     0x00000000, // DMA2 stream 5 number of data register
     DMA2_S5PAR_ADDRESS,      0x00000000, // DMA2 stream 5 peripheral address register
     DMA2_S5M0AR_ADDRESS,     0x00000000, // DMA2 stream 5 memory 0 address register
     DMA2_S5M1AR_ADDRESS,     0x00000000, // DMA2 stream 5 memory 1 address register
     DMA2_S5FCR_ADDRESS,      0x00000000, // DMA2 stream 5 FIFO control register

     DMA2_S6CR_ADDRESS,       0x00000000, // DMA2 stream 6 configuration register
     DMA2_S6NDTR_ADDRESS,     0x00000000, // DMA2 stream 6 number of data register
     DMA2_S6PAR_ADDRESS,      0x00000000, // DMA2 stream 6 peripheral address register
     DMA2_S6M0AR_ADDRESS,     0x00000000, // DMA2 stream 6 memory 0 address register
     DMA2_S6M1AR_ADDRESS,     0x00000000, // DMA2 stream 6 memory 1 address register
     DMA2_S6FCR_ADDRESS,      0x00000000, // DMA2 stream 6 FIFO control register

     DMA2_S7CR_ADDRESS,       0x00000000, // DMA2 stream 7 configuration register
     DMA2_S7NDTR_ADDRESS,     0x00000000, // DMA2 stream 7 number of data register
     DMA2_S7PAR_ADDRESS,      0x00000000, // DMA2 stream 7 peripheral address register
     DMA2_S7M0AR_ADDRESS,     0x00000000, // DMA2 stream 7 memory 0 address register
     DMA2_S7M1AR_ADDRESS,     0x00000000, // DMA2 stream 7 memory 1 address register
     DMA2_S7FCR_ADDRESS,      0x00000000, // DMA2 stream 7 FIFO control register
//
// Extended interrupts and events controller
//
     EXTI_IMR_ADDRESS,        0x00000000, // Interrupt mask register
     EXTI_EMR_ADDRESS,        0x00000000, // Event mask register
     EXTI_RTSR_ADDRESS,       0x00000000, // Rising trigger selection register
     EXTI_FTSR_ADDRESS,       0x00000000, // Falling trigger selection register
     EXTI_SWIER_ADDRESS,      0x00000000, // Software interrupt event register
     EXTI_PR_ADDRESS,         0x00000000, // Pending register
//
// A/D 1 Converters
//
     ADC1_SR_ADDRESS,         0x00000000, // ADC1 status register
     ADC1_CR1_ADDRESS,        0x00000000, // ADC1 control register 1
     ADC1_CR2_ADDRESS,        0x00000000, // ADC1 control register 2
     ADC1_SMPR1_ADDRESS,      0x00000000, // ADC1 sample time register 1
     ADC1_SMPR2_ADDRESS,      0x00000000, // ADC1 sample time register 2
     ADC1_JOFR1_ADDRESS,      0x00000000, // ADC1 injected channel data offset register 1
     ADC1_JOFR2_ADDRESS,      0x00000000, // ADC1 injected channel data offset register 2
     ADC1_JOFR3_ADDRESS,      0x00000000, // ADC1 injected channel data offset register 3
     ADC1_JOFR4_ADDRESS,      0x00000000, // ADC1 injected channel data offset register 4
     ADC1_HTR_ADDRESS,        0x00000FFF, // ADC1 watchdog higher threshold register
     ADC1_LTR_ADDRESS,        0x00000000, // ADC1 watchdog lower threshold register
     ADC1_SQR1_ADDRESS,       0x00000000, // ADC1 regular sequence register 1
     ADC1_SQR2_ADDRESS,       0x00000000, // ADC1 regular sequence register 2
     ADC1_SQR3_ADDRESS,       0x00000000, // ADC1 regular sequence register 3
     ADC1_JSQR_ADDRESS,       0x00000000, // ADC1 injected sequence register
     ADC1_JDR1_ADDRESS,       0x00000000, // ADC1 injected data register 1
     ADC1_JDR2_ADDRESS,       0x00000000, // ADC1 injected data register 2
     ADC1_JDR3_ADDRESS,       0x00000000, // ADC1 injected data register 3
     ADC1_JDR4_ADDRESS,       0x00000000, // ADC1 injected data register 4
     ADC1_DR_ADDRESS,         0x00000000, // ADC1 regular data register
//
// A/D 2 Converters
//
     ADC2_SR_ADDRESS,         0x00000000, // ADC2 status register
     ADC2_CR1_ADDRESS,        0x00000000, // ADC2 control register 1
     ADC2_CR2_ADDRESS,        0x00000000, // ADC2 control register 2
     ADC2_SMPR1_ADDRESS,      0x00000000, // ADC2 sample time register 1
     ADC2_SMPR2_ADDRESS,      0x00000000, // ADC2 sample time register 2
     ADC2_JOFR1_ADDRESS,      0x00000000, // ADC2 injected channel data offset register 1
     ADC2_JOFR2_ADDRESS,      0x00000000, // ADC2 injected channel data offset register 2
     ADC2_JOFR3_ADDRESS,      0x00000000, // ADC2 injected channel data offset register 3
     ADC2_JOFR4_ADDRESS,      0x00000000, // ADC2 injected channel data offset register 4
     ADC2_HTR_ADDRESS,        0x00000000, // ADC2 watchdog higher threshold register
     ADC2_LTR_ADDRESS,        0x00000000, // ADC2 watchdog lower threshold register
     ADC2_SQR1_ADDRESS,       0x00000000, // ADC2 regular sequence register 1
     ADC2_SQR2_ADDRESS,       0x00000000, // ADC2 regular sequence register 2
     ADC2_SQR3_ADDRESS,       0x00000000, // ADC2 regular sequence register 3
     ADC2_JSQR_ADDRESS,       0x00000000, // ADC2 injected sequence register
     ADC2_JDR1_ADDRESS,       0x00000000, // ADC2 injected data register 1
     ADC2_JDR2_ADDRESS,       0x00000000, // ADC2 injected data register 2
     ADC2_JDR3_ADDRESS,       0x00000000, // ADC2 injected data register 3
     ADC2_JDR4_ADDRESS,       0x00000000, // ADC2 injected data register 4
     ADC2_DR_ADDRESS,         0x00000000, // ADC2 regular data register
//
// A/D 3 Converters
//
     ADC3_SR_ADDRESS,         0x00000000, // ADC3 status register
     ADC3_CR1_ADDRESS,        0x00000000, // ADC3 control register 1
     ADC3_CR2_ADDRESS,        0x00000000, // ADC3 control register 2
     ADC3_SMPR1_ADDRESS,      0x00000000, // ADC3 sample time register 1
     ADC3_SMPR2_ADDRESS,      0x00000000, // ADC3 sample time register 2
     ADC3_JOFR1_ADDRESS,      0x00000000, // ADC3 injected channel data offset register 1
     ADC3_JOFR2_ADDRESS,      0x00000000, // ADC3 injected channel data offset register 2
     ADC3_JOFR3_ADDRESS,      0x00000000, // ADC3 injected channel data offset register 3
     ADC3_JOFR4_ADDRESS,      0x00000000, // ADC3 injected channel data offset register 4
     ADC3_HTR_ADDRESS,        0x00000000, // ADC3 watchdog higher threshold register
     ADC3_LTR_ADDRESS,        0x00000000, // ADC3 watchdog lower threshold register
     ADC3_SQR1_ADDRESS,       0x00000000, // ADC3 regular sequence register 1
     ADC3_SQR2_ADDRESS,       0x00000000, // ADC3 regular sequence register 2
     ADC3_SQR3_ADDRESS,       0x00000000, // ADC3 regular sequence register 3
     ADC3_JSQR_ADDRESS,       0x00000000, // ADC3 injected sequence register
     ADC3_JDR1_ADDRESS,       0x00000000, // ADC3 injected data register 1
     ADC3_JDR2_ADDRESS,       0x00000000, // ADC3 injected data register 2
     ADC3_JDR3_ADDRESS,       0x00000000, // ADC3 injected data register 3
     ADC3_JDR4_ADDRESS,       0x00000000, // ADC3 injected data register 4
     ADC3_DR_ADDRESS,         0x00000000, // ADC3 regular data register
//
// A/D Common Registers
//
     ADC_CSR_ADDRESS,         0x00000000, // ADC Common status register
     ADC_CCR_ADDRESS,         0x00000000, // ADC common control register
     ADC_CDR_ADDRESS,         0x00000000, // ADC common regular data register for dual and triple modes
//
// D/A Converter
//
     DAC_CR_ADDRESS,          0x00000000, // DAC Control Register
     DAC_SWTRIGR_ADDRESS,     0x00000000, // DAC Software Trigger Register
     DAC_DHR12R1_ADDRESS,     0x00000000, // DAC channel1 12-bit right-aligned data holding register
     DAC_DHR12L1_ADDRESS,     0x00000000, // DAC channel1 12-bit left aligned data holding register
     DAC_DHR8R1_ADDRESS,      0x00000000, // DAC channel1 8-bit right aligned data holding register
     DAC_DHR12R2_ADDRESS,     0x00000000, // DAC channel2 12-bit right aligned data holding register
     DAC_DHR12L2_ADDRESS,     0x00000000, // DAC channel2 12-bit left aligned data holding register
     DAC_DHR8R2_ADDRESS,      0x00000000, // DAC channel2 8-bit right-aligned data holding register
     DAC_DHR12RD_ADDRESS,     0x00000000, // DAC Dual DAC 12-bit right-aligned data holding register
     DAC_DHR12LD_ADDRESS,     0x00000000, // DAC DUAL DAC 12-bit left aligned data holding register
     DAC_DHR8RD_ADDRESS,      0x00000000, // DAC DUAL DAC 8-bit right aligned data holding register
     DAC_DOR1_ADDRESS,        0x00000000, // DAC channel1 data output register (DAC_DOR1)
     DAC_DOR2_ADDRESS,        0x00000000, // DAC channel2 data output register (DAC_DOR2)
     DAC_SR_ADDRESS,          0x00000000, // DAC status register
//
// Timer 1
//
     TIM1_CR1_ADDRESS,        0x00000000, // TIM1 control register 1
     TIM1_CR2_ADDRESS,        0x00000000, // TIM1 control register 2
     TIM1_SMCR_ADDRESS,       0x00000000, // TIM1 slave mode control register
     TIM1_DIER_ADDRESS,       0x00000000, // TIM1 DMA/interrupt enable register
     TIM1_SR_ADDRESS,         0x00000000, // TIM1 status register
     TIM1_EGR_ADDRESS,        0x00000000, // TIM1 event generation register
     TIM1_CCMR1_ADDRESS,      0x00000000, // TIM1 capture/compare mode register 1
     TIM1_CCMR2_ADDRESS,      0x00000000, // TIM1 capture/compare mode register 2
     TIM1_CCER_ADDRESS,       0x00000000, // TIM1 capture/compare enable register
     TIM1_CNT_ADDRESS,        0x00000000, // TIM1 counter
     TIM1_PSC_ADDRESS,        0x00000000, // TIM1 prescaler
     TIM1_ARR_ADDRESS,        0xFFFFFFFF, // TIM1 auto-reload register
     TIM1_RCR_ADDRESS,        0x00000000, // TIM1 repetition counter register
     TIM1_CCR1_ADDRESS,       0x00000000, // TIM1 capture/compare register 1
     TIM1_CCR2_ADDRESS,       0x00000000, // TIM1 capture/compare register 2
     TIM1_CCR3_ADDRESS,       0x00000000, // TIM1 capture/compare register 3
     TIM1_CCR4_ADDRESS,       0x00000000, // TIM1 capture/compare register 4
     TIM1_BDTR_ADDRESS,       0x00000000, // TIM1 break and dead-time register
     TIM1_DCR_ADDRESS,        0x00000000, // TIM1 DMA control register
     TIM1_DMAR_ADDRESS,       0x00000000, // TIM1 DMA address for full transfer
     TIM1_CCMR3_ADDRESS,      0x00000000, // TIM1 capture/compare mode register 3
     TIM1_CCR5_ADDRESS,       0x00000000, // TIM1 capture/compare register 5
     TIM1_CCR6_ADDRESS,       0x00000000, // TIM1 capture/compare register 6
//
// Timer 8
//
     TIM8_CR1_ADDRESS,        0x00000000, // TIM8 control register 1
     TIM8_CR2_ADDRESS,        0x00000000, // TIM8 control register 2
     TIM8_SMCR_ADDRESS,       0x00000000, // TIM8 slave mode control register
     TIM8_DIER_ADDRESS,       0x00000000, // TIM8 DMA/interrupt enable register
     TIM8_SR_ADDRESS,         0x00000000, // TIM8 status register
     TIM8_EGR_ADDRESS,        0x00000000, // TIM8 event generation register
     TIM8_CCMR1_ADDRESS,      0x00000000, // TIM8 capture/compare mode register 1
     TIM8_CCMR2_ADDRESS,      0x00000000, // TIM8 capture/compare mode register 2
     TIM8_CCER_ADDRESS,       0x00000000, // TIM8 capture/compare enable register
     TIM8_CNT_ADDRESS,        0x00000000, // TIM8 counter
     TIM8_PSC_ADDRESS,        0x00000000, // TIM8 prescaler
     TIM8_ARR_ADDRESS,        0xFFFFFFFF, // TIM8 auto-reload register
     TIM8_RCR_ADDRESS,        0x00000000, // TIM8 repetition counter register
     TIM8_CCR1_ADDRESS,       0x00000000, // TIM8 capture/compare register 1
     TIM8_CCR2_ADDRESS,       0x00000000, // TIM8 capture/compare register 2
     TIM8_CCR3_ADDRESS,       0x00000000, // TIM8 capture/compare register 3
     TIM8_CCR4_ADDRESS,       0x00000000, // TIM8 capture/compare register 4
     TIM8_BDTR_ADDRESS,       0x00000000, // TIM8 break and dead-time register
     TIM8_DCR_ADDRESS,        0x00000000, // TIM8 DMA control register
     TIM8_DMAR_ADDRESS,       0x00000000, // TIM8 DMA address for full transfer
     TIM8_CCMR3_ADDRESS,      0x00000000, // TIM1 capture/compare mode register 3
     TIM8_CCR5_ADDRESS,       0x00000000, // TIM1 capture/compare register 5
     TIM8_CCR6_ADDRESS,       0x00000000, // TIM1 capture/compare register 6
//
// Timer 2
//
     TIM2_CR1_ADDRESS,        0x00000000, // TIM2 control register 1
     TIM2_CR2_ADDRESS,        0x00000000, // TIM2 control register 2
     TIM2_SMCR_ADDRESS,       0x00000000, /// TIM2 slave mode control register
     TIM2_DIER_ADDRESS,       0x00000000, // TIM2 DMA/interrupt enable register
     TIM2_SR_ADDRESS,         0x00000000, // TIM2 status register
     TIM2_EGR_ADDRESS,        0x00000000, // TIM2 event generation register
     TIM2_CCMR1_ADDRESS,      0x00000000, // TIM2 capture/compare mode register 1
     TIM2_CCMR2_ADDRESS,      0x00000000, // TIM2 capture/compare mode register 2
     TIM2_CCER_ADDRESS,       0x00000000, // TIM2 capture/compare enable register
     TIM2_CNT_ADDRESS,        0x00000000, // TIM2 counter
     TIM2_PSC_ADDRESS,        0x00000000, // TIM2 prescaler
     TIM2_ARR_ADDRESS,        0xFFFFFFFF, // TIM2 auto-reload register
     TIM2_RCR_ADDRESS,        0x00000000, // TIM2 repetition counter register
     TIM2_CCR1_ADDRESS,       0x00000000, // TIM2 capture/compare register 1
     TIM2_CCR2_ADDRESS,       0x00000000, // TIM2 capture/compare register 2
     TIM2_CCR3_ADDRESS,       0x00000000, // TIM2 capture/compare register 3
     TIM2_CCR4_ADDRESS,       0x00000000, // TIM2 capture/compare register 4
     TIM2_DCR_ADDRESS,        0x00000000, // TIM2 DMA control register
     TIM2_DMAR_ADDRESS,       0x00000000, // TIM2 DMA address for full transfer
     TIM2_OR_ADDRESS,         0x00000000, // TIM2 option register
//
// Timer 3
//
     TIM3_CR1_ADDRESS,        0x00000000, // TIM3 control register 1
     TIM3_CR2_ADDRESS,        0x00000000, // TIM3 control register 2
     TIM3_SMCR_ADDRESS,       0x00000000, // TIM3 slave mode control register
     TIM3_DIER_ADDRESS,       0x00000000, // TIM3 DMA/interrupt enable register
     TIM3_SR_ADDRESS,         0x00000000, // TIM3 status register
     TIM3_EGR_ADDRESS,        0x00000000, // TIM3 event generation register
     TIM3_CCMR1_ADDRESS,      0x00000000, // TIM3 capture/compare mode register 1
     TIM3_CCMR2_ADDRESS,      0x00000000, // TIM3 capture/compare mode register 2
     TIM3_CCER_ADDRESS,       0x00000000, // TIM3 capture/compare enable register
     TIM3_CNT_ADDRESS,        0x00000000, // TIM3 counter
     TIM3_PSC_ADDRESS,        0x00000000, // TIM3 prescaler
     TIM3_ARR_ADDRESS,        0xFFFFFFFF, // TIM3 auto-reload register
     TIM3_RCR_ADDRESS,        0x00000000, // TIM3 repetition counter register
     TIM3_CCR1_ADDRESS,       0x00000000, // TIM3 capture/compare register 1
     TIM3_CCR2_ADDRESS,       0x00000000, // TIM3 capture/compare register 2
     TIM3_CCR3_ADDRESS,       0x00000000, // TIM3 capture/compare register 3
     TIM3_CCR4_ADDRESS,       0x00000000, // TIM3 capture/compare register 4
     TIM3_DCR_ADDRESS,        0x00000000, // TIM3 DMA control register
     TIM3_DMAR_ADDRESS,       0x00000000, // TIM3 DMA address for full transfer
//
// Timer 4
//
     TIM4_CR1_ADDRESS,        0x00000000, // TIM4 control register 1
     TIM4_CR2_ADDRESS,        0x00000000, // TIM4 control register 2
     TIM4_SMCR_ADDRESS,       0x00000000, // TIM4 slave mode control register
     TIM4_DIER_ADDRESS,       0x00000000, // TIM4 DMA/interrupt enable register
     TIM4_SR_ADDRESS,         0x00000000, // TIM4 status register
     TIM4_EGR_ADDRESS,        0x00000000, // TIM4 event generation register
     TIM4_CCMR1_ADDRESS,      0x00000000, // TIM4 capture/compare mode register 1
     TIM4_CCMR2_ADDRESS,      0x00000000, // TIM4 capture/compare mode register 2
     TIM4_CCER_ADDRESS,       0x00000000, // TIM4 capture/compare enable register
     TIM4_CNT_ADDRESS,        0x00000000, // TIM4 counter
     TIM4_PSC_ADDRESS,        0x00000000, // TIM4 prescaler
     TIM4_ARR_ADDRESS,        0xFFFFFFFF, // TIM4 auto-reload register
     TIM4_RCR_ADDRESS,        0x00000000, // TIM4 repetition counter register
     TIM4_CCR1_ADDRESS,       0x00000000, // TIM4 capture/compare register 1
     TIM4_CCR2_ADDRESS,       0x00000000, // TIM4 capture/compare register 2
     TIM4_CCR3_ADDRESS,       0x00000000, // TIM4 capture/compare register 3
     TIM4_CCR4_ADDRESS,       0x00000000, // TIM4 capture/compare register 4
     TIM4_DCR_ADDRESS,        0x00000000, // TIM4 DMA control register
     TIM4_DMAR_ADDRESS,       0x00000000, // TIM4 DMA address for full transfer
//
// Timer 5
//
     TIM5_CR1_ADDRESS,        0x00000000, // TIM5 control register 1
     TIM5_CR2_ADDRESS,        0x00000000, // TIM5 control register 2
     TIM5_SMCR_ADDRESS,       0x00000000, // TIM5 slave mode control register
     TIM5_DIER_ADDRESS,       0x00000000, // TIM5 DMA/interrupt enable register
     TIM5_SR_ADDRESS,         0x00000000, // TIM5 status register
     TIM5_EGR_ADDRESS,        0x00000000, // TIM5 event generation register
     TIM5_CCMR1_ADDRESS,      0x00000000, // TIM5 capture/compare mode register 1
     TIM5_CCMR2_ADDRESS,      0x00000000, // TIM5 capture/compare mode register 2
     TIM5_CCER_ADDRESS,       0x00000000, // TIM5 capture/compare enable register
     TIM5_CNT_ADDRESS,        0x00000000, // TIM5 counter
     TIM5_PSC_ADDRESS,        0x00000000, // TIM5 prescaler
     TIM5_ARR_ADDRESS,        0xFFFFFFFF, // TIM5 auto-reload register
     TIM5_RCR_ADDRESS,        0x00000000, // TIM5 repetition counter register
     TIM5_CCR1_ADDRESS,       0x00000000, // TIM5 capture/compare register 1
     TIM5_CCR2_ADDRESS,       0x00000000, // TIM5 capture/compare register 2
     TIM5_CCR3_ADDRESS,       0x00000000, // TIM5 capture/compare register 3
     TIM5_CCR4_ADDRESS,       0x00000000, // TIM5 capture/compare register 4
     TIM5_DCR_ADDRESS,        0x00000000, // TIM5 DMA control register
     TIM5_DMAR_ADDRESS,       0x00000000, // TIM5 DMA address for full transfer
     TIM5_OR_ADDRESS,         0x00000000, // TIM5 option register
//
// Timer 9
//
     TIM9_CR1_ADDRESS,        0x00000000, // TIM9 Control Reister 1
     TIM9_SMCR_ADDRESS,       0x00000000, // TIM9 Slave Mode Control Register
     TIM9_DIER_ADDRESS,       0x00000000, // TIM9 Interrupt Enable Register
     TIM9_SR_ADDRESS,         0x00000000, // TIM9 Status Register
     TIM9_EGR_ADDRESS,        0x00000000, // TIM9 Event Generation Register
     TIM9_CCMR1_ADDRESS,      0x00000000, // TIM9 Capture/Compare Mode Register 1
     TIM9_CCER_ADDRESS,       0x00000000, // TIM9 Capture/Compare Enable Register
     TIM9_CNT_ADDRESS,        0x00000000, // TIM9 Counter
     TIM9_PSC_ADDRESS,        0x00000000, // TIM9 Prescaler
     TIM9_ARR_ADDRESS,        0x00000000, // TIM9 Auto-Reload Register
     TIM9_CCR1_ADDRESS,       0x00000000, // TIM9 Capture/Compare Register 1
     TIM9_CCR2_ADDRESS,       0x00000000, // TIM9 Capture/Compare Register 2
//
// Timer 12
//
     TIM12_CR1_ADDRESS,       0x00000000, // TIM12 Control Reister 1
     TIM12_SMCR_ADDRESS,      0x00000000, // TIM12 Slave Mode Control Register
     TIM12_DIER_ADDRESS,      0x00000000, // TIM12 Interrupt Enable Register
     TIM12_SR_ADDRESS,        0x00000000, // TIM12 Status Register
     TIM12_EGR_ADDRESS,       0x00000000, // TIM12 Event Generation Register
     TIM12_CCMR1_ADDRESS,     0x00000000, // TIM12 Capture/Compare Mode Register 1
     TIM12_CCER_ADDRESS,      0x00000000, // TIM12 Capture/Compare Enable Register
     TIM12_CNT_ADDRESS,       0x00000000, // TIM12 Counter
     TIM12_PSC_ADDRESS,       0x00000000, // TIM12 Prescaler
     TIM12_ARR_ADDRESS,       0x00000000, // TIM12 Auto-Reload Register
     TIM12_CCR1_ADDRESS,      0x00000000, // TIM12 Capture/Compare Register 1
     TIM12_CCR2_ADDRESS,      0x00000000, // TIM12 Capture/Compare Register 2
//
// Timer 10
//
     TIM10_CR1_ADDRESS,       0x00000000, // TIM10 Control Reister 1
     TIM10_SMCR_ADDRESS,      0x00000000, // TIM10 Slave Mode Control Register
     TIM10_DIER_ADDRESS,      0x00000000, // TIM10 Interrupt Enable Register
     TIM10_SR_ADDRESS,        0x00000000, // TIM10 Status Register
     TIM10_EGR_ADDRESS,       0x00000000, // TIM10 Event Generation Register
     TIM10_CCMR1_ADDRESS,     0x00000000, // TIM10 Capture/Compare Mode Register 1
     TIM10_CCER_ADDRESS,      0x00000000, // TIM10 Capture/Compare Enable Register
     TIM10_CNT_ADDRESS,       0x00000000, // TIM10 Counter
     TIM10_PSC_ADDRESS,       0x00000000, // TIM10 Prescaler
     TIM10_ARR_ADDRESS,       0x00000000, // TIM10 Auto-Reload Register
     TIM10_CCR1_ADDRESS,      0x00000000, // TIM10 Capture/Compare Register 1
//
// Timer 11
//
     TIM11_CR1_ADDRESS,       0x00000000, // TIM11 Control Reister 1
     TIM11_SMCR_ADDRESS,      0x00000000, // TIM11 Slave Mode Control Register
     TIM11_DIER_ADDRESS,      0x00000000, // TIM11 Interrupt Enable Register
     TIM11_SR_ADDRESS,        0x00000000, // TIM11 Status Register
     TIM11_EGR_ADDRESS,       0x00000000, // TIM11 Event Generation Register
     TIM11_CCMR1_ADDRESS,     0x00000000, // TIM11 Capture/Compare Mode Register 1
     TIM11_CCER_ADDRESS,      0x00000000, // TIM11 Capture/Compare Enable Register
     TIM11_CNT_ADDRESS,       0x00000000, // TIM11 Counter
     TIM11_PSC_ADDRESS,       0x00000000, // TIM11 Prescaler
     TIM11_ARR_ADDRESS,       0x00000000, // TIM11 Auto-Reload Register
     TIM11_CCR1_ADDRESS,      0x00000000, // TIM11 Capture/Compare Register 1
//
// Timer 13
//
     TIM13_CR1_ADDRESS,       0x00000000, // TIM13 Control Reister 1
     TIM13_SMCR_ADDRESS,      0x00000000, // TIM13 Slave Mode Control Register
     TIM13_DIER_ADDRESS,      0x00000000, // TIM13 Interrupt Enable Register
     TIM13_SR_ADDRESS,        0x00000000, // TIM13 Status Register
     TIM13_EGR_ADDRESS,       0x00000000, // TIM13 Event Generation Register
     TIM13_CCMR1_ADDRESS,     0x00000000, // TIM13 Capture/Compare Mode Register 1
     TIM13_CCER_ADDRESS,      0x00000000, // TIM13 Capture/Compare Enable Register
     TIM13_CNT_ADDRESS,       0x00000000, // TIM13 Counter
     TIM13_PSC_ADDRESS,       0x00000000, // TIM13 Prescaler
     TIM13_ARR_ADDRESS,       0x00000000, // TIM13 Auto-Reload Register
     TIM13_CCR1_ADDRESS,      0x00000000, // TIM13 Capture/Compare Register 1
//
// Timer 14
//
     TIM14_CR1_ADDRESS,       0x00000000, // TIM14 Control Reister 1
     TIM14_SMCR_ADDRESS,      0x00000000, // TIM14 Slave Mode Control Register
     TIM14_DIER_ADDRESS,      0x00000000, // TIM14 Interrupt Enable Register
     TIM14_SR_ADDRESS,        0x00000000, // TIM14 Status Register
     TIM14_EGR_ADDRESS,       0x00000000, // TIM14 Event Generation Register
     TIM14_CCMR1_ADDRESS,     0x00000000, // TIM14 Capture/Compare Mode Register 1
     TIM14_CCER_ADDRESS,      0x00000000, // TIM14 Capture/Compare Enable Register
     TIM14_CNT_ADDRESS,       0x00000000, // TIM14 Counter
     TIM14_PSC_ADDRESS,       0x00000000, // TIM14 Prescaler
     TIM14_ARR_ADDRESS,       0x00000000, // TIM14 Auto-Reload Register
     TIM14_CCR1_ADDRESS,      0x00000000, // TIM14 Capture/Compare Register 1
//
// Timer 6
//
     TIM6_CR1_ADDRESS,        0x00000000, // TIM6 Control Reister 1
     TIM6_CR2_ADDRESS,        0x00000000, // TIM6 Control Reister 2
     TIM6_DIER_ADDRESS,       0x00000000, // TIM6 Interrupt Enable Register
     TIM6_SR_ADDRESS,         0x00000000, // TIM6 Status Register
     TIM6_EGR_ADDRESS,        0x00000000, // TIM6 Event Generation Register
     TIM6_CNT_ADDRESS,        0x00000000, // TIM6 Counter
     TIM6_PSC_ADDRESS,        0x00000000, // TIM6 Prescaler
     TIM6_ARR_ADDRESS,        0xFFFFFFFF, // TIM6 Auto-Reload Register
//
// Timer 7
//
     TIM7_CR1_ADDRESS,        0x00000000, // TIM7 Control Reister 1
     TIM7_CR2_ADDRESS,        0x00000000, // TIM7 Control Reister 2
     TIM7_DIER_ADDRESS,       0x00000000, // TIM7 Interrupt Enable Register
     TIM7_SR_ADDRESS,         0x00000000, // TIM7 Status Register
     TIM7_EGR_ADDRESS,        0x00000000, // TIM7 Event Generation Register
     TIM7_CNT_ADDRESS,        0x00000000, // TIM7 Counter Register
     TIM7_PSC_ADDRESS,        0x00000000, // TIM7 Prescaler Register
     TIM7_ARR_ADDRESS,        0xFFFFFFFF, // TIM7 Auto-Reload Register
//
// Low Power Timer 1
//
     LPTIM1_ISR_ADDRESS,      0x00000000, // LPTIM1 Interrupt and Status Register
     LPTIM1_ICR_ADDRESS,      0x00000000, // LPTIM1 Interrupt Clear Register
     LPTIM1_IER_ADDRESS,      0x00000000, // LPTIM1 Interrupt Enable Register
     LPTIM1_CFGR_ADDRESS,     0x00000000, // LPTIM1 Configuration Register
     LPTIM1_CR_ADDRESS,       0x00000000, // LPTIM1 Control Register
     LPTIM1_CMP_ADDRESS,      0x00000000, // LPTIM1 Compare Register
     LPTIM1_ARR_ADDRESS,      0x00000001, // LPTIM1 Auto-Reload Register
     LPTIM1_CNT_ADDRESS,      0x00000000, // LPTIM1 Counter Register
     LPTIM1_OR_ADDRESS,       0x00000000, // LPTIM1 Option Register
//
// Independent Watch Dog Timer
//
     IWDG_KR_ADDRESS,         0x00000000, // IWDG Key Register
     IWDG_PR_ADDRESS,         0x00000000, // IWDG Prescale Register
     IWDG_RLR_ADDRESS,        0x00000FFF, // IWDG Reload Register
     IWDG_SR_ADDRESS,         0x00000000, // IWDG Status Register
     IWDG_WINR_ADDRESS,       0x00000FFF, // IWDG Window Register
//
// Window Watch Dog Timer
//
     WWDG_CR_ADDRESS,         0x0000007F, // WWDG Control Register
     WWDG_CFR_ADDRESS,        0x0000007F, // WWDG Configuration Register
     WWDG_SR_ADDRESS,         0x00000000, // WWDG Status Register
//
// Real Time Clock
//
     RTC_TR_ADDRESS,          0x00000000, // RTC Time Register
     RTC_DR_ADDRESS,          0x00002101, // RTC Date Register
     RTC_CR_ADDRESS,          0x00000000, // RTC Control Register
     RTC_ISR_ADDRESS,         0x00000007, // RTC Initialization and Status Register
     RTC_PRER_ADDRESS,        0x007F00FF, // RTC Prescale Register
     RTC_WUTR_ADDRESS,        0x0000FFFF, // RTC Wakeup Timer Register
     RTC_ALRMAR_ADDRESS,      0x00000000, // RTC Alarm A Register
     RTC_ALRMBR_ADDRESS,      0x00000000, // RTC Alarm B Register
     RTC_WPR_ADDRESS,         0x00000000, // RTC Write Protection Register
     RTC_SSR_ADDRESS,         0x00000000, // RTC Sub Second Register
     RTC_SHIFTR_ADDRESS,      0x00000000, // RTC Shift Control Register
     RTC_TSTR_ADDRESS,        0x00000000, // RTC Timestamp Time Register
     RTC_TSDR_ADDRESS,        0x00000000, // RTC Timestamp Date Register
     RTC_TSSSR_ADDRESS,       0x00000000, // RTC Calibration Register
     RTC_CALR_ADDRESS,        0x00000000, // RTC Calibration Register
     RTC_TAMPCR_ADDRESS,      0x00000000, // RTC Tamper Configuration Register
     RTC_ALRMASSR_ADDRESS,    0x00000000, // RTC Alarm A Sub Second Register
     RTC_ALRMBSSR_ADDRESS,    0x00000000, // RTC Alarm B Sub Second Register 
     RTC_OR_ADDRESS,          0x00000000, // RTC Option Register
     RTC_BKP0R_ADDRESS,       0x00000000, // RTC Backup Register 0
     RTC_BKP1R_ADDRESS,       0x00000000, // RTC Backup Register 1
     RTC_BKP2R_ADDRESS,       0x00000000, // RTC Backup Register 2
     RTC_BKP3R_ADDRESS,       0x00000000, // RTC Backup Register 3
     RTC_BKP4R_ADDRESS,       0x00000000, // RTC Backup Register 4
     RTC_BKP5R_ADDRESS,       0x00000000, // RTC Backup Register 5
     RTC_BKP6R_ADDRESS,       0x00000000, // RTC Backup Register 6
     RTC_BKP7R_ADDRESS,       0x00000000, // RTC Backup Register 7
     RTC_BKP8R_ADDRESS,       0x00000000, // RTC Backup Register 8
     RTC_BKP9R_ADDRESS,       0x00000000, // RTC Backup Register 9
     RTC_BKP10R_ADDRESS,      0x00000000, // RTC Backup Register 10
     RTC_BKP11R_ADDRESS,      0x00000000, // RTC Backup Register 11
     RTC_BKP12R_ADDRESS,      0x00000000, // RTC Backup Register 12
     RTC_BKP13R_ADDRESS,      0x00000000, // RTC Backup Register 13
     RTC_BKP14R_ADDRESS,      0x00000000, // RTC Backup Register 14
     RTC_BKP15R_ADDRESS,      0x00000000, // RTC Backup Register 15
     RTC_BKP16R_ADDRESS,      0x00000000, // RTC Backup Register 16
     RTC_BKP17R_ADDRESS,      0x00000000, // RTC Backup Register 17
     RTC_BKP18R_ADDRESS,      0x00000000, // RTC Backup Register 18
     RTC_BKP19R_ADDRESS,      0x00000000, // RTC Backup Register 19
     RTC_BKP20R_ADDRESS,      0x00000000, // RTC Backup Register 20
     RTC_BKP21R_ADDRESS,      0x00000000, // RTC Backup Register 21
     RTC_BKP22R_ADDRESS,      0x00000000, // RTC Backup Register 22
     RTC_BKP23R_ADDRESS,      0x00000000, // RTC Backup Register 23
     RTC_BKP24R_ADDRESS,      0x00000000, // RTC Backup Register 24
     RTC_BKP25R_ADDRESS,      0x00000000, // RTC Backup Register 25
     RTC_BKP26R_ADDRESS,      0x00000000, // RTC Backup Register 26
     RTC_BKP27R_ADDRESS,      0x00000000, // RTC Backup Register 27
     RTC_BKP28R_ADDRESS,      0x00000000, // RTC Backup Register 28
     RTC_BKP29R_ADDRESS,      0x00000000, // RTC Backup Register 29
     RTC_BKP30R_ADDRESS,      0x00000000, // RTC Backup Register 30
     RTC_BKP31R_ADDRESS,      0x00000000, // RTC Backup Register 31
//
// UART 1
//
     USART1_CR1_ADDRESS,      0x00000000, // UART1 Control Register 1
     USART1_CR2_ADDRESS,      0x00000000, // UART1 Control Register 2 
     USART1_CR3_ADDRESS,      0x00000000, // UART1 Control Register 3 
     USART1_BRR_ADDRESS,      0x00000000, // UART1 Baud Rate Register
     USART1_GTPR_ADDRESS,     0x00000000, // UART1 Guard Time and Prescale Register
     USART1_SR_ADDRESS,       0x020000C0, // UART1 Interrupt and Status Register
     USART1_DR_ADDRESS,       0x00000000, // UART1 Data Register
//
// UART 2
//
     USART2_CR1_ADDRESS,      0x00000000, // UART2 Control Register 1
     USART2_CR2_ADDRESS,      0x00000000, // UART2 Control Register 2 
     USART2_CR3_ADDRESS,      0x00000000, // UART2 Control Register 3 
     USART2_BRR_ADDRESS,      0x00000000, // UART2 Baud Rate Register
     USART2_GTPR_ADDRESS,     0x00000000, // UART2 Guard Time and Prescale Register
     USART2_SR_ADDRESS,       0x020000C0, // UART2 Interrupt and Status Register
     USART2_DR_ADDRESS,       0x00000000, // UART2 Data Register
//
// End of record (registers loaded)
//
     0xFFFFFFFF,              0x00000000, // Last entry in database
  };
/*
// =============================================================================
// test with 0's removed
const unsigned int init_arrayB[][2] =
  {
     PWR_CR_ADDRESS,          0x00004000, // PWR Power control register
     RCC_AHB1ENR_ADDRESS,     0x0010007F, // RCC AHB1 peripheral clock enable register
     RCC_APB1ENR_ADDRESS,     0x10420000, // RCC APB1 peripheral clock enable register
     RCC_APB2ENR_ADDRESS,     0x00005010, // RCC APB2 peripheral clock enable register
     RCC_AHB1LPENR_ADDRESS,   0x0003007F, // RCC AHB1 peripheral clock enable in low power
     RCC_APB1LPENR_ADDRESS,   0x104008FF, // RCC APB1 peripheral clock enable in low power mode register
     RCC_APB2LPENR_ADDRESS,   0x00075F13, // RCC APB2 peripheral clock enable in low power mode register
     RCC_PLLI2SCFGR_ADDRESS,  0x20003000, // RCC PLLI2S configuration register

     GPIOA_AFRL_ADDRESS,      0x55557000, // Alternate Function Low Register
     GPIOA_AFRH_ADDRESS,      0x000AA000, // Alternate Function High Register
     GPIOA_MODER_ADDRESS,     0xAA84A980, // Port Mode Register 
     GPIOA_OSPEEDR_ADDRESS,   0xFEB0FFC0, // Output Speed Register
     GPIOA_PUPDR_ADDRESS,     0x6400AA00, // Push Up-Pull Down Register

     GPIOB_AFRH_ADDRESS,      0xAA000000, // Alternate Function High Register
     GPIOB_ODR_ADDRESS,       0x00000040, // Output Data Register
     GPIOB_MODER_ADDRESS,     0xA0001010, // Port Mode Register
     GPIOB_OSPEEDR_ADDRESS,   0xA00013C0, // Output Speed Register
     GPIOB_PUPDR_ADDRESS,     0x00000105, // Push Up-Pull Down Register
     
     GPIOC_MODER_ADDRESS,     0x00010100, // Port Mode Register
     GPIOC_ODR_ADDRESS,       0x00000110, // Output Data Register

     GPIOD_AFRL_ADDRESS,      0x00700000, // Alternate Function Low Register
     GPIOD_MODER_ADDRESS,     0x51000800, // Port Mode Register
     GPIOD_OSPEEDR_ADDRESS,   0x03000400, // Output Speed Register
     GPIOD_PUPDR_ADDRESS,     0x02540000, // Push Up-Pull Down Register

     GPIOE_PUPDR_ADDRESS,     0x55550000, // Push Up-Pull Down Register

     GPIOF_AFRL_ADDRESS,      0x00000044, // Alternate Function Low Register
     GPIOF_MODER_ADDRESS,     0x5540501A, // Port Mode Register // takes I2C2 SCL line low if not
     GPIOF_OTYPER_ADDRESS,    0x00000001, // Output Type Register  // important for I2C (pin 0 SDA) open drain
     GPIOF_OSPEEDR_ADDRESS,   0x5540201F, // Output Speed Register
     GPIOF_PUPDR_ADDRESS,     0x00008028, // Push Up-Pull Down Register
     GPIOF_ODR_ADDRESS,       0x00000040, // Output Data Register , Touch Screen INT set high
     
     GPIOG_MODER_ADDRESS,     0x00005555, // Port Mode Register
     GPIOG_OSPEEDR_ADDRESS,   0x0000FFFF, // Output Speed Register

     SYSCFG_EXTICR2_ADDRESS,  0x00000500, // SYSCFG external interrupt configuration register 2

     ADC1_HTR_ADDRESS,        0x00000FFF, // ADC1 watchdog higher threshold register

     TIM1_ARR_ADDRESS,        0xFFFFFFFF, // TIM1 auto-reload register
     TIM8_ARR_ADDRESS,        0xFFFFFFFF, // TIM8 auto-reload register
     TIM2_ARR_ADDRESS,        0xFFFFFFFF, // TIM2 auto-reload register
     TIM3_ARR_ADDRESS,        0xFFFFFFFF, // TIM3 auto-reload register
     TIM4_ARR_ADDRESS,        0xFFFFFFFF, // TIM4 auto-reload register
     TIM5_ARR_ADDRESS,        0xFFFFFFFF, // TIM5 auto-reload register
     TIM6_ARR_ADDRESS,        0xFFFFFFFF, // TIM6 Auto-Reload Register
     TIM7_ARR_ADDRESS,        0xFFFFFFFF, // TIM7 Auto-Reload Register
     LPTIM1_ARR_ADDRESS,      0x00000001, // LPTIM1 Auto-Reload Register

     IWDG_RLR_ADDRESS,        0x00000FFF, // IWDG Reload Register
     IWDG_WINR_ADDRESS,       0x00000FFF, // IWDG Window Register

     WWDG_CR_ADDRESS,         0x0000007F, // WWDG Control Register
     WWDG_CFR_ADDRESS,        0x0000007F, // WWDG Configuration Register

     RTC_DR_ADDRESS,          0x00002101, // RTC Date Register
     RTC_ISR_ADDRESS,         0x00000007, // RTC Initialization and Status Register
     RTC_PRER_ADDRESS,        0x007F00FF, // RTC Prescale Register
     RTC_WUTR_ADDRESS,        0x0000FFFF, // RTC Wakeup Timer Register

     USART1_SR_ADDRESS,       0x020000C0, // UART1 Interrupt and Status Register

     USART2_SR_ADDRESS,       0x020000C0, // UART2 Interrupt and Status Register
// End of record (registers loaded)
     0xFFFFFFFF,              0x00000000, // Last entry in database
  };
*/

// =============================================================================
  uint32_t i = 0;
  uint32_t *reg_address;
  uint32_t reg_intval;
  uint32_t reg_value;

void initialize_registers(void)
{  
  // Extract data from array
  while ( init_array[i][0] != 0xFFFFFFFF )
  {
    reg_address = (uint32_t *) init_array[i][0];
    reg_intval = init_array[i][0];
    reg_value = init_array[i][1];
    // Check if 16 bit transfer.
    // For 16 bit transfers change 0x8XXXXXXX to 0x4XXXXXXX.
    if ((reg_intval & 0xf0000000) == 0x80000000)
    {
      reg_address = (uint32_t *) ((reg_intval & 0x0FFFFFF) | 0x40000000); 
      *reg_address = (uint16_t) reg_value;
    }
    // Process normal 32 bit transfers
    else *reg_address = reg_value;      
    // Get the next item to process
    i++;
  } 
} 
// =============================================================================
//
//   fast startup version
//   does not write if table value for the register is zero.
//
//   will work at powerup, may need to zero registers on a program restart
//
// status: written tested, working 

void initialize_registers_fast(void)
{  
  // Extract data from array
  while ( init_array[i][0] != 0xFFFFFFFF )
  {
    if( init_array[i][1] )
    {
      reg_address = (uint32_t *) init_array[i][0];
      reg_intval = init_array[i][0];
      reg_value = init_array[i][1];
      // Check if 16 bit transfer.
      // For 16 bit transfers change 0x8XXXXXXX to 0x4XXXXXXX.
      if ((reg_intval & 0xf0000000) == 0x80000000)
      {
        reg_address = (uint32_t *) ((reg_intval & 0x0FFFFFF) | 0x40000000); 
        *reg_address = (uint16_t) reg_value;
      }
      // Process normal 32 bit transfers
      else *reg_address = reg_value;     
    }
    // Get the next item to process
    i++;
  } 
} 
/*
// same as standard but using smaller table init_arrayB
// no noticeable improvement

void initialize_registers_fast(void)
{  
  // Extract data from array
  while ( init_arrayB[i][0] != 0xFFFFFFFF )
  {
    reg_address = (uint32_t *) init_arrayB[i][0];
    reg_intval = init_arrayB[i][0];
    reg_value = init_arrayB[i][1];
    // Check if 16 bit transfer.
    // For 16 bit transfers change 0x8XXXXXXX to 0x4XXXXXXX.
    if ((reg_intval & 0xf0000000) == 0x80000000)
    {
      reg_address = (uint32_t *) ((reg_intval & 0x0FFFFFF) | 0x40000000); 
      *reg_address = (uint16_t) reg_value;
    }
    // Process normal 32 bit transfers
    else *reg_address = reg_value;     
    // Get the next item to process
    i++;
  } 
}   
*/
