//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Feb 20, 2018	             Design Documents                         CSI_MC59.c
//
//			routines for MC59 keypad microphone
//
// Programmer
//  Jerry Wanger
//
// Copyright
//  Connect Systems Inc.
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  process_keypad                    Handle keyboard data
//  lowpulseDM                        pulse DM line
//  mc59_read_keypad                  read keypad for any keypresses
//  mc59_getkey                       gets keynumber
//  mc59_process_data                 extracts key press data from raw
//  mc59_get_data                     gets raw data from keypad

//==============================================================================
#include <stdint.h>
#include "CSI_MC59.h"
#include "CSI_GPIO.h"
#include "CSI_SSD1963.h"
#include "CSI_TIMERS.h"
#include "CSI_SWITCHES.h"

uint8_t ascii_convert_string[17] = {
  " 0123456789C<>O*#"
};

uint32_t dtmf_codes_timer;

uint32_t mc59_raw_data;
uint32_t mc59_flag;
uint32_t mc59_key;
uint32_t mc59_release_flag;
uint32_t mc59_temp_raw_data;

uint32_t process_keypad_state;
uint32_t parm_char;
uint32_t parm_char_flag;
uint32_t radio_keypad_char;
uint32_t radio_keypad_flag;
uint32_t radio_keypad_release_char;
uint32_t radio_keypad_release_flag;

//
//******************************************************************************
//
// Feb 20, 2018			Design Documents                 process_keypad
//
// This program processes the keyboard data for both the radio and for the
// other functions.
//
// For the radio, we need both a key pressed and a key released function
//
// For the special code sequences, we need only key pressed and return true
// ASCII data
//
// For the programming program, we need only key pressed and return true ASCII
// data
//
// This program goes into a 1 ms timer interrupt
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  if key press, the following variables are filled and flags set:
//    parm_char, parm_char_flag
//    dtmf_code_char, dtmf_code_flag
//    radio_keypad_char, radio_keypad_flag
//  If key released, the following variables and filled and flags set:
//    radio_keypad_release_char, radio_keypad_release_flag
//    
// External functions called
//  
//
// External variables used
//  parm_char
//  parm_char_flag
//  dtmf_code_char
//  dtmf_code_flag
//  long_timer_value
//  process_keypad_state
//  radio_keypad_char
//  radio_keypad_flag
//  radio_keypad_release_char
//  radio_keypad_release_flag
//
// Internal variables used
//  ascii_convert_string 
//
// Constants
//
// History
//    10/01/2018 revised to use switch table
//    11/30/2018 added mc59_release_flag = 0; to release section.
//
// Status
//  written
//

void process_keypad(void)
{
  // Handle keyboard data for special codes and programming mode
  if (mc59_flag)
  {
    // convert raw data to ASCII data and pass it on to other programs
    parm_char = ascii_convert_string[mc59_key];
    parm_char_flag = 1;
    dtmf_code_char =  parm_char;
    dtmf_code_flag = 1;
  }
  // First handles keyboard data for the radio
  if (process_keypad_state == 0)
  {
    if (mc59_flag)
    {
      radio_keypad_char = mc59_key;
      radio_keypad_release_char = radio_keypad_char;
      CS800_buttons[13].press_flag = 1;
      mc59_flag = 0;
      process_keypad_state++;
    }
    return;
  }
  if (process_keypad_state == 1)
  {
    if (mc59_release_flag)
    {
      CS800_buttons[13].release_flag = 1;
      mc59_release_flag = 0;
      process_keypad_state = 0;
    }
    return;
  }  
  process_keypad_state = 0;
}
//
//******************************************************************************
//
// Feb 22, 2018			Design Documents                mc59_read_keypad
//
// This program reads the keypad inside the MC59 style hand mike. The key to 
// understanding this routine is to understand that a single line called "DM"
// is used for clocking data from the hand mike back to the microprocessor. The
// sequence is the rising edge of the pulse output from the microprocessor is
// used to advance the circuitry inside the hand mike. After the rising edge 
// pulse the microprocessor then reads the data on the same line. After a total
// of 30 pulses, the microprocessor has to then interpret the results of the
// data. The possible sequences for the 16 keys is as follows:
//
// 1  11111 00000 01000 11111 11111 11111 
// 2  11111 00000 00100 11111 11111 11111 
// 3  11111 00000 00010 11111 11111 11111 
// C  11111 00000 00001 11111 11111 11111 
//
// 4  11111 00000 11111 01000 11111 11111 
// 5  11111 00000 11111 00100 11111 11111 
// 6  11111 00000 11111 00010 11111 11111 
// ^  11111 00000 11111 00001 11111 11111 
//
// 7  11111 00000 11111 11111 01000 11111
// 8  11111 00000 11111 11111 00100 11111
// 9  11111 00000 11111 11111 00010 11111
// v  11111 00000 11111 11111 00001 11111
//
// *  11111 00000 11111 11111 11111 01000
// 0  11111 00000 11111 11111 11111 00100
// #  11111 00000 11111 11111 11111 00010
// O  11111 00000 11111 11111 11111 00001
//
// If no key is pressed, then the following pattern is generated:
//    11111 00000 11111 11111 11111 11111
//
// We will use this table to compare the results. Because an integer in the 
// microprocessor we are using is 32 bits, we will set the 2 most significant
// bits to 0. A revised table is as follows:
//
// 1  0011 1110 0000 0100 0111 1111 1111 1111 0x3E047FFF
// 2  0011 1110 0000 0010 0111 1111 1111 1111 0x3E027FFF
// 3  0011 1110 0000 0001 0111 1111 1111 1111 0x3E017FFF
// C  0011 1110 0000 0000 1111 1111 1111 1111 0x3E00FFFF
//
// 4  0011 1110 0000 1111 1010 0011 1111 1111 0x3E0FA3FF
// 5  0011 1110 0000 1111 1001 0011 1111 1111 0x3E0F93FF
// 6  0011 1110 0000 1111 1000 1011 1111 1111 0x3E0F8BFF
// ^  0011 1110 0000 1111 1000 0111 1111 1111 0x3E0F87FF
//
// 7  0011 1110 0000 1111 1111 1101 0001 1111 0x3E0FFD1F
// 8  0011 1110 0000 1111 1111 1100 1001 1111 0x3E0FFC9F
// 9  0011 1110 0000 1111 1111 1100 0101 1111 0x3E0FFC5F
// v  0011 1110 0000 1111 1111 1100 0011 1111 0x3E0FFC3F
//
// *  0011 1110 0000 1111 1111 1111 1110 1000 0x3E0FFFE8
// 0  0011 1110 0000 1111 1111 1111 1110 0100 0x3E0FFFE4
// #  0011 1110 0000 1111 1111 1111 1110 0010 0x3E0FFFE2
// O  0011 1110 0000 1111 1111 1111 1110 0001 0x3E0FFFE1
//
// If no key is pressed, then the following pattern is generated:
//    0011 1110 0000 1111 1111 1111 1111 1111 0x3E0FFFFF
//
// This program should be called from a periodic interrupt. However, it could
// be called from the idle loop if the idle loop does not have any software
// delays embedded within.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  mc59_key indicates the key that was pressed
//  mc59_flag indicates the MC59_key is valid data
//  mc_59_release_flag indicates previous key was released
//
// External functions called
//  mike_dm_high
//  mike_dm_low
//  read_mike_dm
//
// External variables used
//  mc59_raw_data
//  mc59_temp_raw_data
//
// Internal variables used
//  mc59_state
//
// Constants
//
// History
//    04/04/2019 revised to run on 10 usec timer every 5 millisec
//    08/19/2019  revised, made state switch, moved state variable to local
//                now working with both new mic and old
//
// Status
//  written, tested
//
// ===============
// counter for keypad process burst
uint8_t mc59_keypad_timer = 0;
extern void timer_10micro_enable( int onoff );
// =============================================================================
//
//    Low pulse DM
//       toggles the Dm line low then high.
//       When 30 pulses have been sent it turns off the 10 usec timer. 
//
void lowpulseDM( void )
{
    static uint8_t pulse_count = 0;
    
    mike_dm_low();
    delay(5);
    mike_dm_high();
    if(pulse_count++ > 28 )
    {
      pulse_count = 0;
      // turn off timer
      timer_10micro_enable( 0 );      
    }
}
// =============================================================================

void mc59_read_keypad(void)
{
  static int mc59_state;
  
  switch(  mc59_state )
  {
  // The data is not read at this time to allow the DM line to stabalize.
  case 0:
    {
      mc59_state++;
      break;
    }
  // State 1 reads the data and then sends a DM line pulse.
  // It also begins processing the data.
  case 1:
    {
      // Put a new bit in shift register. 
      mc59_raw_data = mc59_raw_data << 1;
      if (read_mike_dm())
      {
        mc59_raw_data |= 1;
      }
      mc59_raw_data &= 0x3FFFFFFF;
      lowpulseDM();

      // If we don't have 0x3E0XXXXX, then we are not ready to process the data
      if ((mc59_raw_data & 0xFFF00000) != 0x3E000000)
      {
        mc59_state = 0;
        break;
      }

      // If the pattern is no keys pressed,
      // then go back to state 0 and wait for more data
      if ((mc59_raw_data & 0x3FFFFFFF) == 0x3E0FFFFF)
      {
        mc59_raw_data = 0;
        mc59_state = 0;
        break;      
      }

      // If we got here we have data that can be processed.
      // Store the results and advance to the next state.
      mc59_temp_raw_data = mc59_raw_data;
      mc59_raw_data = 0;
      mc59_state++;
      break; 
    }
  // The data is not read at this time to allow the DM line to stabalize.
  case 2:
    {
      mc59_state++;
      break;
    }    
  // State 3 reads the data and sets the DM line low. It also processes the data
  // at this time. We are looking for two sets of data that are the same to 
  // debounce the key
  case 3:
    {
      // shift left, put a new 0 at bit 0
      mc59_raw_data <<= 1;
      // If a 1 put new bit in shift register.
      if (read_mike_dm())
      {
        mc59_raw_data |= 1;
      }
      mc59_raw_data &= 0x3FFFFFFF;
      lowpulseDM();

      // If we don't have 0x3E0XXXXX, then we are not ready to process the data
      if ((mc59_raw_data & 0xFFF00000) != 0x3E000000)
      {
        mc59_state = 2;
        break;
      }

      // If the current mc59_raw_data and the temp_mc59_raw_data does not match,
      // go back to state 0 and try for another set of data
      if ((mc59_raw_data & 0x3FFFFFFF) != (mc59_temp_raw_data & 0x3FFFFFFF))
      {
        mc59_state = 0;
        break;
      }
      // If we match one of the 16 possible keys, we pass the information to
      // another program and advance to the next state waiting for a key release.
      switch(mc59_raw_data - 0x3E000000)
      {
      case 0xFFFE4: mc59_key = 0x01; break; // 0 key
      case 0x47FFF: mc59_key = 0x02; break; // 1 
      case 0x27FFF: mc59_key = 0x03; break; // 2 
      case 0x17FFF: mc59_key = 0x04; break; // 3 
      case 0xFA3FF: mc59_key = 0x05; break; // 4 
      case 0xF93FF: mc59_key = 0x06; break; // 5
      case 0xF8BFF: mc59_key = 0x07; break; // 6
      case 0xFFD1F: mc59_key = 0x08; break; // 7
      case 0xFFC9F: mc59_key = 0x09; break; // 8
      case 0xFFC5F: mc59_key = 0x0A; break; // 9
      case 0x0FFFF: mc59_key = 0x0B; break; // C
      case 0xF87FF: mc59_key = 0x0C; break; // ^
      case 0xFFC3F: mc59_key = 0x0D; break; // v
      case 0xFFFE1: mc59_key = 0x0E; break; // O
      case 0xFFFE8: mc59_key = 0x0F; break; // *
      case 0xFFFE2: mc59_key = 0x10; break; // #
      default: mc59_state = 2; return;
      }    
      mc59_flag = 1;
      mc59_raw_data = 0;
      mc59_state++;
      break;
    }
  // State 4 is now waiting for a key release. A key release is defined as no
  // key pressed. Sets the DM line high thereby triggerring the keyboard logic.
  // The data is not read at this time to allow the DM line to stabalize.
  case 4:
  {
    mc59_state++;
    break;
  }
  // State 5 reads the data and processes the data at this time.
  case 5:
    {
      // Put new bit in shift register. 
      mc59_raw_data = mc59_raw_data << 1;
      if (read_mike_dm())
      {
        mc59_raw_data |= 1;
      }
      mc59_raw_data &= 0x3FFFFFFF;
      lowpulseDM();

      // If we don't have 0x3E0XXXXX, then we are not ready to process the data
      if ((mc59_raw_data & 0xFFF00000) != 0x3E000000)
      {
        mc59_state = 4;
        return;
      }

      // If the current data is the same as the previous data, the key is still
      // pressed so continue processing data
      if ((mc59_raw_data & 0x3FFFFFFF) == (mc59_temp_raw_data & 0x3FFFFFFF))
      {
        mc59_raw_data = 0;
        mc59_state = 4;
        break;
      }

      // Any other pattern indicates the key is released.
      // Set the flag to indicate the key has been released and
      // go back to state 0 waiting to process more keys.
      // REVISED, only a valid "no key" is a release, prevents keys repeating
      if ( mc59_raw_data == 0x3E0FFFFF )
      {
        mc59_release_flag = 1;
        mc59_raw_data = 0;
        mc59_state = 0;
      }
      break;
    }
  default:
    {
      // If we got here then there is an error in the program.
      // Set the state back to state 0
      mc59_state = 0;
      break;
    }
  }
}
// =============================================================================
// revision
//   mc59_getkey
//   mc59_process_data
//   mc59_get_data
//
//  08/19/2019  Tested, working with both new mic and old
// =============================================================================
//
//  get key number from raw data
//
//  if the data does not match any key, return 0

uint32_t mc59_getkey( uint32_t raw_data )
{
  uint32_t key = 0;
  
  switch(raw_data - 0x3E000000)
  {
    case 0xFFFE4: key = 0x01; break; // 0 key
    case 0x47FFF: key = 0x02; break; // 1 
    case 0x27FFF: key = 0x03; break; // 2 
    case 0x17FFF: key = 0x04; break; // 3 
    case 0xFA3FF: key = 0x05; break; // 4 
    case 0xF93FF: key = 0x06; break; // 5
    case 0xF8BFF: key = 0x07; break; // 6
    case 0xFFD1F: key = 0x08; break; // 7
    case 0xFFC9F: key = 0x09; break; // 8
    case 0xFFC5F: key = 0x0A; break; // 9
    case 0x0FFFF: key = 0x0B; break; // C
    case 0xF87FF: key = 0x0C; break; // ^
    case 0xFFC3F: key = 0x0D; break; // v
    case 0xFFFE1: key = 0x0E; break; // O
    case 0xFFFE8: key = 0x0F; break; // *
    case 0xFFFE2: key = 0x10; break; // #
    default: break;
  }
  return key;
}
// =============================================================================
// 
//  process data is run when mc59_get_data() has
//    placed 30 bits of data in mc59 temp raw data 
//
uint8_t mc59_pulse_synch = 0;

void mc59_process_data(void)
{
  static uint8_t  mc59_keydown = 0;
  uint32_t key = 0;
  uint32_t tr_data;
  
  tr_data = mc59_temp_raw_data;

  // set synch flag if aligned
  if( (tr_data & 0x3FF00000) == 0x3E000000 )
  {
    mc59_pulse_synch = 1;
  }
  else mc59_pulse_synch = 0;  
  
  int i = 0;
  // align data with synch
  while ((tr_data & 0xFFF00000) != 0x3E000000)
  {
    // if a low bit needs to roll to high bit
    if( tr_data & 1 ) tr_data |= 0x40000000;
    // shift right 1
    tr_data >>= 1;
    if( i++ > 30) return;
  }

  // synch was found, get the key
  key = mc59_getkey( tr_data );
  if( short_press_timeout == 0 )
  {
    // if key was down
    if( mc59_keydown )
    {
      // test for button release, is current a nokeypress? 
      if( key == 0 )
      {
        mc59_keydown = 0;
        // setup key release info
        mc59_key = key;
        mc59_release_flag = 1;
        short_press_timeout = 200;
      }
    }
    else  // key was up
    {
      // test for button push
      if( key )
      {    
        mc59_keydown = 1;
        // setup key press info
        mc59_key = key;  
        mc59_flag = 1; 
        short_press_timeout = 200;      
      }
    }
  }
}
// =============================================================================
//
// get data runs on the 10 micro timer
//  bursts of 30 cycles then a 5 millisec break
//  data is reading 1 extra bit to allow for synch finder to add one bit
//  not in synch.

void mc59_get_data( void )
{
  static int pulse_count = 0;
  static int p_timer = 4;
  
  // read every fourth cycle
  if( p_timer == 0 )
  {
    p_timer = 4;
    // get data
    mc59_raw_data = mc59_raw_data << 1;
    if (read_mike_dm())
    {
      mc59_raw_data |= 1;
    }

    mike_dm_low();
    delay(5);
    if( pulse_count < 29)
    {
      mike_dm_high();
    }
    pulse_count++;
  }
  p_timer--;  
  
  if(pulse_count > 30 )
  {
    pulse_count = 0;
    // turn off timer
    timer_10micro_enable( 0 );
    // transfer data
    mc59_temp_raw_data = mc59_raw_data;
    // if synch align read one more bit roll extra bit to last bit
    if( mc59_temp_raw_data & 0x40000000 )
    {
      mc59_temp_raw_data |= 1;
      mc59_temp_raw_data &= 0x3FFFFFFF;
    }    
    // process the data when it's ready (will occur every 5 msec)
    mc59_process_data();
    // if the processor was not synched read one extra bit
    if( mc59_pulse_synch == 0 )
    {
      pulse_count = -1;
    }
  }    
}

// end of MC59.c

