//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// August 2, 2018	    Design Documents                         CSI_menus.c 
// 
//             Routines for display of the radio menu.
//                  for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  08/02/2018  started, moved portions from process packets
//  10/15/2018  added menu revision for 5 tab analog channel
//  06/07/2019  refactored to use RGB_colors
//  06/24/2019  revised menu tracking to use menu_stack
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//  
//  display_menu                 displays the CS800d radio menu
//  display_submenu              displays a submenu 
//  get_menu_index               get index of current menu item from table
//  get_nav_key                  gets a navigation key, up, down, select, cancel
//  display_text_box             displays a two line text box
//  display_panel_menu           displays panel menu
//  push_menu_stack              put a menu item on the menu stack
//  pop_menu_stack               clear the top menu item, & point to previous
//  get_menu_stack               get the top menu item on the stack
//  reset_menu_stack             clear the menu stack and point to first
//  process_panel_menu           handles the panel menu, and submenus
//  display_error_list           displays error list screen
//  display_event_list           displays event list screen
//  question_box                 pop up question box, for yes or no response
//  process_question_box         save or cancel for color settings
//  reset_panel_menu             reset panel menu variables to start
//  message_box                  displays a message box
//  display_sub_menu             displays a submenu screen
//  display_radio_choices        display radio selection screen
//  process_radio_select         handle radio selection screen
//  process_color_selector       handle color selector screen
//  process_menu_level           handle the first menu level
//  process_menu_level2          handle the second menu level
//  process_e_list               handle error or event list
//
//==============================================================================
#include "CSI_Menus.h"
#include <stdint.h>
#include <string.h>
#include "CSI_MC59.h"
#include "CSI_eeprom.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_SCREENS.h"
#include "CSI_GRAPHICS.h"
#include "CSI_COLOR.h"
#include "CSI_SWITCHES.h"
#include "CSI_TouchPanel.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_SSD1963.h"
#include "CSI_FEATURES.h"
#include "CSI_RADIO.h"
#include "CSI_diagnostics.h"
#include "CSI_i2c.h"

#include "CSI_CS800_TGMonitor.h"

#ifdef radio_FT857D
#include "CSI_FT857_SCREENS.h"
#include "CSI_FT857D.h"
#endif /* FT857D */

// menu navigation variables
int menu_current_level = 0;
int menu_current_item = 0;
// menu previous level is always menu_current_level-1
// menu item in previous menu level that was before the current menu level
int menu_prev_item = 0;
int menu_table_items;  // is the total number of items in the table

uint16_t menu_tab_title_x = 10;
uint16_t menu_tab_title_y = 75;
uint8_t msg1_buf[16];
uint8_t msg2_buf[16];

int select_box_width = 315;
int select_box_height = 48;

// location of popup boxes for each menupage
// in mode 2, menus
// menupage is the index
// xloc, yloc, width, height

box_spec info_box_loc[6] = {
{ 400, 220, 360, 100 },   // contacts
{ 400, 220, 360, 100 },   // scan
{ 20, 220, 360, 100 },    // zone
{ 20, 270, 360, 100 },    // messages  
{ 20, 270, 360, 100 },    // call log
{ 20, 270, 360, 100 }     // utilities
};

// menu data

// level
// item_number
// item_text
// previous_level_item_number
// next_level_item_number
// x_loc
// y_loc

// packet types for different menitem types
// packet  4  - display text information
// packet  6  - main menu (tabs)
// packet  7  - sub menu
// packet  8  - 
// packet  9  - display stored information, no action, editable?
// packet 11  - on off selector
// packet 12  - list item selector


/* in header, here for reference 
// menu_item
typedef struct { 
  int  level;
  int  item_num;
  int  action_type;
  int  sub_indx;
  int  pos_x;
  int  pos_y;
  char msg1[16];
  char msg2[16];
} Menu_table;
*/


// the total number of items in the table
int CS800D_table_items = 85;  

Menu_item CS800D_Menu[85] =
{
// menu level, item number, action type, index to submenu, x,y, font, msg1, msg2  
  {1, 1, MAIN,  0, 10, 75, 4, "Contacts", "" },
  {1, 2, MAIN,  0, 10, 75 + 132, 4, "Scan", "" },
  {1, 3, MAIN,  0, 10, 75 + 2 * 132, 4, "Zone" },
  {1, 4, MAIN,  6, 10, 75 + 3 * 132, 4, "Messages", "" },
  {1, 5, MAIN, 13, 10, 75 + 4 * 132, 4, "Call_Log", "" },
  {1, 6, MAIN, 16, 10, 75 + 5 * 132, 4, "Utilities", "" },

  {2, 1, SUB,   0, 370, 110, 4, "Messages", "Inbox" },  
  {2, 2, SUB,   0, 370, 135, 4, "Messages", "Write" }, 
  {2, 3, SUB,   0, 370, 160, 4, "Messages", "Quick Text" }, 
  {2, 4, SUB,   0, 370, 185, 4, "Messages", "Sent Items" }, 
  {2, 5, SUB,   0, 370, 210, 4, "Messages", "Drafts" }, 
  {2, 6, SUB,   0, 370, 235, 4, "Messages", "Digital Voice Pl" }, 
  {2, 7, SUB,   0, 370, 260, 4, "Messages", "Digital Voice Re" },
  
  {2, 1, SUB,   0, 500, 110, 4, "Call Log", "Missed" },  
  {2, 2, SUB,   0, 500, 135, 4, "Call Log", "Answered" }, 
  {2, 3, SUB,   0, 500, 160, 4, "Call Log", "Outgoing" }, 
  
  {2, 1, SUB,  20, 540, 110, 4, "Utilities", "Radio Settings" },  
  {2, 2, SUB,  29, 540, 135, 4, "Utilities", "Radio Info" }, 
  {2, 3, SUB,  42, 540, 160, 4, "Utilities", "Radio Program" }, 
  {2, 4, SUB,  44, 540, 185, 4, "Utilities", "GPS" }, 

  {3, 1, SUB,  49, 540, 110, 4, "Radio Settings", "Tone/Alerts" },  
  {3, 2, SUB,  66, 540, 135, 4, "Radio Settings", "Power" }, 
  {3, 3, SUB,  69, 540, 160, 4, "Radio Settings", "Backlight" }, 
  {3, 4, SUB,  71, 540, 185, 4, "Radio Settings", "Intro screen" }, 
  {3, 5, SUB,  73, 540, 210, 4, "Radio Settings", "Language" }, 
  {3, 6, SUB,  77, 540, 235, 4, "Radio Settings", "LED indicator" }, 
  {3, 7, SUB,  79, 540, 260, 4, "Radio Settings", "Encrypt" },
  {3, 8, SUB,  81, 540, 285, 4, "Radio Settings", "Passwrd Lock" },  
  {3, 9, SUB,  83, 540, 310, 4, "Radio Settings", "Display Mode" },  
  
  // two column list
  {3, 1, SUB,   0, 310, 110, 4, "Radio Info", "Radio Alias" },  
  {3, 2, SUB,   0, 310, 135, 4, "Radio Info", "My Radio ID" }, 
  {3, 3, SUB,   0, 310, 160, 4, "Radio Info", "Model Name" }, 
  {3, 4, SUB,   0, 310, 185, 4, "Radio Info", "Model Number" }, 
  {3, 5, SUB,   0, 310, 210, 4, "Radio Info", "Firmware Ver." }, 
  {3, 6, SUB,   0, 310, 235, 4, "Radio Info", "Hardware Ver." }, 
  {3, 7, SUB,   0, 310, 260, 4, "Radio Info", "Bootloader Ver." },
  {3, 8, SUB,   0, 310, 285, 4, "Radio Info", "Front Panel Ver." },  
  {3, 9, SUB,   0, 310, 310, 4, "Radio Info", "Database Ver." },    
  {3, 10, SUB,  0, 310, 335, 4, "Radio Info", "Serial Number" },
  {3, 11, SUB,  0, 540, 110, 4, "Radio Info", "Ext Device ID" },    
  {3, 12, SUB,  0, 540, 135, 4, "Radio Info", "Freq Range" },
  {3, 13, SUB,  0, 540, 160, 4, "Radio Info", "Last Prog Time" }, 

  {3, 1, SUB,   0, 540, 110, 4, "Radio Program", "CH Program" },  
  {3, 2, SUB,   0, 540, 135, 4, "Radio Program", "Contact Prog" },

  {3, 1, SUB,   0, 540, 110, 4, "GPS", "GPS Switch" },  
  {3, 2, SUB,   0, 540, 135, 4, "GPS", "View Location" }, 
  {3, 3, SUB,   0, 540, 160, 4, "GPS", "Time Zone" }, 
  {3, 4, SUB,   0, 540, 185, 4, "GPS", "Pos Info Tx" }, 
  {3, 5, SUB,   0, 540, 210, 4, "GPS", "Sender Location" },
  
  {4, 1, SUB,   0, 310, 110, 4, "Tone/Alerts", "All Tones" },  
  {4, 2, SUB,   0, 310, 135, 4, "Tone/Alerts", "ChaFreeInd" },
  {4, 3, SUB,   0, 310, 160, 4, "Tone/Alerts", "Keypad Tones" },
  {4, 4, SUB,   0, 310, 185, 4, "Tone/Alerts", "Power Up" },
  {4, 5, SUB,   0, 310, 210, 4, "Tone/Alerts", "Private Call" },
  {4, 6, SUB,   0, 310, 235, 4, "Tone/Alerts", "Group Call" },
  {4, 7, SUB,   0, 310, 260, 4, "Tone/Alerts", "Text Message" },  
  {4, 8, SUB,   0, 310, 285, 4, "Tone/Alerts", "Call Alert" },
  {4, 9, SUB,   0, 310, 310, 4, "Tone/Alerts", "Talk Permit" },
  {4, 10, SUB,  0, 310, 335, 4, "Tone/Alerts", "LoneWorkerPre" },
  {4, 11, SUB,  0, 540, 110, 4, "Tone/Alerts", "Tx Forbidden" },
  {4, 12, SUB,  0, 540, 135, 4, "Tone/Alerts", "TOT-Pre" },
  {4, 13, SUB,  0, 540, 160, 4, "Tone/Alerts", "Low Battery" },
  {4, 14, SUB,  0, 540, 185, 4, "Tone/Alerts", "Scan" },
  {4, 15, SUB,  0, 540, 210, 4, "Tone/Alerts", "ChaAnnunciate" },
  {4, 16, SUB,  0, 540, 235, 4, "Tone/Alerts", "Voice Prompt" },
  {4, 17, SUB,  0, 540, 260, 4, "Tone/Alerts", "GrpCallAlert" },

  {4, 1, TEXT,  21, 540, 110, 4, "Power", "Low" },  
  {4, 2, TEXT,  21, 540, 135, 4, "Power", "Middle" },
  {4, 3, TEXT,  21, 540, 160, 4, "Power", "High" }, 

  {4, 1, TEXT,  22, 540, 110, 4, "Backlight", "Turn Off" },   
  {4, 2, TEXT,  22, 540, 135, 4, "Backlight", "Auto On" },

  {4, 1, TEXT,  23, 540, 110, 4, "Intro screen", "Turn Off" },   
  {4, 2, TEXT,  23, 540, 135, 4, "Intro screen", "Turn On" },   

  {4, 1, TEXT,  24, 540, 110, 4, "Language", "English" },   
  {4, 2, TEXT,  24, 540, 135, 4, "Language", "Sim Chinese" },   
  {4, 3, TEXT,  24, 540, 160, 4, "Language", "Trad Chinese" },   
  {4, 4, TEXT,  24, 540, 185, 4, "Language", "Russian" }, 

  {4, 1, TEXT,  25, 540, 110, 4, "LED indicator", "Turn Off" },   
  {4, 2, TEXT,  25, 540, 135, 4, "LED indicator", "Turn On" },
  
  {4, 1, TEXT,  26, 540, 110, 4, "Encrypt", "Turn Off" },   
  {4, 2, TEXT,  26, 540, 135, 4, "Encrypt", "Turn On" },

  {4, 1, TEXT,  27, 540, 110, 4, "Passwrd Lock", "???" },   
  {4, 2, TEXT,  27, 540, 135, 4, "Passwrd Lock", "???" },

  {4, 1, TEXT,  28, 540, 110, 4, "Display Mode", "CH Mode" },   
  {4, 2, TEXT,  28, 540, 135, 4, "Display Mode", "Freq Mode" }  
  
};

Menu_item Front_Panel_Menu[29] =
{
// menu level, item number, action type, index to submenu, x,y, font, msg1, msg2
  {1, 1, MAIN, 22, 50,  30, 3, "Advanced features", ""},
  {1, 2, MAIN,  8, 50,  70, 3, "Color settings", ""},
  {1, 3, MAIN,  0, 50, 110, 3, "Button names", ""},
  {1, 4, MAIN,  0, 50, 150, 3, "Radio type",""},
  {1, 5, MAIN,  0, 50, 190, 3, "Error log",""},
  {1, 6, MAIN,  0, 50, 230, 3, "Event log",""},
  {1, 7, MAIN,  0, 50, 270, 3, "Diagnostics",""},
  {1, 8, MAIN,  0, 50, 310, 3, "Unknown packet",""},
  
  {2, 1, COLR,   0, 50,  30, 4, "Text Font", ""},
  {2, 2, COLR,   0, 50,  60, 4, "Text Background", ""},
  {2, 3, COLR,   0, 50,  90, 4, "Border", ""},
  {2, 4, COLR,   0, 50, 120, 4, "Highlight", ""},
  {2, 5, COLR,   0, 50, 150, 4, "Shadow", ""},
  {2, 6, COLR,   0, 50, 180, 4, "Greyout", ""},
  {2, 7, COLR,   0, 50, 210, 4, "TouchPoint", ""},
  {2, 8, COLR,   0, 50, 240, 4, "Progress Bar", ""},
  {2, 9, COLR,   0, 50, 270, 4, "Active", ""},
  {2, 10, COLR,  0, 50, 300, 4, "Alert", ""},
  {2, 11, SUB,   0, 50, 330, 4, "Palette", ""},
  {2, 12, SUB,   0, 50, 360, 4, "Default Colors", ""},
  {2, 13, SLID,  0, 380, 100, 4, "Day  ", ""},       //20
  {2, 14, SLID,  0, 380, 150, 4, "Night", ""},       //21
  
  {2, 1, SUB,    0, 50,  30, 3, "Enhanced Monitor Mode", ""},
  {2, 2, SUB,    0, 50,  70, 3, "Enhanced Channel Entry", ""},
  {2, 3, SUB,    0, 50, 110, 3, "Enhanced Scanning Mode", ""},
  {2, 4, SUB,    0, 50, 150, 3, "VFO mode", ""},
  {2, 5, SUB,    0, 50, 190, 3, "Zone Favorites", ""},
  {2, 6, SUB,    0, 50, 230, 3, "Channel Favorites", ""},
  {2, 7, SUB,    0, 50, 270, 3, "Spectrum View", ""},
  
};

// store menu item chosen each level

uint8_t num_menu_levels  = 6;

int menu_chain[6];


// =============================================================================
//
//   display menu
//
//    This menu is for the standard CS800D radio menu.
//    It duplicates the icons in tabs at the top of the screen.
//
// menu_page is just for the top level menu, because it is in the packet.
// action type is determined by the recent packet that is requesting a display
// msg1 and msg2 are what was sent in the packet, n1 and n2 are the length of the strings
//
// History
//  10/15/2018  added revision for 5 tab analog channel
//
// Status: Working. 
//

int  menu_table_index = 0;
uint8_t menu_page = 0;
uint8_t prev_menu_page = 0;
int prev_sub_indx = 0;
int cur_sub_indx = 0;
int cur_top_indx = 0;


void display_menu( uint8_t m_page, uint8_t action_type,
                  char * msg_1, int n1, char * msg_2, int n2 )
{
  int i;
  int nav_key;
 
  if( menu_current_level == 0)
  {
    for( i=0; i < num_menu_levels; i++) menu_chain[i] = 0;
    menu_current_level = 1;
  }
  nav_key = get_nav_key();

  // if up or left,     item --
  // if down or right,  item ++
  // if cancel,         level --
  
  // display based on menu and item numbers
  // starting menu page 
  if((display_mode == 0) || (display_mode == 2))
  {
    // switch based on action type (which packet sent the message)
    switch(action_type)
    {
    case MAIN:
      {
        // switch to menu screen
        if(active_screen_num != 3) display_screen( 3 , 1);
        
        // clear whole tab titles row of previous message
        clear_rectangle( 10 , menu_tab_title_y, 790, 24);
                       
        // sometimes a packet 6 has no data for the menu & the byte is 0x70
        // otherwise use the menupage it sends 
        if( m_page != 0x70)
        {
          // set menu chain for level 1
          menu_chain[1] = m_page;
          menu_current_item = m_page;
          // set text x location, y doesn't change
          menu_tab_title_x = (m_page * 132) + 10;      
          // write tab title
          ssd1963_cg_write_line(n1, 4, menu_tab_title_x , menu_tab_title_y, 4);
          if(m_page != prev_menu_page)
          {
            // draw previous icon grayed out
            draw_menutab( prev_menu_page, 0);    
            // activate current tab
            draw_menutab( m_page, 1); 
            draw_navbuttons( 1 ); 
            // clear the prev page
            clear_menupage( prev_menu_page );
            prev_menu_page = m_page;
            // set line counter to 0 (top of list)
            menu_line_num = 0;
            // clear list data
            for(i = 0; i< lines_per_page; i++)
            {
              Scroll_list[i].has_data = 0;
            }
          }
          // if returning from a submenu
          if(nav_key == CANCEL)
          {
            // clear all sub menus
            clear_menupage( 0 );
          }
          menu_current_level = 1;   // now at this level
        }
        break;
      }
    case SUB:
      {
        // msg 1 has the menu title, msg2 has the menu item
        // on the first level (tabs) menupage has the sub menu
        menu_table_index = get_menu_index( menu_current_level, menu_current_item);
        
        // now we have the index corresponding for menu level and item number
        // sub menus can be on the same menu level, or next level up or down
        // get navigation
        if(nav_key == CANCEL) menu_current_level--; 
        
        if(nav_key == UP) // left or up
        {
          menu_current_item--;
          prev_sub_indx = cur_sub_indx;
          cur_sub_indx--;
          // if up at top, go to last
          if(menu_current_item < 1 )
          {
            cur_sub_indx += submenu_num_items;            
            menu_current_item = submenu_num_items;
          }
          // set menu chain for current level
          menu_chain[menu_current_level] = cur_sub_indx;           
        }
        if(nav_key == DOWN) // right or down
        {
          menu_current_item++;
          prev_sub_indx = cur_sub_indx;          
          cur_sub_indx++;
          // if down at bottom go to top
          if(menu_current_item > submenu_num_items )
          {
            cur_sub_indx -= submenu_num_items;
            menu_current_item = 1;            
          }
          // set menu chain for current level
          menu_chain[menu_current_level] = cur_sub_indx;          
        }
        if(nav_key == SELECT ) menu_current_level++;  // selected

        // if select display a new menu
        if(menu_current_level > 1 && nav_key == SELECT)  
        {
          // clear all sub menus
          clear_menupage( 0 );    
          if(menu_current_level == 2)
          {
            display_submenu( CS800D_Menu[menu_table_index].sub_indx );
            cur_sub_indx = CS800D_Menu[menu_table_index].sub_indx;            
          }
          else 
          {
            display_submenu( CS800D_Menu[cur_sub_indx].sub_indx );
            cur_sub_indx = CS800D_Menu[cur_sub_indx].sub_indx;     
          }
          //
          active_button_mode = 0;  
          draw_navbuttons( 2 );

          prev_sub_indx = cur_sub_indx;
          // set to first item
          menu_current_item = 1;
          // set menu chain for current level
          menu_chain[menu_current_level] = cur_sub_indx;          
          // highlight first item
          draw_frame(CS800D_Menu[cur_sub_indx].pos_x + 7,
                  CS800D_Menu[cur_sub_indx].pos_y + 7, 225, 30, 3, highlight );
          
          break;
        }
        if(menu_current_level == 1 && nav_key == CANCEL)  // cancel
        {
          // it doesn't look like this code ever gets run 
          menu_exit = 0;     // <--- testing here B
          nav_key = NONE;    // 
          display_screen( 1 , 1);  
          break;
        }
        // if current level == 2 cancel is handled by MAIN
        // because packet 6 is sent
        
        // for submenu cancel to previous submenu...
        if(menu_current_level > 1 && nav_key == CANCEL)  // cancel
        {
          // clear all sub menus
          clear_menupage( 0 );          
          // menu level is lower, read previous index
          cur_sub_indx = menu_chain[menu_current_level];
          // get new index to first item in submenu
          cur_top_indx = cur_sub_indx - (CS800D_Menu[cur_sub_indx].item_num - 1);
          
          // redisplay previous
          display_submenu( cur_top_indx );
          // highlight the active item          
          draw_frame(CS800D_Menu[cur_sub_indx].pos_x + 7,
                 CS800D_Menu[cur_sub_indx].pos_y + 7, 225, 30, 3, highlight );              
          
          break;
        }  
        // ===============================
        // not selected and not cancel
        // a direction key, up or down the list  
        
        // menu is displayed, menu list has the data
        // clear previous highlight   
        draw_frame( CS800D_Menu[prev_sub_indx].pos_x + 7,
              CS800D_Menu[prev_sub_indx].pos_y + 7, 225, 30, 3, cg_background);    
        // highlight active menuitem      
        draw_frame( CS800D_Menu[cur_sub_indx].pos_x + 7,
                CS800D_Menu[cur_sub_indx].pos_y + 7, 225, 30, 3, highlight );  
        
        break;
      }
    default: break;
    }
  }    
}
// =============================================================================
//
//  Display a submenu list
// 
// in a sub type msg 1 has the submenu title
// msg2 is the items in the menu

// The index is to the first item in the submenu to be displayed
// the items are numbered sequentialy the comparison to the item number
// will end when a new submenu is encountered


int submenu_num_items = 0;
int submenu_max_lines = 10;

List_text Menu_list[25];
           
void display_submenu( int index )
{
  int i;
  int item = 1;
  int col_width;
  
  // int top_index = index;
  submenu_num_items = 0;
  int submenu_x = CS800D_Menu[index].pos_x;
  int submenu_y = CS800D_Menu[index].pos_y;
  
  // clear the list
  for( i = 0; i < 25; i++) Menu_list[i].has_data = 0;  
  
  // copy menuitems to the array
  while(CS800D_Menu[index].item_num == item)
  {
    strncpy( Menu_list[item].text, CS800D_Menu[index].msg2 , 16);
    Menu_list[item].has_data = 1;    
    submenu_num_items++;
    index++;
    item++;
  }
  if( submenu_num_items > submenu_max_lines )
  {
    col_width = 485; // two column
    wide_list( submenu_num_items + 1, 0, submenu_x, submenu_y,
                 col_width, 25 * (submenu_max_lines + 1) );
  }
  else  // single column
  {
    col_width = 255; // one column
    wide_list( submenu_num_items + 1, 0, submenu_x, submenu_y,
                 col_width, 25 * (submenu_num_items + 1));
  }
}
//
// =============================================================================
//
//   Get index to the current menu/submenu item
//     returns the table index 
//

int get_menu_index( int current_level, int current_item)   
{
  int table_index = 0;
  // using current menu level, move to that section of the table
  while( current_level != CS800D_Menu[table_index].level)
  {
    table_index++;           
    if(table_index > CS800D_table_items) 
    {
      error_log( 3041 );
      break;
    }
  }
  // starting at the index, move to the item in the menu
  while( current_item != CS800D_Menu[table_index].item_num-1)
  {
    table_index++;           
    if(table_index > CS800D_table_items) 
    {
      error_log( 3041 );
      break;
    }
  }
  
  return table_index;
}           
// =============================================================================
//
//  Get menu navigation key
//
//  key assignement per CPS, make changes here 
//
// History 8/31/2018 ver 1.0.2 added knob controls
//
int get_nav_key( void )
{
  int nav_key = NONE;
  // front panel buttons          microphone keys
  if((last_key_press == 0x15) || (last_key_press == 0x0B)) nav_key = CANCEL;
  if((last_key_press == 0x16) || (last_key_press == 0x0C)) nav_key = UP;
  if((last_key_press == 0x17) || (last_key_press == 0x0D)) nav_key = DOWN;
  if((last_key_press == 0x18) || (last_key_press == 0x0E)) nav_key = SELECT;
  // when on a screen that uses the knobs for navigation
  // knob data to radio needs to be deactivated before using this.
  // right knob.
  if((last_key_press == 0x1C) && (key_type == 2)) nav_key = DOWN;
  if((last_key_press == 0x1C) && (key_type == 3)) nav_key = UP; 
  // left knob. works, but only using one for now, could be right, left
  //if(last_key_press == 0x19) nav_key = UP;
  //if(last_key_press == 0x1A) nav_key = DOWN;   

  return nav_key;
}
// =============================================================================
//
// Display a two line text box
// 
// strings in the buffer should be null terminated.
// cg_write_line needs the number of characters, but exits early on a null

uint8_t info_box_active = 0;

uint8_t prev_msg1[16];
uint8_t prev_msg2[16];
 
void display_text_box( uint16_t x_loc, uint16_t y_loc, uint16_t width, uint16_t height)
{
  if(info_box_active == 0)
  {
    // clear
    clear_rectangle( x_loc, y_loc, width, height);    
    // draw frame
    draw_frame(x_loc, y_loc, width, height, 5, highlight );     
    info_box_active = 1;
  }
  else
  {
    // only the line if changed
    if(strncmp((char const * ) prev_msg1,
               (char const * ) msg1_buf, 16)) 
    clear_rectangle( x_loc + 5, y_loc + 5, width - 10, 40);
    
    if(strncmp((char const * ) prev_msg2,
               (char const * ) msg2_buf, 16)) 
    // only clear the text
    clear_rectangle( x_loc + 5, y_loc + 45, width - 10, 40);    
    
  }
  // write data
  strncpy( (char*) cg_line_buffer, (char const*) msg1_buf, 16);
  ssd1963_cg_write_line(16, 3, x_loc + 12, y_loc + 10, 4);
  strncpy( (char*) cg_line_buffer, (char const*) msg2_buf, 16);
  ssd1963_cg_write_line(16, 3, x_loc + 12, y_loc + 50, 4);
  
  // if we're in the menu
  if(active_screen_num == 3)
  {
    menu_current_level++;
  }
  // store for next compare
  strncpy( (char*) prev_msg1, (char const*) msg1_buf, 16);
  strncpy( (char*) prev_msg2, (char const*) msg2_buf, 16);
}
// =============================================================================
//
//  clears the previuos selector box
//
uint16_t prev_sel_x;
uint16_t prev_sel_y;
uint16_t prev_sel_width;
uint16_t prev_sel_height;
uint8_t  prev_sel_border;

void clear_previous_selector( void )
{
      draw_frame(prev_sel_x, prev_sel_y, prev_sel_width, prev_sel_height,
                prev_sel_border, cg_background );  
}
// =============================================================================
//
//  Display the panel controls menu
//
// History  09/28/2018
//
void display_panel_menu( void )
{
  int i;
  // menu items
  for(i = 0; i < 8; i ++)
  {
    strncpy( (char*) cg_line_buffer, Front_Panel_Menu[i].msg1  , 18); 
    ssd1963_cg_write_line(18, 3, Front_Panel_Menu[i].pos_x,
                          Front_Panel_Menu[i].pos_y, 3);
  }
  // set select box variables
  prev_sel_x = Front_Panel_Menu[0].pos_x - 10;
  prev_sel_y = Front_Panel_Menu[0].pos_y - 5;
  select_box_width = 315;
  prev_sel_width = select_box_width;
  select_box_height = 48;
  prev_sel_height = select_box_height;
  prev_sel_border = 5;
  
  // draw selection frame
  draw_frame(Front_Panel_Menu[0].pos_x - 10, Front_Panel_Menu[0].pos_y - 5,
               315, 48, 5, highlight ); 
}
// =============================================================================
//
//
void process_nopage( uint8_t np )
{
  // do nothing. To prevent possible null pointer
}
// =============================================================================
// 06/24/2019
//
//  Menu handling stack
//
//  Routines to do menu page tracking
//
//  push_menu_stack                 Push menu page number onto stack
//  pop_menu_stack                  Pop menu page number off stack
//  get_menu_stack                  Get menu page number at top of stack
//  reset_menu_stack                Reset stack
//
//
uint8_t menu_stack[ 8 ];  // 8 levels max, only using 5
uint8_t menu_stack_pointer = 0;

// =============================================================================
//  Push
//
void push_menu_stack( uint8_t page_num )
{
  menu_stack_pointer++;
  if( menu_stack_pointer > 7 ) 
  {
    menu_stack_pointer = 7;
    error_log( 3500 );
  }
  menu_stack[ menu_stack_pointer ] = page_num; 
}
// =============================================================================
//  Pop
//
void pop_menu_stack( void )
{
  menu_stack[ menu_stack_pointer ] = 0;   
  if ( menu_stack_pointer > 0 ) menu_stack_pointer--;
}

// =============================================================================
// Get top of stack
//
uint8_t get_menu_stack( void )
{
  return menu_stack[ menu_stack_pointer ];
}
// =============================================================================
// Reset stack
//  clear all and set pointer to 0
//  
void reset_menu_stack( void )
{
  int i;
  for( i = 0 ; i < 8; i++) menu_stack[ i ] = 0;
  menu_stack_pointer = 0;
}

// =============================================================================
//
//  Process panel menu
//
//  handle controls of the front panel menu
//  run in main loop.
//
// Status: written, needs revision. Too big, make subroutines
//
// History  10/01/2018

int pm_cur_menu_item = 0;
int last_sub_idx = 0;
int first_sub_idx = 0;
int sub_item_num = 0;
uint8_t colr_idx = 0;
int pal_num = 0;

uint8_t in_palette_adjust = 0;
uint8_t in_rgb_adjust = 0;
uint8_t colors_changed = 0;

uint8_t menu_exit = 0;  // flag to exit the menu from sub menu pages

// advanced menu processes table
// menu number, menu process function
struct adv_menu {
    uint8_t page_num;               // menu page number (use menu_page enums)
    void (*mpage_proc)( uint8_t );     // function for the menu page
  };  
struct adv_menu adv_menu_proc[13] = 
{
  {NOPAGE,  process_nopage },   // a do nothing function
  {VFOMODE, process_vfo_mode },
  {ENHMON,  process_EMon_mode },
  {ECHAN,   process_e_chan_mode },
  {ENHSCAN, process_enhscan_mode },
  {CHANFAV, process_channel_favs },
  {ZONEFAV, process_zone_favs },
  {SPECTV,  process_spectrum_view },
  {RADSEL,  process_radio_select },
  {ERRLIST, process_e_list },
  {EVTLIST, process_e_list },
  {COLRSEL, process_color_selector },
  {QBOX,    process_question_box } 
};
  
uint8_t adv_menu_page = 0;

extern uint8_t current_radio_type;
RGB_color fr_color;

void process_panel_menu( void )
{
  int nav_key;
  fr_color.red   = 200;
  fr_color.green = 255;
  fr_color.blue  = 255;
  
  // if active screen = 3 the radio menu is on.
  // this routine gets called to exit
  if(active_screen_num == 3 ) 
  {
    if( menu_exit )    
    {
      pm_cur_menu_item = 0;
      display_mode = 0;
      menu_exit = 0;
      display_screen( 1 , 1 ); 
      return;
    }
  }
    
  if(active_screen_num != 6 ) return;
  
  // menu exit from sub menu pages
  if( menu_exit )
  {
    // cancel
    pm_cur_menu_item = 0;
    display_mode = 0;
    switch( current_radio_type )
    {
    case 0:
      {
        display_screen( 1 , 1 );
        break;
      }
#ifdef radio_FT857D      
    case 1:
      {
        FT857_display_screen( 1, 1 );
        active_screen_num = 0;
        last_key_press = 0; 
        special_encoder_flag ^= 1;  // undo duplicate toggle
        break;
      }
    case 2:
      {
        FT857_display_screen( 1, 1 );
        active_screen_num = 0;       
        last_key_press = 0;
        special_encoder_flag ^= 1; // undo duplicate toggle
        break;
      }
#endif /* FT857D */      
    default: break;
    }
    menu_exit = 0;
    return;
  }
  
  nav_key = get_nav_key();  
  adv_menu_page = get_menu_stack();
  //  run the menu page process
  if( adv_menu_page > 0 )
  {
    adv_menu_proc[ adv_menu_page ].mpage_proc( nav_key );
    last_key_press = 0;
    return;      
  }

  // only handle the rest if there's a key
  if(nav_key == NONE && dtmf_code_flag == 0) return;

  // first menu level
  if(Front_Panel_Menu[pm_cur_menu_item].level == 1)
  {
    process_menu_level( nav_key );
    last_key_press = 0;    
    return;    
  }

  // if in menu level 2
  if(Front_Panel_Menu[pm_cur_menu_item].level == 2)
  {
    process_submenu_level2( nav_key );  
  }
  // done handling clear the lastkey
  last_key_press = 0;
}

// =============================================================================
//
//  display error list
//    displays error message of the last 32 errors in the log
//

void display_error_list( void )
{
  int i;
  int err_count = 0;
  int erl_idx;
  uint16_t pos_x = 10;
  uint16_t pos_y = 10;
    
  if(error_count > 0)
  {
    // start list 
    erl_idx = current_error - 32;
    if( erl_idx < 0 ) erl_idx = erl_idx + 100;
    
    for( i = 0; i < 32; i++)
    {
      if(err_count == 16)
      {
        pos_x = 400;
        pos_y = 10;
      }
      
      if(error_record[erl_idx] > 0)
      {
        strncpy( (char*)cg_line_buffer, get_error_text( error_record[erl_idx]), 20); 
        // send to sceen
        ssd1963_cg_write_line( 20, 4, pos_x , pos_y, 4);
        pos_y += 26;
        err_count++;
      } 
      erl_idx++;
      if( erl_idx > 99 ) erl_idx = 0;
    }
  }
  else
  {
    strncpy( (char*)cg_line_buffer, "No errors\n" , 9);
    ssd1963_cg_write_line( 9, 3, 10 , 10, 4);    
  }
}

// =============================================================================
//
//  display event list
//    displays event message of the last 32 events 
//

void display_event_list( void )
{
  int i;
  int ev_count = 0;
  int ev_idx;
  uint16_t pos_x = 10;
  uint16_t pos_y = 10;

  // start list 
  ev_idx = event_log_indx - 32;
  if( ev_idx < 0 ) ev_idx = ev_idx + 100;
  
  for( i = 0; i < 32; i++)
  {
    if(ev_count == 16)
    {
      pos_x = 400;
      pos_y = 10;
    }
    
    if(strlen(get_event( ev_idx )) > 0)
    {
      strncpy( (char*)cg_line_buffer, get_event( ev_idx ), 20); 
      // send to sceen
      ssd1963_cg_write_line( 20, 4, pos_x , pos_y, 4);
      pos_y += 26;
      ev_count++;
    } 
    ev_idx++;
    if( ev_idx > 99 ) ev_idx = 0;
  }
}

// =============================================================================
//
//  Pop up options box
//   for: a question expecting a yes or no answer
//   Buttons are for yes, no
// 
//  Todo: make generic for:
//        ok, cancel
//        save, cancel 
//

void question_box( uint16_t pos_x, uint16_t pos_y, uint16_t width,
               uint16_t height, char* question, uint8_t butn1, uint8_t butb2 )
{
  // draw frame
  draw_frame( pos_x, pos_y, width, height, 5,
             current_palette[get_pal_color( 2 )] );

  // display question
  strncpy( (char*) cg_line_buffer, (char const*) question, 16);
  ssd1963_cg_write_line(16, 3, pos_x + 15 , pos_y + 15, 4);  

  // clear button area
  clear_rectangle(  340 , 420, 410 , 50);

  draw_button( 9, get_pal_color( 2 ), get_pal_color( 3 ) );
  draw_button( 10, get_pal_color( 2 ), get_pal_color( 3 ) );
  
  active_button_mode = 0;
  push_menu_stack( QBOX );
}
// =============================================================================
//
//  options box for color settings save, cancel
//
//
void process_question_box( uint8_t nav_key )
{
  switch(nav_key)
  {
  case SELECT:       // yes, save
    {
      // if color choices are ok
      if(color_check( current_palette ) == 1)
      {
        // set I2C to eeprom
        i2c2_device_active = I2C_eprom; 
        // save to eeprom
        save_colorset( );
        log_event( "Colors saved"); 
        // switch back to touch panel      
        i2c2_device_active = I2C_touch;
        pop_menu_stack();
      }
      // else display bad color message
      else
      {
        error_log( 4010 );
        // change message in question box.
        clear_rectangle(  450 , 100, 300 , 100);        
        question_box( 450, 100, 300, 100, "Bad Colors", 9, 10 );
        strncpy( (char*) cg_line_buffer, "Text = Background", 17);
        ssd1963_cg_write_line(17, 4, 460 , 150, 2);
        // clear button area
        clear_rectangle(  340 , 420, 410 , 50);        
        draw_cancel_button();
        
      }
      break;
    }
  case CANCEL: // no don't save
    {
      pop_menu_stack();     
      break;
    }
  default:break;  
  }
  // clear question box
  //  if navkey was not SELECT or CANCEL doesn't clear
  if( get_menu_stack() != QBOX)
  {
    // clear buttons
    clear_rectangle(  340 , 420, 410 , 50);
    // clear popup
    clear_rectangle(  450 , 100, 300 , 100);
    // redisplay nav  buttons
    draw_navbuttons( 2 );
  }
}
// =============================================================================
//
//  Reset panel menus
//    prevents menu from being left in some active state,
//    when the knob switch changes back to the main screen
//    always restarts at the beginning.
//
void reset_panel_menu( void ) 
{
  pm_cur_menu_item = 0;
  last_sub_idx = 0;
  first_sub_idx = 0;
  colr_idx = 0;
  pal_num = 0;  
  //in_color_selector = 0;
  in_palette_adjust = 0;
  in_rgb_adjust = 0;
  //in_error_list = 0;
  //in_event_list = 0; 
  //in_question_box = 0;
  reset_menu_stack();
  
  // clear state variables, so menus restart properly
  zf_state = 0;
  cf_state = 0;
  vfo_state = 0;
  e_chan_state = 0;
  chansc_state = 0;
  gsd_state = 0;
  sv_state = 0;
  enhscan_state = 0;

  //in_radio_select = 0;
  //in_e_chan_mode = 0;  
  //in_vfo_mode = 0;
  //in_chan_fav = 0;
  //in_zone_fav = 0;  
  menu_exit = 0;  
}

// =============================================================================
//
//  Pop up message box
//    single message pop up
//    20 charcter max
//
void message_box( uint16_t pos_x, uint16_t pos_y,
                 uint16_t width, uint16_t height, char* message )
{
  int font;
  if(strlen( message ) > 12) font = 4;
  else font = 3;
  // draw frame
  draw_frame( pos_x, pos_y, width, height, 5,
             current_palette[get_pal_color( 2 )] );

  // display message
  strncpy( (char*) cg_line_buffer, (char const*) message, 20);
  ssd1963_cg_write_line(20, font, pos_x + 15 , pos_y + 15, 4); 
  
}

// =============================================================================
//
//  displays a sub menu starting at the index location
//
//   use for both first display and return from higer level menu
//   select num is which item to highlight, 0 for first display,
//   whichever the selection was for returning to this menu.
//   box_width is the width of the selection box

// 01/24/2019 added font so that submenus could be displayed with different fonts

void display_sub_menu( int sub_num, uint8_t select_num, uint16_t box_width, uint8_t font )
{
  int fp_idx;
  int font_height;
  // get height of a selector box for the font
  switch(font)
  {
  case 0: font_height = 116; break;
  case 1: font_height = 80; break;
  case 2: font_height = 65; break;
  case 3: font_height = 48; break;
  case 4: font_height = 34; break;
  }
  
  // display sub menu
  fp_idx = Front_Panel_Menu[ sub_num ].sub_indx;
  first_sub_idx = fp_idx;
  sub_item_num = 0;   // set to first
  // display sub menu
  while(Front_Panel_Menu[fp_idx].item_num == sub_item_num + 1)
  {
    sub_item_num = Front_Panel_Menu[fp_idx].item_num;
    
    strncpy( (char*) cg_line_buffer, Front_Panel_Menu[fp_idx].msg1  , 24);
    ssd1963_cg_write_line(24, font, Front_Panel_Menu[fp_idx].pos_x,
                          Front_Panel_Menu[fp_idx].pos_y, 4);
    fp_idx++;
  }
  last_sub_idx = fp_idx -1;
  //   // reset index to the item selected
  fp_idx = first_sub_idx + select_num;  
  // set current 
  pm_cur_menu_item = fp_idx;
  
  prev_sel_x = Front_Panel_Menu[fp_idx].pos_x - 10;
  prev_sel_y = Front_Panel_Menu[fp_idx].pos_y - 5;
  prev_sel_width = box_width;
  prev_sel_height = font_height;
  prev_sel_border = 3;
  // draw new selector frame in the sub menu
  draw_frame(Front_Panel_Menu[fp_idx].pos_x - 10,
             Front_Panel_Menu[fp_idx].pos_y - 5,
             box_width, font_height, 3, highlight );
}
// =============================================================================
//
//  display list of available radios
//
//  full screen, 2 radios only for now
//
// Programmer Dean S. Graham
//
// History: 02/20/2019 written for switching to FT857
//
// Status: writing

// structure for radio types
typedef struct
{
  uint8_t radio_num;
  char    name[17];
} radio_type_info;

uint8_t num_radio_types = 1;

radio_type_info radio_type[3] = {
{ 0, "CS800D UART1"},
{ 1, "FT857D UART1"},
{ 2, "FT857D UART2"},
};

void display_radio_choices( uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  int i;
  int column = 0;
  int row = 0;
  
  #ifdef radio_FT857D
  num_radio_types = 3;
  #endif /* FT857D */
  
  for(i = 0; i < num_radio_types; i++ )
  {
    strncpy( (char *) cg_line_buffer, radio_type[i].name, 17);
    ssd1963_cg_write_line(17, font, pos_x + column , pos_y + row * 40, 3);
    row++;
    /* not needed til there more than 7 radios to choose
    if( i == 7)
    {
      row = 0;
      column = 400;
    }
    */
  }
}

// =============================================================================
// 02/20/2019
//
//     Process radio selection page
//
//

uint8_t rtyp_state = 0; // radio type state
int rtyp_num = 0;       // radio number on page, signed int so < 0 works

uint16_t rtyp_x_pos = 30;  // starting corner of list
uint16_t rtyp_y_pos = 80;

uint8_t radio_select_changed = 0;

extern uint8_t current_radio_type;

extern void __cmain( void );

void process_radio_select( uint8_t nav_key )
{
  int i;
  static int row;
  static int column;
  
  // uint8_t prev_dtmf = 0;
   
  switch( rtyp_state )
  {
  case 0:   // display page
    {
      // clear
      ssd1963_cg_clear_display();
      active_button_mode = 0;   // force redraw

      // display title
      strncpy( (char*) cg_line_buffer, "Select Active Radio", 19);
      ssd1963_cg_write_line(19, 2, 225 ,20, 4);
      // display radio choices
      display_radio_choices( rtyp_x_pos, rtyp_y_pos, 3 );
      column = 0;
      row = 0;
      rtyp_num = 0;
      select_box_width = 325;        
      // draw selector box at first
      draw_frame(rtyp_x_pos + column - 10, rtyp_y_pos + ( row * 40) - 5,
                 select_box_width, 45, 3, highlight );      
  
      //draw buttons (up down)
      draw_navbuttons( 2 );
      last_key_press = 0;
      dtmf_code_flag = 0;

      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4
      
      // activate touch points, the same as a favorites list 
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 1;           
    
      push_menu_stack( RADSEL );
      rtyp_state++;
      break;
    }
  case 1:  // navigate list
    {
      // navigate
      nav_key = get_nav_key();
      if(nav_key == NONE) break;
      last_key_press = 0;        
      switch( nav_key )
      {
      case UP:
        {
          // erase previous selector
          draw_frame(rtyp_x_pos + column - 10, rtyp_y_pos + ( row * 40) - 5,
                         select_box_width, 45, 3, cg_background);
          rtyp_num--;
          if( rtyp_num < 0 )rtyp_num = num_radio_types - 1;
          if( rtyp_num < 8 )
          {
            column = 0;
            row = rtyp_num;
          }
          else
          {
            column = 400;
            row = rtyp_num - 8;            
          }
          // draw new selector box
          draw_frame(rtyp_x_pos + column - 10, rtyp_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight );          
          
          break;
        }
      case DOWN:
        {
          // erase previous selector
          draw_frame(rtyp_x_pos + column - 10, rtyp_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, cg_background);
          
          rtyp_num++;
          if( rtyp_num > num_radio_types - 1 )rtyp_num = 0;
          if( rtyp_num < 8 )
          {
            column = 0;
            row = rtyp_num;
          }
          else
          {
            column = 400;
            row = rtyp_num - 8;            
          }          
          // draw new selector box
          draw_frame(rtyp_x_pos + column - 10, rtyp_y_pos + ( row * 40) - 5,
                     select_box_width, 45, 3, highlight );          
          break;
        }      
      case SELECT:
        {
          // set variable
          current_radio_type = rtyp_num;
          // store in eeprom
          save_radio_type( current_radio_type );
          // event log is cleared on radio change, message never shows
          // log_event( "Change radio type" );
          msec_delay(30);
          pop_menu_stack();
          // full reset, restart with new radio type
          //__iar_program_start();
          __cmain();
          return; // this never gets used, the call to cmain resets the stack
        }
      case CANCEL:
        {
          rtyp_state++;  // don't change radio type, jump to exit
          break;
        }        
      default: break;
      }
      break;
    }
 case 2:   // return to previous menu
    {
      // turn off touch points, turned back on by radio screen
      for(i = 0; i < 16; i++) CS800D_touch_point[18 + i].is_active = 0;  
      // clear the page
      clear_rectangle( 0,0, 800, 410);
      pop_menu_stack();
      rtyp_state = 0;
      menu_exit = 1;      
      return;
    }
  default:
    {
      error_log( 3001 );
      break;
    }
  } // end of state switch
}
// =============================================================================
//
//  Handle day and night brightness sliders
//
uint8_t in_brightness_adjust = 0;

void process_brightness_adjust ( uint8_t nav_key )
{
  switch( nav_key )
  {
  case UP:
    {
      if( pm_cur_menu_item == 20 )
      {
        if( day_bright_val < 201 ) day_bright_val++;
        // set brightness control
        ssd1963_set_brightness( day_bright_val );
        // redisplay slider
        show_brightness_control( 0 ); 
      }
      if( pm_cur_menu_item == 21 )
      {
        if( night_bright_val < 201 ) night_bright_val++;
        // set brightness control
        ssd1963_set_brightness( night_bright_val );
        // redisplay slider
        show_brightness_control( 1 );
      }     
      colors_changed = 1;
      break;
    }
  case DOWN:
    {
      if( pm_cur_menu_item == 20 )
      {
        if( day_bright_val > 0 ) day_bright_val--;
        // set brightness control
        ssd1963_set_brightness( day_bright_val );
        // redisplay slider
        show_brightness_control( 0 ); 
      }
      if( pm_cur_menu_item == 21 )
      {
        if( night_bright_val > 0 ) night_bright_val--;
        // set brightness control
        ssd1963_set_brightness( night_bright_val );
        // redisplay slider
        show_brightness_control( 1 );
      }
      colors_changed = 1;
      break;
    }    
  case SELECT:
    {
      if( pm_cur_menu_item == 20 )
      {
        // set brightness control
        ssd1963_set_brightness( day_bright_val );
      }
      if( pm_cur_menu_item == 21 )
      {
        // set brightness control
        ssd1963_set_brightness( night_bright_val );
      }        
      break;
    }
  case CANCEL:
    {
      // deselect
      in_brightness_adjust = 0;
      pop_menu_stack();
      break;
    }
  default: break;
  }
}

// =============================================================================
//
//   Handle color selector screen
//

void process_color_selector( uint8_t nav_key )
{
  int i;
  
  if( in_brightness_adjust )
  {
    process_brightness_adjust( nav_key );
    return;
  }
 
  switch(nav_key)
  {
  case UP:   // left
    {
      if( pal_num > 0) pal_num--;
      else pal_num = 15;
      // repaint palette with new selected frame 
      show_palette( 50, 350, current_palette );
      show_pal_selector_frame( 50, 350, current_palette, pal_num );        
      break;
    }
  case DOWN:  // right
    {
      pal_num++;
      if(pal_num > 15) pal_num = 0;
      // repaint palette with new selected frame 
      show_palette( 50, 350, current_palette );        
      show_pal_selector_frame( 50, 350, current_palette, pal_num );          
      break;
    }
  case SELECT:
    {
      if(in_palette_adjust)
      {
        // handle rgb color change
        if(in_rgb_adjust)
        {
          // store color back in palette
          RGB_copy( &current_palette[pal_num], &adj_color );
          // redisplay
          show_palette( 50, 350, current_palette);
          break;
        }
        // display RGB adjustment
        // turn on the tp's for color adjust
        for( i=12; i< 18; i++) CS800D_touch_point[i].is_active = 1;
        
        // clear brightness adjust 
        clear_rectangle( 350, 80, 400, 120 );
        
        show_palette( 50, 350, current_palette);
        display_color( 350, 90, current_palette[pal_num] );
        
        // set variables for adjusting
        RGB_copy( &adj_color, &current_palette[pal_num] );          
        
        color_adj_x = 350;
        color_adj_y = 90;
        in_rgb_adjust = 1;          
      }
      // handle brightness controls

      else
      {
        // handle palette color change
        log_event( "Colors changed"); 
        set_pal_color( Front_Panel_Menu[pm_cur_menu_item].item_num-1, pal_num );
        // draw color frame in menu
        draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x + 248,
                   Front_Panel_Menu[pm_cur_menu_item].pos_y - 2,
                   34, 34, 2, fr_color); 
              
        // draw color box
        display_color_block( Front_Panel_Menu[pm_cur_menu_item].pos_x + 250,
                             Front_Panel_Menu[pm_cur_menu_item].pos_y,
                             Front_Panel_Menu[pm_cur_menu_item].item_num, 30, 30);        
      }
      break;
    }      
  case CANCEL:
    {
      if(in_palette_adjust)
      {
        if(in_rgb_adjust)
        {
          // clear color adjust area
          clear_rectangle( 350, 90, 200, 270); 
          in_rgb_adjust = 0;   
          // turn off the tp's for color adjust
          for( i=12; i< 18; i++) CS800D_touch_point[i].is_active = 0;
          // redisplay
          show_palette( 50, 350, current_palette);
          // reshow the menu
          
          break;
        }  
        in_palette_adjust = 0;
        break;
      }
      pop_menu_stack();
      // clear palette block
      clear_rectangle( 50, 350, 650, 70);
      draw_navbuttons( 2 );
      // repaint menu
      display_color_menupage();
      break;
    }      
  default: break;
  }
}
// =============================================================================
//
//  Display color selector menu page
//
//  added as a separate routine because it's needed when
//  palette display is cleaed and the menu needs to be re-painted

void display_color_menupage( void )
{
  int fp_idx = 0;  
  // clear previous selector frame
  clear_previous_selector();          

  select_box_width =  315;
  // special box size for brightness controls
  if( pm_cur_menu_item == 20 || pm_cur_menu_item == 21 )
  {
    select_box_width = 325;
    select_box_height = 42;
  } 

  // display sub menu
  fp_idx = Front_Panel_Menu[1].sub_indx;
  first_sub_idx = fp_idx;
  sub_item_num = 0;   // set to first
  // sub menu
  while(Front_Panel_Menu[fp_idx].item_num == sub_item_num + 1)
  {
    sub_item_num = Front_Panel_Menu[fp_idx].item_num;
    
    strncpy( (char*) cg_line_buffer, Front_Panel_Menu[fp_idx].msg1  , 18);
    ssd1963_cg_write_line(18, 4, Front_Panel_Menu[fp_idx].pos_x,
                          Front_Panel_Menu[fp_idx].pos_y, 4);
    
    if(Front_Panel_Menu[fp_idx].action_type == COLR)
    {
      // draw color frame
      draw_frame(Front_Panel_Menu[fp_idx].pos_x + 248,
                 Front_Panel_Menu[fp_idx].pos_y - 2,
                 34, 34, 2, fr_color); 
      
      // draw color box
      display_color_block( Front_Panel_Menu[fp_idx].pos_x + 250,
                           Front_Panel_Menu[fp_idx].pos_y,
                           Front_Panel_Menu[fp_idx].item_num, 30, 30);
    }
    if(Front_Panel_Menu[fp_idx].action_type == SLID)
    {
      //get stored from eeprom
      //day_bright_val = 50;
      //night_bright_val = 50;            
      // index is 20 for day and 21 for night, control is 0 or 1
      show_brightness_control( fp_idx - 20 );
    }
    fp_idx++;
  }
  last_sub_idx = fp_idx -1;
  // reset index to first
  fp_idx = first_sub_idx;
  // set current 
  pm_cur_menu_item = fp_idx;
  select_box_width = 240;
  select_box_height = 34;
  // set box variables
  prev_sel_x = Front_Panel_Menu[fp_idx].pos_x - 10;
  prev_sel_y = Front_Panel_Menu[fp_idx].pos_y - 5;
  prev_sel_width = select_box_width;
  prev_sel_height = select_box_height;
  prev_sel_border = 3;
  // draw new selector frame in the sub menu
  draw_frame(Front_Panel_Menu[fp_idx].pos_x - 10,
             Front_Panel_Menu[fp_idx].pos_y - 5,
             select_box_width, select_box_height, 3, highlight );
  last_key_press = 0;
}

// =============================================================================
//
//   Handle the first menu level 
//

void process_menu_level( uint8_t nav_key )
{
  // set select box variables
  prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
  prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
  prev_sel_width = select_box_width;
  prev_sel_height = select_box_height; 
  prev_sel_border = 5;
  switch(nav_key)
  {
  case UP:
    {
    // clear selector frame
    clear_previous_selector();      
    
    pm_cur_menu_item--;
    if( pm_cur_menu_item < 0)
    {
      pm_cur_menu_item = 7;
    }
    // set select box variables
    prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
    prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
    prev_sel_width = 315;
    prev_sel_height = 48;
    prev_sel_border = 5;
    // draw new selector frame
    draw_frame(prev_sel_x, prev_sel_y, prev_sel_width, prev_sel_height, 
               5, highlight );
    break;
    }
  case DOWN:
    {
    // clear selector frame
    clear_previous_selector();   
    
    pm_cur_menu_item++;
    if( pm_cur_menu_item > 7)
    {
      pm_cur_menu_item = 0;
    }
    // set select box variables
    prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
    prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
    prev_sel_width = 315;
    prev_sel_height = 48;
    prev_sel_border = 5;
    // draw new selector frame
    draw_frame(prev_sel_x, prev_sel_y, prev_sel_width, prev_sel_height, 
               5, highlight );     
     break;
    }
  case SELECT:
    {
      // clear main menu
      clear_rectangle( 40, 20, 315, 360);
      // set select box variables
      prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
      prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
      prev_sel_width = select_box_width;
      prev_sel_height = select_box_height;
      prev_sel_border = 5;
      switch( pm_cur_menu_item )
      {
      case 0: // advanced features
        {
          if(current_radio_type == 1 || current_radio_type == 2)  // FT857D
          {
            // display not available
            display_message("This feature is not available yet", 150, 350, 3);
            last_key_press = 0;
            // redraw main menu
            display_panel_menu();
            active_button_mode = 0;  
            draw_navbuttons( 2 );
            
            // back to main menu 
            pm_cur_menu_item = 0;
            // redraw selector
            draw_frame(Front_Panel_Menu[0].pos_x - 10,
                 Front_Panel_Menu[0].pos_y - 5, 315, 48, 5, highlight );                
          }
          if(current_radio_type == 0)  // CS800D
          {
            // clear selector frame
            draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                       Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                       select_box_width, 48, 5, cg_background); 
            
            select_box_width = 460;
            prev_sel_height = 48;
            prev_sel_border = 5;
            prev_sel_width = select_box_width;
            last_key_press = 0;
            // display sub menu
            display_sub_menu( 0, 0, 460, 3 );
          } 
          return;    // don't move to next level till a new key            
        }
      case 1: // color settings
        {
          display_color_menupage();          
          return;    // don't move to next level till a new key
        }
      case 2: // button names
        {
          // display not available
          display_message("This feature is not available yet", 150, 350, 3);              
          last_key_press = 0;
          // redraw main menu
          display_panel_menu();
          active_button_mode = 0;  
          draw_navbuttons( 2 );
          
          // back to main menu 
          pm_cur_menu_item = 0;
          // redraw selector
          draw_frame(Front_Panel_Menu[2].pos_x - 10,
               Front_Panel_Menu[2].pos_y - 5, 315, 48, 5, highlight );            
          
          return;
          break;
        }
      case 3: // radio type
        {
          // display radio selection page
          process_radio_select( 0 );
          push_menu_stack( RADSEL );
          break;
        }
      case 4: // error log
        {
          // clear buttons
          clear_rectangle(  350 , 420, 400 , 50);
          // only draw the cancel icon
          draw_cancel_button();
          display_error_list();
          push_menu_stack( ERRLIST );
          break;
        }
      case 5: // event log
        {
          // clear buttons
          clear_rectangle(  350 , 420, 400 , 50);
          // only draw the cancel icon
          draw_cancel_button();
          display_event_list();
          push_menu_stack( EVTLIST );
          break;
        }
      case 6: // diagnostics
        {
          display_screen( 2 , 1);
          process_board_tests();
          msec_delay(2000);
          // restore keys
          activate_all_keypress();
          // redraw panel menu
          display_screen( 6 , 1);
          select_box_width = 315;
          // clear selector frame
          draw_frame(Front_Panel_Menu[0].pos_x - 10,
                     Front_Panel_Menu[0].pos_y - 5,
                     select_box_width, 48, 5, cg_background);          
          active_button_mode = 0;  
          draw_navbuttons( 2 );
        
          pm_cur_menu_item = 5;
          // redraw selector
          draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                     Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                     select_box_width, 48, 5, highlight );
          
          return;   // return, not break       
        }  
      case 7: // unknown packet
        {
          display_packet_dump( unknown_packet_error_buffer );
          return; // return, not break
        }           
      default: break;
      }    // end of menu item case
    break;
    }       // end of Select case
  case CANCEL:
    {
      // cancel
      pm_cur_menu_item = 0;
      display_mode = 0;
      menu_exit = 1;

    break;
    }
  default: break;
  }  
}

// =============================================================================
//
//   Handle second submenu level 
//
//
void process_submenu_level2( uint8_t nav_key )
{
  if (Front_Panel_Menu[pm_cur_menu_item].font == 3 ) select_box_height = 48;
  if (Front_Panel_Menu[pm_cur_menu_item].font == 4 ) select_box_height = 34;
 
  switch(nav_key)
  {
  case UP:
    {
      // clear previous selector frame
      clear_previous_selector();
      pm_cur_menu_item--;
      if( pm_cur_menu_item < first_sub_idx)
      {
        pm_cur_menu_item = last_sub_idx;
      }
      if( pm_cur_menu_item > 7 && pm_cur_menu_item < 22 ) select_box_width = 240;
      // special box size for brightness controls
      if( pm_cur_menu_item == 20 || pm_cur_menu_item == 21 )
      {
        select_box_width = 325;
        select_box_height = 42;
      }      
      // set box variables
      prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
      prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
      prev_sel_width = select_box_width;
      prev_sel_height = select_box_height;
      prev_sel_border = 3;
      // draw new selector frame
      draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                 Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                 select_box_width, select_box_height, 3, highlight );
      
      break;
    }
  case DOWN:
    {
      // down
      // clear selector frame
      clear_previous_selector();      
     
      pm_cur_menu_item++;
      if( pm_cur_menu_item > last_sub_idx)
      {
        pm_cur_menu_item = first_sub_idx;
      }
      if( pm_cur_menu_item > 7 && pm_cur_menu_item < 22 ) select_box_width = 240;
      // special box size for brightness controls
      if( pm_cur_menu_item == 20 || pm_cur_menu_item == 21 )
      {
        select_box_width = 325;
        select_box_height = 42;
      }
      prev_sel_x = Front_Panel_Menu[pm_cur_menu_item].pos_x - 10;
      prev_sel_y = Front_Panel_Menu[pm_cur_menu_item].pos_y - 5;
      prev_sel_width = select_box_width;
      prev_sel_height = select_box_height;
      prev_sel_border = 3;
      
      // draw new selector frame
      draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                 Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                 select_box_width, select_box_height, 3, highlight );        
      break;
    }    
  case SELECT:
    {
      if(Front_Panel_Menu[pm_cur_menu_item].action_type == COLR)
      {
        if( pm_cur_menu_item < 18 )
        {
          // set selector to current color
          colr_idx = Front_Panel_Menu[pm_cur_menu_item].item_num - 1;
          pal_num = get_pal_color( colr_idx );
          // clear palette space
          clear_rectangle( 40, 330, 600, 100);           
          // show palette selector
          show_pal_selector( 50, 350, current_palette, colr_idx );
        }
      
        draw_navbuttons( 1 );
        colors_changed = 1;
      }
      if(Front_Panel_Menu[pm_cur_menu_item].action_type == SUB)
      {
        switch( pm_cur_menu_item )
        {
        case 18:  // palette adjust
          {
            // clear selector frame
            draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                       Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                       select_box_width, 34, 3, cg_background);
            
            // start at 0
            pal_num = 0;
            // clear palette space
            clear_rectangle( 40, 330, 600, 100);         
            show_palette( 50, 350, current_palette );
            show_pal_selector_frame( 50, 350, current_palette, 0 );
            in_palette_adjust = 1;
            active_button_mode = 0;  
            draw_navbuttons( 1 );
            colors_changed = 1;
            break;
          }
        case 19:  // restore defaults
          {
            // clear selector frame
            draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
                       Front_Panel_Menu[pm_cur_menu_item].pos_y - 5,
                       select_box_width, 34, 3, cg_background);
            
            // add popup message "defaults restored"
            message_box( 450, 220, 300, 100, "defaults restored" );            
            // load default
            load_palette( (RGB_color*) default_palette );
            restore_default_colors();
            log_event( "Colors reset");
            colors_changed = 1;  
            break;
          }
        case 22: // Enhanced Monitor Mode
          {
             process_EMon_mode( 0 );
             break;
          }
        case 23: // Enhanced Channel Entry
          {
            process_e_chan_mode( 0 );
            break;
          }
        case 24: // Enhanced Scanning Mode
          {
            process_enhscan_mode( 0 );
            break;
          }
        case 25: // VFO mode
          {
            process_vfo_mode( 0 );
            break;
          }
        case 26: // Zone Favorites
          {
            process_zone_favs( 0 );
            break;
          }
        case 27: // Channel favorites
          {
            process_channel_favs( 0 );
            break;
          }
        case 28: // Spectrum view
          {
            process_spectrum_view( 0 );

            break;
          } 

        default: break;
        }
      }
      if(Front_Panel_Menu[pm_cur_menu_item].action_type == SLID)
      {
        switch( pm_cur_menu_item )
        {
        case 20:  // day brightness adjust
          {
            // highlight selected box
            draw_frame( prev_sel_x, prev_sel_y, prev_sel_width, prev_sel_height,
                       prev_sel_border, current_palette[ 4 ] );
         
            in_brightness_adjust = 1;
            break;
          }
        case 21:  // night brightness adjust  
          {
            // highlight selected box
            draw_frame( prev_sel_x, prev_sel_y, prev_sel_width, prev_sel_height,
                       prev_sel_border, current_palette[ 4 ] );            
            in_brightness_adjust = 1;
            break;
          }
        default: break;
        }
        
      }
      if((pm_cur_menu_item > 7) && (pm_cur_menu_item < 22))
      {
        if( get_menu_stack() != COLRSEL ) push_menu_stack( COLRSEL );
      }
      break;
    }    
  case CANCEL:
    {
      // clear sub menu
      clear_rectangle( 20, 10, 780, 400);
      // redraw main menu
      display_panel_menu();
      active_button_mode = 0;  
      draw_navbuttons( 2 );
      
      // back to main menu 
      pm_cur_menu_item = 0;
      // redraw selector
      draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
               Front_Panel_Menu[pm_cur_menu_item].pos_y - 5, 315, 48, 5,
               highlight );
      
      if( colors_changed == 1 )
      {
        // pop up question save, cancel question box
        question_box( 450, 100, 300, 100, "Save changes?", 9, 10 ); 
        colors_changed = 0;
      }
      break;
    }    
  default: break;
  }
}
// =============================================================================
//
//   handle error or event list
//

void process_e_list( uint8_t nav_key )
{
  // one case only, close. No need for a switch
  if(nav_key == CANCEL)
  {
    // clear list
    clear_rectangle( 0, 0, 700, 479);
    // redraw main menu
    display_panel_menu();
    // clear starting selector
    draw_frame(Front_Panel_Menu[0].pos_x - 10,
               Front_Panel_Menu[0].pos_y - 5, 315, 48, 5, cg_background); 
    if( get_menu_stack() == ERRLIST )
    {
      log_event( "Error log"); 
      // back to main menu 
      pm_cur_menu_item = 4;
    }
    if( get_menu_stack() == EVTLIST )
    {
      log_event( "Event log"); 
      // back to main menu 
      pm_cur_menu_item = 5;
    }
    // redraw selector at correct line
    draw_frame(Front_Panel_Menu[pm_cur_menu_item].pos_x - 10,
             Front_Panel_Menu[pm_cur_menu_item].pos_y - 5, 315, 48, 5,
             highlight );
    
    active_button_mode = 0;  
    draw_navbuttons( 2 );
    pop_menu_stack();
  }  
}


