//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// May 3, 2018		        Design Documents                   CSI_screens.c 
// 
//       Routines for display of radio screens and screen information
//           for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  05/03/2018  started
//  06/07/2019  refactored to use RGB_colors, revised the index
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
// display_screen                    Displays a screen
// display_volume_bar                Display the volume bar
// draw_rssi_icon                    Display the RSSI ICON with signal strength
// display_channel_name              Display the current channel name
// draw_scan_icon                    Display the scan icon
// display_tx_power                  Display the transmit power level
// draw_lock_icon                    Display the lock icon
// draw_envelope_icon                Display the envelope icon
// draw_inboxfull_icon               Display the inbox full icon
// draw_tone_disable_icon            Display the tone disable icon
// draw_talk_around_icon             Display the talk around icon
// draw_monitor_icon                 Display the monitor (speaker) icon
// draw_button                       Display a button
// draw_menutab                      Draw a menu page tab
// draw_navbuttons                   Draw navigation buttons on the screen
// draw_cancel_button                Draw the cancel button
// clear_scroll_list                 clear the data in the scroll list
// draw_list                         Scrolling list for contacts and zones
// clear_menupage                    Clear a submenu page
// draw_TXonoff                      Display the "On Air" transmit indicator
// draw_RXonoff                      Display the "Receive" indicator
// display_POM_message               Display the power on or off message
// draw_call_icon                    Display the call icon 
// popup_lastcall_list               popup list for the last call table
// display_lastcall_icon             Display the last call icon
// popup_list                        Draw or clear a list box on the screen
// wide_list                         Draw a menu box that uses multiple columns
// blink_red                         Blink red led
// blink_green                       Blink green led
// blink_alternate                   Alternate blink red green
// display_color                     display color , and adjustment numbers
// display_packet_dump               Packet data dump, for debugging
// display_dmr_color                 Displays the DMR color code
// display_dmr_slot                  Displays the DMR slot number
// display_radio_wait                Display waiting for radio message
// display_radio_connected           Display connected to radio message
// display_heartbeat                 Display heartbeat
// display_message                   Display message, wait 3 sec, clear, return
// display_RSSI_graph                Display RSSI graph

// Status: written, tested, not used
// 
// draw_battery_icon                 Displays the battery level icon
// display_chan_selector             Display a channel list
//==============================================================================
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "CSI_SCREENS.h"
#include "CSI_GPIO.h"
#include "CSI_TIMERS.h"
#include "CSI_SSD1963.h"
#include "CSI_GRAPHICS.h"
#include "CSI_ICONS.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_COLOR.h"
#include "CSI_SWITCHES.h"
#include "CSI_Menus.h"
#include "CSI_TouchPanel.h"
#include "CSI_RADIO.h"

#ifdef radio_FT857D
#include "CSI_FT857_SCREENS.h"
#include "CSI_FT857D.h"
#endif /* FT857D */

// =============================================================================
// May 4, 2018  		Design Documents                  display_screen
//
//     Display a screen
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  05/04/2018 moved from main
//  06/19/2018 revision for Board test
//
// Status
//  writing
//

// large panel software revision number for splash 
const char revision_number[7] = " 1.1.7";
// for use with radio firmware MS30.11 30.14

// messages for test screen
// all 20 characters long
// font16 20wide +2 spacing 20*22 = 440 two columns on 800 wide screen
// use proportional spacing,  adjust to fit if too wide
test_info test_list[24] =
  {
    0, "Clock speed",     
    1, "LCD display",
    1, "SSD1963 LCD control",
    0, "Ext Flash mfrID",
    0, "Ext Flash UID",
    0, "Ext Flash R/W",
    0, "EEPROM R/W",
    0, "Switch 1",
    0, "Switch 2",
    0, "Switch 3",
    0, "Switch 4", //10
    0, "Left Knob button",
    0, "Left Knob rotate", 
    0, "Right Knob button",
    0, "Right Knob rotate",
    0, "PTT",
    0, "Mic keypad",
    0, "Touch screen",
    0, "Radio volume",
    0, "Radio channel",
    0, "UART1", //20
    0, "UART2",
    0, "USB1",  
    0, "USB2"
      
  };

tab_info screen_tab[7] =
{
  {0,   5, ICON_CONTACT},
  {132, 5, ICON_SCAN_small},
  {264, 5, ICON_ZONE},
  {396, 5, ICON_ENVELOPE},
  {528, 5, ICON_CALL_LOG},
  {660, 5, ICON_WRENCH},
  {528, 5, ICON_WRENCH}
};

int active_screen_num = 0;
uint8_t cur_call_active = 0;
uint8_t chan_nums_active = 1;     // channel numbers display flag

// for display modes
// 0 normal
// 1 original small screen
// 2 original style menus and submenus
// 3 no button pushes passed to the radio, for front panel menu
// multiple screens may be used by a mode

int display_mode = 0;
uint8_t screen_refresh = 0; // for refreshing icons 

// location on mode 0 screen 1 for standard messages
int message1_X = 30;
int message1_Y = 405;
int message2_X = 30;
int message2_Y = 435;

//extern unsigned int ticks;
//int char_time = 0;

void display_screen( uint8_t screen_num, uint8_t clear )
{
  int i,j;
  int page_row_number = 0;
  int page_column_number = 0;
  // number of tests to process
  int num_tests = 24;  

  switch( screen_num )
  {
  case 1:     // normal radio display screen
    {
    // screen 1 uses display mode 0
    display_mode = 0;
    
    // location for standard messages
    message1_X = 30;
    message1_Y = 410;
    message2_X = 30;
    message2_Y = 435;    
    
    // font colors 
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    // set shadow color
    RGB_copy( &highlight, &current_palette[get_pal_color( 4 )]); 
    // set border color
    RGB_copy( &border, &current_palette[get_pal_color( 2 )]);

    if(clear)
    {
      ssd1963_cg_clear_display();
      // set items to new for screen refresh (all) for now
      cs_channel_new = 0x7FFFFF;
      screen_refresh = 1;
    }
    // set active after clearing
    active_screen_num = screen_num;
    // border shaded frame
    draw_shaded_frame( 0, 90, 800, 390, 5, border,
                      highlight);
    // Radio shaded frame
    draw_shaded_frame( 10, 100, 780, 95, 5, border,
                      highlight);
    // 
    if(cs_channel_alias_length > 0 )
    {
      display_channel_alias();
    }
    
    /*
    // read timer again
    char_time = ticks - char_time;
    // break after
    */
   
    // turn on all buttons in case they've been turned off
    activate_all_keypress();
    r_knob_for_nav = 0;   // turn off knob sending button pushes in menu
    
    // info boxes
    draw_shaded_frame(  10, 200, 130, 160, 5, border, highlight);
    //draw_shaded_frame( 142, 200, 470, 160, 5, border, highlight);  
    draw_shaded_frame( 142, 200, 450, 160, 5, border, highlight);  
    //draw_shaded_frame( 614, 200, 176, 160, 5, border, highlight);
    draw_shaded_frame( 616, 200, 179, 160, 5, border, highlight);
    
    // draw day night buttons
    //draw_rectangle( 592, 200, 24, 78, current_palette[get_pal_color( 3 )] );
    draw_rectangle( 592, 200, 24, 78, current_palette[ 13 ] );
    draw_rectangle( 592, 280, 24, 78, current_palette[get_pal_color( 4 )] );
    draw_icon( ICON_SUN, 592, 230, 6, 13 );
    draw_icon( ICON_MOON, 592, 310, 0, 11 );
    
    // draw some stars
    draw_pixel( 595, 295, current_palette[ 0 ] );
    draw_pixel( 605, 345, current_palette[ 0 ] );
    
    // handled by packets
    // if there's data
    if(cs_channel_number_in_zone > 0)
    {
      // Zone
      display_zone_number( cs_channel_zone_number);
      display_zone_alias();
      //   Channel Number  
      if ( chan_nums_active )
      {
        display_channel_number( cs_channel_zone_number );
        display_channel_number_in_zone( cs_channel_number_in_zone );
      }
      //   Channel Frequency
      display_channel_rx_frequency();
      display_channel_tx_frequency();
    }
    
    //   Channel type
    display_channel_type(cs_channel_type, 630, 230, 2);    

    // channel number adjust icons
    draw_icon( ICON_UP_ADJ, 45, 240, get_pal_color( 6 ), cur_background );    
    draw_icon( ICON_DOWN_ADJ, 45, 335, get_pal_color( 6 ), cur_background );      
      
    //   Power level
    display_tx_power(cs_channel_power_level, 0, 630, 295, 2 );
    
    //   Volume 1-32  
    if(cs_channel_volume > 0 && cs_channel_volume < 33 ) display_volume_bar(cs_channel_volume);
    else display_volume_bar( 1 );
    // Icons
    // RSSI
    if( rssi_graph_on == 0 )
    {
      rssi_icon_isactive = 0;  // force redraw
      draw_rssi_icon( 0, 690, 15 );
    }
    // lock
    draw_lock_icon( 0, 206, 15 );
    // Envelope icon
    draw_envelope_icon( prev_env, 134, 18 );
    
    // tone disable icon
    draw_tone_disable_icon( prev_tdis );
    
    // talk around, always redraw on screen paint
    draw_talk_around_icon( prev_talka );  
    
    // Battery - No battery now
    // draw_battery_icon( 0 );
    
    // monitor 
    draw_monitor_icon( prev_monitor, 40, 15 );

    draw_scan_icon( prev_scan, 520, 15 );
    
    // Private/ Group
    draw_call_icon( prev_call_type );

    // GPS
    draw_gps();
    // lastcall icon
    if( Last_call_table[0].has_data ==1) display_lastcall_icon();
    
    // display last contact info 
    if(cur_call_active)
    {
      strncpy( (char*) cg_line_buffer, (char const*) Last_call_table[last_call_indx].text, 16);  
      // clear the current contact 
      clear_rectangle(  30 , 375, 280, 35 );
      ssd1963_cg_write_line(16, 3, 30 , 375, 4);    
    }
    // pop up list icon
    if(list_hasdata) draw_icon_rle( ICON_LASTCALL_UP_MC, 300, 425 );

    // draw buttons P1-P4
    draw_button( 1, get_pal_color( 2 ), get_pal_color( 3 ));
    draw_button( 2, get_pal_color( 2 ), get_pal_color( 3 ));
    draw_button( 3, get_pal_color( 2 ), get_pal_color( 3 ));
    draw_button( 4, get_pal_color( 2 ), get_pal_color( 3 ));
    
    CS800D_touch_point[0].is_active = 1;  // volume up
    CS800D_touch_point[1].is_active = 1;  // volume down
    CS800D_touch_point[2].is_active = 1;  // channel up
    CS800D_touch_point[3].is_active = 1;  // channel down
    CS800D_touch_point[4].is_active = 1;  // lastcall popup
    CS800D_touch_point[5].is_active = 1;  // zone alias
    CS800D_touch_point[6].is_active = 1;  // channel alias
    CS800D_touch_point[7].is_active = 1;  // P1
    CS800D_touch_point[8].is_active = 1;  // P2
    CS800D_touch_point[9].is_active = 1;  // P3
    CS800D_touch_point[10].is_active = 1; // P4
    CS800D_touch_point[11].is_active = 1; // Power icon
    CS800D_touch_point[34].is_active = 1; // rssi icon
    CS800D_touch_point[63].is_active = 1; // day brightness
    CS800D_touch_point[64].is_active = 1; // night brightness
    
    /*
    // draw buttons with menu titles
    draw_button( 5, 9, 8 );
    draw_button( 6, 9, 8 );
    draw_button( 7, 9, 8 );
    draw_button( 8, 9, 8 );    
    */
    // power button
    draw_icon( ICON_POWER, 720, 425, cur_foreground, cur_background );
    
    // done refeshing
    screen_refresh = 0;
    
    break;
    }
  // =================================================================
  case 2:  // board test screen
    {
     // font colors 
    set_font_colors(  get_pal_color( 0 ), get_pal_color( 1 ) );
    if(clear) ssd1963_cg_clear_display();
    // set active after clearing
    active_screen_num = screen_num;    
    // border shaded frame
    draw_shaded_frame( 0, 0, 800, 480, 5, border,
                      highlight);
    
    for(j = 0; j <  num_tests; j++)
    {
      if(test_list[j].passed == 0) set_font_colors(  get_pal_color( 8 ), get_pal_color( 1 ) ); // unpassed in red
      else set_font_colors( get_pal_color( 7 ), get_pal_color( 1 ));  // passed in green
      
      for ( i = 0 ; i < 20; i++) cg_line_buffer[i] = test_list[j].text[i];
      ssd1963_cg_write_line( 20, 4, 12 + page_column_number * 400, 7 + page_row_number * 24, 2);  
      
      page_row_number++;
      if(page_row_number > 18)
      {
        page_row_number = 0;
        page_column_number = 1;
      }
    }
    // display error count
    strncpy( (char*) cg_line_buffer, "Error Count ", 12);
    sprintf((char*) cg_line_buffer + 12, "%4i ", error_count);
    ssd1963_cg_write_line( 16, 3, 450 , 320, 2);
    
    
    break;
    }
  // =================================================================
  case 3:  // menu screen
    {
      //display_mode = 0;   // full menu mode(not working)
      display_mode = 2;   // original style menu mode
      
      // deactivate knobs sending to the radio
      deactivate_keypress( 7 );
      deactivate_keypress( 9 );
      // if in this menu screen and the knob turns, need to convert the knob
      // to a standard button push that is sent to the radio.
      r_knob_for_nav = 1;
      
      message1_X = 30;
      message1_Y = 110;
      message2_X = 30;
      message2_Y = 160;        
      
      // font colors 
      //set_font_colors( 3, 13 );
      set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num;      
      
      for(i = 0; i< lines_per_page; i++)
      {
        Scroll_list[i].has_data = 0;
      }
      list_top_item = 0;
      menu_line_num = 0;
           
      if( dmr_ana == 0) // DMR
      {
        // draw tabs
        draw_menutab( 0, 1);
        draw_menutab( 1, 0);
        draw_menutab( 2, 0);
        draw_menutab( 3, 0);
        draw_menutab( 4, 0);
        draw_menutab( 5, 0);        

        num_menu_levels = 6;   
      }
      
      if( dmr_ana == 1) // Analog
      {
        // draw tabs
        draw_menutab( 0, 1);
        draw_menutab( 1, 0);
        draw_menutab( 2, 0);
        draw_menutab( 3, 0);
        draw_menutab( 6, 0);
        // draw underline for unused last tab
        draw_rectangle( screen_tab[5].x_loc + 3, screen_tab[5].y_loc + 66,
                       129, 3, cg_font );         
        
        num_menu_levels = 5;   
      }
 
      // use icons for buttons
      active_button_mode = 0;      
      draw_navbuttons( 1 );
    
      all_touches_off();
      // turn on only the ones for this screen
      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4      
      
      break;
    }
  // =================================================================
  // packet dump screen for debugging and test  
  case 4:  
    {
      // font colors 
      set_font_colors( 3, 13 );
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num;      
      
      break;
    }
  // =================================================================
  // CS800D original screen mode, for debugging
    // screen 5 uses display mode 1
  case 5:  
    {
      display_mode = 1;
      // location on screen 5 for messages
      message1_X = 30;
      message1_Y = 110;
      message2_X = 30;
      message2_Y = 160;      
      // debug_p_line = 0;
      // font colors 
      //set_font_colors( 5, 3 );   // Orange on black
      //set_font_colors( 7, 3 );   // Green on black
      set_font_colors( 3, 14 );  // black on pale green
      // set highlight color
      highlight.red = 128;
      highlight.green = 96;
      highlight.blue = 0;
      // set border color
      border.red = 255;
      border.green = 192;
      border.blue = 0;       
      
      if(clear)
      {
          ssd1963_cg_clear_display();
          cs_channel_new = 0x7FFFFF;
      }
      // set active after clearing
      active_screen_num = screen_num;      
      // Radio frame
      draw_shaded_frame( 10, 100, 780, 120, 5, border,
                      highlight);
      rssi_icon_isactive = 0;
      if( rssi_graph_on == 0 ) draw_rssi_icon( 0, 690, 15 ); 
      display_zone_alias();
      display_channel_alias(); 
      display_tx_power( prev_powlvl, 0, 630, 295, 2 );
      
      // power button icon
      draw_icon( ICON_POWER, 720, 425, cur_foreground, cur_background );
      
      // no touch points on this screen
      all_touches_off();
      // turn on all buttons in case they've been turned off
      activate_all_keypress();
    
      break;
    }
  // =================================
  // front panel controls screen  
  case 6:  
    { 
      display_mode = 3;   // no packet displays for now
      all_touches_off();
      // writing here... add knob controls to panel menu
      
      // set only the four buttons as touch points
      CS800D_touch_point[7].is_active = 1;  // P1
      CS800D_touch_point[8].is_active = 1;  // P2
      CS800D_touch_point[9].is_active = 1;  // P3
      CS800D_touch_point[10].is_active = 1; // P4
      
      // turn off sending buttons to radio
      deactivate_all_keypress();

      active_screen_num = screen_num;
      // font colors 
      set_font_colors( 3, 14 );   // 
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num;      
      // reset to always start at the beginning
      reset_panel_menu(); 
      // menu items
      display_panel_menu();
      
      active_button_mode = 0;
      draw_navbuttons( 2 );
      
      log_event( "Panel menu"); 
      
      break;
    }
  // =================================
  // Start up splash screen  
  case 7:  
    {
      uint8_t c_red;
      uint8_t c_green;
      uint8_t c_blue;

      uint8_t cbk_red;
      uint8_t cbk_green;
      uint8_t cbk_blue;
      
      display_mode = 0;
     
      // store current colors for FT857 which has changed colors to those
      // for the radio screen before the splash screen runs
      c_red   = cg_font.red;
      c_green = cg_font.green;
      c_blue  = cg_font.blue;      
      cbk_red   = cg_background.red;
      cbk_green = cg_background.green;
      cbk_blue  = cg_background.blue;
      
      // set foreground d blue
      cg_font.red = 10;
      cg_font.green = 100;
      cg_font.blue = 150;
      // set background lt blie
      cg_background.red = 170;
      cg_background.green = 190;
      cg_background.blue = 220;
      
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num; 
      
      // reset colors
      // FT857 receives color set packet while screen clears
      // set foreground d blue
      cg_font.red = 10;
      cg_font.green = 100;
      cg_font.blue = 150;
      // set background lt blie
      cg_background.red = 170;
      cg_background.green = 190;
      cg_background.blue = 220;      
      
      draw_icon_rle( ICON_LOGO_C, 120, 170 );      
      
      draw_icon_rle( ICON_LOGO_S, 320, 170 );
      
      draw_icon_rle( ICON_LOGO_ANT, 530, 170 );

      draw_icon_mirrored( ICON_LOGO_Waves, 550, 0, 1 );
      
      // invert colors
      invert_font_colors();
     
      // fill a rectangle
      draw_rectangle(110, 375, 580, 60, cg_background );
            
      char *message = "Connect Systems Inc.";
      strncpy( (char *) cg_line_buffer , message, 20);
      ssd1963_cg_write_line( 20, 2, 125, 375, 6);
            
      // restore normal text colors
      // invert colors
      invert_font_colors();
      
      char *version = "Version ";
      strncpy( (char *) cg_line_buffer , version, 8);
      ssd1963_cg_write_line( 8, 3, 270, 440, 4);
      
      strncpy( (char *) cg_line_buffer , revision_number, 7);
      ssd1963_cg_write_line( 7, 3, 420, 440, 4);
    
      // restore current colors 
      cg_font.red   = c_red;
      cg_font.green = c_green;
      cg_font.blue  = c_blue;      
      cg_background.red   = cbk_red;
      cg_background.green = cbk_green;
      cg_background.blue  = cbk_blue;
      
      break;
    }
  // =================================
  // Start up programming screen  
  case 8:  
    {
      display_mode = 0;
      set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );      
      
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num;      
      
      char *txt_1 = "Program Mode";
      strncpy( (char *) cg_line_buffer , txt_1, 12);
      ssd1963_cg_write_line( 12, 2, 200, 150, 6);
      
      char *txt_2 = "Wait for the firmware update to finish.";
      strncpy( (char *) cg_line_buffer , txt_2, 39);
      ssd1963_cg_write_line( 39, 3, 125, 250, 2);

      char *txt_3 = "Then turn off the radio and restart.";
      strncpy( (char *) cg_line_buffer , txt_3, 36);
      ssd1963_cg_write_line( 36, 3, 150, 300, 2);      
      
      break;
    }
  // =================================
  // Start up RCDB screen  
  case 9:  
    {
      display_mode = 0;
      set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );      
      
      if(clear) ssd1963_cg_clear_display();
      // set active after clearing
      active_screen_num = screen_num;      
      
      char *txt_3 = "RCDB Reset";
      strncpy( (char *) cg_line_buffer , txt_3, 10);
      ssd1963_cg_write_line( 10, 2, 220, 150, 6);
      
      char *txt_4 = "Wait for the radio to finish.";
      strncpy( (char *) cg_line_buffer , txt_4, 29);
      ssd1963_cg_write_line( 29, 3, 175, 250, 2);

      char *txt_5 = "Then turn off the radio and restart.";
      strncpy( (char *) cg_line_buffer , txt_5, 36);
      ssd1963_cg_write_line( 36, 3, 150, 300, 2);      
      
      break;
    }     
  default: break;  
  }
}

// =============================================================================
// April 26, 2018		Design Documents              display_volume_bar
//
// This program displays the volume bar. 
//
//  Uses draw_progress_bar which:
//   Locates the bar on the screen at x,y location
//   defines the width and height
//   selects the border, bar, and background colors
//
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   volume_value: This is the value of the volume bar and is passed as
//   a parameter
//
// Exit Conditions
//  The volume bar is displayed.
//
// External functions called
//  draw_progress_bar 
//
// History 
//  05/03/2018 moved from main. size and location are not variable
//  11/11/2019 added volume number display
//
// Status
//  written , need to make location moveable
//
uint16_t current_radio_volume = 0;

// touch button location variables, for screen 1
// x and y are swapped, need to adjust in controller

// xy adjusted here to test
uint16_t volume_up_touch_miny = 760;
uint16_t volume_up_touch_maxy = 800; 
uint16_t volume_up_touch_minx = 375; 
uint16_t volume_up_touch_maxx = 415;

uint16_t volume_dn_touch_miny = 535;
uint16_t volume_dn_touch_maxy = 575;
uint16_t volume_dn_touch_minx = 375;
uint16_t volume_dn_touch_maxx = 415;

void display_volume_bar(uint8_t volume_value)
{
  uint16_t val;
  // if volume is displayed on the screen
  // update volume display
  
  // volume level is 1-32
  // progress bar is 0-100%
  // subtract 1  0 - 31
  // multply by 100  3100 max
  // divide by 31  0 to 100 
  val = (uint16_t) ((volume_value - 1) * 100 ) / 31;
  current_radio_volume = val;
  if(val > 100) val = 100;  // in case of a garbage value

  // one screen version (for now) 
  strncpy( (char * ) cg_line_buffer, "Volume", 6);
  ssd1963_cg_write_line( 6, 4, 463 , 381, 2);
  
  draw_progress_bar( val, 0, 600, 380, 170, 30, get_pal_color( 2 ), get_pal_color( 7 ), 1);
  draw_icon( ICON_LEFT_ADJ, 579, 378, get_pal_color( 6 ), cur_background );
  draw_icon( ICON_RIGHT_ADJ, 774, 378, get_pal_color( 6 ), cur_background );

  // show volume number
  sprintf((char*) cg_line_buffer, "%3u ", volume_value); 
  ssd1963_cg_write_line( 3, 4, 545 , 381, 2);    

}

// =============================================================================
// May 5, 2018  		Design Documents                  draw_rssi_icon
//
//     Displays the RSSI icon with signal strength bars. 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   rssi_value: This is the value of the rssi bar and is passed as
//   a parameter
//
// Exit Conditions
//  The rssi icon and bars are displayed.
//
// External functions called
//  draw_rectangle
//  draw_icon
//
// External variables used
//
// Internal variables used
//
// History 
//  05/03/2018 moved from main, added 5th bar
//  05/04/2018 removed 5th bar
//  11/07/2018 made relocatable x, y
//
// Status
//  written, tested, working
//

// x 690
// y 15 

uint8_t rssi_icon_isactive = 0;

void draw_rssi_icon( uint8_t rssi_value, uint16_t pos_x, uint16_t pos_y )
{
  static uint8_t last_value = 0;
  RGB_color r_color;
  RGB_copy( &r_color, &current_palette[get_pal_color( 8 )]);
  
  // draw the icon first
  if(!rssi_icon_isactive)
  {
    // erase
    clear_rectangle( pos_x - 2, pos_y - 2, 90, 76 );
    // Antenna icon  
    draw_icon( ICON_ANTENNA, pos_x, pos_y, cur_foreground, cur_background );
    // underline
    draw_rectangle( pos_x + 24, pos_y + 67, 48, 5, cg_font );
    rssi_icon_isactive = 1;
  }
  if(rssi_value != last_value)
  {
    // background blocks to erase previous
    // erase bars 1 & 2
    clear_rectangle( pos_x + 30, pos_y + 41, 20, 24 );
    // // erase bars 3-4
    clear_rectangle( pos_x + 52, pos_y + 5, 20, 60 );
    // rssi bars
    if(rssi_value > 0) draw_rectangle( pos_x + 30, pos_y + 53, 7, 12, r_color);
    if(rssi_value > 1) draw_rectangle( pos_x + 41, pos_y + 41, 7, 24, r_color);
    if(rssi_value > 2) draw_rectangle( pos_x + 52, pos_y + 29, 7, 36, r_color);
    if(rssi_value > 3) draw_rectangle( pos_x + 63, pos_y + 17, 7, 48, r_color);
    // if(rssi_value > 4) draw_rectangle( 764, 20, 7, 60, 255, 0, 0);
    last_value = rssi_value;
  }
  
}
// =============================================================================
// May 5, 2018  		Design Documents            display_channel_name
//
//     Displays the current channel name 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  string to display is loaded in cg_line_buffer[]
//  number of characters to display is passed
//
// Exit Conditions
//
// External functions called
//  strncmp
//  draw_rectangle
//  ssd1963_cg_write_line
//
// External variables used
//
// Internal variables used
//
// History 
//  05/03/2018 moved from main
//
// Status
//  written, working
//

char prev_message[20];
uint8_t channelname_isactive = 0;

// touch button location variables, for screen 1
// x and y are swapped, need to adjust in controller

// xy adjusted here to test
uint16_t channel_up_touch_miny = 0;
uint16_t channel_up_touch_maxy = 120; 
uint16_t channel_up_touch_minx = 220; 
uint16_t channel_up_touch_maxx = 280;

uint16_t channel_dn_touch_miny = 0;
uint16_t channel_dn_touch_maxy = 120;
uint16_t channel_dn_touch_minx = 320;
uint16_t channel_dn_touch_maxx = 380;

uint16_t lastcall_touch_miny = 50;
uint16_t lastcall_touch_maxy = 300;
uint16_t lastcall_touch_minx = 400;
uint16_t lastcall_touch_maxx = 450;

// Font 1 (48) 16 characters @ 63 max + 3 spacing  = 1056 won't fit on screen
// Font 2 (36) 16  * 46 + 3  = 784 , barely

void display_channel_name( uint8_t numchars )
{
    // prevent overflowing array
    if(numchars > 20) numchars = 20;
    int i;
    // if it's not active, make active
    if(!channelname_isactive) 
    {
      // copy channel alias data to previous array
      for(i = 0; i < numchars; i++) prev_message[i] = cg_line_buffer[i];
      // erase 
      clear_rectangle( 170, 210, 385, 73 );
      // write new
      ssd1963_cg_write_line(numchars , 2, 170, 210,3);
      channelname_isactive = 1;
    }
    // if the current message is different from the previous, display the new
    if(strncmp((char const * ) prev_message, (char const * ) cg_line_buffer, numchars) != 0)
    {
      // copy channel alias data to array
      for(i = 0; i < numchars; i++) prev_message[i] = cg_line_buffer[i];
      // erase previous
      clear_rectangle( 170, 210, 380, 73 );
      // write new
      ssd1963_cg_write_line(numchars , 1, 170, 210,3); 
    }
}
// =============================================================================
// May 5, 2018  		Design Documents                  draw_scan_icon
//
//     Displays the scan icon, active or inactive 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  scan bit is passed
//
// Exit Conditions
//
// External functions called
//  draw_icon
//
// History 
//  05/05/2018 moved from main
//  11/07/2018 made relocatable x, y
//
// Status
//  writen, tested 
//

uint8_t scanicon_isactive = 0; 
uint8_t prev_scan = 0;

void draw_scan_icon( uint8_t scan, uint16_t pos_x, uint16_t pos_y)
{
  if(prev_scan != scan || screen_refresh)
  {
    // scanning
    if(scan == 1)
    {
      draw_icon( ICON_SCAN, pos_x, pos_y, get_pal_color( 8 ), cur_background );
      //draw_icon( ICON_SCAN, 520, 15, get_pal_color( 8 ), cur_background );
      prev_scan = 1;
    }
    else  // not scanning
    {
      // erase 
      // clear_rectangle( 520, 15, 72, 73 );
      draw_icon( ICON_SCAN, 520, 15, get_pal_color( 5 ), cur_background ); // in grey
      prev_scan = 0; 
    }
  }
}
// =============================================================================
// May 5, 2018  		Design Documents                display_tx_power
//
//     Displays the transmit power level
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  power level is passed
//  colors are based on loaded simple palette 
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  set_font_colors
//  ssd1963_cg_write_line
//  error_log
//
// History 
//  05/05/2018 moved from main
//  11/07/2018 made relocatable x, y, font
//
// Status
//  written
//

uint8_t prev_powlvl = 0;
uint8_t prev_txrx = 0;  

void display_tx_power( uint8_t powlvl, uint8_t txonoff,
                      uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  unsigned char *message;
  int i;
  uint8_t cur_fg;
  uint8_t cur_bg;
  
  // store before changing, because setting changes current
  cur_fg = cur_foreground;
  cur_bg = cur_background;
  
  if(( powlvl != prev_powlvl) || (prev_txrx != txonoff) || screen_refresh)
  {
    if(display_mode == 0)
    {
      // erase old
      clear_rectangle( pos_x, pos_y, 152, 55 );  
      // set color for TX
      if(txonoff)
      {
        // set font color to active (red)
        set_font_colors( get_pal_color( 8 ), cur_bg );
      }
      else
      {
        // set to normal
        set_font_colors( cur_fg,  cur_bg );
      }
      // display new
      switch (powlvl)
      {
        case 0:
          {
            message = "LOW";
            for ( i = 0 ; i < 3; i++) cg_line_buffer[i] = message[i];
            ssd1963_cg_write_line( 3, font, pos_x, pos_y, 4);
            break;
          }
        case 1:
          {
            message = "MED";
            for ( i = 0 ; i < 3; i++) cg_line_buffer[i] = message[i];
            ssd1963_cg_write_line( 3, font, pos_x, pos_y, 4);
            break;
          }
        case 2:
          {
            message = "HIGH";
            for ( i = 0 ; i < 4; i++) cg_line_buffer[i] = message[i];
            ssd1963_cg_write_line( 4, font, pos_x, pos_y, 4);
            break;
          }
        default: error_log(3001); break;
      }
    }
  if(display_mode == 1)
  {
    // leave colors unchanged
    // erase old
    clear_rectangle( 20, 20, 65, 75 );
    // display new
    switch (powlvl)
    {
      case 0:
        {
          ssd1963_cg_output_character('L', 1, 20, 20, 4);
          break;
        }
      case 1:
        {
          message = "M";
          ssd1963_cg_output_character('M', 1, 20, 20, 4);
          break;
        }
      case 2:
        {
          message = "H";
          ssd1963_cg_output_character('H', 1, 20, 20, 4);          
          break;
        }
      default: error_log(3001); break;
    }    
  }
  // set font back to before  
  set_font_colors( cur_fg, cur_bg );
  // save previous for next time
  prev_powlvl = powlvl;
  prev_txrx = txonoff;
  }
}

// =============================================================================
// May 6, 2018  		Design Documents                  draw_lock_icon
//
//     Displays the lock icon, either locked, unlocked or none
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  lock variable is passed, 0 none, 1 open, 2 closed
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  error_log
//
// History 
//  05/06/2018
//  11/07/2018 made relocatable x, y
//
// Status
//  written, tested

// x 206
// y 15 

uint8_t prev_lock = 0;

void draw_lock_icon( uint8_t lock, uint16_t pos_x, uint16_t pos_y )
{
  // only redraw if it has changed
  if(prev_lock != lock || screen_refresh)
  {
    switch(lock)
    {
      case 0: // no icon
        {
          clear_rectangle( pos_x - 2, pos_y - 2, 80, 76 );
          //draw_icon( ICON_UNLOCKED, 206, 15, get_pal_color( 5 ), cur_background ); // grey
          draw_icon( ICON_UNLOCKED, pos_x, pos_y, get_pal_color( 5 ), cur_background ); // grey
          break;
        }
      case 1: // open lock 
        {
          clear_rectangle( pos_x - 2, pos_y - 2, 80, 76 );
          draw_icon( ICON_UNLOCKED, pos_x, pos_y, cur_foreground, cur_background );
          break;        
        }
      case 2:  // closed lock
        {
          clear_rectangle( pos_x - 2, pos_y - 2, 80, 76 );
          draw_icon( ICON_LOCKED, pos_x + 24, pos_y, cur_foreground, cur_background );
          break;          
        }
      default:  error_log(3001); break;
    }
      prev_lock = lock;
  }
}

// =============================================================================
// May 6, 2018  		Design Documents              draw_envelope_icon
//
//     Displays the envelope icon
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  envelope variable is passed 0 - no message,  1 - message
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  ssd1963_cg_write_line
//  
// History 
//  05/06/2018
//  11/07/2018 made relocatable x, y
//
// Status
//  written, tested

// x 134
// y 18

uint8_t prev_env = 0;

void draw_envelope_icon( uint8_t env, uint16_t pos_x, uint16_t pos_y )
{
  unsigned char *message;
  int i;
  // only redraw if it has changed
  if(prev_env != env || screen_refresh)
  {
    if(env)
    {
      clear_rectangle( pos_x - 2, pos_y - 2, 68, 52 );
      
      draw_icon_rle( ICON_ENVELOPE_MC, pos_x, pos_y );
      // draw_icon( ICON_ENVELOPE, 134, 18, cur_foreground, cur_background );
      message = "message";  
      for ( i = 0 ; i < 7; i++) cg_line_buffer[i] = message[i];
      ssd1963_cg_write_line( 7, 4, pos_x - 9 , pos_y + 47, 2);  
    }
    else
    {
      // erase icon
      clear_rectangle( pos_x - 2, pos_y - 2, 68, 52 );
      // erase text
      clear_rectangle( pos_x - 9, pos_y + 47, 80, 24 );
      draw_icon( ICON_ENVELOPE, pos_x, pos_y, get_pal_color( 5 ), cur_background ); // grey      
    }
    prev_env = env;
  }
}
// =============================================================================
// May 6, 2018  		Design Documents             draw_inboxfull_icon
//
//     Displays the inbox full
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  inbocfull variable is passed 
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  
// History 
//  05/06/2018 
//
// Status
//  written

uint8_t prev_inbox = 0;

void draw_inboxfull_icon( uint8_t inbox )
{
  unsigned char *message;
  int i;
  // only redraw if it has changed
  if(prev_inbox != inbox || screen_refresh)
  {
    if(inbox)
    {
      clear_rectangle( 132, 16, 68, 52 );
      draw_icon( ICON_INBOXFULL, 105, 18, cur_foreground, cur_background );
      message = "inbox full";  
      for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
      ssd1963_cg_write_line( 10, 4, 110, 65, 2);  
    }
    else
    {
      // erase icon
      clear_rectangle( 132, 16, 68, 52 );
      // erase text
      clear_rectangle( 110, 65, 80, 24 );
    }
    prev_inbox = inbox;
  }
}
// =============================================================================
// May 6, 2018  		Design Documents          draw_tone_disable_icon
//
//     Displays the tone disable icon
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  tone disable variable is passed
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  
// History 
//  05/06/2018 
//
// Status
//  written, tested

uint8_t prev_tdis = 1;

void draw_tone_disable_icon( uint8_t tdis )
{
  // only redraw if it has changed
  if(prev_tdis != tdis || screen_refresh)
  {
    if(tdis)
    {
      //clear_rectangle( 288, 13, 76, 76 );
      draw_icon( ICON_TONEDISABLE, 290, 15, cur_foreground, cur_background );
    }
    else
    {
      // erase icon
      //clear_rectangle( 288, 13, 76, 76 );
      draw_icon( ICON_TONEDISABLE, 290, 15, get_pal_color( 5 ), cur_background ); // grey
    }
    prev_tdis = tdis;
  }
}
// =============================================================================
// May 6, 2018  		Design Documents          draw_talk_around_icon
//
//     Displays the talk around icon
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  talk-around variable is passed
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  
// History 
//  05/06/2018
//  09/04/2018  changed to always redraw, moved the "if needed" to
//              new update_talk_around_icon routine
//
// Status
//  written, tested

uint8_t prev_talka = 0;

void draw_talk_around_icon( uint8_t talka )
{
  if(prev_talka != talka|| screen_refresh)
  {
    if(talka)
    {
      //clear_rectangle( 440, 13, 76, 76 );
      // draw vertical bars
      draw_rectangle( 440, 15, 8, 72, current_palette[get_pal_color( 0 )] );
      draw_rectangle( 504, 15, 8, 72, current_palette[get_pal_color( 0 )] );
      draw_icon( ICON_ARROW, 460, 40, cur_foreground, cur_background );
    }
    else
    {
      // erase icon
      // clear_rectangle( 440, 13, 76, 76 );
      // draw vertical bars
      draw_rectangle( 440, 15, 8, 72, current_palette[get_pal_color( 5 )] );
      draw_rectangle( 504, 15, 8, 72, current_palette[get_pal_color( 5 )] );
      draw_icon( ICON_ARROW, 460, 40, get_pal_color( 5 ), cur_background );
    }
    prev_talka = talka;
  }
}
// =============================================================================
// May 6, 2018  		Design Documents               draw_battery_icon
//
//     Displays the battery level icon.
//      !! this is not being used with the mobile !!
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  batt variable is passed (does not exist yet) 
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  
// History 
//  05/06/2018 
//
// Status
//  written, tested, not being used

uint8_t batt_icon_isactive = 0;
uint8_t prev_batt = 0;

void draw_battery_icon( uint8_t batt )
{
  // if it's being used draw the frame
  if(!batt_icon_isactive)
  {
    clear_rectangle( 13, 13, 52, 76 );
    draw_icon( ICON_BATTERY, 15, 15, cur_foreground, cur_background );
    batt_icon_isactive = 1;
  }
  
  // only redraw if it has changed
  if(prev_batt != batt)
  {
    // erase just the bars
    draw_rectangle( 24, 26, 31, 50, cg_background );
    // draw new bars
    if(batt > 0) draw_rectangle( 24, 70, 31, 9,
                                current_palette[get_pal_color( 7 )]);
    if(batt > 1) draw_rectangle( 24, 59, 31, 9,
                                current_palette[get_pal_color( 7 )]);
    if(batt > 2) draw_rectangle( 24, 48, 31, 9,
                                current_palette[get_pal_color( 7 )]);
    if(batt > 3) draw_rectangle( 24, 37, 31, 9,
                                current_palette[get_pal_color( 7 )]);
    if(batt > 4) draw_rectangle( 24, 26, 31, 9,
                                current_palette[get_pal_color( 7 )]);
    prev_batt = batt;
  }
}
// =============================================================================
// May 6, 2018  		Design Documents               draw_monitor_icon
//
//     Displays the monitor (speaker)  icon
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  monitor variable is passed
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  draw_icon
//  
// History 
//  05/06/2018 written
//  11/07/2018 added x,y locators
//  06/17/2019 prevented icon on an anlog channel
//
// Status
//  written, tested

uint8_t prev_monitor = 0;

void draw_monitor_icon( uint8_t monitor, uint16_t pos_x, uint16_t pos_y )
{
  // only redraw if it has changed
  if(prev_monitor != monitor || screen_refresh)
  {
    if(monitor == 1 && cs_channel_type == 0 )
    {
      // draw icon
      draw_icon( ICON_SPEAKER, pos_x, pos_y, cur_foreground, cur_background );
    }
    else
    {
      // draw icon in greyout color
      draw_icon( ICON_SPEAKER, pos_x, pos_y, get_pal_color( 5 ), cur_background ); 
    }
    prev_monitor = monitor;

  }
}
// =============================================================================
// May 9, 2018  		Design Documents                     draw_button
//
//     Displays a button
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  button info array has text and location, which button to display is passed
//
// Exit Conditions
//
// External functions called
//  draw_rectangle
//  ssd1963_cg_write_line
//  
// History 
//  05/09/2018 
//
// Status
//  written, working

// data structure to contain variable button info, can add buttons or 
// change button text or location

struct button_data
  {
  unsigned int button_number;
  char *button_text;
  unsigned int xpos;
  unsigned int ypos;
  }button_list[11] =
  {
  0,  "  ", 0 , 0,     // button 0, no button    
  1,  "P1", 365, 420,
  2,  "P2", 450, 420,
  3,  "P3", 535, 420,
  4,  "P4", 620, 420,
  5,  "Moni", 355, 420,
  6,  "Encr", 440, 420,
  7,  "Scan", 525, 420,
  8,  "Menu", 610, 420,
  9,  "NO",   345, 420,
  10, "YES",  610, 420
  
  };
// old button locations for proto board
/*
  1,  "P1", 135, 420,
  2,  "P2", 315, 420,
  3,  "P3", 495, 420,
  4,  "P4", 675, 420 */

void draw_button( uint8_t B_num, uint8_t bordr_color, uint8_t face_color )
{
  unsigned int bordr_size;
  unsigned int tlen;
  // int i;
  
  if(B_num < 5)
  {
    bordr_size = 4;
    // draw buttons 60 wide 50 high
    draw_shaded_frame( button_list[B_num].xpos, button_list[B_num].ypos,
                    60, 50, bordr_size, current_palette[face_color],
                    current_palette[bordr_color] );
    // write text
    tlen = strlen(button_list[B_num].button_text);
    
    memcpy( cg_line_buffer, button_list[B_num].button_text, tlen );  
    //for( i = 0 ;i < tlen; i++) cg_line_buffer[i] = button_list[B_num].button_text[i]; 

    ssd1963_cg_write_line( tlen, 3, button_list[B_num].xpos + bordr_size + 8,
                                    button_list[B_num].ypos + bordr_size + 4, 2);  
  }
  else
  {
    bordr_size = 3;
    // draw buttons 80 wide 50 high
    draw_shaded_frame( button_list[B_num].xpos, button_list[B_num].ypos,
                    80, 50, bordr_size, current_palette[face_color],
                    current_palette[bordr_color] );
    // write text
    tlen = strlen(button_list[B_num].button_text);
    
    memcpy( cg_line_buffer, button_list[B_num].button_text, tlen );
    // for( i = 0 ;i < tlen; i++) cg_line_buffer[i] = button_list[B_num].button_text[i];                       
    ssd1963_cg_write_line( tlen, 4, button_list[B_num].xpos + bordr_size + 8,
                                    button_list[B_num].ypos + bordr_size + 10, 1);    
  }
  
}
// =============================================================================
//
// GPS ICON
//
void draw_gps(void)
{
  draw_icon( ICON_GPS, 620, 15, 2, cur_background );
}
// =============================================================================
// July 20, 2018  		Design Documents                    draw_menutab
//
//    Draw a menu page tab in foreground color or greyed out
//    uses tab data stored in the tab info array 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  07/20/2018
//  09/20/2019 fixed greyout icon for analog utilities
//
// Status
//  written, working
//

void draw_menutab( uint8_t tabnum, uint8_t active)
{
  int tnum;
  //uint8_t draw_color;
  // set colors
      set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
  //if(active)  draw_color = 3; // black
  //if(active)  draw_color = get_pal_color( 0 ); // font
  //else draw_color = get_pal_color( 5 );  // 2 grey

  // left side
  draw_icon( ICON_TAB_L,screen_tab[tabnum].x_loc,
                        screen_tab[tabnum].y_loc, get_pal_color( 0 ), get_pal_color( 1 ) );       
  // top
  draw_rectangle( screen_tab[tabnum].x_loc+32, screen_tab[tabnum].y_loc, 72, 3,
                  current_palette[get_pal_color( 0 )] );
  // right side
  draw_icon( ICON_TAB_R, screen_tab[tabnum].x_loc+32+72,
                         screen_tab[tabnum].y_loc, get_pal_color( 0 ), get_pal_color( 1 ) ); 
    
  // draw icon
  if(active)
  {
    switch(tabnum)
    {
    case 0:  // contacts
      {
      draw_icon_rle( ICON_SMILEY_MC, screen_tab[tabnum].x_loc+42,
                      screen_tab[tabnum].y_loc+10 ); 
      break;
      }
    case 1:  // scan
      {
      draw_icon_rle( ICON_SCAN_MC, screen_tab[tabnum].x_loc+42,
                      screen_tab[tabnum].y_loc+10 ); 
      break;
      }      
    case 2:  // zones
      {
      draw_icon_rle( ICON_ZONE_MC, screen_tab[tabnum].x_loc+42,
                         screen_tab[tabnum].y_loc+10 );    
      break;
      }
    case 3:  // messages
      {
      draw_icon_rle( ICON_ENVELOPE_MC, screen_tab[tabnum].x_loc+42,
                         screen_tab[tabnum].y_loc+10 );    
      break;
      }
    case 4:  // call log
      {
        // analog menu
        if( dmr_ana == 1)
        {
          draw_icon_rle( ICON_UTILITY_MC, screen_tab[tabnum].x_loc+42,
                         screen_tab[tabnum].y_loc+10 ); 
        }
        else  // DMR
        {
        draw_icon_rle( ICON_CALLOG_MC, screen_tab[tabnum].x_loc+42,
                         screen_tab[tabnum].y_loc+10 );
        }
      break;
      }
    case 5:  // utilities
      {
      draw_icon_rle( ICON_UTILITY_MC, screen_tab[tabnum].x_loc+42,
                         screen_tab[tabnum].y_loc+10 );    
      break;
      }      
    default:  break;          
    }
  }
  else // greyed out icons
  {
    // analog menu
    if( dmr_ana == 1)
      {
        tnum = tabnum;
        if(tabnum == 4 ) tnum = 5;
        draw_icon( screen_tab[tnum].icon, screen_tab[tabnum].x_loc+42,
          screen_tab[tabnum].y_loc+10, get_pal_color( 5 ), get_pal_color( 1 ) );
      }
    // DMR
    else
    {
      draw_icon( screen_tab[tabnum].icon, screen_tab[tabnum].x_loc+42,
        screen_tab[tabnum].y_loc+10, get_pal_color( 5 ), get_pal_color( 1 ) );      
    }
  }
  
  // draw underline after side icons 
  if(active)
  {
    // clear underline
    // draw underline in backround color
    draw_rectangle( screen_tab[tabnum].x_loc + 3, screen_tab[tabnum].y_loc + 66,
                   129, 3, cg_background );      
  }
  else //  greyed out
  {
    // draw underline in foreground color
    draw_rectangle( screen_tab[tabnum].x_loc + 3, screen_tab[tabnum].y_loc + 66,
                   129, 3, cg_font);  
  }  
}
// =============================================================================
// July 23, 2018  		Design Documents                 draw_navbuttons
//
//    Draw navigation buttons on the screen
//     Several different modes depending on which buttons and where
//   
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  07/23/2018
//  10/2/2018  changed to use existing foreground and background colors
//
// Status
//  written, working
//

// mode 0 at start because setting to one draws it the first time
uint8_t active_button_mode = 0;

void draw_navbuttons( uint8_t mode )
{
  // only redraw if changed
  if(  mode != active_button_mode)
  {
    // clear button area
    clear_rectangle(  350 , 420, 400 , 55);
    if(mode == 1 )
    {
      // first menu level, right - left
      // use icons for buttons

      draw_icon( ICON_NAV_X,     385, 425, 4, cur_background );
      draw_icon( ICON_NAV_LEFT,  470, 425, cur_foreground, cur_background );    
      draw_icon( ICON_NAV_RIGHT, 560, 425, cur_foreground, cur_background );
      draw_icon( ICON_NAV_DOT,   630, 425, cur_foreground, cur_background );
    }
    if(mode == 2 )
    {
      // second menu level, up down
      draw_icon( ICON_NAV_X,    385, 425, 4, cur_background );  
      draw_icon( ICON_NAV_UP,   460, 430, cur_foreground, cur_background );    
      draw_icon( ICON_NAV_DOWN, 545, 440, cur_foreground, cur_background );
      draw_icon( ICON_NAV_DOT,  630, 425, cur_foreground, cur_background );
    
    }
    active_button_mode = mode;
  }
}
// =============================================================================
//
// Draw the cancel button
//   for screens that only have close option
//   short routine to keep icon calls in screens.c


void draw_cancel_button( void )
{
   draw_icon( ICON_NAV_X, 385, 425, 4, cur_background );
}


// =============================================================================
//
// clear the data in the scroll list 
//
//
void clear_scroll_list( void )
{
  int i;
  // clear list data
  for(i = 0; i< lines_per_page; i++)
  {
    Scroll_list[i].text[0] = '\0';
    Scroll_list[i].has_data = 0;
  }
  menu_line_num = 0;      
 
}

// =============================================================================
//
//   Scrolling list for contacts and zones
//
// updown is 0 for down 1 for up, it can be sent as 2 or something else
//   which will not scroll

// for sub menu list
uint8_t  menu_line_num = 0;
uint8_t  lines_per_page = 9;
uint32_t list_items_shown = 0;

List_text Scroll_list[9];
// List_text Zone_list[250];
// pointer to which record in the list begins the page
int list_top_item = 0;
int list_bottom;

void draw_list( uint16_t x_loc, uint16_t y_loc, uint32_t txt_len, uint8_t updown )
{
  int i;
  
  // if((active_screen_num == 3) && (cs_menu_page == 0)) // just for contacts
  if( active_screen_num == 3 )  // testing for all menu pages
  {
    // change button icons
    draw_navbuttons( 2 );
    // draw frame for list
    draw_frame( x_loc-11, y_loc - 7, 332  , 290, 3,
                   highlight);     
    // four cases:
    // up arrow at top of list
    // up arrow at bottom or middle of list
    // down arrow at top or middle of list    
    // down arrow at bottom of list
         
    // ===============================
    // up 
    if(updown == 1)
    { 
      // up arrow at top of list
      if( menu_line_num == 0)
      {
        // scroll down, write all new page
        list_top_item--;
        if(list_top_item < 0) list_top_item = lines_per_page -1;
        // put new data into list
        // copy and pad with spaces
        for (i = 0; i < 16; i++)
        {
          if( i < txt_len) Scroll_list[list_top_item].text[i] = cg_line_buffer[i];
          else Scroll_list[list_top_item].text[i] = cg_line_buffer[i] = ' ';
        }
        Scroll_list[list_top_item].has_data = 1;         
        // clear inside the frame, then no need to redraw
        clear_rectangle(x_loc-8, y_loc, 322, 280);
        // clear page
      
        // make sub
        // write full list
        int j;
        int top_idx;
        for( j = 0; j < lines_per_page; j++)
        {
          top_idx = list_top_item + j;
          if( top_idx > lines_per_page-1) top_idx -= lines_per_page;
          // if not blank write data
          if( Scroll_list[top_idx].has_data )
          {
            // load screen buffer
            for (i = 0; i < 16; i++)
            {
              cg_line_buffer[i] = Scroll_list[top_idx].text[i];
            }            
            // display text
            ssd1963_cg_write_line(16, 4, x_loc, y_loc + (30 * j) , 4);
          }          
        }
        // redraw highlight at top because page was cleared
        // draw new selection frame 
        draw_frame( x_loc-8, y_loc - 5, 320  , 34, 3,
                 highlight);          
      }
    
      else 
      {
      // ===============================  
        // up arrow at bottom or middle of list 
         // move highlight up
        // clear current frame
        draw_frame( x_loc-8, y_loc + (30 * menu_line_num )-5, 320  , 34,
                 3, cg_background );      
        // move current up
        menu_line_num--;
        // draw new selection frame 
        draw_frame( x_loc-8, y_loc + (30 * menu_line_num) - 5, 320  , 34, 3,
                   highlight);      
         // if blank write data
        if( !Scroll_list[menu_line_num].has_data )
        {
          ssd1963_cg_write_line(txt_len, 4, x_loc,
                                y_loc + (30 * menu_line_num) , 4);
        }
      }
    }
    // ===============================  
    // down
    if(updown == 0)
    {  
      // down arrow at bottom of list
      if( menu_line_num == lines_per_page -1)
      {
        // scroll up, write all new page
        list_bottom = list_top_item;
        list_top_item++;
        if(list_top_item > lines_per_page -1) list_top_item = 0;
        // put new data into list
        // copy and pad with spaces
        for (i = 0; i < 16; i++)
        {
          if( i < txt_len) Scroll_list[ list_bottom ].text[i] = cg_line_buffer[i];
          else Scroll_list[list_bottom].text[i] = cg_line_buffer[i] = ' ';
        }
        Scroll_list[list_bottom].has_data = 1;
        
        // clear inside the frame, then no need to redraw
        clear_rectangle(x_loc-8, y_loc, 322, 280);
        
        // clear page
        //clear_menupage( cs_menu_page );
        // draw frame for list
        //draw_frame( x_loc-11, y_loc - 7, 332  , 290, 3,
        //           highlight);             

        // write full list
        int j;
        int top_idx;
        for( j = 0; j < lines_per_page; j++)
        {
          top_idx = list_top_item + j;
          if( top_idx > lines_per_page-1) top_idx -= lines_per_page;
          // if not blank write data
          if( Scroll_list[top_idx].has_data )
            {
              // load screen buffer
              for (i = 0; i < 16; i++)
              {
                cg_line_buffer[i] = Scroll_list[top_idx].text[i];
              }            
              // display text
              ssd1963_cg_write_line(16, 4, x_loc, y_loc + (30 * j) , 4);
            }          
        }
        // redraw highlight at bottom because page was cleared
        // draw new selection frame 
        draw_frame( x_loc-8, y_loc + (30 * (lines_per_page-1)) - 5, 320  , 34, 3,
                 highlight);      
      }
      else 
      {
      // ===============================          
      // down arrow at top or middle of list
        if(Scroll_list[menu_line_num].has_data)
        {
          // move highlight down
          // clear current frame
          draw_frame( x_loc-8, y_loc + (30 * menu_line_num )-5, 320  , 34,
                   3, cg_background );      
          // move current down
          menu_line_num++;
        }
        // draw new selection frame 
        draw_frame( x_loc-8, y_loc + (30 * menu_line_num) - 5, 320  , 34, 3,
                 highlight);   
         // if blank store data
        if( Scroll_list[menu_line_num].has_data == 0 )
        {
          //ssd1963_cg_write_line(txt_len, 4, x_loc,
          //                       y_loc + (30 * menu_line_num) , 4);
          // put new data into list
          // copy and pad with spaces
          for (i = 0; i < 16; i++)
          {
            if( i < txt_len) Scroll_list[ menu_line_num ].text[i] = cg_line_buffer[i];
            else Scroll_list[menu_line_num].text[i] = cg_line_buffer[i] = ' ';
          }
        
          Scroll_list[menu_line_num].has_data = 1; 
        }
        // whether it has data or is new send to screen anyway 
        ssd1963_cg_write_line(txt_len, 4, x_loc,
                                y_loc + (30 * menu_line_num) , 4);          
      } 
    } 
  }
}
// =============================================================================
//
//  Clear a submenu page
//
// height 305 = 410 top of button icon zone - 105 height of tabs and title
//       - 5 x and y to cover selection frame

void clear_menupage( uint8_t menupage )
{
  int start_x;
  start_x = screen_tab[menupage].x_loc-11;
  if( start_x < 0)start_x = 0;

  clear_rectangle( start_x , 97, 800-start_x, 305);
}

// =============================================================================
//
//  On Air transmit indicator
//
// History
//     05/28/2019 added table for x,y location on different screens

// location of on air for each screen
// xloc, yloc, width, height  (not using width and height here) 

box_spec onair_loc[4] = {
{ 580, 125, 100, 100 },   // main page
{ 580, 200, 100, 100 },   // enhanced entry
{ 580,  10, 100, 100 },   // vfo mode
{ 580, 125, 100, 100 },   // enhanced scanning  
};

void draw_TXonoff( uint8_t onoff )
{
  uint8_t cur_foreg;
  uint8_t cur_backg;
  uint16_t cur_x;
  uint16_t cur_y;
  uint8_t mpage = 0;

  uint8_t loc_indx = 0;  // default is main screen
  // get x,y for screen
  mpage = get_menu_stack(); 
  if( mpage == ECHAN ) loc_indx = 1;
  else if ( mpage == VFOMODE ) loc_indx = 2;  
  else if ( mpage == ENHSCAN ) loc_indx = 3;
  
  cur_x = onair_loc[loc_indx].x_loc;
  cur_y = onair_loc[loc_indx].y_loc;
  
  cur_foreg = cur_foreground;
  cur_backg = cur_background;
  
    if(onoff)
    {
      // only show if radio is communicating 
      if (radio_watchdog_timer > 0)
      {
        // font colors 
        set_font_colors( 0, 4 );    // white on red
        // paint alert area
        clear_rectangle( cur_x - 15, cur_y - 5, 205 , 60 );    

        strncpy( (char * ) cg_line_buffer, "ON AIR", 6);
        ssd1963_cg_write_line( 6, 2, cur_x, cur_y, 3);
        // restore font colors 
        set_font_colors( cur_foreg, cur_backg );
        log_event( "transmit" );
        radio_transmiting = 1;
      }
    }
    // if it is off always clear
    else
    {
      radio_transmiting = 0;
      ptt_timer = 0;

      // only on main screen
      if( active_screen_num == 1 )
      {
        // clear message area
        clear_rectangle( cur_x - 15, cur_y - 5, 205 , 60 );          
      }
      
      if (check_channel_bits(3))
      { 
        display_zone_alias();        
      }  
      
    }
}
// =============================================================================
//
//  Receiving indicator
//
// History 12/05/2019  written
//     
//  uses the same locations as On Air

void draw_RXonoff( uint8_t onoff )
{
  uint8_t cur_foreg;
  uint8_t cur_backg;
  uint16_t cur_x;
  uint16_t cur_y;
  uint8_t mpage = 0;

  uint8_t loc_indx = 0;  // default is main screen
  // get x,y for screen
  mpage = get_menu_stack(); 
  if( mpage == ECHAN ) loc_indx = 1;
  else if ( mpage == VFOMODE ) loc_indx = 2;  
  else if ( mpage == ENHSCAN ) loc_indx = 3;
  
  cur_x = onair_loc[loc_indx].x_loc;
  cur_y = onair_loc[loc_indx].y_loc;
  
  cur_foreg = cur_foreground;
  cur_backg = cur_background;
  
    if(onoff)
    {
      // only show if radio is communicating and on an allowed screen
      if (radio_watchdog_timer > 0)
      {
        // eliminate default (when not really on main screen)
        if(( loc_indx == 0 && active_screen_num == 1 ) || ( loc_indx > 0 ))
        {
          // font colors 
          set_font_colors( 0, 7 );    // white on green
          // paint alert area
          clear_rectangle( cur_x - 15, cur_y - 5, 205 , 60 );    

          strncpy( (char * ) cg_line_buffer, "Receive", 7);
          ssd1963_cg_write_line( 7, 2, cur_x, cur_y, 3);
          // restore font colors 
          set_font_colors( cur_foreg, cur_backg );
          // log_event( "receiving" );
          radio_receiving = 1;
        }
      }
    }
    // if it is off always clear
    else
    {
      radio_receiving = 0;
      
      // only clear on allowed screens
      if(( loc_indx == 0 && active_screen_num == 1 ) || ( loc_indx > 0 ))
        {
          // don't clear if transmitting
          if( !radio_transmiting )
          {
            // clear message area
            clear_rectangle( cur_x - 15, cur_y - 5, 205 , 60 );    
          }
        }

      // only on main screen
      if( active_screen_num == 1 )
      {     
        if (check_channel_bits(3))
        { 
          display_zone_alias();        
        }  
      }
    }
}
// =============================================================================
//
//   Display power on message, and "powering down" on shut down
//
void display_POM_message( uint8_t msg_num, uint8_t msg_length )
{
  uint8_t font;
  
  font = 2;
  // check pixel width, decrease font size if it doesn't fit
  while ( pixel_line_width( (char *) cg_line_buffer, font, 4 ) > 370 )
  {
    font++;
    if(font > 4)
    {
      font = 4;
      error_log( 3061 );
      cg_line_buffer[12] = '\0';
      break;
    }
  }
  
  // write message1
  if(msg_num == 1)
  {
    // clear whole radio box with msg 1
    clear_rectangle( 15, 105, 768 , 83);  

    // cg_buffer should already be loaded with msg 1
    ssd1963_cg_write_line( msg_length, font, 40 , 125, 4);
  }
  // write message 2
  if(msg_num == 2)
  {
    // cg_buffer should already be loaded with msg 2
    ssd1963_cg_write_line( msg_length, font, 410 , 125, 4);    
  } 
  // set message flags, if they're on, won't display.
  wait_for_radio_onscreen = 1;
  radio_connected_onscreen = 1;
}
// =============================================================================
//
//   Display call icon 
//
uint8_t prev_call_type = 0;

void draw_call_icon( uint8_t call_type )
{
  // clear icon area
  clear_rectangle(364 , 15, 72, 72);  
  if(prev_call_type != call_type || screen_refresh)
  {
    // if the call type is not these it will be blanked by the clear  
    if(call_type == 0) draw_icon( ICON_PRIVATE, 376, 15,
                           cur_foreground, cur_background );
    if(call_type == 1) draw_icon( ICON_GROUP, 364, 15,
                           cur_foreground, cur_background );
    if(call_type == 2) draw_icon_rle( ICON_SMILEY_MC, 364, 15 );
    
    prev_call_type = call_type;
  }
  /*  Radio info problem, what is the smiley icon for really?
          It's probably not all call     
  if(call_type > 1)
  {
    // all call doesn't have an icon yet 
    // log_event( "all call" );  // log turned off, came through too many times
  }
  */
}
// =============================================================================
//
//   popup list for the last call table
//
// History 
//  08/03/2018 wrote generalized popup, modified this to call it  

uint8_t lc_poped_up = 0;
uint8_t list_hasdata = 0;

void popup_lastcall_list( void )
{
  int i;
  list_hasdata = 0;
  // copy last call list data to pop up list
  for(i = 0; i < last_call_recs; i++)
  {
      strncpy( pop_up_list[i].text, Last_call_table[i].text , 16);
      pop_up_list[i].has_data = Last_call_table[i].has_data;
      if(pop_up_list[i].has_data == 1) list_hasdata = 1;
  }
  
  popup_list( lc_poped_up, last_call_recs, last_call_indx,
       30, 100, 280, 270);
  
  lc_poped_up ^= 1;
  // redisplay rx freq if 
  display_channel_rx_frequency();
  // display icon for touch
  if(lc_poped_up) draw_icon_rle( ICON_LASTCALL_DOWN_MC, 300, 425 ); 
  else 
  {
      if(list_hasdata) draw_icon_rle( ICON_LASTCALL_UP_MC, 300, 425 ); 
    else clear_rectangle(300, 425, 40, 48);  
  }
  log_event( "popup lastcall" );
}

// separated icon display so that the up icon can be displayed when the first
// contact comes in, without popping up the box 
// keeping all icon calls in screens.c 

void display_lastcall_icon( void )
{
  // display icon for touch
  draw_icon_rle( ICON_LASTCALL_UP_MC, 300, 425 );   
}


// =============================================================================
// August 3, 2018  		Design Documents                     pop_up_list
//
//    Draw, clear a list box on the screen
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  08/03/2018 generalized the last call popup  
//
// Status
//    written, working

List_text pop_up_list[20];

void popup_list( uint8_t poped_up, uint8_t numrecs, uint8_t list_indx,
       uint16_t pop_x, uint16_t pop_y, uint16_t pop_width, uint16_t pop_height)
{
  int j;
  
  if( poped_up == 0 )
  {
    // not popped up, display
    // clear space
    clear_rectangle(pop_x , pop_y, pop_width, pop_height);
    // draw frame
    draw_shaded_frame( pop_x , pop_y, pop_width, pop_height, 5, border, 
                      highlight);    
   
    // set list pointer to the next one so the most recent will be at the bottom
    list_indx++;
    if(list_indx > numrecs-1) list_indx = 0;
    
    int list_top = 0;
    
    // write the list
    for( j = 0; j < numrecs; j++)
    {
      // only display if there is data
      if(pop_up_list[list_indx].has_data == 1)
      {
        // load the diaplay buffer
        strncpy( (char*)cg_line_buffer, (char const*) pop_up_list[list_indx].text, 16); 
        ssd1963_cg_write_line( 16, 4, pop_x + 15 , pop_y + 10 + ( 25 * list_top++ ), 3);
      }
      list_indx++;
      if(list_indx > numrecs-1) list_indx = 0;      
    }
  }
  else
  {
    // popped up, clear
    // if there's no data, don't display call
    if(pop_up_list[list_indx].has_data == 0) cur_call_active = 0;    
    // clear space
    clear_rectangle(pop_x , pop_y, pop_width, pop_height);
    display_screen( 1, 0 );
  }
}
// =============================================================================
// August 9, 2018  		Design Documents                       wide_list
//
//    Draw a menu box that uses multiple columns
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  08/09/2018 variant of popup list for wide menus 
//
// Status
//    writing

void wide_list( uint8_t numrecs, uint8_t list_indx,
       uint16_t loc_x, uint16_t loc_y, uint16_t width, uint16_t height)
{
  int j;
  int col_x;
  int list_top = 0;
   
  // clear space
  clear_rectangle(loc_x , loc_y, width, height);
  // draw frame
  draw_shaded_frame( loc_x , loc_y, width, height, 5,
                    border, 
                    highlight);    
  
  col_x = loc_x;  
  
  // write the list
  for( j = 1; j < numrecs + 1; j++)
  {
    if(list_top > submenu_max_lines -1) 
    {
      list_top = 0;
      col_x += 230;  
    }
    // only display if there is data
    if(Menu_list[list_indx].has_data == 1)
    {
      // load the diaplay buffer
      strncpy( (char*)cg_line_buffer, (char const*) Menu_list[list_indx].text, 16); 
      ssd1963_cg_write_line( 16, 4, col_x + 17 , loc_y + 10 + ( 25 * list_top++ ), 3);
    }
    list_indx++;
    if(list_indx > numrecs-1) list_indx = 0;      
  }
}
// =============================================================================
//
// blink red LED (tx)
//
// diagnostic 1
//
// History
//     09/11/2018 moved here from diagnostics
//

void blink_red ( unsigned int blinks )
{
  int static toggle;

  // using NOP micro second delay
  while( blinks-- > 0)
  {
    toggle ^= 1;
    if(toggle) tx_led_on();
    else tx_led_off();
    delay(50000);
  }  
}
//========================================================
//
// blink green LED (rx)
//
// diagnostic 2
//
// History
//     09/11/2018 moved here from diagnostics
//
void blink_green ( unsigned int blinks )
{
  int static toggle;
  while( blinks-- > 0)
  {
    toggle ^= 1;
    if(toggle) rx_led_on();
    else rx_led_off();
    delay(50000);
  }

}
//========================================================
//
// Alternate blink red green
//
//  Run from a timer. Switches color each time it's called
//
// History
//     09/11/2018 moved here from diagnostics
//
void blink_alternate( void )
{
  static int toggle = 0;

  toggle ^= 1;
  if(toggle)
  {
    tx_led_off(); 
    rx_led_on();
  }
  else
  {
    rx_led_off(); 
    tx_led_on();
  }
    
}
// =============================================================================
//
//   Display a color for adjustmebt
//
// color adjustment numbers for use by touch points
RGB_color adj_color;

uint16_t color_adj_x;
uint16_t color_adj_y;

void display_color(  uint16_t startX, uint16_t startY, RGB_color c_color )
{
  strncpy( (char*) cg_line_buffer, "red", 3);  
  ssd1963_cg_write_line( 3, 4, startX, startY + 110, 2);
  sprintf((char*) cg_line_buffer, "%3u ", c_color.red);    
  ssd1963_cg_write_line( 3, 4, startX + 100, startY + 110, 2);
  
  strncpy( (char*) cg_line_buffer, "green", 5);  
  ssd1963_cg_write_line( 5, 4, startX, startY + 150, 2); 
  sprintf((char*) cg_line_buffer, "%3u ", c_color.green);
  ssd1963_cg_write_line( 3, 4, startX + 100, startY + 150, 2);    
  
  strncpy( (char*) cg_line_buffer, "blue", 4);  
  ssd1963_cg_write_line( 4, 4, startX, startY + 190, 2);  
  sprintf((char*) cg_line_buffer, "%3u ", c_color.blue);
  ssd1963_cg_write_line( 3, 4, startX + 100, startY + 190, 2);

  // adjusment icons
  draw_icon( ICON_LEFT_ADJ, startX + 75 , startY + 106, 10, cur_background );
  draw_icon( ICON_LEFT_ADJ, startX + 75 , startY + 146, 10, cur_background );
  draw_icon( ICON_LEFT_ADJ, startX + 75 , startY + 186, 10, cur_background );
  draw_icon( ICON_RIGHT_ADJ, startX + 142 , startY + 106, 10, cur_background );
  draw_icon( ICON_RIGHT_ADJ, startX + 142 , startY + 146, 10, cur_background );
  draw_icon( ICON_RIGHT_ADJ, startX + 142 , startY + 186, 10, cur_background );
  // set touch point locations
  // red +
  CS800D_touch_point[12].xmin = startY + 106;  
  CS800D_touch_point[12].xmax = startY + 176;  
  CS800D_touch_point[12].ymin = startX + 142;
  CS800D_touch_point[12].ymax = startX + 172;
  // red -
  CS800D_touch_point[13].xmin = startY + 106;
  CS800D_touch_point[13].xmax = startY + 176;
  CS800D_touch_point[13].ymin = startX + 70;
  CS800D_touch_point[13].ymax = startX + 100;
  // green +
  CS800D_touch_point[14].xmin = startY + 146; 
  CS800D_touch_point[14].xmax = startY + 186;
  CS800D_touch_point[14].ymin = startX + 142;
  CS800D_touch_point[14].ymax = startX + 172;
  // green -
  CS800D_touch_point[15].xmin = startY + 146;
  CS800D_touch_point[15].xmax = startY + 186;
  CS800D_touch_point[15].ymin = startX + 70;
  CS800D_touch_point[15].ymax = startX + 100;
  // blue +
  CS800D_touch_point[16].xmin = startY + 186; 
  CS800D_touch_point[16].xmax = startY + 226;
  CS800D_touch_point[16].ymin = startX + 142;
  CS800D_touch_point[16].ymax = startX + 172;
  // blue -
  CS800D_touch_point[17].xmin = startY + 186;
  CS800D_touch_point[17].xmax = startY + 226;
  CS800D_touch_point[17].ymin = startX + 70;
  CS800D_touch_point[17].ymax = startX + 100;  
  
  
  // display the color
  draw_rectangle( startX + 70, startY, 100, 100, c_color );  
    
} 
// =============================================================================
//
//   Packet data dump
//    for debugging puts packet data to screen
//    then waits for keypress on right knob
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 07/26/2018
//         10/12/2018 moved to screens.c from process_CS800D
//
// Status
//  written, working
//
void display_packet_dump( uint8_t * pk_buffer )
{
  int i;
  int n;
  
  int numbytes = 0;
  int cur_screen = active_screen_num;
  int xloc = 20;
  int yloc = 20;
  
  active_screen_num = 0; // to stay off while waiting
  // turn off all touches
  all_touches_off();
  // except cancel
  CS800D_touch_point[7].is_active = 1;  // P1 cancel button
  // clear screen
  ssd1963_cg_clear_display();
  // dump packet in hex
  // get num of bytes
  numbytes = pk_buffer[2] + 4;
  
  for(i = 0; i < numbytes; i++)
  {
    n = sprintf((char*) cg_line_buffer, "%02X ", pk_buffer[i]);
    ssd1963_cg_write_line(n, 4, xloc , yloc, 3);
    xloc += 60;     // next byte
    if( xloc > 600 )  
    {
      xloc =  20;
      yloc += 24;   // next row
    }
  }
  // display cancel icon only
  draw_cancel_button();
  // activate only the cancel button touch
  CS800D_touch_point[7].is_active = 1;  // P1
  
  // hang here and wait for a press of the right encoder switch
  while((last_key_press != 0x15) && (read_switch_s2() == 0));
  //while( read_switch_s2() != 0 );
    
  // back to last screen
  display_screen( cur_screen , 1);
}
// =============================================================================
//
//   packet sniffing data dump
//    for debugging puts packet data to screen
//    then waits for keypress on right knob
//
//    call with pointer to UART1_rx_buffer to view rx buffer directly
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 05/15/2019 revised version for FT857D packet sniffing
//
// Status
//  written
//

// only for FT857
#ifdef radio_FT857D

void display_rxdata_dump( uint8_t* rx_buffer )
{
  int i;
  int n;
  
  int numbytes = 0;
  int xloc = 5;
  int yloc = 20;
  
  active_screen_num = 0; // to stay off while waiting
  // clear screen
  ssd1963_cg_clear_display();
  // dump packet in hex
  // use whole rx buffer
  numbytes = 256;
  
  for(i = 0; i < numbytes; i++)
  {
    n = sprintf((char*) cg_line_buffer, "%02X ", rx_buffer[i]);
    ssd1963_cg_write_line(n, 4, xloc , yloc, 3);
    xloc += 50;     // next byte
    if( xloc > 770 )  
    {
      xloc =  5;
      yloc += 24;   // next row
    }
  }
  // hang here and wait for a press of p2
  while(read_switch_s4() == 0);
    
  // back to last screen
  FT857_display_screen( 1, 1 );
}
#endif /* FT857D */

// =============================================================================
// Nov 7, 2018  		Design Documents               display_dmr_color
//
//     Displays the color code of a DMR communication
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  color code is passed as an integer
//
// Exit Conditions
//
// External functions called
//  sprintf
//  clear_rectangle
//  ssd1963_cg_write_line
//
// History 
//  11/07/2018 written for MS20.41
//
// Status
//  writing
//

int prev_dmr_color;

void display_dmr_color( int color_code, uint16_t pos_x, uint16_t pos_y, uint8_t font)
{
  int n;

  // always display
  // if it's different
  // if(color_code != prev_dmr_color)
  // {
    // make text line
    strncpy((char*) cg_line_buffer, "CC",2);
    // convert int to string
    n = sprintf((char*) cg_line_buffer+2, "%2u", color_code); 
    // erase 
    clear_rectangle( pos_x, pos_y, 140, 20 );
    // write new
    ssd1963_cg_write_line(2 + n , font, pos_x, pos_y, 3);
    prev_dmr_color = color_code;
  // }
}                         
// =============================================================================
// Nov 7, 2018  		Design Documents                display_dmr_slot
//
//     Displays the slot number of a DMR communication
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  slot number is passed as an integer
//
// Exit Conditions
//
// External functions called
//  sprintf
//  clear_rectangle
//  ssd1963_cg_write_line
//
// History 
//  11/07/2018 written for MS20.41
//
// Status
//  writing
//

int prev_dmr_slot;

void display_dmr_slot( int slot_num, uint16_t pos_x, uint16_t pos_y, uint8_t font)
{
  int n;

  // if it's differeent
  // if(slot_num != prev_dmr_slot)
  //{
    // make text line
    strncpy((char*) cg_line_buffer, "Slot ",5);
    // convert int to string
    n = sprintf((char*) cg_line_buffer+5, "%1u", slot_num);  
      // erase 
    clear_rectangle( pos_x, pos_y, 60, 20 );
    // write new
    ssd1963_cg_write_line(5 + n , font, pos_x, pos_y,3);
    prev_dmr_slot = slot_num;
  // }
}                         
// =============================================================================
//
//   Pop up channel selector
//
//   display a channel list for direct jump to a channel

// History
//      11/13/2018 writing
//
// Status: unfinished, needs touch controls

void display_chan_selector( uint8_t current_chan, uint16_t pos_x, uint16_t pos_y )
{
  int i;
  int n;
  // clear popup area
  clear_rectangle( pos_x, pos_y-5, 800, 60 );
  // display number boxes
  for( i = 0; i < 16; i++)
  {
    if(i == current_chan)
    {
      // highlight current
      draw_frame(pos_x + (50*i), pos_y, 50, 50, 4,
                  highlight);        
    }
    else
    {
      // frame
      draw_frame(pos_x + (50*i), pos_y, 50, 50, 4,
                    border);
    }
    // number
    n = sprintf((char*) cg_line_buffer, "%2u", i+1);
    if(i < 10) ssd1963_cg_write_line( n , 3, pos_x + 5 + (50*i), pos_y + 5 ,3);
    else ssd1963_cg_write_line( n , 3, pos_x + 10 + (50*i), pos_y + 5 ,3);
  }
  
  // set touch point data
  
  // deactivate other touches
  
  // activate touch points
 
}

// =============================================================================
//
//  display waiting for radio message, or clear the message if connected
//
// History: 12/04/2018 written
//          12/21/2018 split waiting and connected to separate routines
//
// Status: working.

int wait_for_radio_onscreen = 0;
int radio_connected_onscreen = 0;

void display_radio_wait( uint8_t font, uint16_t pos_x, uint16_t pos_y, uint8_t spacing)
{
  if(wait_for_radio_onscreen == 0)
  {
    if( active_screen_num == 1 || active_screen_num == 5 ) 
    {
      // clear 
      clear_rectangle(  pos_x , pos_y, 700, 55 );      
      strncpy( (char*) cg_line_buffer, "Waiting for radio...", 20);        
      ssd1963_cg_write_line( 20, font, pos_x , pos_y, spacing);
      wait_for_radio_onscreen = 1;   // waiting, displayed 
      radio_connected_onscreen = 0;
    }
  }
}
// =============================================================================
//
//  display connected to radio message, or clear the message
//
// History: 12/04/2018 written
//          12/21/2018 split waiting and connected to separate routines
//
// Status: written, working

void display_radio_connected( uint8_t font, uint16_t pos_x,
                             uint16_t pos_y, uint8_t spacing)
{
  // only show if it's off and waiting for radio is on
  if(radio_connected_onscreen == 0 && wait_for_radio_onscreen == 1)
  {
    if( active_screen_num == 1 || active_screen_num == 5 )
    {
      // is displayed, clear message
      clear_rectangle(  pos_x , pos_y, 730, 55 );
      // block display for first packets so radio Id can stay displayed
      if( packet_counter > 2 )
      {
        strncpy( (char*) cg_line_buffer, "Connected", 9);        
        ssd1963_cg_write_line( 9, font, pos_x , pos_y, spacing);
        radio_connected_onscreen = 1;
      }
    }
  }
}
// =============================================================================
//
//  display heartbeat
//
//
// History: 12/06/2018 written
//
// Status: working. Customers made positive comments.

void display_heartbeat( uint16_t pos_x, uint16_t pos_y, uint8_t count )
{
  // clear previous
  clear_rectangle(  pos_x , pos_y, 80, 10 );
  // show heartbeat box
  draw_rectangle( pos_x + (10 * count), pos_y, 10, 10,
                 current_palette[get_pal_color( 9 )] );
}

// =============================================================================
//
//  Display message, wait 3 sec, clear, return
// 
//
// History: 12/12/2018 written
//

void display_message( char* msg, uint16_t pos_x, uint16_t pos_y, uint8_t font )
{
  unsigned int f_width;

  f_width = pixel_line_width( msg, font, 3 ) + 5;
  // draw a frame    
  draw_frame( pos_x - 5, pos_y - 5, f_width  , cg_max_vert_size + 10,
              3, highlight); 
    
  // display the message
  strncpy( (char*) cg_line_buffer, msg, 40);
  ssd1963_cg_write_line( strlen(msg), font, pos_x , pos_y, 2);
  // wait 3 seconds
  msec_delay( 3000 );
  // clear the message
  clear_rectangle(  pos_x - 5 , pos_y - 5, f_width, cg_max_vert_size + 10 );
  
}
// =============================================================================
//
//  Display RSSI graph
//    uses new MS20.53 firmware rssi info packet
//    samples once per second. call placed in one second timer section of systick
//    180 samples gives 3 minutes of history
//
//    Radio's returned rssi adjusted by ((rssi_value - 45) ^2 ) / 150
//      to fit in zone window.
//
// History: 01/24/2019 written
//

uint8_t rssi_graph_on = 0;
uint16_t graph_xpos = 615;
uint16_t graph_ypos = 10;

// in the zone window
//uint16_t graph_xpos = 600;
//uint16_t graph_ypos = 110;

void display_RSSI_graph( void )
{
  int i;
  uint8_t bar_size;
  uint8_t bar_index;
  
  // bar index starts at oldest data
  bar_index = RSSI_hist_index;

  // display history
  for( i = 0; i < 180; i++)
  {
    // ( old version ) squared and scaled
    // bar_size =  RSSI_history[ bar_index ] * RSSI_history[ bar_index ]  / 160;
    // graph scale only calc'd when rssi range changes 
    bar_size = ( uint8_t ) ( RSSI_history[ bar_index ] * RSSI_graph_scale );

    if(bar_size > 75 ) bar_size = 75;  // limit bad data
    bar_index++;
    if(bar_index > 179 ) bar_index = 0;
    // clear top
    clear_rectangle( graph_xpos + i, graph_ypos, 1, 75 - bar_size);
    // display bar
    draw_rectangle( graph_xpos + i, graph_ypos + 75 - bar_size, 1, bar_size,
                   current_palette[get_pal_color( 8 )] );
  }  
}


