//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 29, 2018             Design Documents                           CSI_spi.c
//
//               Routines for handling SPI  1 peripheral
//                  For large panel project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  03/29/2018 copied from CS7000 prototype for STM32F7
//  05/23/2018 major modifications while testing for STM32F405
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  spi1_init                   initialize SPI 1
//  spi1_check_error            reads status register for errors
//  spi1_write_bytes            write bytes to spi 1
//  spi1_enable                 turn on spi module
//  spi1_csn                    set chip select line
//  SPI1_tx_buffer_add          put data into spi buffer, update index
//  SPI1_Handler                interrupt handler
//  spi1_interrupts             turn on or off spi interrupts
//
// Status: written, not used
//
//  SPI1_change_dff             change data frame format
//  spi1_waitfor_rx             waits with timeout for rx data
//  spi1_sleep                  set to low power mode
//  spi_rxtx                    set spi rx/tx mode
//==============================================================================
//          SPI for flash on front panel project
//  GPIO configuration for Flash
//          
//  FLASH CS      gpio        A   4 (flash CS controlled by GPIO, not SPI1 NSS)
//  FLASH SCLK    SPI1 SCK    A   5
//  FLASH DO      SPI1 MISO   A   6
//  FLASH DI      SPI1 MOSI   A   7
//
//==============================================================================
#include <stdint.h>
#include <assert.h>
#include "CSI_spi.h"
#include "CSI_GPIO.h"
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_interrupt.h"
#include "CSI_ERROR_LOG.h"

extern uint32_t *RCC_APB2ENR; 

// make pointers to the registers
// SPI 1
SPI_CR1_reg *SPI1_CR1 = (SPI_CR1_reg *) SPI1_CR1_ADDRESS;
SPI_CR2_reg *SPI1_CR2 = (SPI_CR2_reg *) SPI1_CR2_ADDRESS;
uint16_t *SPI1_CRCPR  = (uint16_t *) SPI1_CRCPR_ADDRESS;
// different sized pointers to the same data register
uint8_t  volatile *SPI1_DR8    = (uint8_t *) SPI1_DR_ADDRESS;
uint16_t volatile *SPI1_DR16   = (uint16_t *) SPI1_DR_ADDRESS;

SPI_SR_reg volatile *SPI1_SR   = (SPI_SR_reg *) SPI1_SR_ADDRESS;

// make global variables with the structures
SPI_CR1_reg  SPI1_CR1_temp;
SPI_CR2_reg  SPI1_CR2_temp;
SPI_SR_reg   SPI1_SR_temp;

// set up rolling buffers for read and write
// write data at the head, this transmits data from the tail until the tail
// catches up to the head
// this receives data and places it at the head, read data from the tail until
// what's read catches up to the head

uint8_t spi_receive_buffer[256];
uint8_t rx_bufr_head = 0;
uint8_t rx_bufr_tail = 0;

uint8_t spi_transmit_buffer[256];
uint8_t tx_bufr_head = 0;
uint8_t tx_bufr_tail = 0;

int volatile spi1_num_bytes_to_rx = 0;
int volatile spi1_num_bytes_to_tx = 0;

//******************************************************************************
//
// Mar 10, 2016			Design Documents                       spi1_init
// 
//   Initialize SPI 1 for FLASH
//
//   SPI configuration:
//   Set for full duplex.
//   Commands need to be sent and data sent and received.
//   Set for 8 bit only, most commands are 8 bit and the module needs to 
//     be disabled to switch between 8 bit and 16  
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//   SPI peripheral ready for writing or reading
// Functions used
//  nvic_enableIRQ() 
//
// History
//  03/10/2016 written
//  03/14/2016 documentation
//  03/29/2016 work stopped, switched to bit-bang routines
//  03/29/2018 revised for large display project
//
// Status
//  written, tested, working
//
//******************************************************************************

int spi1_init( void )
{
  spi1_csn( 1 );
  
  uint32_t volatile *RCC_APB2ENR = (uint32_t *) RCC_APB2ENR_ADDRESS;
  // turn on spi 1 clock, apb2 clock is default sysclk/4 42 Mhz
  *RCC_APB2ENR |= RCC_APB2ENR_SPI1EN;
  
  // enable interrupt for SPI1 interrupt #35
  nvic_enableIRQ( SPI1_INTR );
  
  // disable spi to set controls
  SPI1_CR1->spe = 0; 
  
  // set up SPI1_CR1_temp 
  SPI1_CR1_temp.rate     = 4; // peripheral clock 2=  / 8  = 6.75 Mhz 3= /16
                              //  4= /32 1.3125 Mhz 6= /128 328,125 Khz
  SPI1_CR1_temp.mstr     = 1; // master 
  SPI1_CR1_temp.cpha     = 0; // clock phase
  SPI1_CR1_temp.cpol     = 0; // clock polarity
  SPI1_CR1_temp.rxonly   = 0; // must be clear to transmit
  SPI1_CR1_temp.lsbfirst = 0; // Send MSB first
  SPI1_CR1_temp.ssm      = 0; // (1 software) (0 hardware) controls slave select
  SPI1_CR1_temp.ssi      = 0;
  SPI1_CR1_temp.dff      = 0; // 8 bit data frame
  SPI1_CR1_temp.crcen    = 0; // leave CRC off 
  SPI1_CR1_temp.bidioe   = 0; // not transmit only 
  SPI1_CR1_temp.bidimode = 0; // bi directional
  
  // put setup data into register
  *SPI1_CR1 = SPI1_CR1_temp;
  
  // set up SPI1_CR2_temp 
  SPI1_CR2_temp.ssoe     = 1; // ss output as master 
  SPI1_CR2_temp.frf      = 0; // frame format is motorola 
  SPI1_CR2_temp.rxneie   = 1; // enable rx interrupt
  SPI1_CR2_temp.txeie    = 0; // disable tx interrupt, must start with it off
  SPI1_CR2_temp.errie    = 1; // enable err interrupt
    
  // put setup data into register
  *SPI1_CR2 = SPI1_CR2_temp;
  
  // other SPI 1 registers can be left at default reset settings 
  
  spi1_num_bytes_to_rx = 0;
  spi1_num_bytes_to_tx = 0;
  rx_bufr_head = 0;
  rx_bufr_tail = 0;
  tx_bufr_head = 0;
  tx_bufr_tail = 0;

  // only enable spi before transmitting 
  //SPI1_CR1->spe = 1;
  //spi1_csn( 1 );
    
  return(1);
}
// =============================================================================
//
// May 23, 2018			Design Documents                 SPI1_change_dff
//
//              Change data frame format
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   to change data frame format SPI umust be disabled
//   dff set to 0 is 8 bit, 1 is 16 bit
//
// History
//  05/23/2018 written
// Status
//  written, working, but unneeded, using 8 bit only
//
//******************************************************************************
//
//  change data frame format
//
 
void SPI1_change_dff ( uint8_t dff )
{
  // be sure it's not busy
  while(SPI1_SR->bsy);

  // disable  
  SPI1_CR1->spe = 0;
  // change 
  SPI1_CR1->dff = dff;
  // enable  
  SPI1_CR1->spe = 1;    
}
  

//******************************************************************************
//
// Mar 14, 2016			Design Documents                 check_spi_error
// 
//   Check SPI status for error conditions 
//
// Algorithms 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//   returns 1 without an error returns 1 and error is logged if error occurred
// Functions used
//   error_log()
// External variables used
//  *SPI1_SR
//  SPI1_SR_temp - status register structure variable
// Internal variables used
//    err
// Constants
//
// History
//  03/14/2016 written
// Status
//  written, tested, working
//
//******************************************************************************
//
//  Check for SPI errors
//    log error if it occurs
//
int spi1_check_error( void )
{
  int err = 0;

  // read status register
  SPI1_SR_temp = *SPI1_SR;
  // test for any error
  if( SPI1_SR_temp.udr |
      SPI1_SR_temp.ovr |
      SPI1_SR_temp.crcerr |
      SPI1_SR_temp.modf |
      SPI1_SR_temp.fre )
  {
    err = 1;
    // check and log errors
    // udr, CRC, Mode, odr, fre
    if(SPI1_SR_temp.udr)    error_log( 1601 );
    if(SPI1_SR_temp.ovr)    error_log( 1602 );     
    if(SPI1_SR_temp.crcerr) error_log( 1603 );
    if(SPI1_SR_temp.modf)   error_log( 1604 );
    if(SPI1_SR_temp.fre)    error_log( 1605 );
  }
  return (err);
  
}
//******************************************************************************
//
// Mar 14, 2016			Design Documents                 spi1_waitfor_rx
//
//    Wait for rx buffer to have data
//
// Algorithms 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//   data read returned
// Functions used
//   spi_check_error()
//   error_log()
// External variables used
//  SPI1_CR1 - spi control register 1
//
// History
//  03/14/2016 written
//  03/30/2018 added wait for data with timeout and error
// Status
//  written, working but not being used
//
//******************************************************************************

#define spi_rxtx_WAIT   100000
//==============================================================================
//
//  wait with timeout for receiving data
//   returns 1 if data is available
//           0 if no data has been received within timeout
//
int spi1_waitfor_rx( unsigned int timeout )
{
  unsigned int recdata_timeout = timeout;
  while(rx_bufr_tail == rx_bufr_head)
  {
    if(--recdata_timeout == 0) error_log( 1606 );
    return (0);
  }
  return (1);  
}
//******************************************************************************
//
// Mar 14, 2016			Design Documents                spi1_write_bytes
//
//    Write to SPI1 8 bit, multiple bytes
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  Transmit buffer is loaded with commands and dummy bytes
//
// Exit Conditions
//   data is sent to SPI1 
// Functions used
//   spi_check_error())
// External variables used
//
// History
//  03/14/2016 written 
//  03/30/2018 added wait for data with timeout and error
// Status
//  written, tested, working

// =============================================================================
//
//  Write SPI1 multiple bytes
//    TX buffer must be pre-loaded with bytes to be sent
//    start transmission by sending the first byte in the buffer
//    Full duplex, the number of writes = the number of reads
//
// Status
//  written, tested, working

int spi1_write_bytes( void )
{
  // check for error before writing
  if (spi1_check_error()) return(0) ;
  
  spi1_csn( 0 );
 
  spi1_enable( 1 );
  // transmit begins as soon as data is written so decrement the counter
  // before the data is sent
  spi1_num_bytes_to_tx--;
  // write to SPI 1 data register
  *SPI1_DR8 = spi_transmit_buffer[tx_bufr_tail++];

  // turn on transmit interrupt AFTER writing data, otherwise
  // the data register is empty and will cause an immediate interrupt
  SPI1_CR2->txeie = 1; 
  
  return(1);
}
// *****************************************************************************
//
//  Set SPI1 NSS line (CSN for FLASH) 
//     direct hardware control of cs
//     Low = 0 or high = 1 
//
//  Use A4 as gpio port (NSS not controlled by spi module)
// Status
//  written, tested, working

void spi1_csn( int hilo )
{
  
  // using gpio control of csn
  if(hilo) *GPIO_A_BSRR = BIT04_ON;
  else *GPIO_A_BSRR = BIT04_OFF; 
 
}

// *****************************************************************************
// 
//  Enable or Disable SPI1
//
//  Disabling SPI does not change the module settings
//  It can be re-enabled when communication is resumed
//
// Status
//  written, tested, working
//
void spi1_enable( int onoff )
{
  if( onoff ) SPI1_CR1->spe = 1;
  else SPI1_CR1->spe = 0;
}


//******************************************************************************
//
// Mar 16, 2016			Design Documents                      spi1_sleep
//                                                                  
//   Turn off SPI 1 interface 
//
// Algorithms 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//   
// External variables used
//  RCC_APB2ENR
//  SPI1_SR register structure
//  SPI1_CR1 register structure
// Internal variables used
//    
// Constants
//  RCC_APB2ENR_SPI1EN
// History
//  03/16/2016 written
//  04/05/2016 documentation
// Status
//  written, untested
//
//******************************************************************************
//
// Put spi 1 peripheral into low power sleep
//
//
int spi1_sleep( void )
{
  //wait for not busy
  while (SPI1_SR->bsy);
  
  // disable SPI 
  SPI1_CR1->spe = 0;
  
  // turn off spi clock
  *RCC_APB2ENR &= ~RCC_APB2ENR_SPI1EN;
  
  return(1);
}

// *****************************************************************************
//
//   set spi for receive only, tx only, or full duplex
//     when set to ( 0 )
//     spi becomes transmit only
//     when onoff set to ( 1 )  
//     turns off spi transmitter, sends clock
//     spi becomes receive only 
//     when set to ( 2 )
//     Spi is full duplex.
//
// !!!    This can't be used in the middle of communication.
//        bidimode and bidioe should be set before enabling SPI
// Status
//  written, tested, working but not being used

void spi_rxtx( int rxtx )
{
  switch( rxtx )  
  {
  case 0: // transmit only
    {
      SPI1_CR1->bidimode = 1;
      SPI1_CR1->bidioe = 1;
      break;
    }
  case 1:  // receive only 
      {
      SPI1_CR1->bidimode = 1;
      SPI1_CR1->bidioe = 0;
      break;
      }
  case 2:  //full duplex, both tx and rx
    {
      SPI1_CR1->bidimode = 0;
      SPI1_CR1->bidioe = 0;      
      SPI1_CR1->rxonly = 0;
      break;  
    }
  default: error_log( 3001 ); break;
  }
}
//==============================================================================
//
//  put data into tx buffer
//   data goes in at the head
//   data is taken out from the tail
//   the buffer is 256 bytes so the 8 bit index wraps without adjustment
//
// Status
//  written, tested, working
//
void SPI1_tx_buffer_add( uint8_t data )
{
  spi_transmit_buffer[ tx_bufr_head++ ] = data;
  // report buffer error
  if( tx_bufr_tail == tx_bufr_head )
  {
    error_log( 3023 );
  }
  
}

//==============================================================================
//
//  SPI1 interrupt handler
//    interrupt #35
//    overrides __weak definition in startupCSI

// the transmit and receive buffers are 8 bit 
// this ISR only handles 8 bit now
// 
// Status
//  written, tested, working

// flag for ISR to send dummy tx for continuous reads
uint8_t spi_repeat_tx = 0;

void SPI1_Handler( void )
{
   uint8_t temp;
   
   // doesn't work properly when head has wrapped around 
   //if( tx_bufr_tail > tx_bufr_head) error_log( 3023 ); 
   
   spi1_interrupts( 0 );
   // read status register to determine what caused the interrupt
   SPI1_SR_temp = *SPI1_SR;
   
   // ===========================
   // a TXE flag, SPI data buffer is empty, has space for more data to send
   if(SPI1_SR_temp.txe)
   {
     // if there's no more data in the buffer to send
     if(spi1_num_bytes_to_tx == 0)
     {
       // send dummy data while receiving
       if ( spi_repeat_tx )
       {
         *SPI1_DR8 = 0;
       }
       else
       {
         // if there is no more to receive, done
         if( spi1_num_bytes_to_rx == 0 )
         {
           while(SPI1_SR->bsy);
           spi1_csn( 1 );
           SPI1_CR2->txeie = 0;  // turn off TX interrupt               
           spi1_enable( 0 );  // turn off SPI1
         }
       }
     }
     else // there is more data in the buffer to send 
     {
         *SPI1_DR8 = spi_transmit_buffer[ tx_bufr_tail++ ];
         spi1_num_bytes_to_tx--;
     }
   }
   // ===========================
   // a RXNE flag, receiver buffer has data
   if(SPI1_SR_temp.rxne)
   {
     // always read to prevent an OVR error
     spi_receive_buffer[rx_bufr_head++] = *SPI1_DR8;
     spi1_num_bytes_to_rx--;
     // if tx was sending dummies, turn it off  n-1
     if(spi1_num_bytes_to_rx == 1) spi_repeat_tx = 0;
 
   }   
   // if there are no more bytes to read or write, stop
   if( (spi1_num_bytes_to_rx <= 0) && (spi1_num_bytes_to_tx <= 0) )
   {
     while(SPI1_SR->bsy);
     spi1_csn( 1 );
     SPI1_CR2->txeie = 0;  // turn off TX interrupt 
     spi1_enable( 0 );  // turn off SPI1
   }

   // ===========================   
   // handle any errors
   // under run
   if(SPI1_SR_temp.udr)    error_log( 1601 );
   // over run
   if(SPI1_SR_temp.ovr)
   {
     // read DR
     temp = *SPI1_DR8;
     if(temp); // to eliminate compiler warning
     // read status
     SPI1_SR_temp = *SPI1_SR;
     error_log( 1602 ); 
   }
   // CRC err  
   if(SPI1_SR_temp.crcerr) error_log( 1603 ); 
   // mode fault     
   if(SPI1_SR_temp.modf)   error_log( 1604 );
   // frame format err
   if(SPI1_SR_temp.fre)    error_log( 1605 );    

     spi1_interrupts( 1 );
 }

// =============================================================================
//                                                                    
// turn on or off spi interrupts
//
// Status
//  written, tested, working
//
void spi1_interrupts( int onoff )
{
  if (onoff)
  {  // 1 enable
    SPI1_CR2->rxneie   = 1; // enable rx interrupt
    SPI1_CR2->txeie    = 0; // disable tx interrupt !! leave off, control it 
    SPI1_CR2->errie    = 1; // enable err interrupt  
  }
  else
  {  // 0 disable
    SPI1_CR2->rxneie   = 0; // disable rx interrupt
    SPI1_CR2->txeie    = 0; // disable tx interrupt
    SPI1_CR2->errie    = 0; // disable err interrupt 
  }
}
