//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Feb 14, 2018 	    Design Document   CSI_SSD1963_CHARACTER_GENERATION.c 
// 
//       Routines for display text characters on the LCD screen
//                    for the large panel project
//
// Programmer
//  Jerry Wanger, Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  02/14/2018  started
//  01/28/2019  added courier fonts
//  02/13/2019  revised write_byte, screen clear, for faster screen display
//
//==============================================================================
// NAME OF ROUTINE                      Function
//
// Status: written, tested, working
//
// ssd1963_cg_initialize                Initialize the display
// ssd1963_cg_read_array                Reads data from character array
// ssd1963_cg_read_descriptor           Reads character description
// ssd1963_cg_read_char_size            Reads parameters for character type
// ssd1963_cg_write_byte                Writes 1 - 8 bits to the display
// ssd1963_cg_blank                     Writes 1-255 backgroud bits to display
// ssd1963_cg_line                      Writes a line to the display
// ssd1963_cg_write_progressive         Output progressive character to display
// ssd1963_cg_write_centered            Output centered character to display
// ssd1963_cg_location                  Set the controller chip to output a char
// ssd1963_cg_next_location             Points to location of next character
// ssd1963_cg_output_character          Output character to the display.
// ssd1963_cg_write_line                Output a line of characters to display
// ssd1963_cg_clear_line                Clear a line on the display
// ssd1963_cq_calculate_center          Calculate starting location for center

// Status: written, unused

// ssd1963_cq_write_horizontal_line     Write a single horizontal line
// ssd1963_cq_clear_horizontal_line     Clear a single horizontal line
// ssd1963_cq_display_center            Display message centered

// =============================================================================
#include <stdint.h>
#include "CSI_SSD1963.h"
#include "CSI_GPIO.h"
#include "CSI_TIMERS.h"
#include "CSI_Times_New_Roman_FONT12.h"
#include "CSI_Times_New_Roman_FONT16.h"
#include "CSI_Times_New_Roman_FONT24.h"
#include "CSI_Times_New_Roman_FONT36.h"
#include "CSI_Times_New_Roman_FONT48.h"
#include "CSI_Times_New_Roman_FONT72.h"
#include "CSI_Courier_New_Font16.h"
#include "CSI_Courier_New_Font20.h"
#include "CSI_Courier_New_Font24.h"
#include "CSI_Courier_New_Font28.h"
#include "CSI_Courier_New_Font36.h"
#include "CSI_Courier_New_Font48.h"
#include "CSI_COLOR.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_REGISTERS.h"
#include "CSI_GRAPHICS.h"

// for debug
#include "CSI_PROCESS_CS800D.h"
#include <string.h>

// cg variables
uint16_t cg_offset; 
uint16_t cg_space_above; 
uint16_t cg_space_below; 
uint16_t cg_space_before;
uint16_t cg_space_after; 
uint16_t cg_max_hori_size;
uint16_t cg_max_vert_size;
uint16_t cg_spacing;
uint16_t cg_hori_size;
uint16_t cg_vert_size;
uint16_t cg_index;
uint16_t cg_next_address;  // set inprogramming.c but never used 

uint16_t cg_hori_location;
uint16_t cg_vert_location;

uint16_t cg_character_set;
uint8_t  cg_line_buffer[32];

//******************************************************************************
//
// March 24, 2018		Design Documents           ssd1963_cg_initialize
//
// Initializes the display
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The display is initialized
//
// Functions used
//  lcd_reset_high
//  lcd_reset_low
//  delay
//  ssd1963_set_pll_mn
//  ssd1963_set_pll
//  ssd1963_soft_reset
//  ssd1963_set_lshift_freq
//  ssd1963_set_lcd_mode
//  ssd1963_set_hori_period
//  ssd1963_set_vert_period
//  ssd1963_set_address_mode
//  ssd1963_set_pixel_data_interface
//  ssd1963_set_display_on
//
// History
//        09/26/2019 added red led blink when clock PLL fails
//
// Status
//  written
//
#define HDP 799      //Horizontal Display Period
#define HT  928      //Horizontal Total Period = (HT + 1)pixels
#define HPS  54      //Horizontal Sync Pulse Start Position (HPS + 1) pixels
#define LPS  40      //Horizontal Display Period Start Position = LPS pixels
#define HPW 128      //Horizontal Sync Pulse Width = (HPW + 1) pixels
#define VDP 479      //Vertical Display Period
#define VT  530      //Vertical Total = (VT + 1) Lines
#define VPS  50      //Vertical Sync Pulse Start Position = VPS lines
#define FPS  30      //Vertical Display Period Start Position = FPS lines
#define VPW  21      //Vertical Sync Pulse Width = (VPW + 1) lines

void ssd1963_cg_initialize(void)
{
  // set all port control pins high
  lcd_cs_high();
  lcd_d_c_high();
  lcd_wr_high();
  lcd_rd_high();      

  // Generate a hardware reset for the LCD display. The SSD1963 specs says the 
  // Reset must be low for a minimum of 100 uS
  lcd_reset_high();
  delay(10);
  lcd_reset_low();
  delay(100);
  lcd_reset_high();
  delay(200);

  // Set up the PLL for 800 x 480 display

  // POR default M = 0x2D 45 N = 3
  // refClk = 10 Mhz external crystal
  // m = 48  refClk * (m + 1) = 490 VCO divide / (N + 1)  245 Mhz PLL freq
  // maybe m should be = 47 480 VCO 240 PLL
  ssd1963_set_pll_mn(0x30,0x01,0x04);
  // registers can't be set faster than ref clock (10 Mhz) / 2 = 5Mhz, 200 nsec
  // write command with 1 variable is about 20 nsec, 3 vars 60 nsec
  delay(1);
  ssd1963_set_pll(0x01);
  //delay(1);
  // needs 100usec time for pll to lock
  //   delay(100);
  // or wait for lock..
  uint32_t plltimeout =  100000;
  while(ssd1963_get_pll_status() != 0x04)
  {
    delay(1);  // wait min 200n between register commands
    if(plltimeout-- == 0)
    {
      error_log( 2100 );
      plltimeout =  100000;
      blink_red(1);
    }
  }

  ssd1963_set_pll(0x03);
  delay(5);

  // Generate a software reset
  ssd1963_soft_reset();
  // "The host processor must wait 5ms before sending any new commands to a
  // SSD1963 following this command."  pg.24
  //delay(5);  
  delay(5000);

  // Set lshift frequency
  // POR default 524287 0x07FFFF
  // (110484 +1) / 2^20 
  ssd1963_set_lshift_freq(0x0001AF94);

  // Set LCD Mode
  ssd1963_set_lcd_mode(0x20,0x00,HDP,VDP,0x00);
  delay(5);
  // Set horizontal period
  ssd1963_set_hori_period(HT,HPS,HPW,LPS,0x00);
  // Set vertical period
  ssd1963_set_vert_period(VT,VPS,VPW,FPS);

  // Set address mode. This sets the definition of transfers from the processor
  // to the frame buffer and from the frame buffer to the display.
  ssd1963_set_address_mode(0x00); // POR default 0x00 can skip this

  // this is the parallel port width to CPU, should be 8
  //ssd1963_set_pixel_data_interface(0x05);  // POR default is 0x05 24 bit
  ssd1963_set_pixel_data_interface(0x00);
  delay(5);

  // Turn on the display
  ssd1963_set_display_on();
}
//
//******************************************************************************
// Feb 14, 2018			Design Documents           ssd1963_cg_read_array
//
// Reads a byte from the specified dot matrix array. There are currently eleven
// character sets as follows:
//   Size   Width  Height  Font Type        Index Number
//   72 pt  93     87      Times New Roman  0
//   48 pt  63     48      Times New Roman  1
//   36 pt  46     44      Times New Roman  2
//   24 pt  32     29      Times New Roman  3
//   16 pt  20     20      Times New Roman  4
//   48 pt  68     69      Courier New      5
//   36 pt  51     50      Courier New      6
//   28 pt  39     39      Couruer New      7
//   24 pt  34     36      Courier New      8
//   20 pt  29     30      Courier New      9
//   16 pt  23     23      Courier New     10
//   
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_offset is the current position in the dot matrix array
//  cg_character_set determines which dot matrix array to use
//  
//
// Exit Conditions
//  The dot matrix value pointed to by cg_offset is returned
// 
// External variables used
//  Times_New_Roman_72_Bitmap[] dot matrix for first character set
//  Times_New_Roman_48_Bitmap[] dot matrix for second character set
//  Times_New_Roman_36_Bitmap[] dot matrix for third character set
//  Times_New_Roman_24_Bitmap[] dot matrix for fourth character set
//  Times_New_Roman_16_Bitmap[] dot matrix for fifth character set
//  courier_New_48_bitmap[]  dot matrix for sixth character set
//  courier_New_36_bitmap[]  dot matrix for seventh character set
//  courier_New_28_bitmap[]  dot matrix for eight character set
//  courier_New_24_bitmap[]  dot matrix for nineth character set
//  courier_New_20_bitmap[]  dot matrix for tenth character set
//  courier_New_16_bitmap[]  dot matrix for eleventh character set
//
// History
//  01/28/2019  added courier fonts
//  11/07/2019  added 12pt TimesNR
//
// Status:
//  written, working
//
uint8_t ssd1963_cg_read_array(uint16_t cg_character_set, uint16_t cg_offset)
{
  uint8_t byt;
  switch(cg_character_set)
  {
    case 0:  byt = Times_New_Roman_72_Bitmap[cg_offset];break;
    case 1:  byt = Times_New_Roman_48_Bitmap[cg_offset];break;
    case 2:  byt = Times_New_Roman_36_Bitmap[cg_offset];break;
    case 3:  byt = Times_New_Roman_24_Bitmap[cg_offset];break;
    case 4:  byt = Times_New_Roman_16_Bitmap[cg_offset];break;
    case 5:  byt = Courier_New_48_bitmap[cg_offset];break;
    case 6:  byt = Courier_New_36_bitmap[cg_offset];break;
    case 7:  byt = Courier_New_28_bitmap[cg_offset];break;
    case 8:  byt = Courier_New_24_bitmap[cg_offset];break;
    case 9:  byt = Courier_New_20_bitmap[cg_offset];break;
    case 10: byt = Courier_New_16_bitmap[cg_offset];break;
    case 11: byt = Times_New_Roman_12_Bitmap[cg_offset];break;
    default:
      byt = 0;
      break;
  }
  return byt;
}
//
//******************************************************************************
//
// March 8, 2018		Design Documents      ssd1963_cg_read_descriptor
//
// Reads four parameters from the descriptor array. There are currently five
// character sets as follows:
//   Size   Width  Height
//   72 pt  93     87 
//   48 pt  63     48
//   36 pt  46     44
//   24 pt  32     29
//   16 pt  20     20
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  chars is the character you want to look up
//  cg_character_set determines which dot matrix array to use
//  
//
// Exit Conditions
//  cg_hori_size is the horizontal size of the character
//  cg_vert_size is the vertical size of the character
//  cg_space_after is how many bits until next character starts
//  cg_offset is the start of the character in the dot matrix array
// 
// External variables used
//  TN_char72_descriptors[][] description first character set
//  TN_char48_descriptors[][] description second character set
//  TN_char36_descriptors[][] description third character set
//  TN_char24_descriptors[][] description fourth character set
//  TN_char16_descriptors[][] description fifth character set
//  CN_char48_descriptors[][] description sixth character set
//  CN_char36_descriptors[][] description seventh character set
//  CN_char28_descriptors[][] description eight character set
//  CN_char24_descriptors[][] description nineth character set
//  CN_char20_descriptors[][] description tenth character set
//  CN_char16_descriptors[][] description eleventh character set
//
// History
//  01/28/2019  added courier fonts
//
// Status
//  written
//
void ssd1963_cg_read_descriptor(uint16_t cg_character_set, uint8_t chars)
{
const uint16_t *current_character_set;
//
// set up pointer to correct matrix. If the case is out of bounds, set the 
// the pointer to the smallest character set.
//
  current_character_set = &TN_char16_descriptors[chars][0];
  switch(cg_character_set)
  {
    case 0:  current_character_set = &TN_char72_descriptors[chars][0]; break;
    case 1:  current_character_set = &TN_char48_descriptors[chars][0]; break;
    case 2:  current_character_set = &TN_char36_descriptors[chars][0]; break;
    case 3:  current_character_set = &TN_char24_descriptors[chars][0]; break;
    case 4:  current_character_set = &TN_char16_descriptors[chars][0]; break;
    case 5:  current_character_set = &CN_char48_descriptors[chars][0]; break;
    case 6:  current_character_set = &CN_char36_descriptors[chars][0]; break;
    case 7:  current_character_set = &CN_char28_descriptors[chars][0]; break;
    case 8:  current_character_set = &CN_char24_descriptors[chars][0]; break;
    case 9:  current_character_set = &CN_char20_descriptors[chars][0]; break;
    case 10: current_character_set = &CN_char16_descriptors[chars][0]; break;
    case 11: current_character_set = &TN_char12_descriptors[chars][0]; break;
  default:
    break;
  }
//
// Load the parameters and return
//  
  cg_hori_size = current_character_set[0];
  cg_vert_size = current_character_set[1];
  cg_space_below = current_character_set[2];
  cg_offset = current_character_set[3];
}
//
//******************************************************************************
//
// March 9, 2018		Design Documents       ssd1963_cg_read_char_size
//
// Reads three parameters from the char size array. There are currently five
// character sets as follows:
//   Size   Width  Height
//   72 pt  93     87 
//   48 pt  63     48
//   36 pt  46     44
//   24 pt  32     29
//   16 pt  20     20
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_character_set determines which array to use
//  
//
// Exit Conditions
//  cg_max_hori_size   maximum width of character
//  cg_max_vert_size   maximum height of character
//  cg_spacing         spacing between characters
// 
// External variables used
//  TN_char72_char_size[] size and spacing of first character set
//  TN_char48_char_size[] size and spacing of second character set
//  TN_char36_char_size[] size and spacing of third character set
//  TN_char24_char_size[] size and spacing of fourth character set
//  TN_char16_char_size[] size and spacing of fifth character set
//  CN_char48_char_size[] size and spacing of sixth character set
//  CN_char36_char_size[] size and spacing of seventh character set
//  CN_char28_char_size[] size and spacing of eight character set
//  CN_char24_char_size[] size and spacing of nineth character set
//  CN_char20_char_size[] size and spacing of tenth character set
//  CN_char16_char_size[] size and spacing of eleventh character set
//
// History
//
// Status
//  written
//
void ssd1963_cg_read_char_size(uint16_t cg_character_set)
{
  const uint16_t *current_character_set;
  //
  // set up pointer to correct matrix. If the case is out of bounds, set the 
  // the pointer to the smallest character set.
  //
  current_character_set = &TN_char16_char_size[0];
  switch(cg_character_set)
  {
    case 0:  current_character_set = &TN_char72_char_size[0]; break;
    case 1:  current_character_set = &TN_char48_char_size[0]; break;
    case 2:  current_character_set = &TN_char36_char_size[0]; break;
    case 3:  current_character_set = &TN_char24_char_size[0]; break;
    case 4:  current_character_set = &TN_char16_char_size[0]; break;
    case 5:  current_character_set = &CN_char48_char_size[0]; break;
    case 6:  current_character_set = &CN_char36_char_size[0]; break;
    case 7:  current_character_set = &CN_char28_char_size[0]; break;
    case 8:  current_character_set = &CN_char24_char_size[0]; break;
    case 9:  current_character_set = &CN_char20_char_size[0]; break;
    case 10: current_character_set = &CN_char16_char_size[0]; break;
  }
  //
  // Load the parameters and return
  //  
  cg_max_hori_size = current_character_set[0];
  cg_max_vert_size = current_character_set[1];
  cg_spacing       = current_character_set[2];
}
//
//******************************************************************************
//
// Feb 13, 2018			Design Documents           ssd1963_cg_write_byte
//
// Writes a single byte to the display. If the bit to be displayed is a "1" then
// output the red, green, and blue values. If the bit to be displayed is a "0"
// then output the background_red, background_green, and background_blue values.
//
// Programmer
//  Jerry Wanger, Dean S. Graham 
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  chip select must be done before this is called
//  cg_data is the data to be displayed
//  cg_bits is the number of bits to use from cg_data. MSB outputs first
//
// Exit Conditions
//  The specified number of bits is displayed 
// 
// External variables used
//  cg_red is the red foreground byte
//  cg_green is the green foreground byte
//  cg_blue is the blue foreground byte
//
//  cg_background_red is the red background byte
//  cg_background_green is the green background byte
//  cg_background_blue is the blue background byte
// 
// History
//     08/21/2018 faster version
// =============================================================================
// faster version
// chip select must be done before this is called
// direct to gpio, no subroutine call
//
// Programmer
//  Dean S. Graham 
//
//  History
//  02/13/2019 removed NOP waits, subroutine call is enough time. use WR_cycle() 

void ssd1963_cg_write_byte(uint8_t cg_data,uint8_t cg_bits)
{
  uint8_t i;
  volatile uint16_t *G_ODR = GPIO_G_ODR;

  uint8_t f_red   = cg_font.red;
  uint8_t f_green = cg_font.green;
  uint8_t f_blue  = cg_font.blue;
  
  uint8_t b_red   = cg_background.red;
  uint8_t b_green = cg_background.green;
  uint8_t b_blue  = cg_background.blue;  
  
  for (i = cg_bits; i != 0; i--)
  {
    if (cg_data & 0x80)
    {
      *G_ODR = f_red;
      WR_cycle();
      *G_ODR = f_green;
      WR_cycle();
      *G_ODR = f_blue;
      WR_cycle();
    }
    else
    {
      *G_ODR = b_red;
      WR_cycle();      
      *G_ODR = b_green;
      WR_cycle();
      *G_ODR = b_blue;
      WR_cycle();
    }
    cg_data = cg_data << 1;
  }
}


//******************************************************************************
//
// Feb 13, 2018			Design Documents                ssd1963_cg_blank
//
// Writes the specified number of backgroud bits to the display.
//
// Programmer
//  Jerry Wanger, Dean S. Graham 
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_bits is the number of bits to output to ther display
//
// Exit Conditions
//  The specified number of bits is displayed 
// 
// External variables used
//  cg_background_red is the red background byte
//  cg_background_green is the green background byte
//  cg_background_blue is the blue background byte
// 
// Internal variables used
//  i is a temporary index counter
//
// Functions used
//  ssd1963_write_data(uint8_t)
//
// History
//     08/21/2018 revised to run faster, moved chip select before this is called
//     02/13/2019 removed NOP waits, subroutine call is enough time. use WR_cycle() 
//
// Status
//  written, working
//
void ssd1963_cg_blank(uint32_t cg_bits)
{
  uint32_t i;
  volatile uint16_t *G_ODR = GPIO_G_ODR;
  uint8_t b_red   = cg_background.red;
  uint8_t b_green = cg_background.green;
  uint8_t b_blue  = cg_background.blue;    

  for (i = cg_bits; i != 0; i--)
  {
      *G_ODR = b_red;
      WR_cycle();
      *G_ODR = b_green;
      WR_cycle(); 
      *G_ODR = b_blue;
      WR_cycle();  
  }
}
//******************************************************************************
//
// Feb 13, 2018			Design Documents                 ssd1963_cg_line
//
// Writes a single line of active character data to the display. It should be
// noted if the complete byte is not used by the time we get to the end of the
// line, then the remaining bits in that byte will be discarded.
//
// As an example, a 5 x 7 character takes only 35 bits to display, but because
// we are discarding the last 3 bits in each byte, we are storing that character
// in 7 bytes or 56 bits.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_character_set is the font being used
//  cg_hori_size is the number of bits to output to the display
//  cg_offset points to the character that has to be outputed
//  cg_spacing is the number of blank bits after the character
//
// Exit Conditions
//  The specified number of bits is outputed to the display.
//  cg_offset points to the next set of bytes that have to be outputed.
// 
// External variables used
//  None
// 
// Internal variables used
//  cg_bits is number of bits to output
//    
// Functions used
//  ssd1963_cg_read_array
//  ssd1963_cg_write_byte
//  ssd1963_cg_blank
//
// History
//       08/21/2018 Faster, moved chip select before this is called
//
// Status
//  written, working
//
uint16_t ssd1963_cg_line(uint16_t cg_character_set, uint16_t cg_offset,
                         uint16_t cg_hori_size, uint16_t cg_spacing)
{
  uint8_t cg_bits;
  uint8_t cg_data;  

  while (cg_hori_size)
  {
    // Determine number of bits to be output. if cg_hori_size is greater than or
    // equal to 8, output 8 bits. If less than 8, output specified number.
    if (cg_hori_size < 9) cg_bits = cg_hori_size;
    else cg_bits = 8;

    // Get the data to be output and then send it to the display.
    cg_data = ssd1963_cg_read_array(cg_character_set, cg_offset); 
    ssd1963_cg_write_byte(cg_data, cg_bits); 

    // Adjust the number of bits left by subtracting the number of bits just sent
    // to the display from the cg_hori_size. Point to the next character to be
    // outputed. Note that if the last byte does not use all the bits for the 
    // display then the rest of the bits in that byte are wasted.
    cg_hori_size = cg_hori_size - cg_bits;
    cg_offset++;
  }
  
  // Output the spacing after the character if used for progressive characters 
  // else ignore if centered characters.
  if (cg_spacing)
  {
    ssd1963_cg_blank(cg_spacing);
  }
  // Return pointer so it now points to the begining of the next line.
  return cg_offset;
}
//
//******************************************************************************
//
// Feb 14, 2018			Design Documents    ssd1963_cg_write_progressive
//
// Writes the specified character to the display in progressive mode. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_offset is the index into the character.
//  cg_spacing is the number of blank dots to the right of the character
//  cg_space_above is the number of blank lines above the character
//  cg_space_below is the number of blank lines below the character
//  cg_vert size is the size of the character in number of lines
//  cg_hori_size is the active horizontal size of the character in bits
//
// Exit Conditions
//  The character is written to the display.
// 
// External variables used
//  None
// 
// Internal variables used
//  i is used as a loop counter
//
// Functions used
//  ssd1963_cg_blank
//  ssd1963_cg_line
//
// History
//        08/21/2018 faster, moved chip select here
//
// Status
//  written, working
//
void ssd1963_cg_write_progressive(uint16_t cg_character_set, uint16_t cg_offset,
                               uint16_t cg_space_above, uint16_t cg_space_below,
                               uint16_t cg_hori_size, uint16_t cg_vert_size,
                               uint16_t cg_spacing)                             
{
  uint32_t i;  
  // chip select begin
  lcd_cs_low();
  lcd_d_c_high();  
  // wait_12_nanoseconds();
  
  // Write blank lines to the space above the display
  ssd1963_cg_blank(cg_space_above * (cg_hori_size + 1 + cg_spacing)); 
  
  // Write the character to be displayed
  for (i = cg_vert_size; i != 0; i--)
  {
    cg_offset = ssd1963_cg_line(cg_character_set, cg_offset, cg_hori_size, cg_spacing + 1);
  }

  // Write the blank lines to the space below the display
  ssd1963_cg_blank(cg_space_below  * (cg_hori_size + 1 + cg_spacing));
  
  // chip select end
  lcd_cs_high();
  //wait_12_nanoseconds();   
}
//******************************************************************************
//
// Feb 14, 2018			Design Documents       ssd1963_cg_write_centered
//
// Writes the specified character to the display in centering mode. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_offset is the index into the character.
//  cg_space_above is the number of blank lines above the character
//  cg_space_below is the number of blank lines below the character
//  cg_space_before is the number of blank dots before the character
//  cg_space_after is the number of blank dots after the character
//  cg_max_hori_size is the horizontal size of character including spaces
//  cg_vert size is the size of the character
//
// Exit Conditions
//  The character is written to the display
// 
// External variables used
//  None
// 
// Internal variables used
//  i is used as a loop counter
//    
// Functions used
//  ssd1963_cg_blank
//  ssd1963_cg_line
//
// History
//        08/21/2018 faster, moved chip select here
//
// Status
//  written, working
//
void ssd1963_cg_write_centered(uint16_t cg_character_set, uint16_t cg_offset,
                             uint16_t cg_space_above, uint16_t cg_space_below,
                             uint16_t cg_space_before, uint16_t cg_space_after,
                             uint16_t cg_max_hori_size, uint16_t cg_vert_size)
{
  uint32_t i;
  // chip select begin
  lcd_cs_low();
  lcd_d_c_high();  
  //wait_12_nanoseconds();

  // Write blank lines to the space above the display  
  ssd1963_cg_blank(cg_space_above * (cg_max_hori_size + 1));

  // Write the character to be displayed
  for (i = cg_vert_size; i != 0; i--)
  {
    ssd1963_cg_blank(cg_space_before);    
    cg_offset = ssd1963_cg_line(cg_character_set, cg_offset, cg_hori_size, 0);
    ssd1963_cg_blank(cg_space_after);
  }

  // Write the blank lines to the space below the display
  ssd1963_cg_blank(cg_space_below * (cg_max_hori_size + 1));
  // chip select end
  lcd_cs_high();
  //wait_12_nanoseconds();     
  
}
//
//******************************************************************************
//
// Feb 16, 2018			Design Documents             ssd1963_cg_location
//
// Sets the controller chip to output data within a rectangular location on the
// display. This allows the programs to output data without worrying about
// setting pointers to the next line when approriate.
//
// This routine handles both the centered character and the progressive
// character. For the centered character, the size is 100% determined by the
// maximum vertical size and maximum horizontal size. For the progressive
// character, the vertical size is determined by the maximum vertical size
// while the horizontal size is determined by the horizontal size of the 
// character plus the spacing between characters.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_spacing is the space between characters. For centered char value is zero.
//  cg_hori_location is the location of the top left corner of the character
//  cg_vert_location is the location of the top left corner of the character
//  cg_hori_size is the horizontal size of the character without spaces
//  cg_max_hori_size is the horizontal size of character including spaces
//  cg_max_vert size is the size of the character
//
// Exit Conditions
//  The controller is set up to accept data for a character
// 
// External variables used
//  None
// 
// Internal variables used
//  Not used
// 
// Functions used
//  ssd1963_set_column_address
//  ssd1963_set_row_address
//  ssd1963_write_memory_start
//
// History
//
// Status
//  written, working
//
void ssd1963_cg_location(uint16_t cg_spacing_type, uint16_t cg_hori_location, 
                         uint16_t cg_vert_location, uint16_t cg_hori_size,
                         uint16_t cg_max_hori_size, uint16_t cg_max_vert_size)
{

  // Process centered characters. The size of the centered characters is entirely
  // dependent on the cg_max_hori_size and cg_max_vert_size variables. 
  if (cg_spacing_type == 0)
  {
    ssd1963_set_column_address(cg_hori_location, 
                               cg_hori_location + cg_max_hori_size);
    ssd1963_set_row_address(cg_vert_location, 
                               cg_vert_location + cg_max_vert_size);
  }
  else
  {
    ssd1963_set_column_address(cg_hori_location, 
                               cg_hori_location + cg_hori_size + cg_spacing);
    ssd1963_set_row_address(cg_vert_location, 
                               cg_vert_location + cg_max_vert_size);    
  }
  ssd1963_write_memory_start();
}
//******************************************************************************
//
// Feb 16, 2018			Design Documents        ssd1963_cg_next_location
//
// Sets the location to the next character. The calculation is different
// dependent on if the character is centered or progressive.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_spacing is the space between characters. For centered char value is zero.
//  cg_hori_location is the location of the top left corner of the character
//  cg_hori_size is the horizontal size of the character without spaces
//  cg_max_hori_size is the horizontal size of character including spaces
//
// Exit Conditions
//  The location of the next character is returned
// 
// External variables used
//  None
// 
// Internal variables used
//  Not used
//
// History
//
// Status
//  written
//
uint16_t ssd1963_cg_next_location(uint16_t cg_spacing, uint16_t cg_hori_location,  
                                  uint16_t cg_hori_size,
                                  uint16_t cg_max_hori_size)  
{
  uint16_t hloc;
  // Process centered characters. The next location of the centered characters is 
  // entirely dependent on the cg_max_hori_size variable. 
  if (cg_spacing == 0)
  {
    hloc = cg_hori_location + cg_max_hori_size;
  }
  // Process progressive characters. The next location of the progressive
  // character is the starting location plus the size of the character plus the
  // spacing between characters.
  else 
  {
    hloc = cg_hori_location + cg_hori_size + cg_spacing;
  }
  return hloc;
}

//******************************************************************************
//
// Feb 16, 2018			Design Documents     ssd1963_cg_output_character
//
// This program outputs a single character to the display. There are many 
// paramaters to consider besides the character itself. The details are listed
// below.
//
// 1. Character set. This product can accomodate many different character sets.
//    The character set will be defined as a single byte. Each of those sets
//    needs the bit mapped characters and the definition of the characters. 
//
// 2. The number of vertical bits is a variable for all of the characters in the
//    character set. This is to minimize the storage space of the characters. 
//
// 3. The number of horizontal bits is a variable for all of the characters in 
//    the character set. This is to minimize the storage space of the characters
//
// 3. The characters do not always take the full vertical space allowed for each
//    character. The number of blank lines above and the number of blank lines
//    below the character are defined.
//
// 4. Being that each character has a variable number of bits required to 
//    represent itself, the starting offset for that characters has to be
//    defined.
//
// 5. There should be a space between characters. There is a parameter that
//    defines the number of bits to the right of the character. If the value
//    of that paramater is zero, then the character is centered horizontally
//    in its allocated space.
//
// 6. Character size. This is particularly important when you have fixed
//    sizes for each character. For proportional spacing it is not as important.
//    The vertical spacing can be calculated by taking a character and adding
//    the blank space above and below the character.
//
// 7. Background and Foreground colors.
//
// Note the character below and the bit map is set for a 15 x 22 matrix
//
//                           0000000001111111111222
// @45 '"' (22 pixels wide)  1234567890123456789012
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //    Note there are 3 blank lines above character                   
//      0x00, 0x00, 0x00, //                       
//      0x00, 0xD8, 0x00, //         ## ##         
//      0x00, 0xD8, 0x00, //         ## ##         
//      0x00, 0xD8, 0x00, //         ## ##         
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //  Note there are 9 blank lines below character                      
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, //                       
//      0x00, 0x00, 0x00, // 
//
// The compressed character below is using only a 3 x 5 matrix.
// @9 '"' (5 pixels wide)
//      0xD8, // ## ##
//      0xD8, // ## ##
//      0xD8, // ## ##
//
// If we were going to center it, we would subtract 5 from 22 = 17. Then I 
// would have 8 dots before the character and 9 dots after the character
// for a total of 22 dots.
//
// cg_character_set_data[char set][3] 
//   uint16_t cg_spacing        //character set 0
//   uint16_t cg_max_vert_size  //character set 0
//   uint16_t cg_max_hori_size  //character set 0
//      .
//      .
//   uint16_t cg_spacing        //character set 4
//   uint16_t cg_max_vert_size  //character set 4
//   uint16_t cg_max_hori_size  //character set 4
//
// cg_character_definition[char set][char][4]
//   uint16_t cg_vert_size      //character set n, char p
//   uint16_t cg_hori_size
//   uint16_t cg_space_below
//   uint16_t cg_offset
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_character
//  cg_character_set
//  cg_hori_location
//  cg_vert_location
//  cg_spacing_type
//
// Exit Conditions
//  The specified character is displayed 
//  The next location for the next character is returned
// 
// External variables used
//  cg_red is the red byte
//  cg_green is the green byte
//  cg_blue is the red byte
//
//  cg_background_red is the red background byte
//  cg_background_green is the green background byte
//  cg_background_blue is the red background byte
//
//  cg_character_set[5][3]
//  cg_character_definition[5][98][4]
// 
// Internal variables used
//  cg_spacing_type
//  cg_max_vert_size
//  cg_max_hori_size
//  cg_vert_size
//  cg_hori_size
//  cg_space_above
//  cg_space_below
//  cg_offset
//
// Functions used
//  ssd1963_cg_write_centered
//  ssd1963_cg_write_progressive
//  ssd1963_cg_location
//  ssd1963_cg_next_location
// History
//
// Status
//  written
//
uint8_t temp_cg_character;

uint16_t ssd1963_cg_output_character(uint8_t cg_character,uint8_t character_set,
                           uint16_t cg_hori_location, uint16_t cg_vert_location, 
                           uint16_t cg_spacing_type)
{
//
// This routine is modified for use with the FT857D. There a these special
// characters that have to be accomodated for the courier new 28 
//  0x7E  --> 0x7F   Running man 1
//  0x7F  --> 0x80   Running man 2
//  0x1F  --> 0x81   Special Star
//  0x1C  --> 0x82   Special dot
//  0x83  --> 0x83   special Double plus
//  0x7D  --> 0x84   lock
//
// These characters are for the courier new 48
//  0x19  --> 0x7F   Return arrow
//
    temp_cg_character = cg_character;
    if (character_set == 7)  // Courier 28 point
    {
      if (cg_character == 0x7D) cg_character = 0x84; // Lock
      if (cg_character == 0x7F) cg_character = 0x80; // running man 2
      if (cg_character == 0x7E) cg_character = 0x7F; // running man 1
 

      if (cg_character == 0x1F) cg_character = 0x81; // special star
      if (cg_character == 0x1C) cg_character = 0x82; // special dot
      if ((cg_character <0x20) || (cg_character > 0x84))
      {
        cg_character = 0x20;
      }
    }
    
    if (character_set == 5)  //Courier 48 point
    {
      if (cg_character == 0x19) cg_character = 0x7F; // Return arrow
      if ((cg_character <0x20) || (cg_character > 0x7F))
      {
        cg_character = 0x20;
      }
    }
  
  //  assert (cg_character >31);
  //  assert (cg_character <95);

    // Get the various parameters associated with this character set and character
  if(cg_character > 0x20) cg_index = cg_character - 0x20;
  else cg_index = '_' - 0x20;  // print underlines for bad characters < 32
  if (cg_index >100) // changed from 94 --> 99 for special characters *************for FT857D only
  {
    cg_index = '?' - 0x20;  // ?'s for bad chacaters > 94
    //display_packet_dump();
  }
  
  // get font max sizes, and spacing
  ssd1963_cg_read_char_size( character_set );
  // get character sizes and spacing
  ssd1963_cg_read_descriptor( character_set, cg_index );

  // Space above is a cacluated value. Its whats left when you take into account
  // the character size and space below. Space before and space after is also
  // calculated value and is whats left when you take into account the character
  // size.
  //
  cg_space_above = cg_max_vert_size - cg_vert_size - cg_space_below;
  cg_space_before = (cg_max_hori_size - cg_hori_size)/2;
  cg_space_after = cg_space_before;
  // if it's an odd number, 0.5 part of integer was truncated, need one more bit
  if((cg_max_hori_size - cg_hori_size) & 1) cg_space_after++;
  cg_space_after++;
  // if ((cg_space_before + cg_space_after) < cg_max_hori_size) cg_space_after++;
  
  // special case for a space
  if( cg_character == ' ') 
  {
    uint16_t spacewidth;
    // recalc hori sizes so location and next location are correct
    if(cg_spacing_type == 0)    // for centered
    {  
      spacewidth = cg_max_hori_size;
      cg_max_hori_size -= 1;
      cg_max_vert_size -= 1;      
    }
    else  // for propoptional 
    {  
      spacewidth = 5 * cg_spacing_type;
      cg_hori_size = spacewidth - cg_spacing_type - 4;
      cg_max_vert_size -= 1;
    }
    // Calculate the size of the character so the controller chip can be
    // optimized for entering the data.
    //
    ssd1963_cg_location(cg_spacing_type, cg_hori_location, cg_vert_location, 
                      cg_hori_size, cg_max_hori_size, cg_max_vert_size);
   
    // chip select begin
    lcd_cs_low();
    lcd_d_c_high();  
    //wait_12_nanoseconds(); subroutine call takes enough time
    // write blanks for the space
    ssd1963_cg_blank(cg_max_vert_size * (spacewidth + 1));
    // chip select end
    lcd_cs_high();
    //wait_12_nanoseconds();     
  }
  else
  {
    // Calculate the size of the character so the controller chip can be
    // optimized for entering the data.
    ssd1963_cg_location(cg_spacing_type, cg_hori_location, cg_vert_location, 
                      cg_hori_size, cg_max_hori_size, cg_max_vert_size);
    // Now that everything is calculated, it is time to output the character.

    if (cg_spacing_type == 0)
    {
      ssd1963_cg_write_centered(character_set, cg_offset, cg_space_above,
                                cg_space_below, cg_space_before, cg_space_after,
                                cg_max_hori_size, cg_vert_size);
    }
    else
    {
      ssd1963_cg_write_progressive(character_set, cg_offset, cg_space_above,
                                   cg_space_below, cg_hori_size, cg_vert_size,
                                   cg_spacing);
    }
  }

  // Return the location of the next character.
  return ssd1963_cg_next_location(cg_spacing_type, cg_hori_location, cg_hori_size + 1, 
                                  cg_max_hori_size + 1);  
}
//
//******************************************************************************
//
// Feb 16, 2018			Design Documents           ssd1963_cg_write_line
//
// Outputs a line of characters to the display. This routine only outputs a ram
// buffer. If you want to output fixed message, the message must first be 
// transfered to that buffer
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_length is the number of characters to be displayed
//  cg_character_set indicates which character set to use.
//  cg_hori_location is the location of the top left corner of the character
//  cg_vert_location is the location of the top left corner of the character
//  cg_spacing indicates number of blanks bits between characters. 
//
// Exit Conditions
//  The specified data is written to the display
// 
// External variables used
//  cg_line_buffer[80]
// 
// Internal variables used
//  i is used as a loop counter
// 
// Functions used
//  ssd1963_cg_output_character
//
// History
//   07/31/2018 added break loop on zero character
//
// Status
//  written
//
void ssd1963_cg_write_line(uint8_t cg_length,uint8_t character_set,
                           uint16_t cg_hori_location, uint16_t cg_vert_location, 
                           uint16_t cg_spacing_type)
{
  uint32_t i;
  uint8_t cg_character;
  //
  // Output data to the display
  //  must use incrementing index to get data out of buffer
  for (i = 0; i < cg_length; i++)
  {
    cg_character = cg_line_buffer[i];
    if( cg_character == 0) break;
    cg_hori_location = ssd1963_cg_output_character(cg_character, character_set,
                               cg_hori_location, cg_vert_location, cg_spacing_type);
    //
    if( cg_hori_location > (HDP - cg_hori_size) )
    {
       cg_hori_location = 0;
       cg_vert_location += cg_vert_size + 5;  // <- make a line space variable
    }
  }   
}    
// =============================================================================
// write to screen from char* instead of fixed buffer
//  using null terminated strings, length is not required
//  

// 08/13/2018 written, untested

void ssd1963_write_line(char* txt_data, uint8_t character_set, uint16_t cg_hori_location,
                        uint16_t cg_vert_location, uint16_t cg_spacing_type)
{
  uint32_t i;
  uint8_t ch;
  // Output data to the display
  i = 0;
  while( txt_data[i] != 0)
  {
    ch = txt_data[i];
    cg_hori_location = ssd1963_cg_output_character( ch, character_set,
                       cg_hori_location, cg_vert_location, cg_spacing_type);
    //
    if( cg_hori_location > (HDP - cg_hori_size) )
    {
       cg_hori_location = 0;
       cg_vert_location += cg_vert_size + 5;  // <- make a line space variable
    }
    i++;
  }   
}


// =============================================================================
//
//******************************************************************************
//
// Feb 16, 2018			Design Documents           ssd1963_cg_clear_line
//
// clears a line of data. The size of the line is arbitrary and can include 
// anything from part of a line to the whole display.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_hori_location is the location of the top left corner of the character
//  cg_vert_location is the location of the top left corner of the character
//  cg_hori_size is the horizontal size of the rectangle to be erased
//  cg_vert_size is the vertical size of the rectangle to be erased
//
// Exit Conditions
//  The specified line is erased
// 
// External variables used
//  cg_background_red
//  cg_background_green
//  cg_background_blue
// 
// Internal variables used
//  i is used as a loop counter
// 
// Functions used
//  ssd1963_set_column_address
//  ssd1963_set_row_address
//  ssd1963_write_memory_start
//  ssd1963_write_data
//
// History
//
// Status
//  written
//
void ssd1963_cg_clear_line(uint16_t cg_hori_location, uint16_t cg_vert_location, 
                           uint16_t cg_hori_size, uint16_t cg_vert_size)
{
  uint32_t i;
  volatile uint16_t *G_ODR = GPIO_G_ODR;
  uint8_t b_red   = cg_background.red;
  uint8_t b_green = cg_background.green;
  uint8_t b_blue  = cg_background.blue;   
  
  //
  // Set up rectangle
  //
  ssd1963_set_column_address(cg_hori_location, cg_hori_location + cg_hori_size);
  ssd1963_set_row_address(cg_vert_location, cg_vert_location + cg_vert_size);
  ssd1963_write_memory_start();
  // Clear Rectangle
  // chip select begin
  lcd_cs_low();
  lcd_d_c_high();  
  // wait_12_nanoseconds();
  
  for (i = (cg_hori_size * cg_vert_size); i != 0;  i--)
  {
      *G_ODR = b_red;
      WR_cycle();      
      *G_ODR = b_green;
      WR_cycle();
      *G_ODR = b_blue;
      WR_cycle();
  }
  // chip select end
  lcd_cs_high();
}    
//
//******************************************************************************
//
// March 6, 2018		Design Documents        ssd1963_cg_clear_display
//
// This program clears the entire display           
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The entire display is erased
// 
// External variables used
//  cg_background_red
//  cg_background_green
//  cg_background_blue
// 
// Internal variables used
//  i is used as a loop counter
// 
// Functions used
//  ssd1963_set_column_address
//  ssd1963_set_row_address
//  ssd1963_write_memory_start
//  ssd1963_write_data
//
// History
//      Old version, slow due to using write_data,
//                    replaced by faster clear_display
//
// Status
//  written, replaced
//
/*
void ssd1963_cg_clear_display_old(void)
{
uint32_t i;
//
// Set up rectangle
//
  cg_hori_location = 0;
  cg_hori_size = 800;
  cg_vert_location = 0;
  cg_vert_size = 480;
  
  ssd1963_set_column_address(cg_hori_location, cg_hori_location + cg_hori_size);
  ssd1963_set_row_address(cg_vert_location, cg_vert_location + cg_vert_size);
  ssd1963_write_memory_start();
//
// Clear Rectangle
//
  for (i = 0; i < cg_hori_size * cg_vert_size; i++)
  {
    ssd1963_write_data(cg_background_red);
    ssd1963_write_data(cg_background_green);
    ssd1963_write_data(cg_background_blue);    
  }
}
*/
// =============================================================================
// 
//    Fast screen clear
//      Replaces slow version
//
// Programmer
//  Dean S. Graham
//
// for screen clear timing test
// extern unsigned int ticks;
// int char_time = 0;
//
// History
//  02/13/2019 removed NOP waits, subroutine call is enough time. use WR_cycle() 

void ssd1963_cg_clear_display(void)
{
  volatile uint16_t *G_ODR = GPIO_G_ODR;
  uint32_t i;
  uint8_t cbk_red;
  uint8_t cbk_green;
  uint8_t cbk_blue;
  
  // store colors so they aren't changed during screen clear
  cbk_red   = cg_background.red;
  cbk_green = cg_background.green;
  cbk_blue  = cg_background.blue;
      
  // Set up rectangle
  //  char_time = ticks;  // for clear screen timing test

  cg_hori_location = 0;
  cg_hori_size = 800;
  cg_vert_location = 0;
  cg_vert_size = 480;
  
  ssd1963_set_column_address(cg_hori_location, cg_hori_location + cg_hori_size);
  ssd1963_set_row_address(cg_vert_location, cg_vert_location + cg_vert_size);

  // set port to output
  ssd1963_set_port_direction( 1 );
  // send write command
  ssd1963_write_memory_start();
  
  // Clear Rectangle
  // write memory start ends with CS high and DC low
  //   
  lcd_cs_low();
  lcd_d_c_high(); 
  
  for (i = (cg_hori_size + 1) * cg_vert_size; i != 0; i--)
  {
    // put data in port, red
    *G_ODR = cbk_red;
    WR_cycle();
    // put data in port, green
    *G_ODR = cbk_green;    
    WR_cycle();
    // put data in port, blue
    *G_ODR = cbk_blue;      
    WR_cycle();
  }
  lcd_cs_high();
  
  /* for screen clear timing test
  // read timer again
  char_time = ticks - char_time;
  // break after
  char_time = 0; 
  */
  
}    


//
//******************************************************************************
//
// March 6, 2018	       Design Documents ssd1963_cq_write_horizontal_line
//
// This program writers a single horizontal line
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_hori_location indicates the starting horizontal location
//  cg_vert_location indicates the starting vertical location
//  cg_hori_size indicates the length of the line
//
// Exit Conditions
//  A single line is written
// 
// External variables used
//  cg_red
//  cg_green
//  cg_blue
// 
// Internal variables used
//  i is used as a loop counter
// 
// Functions used
//  ssd1963_set_column_address
//  ssd1963_set_row_address
//  ssd1963_write_memory_start
//  ssd1963_write_data
//
// History
//
// Status
//  written, not in use, uses write_data, slow
//              
void ssd1963_cq_write_horizontal_line(uint16_t cg_hori_location,
                                      uint16_t cg_vert_location,
                                      uint16_t cg_hori_size)
{
uint32_t i;
//
// Set up line
//
  ssd1963_set_column_address(cg_hori_location, cg_hori_location + cg_hori_size);
  ssd1963_set_row_address(cg_vert_location, cg_vert_location + 1);
  ssd1963_write_memory_start();
//
// Write line
//
  for (i = (cg_hori_size * cg_vert_size); i != 0; i--)
  {
    ssd1963_write_data(cg_font.red);
    ssd1963_write_data(cg_font.green);
    ssd1963_write_data(cg_font.blue);    
  }
}
//
//******************************************************************************
//
// March 6, 2018	       Design Documents ssd1963_cq_clear_horizontal_line
//
// This program clears a single horizontal line
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_hori_location indicates the starting horizontal location
//  cg_vert_location indicates the starting vertical location
//  cg_hori_size indicates the length of the line
//
// Exit Conditions
//  A single line is cleared
// 
// External variables used
//  cg_background_red
//  cg_background_green
//  cg_background_blue
// 
// Internal variables used
//  i is used as a loop counter
// 
// Functions used
//  ssd1963_set_column_address
//  ssd1963_set_row_address
//  ssd1963_write_memory_start
//  ssd1963_write_data
//
// History
//
// Status
//  written not in use, uses write_data, slow
//   
void ssd1963_cq_clear_horizontal_line(uint16_t cg_hori_location,
                                      uint16_t cg_vert_location,
                                      uint16_t cg_hori_size)            
{
  uint32_t i;
  // Set up line
  ssd1963_set_column_address(cg_hori_location, cg_hori_location + cg_hori_size);
  ssd1963_set_row_address(cg_vert_location, cg_vert_location + 1);
  ssd1963_write_memory_start();
  // Clear Line
  for (i = (cg_hori_size * cg_vert_size); i != 0; i--)
  {
    ssd1963_write_data(cg_background.red);
    ssd1963_write_data(cg_background.green);
    ssd1963_write_data(cg_background.blue);    
  }
}
//
//******************************************************************************
//
// March 28, 2018	       Design Documents      ssd1963_cq_calculate_center
//
// This program calculates the starting point of a line if you want the line to
// be centered in the screen
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_character_set inidcates which character set we are using.
//  cg_number_of_characters is the number of characters to be displayed
//
// Exit Conditions
//  Starting location from the left side of the screen is calculated to enable
//  the message to be centered in the screen.
// 
// Functions used
//  ssd1963_cg_read_char_size
//
// History
//
// Status
//  written
//   
uint16_t ssd1963_cq_calculate_center(uint16_t cg_character_set,
                                 uint16_t cg_number_of_characters)          
{
//
// Read the characteristics of the character set we are using. Each character
// takes up cg_max_hori_size + cg_spacing
//
  ssd1963_cg_read_char_size(cg_character_set);
//
// Calculate the space need on the left to center the message. Remember the
// total size of the screen is 800 and there should be equal space on the left
// as well as the right.
//
  return (800 - ((cg_max_hori_size + cg_spacing) * cg_number_of_characters))/2;
}
//
//******************************************************************************
//
// March 28, 2018	       Design Documents        ssd1963_cq_display_center
//
// This program displays a message in the center of the screen
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  cg_character_set inidcates which character set we are using.
//  cg_number_of_characters is the number of characters to be displayed
//  cg_vert_location is the vertical location to display message
//
// Exit Conditions
//  Starting location from the left side of the screen is calculated to enable
//  the message to be centered in the screen.
// 
// Functions used
//  ssd1963_cg_read_char_size
//  ssd1963_cg_clear_line
//  ssd1963_cq_calculate_center
//  ssd1963_cg_write_line
//
// History
//
// Status
//  written
//   
void ssd1963_cq_display_center(uint16_t cg_character_set,
                                   uint16_t cg_number_of_characters,
                                   uint16_t cg_vert_location)          
{
  // Erase the line.
  ssd1963_cg_read_char_size(cg_character_set);
  ssd1963_cg_clear_line(0,cg_vert_location, 800, cg_max_vert_size);

  // Calculate the starting location of the line to be displayed 
  cg_hori_location = ssd1963_cq_calculate_center(cg_character_set,
                                                 cg_number_of_characters); 
  // Output the line to the display
  ssd1963_cg_write_line(cg_number_of_characters, cg_character_set, 
                        cg_hori_location, cg_vert_location, cg_spacing);
                           
}

