//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Feb 8,2018             Design Documents               CSI_SSD1963_UTILITIES.c
//
//		routines for SSD1963 LCD screen controller
//                  for the large panel project
//
// Programmer
//  Jerry Wanger
//
// Copyright
//  Connect Systems Inc.
//
//==============================================================================
// NAME OF ROUTINE                      Function
//
// ssd1963_write_command                Write a command byte to the controller
// ssd1963_write_data                   Write a data byte to the controller
// ssd1963_read_data                    Reads a data byte from the controller
// ssd1963_set_port_direction           Set GPIO port direction
//
//                                   COMMAND CODE
// ssd1963_nop                          0x00
// ssd1963_soft_reset                   0x01
// ssd1963_get_power_mode               0x0A
// ssd1963_get_address_mode             0x0B
// ssd1963_get_display_mode             0x0D
// ssd1963_get_tear_effect_status       0x0E
// ssd1963_enter_sleep_mode             0x10
// ssd1963_exit_sleep_mode              0x11
// ssd1963_enter_partial_mode           0x12
// ssd1963_enter_normal_mode            0x13
// ssd1963_exit_invert_mode             0x20
// ssd1963_enter_invert_mode            0x21
// ssd1963_set_gamma_curve              0x26
// ssd1963_set_display_off              0x28
// ssd1963_set_display_on               0x29
// ssd1963_set_column_address           0x2A
// ssd1963_set_row_address              0x2B
// ssd1963_write_memory_start           0x2C
// ssd1963_read_memory_start            0x2E
// ssd1963_set_partial_area             0x30
// ssd1963_set_tear_off                 0x34
// ssd1963_set_tear_on                  0x35
// ssd1963_set_address_mode             0x36
// ssd1963_exit_idle_mode               0x38
// ssd1963_enter_idle_mode              0x39
// ssd1963_write_memory_continue        0x3C
// ssd1963_read_memory_continue         0x3E
// ssd1963_set_tear_scanline            0x44 
// ssd1963_get_scanline                 0x45
// ssd1963_read_device_desciptor        0xA1
// ssd1963_set_lcd_mode                 0xB0
// ssd1963_set_hori_period              0xB4
// ssd1963_set_vert_period              0xB6
// ssd1963_set_post_proc                0xBC
// ssd1963_get_post_proc                0xBD
// ssd1963_set_pwm_conf                 0xBC
// ssd1963_set_dbc_conf                 0xD0
// ssd1963_set_dbc_th                   0xD4
// ssd1963_set_pll                      0xE0
// ssd1963_set_pll_mn                   0xE2
// ssd1963_get_pll_status               0xE4
// ssd1963_set_deep_sleep               0xE5
// ssd1963_set_lshift_freq              0xE6
// ssd1963_set_pixel_data_interface     0xF0
//
//==============================================================================
#include <stdint.h>
#include "CSI_SSD1963.h"
#include "CSI_GPIO.h"
#include "CSI_TIMERS.h"
#include "CSI_REGISTERS.h"
//==============================================================================
//
// Feb 8, 2018			Design Documents           ssd1963_write_command
//
// Writes a command byte to the SSD1963 controller. Assumes 8080 timing
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  Command byte is passed as a parameter
//
// Exit Conditions
//  The Command byte is written into the controller
//
// External functions called
//  write_lcd_port(uint8_t)
//  lcd_cs_low(void)
//  lcd_cs_high(void)
//  lcd_d_c_low(void)
//  lcd_wr_low(void)
//  lcd_wr_high(void)
//  wait_12_nanoseconds(void)
//  
// Internal variables used
//    
// Constants
//
// History
//
// Status
//  written
//
void ssd1963_write_command(uint8_t command_byte)
{
  write_lcd_port(command_byte);
  lcd_cs_low();
  lcd_d_c_low();
  lcd_wr_low();
  //wait_12_nanoseconds();
  lcd_wr_high();             //Tpwlw 14 nS
  lcd_cs_high();             //Tpwcsl 16 nS
  //wait_12_nanoseconds();     //Tpwcsh 14 nS
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents              ssd1963_write_data
//
// Writes a data byte to the SSD1963 controller. Assumes 8080 timing
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  Data byte is passed as a parameter
//
// Exit Conditions
//  The Data byte is written into the controller
//
// External functions called
//  write_lcd_port(uint8_t)
//  lcd_cs_low(void)
//  lcd_cs_high(void)
//  lcd_d_c_high(void)
//  lcd_wr_low(void)
//  lcd_wr_high(void)
//     eliminated -  wait_12_nanoseconds(void)
// 
// Internal variables used
//    
// Constants
//
// History
//    03/05/2019 eliminated timing call. function call uses enough time
//
// Status
//   written, tested 
//
void ssd1963_write_data(uint8_t data_byte)
{
  write_lcd_port(data_byte);
  lcd_cs_low();
  lcd_d_c_high();
  lcd_wr_low();
  //wait_12_nanoseconds();
  lcd_wr_high();
  lcd_cs_high();
  //wait_12_nanoseconds();
}

//******************************************************************************
//
// Feb 8, 2018			Design Documents               ssd1963_read_data
//
// Reads a data byte from the SSD1963 controller.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  Data byte is passed as a parameter
//
// Exit Conditions
//  The Data byte is read from the controller
//
// External functions called
//  uint8_t read_lcd_port(void)
//  lcd_cs_low(void)
//  lcd_cs_high(void)
//  lcd_d_c_high(void)
//  lcd_rd_low(void)
//  lcd_rd_high(void)
//  wait_36_nanoseconds(void)
//  wait_100_nanoseconds(void)
// 
// Internal variables used
//    
// Constants
//
// History
//
// Status
//  written
//
uint8_t ssd1963_read_data(void)
{
uint32_t i;

  wait_100_nanoseconds();
  lcd_cs_low();
  lcd_d_c_high();
  lcd_rd_low();
  wait_36_nanoseconds();
  i = read_lcd_port();
  wait_36_nanoseconds();  
  lcd_rd_high();
  wait_36_nanoseconds(); 
  lcd_cs_high();
  wait_100_nanoseconds();
  return i;
}
//==============================================================================
//
//  Set port drection
//
// 
#define INPUT     0
#define OUTPUT    1

void ssd1963_set_port_direction( uint8_t direction )
{
  uint32_t volatile *GPIO_G_MODER = (uint32_t *) GPIOG_MODER_ADDRESS;
  
  if(direction) *GPIO_G_MODER = 0x00005555; //Set port to output
  else *GPIO_G_MODER = 0x00000000;          //Set port to input
}

//******************************************************************************
//
// Feb 8, 2018			Design Documents                     ssd1963_nop
//
// Executes the  SSD1963 nop command.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 nop command is executed
//
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_nop(void)
{
  ssd1963_write_command(0x00);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents              ssd1963_soft_reset
//
// Executes the  SSD1963 soft_reset command. The host processor must wait 5ms
// before sending any new commands to the SSD1963 following this command.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 soft reset command is executed
//
// Functions used
//  ssd1963_write_command
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written, tested, working
//
void ssd1963_soft_reset(void)
{
  ssd1963_write_command(0x01);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents          ssd1963_get_power_mode
//
// Executes the  SSD1963 get_power_mode command. Results are as follow:
//   A[6] Idle mode on/off (POR = 0)
//          0   Idle mode off
//          1   Idle mode on
//   A[5] Partial mode on/off (POR = 0)
//          0   Partial mode off
//          1   Partial mode on
//   A[4] Sleep mode on/off (POR = 0)
//          0   Sleep mode on
//          1   Sleep mode off
//   A[3] Display normal mode on/off (POR = 1)
//          0   Display normal mode off
//          1   Display normal mode on (Partial mode and vertical scroll off)
//   A[2] Display on/off (POR = 0)
//          0   Display is off
//          1   Display is on
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 nop command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint8_t ssd1963_get_power_mode(void)
{
  ssd1963_write_command(0x0A);
  return ssd1963_read_data();   
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_get_address_mode
//
// Executes the  SSD1963 get_address_mode command. Results are as follow:
//   A[7] Page Address Order (POR = 0)
//          0   Top to bottom
//          1   Bottom to top
//   A[6] Column address order (POR = 0)
//          0   Left to right
//          1   Right to left
//   A[5] Page column order (POR = 0)
//          0   Normal mode
//          1   Reverse mode
//   A[4] Line address order (POR = 0)
//          0   LCD refresh top to bottom
//          1   LCD refresh bottom to top
//   A[3] RGB/BGR order (POR = 0)
//          0   RGB
//          1   BGR
//   A[2] Display data latch data (POR = 0)
//          0   LCD refresh left to right
//          1   LCD refresh right to left
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 get address mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint8_t ssd1963_get_address_mode(void)
{
  ssd1963_write_command(0x0B);
  return ssd1963_read_data();   
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_get_display_mode
//
// Executes the  SSD1963 get_display_mode command. Results are as follow:
//   A[7] Vertical scrolling on/off (POR = 0)
//          0   Vertical scrolling is off
//          1   Vertical scrolling is on
//   A[5] Invert mode on/off (POR = 0)
//          0   Inversion is off
//          1   Inversion is on
//   A[2:0] Gamma curve selection (POR = 011)
//          000 Gamma curve 0
//          001 Gamma curve 1
//          010 Gamma curve 2
//          011 Gamma curve 3
//          100 Reserved
//          101 Reserved
//          110 Reserved
//          111 Reserved          
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 get display mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint8_t ssd1963_get_display_mode(void)
{
  ssd1963_write_command(0x0D);
  return ssd1963_read_data();   
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents  ssd1963_get_tear_effect_status
//
// Executes the  SSD1963 get_tear_effect_status command. Results are as follow:
//   A[7] Tearing effect line mode (POR = 0)
//          0   Tearing effect off
//          1   Tearing effect on       
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 get tear effect status command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint8_t ssd1963_get_tear_effect_status(void)
{
  ssd1963_write_command(0x0E);
  return ssd1963_read_data();   
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_enter_sleep_mode
//
// Executes the  SSD1963 enter_sleep_mode command. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 enter sleep mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_enter_sleep_mode(void)
{
  ssd1963_write_command(0x10);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_exit_sleep_mode
//
// Executes the  SSD1963 exit_sleep_mode command. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 exit sleep mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_exit_sleep_mode(void)
{
  ssd1963_write_command(0x11);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents      ssd1963_enter_partial_mode
//
// Executes the  SSD1963 enter_partial_mode command. Once enter partial mode is
// triggered, the Partial Display Mode window is decribed by the 
// set_partial_area(0x30). Once enter_normal_mode(0x13) is triggered, partial
// display mode will end.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 enter_partial_mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_enter_partial_mode(void)
{
  ssd1963_write_command(0x12);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents       ssd1963_enter_normal_mode
//
// Executes the  SSD1963 enter_normal_mode command. Normal mode is defined as
// partial display and vertical scroll mode off. That means the whole display
// area is isedd for image display.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The ssd1963 enter normal mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_enter_normal_mode(void)
{
  ssd1963_write_command(0x13);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_exit_invert_mode
//
// Executes the  SSD1963 exit_invert_mode command. This command causes the
// SSD1963 to stop inverting the image data on the display panel. The frame
// buffer remains unchanged.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The ssd1963 exit invert mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written, tested, working
//
void ssd1963_exit_invert_mode(void)
{
  ssd1963_write_command(0x20);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents       ssd1963_enter_invert_mode
//
// Executes the  SSD1963 enter_invert_mode command. This command causes the
// SSD1963 to invert the image data on the display panel. The frame buffer
// remains unchanged.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The ssd1963 enter invert mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written, tested, working
//
void ssd1963_enter_invert_mode(void)
{
  ssd1963_write_command(0x21);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents         ssd1963_set_gamma_curve
//
// Executes the  SSD1963 set_gamma_curve command. The choice is gamma curve 0
// to gamma curve 3. We have set the gamma curve to 1 for this project.
//   A[3:0]   Gamma curve selection (POR = 1000)
//   0000     Gamma curve 0
//   0001     Gamma curve 0
//   0010     Gamma curve 1
//   0100     Gamma curve 2
//   1000     Gamma curve 3
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The ssd1963 set gamma curve command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_gamma_curve(void)
{
  ssd1963_write_command(0x26);
  ssd1963_write_data(0x02);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents         ssd1963_set_display_off
//
// Executes the  SSD1963 set_display_off command. This command blanks the
// display panel. The frame buffer remains unchanged.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The set display off command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_display_off(void)
{
  ssd1963_write_command(0x28);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents          ssd1963_set_display_on
//
// Executes the  SSD1963 set_display_on command. This command shows the image
// on the display panel. The frame buffer remains unchanged.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The set display on command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_display_on(void)
{
  ssd1963_write_command(0x29);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents      ssd1963_set_column_address
//
// Set the column address of frame buffer accessed by host processor with the 
// read_memory_continue (0x3E) and write_memory_continue(0x3C). Start column
// must be less than or equal to end column.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  start_column supplied by calling program (POR = 0x0000)
//  end_column supplied by calling program (POR = 0x0000)
//
// Exit Conditions
//  Starting and ending column pointers set
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  WR_cycle()
// 
// History
//    03/08/2019  faster version, not toggling chip select each data byte
//
// Status
//  written, tested, working
//
/*
void ssd1963_set_column_address(uint16_t start_column, uint16_t end_column)
{
  ssd1963_write_command(0x2A);
  ssd1963_write_data(start_column >>8);
  ssd1963_write_data(start_column);
  ssd1963_write_data(end_column >>8);
  ssd1963_write_data(end_column);  
}
*/

extern void WR_cycle( void );

// faster version, do not use write data, keep cs low for all data bytes
void ssd1963_set_column_address(uint16_t start_column, uint16_t end_column)
{
  // set data port for direct use
  uint16_t volatile *GPIO_G_ODR = (uint16_t *) GPIOG_ODR_ADDRESS;
  
  // send command
  ssd1963_write_command(0x2A);
  // send start column 
  *GPIO_G_ODR = start_column >>8;    
  lcd_cs_low();
  lcd_d_c_high(); 
  WR_cycle();
  *GPIO_G_ODR = start_column;
  WR_cycle();  
   // send end column 
  *GPIO_G_ODR = end_column >>8;    
  WR_cycle();
  *GPIO_G_ODR = end_column;
  WR_cycle();

  lcd_cs_high();
}


//******************************************************************************
//
// Feb 8, 2018			Design Documents        ssd1963_set_row_address
//
// Set the row address of frame buffer accessed by host processor with the 
// read_memory_continue (0x3E) and write_memory_continue(0x3C). Start row
// must be less than or equal to end row.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  start_row supplied by calling program (POR = 0x0000)
//  end_row supplied by calling program (POR = 0x0000)
//
// Exit Conditions
//  Starting and ending row pointers set
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  WR_cycle()
// 
// History
//    03/08/2019  faster version, not toggling chip select each data byte
//
// Status
//  written, tested, working
//
void ssd1963_set_row_address(uint16_t start_row, uint16_t end_row)
{
  // set data port for direct use
  uint16_t volatile *GPIO_G_ODR = (uint16_t *) GPIOG_ODR_ADDRESS;  
  // send command
  ssd1963_write_command(0x2B);
  // send start row 
  *GPIO_G_ODR = start_row >>8;    
  lcd_cs_low();
  lcd_d_c_high(); 
  WR_cycle();
  *GPIO_G_ODR = start_row;
  WR_cycle();  
   // send end column 
  *GPIO_G_ODR = end_row >>8;    
  WR_cycle();
  *GPIO_G_ODR = end_row;
  WR_cycle();

  lcd_cs_high();
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents      ssd1963_write_memory_start
//
// Transfer image information from the host processor interface to the SSD1963
// starting at the location provided by the set_column_address(0x2A) and 
// set_row_address(0x2B) commands.
//
// If set_address_mode(0x36) A[5] = 0 (Normal mode)
//   The column and row address are reset to the Start Column (SC) and 
//   Start Row (SR), respectively.
//
//   Pixel Data is stored in frame buffer at (SC,SR). The column address is then 
//   incremented and pixels are written to the frame buffer until the column 
//   address equals the End Column (EC) value. The column address is then reset 
//   to SC and the row address is incremented. Pixels are written to the frame 
//   buffer until the row address equals the end row (ER) value and the column 
//   address equals the EC value, or the host processor sends another command.
//
//   If the number of pixels exceed (EC-SC+1)*(ER-SR+1) the extra pixels are 
//   ignored.
//
// If set_address_mode(0x36) A[5] = 1 (Reverse mode); 
//   The column and row address are reset to the Start Column (SC) and 
//   Start Row (SR), respectively.
//
//   Pixel Data is stored in frame buffer at (SC,SR). The row address is then 
//   incremented and pixels are written to the frame buffer until the row 
//   address equals the End row (ER) value. The row address is then reset 
//   to SR and the column address is incremented. Pixels are written to the 
//   frame buffer until the row address equals the end row (ER) value and the 
//   column address equals the EC value, or the host processor sends another 
//   command.
//
//   If the number of pixels exceed (EC-SC+1)*(ER-SR+1) the extra pixels are 
//   ignored.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The write memory start command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written, tested, working
//
void ssd1963_write_memory_start(void)
{
  ssd1963_write_command(0x2C);
}
//******************************************************************************
//
// Feb 8, 2018			Design Documents      ssd1963_read_memory_start
//
// Transfer image data information from the SSD1963 to the the host processor 
// starting at the location provided by the set_column_address(0x2A) and 
// set_row_address(0x2B) commands.
//
// If set_address_mode(0x36) A[5] = 0 (Normal mode)
//   The column and row address are reset to the Start Column (SC) and 
//   Start Row (SR), respectively.
//
//   Pixel Data is read from frame buffer at (SC,SR). The column address is then 
//   incremented and pixels are written to the frame buffer until the column 
//   address equals the End Column (EC) value. The column address is then reset 
//   to SC and the row address is incremented. Pixels are written to the frame 
//   buffer until the row address equals the end row (ER) value and the column 
//   address equals the EC value, or the host processor sends another command.
//
// If set_address_mode(0x36) A[5] = 1 (Reverse mode); 
//   The column and row address are reset to the Start Column (SC) and 
//   Start Row (SR), respectively.
//
//   Pixel Data is read from frame buffer at (SC,SR). The row address is then 
//   incremented and pixels are written to the frame buffer until the row 
//   address equals the End row (ER) value. The row address is then reset 
//   to SR and the column address is incremented. Pixels are written to the 
//   frame buffer until the row address equals the end row (ER) value and the 
//   column address equals the EC value, or the host processor sends another 
//   command.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The read memory start command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_read_memory_start(void)
{
  ssd1963_write_command(0x2E);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents        ssd1963_set_partial_area
//
// This command defines the Partial Display mode's display area. There are two
// parameters associated with this command, the first defines the Start Row (SR)
// and the second the End Row(ER). SR and ER referes to the Frame buffer Line 
// Pointer.
//
// SR[15:0] amd ER[15:0] cannot ne 0000h nor exceed the last vertical line 
// number.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  start_row supplied by calling program (POR = 0x0000)
//  end_row supplied by calling program (POR = 0x0000)
//
// Exit Conditions
//  Starting and ending row pointers set
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written, not used 
//
void ssd1963_set_partial_area(uint16_t start_row, uint16_t end_row)
{
  // set data port for direct use
  uint16_t volatile *GPIO_G_ODR = (uint16_t *) GPIOG_ODR_ADDRESS; 
  
  // send command
  ssd1963_write_command(0x30);
  // send start row 
  *GPIO_G_ODR = start_row >>8;    
  lcd_cs_low();
  lcd_d_c_high(); 
  WR_cycle();
  *GPIO_G_ODR = start_row;
  WR_cycle();  
   // send end column 
  *GPIO_G_ODR = end_row >>8;    
  WR_cycle();
  *GPIO_G_ODR = end_row;
  WR_cycle();

  lcd_cs_high(); 
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents            ssd1963_set_tear_off
//
// Executes the  SSD1963 set_tear_off command. This command inhibits the TE
// signal from being sent to the host processor.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The set tear off command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written, not used
//
void ssd1963_set_tear_off(void)
{
  ssd1963_write_command(0x34);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents             ssd1963_set_tear_on
//
// Executes the  SSD1963 set_tear_on command. This command enables the TE signal
// going to the host processor.
//
// A[0]: Tearing effect line mode (POR = 0)
//       0     The tearing effect output line consist of V-blanking information
//             only.
//       1     The tearing effect output line consist of both V-Blanking and 
//             H-blanking information by set_tear_scanline(0x44).
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The set tear on command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void  ssd1963_set_tear_on(uint8_t data_byte)
{
  ssd1963_write_command(0x35);
  ssd1963_write_data(data_byte);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents        ssd1963_set_address_mode
//
// Set the read order from host processor to frame buffer by A[7:5] amd A[3] and
// from frame buffer to the display panel by A[2:0] and A[4].
//
// A[7]: Row address order (POR = 0)
// This bit controls the order that rows of data are transfered from the host 
// processor to the SSD1963's frame buffer.
//     0      Top to bottom, pages transfered from Start Row to End Row
//     1      Bottom to top, pages transfered from End Row to Start Row
//
// A[6]: Column address order (POR = 0)
// This bit controls the order that columns of data are transfered from the host
// processor to the SSD1963's frame buffer.
//     0      Left to right, columns transfered from Start column to End column
//     1      Right to left, columns transfered from End column to Start column
// 
// A[5]: Page/Column order (POR = 0)
// This bit controls the order that columns of data are transfered from the host
// processor to the SSD1963's frame buffer.
//     0      Normal Mode
//     1      Reverse Mode
//
// A[4]: Line address order (POR = 0)
// This bit controls the display panel's horizontal line refresh order. The 
// image shown on the display panel is unaffected regardless of the bit setting.
//     0      LCD refresh from top line to bottom line.
//     1      LCD refresh from bottom line to top line.
//
// A[3]: RGB/BGR order (POR = 0)
// This bit controls the RGB data order transferred from the SSD1963's frame 
// buffer to the display panel.
//     0      RGB
//     1      BGR
//
// A[2]: Display data latch data (POR = 0)
// This bit controls the display panel's vetical line data latch order. The 
// image shown on the display panel is unaffected, regardless of the bit setting
//     0      LCD refresh from left side to right side.
//     1      LCD refresh from right side to left side.
//
// A[1]: Flip Horizontal (POR = 0)
// This bit flips the image shown on the display panel left to right. No change
// is made to the frame buffer
//     0      Normal
//     1      Flipped
//
// A[0]: Flip Vertical (POR = 0)
// This bit flips the image shown on the display panel top to bottom. No change
// is made to the frame buffer
//     0      Normal
//     1      Flipped
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  parameter supplied by calling program
//
// Exit Conditions
//  The set address mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_address_mode(uint8_t data_byte)
{
  ssd1963_write_command(0x36);
  ssd1963_write_data(data_byte);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents          ssd1963_exit_idle_mode
//
// Executes the  SSD1963 exit_idle_mode command. Full color depth is restored
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The exit idle mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_exit_idle_mode(void)
{
  ssd1963_write_command(0x38);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents         ssd1963_enter_idle_mode
//
// Executes the  SSD1963 enter_idle_mode command. In idle mode, color dept is
// reduced. Colors shown on the display panel using the msb of each R, G and B
// color components in the frame buffer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The enter idle mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_enter_idle_mode(void)
{
  ssd1963_write_command(0x39);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents   ssd1963_write_memory_continue
//
// Transfer image information from the host processor interface to the SSD1963
// from the last write_memory_continue(0x3C) or write_memory_start(0x2C)
//
// If set_address_mode(0x36) A[5] = 0 (Normal mode)
//   Data is writtten continuing from the pixel location after the write range 
//   of the previous write_memory_start or write_memory_continue. The column 
//   address is then incremented and pixels are written into the frame buffer 
//   until the column address equals the  End Column value. The column address 
//   is then reset to start column and the row address is incrmented. Pixels are
//   written to the frame buffer until the row address equls the end row value 
//   and the column address equsls the end column value, or the host processor 
//   sends another commnd.
//
//   If the number of pixels exceed (EC-SC+1)*(ER-SR+1) the extra pixels are 
//   ignored.
//
// If set_address_mode(0x36) A[5] = 1 (Reverse mode); 
//   Data is writtten continuing from the pixel location after the write range 
//   of the previous write_memory_start or write_memory_continue. The row 
//   address is then incremented and pixels are written into the frame buffer 
//   until the row address equals the  End Row value. The Row address 
//   is then reset to start row and the column address is incrmented. Pixels are
//   written to the frame buffer until the column address equls the end column  
//   value and the row address equsls the end row value, or the host processor 
//   sends another commnd.
//
//   If the number of pixels exceed (EC-SC+1)*(ER-SR+1) the extra pixels are 
//   ignored.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The write memory continue command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_write_memory_continue(void)
{
  ssd1963_write_command(0x3C);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents    ssd1963_read_memory_continue
//
// Transfer image information from the SSD1963 to the host processor from the 
// last write_memory_continue(0x3C) or write_memory_start(0x2C)
//
// If set_address_mode(0x36) A[5] = 0 (Normal mode)
//   Pixels are read continuing from the pixel location after the read range 
//   of the previous read_memory_start or read_memory_continue. The column 
//   address is then incremented and pixels are read from the frame buffer 
//   until the column address equals the  End Column value. The column address 
//   is then reset to start column and the row address is incrmented. Pixels are
//   read from the frame buffer until the row address equls the end row value 
//   and the column address equsls the end column value, or the host processor 
//   sends another commnd.
//
// If set_address_mode(0x36) A[5] = 1 (Reverse mode); 
//   Pixels are read continuing from the pixel location after the read range 
//   of the previous read_memory_start or read_memory_continue. The row 
//   address is then incremented and pixels are read from the frame buffer 
//   until the row address equals the  End Row value. The Row address 
//   is then reset to start row and the column address is incrmented. Pixels are
//   read from the frame buffer until the column address equls the end column  
//   value and the row address equsls the end row value, or the host processor 
//   sends another commnd.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The read memory continue command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_read_memory_continue(void)
{
  ssd1963_write_command(0x3E);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents       ssd1963_set_tear_scanline
//
// TE signal is sent from the SSD1963 to the host processor when the display
// panel refresh reaches the designated scanline.
//
// N[15:8] High byte of the scanline (POR = 0x00)
// N[7:0] Low byte of the scanline (POR = 0x00)
//
// Valid setting fir TE signal is 0x0000 and 0x0002 to 0xFFFF
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  parameter supplied by calling program
//
// Exit Conditions
//  The set tear scanline command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_tear_scanline(uint16_t scanline)
{
  ssd1963_write_command(0x44);
  ssd1963_write_data(scanline >>8);
  ssd1963_write_data(scanline);
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents            ssd1963_get_scanline
//
// Gets the current scanline
//
// N[15:8] High byte of the current scanline (POR = 0x00)
// N[7:0] Low byte of the current scanline (POR = 0x00)
//
// Valid setting fir TE signal is 0x0000 and 0x0002 to 0xFFFF
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  parameter supplied by calling program
//
// Exit Conditions
//  The get scanline command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint16_t ssd1963_get_scanline(uint16_t scanline)
{
  ssd1963_write_command(0x45);
  return (ssd1963_read_data() << 8) + ssd1963_read_data();
}
//******************************************************************************
//
// Feb 9, 2018			Design Documents   ssd1963_read_device_desciptor
//
// Gets the device descriptor block from the SSD1963. This program is used for
// diagnostic to make sure the firmware could read the display
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  parameter supplied by calling program
//
// Exit Conditions
//  The device descriptor block is put in memory
//    ddb1 always = 0x01 Supplier ID high byte
//    ddb2 always = 0x57 Supplier ID low byte
//    ddb3 always = 0x61 Product ID
//    ddb4 always = 0x01 Revision code
//    ddb5 always = 0xff Exit code
//
//    if all the data is good, the program returns a 1, else it returns a 0.
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written, tested, working
//

// ========================================
//  
//  Read device description data
//
uint16_t ssd1963_read_device_desciptor(void)
{
uint8_t ddb1;
uint8_t ddb2;
uint8_t ddb3;
uint8_t ddb4;
uint8_t ddb5;
  
  ssd1963_write_command(0xA1);
  ddb1 = ssd1963_read_data();
  ddb2 = ssd1963_read_data();
  ddb3 = ssd1963_read_data();
  ddb4 = ssd1963_read_data();
  ddb5 = ssd1963_read_data();
  
  
  if (ddb1 != 0x01) return 0;
  if (ddb2 != 0x57) return 0;
  if (ddb3 != 0x61) return 0;
  if (ddb4 != 0x01) return 0;
  if (ddb5 != 0xFF) return 0;
  
  return 1;
}
//******************************************************************************
//
// Feb 10, 2018			Design Documents            ssd1963_set_lcd_mode
//
// Set the LCD panel mode and resolution.
//  parameter 1
//    A[5]: TFT panel data width (POR = 0)
//          0     18 bit
//          1     24 bit
//    A[4]: TFT color depth enhancement enable (POR = 0)
//          0     Disable FRC or dithering
//          1     Enable FRC or dithering forcolor depth enhancement
//          If the panel data width was set to 23 bit, FRC and dithering feature
//          will be disabled uatomatically regardless of the value of this
//          register.
//    A[3]: TFT FRC enable (POR = 0)
//          0     TFT dithering enable
//          1     TFT FRC enable
//    A[2]: LSHIFT polarity (POR = 0). Set the dot clock pulse polarity.
//          0     Data latch on falling edge
//          1     Data latch on rising edge
//    A[1]  LLINE polarity (POR = 0) Set the horizontal sync pulse polarity.
//          0     Active low
//          1     Active high
//    A[0]  LFRAME polarity (POR = 0). Set the vertical sync pulse polarity.
//          0     Active low
//          1     Active high
//
// parameter 2
//    B[6:5]: TFT type (POR = 01)
//          00    TFT mode
//          01    TFT mode
//          10    Serial RGB mode
//          11    serial RGB + dummy mode
//
// parameter 3
//    HDP[10:0] Horizonatal panel size (POR = 01001111111 = 640)
//
// parameter 4
//    VDP[10:0] Vertical panel size (POR = 00111011111 = 479)
//
// parameter 5
//    G[5:3]: Even line RGB sequence for serial TFT interface (POR = 000)
//          000   RGB
//          001   RBG
//          010   GRB
//          011   GBR
//          100   BRG
//          101   BGR
//          110   Reserved
//          111   Reserved
//
//    G[2:0]: Odd line RGB sequence for serial TFT interface (POR = 000)
//          000   RGB
//          001   RBG
//          010   GRB
//          011   GBR
//          100   BRG
//          101   BGR
//          110   Reserved
//          111   Reserved
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set lcd mode command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_lcd_mode(uint8_t parm1,uint8_t parm2, uint16_t parm3,
                          uint16_t parm4,uint8_t parm5)
{
  ssd1963_write_command(0xB0);
  ssd1963_write_data(parm1);
  ssd1963_write_data(parm2);  
  ssd1963_write_data(parm3 >>8);
  ssd1963_write_data(parm3);
  ssd1963_write_data(parm4 >>8);
  ssd1963_write_data(parm4);
  ssd1963_write_data(parm5);
}
//******************************************************************************
//
// Feb 11, 2018			Design Documents         ssd1963_set_hori_period
//
// Set the front [porch and back porch
//  parameter 1
//    HT[10:0] Horizonatal total period(POR = 01010101111 = 687)
//
// parameter 2
//    HPS(10:0]: Non displayed period between the start of the horizontal sync 
//    (LLINE) signal and the first display data (POR = 000000100000 = 32)
//
//    For TFT: Horizontal Sync Pulse Start Position = HPS pixels
//
//    For serial TFT: Horizontal Sync Pulse Start Position = HPS pixels + LPSPP
//    subpixels.
//
// parameter 3
//    HPW[6:0] Set the horizontal sync pulse width (LLINE) in pixel clock. 
//    (POR = 0000111 = 7). Horizontal sync pulse = (HPW + 1) pixels
//
// parameter 4
//    LPS(10:0) Set the horizontal sync pulse (LLINE) start location in pixel 
//    clock. (POR = 0)
//
// parameter 5
//    LPSPP[1:0]: Set the horizontal sync pulse subpixel start position for 
//    serial TFT interface (POR = 00)
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set hori period command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_hori_period(uint16_t parm1,uint16_t parm2, uint8_t parm3,
                          uint16_t parm4,uint8_t parm5)
{
  ssd1963_write_command(0xB4);
  ssd1963_write_data(parm1>>8);
  ssd1963_write_data(parm1);  
  ssd1963_write_data(parm2 >>8);
  ssd1963_write_data(parm2);
  ssd1963_write_data(parm3);
  ssd1963_write_data(parm4>>8);
  ssd1963_write_data(parm4);  
  ssd1963_write_data(parm5);
}
//******************************************************************************
//
// Feb 11, 2018			Design Documents         ssd1963_set_vert_period
//
// Set the vertical blanking interval between last scan lineand next LFRAME 
// pulse
//
//  parameter 1
//    VT[10:0]: Vertical total (display + non-display) period in lines (POR = 
//    00111101111 = 495)
//
// parameter 2
//    VPS(10:0]: The non display period in lines between the start of the frame 
//    and the first display data in line. (POR = 00000000100 = 4)
//
// parameter 3
//    VPW(6:0): Set the vertical sync pulse width (LFRAME) in lines. 
//    (POR = 000001). Vertical Sync Pulse Width = (VPW + 1) lines.
//
// parameter 4
//    FPS[10:0]: Vertical sync pulse (LFRAME) start location in lines. 
//    (POR = 0). Vertical Display Period Start Position = FLS lines.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set vert period command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_vert_period(uint16_t parm1,uint16_t parm2, uint8_t parm3,
                          uint16_t parm4)
{
  ssd1963_write_command(0xB6);
  ssd1963_write_data(parm1>>8);
  ssd1963_write_data(parm1);  
  ssd1963_write_data(parm2>>8);
  ssd1963_write_data(parm2);  
  ssd1963_write_data(parm3);
  ssd1963_write_data(parm4>>8);
  ssd1963_write_data(parm4);  
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents           ssd1963_set_post_proc
//
// Set the image post processor
//
//  parameter 1
//    A[7:0]: Set the contrast value (POR = 01000000 = 64)
//
// parameter 2
//    B[7:0]: Set the brightness value (POR = 10000000 = 128)
//
// parameter 3
//    Set the satuaration value (POR = 01000000 = 64)
//
// parameter 4
//    Post processpr Enable (POR = 0)
//       0     Disable the post processor
//       1     Enable the post processor
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set post proc command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written, not in use
//
void ssd1963_set_post_proc(uint8_t parm1,uint8_t parm2, uint8_t parm3,
                          uint8_t parm4)
{
  ssd1963_write_command(0xBC);
  ssd1963_write_data(parm1);  
  ssd1963_write_data(parm2);  
  ssd1963_write_data(parm3);
  ssd1963_write_data(parm4);  
}
//******************************************************************************
//
// Feb 8, 2019			Design Documents           ssd1963_get_post_proc
//
// Read the image post processor
//
//  parameter 1
//    A[7:0]: the contrast value (POR = 01000000 = 64)
//
// parameter 2
//    A[15:8]: the brightness value (POR = 10000000 = 128)
//
// parameter 3
//    A[23:16]: the satuaration value (POR = 01000000 = 64)
//    Set 
//
// parameter 4
//    A[32-24] Post processpr Enable (POR = 0)
//       0     Disable the post processor
//       1     Enable the post processor
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  No parameters needed
//
// Exit Conditions
//  The set post proc command is read
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
uint32_t ssd1963_get_post_proc(void)
{
uint32_t contrast;
uint32_t brightness;
uint32_t saturation;
uint32_t enable;

  ssd1963_write_command(0xBD);
  
  contrast =    ((uint32_t) ssd1963_read_data()) &       0x000000FF;   
  brightness = (((uint32_t) ssd1963_read_data()) << 8) & 0x0000FF00;  
  saturation = (((uint32_t) ssd1963_read_data()) << 16)& 0x00FF0000;   
  enable =     (((uint32_t) ssd1963_read_data()) << 24)& 0xFF000000;
  return contrast + brightness + saturation + enable;
}
// =============================================================================
//  10/30/2019            set brightness
//
//   sets brightness to the level sent.  0 to 255
//    brightness above 200 looks washed out, not much brighter
//    starting controller default is 128
//    liniting values 1 to 200
//   
void ssd1963_set_brightness( uint8_t bright_level )
{
  uint32_t post_proc_val;
  uint8_t contrast;
  uint8_t brightness;
  uint8_t saturation;
  uint8_t enable;
  
  // get current settings
  post_proc_val = ssd1963_get_post_proc();
  // modify brightenss to new value, keep the others
  contrast = post_proc_val & 0xFF000000 >> 24;
  brightness = bright_level;
  saturation = post_proc_val & 0x0000FF00 >> 8;
  enable  = 1;
  // send post proc values to the controller
  ssd1963_set_post_proc( contrast, brightness, saturation, enable );
  
}

//******************************************************************************
//
// Feb 12, 2018			Design Documents           ssd1963_set_pwm_conf
//
// Set the PWM configuration
//
//  parameter 1
//    PWMF[7:0]: Set the PWM frequency in system clock (POR = 00000000) PWM
//    signal frequency = PLL clock/(256*(PWMF[7:0]+1))/256
//
// parameter 2
//    PWM[7:0]: Set tghe PWM suty cycle (POR = 00000000)
//    PWM duty cycle = PWM[7:0]/256 for DBC disable (0xD0] A0 = 0
//    If DBC enable (0xD0] A0 = 1, these parameters will be ignored.
//    Note: PWM always 0 if PWM[7:0] = 0x00
//
// parameter 3
//    C[3] PWM configuration (POR = 0)
//          0     PWM controlled by host
//          1     PWM controlled by DMC
//
//    C[0]  PWM enable (POR = 0)
//          0     PWM disabled
//          1     PWM enabled
//
// parameter 4
//    D[7:0]: DBC manual brighness (POR = 00000000)
//    Set the manual brightness level. When Manual Brightness Mode (0xD0) A[6} 
//    is enabled, the final DBC duty cycle output will be multiplied by this 
//    value/255. PWM duty cycle = DBC output*D[7:0]/255.
//          00 Dimmest
//          FF Brightest
//
// parameter 5
//    E[7:0]: DBC minimum brightness (POR = 00000000)
//    Set the minimum brightness level;. When Manual brighness Mpde (0xD0) A6 
//    is enabled, DBC duty cycle output will be limited by this value. This will
//    prevent the backlight from being too dark or off.
//          00 Dimmest
//          FF Brightest
//
// parameter 6
//    F[3:0] Brighness prescaler (POR = 0000)
//    Set the brighness prescaler to control how gradually the manual brightness 
//    is changed between different levels. There is a filter will undergo a 
//    number of interations before the manual brghness is saturaated. This 
//    parameter is valid when Transistion Effect enable (0xD0) A5 = 1.
//
//    The iteration ratio = system frequencyt/Divcode/32768
//          F[3:0]     Divcode
//          0000       off
//          0001       1
//          0010       2
//          0011       3
//          0100       4
//          0101       6
//          0110       8
//          0111       12
//          1000       16
//          1001       24
//          1010       32
//          1011       48
//          1100       64
//          1101       96
//          1110       128
//          1111       192
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set pwm conf command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_pwm_conf(uint8_t parm1, uint8_t parm2, uint8_t parm3,
                          uint8_t parm4, uint8_t parm5, uint8_t parm6)
{
  ssd1963_write_command(0xBE);
  ssd1963_write_data(parm1);
  ssd1963_write_data(parm2);  
  ssd1963_write_data(parm3);
  ssd1963_write_data(parm4);  
  ssd1963_write_data(parm5);
  ssd1963_write_data(parm6);  
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents            ssd1963_set_dbc_conf
//
// Set the Dynamic Backlight Control configuration
//
//  parameter 1
//    A[6]:   DBC Manual Brightness enable (POR = 1)
//            0     Enable
//            1     Disable
//
//    A[5]:   Transition effect (POR = 0)
//            0     Transition effect disabled
//            1     Transition effect enabled
//
//    A[3:2]: Transition effect (POR = 0)
//            00    DBC is disabled
//            01    Consdervative mode
//            10    Normal mode
//            11    Aggresive mode
//
//    A[0]:   Master enable of DBC (POR = 0)
//            0     DBC disabled
//            1     DBC enabled
//
//  The hardware pin PWM is the output signal from the SSD1963 to the system 
//  backlight driver. So you should configure PWM module before enabling DBC.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set dbc conf command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_dbc_conf(uint8_t parm1)
{
  ssd1963_write_command(0xD0);
  ssd1963_write_data(parm1);
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents              ssd1963_set_dbc_th
//
// Set the threshold for each level of power saving
//
//  parameter 1
//    DBC_TH1[16:0] Threshold setting for the conservative mode of DBC (POR = 0)
//
//  parameter 2
//    DBC_TH2[16:0] Threshold setting for the normal mode of DBC (POR = 0)
//
//  parameter 3
//    DBC_TH3[16:0] Threshold setting for the aggressive mode of DBC (POR = 0)
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set dbc th command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_dbc_th(uint32_t parm1, uint32_t parm2, uint32_t parm3)
{
  ssd1963_write_command(0xD4);
  ssd1963_write_data(parm1>>16);
  ssd1963_write_data(parm1>>8);
  ssd1963_write_data(parm1);
  ssd1963_write_data(parm2>>16);
  ssd1963_write_data(parm2>>8);
  ssd1963_write_data(parm2);
  ssd1963_write_data(parm3>>16);
  ssd1963_write_data(parm3>>8);
  ssd1963_write_data(parm3);
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents                 ssd1963_set_pll
//
// Start the PLL. Before the start, the system was operated with the crysta 
// oscillator or clock input.
//
//  parameter 1
//    A[1]:   Lock PLL (POR = 0)
//    After PLL enabled for 100 uSec, can start to lock PLL
//            0     Use reference clock as system clock
//            1     Disable
//
//    A[0]:   Enable PLL (POR = 0)
//            0     disable PLL
//            1     enable PLL
//
// Before enabling PLL, the PLL setting (0xE2) has to be configured first. 
// After PLL enabled for 100 uS, you can startto lock PLL. SDD1963 needed to 
// switch to PLL output as system clock after PLL is locked.
//
// SSD1963 is operating under reference clock before PLL is locked, registers 
// cannot be set faster than half of the reference frequency. For instance, 
// SSD1963 with a 10 MHz reference clock is not allowed to be programmed higher 
// than 5M words/s.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set pll command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_pll(uint8_t parm1)
{
  ssd1963_write_command(0xE0);
  ssd1963_write_data(parm1);
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents              ssd1963_set_pll_mn
//
// Set the MN of PLL
//
//  parameter 1
//    M[7:0] Multiplier (M) of PLL (POR = 00101101 = 45
//
//  parameter 2
//    N[4:0] Divider (N) of PLL (POR = 00011)(POR = 00011 = 3)
//
//  parameter 3
//    C[2] Effectuate MN value (POR = 0)
//            0     Ignore the multiplier (N) and divider (N) values
//            1     Effectuate the multiplier and divider value
//
// VCO = Reference input clock x (M + 1)
// PLL frequency = VCO/(N + 1)
//
// 150 MHz < VCO < 800 MHz
//
// For a 10 MHz reference clock to obtain 100 MHz PLL frequency, user cannot 
// program M = 19 and N = 1. The setting in this situation is setting M = 29 
// and N = 2, where 10 x 30/3 = 100 MHz.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set pll mn command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_pll_mn(uint8_t parm1, uint8_t parm2, uint8_t parm3)
{
  ssd1963_write_command(0xE2);
  ssd1963_write_data(parm1);
  ssd1963_write_data(parm2);
  ssd1963_write_data(parm3);
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents          ssd1963_get_pll_status
//
// Gets the PLL status
//   A[2] PLL Lock
//          0   Not locked
//          1   Locked
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 get pll status command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  uint8_t ssd1963_read_data(void)
// 
// History
//
// Status
//  written
//
uint8_t ssd1963_get_pll_status(void)
{
  ssd1963_write_command(0xE4);
  return ssd1963_read_data();   
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents          ssd1963_set_deep_sleep
//
// Set deep sleep mode. Pll is stopped. Needs to issue 2 dummy read to exit 
// Deep Sleep mode
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The SSD1963 set deep sleep command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_deep_sleep(void)
{
  ssd1963_write_command(0xE5);
}
//******************************************************************************
//
// Feb 12, 2018			Design Documents         ssd1963_set_lshift_freq
//
// Set the LSHIFT (pixel clock frequency.
//
// For parallel LCD interface:
// Configure the pixel clock to PLL frequency x ((LCDC_FPR + 1)/2**20
//
// To obtain PCLCK = 5.3 MHz with PLL Frequency = 100 MHz
// 5.3 MHz = 100 MHz * (LCDC_FPR + 1)/2**20
// LCDC_FPR = 55574
//
//  parameter 1
//    LCDC_FPR[19:0] pixel clock frequency (POR = 01111111111111111111)
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set lshift freq command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_lshift_freq(uint32_t parm1)
{
  ssd1963_write_command(0xE6);
  ssd1963_write_data(parm1>>16);
  ssd1963_write_data(parm1>>8);
  ssd1963_write_data(parm1);
}
//******************************************************************************
//
// Feb 12, 2018		       Design Documents ssd1963_set_pixel_data_interface
//
// Set the pixel data format to 8-bit/9 bit/12 bit/16 bit/16 bit(565)/18 bit/
// 24 bit in the parallel host processor interface. This command is used for 
// display data only as the command format is always 8 bit.
//
// The unused data bus will be driven to ground by SSD1963 so don't connect the 
// unused data bus to the MCU.
//
//  parameter 1
//    A[2:0]:   Pixel Daa Interface Format (POR = 101)
//            000   8 bit
//            001   12 bit
//            010   16 bit packed
//            011   16 bit (565 format)
//            100   18 bit
//            101   24 bit
//            110   9 bit
//            111   Reserved
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the calling program
//
// Exit Conditions
//  The set pixel data interface command is executed
// 
// External functions called
//  ssd1963_write_command(uint8_t)
//  ssd1963_write_data(uint8_t)
// 
// History
//
// Status
//  written
//
void ssd1963_set_pixel_data_interface(uint8_t parm1)
{
  ssd1963_write_command(0xF0);
  ssd1963_write_data(parm1);
}
