//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Fen 20, 2018             Design Documents                      CSI_SWITCHES.c
//
//			Routines for switches
//                   for the Large Display project
//
// Programmer
//  Jerry Wanger, Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//    02/20/2018
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
// process_switch_s1          Handles Switch S1
// process_switch_s2          Handles Switch S2
// process_switch_s3          Handles Switch S3
// process_switch_s4          Handles Switch S4
// process_switch_s5          Handles Switch S5
// process_switch_s6          Handles Switch S6
// process_encoder_1          Handles Encoder 1
// FT857D_encoder_right       Handles FT857 encoder right
// process_encoder_2          Handles Encoder 2
// FT857D_encoder_left        Handles FT857 encoder left
// process_ptt                Handles PTT
// process_switches           sends data to radio body
// deactivate_keypress        stop sending a keypress to the radio
// activate_keypress          allow sending a keypress to the radio
// deactivate_all_keypress    stop sending all keypresses to the radio
// activate_all_keypress      allow sending all keypresses to the radio
// setup_keypad_switches      setup a key buffer to send keypress data
// volume_updown              program control of volume, simulates knob
// channel_updown             program control of channel, simulates keypresses
// short_press_timer          timer sends a key release, after countdown
// knob_nav_control           allows knob to be used instead of button pushes
// FB_button_write            writes CPS button information to the EEPROM
// FB_button_read             reads CPS button information from the EEPROM
// FB_button_compare          compare two sets of button data for a match
// get_button_assign          get which buttons to use for touch screen controls
// radio_to_panel_button      translate radio buttons to panel 
//
// Status: written, not tested, not used
// contact_page_down         program control of menu list, simulates keypresses

//==============================================================================
#include <stdint.h>
#include "CSI_GPIO.h"
#include "CSI_SWITCHES.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_SCREENS.h"
#include "CSI_i2c.h"
#include "CSI_eeprom.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_Menus.h"
#include "CSI_MC59.h"

// *** reference info from CSI_GPIO.c ****************************************** 
// read_switch_s1                  PB0  Encoder left
// read_switch_s2                  PB1  Encoder right
// read_switch_s3                  PE11 Left soft key  - P1
// read_switch_s4                  PE10 Next to left   - P2
// read_switch_s5                  PE9  Next to right  - P3
// read_switch_s6                  PE8  Right soft key - P4         
// read_rotary_encoder_1_0         PE14 Encoder right
// read_rotary_encoder_1_1         PE15 Encoder right
// read_rotary_encoder_2_0         PE12 Encoder left
// read_rotary_encoder_2_1         PE13 Encoder left
// ****************************************************************************
//
// SWITCH Variables
uint32_t key_number_1;
uint32_t key_number_2;
uint32_t key_type;
uint32_t key_active_flag;
uint32_t key_release;

// FT857D variables
uint32_t FT857D_key_released;
uint32_t FT857D_left_encoder_switch;
uint32_t FT857D_right_encoder_switch;
uint32_t FT857D_p1_switch;
uint32_t FT857D_p2_switch;
uint32_t FT857D_p3_switch;
uint32_t FT857D_p4_switch;
uint32_t FT857D_encoder_rt_cw;
uint32_t FT857D_encoder_rt_ccw;
uint32_t FT857D_encoder_lft_cw;
uint32_t FT857D_encoder_lft_ccw;

// from UART1 
uint8_t  key_buffer[32];
uint32_t key_buffer_flag; 
uint8_t  ack_buffer[32]; 
uint32_t ack_buffer_flag;

//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s1
//
// Process switch on rotary encoder 1. This program must be called from a 10
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If key is pressed, then s1_press_flag will be set true
//  If key is released, then s1_release_flag will be set true
//
// External functions called
//  read_switch_s1
//
// External variables used
//
// Internal variables used
//  s1_state
//  s1_timer
//
// Constants
//
// History
//          GPIO switches changed to have 1 for a keydown 0 for keyup
//          Comments need to be revised.
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s1(void)
{
  static uint32_t s1_state;
  static uint32_t s1_timer;  
  
  // State 0 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s1_state == 0) 
  {
    if (read_switch_s1())
    {
      s1_timer = 3;
      s1_state++;
    }
    return;
  }

  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s1_state == 1)
  {
    if (s1_timer) s1_timer--;
    if (!s1_timer)
    {
      CS800_buttons[1].press_flag = 1;
      FT857D_left_encoder_switch = 1;
      s1_timer = 2;      
      s1_state++;
      return;
    }
    if (!read_switch_s1())
    {
      s1_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s1_state == 2)
  {
    if (s1_timer) s1_timer--;
    if (!s1_timer)
    {
      if (!read_switch_s1())
      {
        CS800_buttons[1].release_flag = 1;
        FT857D_key_released = 1;
        s1_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program.
  // Set the state back to state 0
  s1_state = 0;
  error_log( 3002 );
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s2
//
// Process switch on rotary encoder 2. This program must be called from a 10
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If key is pressed, then s2_press_flag will be set true
//  If key is released, then s2_release_flag will be set true
//
// External functions called
//  read_switch_s2
//
// External variables used
//  s2_press_flag
//  s2_release_flag
//
// Internal variables used   
//  s2_state
//  s2_timer
//
// Constants
//
// History
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s2(void)
{
  static uint32_t s2_state;
  static uint32_t s2_timer;  

  // State 0 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s2_state == 0) 
  {
    if (read_switch_s2())
    {
      s2_timer = 3;
      s2_state++;
    }
    return;
  }
  
  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s2_state == 1)
  {
    if (s2_timer) s2_timer--;
    if (!s2_timer)
    {
      FT857D_right_encoder_switch = 1;
      if(r_knob_for_nav) CS800_buttons[6].press_flag = 1; 
      else CS800_buttons[2].press_flag = 1;
      s2_timer = 2;      
      s2_state++;
      return;
    }
    if (!read_switch_s2())
    {
      s2_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s2_state == 2)
  {
    if (s2_timer) s2_timer--;
    if (!s2_timer)
    {
      if (!read_switch_s2())
      {
        FT857D_key_released = 1;
        if(r_knob_for_nav) CS800_buttons[6].release_flag = 1;      
        else CS800_buttons[2].release_flag = 1;
        s2_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program.
  // Set the state back to state 0
  s2_state = 0;
  error_log( 3002 );
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s3
//
// Process front panel switch s3. This program must be called from a 10 
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If key is pressed, then s3_press_flag will be set true
//  If key is released, then s3_release_flag will be set true
//
// External functions called
//  read_switch_s3
//
// External variables used
//  s3_press_flag
//  s3_release_flag
//
// Internal variables used   
//  s3_state
//  s3_timer
//
// Constants
//
// History
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s3(void)
{
  static uint32_t s3_state;
  static uint32_t s3_timer;  
  
  // State 0 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s3_state == 0) 
  {
    if (read_switch_s3())
    {
      s3_timer = 3;
      s3_state++;
    }
    return;
  }

  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s3_state == 1)
  {
    if (s3_timer) s3_timer--;
    if (!s3_timer)
    {
      CS800_buttons[3].press_flag = 1;
      FT857D_p1_switch = 1;
      // log_event( "P1 press"); // for debug
      s3_state++;
      s3_timer = 2;      
      return;
    }
    if (!read_switch_s3())
    {
      s3_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s3_state == 2)
  {
    if (s3_timer) s3_timer--;
    if (!s3_timer)
    {
      if (!read_switch_s3())
      {
        CS800_buttons[3].release_flag = 1;
        FT857D_key_released = 1;
        // log_event( "P1 rel");  // for debug    
        s3_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program.
  // Set the state back to state 0
  s3_state = 0;
  error_log( 3002 );
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s4
//
// Process front panel switch s4. This program must be called from a 10 
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If key is pressed, then s4_press_flag will be set true
//  If key is released, then s4_release_flag will be set true
//
// External functions called
//  read_switch_s4
//
// External variables used
//  s4_press_flag
//  s4_release_flag
//
// Internal variables used   
//  s4_state
//  s4_timer
//
// Constants
//
// History
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s4(void)
{
  static uint32_t s4_state;
  static uint32_t s4_timer;  

  // State 0 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s4_state == 0) 
  {
    if (read_switch_s4())
    {
      s4_timer = 3;
      s4_state++;
    }
    return;
  }

  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s4_state == 1)
  {
    if (s4_timer) s4_timer--;
    if (!s4_timer)
    {
      CS800_buttons[4].press_flag = 1;
      FT857D_p2_switch = 1;
      s4_timer = 2;      
      s4_state++;
      return;
    }
    if (!read_switch_s4())
    {
      s4_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s4_state == 2)
  {
    if (s4_timer) s4_timer--;
    if (!s4_timer)
    {
      if (!read_switch_s4())
      {
        CS800_buttons[4].release_flag = 1;
        FT857D_key_released = 1;
        s4_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program.
  //  Set the state back to state 0
  s4_state = 0;
  error_log( 3002 );  
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s5
//
// Process front panel switch s5. This program must be called from a 10 
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If key is pressed, then s5_press_flag will be set true
//  If key is released, then s5_release_flag will be set true
//
// External functions called
//  read_switch_s5
//
// External variables used
//  s5_press_flag
//  s5_release_flag
//
// Internal variables used   
//  s5_state
//  s5_timer
//
// Constants
//
// History
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s5(void)
{
  static uint32_t s5_state;
  static uint32_t s5_timer;

  // State 0 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s5_state == 0) 
  {
    if (read_switch_s5())
    {
      s5_timer = 3;
      s5_state++;
    }
    return;
  }

  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s5_state == 1)
  {
    if (s5_timer) s5_timer--;
    if (!s5_timer)
    {
      CS800_buttons[5].press_flag = 1;
      FT857D_p3_switch = 1;
      s5_timer = 2;      
      s5_state++;
      return;
    }
    if (!read_switch_s5())
    {
      s5_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s5_state == 2)
  {
    if (s5_timer) s5_timer--;
    if (!s5_timer)
    {
      if (!read_switch_s5())
      {
        CS800_buttons[5].release_flag = 1;
        FT857D_key_released = 1;
        s5_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program.
  // Set the state back to state 0
  s5_state = 0;
  error_log( 3002 );  
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_switch_s6
//
// Process front panel switch s6. This program must be called from a 10 
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If key is pressed, then s6_press_flag will be set true
//  If key is released, then s6_release_flag will be set true
//
// External functions called
//  read_switch_s6
//
// External variables used
//  s6_state
//  s6_timer
//  s6_press_flag
//  s6_release_flag
//
// Internal variables used   
//
// Constants
//
// History
//    08/09/2019 added 20 msec delay to switch release
//
// Status
//  written
//
void process_switch_s6(void)
{
  static uint32_t s6_state;
  static uint32_t s6_timer;  

  // State 1 is waiting for a key to be pressed. When a key is pressed the read
  // switch program returns a zero. When the key is pressed, a 30 millisecond
  // debounce timer is set and an advance to the next state.
  if (s6_state == 0) 
  {
    if (read_switch_s6())
    {
      s6_timer = 3;
      s6_state++;
    }
    return;
  }

  // State 1 is waiting for the debounce_timer to time out before a key is 
  // is released. If time out, set the press flag and advance to the next state
  //
  // If the key is released before the debounce timer times out, then the program
  // returns back to state 0 waiting for a legitimate key.
  if (s6_state == 1)
  {
    if (s6_timer) s6_timer--;
    if (!s6_timer)
    {
      CS800_buttons[6].press_flag = 1;
      FT857D_p4_switch = 1;
      s6_timer = 2;      
      s6_state++;
      return;
    }
    if (!read_switch_s6())
    {
      s6_state = 0;
    }
    return;
  }

  // This state is waiting for the key to be released so it can then start the
  // search for a key press again. If the key is released then the release flag
  // is set.
  if (s6_state == 2)
  {
    if (s6_timer) s6_timer--;
    if (!s6_timer)
    {
      if (!read_switch_s6())
      {
        CS800_buttons[6].release_flag = 1;
        FT857D_key_released = 1;
        s6_state = 0;
      }
    }
    return;
  }

  // If we got here then there is an error in the program. Set the state back to
  // state 0
  s6_state = 0;
  error_log( 3002 );  
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_encoder_1
//
// Process rotary encoder 1. This program looks at the sum of the two pins.
// If the sequence is 0, 2, 3, 1 the encoder is turning clockwise.
// If the sequence is 0, 1, 3, 2 the encoder is turning counter clockwise.
//
// This program is called from the 1 millisecond interrupt. 
//
// add previous two pin sum *4 to current two pin sum
// possible results:
// prev     current   4 * prev + current
// 0         1          1   ccw
// 0         2          2   cw 
// 1         3          7   ccw
// 1         0          4   cw
// 2         0          8   ccw
// 2         3         11   cw 
// 3         2         14   ccw         
// 3         1         13   cw 
//
//  wait for two steps in the same direction before sending a keypress
//
// Programmer
//  Jerry Wanger, Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If rotary encoder is being turned clockwise, then s7_flag is set
//  If rotary encoder is being turned counter clockwise, then s8_flag is set
//  If the knob is being used in the menu, button presses are substituted
//    for the knob rotation, and the knob rotation is not sent to the radio
//
// External functions called
//  read_rotary_encoder_1_0
//  read_rotary_encoder_1_1
//  knob_nav_control
//
// External variables used
//  CS800_buttons[7].press_flag
//  CS800_buttons[8].press_flag
//
// History
//       09/07/2018  eliminated state machine, works better
//       06/14/2019  revised to match FT857 version
//
// Status
//  written, tested, working
//

uint8_t r_knob_for_nav = 0;

void process_encoder_1( void )
{
  static uint8_t enc1_prev_state =0;
  static int rotation = 0;  
  uint8_t enc1_cur_state;
  uint8_t enc_state = 0;

  enc1_cur_state = read_rotary_encoder_1_0() + read_rotary_encoder_1_1();
  if (enc1_cur_state == enc1_prev_state) return;
  // prev * 4 + current
  enc_state = (enc1_prev_state << 2) + enc1_cur_state;

  enc1_prev_state = enc1_cur_state;

  switch( enc_state )
  {
  case 1:  case 7:  case 8:  case 14:
    {
      rotation--;
      break;
    }
  case 2:  case 4:   case 11:  case 13:
    {
      rotation++;
      break;
    }
  default: 
    {
      CS800_buttons[7].press_flag = 0;
      CS800_buttons[8].press_flag = 0;
      rotation = 0;
      break;
    }
  }
  if( rotation > 1 )  // two steps
  {
    if(r_knob_for_nav) knob_nav_control( 1 ); //clockwise
    else CS800_buttons[8].press_flag = 1;
    rotation = 0; 
  }
  if( rotation < -1 )
  {
    if(r_knob_for_nav) knob_nav_control( 0 ); // counter  clockwise
    else CS800_buttons[7].press_flag = 1;
    rotation = 0; 
  }
}
// =============================================================================
// March 3, 2019                Design Documents            FT857D_encoder_right
//
// The program starts by reading the two pins from the encoder. One of the pins
// will give either a 0 or a 1 and the other pin will give either a 0 or a 2.
//
// If you add the two numbers together you get: 0,1,2,3.
// If the encoder is turned clockwise the pin sequence will be 0,1,3,2.
// If the encoder is turned counter clockwise, the pin sequence will be 2,3,1,0
//
// If you store the state of the previous turn and look at the current turn you
// can determine the direction of the turn.
//
// If the encoder is turned right then FT857D_encoder_rt_cw is 1. 
// If the encoder is turned left then FT857D_encoder_rt_ccw is 1.

// revision
// the switch produces two steps per click

// add previous state *4 to current state
// possible results:
// prev     current   shifted prev + current
// 0         1          1   ccw
// 0         2          2   cw 
// 1         3          7   ccw
// 1         0          4   cw
// 2         0          8   ccw
// 2         3         11   cw 
// 3         2         14   ccw         
// 3         1         13   cw 


void FT857D_encoder_right( void )
{
  static uint8_t FT857D_enc_rt_prev_state = 0;
  static int rotation = 0; 
  uint32_t FT857D_enc_rt_cur_state;
  uint8_t enc_state = 0;
  
  FT857D_enc_rt_cur_state = read_rotary_encoder_1_0()+read_rotary_encoder_1_1();
  if (FT857D_enc_rt_cur_state == FT857D_enc_rt_prev_state) return;

  enc_state = (FT857D_enc_rt_prev_state << 2) + FT857D_enc_rt_cur_state;

  FT857D_enc_rt_prev_state = FT857D_enc_rt_cur_state;

  switch( enc_state )
  {
  case 1:  case 7:  case 8:  case 14:
    {
      rotation--;
      break;
    }
  case 2:  case 4:  case 11:  case 13:
    {
      rotation++;
      break;
    }
  default: 
    {
      FT857D_encoder_rt_cw = 0;
      FT857D_encoder_rt_ccw = 0;
      rotation = 0;
      break;
    }
  }
  if( rotation > 1 )  // two steps
  {
    FT857D_encoder_rt_cw = 1;
    rotation = 0; 
  }
  if( rotation < -1 )
  {
    FT857D_encoder_rt_ccw = 1;
    rotation = 0; 
  }
}

//
//******************************************************************************
//
// Feb 20, 2018			Design Documents               process_encoder_2
//
// Process rotary encoder 2. 
//
// This switch is used to simulate the "^" and the "v" keys on the head of the
// radio. 
// When ever the switch goes clockwise, the program has to set both the
// s9_press_flag and the s9_release_flag to simulate the "^" key pressed and
// released
//
// When ever the switch goes counter clockwise, the program has to set both the 
// s10_press_flag and the s10_release_flag to simulate the "v" key pressed and
// released.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If rotary encoder is being turned clockwise, then s9_flag is set
//  If rotary encoder is being turned counter clockwise, then s10_flag is set
//
// External functions called
//  read_rotary_encoder_2_0
//  read_rotary_encoder_2_1
//
// External variables used
//  s9_press_flag
//  s9_release_flag
//  s10_press_flag
//  s10_release_flag
//
// History 
//       09/07/2018  eliminated state machine, works better
//       06/14/2019  revised to match FT857 version
//
// Status
//  written, tested, working
//

void process_encoder_2(void)
{
  static uint8_t enc2_prev_state = 0;
  static int rotation = 0;  
  uint8_t enc2_cur_state;
  uint8_t enc_state = 0;

  enc2_cur_state = read_rotary_encoder_2_0() + read_rotary_encoder_2_1();
  if (enc2_cur_state == enc2_prev_state) return;
  // prev * 4 + current
  enc_state = (enc2_prev_state << 2) + enc2_cur_state;

  enc2_prev_state = enc2_cur_state;

  if(!short_press_timeout && !short_press_holdoff)
  {  
    switch( enc_state )
    {
    case 1:  case 7:  case 8:  case 14:
      {
        rotation--;
        break;
      }
    case 2:  case 4:   case 11:  case 13:
      {
        rotation++;
        break;
      }
    default: 
      {
        CS800_buttons[11].press_flag = 0;
        CS800_buttons[9].press_flag = 0;
        rotation = 0;
        break;
      }
    }

    if( rotation > 1 )  // two steps
    {
      CS800_buttons[9].press_flag = 1;
      short_release_flag = &CS800_buttons[10].release_flag;
      short_press_timeout = 50;     
      rotation = 0; 
    }
    if( rotation < -1 )
    {
      CS800_buttons[11].press_flag = 1;          
      short_release_flag = &CS800_buttons[12].release_flag;
      short_press_timeout = 50;    
      rotation = 0; 
    }
  }  
}
//
// =============================================================================
// March 3, 2019                Design Documents             FT857D_encoder_left
//
// The program starts by reading the two pins from the encoder. One of the pins
// will give either a 0 or a 1 and the other pin will give either a 0 or a 2.
//
// If you add the two numbers together in all possible combinations you get:
// 0,1,2,3. If the encoder is turned clockwise the pin sequence will give you
// 0,1,3,2. If the encoder is turned counter clockwise, the pin sequence will
// give you 2,3,1,0
//
// If you store the state of the previous turn and look at the current turn you
// can determine the direction of the turn.
//
// If the encoder is turned right then FT857D_encoder_lft_cw is 1. 
// If the encoder is turned left then FT857D_encoder_lft_ccw is 1.
//

// =============================================================================
void FT857D_encoder_left( void )
{
  static uint8_t enc_lft_prev_state = 0;
  static int rotation = 0; 
  uint8_t enc_lft_cur_state;
  uint8_t enc_state = 0;
  
  enc_lft_cur_state = read_rotary_encoder_2_0()+read_rotary_encoder_2_1();
  if (enc_lft_cur_state == enc_lft_prev_state) return;

  enc_state = (enc_lft_prev_state << 2) + enc_lft_cur_state;

  enc_lft_prev_state = enc_lft_cur_state;

  switch( enc_state )
  {
  case 1:  case 7:  case 8:  case 14:
    {
      rotation--;
      break;
    }
  case 2:  case 4:   case 11:  case 13:
    {
      rotation++;
      break;
    }
  default: 
    {
      FT857D_encoder_lft_cw = 0;
      FT857D_encoder_lft_ccw = 0;
      rotation = 0;
      break;
    }
  }
  if( rotation > 1 )  // two steps
  {
    FT857D_encoder_lft_cw = 1;
    rotation = 0; 
  }
  if( rotation < -1 )
  {
    FT857D_encoder_lft_ccw = 1;
    rotation = 0; 
  }
}
//
//******************************************************************************
//
// March 15, 2018		Design Documents                     process_ptt
//
// Process PTT. This program must be called from a 10
// millisecond timer.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If key is pressed, then ptt_flag will be set true
//  If key is released, then ptt_release_flag will be set true
//
// External functions called
//  read_mike_ptt
//
// External variables used
//  ptt_state
//  ptt_timer
//  ptt_press_flag
//  ptt_release_flag
//
// Internal variables used
//    
//
// Constants
//
// History
//  05/28/2019 renamed ptt_10_millisecond timer to ptt_timer
//             because debounce time changed to 300 msec
//
// Status
//  written
//
void process_ptt(void)
{
static uint32_t ptt_state;
static uint32_t ptt_timer;  
//
// State 0 is waiting for the PTT to be pressed. When PTT is pressed the read
// PTT program returns a zero. When the key is pressed, a 300 millisecond
// debounce timer is set and an advance to the next state.
//
  if (ptt_state == 0) 
  {
    if (!read_mike_ptt())
    {
      ptt_timer = 30;  // was 10
      ptt_state++;
    }
    return;
  }
//
// State 1 is waiting for the debounce_timer to time out before a key is 
// is released. If time out, set the press flag and advance to the next state
//
// If the key is released before the debounce timer times out, then the program
// returns back to state 0 waiting for a legitimate key.
//
  if (ptt_state == 1)
  {
    if (ptt_timer) ptt_timer--;
    if (!ptt_timer)
    {
      CS800_buttons[0].press_flag = 1;
      ptt_timer = 2;        
      ptt_state++;
      return;
    }
    if (read_mike_ptt())
    {
      ptt_state = 0;
    }
    return;
  }
//
// This state is waiting for the key to be released so it can then start the
// search for a key press again. If the key is released then the release flag
// is set.
//
  if (ptt_state == 2)
  {
    if (ptt_timer) ptt_timer--;
    if (!ptt_timer)
    {
      if (read_mike_ptt())
      {
        CS800_buttons[0].release_flag = 1;
        ptt_state = 0;
      }
    }
    return;
  }
//
// If we got here then there is an error in the program. Set the state back to
// state 0
//
  ptt_state = 0;
  error_log( 3002 );
}
//
//******************************************************************************
//
// March 15, 2018		Design Documents                process_switches
//
// This program processes all the switches. It sends the data back to the main
// radio when a key is pressed or released.
//
// Programmer
//  Jerry Wanger, Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  s1_press_flag indicates switch on encoder 1 pressed
//  s1_release_flag indicates switch on encoder 1 released
//  s2_press_flag indicates switch on encoder 2 pressed
//  s2_release_flag indicates switch on encoder 2 released
//  s3_press_flag indicates switch on front panel pressed
//  s3_release_flag indicates switch on front panel released
//  s4_press_flag indicates switch on front panel pressed
//  s4_release_flag indicates switch on front panel released
//  s5_press_flag indicates switch on front panel pressed
//  s5_release_flag indicates switch on front panel released
//  s6_press_flag indicates switch on front panel pressed
//  s6_release_flag indicates switch on front panel released
//  s7_flag indicates encoder 1 turning clockwise
//  s8_flag indicates encoder 1 turning counter clockwise
//  s9_flag indicates encoder 2 turning clockwise
//  s10_flag indicaes encoder 2 turning counter clockwise
//
//  mc59_flag indicates mike key pressed
//  mc59_release_flag indicates mike key released
//  mc59_key is the value of the mike key that was pressed or released.
//
//  ptt_press_flag indicates PTT switch on microphone pressed
//  ptt_release_flag indicates PTT switch on microphone released
//
// Exit Conditions
//  Data prepared so program can send the data back to the main radio
//  Key press stored in last_key_press for use by other routines
//
// External functions called
//
// External variables used
//
// Internal variables used
//    
//
// Constants
//
// History 
//      10/01/2018 revised to use switch information in a table
//      12/13/2018 made more compact, if(send to radio) a one line
//
// Status
//  written, working

// send_to_radio, key_number_1, key_number_2,
// key_type, press_flag, release_flag
// key type 0-1 for a press release type. 2-3 for knob        

panel_button CS800_buttons[14] = {
{ 1, 0x20, 0x14, 1, 0, 0 },    //PTT  
{ 1, 0x23, 0x1B, 1, 0, 0 },    //s1 left knob button
{ 0,    0,    0, 1, 0, 0 },    //s2 right knob button
{ 1, 0x23, 0x15, 1, 0, 0 },    //s3 P1 left
{ 1, 0x23, 0x16, 1, 0, 0 },    //s4 P2 left middle
{ 1, 0x23, 0x17, 1, 0, 0 },    //s5 P3 right middle
{ 1, 0x23, 0x18, 1, 0, 0 },    //s6 P4 right
{ 1, 0x23, 0x1C, 2, 0, 0 },    //s7 R knob counter clockwise
{ 1, 0x23, 0x1C, 3, 0, 0 },    //s8 R knob clockwise
{ 1, 0x23, 0x19, 1, 0, 0 },    //s9 L knob clockwise press
{ 1, 0x23, 0x19, 0, 0, 0 },    //s9 L knob clockwise release
{ 1, 0x23, 0x1A, 1, 0, 0 },    //s10 L knob counter clockwise press
{ 1, 0x23, 0x1A, 0, 0, 0 },    //s10 L knob counter clockwise release
{ 1, 0x24,    0, 1, 0, 0 }     // all mic keys except ptt 
};

// 
uint32_t last_key_press = 0;
uint32_t ptt_timer = 0;

void process_switches(void)
{
  int bttn;

  // If previous key has not been processed yet, return immediately
  if (key_active_flag)
  {
    error_log(3300);  // missed key error
    return;
  }

  // rewritten to use the button table
  for( bttn = 0; bttn < 14 ; bttn++)
  {
    if(CS800_buttons[bttn].press_flag)
    {
      key_number_1 = CS800_buttons[bttn].key_number_1;
      if( key_number_1 == 0x24 )  // if a mic key
      {
        key_number_2 = mc59_key;
        key_release = mc59_key;
      }
      else  key_number_2 = CS800_buttons[bttn].key_number_2;
      last_key_press = key_number_2;  
      key_type = CS800_buttons[bttn].key_type;
      if(CS800_buttons[bttn].send_to_radio) key_active_flag = 1;
      CS800_buttons[bttn].press_flag = 0;
      // show transmitting active on screen (this could move to a screen routine)
      if(bttn == 0) 
      {
        if(CS800_buttons[0].send_to_radio)
        {
        draw_TXonoff( 1 );
        // set timer to hold "on air" on past the first red led off packet
        ptt_timer = 500;
        }
      }
      // if( bttn == 3) log_event( "P1 active");  // for debug
      
      // in the radio menu, exit does not generate a menu packet      
      // it just sends the keypress
      // we need to clear the menu screen if it's on
      if( bttn == 3 && display_mode == 2 && menu_current_level == 1 && list_active == 0 )
      {
        // with lastkey set to cancel, display menu will handle closing
        display_menu( cs_menu_page, MAIN, "", 0, "" , 0);
        // log_event( "menu close"); // for debug
        // now set lastkey to 0, the cancel has been handled,
        // don't leave it to be used by other processes
        last_key_press = 0;
      }
      return;
    }
    if(CS800_buttons[bttn].release_flag)
    {
      key_number_1 = CS800_buttons[bttn].key_number_1;
      if( key_number_1 == 0x24 )  // if a mic key
      {
        key_number_2 = key_release;
      }
      else key_number_2 = CS800_buttons[bttn].key_number_2;
      key_type = 0;
      if(CS800_buttons[bttn].send_to_radio) key_active_flag = 1;
      CS800_buttons[bttn].release_flag = 0;
      // transmitting not active 
      if(bttn == 0)
      {
        if(CS800_buttons[0].send_to_radio)
        {
          // always off if Ptt is off 
          draw_TXonoff( 0 );
          tx_led_off();          
        }
      }
      return;  
    }
  }
}
// =============================================================================
//
//  deactivate switches - stop sending a keypress data to the radio
//
void deactivate_keypress( uint8_t keynum )
{
   CS800_buttons[keynum].send_to_radio = 0;
   // handle knobs multiple table entries
   if(keynum == 7 || keynum == 8)
   {
      CS800_buttons[7].send_to_radio = 0;     
      CS800_buttons[8].send_to_radio = 0; 
   }
   if(keynum > 8 && keynum < 13)
   {
      CS800_buttons[9].send_to_radio = 0;     
      CS800_buttons[10].send_to_radio = 0; 
      CS800_buttons[11].send_to_radio = 0; 
      CS800_buttons[12].send_to_radio = 0; 
   }
}
// =============================================================================
//
//  activate switch - allow sending a keypress data to the radio
//  
//
void activate_keypress( uint8_t keynum )
{
   CS800_buttons[keynum].send_to_radio = 1;
   // handle knobs multiple table entries
   if(keynum == 7 || keynum == 8)
   {
      CS800_buttons[7].send_to_radio = 1;     
      CS800_buttons[8].send_to_radio = 1; 
   }
   if(keynum > 8 && keynum < 13)
   {
      CS800_buttons[9].send_to_radio = 1;     
      CS800_buttons[10].send_to_radio = 1; 
      CS800_buttons[11].send_to_radio = 1; 
      CS800_buttons[12].send_to_radio = 1; 
   }
}

// =============================================================================
//
// deactivate all keypresses
//
void deactivate_all_keypress( void )
{
  int i;
  for( i = 0; i < 14; i++) CS800_buttons[i].send_to_radio = 0;       
}
// =============================================================================
//
// Activate all keypresses
//
void activate_all_keypress( void )
{
  int i;
  for( i = 0; i < 14; i++) CS800_buttons[i].send_to_radio = 1; 
  // leave right knob off
   CS800_buttons[2].send_to_radio = 0;
}

//     
// *****************************************************************************
//
// March 28, 2018		Design Documents           setup_keypad_switches
//
// This program sets up a buffer to allow the keypad from the mike, the PTT from
// the mike, or one of the switches and encoders to be sent back to the radio.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  key_active_flag indicates data ready to be sent back
//
// Exit Conditions
//  key_active_flag is reset
//  buffer for data is setup
//  key_buffer_flag is setup to indicate buffer is ready to be sent.
//
// External functions called
//
// External variables used
//  key_buffer[]
//
// Internal variables used
//
// History
//       08/08/2018 moved to switches.c from process_packets
//
// Status
//  written
//

void setup_keypad_switches(void)
{
  /*  moved early checks to timer call
      no reason to call and return, the tests can be done there.
  // If this buffer not transmitted, return immediately
  if(key_buffer_flag)
  {
    // error_log(3300);  // many hits when running spectrum
    return;
  }
  // If there are no keys or switches to process, returns immediately. 
  // if (!key_active_flag) return;
  */
  
  // Setup buffer
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  0x07;
  key_buffer[3] =  0x00;
  key_buffer[4] =  key_number_1;
  key_buffer[5] =  key_number_2;
  key_buffer[6] =  0x00;
  key_buffer[7] =  key_type;
  key_buffer[8] =  0x00;
  key_buffer[9] =  key_number_1 + key_number_2 + key_type;  // checksum
  key_buffer[10] = 0xB4;

  // Handle PTT as an exception
  if (key_buffer[4] == 0x20)
  {
    key_buffer[6] =  key_type;
    key_buffer[7] =  0;
  }
  
  // log_event( "Key in bufr");  // for debug

  // Set flag to indicate there is data to be sent to radio
  key_buffer_flag = 1;

  // Clear flag so you can start processing next key
  key_active_flag = 0;  
}
// =============================================================================
//
// July 16, 2018		Design Documents                   volume_updown
//
// Direct volume control by program. Sends data to radio to change the voulme.
// Simulates the knob turning allowing process_switches to do the work.
//  Calling with a 1 increases the volume, a 0 decreases.
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History: 12/6/2018 revised to handle knob assignment per CPS
//

void volume_updown( int updown )
{
  // up
  if( updown )
  {
    // if the volume control is by the knob    
    if( volume_up_button == 7 )    
    {
      if(short_press_holdoff == 0)
      {
        // send knob
        CS800_buttons[7].press_flag = 1;
        // set a holdoff delay to prevent repeating too quickly
        short_press_holdoff = 200;
      }      
    }
    //  up down buttons
    if (volume_up_button == 9 ) 
    {
      // send if it's not active
      if((short_press_timeout == 0) && (short_press_holdoff == 0))
      {
        // use button, set release
        CS800_buttons[ 9 ].press_flag = 1;
        // set timer to release, short press only      
        short_release_flag = &CS800_buttons[ 10 ].release_flag;
        short_press_timeout = 200;
      }
    }
    // normal button
    if( volume_up_button > 0 && volume_up_button < 7 )  
    {
      // use button, set release 
      CS800_buttons[ volume_up_button ].press_flag = 1;
      // set timer to release, short press only      
      short_release_flag = &CS800_buttons[ volume_up_button ].release_flag;
      short_press_timeout = 200;
    }
  }
  // down
  else  
  {
    // if the volume control is by the knob    
    if( volume_dn_button == 8 )    
    {
      if(short_press_holdoff == 0)
      {
        // send knob
        CS800_buttons[8].press_flag = 1;
        // set a holdoff delay to prevent repeating too quickly
        short_press_holdoff = 200;
      }           
    }
    //  up down buttons
    if (volume_dn_button == 11 ) 
    {
      // send if it's not active
      if((short_press_timeout == 0) && (short_press_holdoff == 0))
      {
        // use assigned button
        CS800_buttons[ 11 ].press_flag = 1;
        // set timer to release, short press only      
        short_release_flag = &CS800_buttons[ 12 ].release_flag;
        short_press_timeout = 200;
      }
    }
    // normal button
    if( volume_dn_button > 0 && volume_up_button < 7 )  
    {
      CS800_buttons[ volume_dn_button ].press_flag = 1;
      // set timer to release, short press only      
      short_release_flag = &CS800_buttons[ volume_dn_button ].release_flag;
      short_press_timeout = 200;
    }
  }
}
// =============================================================================
//
// July 16, 2018		Design Documents                  channel_updown
//
// Direct channel control by program. Sends data to radio to change the channel.
// Simulates the knob turning allowing process_switches to do the work.
//  Calling with a 1 increases, a 0 decreases.
//
// This is for the touch screen controls. No button pushes or knob turns
//  cause this. The Panel must simulate the button push and release for the
//  button assigned to channel control
//
//  Simulate the three cases:
//     Normal knob   - send knob turn direction
//     Normal button - send button press, set release on the same button
//     Radio up-down buttons - send press, set release on the release button

// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// External variables used
//  s9_press_flag    ^
//  s9_release_flag  ^
//  s10_press_flag   v
//  s10_release_flag v
//
// extern void msec_delay( int );

// History: 12/6/2018 revised to handle knob assignment per CPS

void channel_updown( int updown )
{
  // up
  if( updown )
  {
    // if the channel control is by the knob    
    if( channel_up_button == 7 )    
    {
      if(short_press_holdoff == 0)
      {
        CS800_buttons[7].press_flag = 1;
        // set a holdoff delay to prevent repeating too quickly
        short_press_holdoff = 100;
      }
    }
    //  up down buttons
    if (channel_up_button == 9 ) 
    {
      // send if it's not active
      if((short_press_timeout == 0) && (short_press_holdoff == 0))
      {
        // use assigned button
        CS800_buttons[ 9 ].press_flag = 1;
        // set timer to release, short press only      
        short_release_flag = &CS800_buttons[ 10 ].release_flag;
        short_press_timeout = 100;
      }
    }
    // normal button
    if( channel_up_button > 0 && channel_up_button < 7 )  
    {
      CS800_buttons[ channel_up_button ].press_flag = 1;
      // set timer to release, short press only      
      short_release_flag = &CS800_buttons[ channel_up_button ].release_flag;
      short_press_timeout = 100;
    }
  }
  // down
  else  
  {
    // if the channel control is by the knob    
    if( channel_dn_button == 8 )    
    {
      if(short_press_holdoff == 0)
      {
        CS800_buttons[8].press_flag = 1;
        // set a holdoff delay to prevent repeating too quickly
        short_press_holdoff = 100;
      }      
    }
    //  up down buttons
    if (channel_dn_button == 11 ) 
    {
      // send if it's not active
      if((short_press_timeout == 0) && (short_press_holdoff == 0))
      {
        // use assigned button
        CS800_buttons[ 11 ].press_flag = 1;
        // set timer to release, short press only      
        short_release_flag = &CS800_buttons[ 12 ].release_flag;
        short_press_timeout = 100;
      }
    }
    // normal button
    if( channel_dn_button > 0 && channel_up_button < 7 )  
    {
      CS800_buttons[ channel_dn_button ].press_flag = 1;
      // set timer to release, short press only      
      short_release_flag = &CS800_buttons[ channel_dn_button ].release_flag;
      short_press_timeout = 100;
    }
  }
}
// =============================================================================
//
// keys activated by the touch panel need to be released. 
// short press release timer
//  in 1 millisec timer wait then release
//  set the release key flag for the key that is active
//  one timer for all keys
//
// History
//       08/28/2018 written
//
// Status
//  written, working

int short_press_timeout = 0;
int short_press_holdoff = 0;  // holdoff after key release to limit repeating

// short release flag is a pointer to the flag to be released.
// It needs to be set when the timeout is started.  

uint32_t *short_release_flag;

// use:
// short_release_flag = &s9_release_flag;
// short_press_timeout = 100;

void short_press_timer( void )
{
  // if there is anything to countdown
  if(short_press_timeout)
  {
    // decrement and send release message to the active key
    if(--short_press_timeout == 0)
    {
       *short_release_flag = 1;
       short_press_holdoff = 200;
    }
  }
  if(short_press_holdoff > 0) short_press_holdoff--;
}

// =============================================================================
//
// July 23, 2018		Design Documents              contacts_page_down
//
// Direct contact list control by program. Sends data to radio to move down
// the contact list in the  menu. Can be used by touch control without
// using the push button. Can be used by progranm to populate a table of
// contacts in the front panel.
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// External variables used
//  s5_flag
//
// Status
//  written, not tested, not being used yet

void contact_page_down( void )
{
  int i;
  // must be on the menu page, and contacts tab
  if((active_screen_num == 3) && cs_menu_page == 0)
  {
    for(i = 0; i < lines_per_page; i++)
    {
      CS800_buttons[5].press_flag = 1; // simulate push of p3
      short_release_flag = (uint32_t *) &CS800_buttons[5].release_flag;
      short_press_timeout = 100;       
    }
  }
}
// =============================================================================
//
//  convert knob rotation to up down button command
//   overrides CPS assignment of knob control, sends a press of
//   the nav buttons to the radio, 
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Status
//  written, working
//
void knob_nav_control( uint8_t updown )
{
  if( updown )
  {
    CS800_buttons[5].press_flag = 1; // knob clockwise -> right middle button
    short_release_flag = &CS800_buttons[5].release_flag;
    short_press_timeout = 100;      
  }
  else
  {
    CS800_buttons[4].press_flag = 1; // knob counter clockwise -> left mid button
    short_release_flag = &CS800_buttons[4].release_flag;
    short_press_timeout = 100;   
  }
}

// =============================================================================
//
// write current CPS function button data to the eeprom
//
// requires MS20.42 and up
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History: 12/06/2018
//
// Status
//  written, working

// eeprom address for Function button settings
// 0x2100 - 0x212D
#define FB_KEYS_HI  0x21
#define FB_KEYS_LO  0x00

void FB_button_write( void )
{
  int i;
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout( FB_KEYS_HI );  
  I2C_put_dataout( FB_KEYS_LO );
  // add key data
  for( i = 0; i < FB_num_keys; i++)
  {
    I2C_put_dataout( *(&current_key_functions.long_press_time + i ) );   
  }
  // write to EEPROM  3+46  49 bytes
  EEPROM_MultipleWrite( 0, 49);
  EEPROM_waitfor_I2C();
}
// =============================================================================
//
// read stored CPS function button data from the eeprom
//
// requires MS20.42 and up
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History: 12/06/2018 
//
// Status
//  written, working

void FB_button_read( void )
{
  int i;
  // read Function Button data
  EEPROM_MemRead( 0, FB_KEYS_HI,FB_KEYS_LO, 49);
    // wait for I2C to finish
  EEPROM_waitfor_I2C();  
  // store data received to structure
  for( i = 0; i < 49; i++)
  {
    *(&stored_key_functions.long_press_time + i ) = I2C_get_datain();
  }
}
// =============================================================================
//
//  Compare two sets of function button data to see if they are the same
//
// requires MS20.42 and up
//
// Programmer
//  Dean S.Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History: 12/06/2018 
//
// Status
//  written, working
//

int FB_button_compare( FB_KeysProgFunctionDef_t* FB_a, FB_KeysProgFunctionDef_t* FB_b )
{
  int i;

  for( i = 0; i < FB_num_keys; i++)
  {
    if( *(&FB_a + i ) != *(&FB_b + i ) ) return 0;   
  }
  return 1;
}
// =============================================================================
//
//  find buttons to use for touch screen controls
//   determines which buttons are set for channel up / down
//   and volume up / down
//   The touch screen simulates a button push or knob turn
//   so it needs to know which button has been assigned to the control
//
//   reads button assignments from current_key_functions
//   current_key_functions needs to be updated with new settings before
//   calling this. 
//  
// History: 12/07/2016 written
//
// Status: Working, not tested with CPS set for P1-P4 buttons
//

uint8_t channel_up_button;
uint8_t channel_up_longp;
uint8_t channel_dn_button;
uint8_t channel_dn_longp;
uint8_t volume_up_button;
uint8_t volume_up_longp;
uint8_t volume_dn_button;
uint8_t volume_dn_longp;

void get_button_assign( void )
{
  int i;
  uint8_t FB_btn;
  uint8_t found_count = 0;
  
  // scan whole list
  for( i = 0; i < FB_num_keys; i++)
  {
    FB_btn = *(&current_key_functions.long_press_time + i );
    
    switch( FB_btn)
    {
    case n_PB_ChannelUp:
      {
        channel_up_button = radio_to_panel_button( i );
        if( i & 1) channel_up_longp = 1;
        else channel_up_longp = 0;
        found_count += 1;
        break;
      }
    case n_PB_ChannelDown:
      {
        channel_dn_button = radio_to_panel_button( i );
        if( i & 1) channel_dn_longp = 1;
        else channel_dn_longp = 0;        
        found_count += 2;        
        break;
      }
    case n_PB_VolumeUp:
      {
        volume_up_button =radio_to_panel_button( i );
        if( i & 1) volume_up_longp = 1;
        else volume_up_longp = 0;        
        found_count += 4;        
        break;
      }
    case n_PB_VolumeDown:
      {
        volume_dn_button = radio_to_panel_button( i );
        if( i & 1) volume_dn_longp = 1;
        else volume_dn_longp = 0;        
        found_count += 8;
        break;
      }
    default: break;      
    }
    if( found_count == 15) break;
  }
  // four, and only four else error
  if( found_count != 15) error_log( 3081 );
  
}
// =============================================================================
// 12/07/2018
//
//  translate radio button numbers to panel numbers
//  returns the panel button number
//  if it's a short press or long press, the same number
//  the longpress variable will be set by checking even/ odd numbers 
//  shortpresses are even, longpresses are odd
//
// History: 12/07/2016 written
//
// Status: Working
//
  
uint8_t radio_to_panel_button( uint8_t btn )
{
  uint8_t p_btn;
  
  switch(btn)
  {
  case 2: case 3:  //p1 short, long
  {
    p_btn = 3;
    break;  
  }
  case 4: case 5:  //p2 short, long
  {
    p_btn = 4;
    break;  
  }
  case 6: case 7:  //p3 short, long
  {
    p_btn = 5;
    break;  
  }
  case 8: case 9:  //p4 short, long
  {
    p_btn = 6;
    break;  
  }
  case 10: case 11:  // radio UP key
  {
    p_btn = 9;
    break;  
  }  
  case 12: case 13:  // radio DOWN key
  {
    p_btn = 11;
    break;  
  }
  case 14: case 15:  // radio knob press
  {
    p_btn = 1;
    break;  
  }  
  case 16:       // radio knob clockwise
  {
    p_btn = 7;
    break;  
  }  
  case 17:       // radio knob counter clockwise
  {
    p_btn = 8;
    break;  
  }    
  
  default: break;
  }
  
  return p_btn;
}



