//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
///
// Mar 15, 2018			Design Documents                    CSI_TIMERS.c 
//   
//                        Timers and clock routines
//                      for the Large Display project
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  03/15/2018 written
//  04/02/2018 systick_init and startup_clocks added
//  06/18/2018 moved 10msec timer to independent ISR for Timer 2
//  03/11/2019 merged with FT857
//  05/31/2019 made conditional compile for FT857D
//
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  timer_100us                     100 micro second timer
//  timer_10ms                      10 millisecond timer
//  timer_100ms                     100 millisecond timer
//  delay                           NOP delay in microseconds
//  wait_12_nanoseconds             NOP 12 nanosecond delay
//  wait_36_nanoseconds             NOP 36 nanosecond delay
//  wait_100_nanoseconds            NOP 100 nanosecond delay
//  init_systick                    initialize system tick timer
//  msec_delay                      delay based on systick, in milliseconds 
//
// Status: written, not used
//  startup_clocks                  initialize CPU clocks
//==============================================================================
#include <stdint.h>
#include "CSI_TIMERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_REGISTERS.h"
#include "CSI_GPIO.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_FEATURES.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_TouchPanel.h"
#include "CSI_SWITCHES.h"
#include "CSI_MC59.h"
#include "CSI_Menus.h"
#include "CSI_UART.h"
#include "CSI_FT857D.h"
#include "CSI_RADIO.h"
#include "CSI_CS800_TGMonitor.h"

uint32_t WatchDogTimer;
uint32_t ms_1_timer;
uint32_t ms_10_timer;
uint32_t pre_100us_timer;
uint32_t pre_10ms_timer;
uint32_t acknowledgement_delay;

unsigned int  volatile ticks = 0;
unsigned int  volatile u_systick = 0;
unsigned long volatile millisec = 0;
unsigned long volatile seconds = 0;

extern void uart_FT857D_send_0xA5(void);
extern void FT857D_auto_initialization(void);

uint32_t FT857D_uart_tx_watch_dog_timer;
uint32_t FT857D_uart_rx_watch_dog_timer;
uint32_t FT857D_auto_init_timer;
uint32_t FT857D_startup_timer;

void timer_100us(void)
{
  static uint8_t process_complete = 1; 
// ======================
#ifdef radio_FT857D  
  if (FT857D_uart_rx_watch_dog_timer) FT857D_uart_rx_watch_dog_timer--;
  if (FT857D_uart_tx_watch_dog_timer) FT857D_uart_tx_watch_dog_timer--;

  // If FT857D_uart_tx_watch_dog_timer reached a value of 1, it meant we did not 
  // get an acknowledgement at the proper time. Try resending the packet. The 
  // watchdog timer is set to a zero to disable it until needed again.
  //
  if (FT857D_uart_tx_watch_dog_timer == 1)
  {
    uart_FT857D_send_0xA5();
    FT857D_uart_tx_watch_dog_timer = 0;
  }
// =====================  
#endif /* FT857D */  
  // hum debug, moved to timer 3
  //mc59_read_keypad();
  if( process_complete == 0 ) error_log( 3200 );
  
  //
  // One millisecond timer
  //
  if (pre_100us_timer++ > 9)
  {
    process_complete = 0;  // debug process flag
    pre_100us_timer = 0;
    if (ms_1_timer) ms_1_timer--;
    if (acknowledgement_delay) acknowledgement_delay--;
    if (uart1_rx_watch_dog_timer) uart1_rx_watch_dog_timer--;
    process_keypad();
    #ifdef radio_FT857D
    if (FT857D_auto_init_timer) FT857D_auto_init_timer--;
    if (RKnob_delay_timer) RKnob_delay_timer--; 
    if (LKnob_delay_timer) LKnob_delay_timer--;
    #endif /* FT857D */ 
    switch( current_radio_type )
    {
    case CS800D:
      {
        process_CS800D();
        uart1_tx_coordinate();
        break;
      }
/// ==========================      
#ifdef radio_FT857D      
    case FT857D:
    case FT857D2:
      {
        if (FT857D_radio_active == 1)
        {
          FT857D_encoder_right();
          FT857D_encoder_left();
        }
        break;
      }
#endif /* FT857D */      
// ===========================      
    default: break;
    }
    
    // need to be outside of current radio to work when switching radios
    process_encoder_1();
    process_encoder_2(); 
    short_press_timer();
    if(ptt_timer) ptt_timer--;    
    
    process_complete = 1;  // debug process finished 
  }   
}
//
// 10_ms timer is called from timer two
//
uint32_t radio_info_timer = 0;
extern int wait_for_radio_onscreen;

void timer_10ms(void)
{
  // if (ms_10_timer) ms_10_timer--;  
  #ifdef radio_FT857D   
  if (FT857D_cursor_timer) FT857D_cursor_timer--;
  #endif /* FT857D */ 
  // heartbeat packets
  if (radio_watchdog_timer) radio_watchdog_timer--;
  else display_radio_wait( 3, 40, 130, 4); 
  // radio info packets
  process_switch_s1();
  process_switch_s2();
  process_switch_s3();
  process_switch_s4();
  process_switch_s5();
  process_switch_s6();
  process_ptt();
  
  switch( current_radio_type )
  {
  case CS800D:
    {
      process_switches(); 
      if( (!key_buffer_flag) && key_active_flag ) setup_keypad_switches();      
      CS800D_process_touches();
      break;
    }
#ifdef radio_FT857D      
    case FT857D:
    case FT857D2:
    {
      if( FT857D_startup_timer )  process_FT857D_packets();
      FT857D_auto_initialization();      
      if ((FT857D_radio_active == 1) && (active_screen_num != 2))
      {
        FT857D_process_touches();  

        // only when the panel menu is on
        if(active_screen_num == 6 ) 
        {
          // need this to make panel menu work
          process_switches();
          //CS800D_process_touches();          
        }
        else
        {
          FT857D_mechanical_switches();
        }
      }
      break;
    }
#endif /* FT857D */     
  default: break;
  }
  if (pre_10ms_timer++ > 8)
  {
    pre_10ms_timer = 0;
    timer_100ms();
  }
}

void timer_100ms(void)
{
  if(dtmf_codes_timer) dtmf_codes_timer--;
  if(WatchDogTimer) WatchDogTimer--;
  #ifdef radio_FT857D 
  if( FT857D_startup_timer) FT857D_startup_timer--;
  if ( FT857D_startup_timer == 2 )
  {
    // send squelch to radio
    FT857D_set_squelch( FT857_squelch_value );
  }
  #endif /* FT857D */    
}
// =============================================================================
//
//  microseconds delay
//
// If there are no other routines interrupting this routine, this routine will
// be accurate to about 1% assuming the CPU clock frequency is 168 MHz.
//

void delay(uint32_t delay_in_microseconds)
{
uint32_t i;
  for (i = 0; i< delay_in_microseconds * 10u; i++)
  {
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");    
    asm("NOP");     
  }
}
void wait_12_nanoseconds(void) // ************* doubled time ***************
{
  asm("NOP");   //should only need 2 nop's 
  asm("NOP");    
  //asm("NOP");    
  //asm("NOP"); 
}
void wait_36_nanoseconds(void) // ************* doubled time ***************
{
  wait_12_nanoseconds();
  wait_12_nanoseconds();
  wait_12_nanoseconds();
}
void wait_100_nanoseconds(void) // ************* doubled time ***************
{
  wait_36_nanoseconds();
  wait_36_nanoseconds();  
  wait_36_nanoseconds();  
}
//==============================================================================
//
// Mar 19, 2018 		Design Documents      		    init_systick
//
// Initializes and enables System Tick timer for 100 microseconds
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  System Tick Timer is set for 100 microseconds
// 
// External variables used
// 
// Internal variables used
//  STK_CTRL pointer to STK_CTRL_ADDRESS
//  STK_LOAD pointer to STK_LOAD_ADDRESS
//  STK_VAL pointer to STK_VAL_ADDRESS
//  SHPR3 pointer to SHPR3_ADDRESS
//
// Constants
//    defined in core_register_definitions.h
//  SYST_CSR_ADDRESS
//  SYST_RVR_ADDRESS
//  SYST_CVR_ADDRESS
//  SHPR3_ADDRESS
//
// History
//  3/21/18 Added Documentation
//
// Status
//  written
//
//==============================================================================
//
// Initialize and enable System Tick timer 
//
void init_systick(void)
{
  uint32_t volatile *STK_CTRL = (uint32_t *) STK_CTRL_ADDRESS;
  uint32_t volatile *STK_LOAD = (uint32_t *) STK_LOAD_ADDRESS;
  uint32_t volatile *STK_VAL  = (uint32_t *) STK_VAL_ADDRESS;
  uint32_t volatile *SHPR3    = (uint32_t *) SHPR3_ADDRESS;

  // set Reload register,  it uses 24 low bits of the register
  // set reload for 100 micro second 168 Mhz / 8 = 21 Mhz / 10000
  // 2100 - 1 for countdown to 0. 
  *STK_LOAD = (uint32_t) 2099; 
  
  // set SysTick priority bits 24-31 are for SysTick/
  *SHPR3 |= 0x02000000;  

  // set SysTick Current value register to 0 it will reload the reload value
  // any write to the register will clear the register to 0
  *STK_VAL = 0;
  
  // set SysTick Control register
  // Enable SysTick  0x01, Enable exception 0x02, Clock source, processor 0x04
  *STK_CTRL = 0x00000003;      // bit 2 = 1 use processor clock 
  //*STK_CTRL = 0x00000003;    // bit 2 = 0 use AHB / 8
}
//==============================================================================
//
// Systick Handler
//  
//  
// extern void test_pin( void );
extern void set_clock_out( int );
extern uint8_t mc59_keypad_timer;
extern void timer_10micro_enable( int );

#pragma call_graph_root = "interrupt"
void SysTick_Handler( void )
{
  static uint8_t dsec = 0;  // counter for 10 millisec
  static uint32_t ksec = 0; // counter for 1 second 
  static uint8_t rssi_sec = 0; // counter for rssi graph
  // systick is 100usec
  ticks++;
  u_systick++;
  timer_100us();
  // test_pin();
    
  // millisec 
  if(u_systick > 9)  
  {
    millisec++;
    dsec++;
    ksec++;
    // timer for advanced scanning 
    if( scan_step_timer ) scan_step_timer--;
    
    // mc59 burst timer  .3 msec for keypad, 5 msec between bursts
    if( mc59_keypad_timer ) mc59_keypad_timer--;
    else 
    {
      mc59_keypad_timer = 6;
      // turn on timer 3
      timer_10micro_enable( 1 );
      mike_dm_high();      
    }
    
    u_systick = 0;
  }
  // 10 millisecond
  if(dsec > 9)
  {
     dsec = 0;
     // timer_10ms();
  }
  // seconds
  if(ksec > 999)
  {
    ksec = 0;
    // once per second
    seconds++;
    // turn on blinking for programming
    // blink_alternate();
    
    if( rssi_graph_on )
    {
      if (rssi_sec++ > 2 )
      {
        // get rssi graph info
        request_RSSI_value();
        rssi_sec = 0;
      }
    }
    if( scan_rx_timer ) scan_rx_timer--;
    // talkgroup monitor
    if( TGM_monitor_on && TGM_call_active )
    {
      TGM_call_seconds++;
    }
    
  }
}

//******************************************************************************
//
// Mar 19, 2018			Design Documents                  startup_clocks
//
// Reset and start up cpu clock
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  CPU clock is set for 168 MHz
// 
// External variables used
// 
// Internal variables used
//  reg_temp
//  timeout
//
// Constants
//    defined in core_register_definitions.h
//  SYST_CSR_ADDRESS
//  SYST_RVR_ADDRESS
//  SYST_CVR_ADDRESS
//  SHPR3_ADDRESS
//
// History
//
// Status
//  written, old. clock startup is done in System_Init().
//    This needs to be updated. 
//
//******************************************************************************
//
// Startup clocks
//
// Initialize_registers uses fixed wait times instead of checking the status
// register. This checks the status register before proceeding.
//
#define BIT16_MASK  0x00010000  
  
void startup_clocks(void)
{
uint32_t volatile *RCC_CR         = (uint32_t *) RCC_CR_ADDRESS;
uint32_t volatile *RCC_PLLCFGR    = (uint32_t *) RCC_PLLCFGR_ADDRESS;
uint32_t volatile *RCC_CFGR       = (uint32_t *) RCC_CFGR_ADDRESS;
uint32_t reg_temp;
uint32_t timeout;
//
// reset and startup clocks
// set the default value which uses the internal
//
  *RCC_CFGR = 0x00000000;
//
// turn on HSE  default at startup is 0x0000XX83
//
  reg_temp = *RCC_CR;
  reg_temp |= BIT16_MASK;
  *RCC_CR = reg_temp;
//  
// wait for HSE ready
//
  timeout = 1000000; 
  while(!(*RCC_CR & 0x00020000))
  {
    if(--timeout == 0)
    {
        error_log( 101 );
        //display_error( 101 );
        return;
    }
  }
//
// configure PLL
// VCO = 8 Mhz * N/M = 336 Mhz ( max 432 Mhz ) OK
// PLL = VCO/P = 168 Mhz  (max 168 Mhz ) OK
// USB OTG FS = VCO/Q = 48 Mhz (required for USB) OK
// default 0x24003010
// Q = 7   0x07000000 bits 24-27
// N = 168 0x00002A00 bits 6-14 (0xA8 << 6)
// M = 4   0x00000004 bits 0-5
// P = 2   0x00000000 bits 16-17 for 00 P = 2
//         0x27002A14
//
  *RCC_PLLCFGR = 0x27002A14;  
//
// turn on PLL
//
  *RCC_CR = 0x01018983;
//
// wait for PLL ready
//
  timeout = 10000000; 
  while(!(*RCC_CR & 0x02000000))
  {
    if(--timeout == 0) error_log( 102 );
    return;
  }
//
// use PLL for sys clock
//
  reg_temp = *RCC_CFGR & 0xFFFFFFFC;
  reg_temp |= 0x00000002; 
  *RCC_CFGR = reg_temp;
}
// ******************************************************************************
//
// Feb 8, 2016			Design Documents      		      msec_delay
//
// Causes a delay for milliseconds using sys tick timer
//
// Algorithms 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// External variables used
//   millisec
// 
// Internal variables used
//   time_start, time_now
//
// History
//  02/08/2016 written
//  03/18/2016 revised to handle wraparound on overflow
//
// Status
//  written
//
// ******************************************************************************

void msec_delay( unsigned int delay )
{
  static unsigned int time_start, time_now;

  time_start = millisec;    // read millisec timer 
  time_now = time_start;
 
  while ((time_now - time_start) < delay)
  {
    time_now = millisec;
    // handle wraparound to 0 on overflow max for unsigned long = 2^32 -1
    // 4,294,967,295 0xFFFFFFFF
    // for millisec timer, the wraparound would occur every 49.7 days
    // if modified to a 10 microsec timer, every 11.9 hours  
    if(time_now < time_start) // millisec wrapped around to 0
    {
      // reduce delay to what's left to time
      delay = delay - (0xFFFFFFFF - time_start);
      // reset starting time 
      time_start = time_now;
    }
  }
  return;
}
