//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// April 27, 2018 		Design Documents      	         CSI_timer_two.c
//
//       Routines for setup and use of timer peripherals 2 & 3
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  04/27/2018 imported CS7000 timer routines for conversion to stm32f4
//  04/04/2019 added 10microsec timer on Timer 3 for MC59
//  05/31/2019 made conditional compile for FT857D
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  init_10milli_timer             initialize 10 millisecond timer
//  timer_10milli_enable           turn on or off timer 2
//  TIM2_Handler                   timer 2 interrupt handler ISR
//  init_10micro_timer             initialize 10 microsecond timer
//  timer_10micro_enable           turn on or off timer 3
//  TIM3_Handler                   timer 3 interrupt handler ISR

//==============================================================================
#include <stdint.h>
#include "CSI_timer_two.h"
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_GPIO.h"
#include "CSI_i2c.h"
#include "CSI_RADIO.h"
#include "CSI_TIMERS.h"
#include "CSI_MC59.h"
#include "CSI_diagnostics.h"

#ifdef radio_FT857D
#include "CSI_FT857D.h"
#endif /* FT857D */

uint32_t volatile *RCC_APB1ENR = (uint32_t *) RCC_APB1ENR_ADDRESS;

uint32_t volatile *NVIC_ISER0  = (uint32_t *) NVIC_ISER0_ADDRESS;
uint32_t volatile *NVIC_ICPR0 = (uint32_t *) NVIC_ICPR0_ADDRESS;
uint32_t volatile *NVIC_IPR07  = (uint32_t *) NVIC_IPR0_ADDRESS + (7 * 4);

uint32_t volatile *TIM2_ARR    = (uint32_t *) TIM2_ARR_ADDRESS; 
uint32_t volatile *TIM2_CNT    = (uint32_t *) TIM2_CNT_ADDRESS;
uint16_t volatile *TIM2_CR1     = (uint16_t *) TIM2_CR1_ADDRESS;
uint16_t volatile *TIM2_DIER    = (uint16_t *) TIM2_DIER_ADDRESS;
uint16_t volatile *TIM2_SR      = (uint16_t *) TIM2_SR_ADDRESS;
uint16_t volatile *TIM2_PSC     = (uint16_t *) TIM2_PSC_ADDRESS;

uint32_t volatile *TIM3_ARR    = (uint32_t *) TIM3_ARR_ADDRESS; 
uint32_t volatile *TIM3_CNT    = (uint32_t *) TIM3_CNT_ADDRESS;
uint16_t volatile *TIM3_CR1     = (uint16_t *) TIM3_CR1_ADDRESS;
uint16_t volatile *TIM3_DIER    = (uint16_t *) TIM3_DIER_ADDRESS;
uint16_t volatile *TIM3_SR      = (uint16_t *) TIM3_SR_ADDRESS;
uint16_t volatile *TIM3_PSC     = (uint16_t *) TIM3_PSC_ADDRESS;

// =============================================================================
//
//  Set up timer for 10 millisecond timer
//    use Timer 2 (a 32 bit general purpose timer)
//    Set timer to generate interrupt at 100 hz
//    Timer 2 interrupt is #28
//    Timer 2 is on clock APB1

// UNRESOLVED: manual says the timer is on APB1 slow speed ( 42 Mhz )
//             But it is working correctly using 84 Mhz high speed to calc
//    Solved, see comment in init_10micro_timer() 
//
void init_10milli_timer( void )
{
  // turn on clock for timer  // bit 0 for timer2
  *RCC_APB1ENR |= 0x01;
  // TIM2_CR1 default is 0x00. no clock divider, upcounting, not one-pulse,
  // interrupt on update event, leave all at default.
  // set timer auto reload 84Mhz / 840000 = 100 hz 
  // 0xCD140 larger than 16 bit requires 32 bit timer 
  *TIM2_ARR =  840000; 
  // set timer counter
  *TIM2_CNT = 0;  
  // set timer prescaler, prescaler = 1   
  *TIM2_PSC = 0;
  
  // setup interrupt for timer 2
  
  *NVIC_ISER0 |= 0x10000000;   // set bit 28 for interrupt 28
  *NVIC_IPR07 |= 0x00000023;   // priority 35?  pr7 is for 28-31
   
    // enable interrupt generation
  *TIM2_DIER = 0x01;
}
// =============================================================================
//
//  turn on or off timer 2
//

void timer_10milli_enable( int onoff )
{
  // CR1 is a 16 bit register
  if(onoff == 1)
  {
    *TIM2_CR1 |= (uint16_t) 0x01;
    // enable interrupt generation
    *TIM2_DIER = 0x01;
  }
  else 
  {
    *TIM2_CR1 &= (uint16_t) ~0x01;
      // disable interrupt generation
      *TIM2_DIER = 0x00;
  }
}

// =============================================================================
//
//   TIMER 2 interrupt handler
//
//  Timer 2 is on interrupt 28

uint32_t two_min_timer = 0;

#pragma call_graph_root = "interrupt"
void TIM2_Handler( void )
{
  // clear interrupt 
  *NVIC_ICPR0 |= 0x10000000;
  // clear interrupt flag in the timer
  *TIM2_SR = 0;
  // run this timer
  timer_10ms();
  
  // =========================
  #ifdef radio_FT857D
  
  if( two_min_timer++ > 12000 )
  {
    two_min_timer = 0;
    // if FT857 
    if( current_radio_type == 1 || current_radio_type == 2 )
    {
      // check to see if any data has changed
      if( (FT857_sq_type != FT857_savd_sq_type) ||
          (FT857_squelch_value != FT857_savd_squelch_value) ||
          (FT857_volume_value != FT857_savd_volume_value))
      {
        // store current FT857 data
        i2c2_device_active = I2C_eprom;
        save_FT857_info( FT857_sq_type, FT857_squelch_value, FT857_volume_value );
          
        i2c2_device_active = I2C_touch;
      }
    }
  }
  #endif /* FT857D */
  // ==========================
  // toggle pin for debug test
  //test_pin();

}
// =============================================================================
//
//  Set up timer for 10 microsecond timer for mc59 keypad handler
//    use Timer 3 (a 16 bit general purpose timer)
//    Set timer to generate interrupt at 100 Khz
//    Timer 3 interrupt is #29
//    Timer 3 is on clock  APB1

// RESOLVED: manual says the timer is on APB1 slow speed ( 42 Mhz )
//          But it is working correctly using 84 Mhz to calculate. 
//    There is a prescaler in the timer modules that multiplies the clock x 2  
//    The register that has the setting for the multiplier is not listed in the 
//    Debugger registers for the CPU. Ref Manual pg 153. RCC_DKCFGR
//    RCC_DCKCFGR1 is in our list of registers. 
//  History:
//      04/04/2019 added this timer for MC59 to eliminate hum
//  Status: written, tested, working.

void init_10micro_timer( void )
{
  // turn on clock for timer  // bit 1 for timer3
  *RCC_APB1ENR |= 0x02;
  // TIM3_CR1 default is 0x00. no clock divider, upcounting, not one-pulse,
  // interrupt on update event, leave all at default.
  // set timer auto reload 84Mhz / 840 = 100 Khz 
  // less than 16 bit can use 16 bit timer
  // *TIM3_ARR =  840; 
  *TIM3_ARR =  840; 
  // set timer counter
  *TIM3_CNT = 0;  
  // set timer prescaler, prescaler = 1   
  *TIM3_PSC = 0;
  
  // setup interrupt for timer 3
  
  *NVIC_ISER0 |= 0x20000000;   // set bit 29 for interrupt 29
  *NVIC_IPR07 |= 0x00002400;   // priority 36  pr7 is for 28-31
   
  // enable interrupt generation, not here.
  // moved to have the burst timer turn on the timer and interrupt
  //*TIM3_DIER = 0x01;
  
}

// =============================================================================
//
//  turn on or off timer 3
//

void timer_10micro_enable( int onoff )
{
  // CR1 is a 16 bit register
  if(onoff == 1)
  {
    *TIM3_CR1 |= (uint16_t) 0x01;
    // enable interrupt generation
    *TIM3_DIER = 0x01;
  }
  else 
  {
    *TIM3_CR1 &= (uint16_t) ~0x01;
      // disable interrupt generation
      *TIM3_DIER = 0x00;
  }
}

// =============================================================================
//
//   TIMER 3 interrupt handler
//
//  Timer 3 is on interrupt 29

#pragma call_graph_root = "interrupt"
void TIM3_Handler( void )
{
  // clear interrupt 
  *NVIC_ICPR0 |= 0x20000000;
  // clear interrupt flag in the timer
  *TIM3_SR = 0;

  // run keypad state machine
  //mc59_read_keypad();   // Jerry's version 
  mc59_get_data();  // Dean's version
  // toggle pin for debug test
  //test_pin();

}



