//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 7,2018             Design Documents                     CSI_TouchPanel.c
//
//			routines for Touch Panel
//                   large panel display project
//
//   Touch Screen Controller is GT911 52 pin  
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//  
// History
//  06/07/2018 started
//  03/12/2019 merged with FT857
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  TouchPanel_int                set to configure I2C addresss for touch panel
//  TouchPanel_reset              set or clear touch panel reset pin
//  TouchPanel_init               initialize touch panel controller
//  TouchPanel_disable            turn off touch panel
//  TouchPanel_interrupt          set interrupt for touch screen
//  TouchPanel_read_multi         read multiple bytes from the touch panel
//  TouchPanel_waitfor_I2C_RX     wait for I2C to finish reading
//  TouchPanel_waitfor_I2C_TX     wait for I2C to finish transmitting
//  TouchPanel_read_touch1        read first touch point data into rx buffer
//  TouchPanel_write              write data to the touch panel controller
//  TouchPanel_get_touch_data     read status and all touch point data
//  TouchPanel_read_touch_buffer  gets touch data from the rx buffer
//  all_touches_off               deactivate handliing of touches
//  tp_button_handler             touch point simulates a button press
//  CS800D_process_touches        check for and handle screen touches
//  FT857D_process_touches        check for and handle screen touches
//  FT857D_touch_state_program    Touch point state machine

// Status: written, tested, not used 
//  TouchPanel_config_check       compare controller configuration with mfr data
//  TouchPanel_read_register      read a single byte register (has bugs)
//  TouchPanel_product_ID         read touch panel controller id
//  TouchPanel_command            send a command to the touch panel controller
//
//==============================================================================
#include "CSI_TouchPanel.h"
#include "CSI_eeprom.h"
#include "CSI_i2c.h"
#include "CSI_interrupt.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_TIMERS.h"
#include "CSI_REGISTERS.h"
#include "CSI_GPIO.h"
#include "CSI_SCREENS.h"
#include "CSI_SWITCHES.h"
#include "CSI_Menus.h"
#include "CSI_FEATURES.h"
#include "CSI_COLOR.h"
#include "CSI_GRAPHICS.h"
#include "CSI_ICONS.h"
#include "CSI_UART.h"
#include "CSI_SSD1963.h"
#include "CSI_CS800_TGMonitor.h"

#include "CSI_FT857D.h"

// The touch Panel interrupt handler is in CSI_interrupt.c

uint8_t volatile TS_status;

// panel configuration data for GT911 from Manufacturer
const uint8_t Touch_config1[186] = {
0x5A, 0xE1, 0x01, 0x21, 0x03, 0x05, 0x34, 0x00, 0x01, 0x08,
0x28, 0x05, 0x50, 0x32, 0x03, 0x05, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x09, 0x17, 0x18, 0x1E, 0x12, 0x8A, 0x2A, 0x0C,
0x17, 0x14, 0x31, 0x0D, 0x00, 0x00, 0x08, 0xC9, 0x04, 0x2D,
0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x64, 0x32, 0x00, 0x00,
0x00, 0x0F, 0x5F, 0x94, 0xC5, 0x02, 0x07, 0x00, 0x00, 0x04,
0xA1, 0x10, 0x00, 0x72, 0x18, 0x00, 0x56, 0x23, 0x00, 0x44,
0x35, 0x00, 0x3E, 0x4F, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x02, 0x04, 0x06, 0x08, 0x0A, 0x0C, 0x0E, 0x10,
0x12, 0x14, 0x16, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x02, 0x04, 0x06, 0x08, 0x0A, 0x0F, 0x10,
0x12, 0x13, 0x16, 0x18, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21,
0x22, 0x24, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x8D, 0x01 };


// config read from chip at start
// it's the same as the config provided
// (except the last byte which needs to be set to 01 to activate the change)
// Configuration data loading has not been written yet.
// the screen came loaded with a working configuration.
// 
/*
const uint8_t Touch_config2[186] = {
0x5A, 0xE1, 0x01, 0x21, 0x03, 0x05, 0x34, 0x00, 0x01, 0x08,
0x28, 0x05, 0x50, 0x32, 0x03, 0x05, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x09, 0x17, 0x18, 0x1E, 0x12, 0x8A, 0x2A, 0x0C,
0x17, 0x14, 0x31, 0x0D, 0x00, 0x00, 0x08, 0xC9, 0x04, 0x2D,
0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x64, 0x32, 0x00, 0x00,
0x00, 0x0F, 0x5F, 0x94, 0xC5, 0x02, 0x07, 0x00, 0x00, 0x04,
0xA1, 0x10, 0x00, 0x72, 0x18, 0x00, 0x56, 0x23, 0x00, 0x44,
0x35, 0x00, 0x3E, 0x4F, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x02, 0x04, 0x06, 0x08, 0x0A, 0x0C, 0x0E, 0x10,
0x12, 0x14, 0x16, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x02, 0x04, 0x06, 0x08, 0x0A, 0x0F, 0x10,
0x12, 0x13, 0x16, 0x18, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21,
0x22, 0x24, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x8D, 0x00 };
*/

/* comparison test, confirmed mfr config  = existing 
int TouchPanel_config_check( void )
{
  int i;
  for( i = 0; i < 186; i++)
  {
    if( Touch_config1[i] != Touch_config2[i]) break;
  }
  return i;
}
*/

// Touch Panel I2C address is 28/29 when RESET is pulled high with INT high
//                            BA/BB when RESET is pulled high with INT low
//  LSB bit of address 0 = write 1 = read 

// =============================================================================
//
//   Touch Panel INT pin control
//    
//    Int pin is configured as an input for the interrupt line
//    use this to set I2C addresss at startup
//    must reconfigure GPIO F6 to make it an output, before using this 
//
// Status: written, tested
//

void TouchPanel_int( uint8_t hilo )
{
  if( hilo ) *GPIO_F_BSRR = BIT06_ON;
  else *GPIO_F_BSRR = BIT06_OFF;
}
// =============================================================================
//
//   Touch Panel RESET pin control
//
// Status: written, tested
//

void TouchPanel_reset( uint8_t hilo )
{
  if( hilo ) *GPIO_F_BSRR = BIT02_ON;
  else *GPIO_F_BSRR = BIT02_OFF;
}
// =============================================================================
//
//   Touch Panel Init
//
//   No need to load configuration data. Controller is pre loaded.
// 
//  I2C2init is the same for both Touch Panel and EEPROM so nothing
//  more needed
//  With int line high (output), raising reset makes TP address 0x28/0x29
//  If int is set low,
//     raising reset makes TP address 0xBA/0xBB   
//  Working using 28/29
//
// History
//   06/07/2018
//   07/03/2018 added interupt priority setting
//
// Status: written, working
// 

void TouchPanel_init( void )
{
  uint32_t volatile *EXTI_IMR = (uint32_t *) EXTI_IMR_ADDRESS;
  uint32_t volatile *GPIOF_MODER = (uint32_t *) GPIOF_MODER_ADDRESS; 
  uint32_t volatile *GPIOF_ODR = (uint32_t *) GPIOF_ODR_ADDRESS;
  
  // INT line F6, mode is set as an output, data high during initialize regs
  // set as an output in case it is not 
  *GPIOF_MODER |= 0x00001000;    
  // reset line going high activates TP controller
  TouchPanel_reset( 1 );
  delay(6000); // GT911 datasheet says > 5 msec!
  // set INT line low
  *GPIOF_ODR &= ~BIT06_MASK;
  delay(51000);  // GT911 datasheet says > 50 msec!
  // change output to input on INT line
  *GPIOF_MODER &= ~0x00001000;  
  
  // load config data
  
  // set for hardware interrrupt
  *EXTI_IMR |= 1 << 6; // set line 6
  // set priority lower than I2C which needs to be used
  // (higher numbers are lower priority)
  // I2C2 events 40 errors 41
  // only high bits used I2C2ev 2 I2C2er 2
  // set to 50, high bits 3
  set_interrupt_priority( EXTI9_5_INTR, 50 );
  
  trigger_enable( 6 , 1 ); // set line 6, 0 is for rising
  i2c2_device_active = I2C_touch;
  nvic_enableIRQ( EXTI9_5_INTR );  // enable external interrupt
  
  /*
  // write to clear data buffer reg
  TouchPanel_write ( GT911_Status, 0 );
  TouchPanel_waitfor_I2C();
  */
}
// =============================================================================
//
//   Touch Panel Disable
//
//   The touch panel runs almost all the time, waiting for touches.
//   The Touch panel is on the same I2C as the EEPROM
//   Turn off the touch panel when using the EERPROM
//   
// Status: written,
// 
void TouchPanel_disable( void )
{
  TouchPanel_reset( 0 );
  TouchPanel_interrupt( 0 );
  if( i2c2_device_active == I2C_touch) i2c2_device_active = 0;
}
// =============================================================================
//
// Touch screen interrupt control
//
//   
// Status: written, working
// 
void TouchPanel_interrupt( uint8_t onoff )
{
  if(onoff) nvic_enableIRQ( EXTI9_5_INTR );
  else nvic_disableIRQ( EXTI9_5_INTR );
  
}

// =============================================================================
//
//   Touch Panel read register
//     register addresses are 16 bit, sent high byte then low
//     Returns the register value which is also put in the datain buffer
//
// History
//     June 7,2018 
//
// Status: written, Has problems, using read multi instead
//     Debugging: suspect I2C with only 1 byte to read is shutting off too soon
//                     
//
uint8_t TouchPanel_read_register( uint16_t regAddr )
{
  uint8_t datain;
  uint8_t control_byte = I2C_touch;  
  TxRxDevAddress = control_byte;
  // put data in the dataout buffer
  I2C_put_dataout( regAddr >> 8 );  
  I2C_put_dataout( regAddr & 0x00FF );

  I2C_rx_count_to_read = 1;  
  I2C_rx_count = 0;
  // start tx, I2C 2, device, 3 bytes
  i2c_start_tx( 2, control_byte, 3 );
  
  // delay(100);
  // wait for I2C to finish
  
  if( TouchPanel_waitfor_I2C_RX() == 1)
  {
    // receiving is done in the ISR
    datain = I2C_get_datain();
  }
  return datain;
}
// =============================================================================
//
//   Touch Panel read multiple registers
//     register addresses are 16 bit, sent high byte then low
//     will read numbytes of sequential registers starting at the address
//     Does not return anything, data read is in the datain buffer
//
// History
//     June 8,2018 
//
// Status: written, working
//
void TouchPanel_read_multi( uint16_t regAddr, uint8_t numbytes )
{
  uint8_t control_byte = I2C_touch;
  TxRxDevAddress = control_byte;
  // put data in the dataout buffer
  I2C_put_dataout( regAddr >> 8 );  
  I2C_put_dataout( regAddr & 0x00FF );

  I2C_rx_count_to_read = numbytes;
  I2C_rx_count = 0;  
  // start tx, I2C 2, device, 3 bytes
  i2c_start_tx( 2, control_byte, 3 );
  
  // receiving is done in the ISR

}
// =============================================================================
//
//   Touch Panel read product ID
//     Does not return anything, data read is in the datain buffer
//
//   Read "911" in ascii with /0 4th byte 
//
// History
//     June 8,2018 
//
// Status: written, working
//
void TouchPanel_product_ID( void )
{
  uint8_t control_byte = I2C_touch;
  TxRxDevAddress = control_byte;
  // put data in the dataout buffer
  I2C_put_dataout( (GT911_Product_ID & 0xFF00) >> 8 );  
  I2C_put_dataout( GT911_Product_ID & 0x00FF );

  I2C_rx_count_to_read = 4;
  I2C_rx_count = 0;  
  // start tx, I2C 2, device, 3 bytes
  i2c_start_tx( 2, control_byte, 3 );
  
  // receiving is done in the ISR

}
// =============================================================================
//
//  Touch panel wait for I2C to finish
//  reset I2C and Touch panel if it times out
//
// The reset was a fix until the I2C was debugged.
//  If the I2C is working correctly, it doesn't timeout
//
//  Leaving it in for error handle

//  08/31/2018 added to trace touch screen hang problem
//  11/21/2018 getting occasional resets
//  02/14/2018 added turn off of i2c before resetting
//             added I2C_rx_count_to_read = 0, so interrupt handler stops
               
int TouchPanel_waitfor_I2C_RX( void )
{
   int TS_waitcount = 40;
   unsigned long msec;
   msec = millisec;
   while(!I2C_2_txrx_complete)
   {
    if(millisec != msec) 
    {
      //msec++;
      msec = millisec;      
      if(--TS_waitcount == 0)
      {
         error_log(2401);
         log_event( "TS-I2C rx reset" );
         // re-initialize I2C and Touch screen
         I2C_Disable( 2 );  // turn off i2c 2
         TouchPanel_init();          
         I2C_rx_count_to_read = 0;
         I2C_2_txrx_complete = 1;
         i2c_2_initialize( I2C_touch );
         return 0;
      }
    }
  }
  return 1;
}
// =======================================
int TouchPanel_waitfor_I2C_TX( void )
{
   int TS_waitcount = 100;
   unsigned long msec;
   msec = millisec;
   while(!I2C_2_txrx_complete)
   {
    if(millisec != msec) 
    {
      //msec++;
      msec = millisec;      
      if(--TS_waitcount == 0)
      {
         error_log(2402);
         log_event( "TS-I2C tx reset" );
         // re-initialize I2C and Touch screen
         I2C_Disable( 2 );  // turn off i2c 2
         TouchPanel_init();          
         I2C_rx_count_to_read = 0;
         I2C_2_txrx_complete = 1;
         i2c_2_initialize( I2C_touch );
         return 0;
      }
    }
  }
  return 1;
}


// =============================================================================
//
//   Touch Panel read coordinate data for touch 1
//     Does not return anything, data read is in the datain buffer
//
//  read  status 1 byte
//        id     1 byte
//        x      2 bytes
//        y      2 bytes
//        size   2 bytes  
//
// History
//     June 8,2018 
//
// Status: writing, not used
//
void TouchPanel_read_touch1( void )
{
  uint8_t control_byte = I2C_touch;
  TxRxDevAddress = control_byte;
  // put address in the dataout buffer
  I2C_put_dataout( (GT911_Status & 0xFF00) >> 8 );  
  I2C_put_dataout( GT911_Status & 0x00FF );

  I2C_rx_count_to_read = 8;
  I2C_rx_count = 0;  
  // start tx, I2C 2, device, 3 bytes
  i2c_start_tx( 2, control_byte, 3 );
  // receiving is done in the ISR
 
}
// =============================================================================
//
//   Touch Panel Command
//
//    Commands are written to the command register:
//    0x00 read coordinate
//    0x01 read diff data or raw data
//    0x02 software reset
//    0x03 baseline update
//    0x04 baseline calibration
//    0x05 screen off  
//
//
// History
//     June 8,2018 
//
// Status: writing, testing, not used
//
void TouchPanel_command ( uint8_t command )
{
  uint8_t control_byte = I2C_touch;
  TxRxDevAddress = control_byte;
  // put register address in the dataout buffer
  I2C_put_dataout( (GT911_Command & 0xFF00) >> 8 );  
  I2C_put_dataout( GT911_Command & 0x00FF );
  // put command in buffer
  I2C_put_dataout( command ); 

  I2C_rx_count_to_read = 0;
  I2C_rx_count = 0;  
  // start tx, I2C 2, device, 4 bytes
  i2c_start_tx( 2, control_byte, 4 );
  
  // receiving is done in the ISR

}
// =============================================================================
//
//   Touch Panel write register
//
//  need to set the "fresh" register after
//  writing to controller registers
//   TouchPanel_write( GT911_Config_fresh, 0x01 );
//
//  Changes to registers are not permanent.
//  The original data is restored after a reset .
//
// History
//     June 11,2018 
//
// Status: written, working
//
void TouchPanel_write ( uint16_t regnum, uint8_t data )
{
  uint8_t control_byte = I2C_touch;
  TxRxDevAddress = control_byte;
  // put register address in the dataout buffer
  I2C_put_dataout( (regnum & 0xFF00) >> 8 );  
  I2C_put_dataout( regnum & 0x00FF );
  // put command in buffer
  I2C_put_dataout( data ); 

  I2C_rx_count_to_read = 0;
  I2C_rx_count = 0;  
  // start tx, I2C 2, device, 4 bytes (cntrl 1, address 2, data 1)
  i2c_start_tx( 2, control_byte, 4 );

}

Touch_info Panel_Touch[5];

// =============================================================================
//
//   Touch Panel read status and coordinate data for touches
//     Does not return anything,
//     data read by I2C is placed in the datain buffer
//     then read from the buffer and stored in the Panel_Touch array
//
//  read  status 1 byte
//        id     1 byte
//        x      2 bytes
//        y      2 bytes
//        size   2 bytes  
//
// History
//     June 13,2018
//
// Status: writing, working.
//   problem: Tries to read when there is no data, causes buffer empty errors
//   this routine is only called once, from the interrupt routine
//
uint8_t prev_t_count = 0;
uint8_t TS_touch_count = 0;

void TouchPanel_get_touch_data( uint8_t status )
{
  uint8_t hi_byte;
  uint8_t lo_byte;
  uint8_t reserve;
  int i;
  //int msec;
  // int gtd_timeout;

  TS_touch_count = status & 0x07;   // three low bits is the number of touches
  if(TS_touch_count > 5) TS_touch_count = 5;
  // status count is 0 to 5, array is 0 to 4
  // a count of 0 is no touch and it isn't valid to put data in the array

  // if count has reduced, clear data in array
  while(prev_t_count > TS_touch_count)
  {
   prev_t_count--;
   Panel_Touch[prev_t_count].id = 0;
   Panel_Touch[prev_t_count].x_coord = 0;
   Panel_Touch[prev_t_count].y_coord = 0;
   Panel_Touch[prev_t_count].size = 0;
  }
  
  // store for next touch
  prev_t_count = TS_touch_count;
  
  // if touch was only 1 it has already been read
  if(TS_touch_count > 1)
  {
    // read status reg and all touches
    TxRxDevAddress = I2C_touch;
    // put address in the dataout buffer
    I2C_put_dataout( GT911_Status >> 8 );  
    I2C_put_dataout( GT911_Status & 0x00FF );

    I2C_rx_count_to_read = 8 * TS_touch_count;
    I2C_rx_count = 0;  
    // start tx, I2C 2, device, 3 bytes
    i2c_start_tx( 2, I2C_touch, 3 );
   
    //log_event( "read TS data" );
    // wait for read to finish
   
    if( TouchPanel_waitfor_I2C_RX() == 1 )
    {
      // get data from buffer
      for( i = 0; i < TS_touch_count; i++)
      {
        // the first byte is the status register
        // after that there is a reserved byte
        if( i == 0) status = I2C_get_datain();
        else reserve = I2C_get_datain();
        if(reserve); // to eliminate compiler warning
        Panel_Touch[i].id =  I2C_get_datain();
        lo_byte = I2C_get_datain();
        hi_byte = I2C_get_datain();
        Panel_Touch[i].x_coord = (hi_byte << 8) + lo_byte;
        lo_byte = I2C_get_datain();
        hi_byte = I2C_get_datain();
        Panel_Touch[i].y_coord = (hi_byte << 8) + lo_byte;
        lo_byte = I2C_get_datain();
        hi_byte = I2C_get_datain();  
        Panel_Touch[i].size = (hi_byte << 8) + lo_byte;
      }
    }
  }  
}
// =============================================================================
//
//  read from buffer and store in touch array
//
//  returns status register, and only the 1st touch point
//  Must have 8 bytes read from the Touch panel into the receive buffer
//
// History
//     June 13,2018 
//
// Status: written, tested, working

uint8_t TouchPanel_read_touch_buffer( void )
{
    uint8_t status;
    uint8_t hi_byte;
    uint8_t lo_byte;
    
    status = 0;
    if( dbufin_readynum() > 7)
    {
      status = I2C_get_datain();
      
      Panel_Touch[0].id =  I2C_get_datain();
      lo_byte = I2C_get_datain();
      hi_byte = I2C_get_datain();
      Panel_Touch[0].x_coord = (hi_byte << 8) + lo_byte;
      lo_byte = I2C_get_datain();
      hi_byte = I2C_get_datain();
      Panel_Touch[0].y_coord = (hi_byte << 8) + lo_byte;
      lo_byte = I2C_get_datain();
      hi_byte = I2C_get_datain();  
      Panel_Touch[0].size = (hi_byte << 8) + lo_byte;
    }
    
    return status;
}

// =============================================================================
//
// touch screen touch location array
// X and Y are switched in the touch panel controller !!
// is_active, touch, tp_state,  minx, maxx, miny, maxy
uint32_t CS800D_num_touch_points = 66; 

// !!! remember
// !!! X and Y are switched in the touch panel controller

  //    Ymin Ymax  Xmin Xmax
Touch_zone CS800D_touch_point[66] = {
  {0, 0, 0, 380, 420, 740, 800},  // volume up
  {0, 0, 0, 380, 420, 580, 680},  // volume down
  {0, 0, 0, 230, 270,  40,  90},  // channel up
  {0, 0, 0, 330, 370,  40,  90},  // channel down
  {0, 0, 0, 430, 480, 290, 320},  // lastcall popup
  {0, 0, 0, 120, 180,  50, 400},  // zone alias
  {0, 0, 0, 260, 330, 160, 460},  // channel alias

  // buttons 7-11
  {0, 0, 0, 420, 470, 365, 425},  // P1
  {0, 0, 0, 420, 470, 450, 510},  // P2
  {0, 0, 0, 420, 470, 535, 595},  // P3
  {0, 0, 0, 420, 470, 620, 680},  // P4
  {0, 0, 0, 420, 470, 705, 765},  // Power bttn
  
  // color adjustment 12-17
  {0, 0, 0, 350, 380, 180, 210},   // red+
  {0, 0, 0, 350, 380, 110, 140},   // red-
  {0, 0, 0, 400, 430, 180, 210},   // green+
  {0, 0, 0, 400, 430, 110, 140},   // green-
  {0, 0, 0, 450, 480, 180, 210},   // blue+
  {0, 0, 0, 450, 480, 110, 140},   // blue-

  // favorites screens 18-33
  {0, 0, 0,  80, 120,  30, 390},  // 1
  {0, 0, 0, 120, 160,  30, 390},  // 2
  {0, 0, 0, 160, 200,  30, 390},  // 3
  {0, 0, 0, 200, 240,  30, 390},  // 4
  {0, 0, 0, 240, 280,  30, 390},  // 5
  {0, 0, 0, 280, 320,  30, 390},  // 6
  {0, 0, 0, 320, 360,  30, 390},  // 7
  {0, 0, 0, 360, 400,  30, 390},  // 8
  {0, 0, 0,  80, 120, 430, 790},  // 9
  {0, 0, 0, 120, 160, 430, 790},  // 10
  {0, 0, 0, 160, 200, 430, 790},  // 11
  {0, 0, 0, 200, 240, 430, 790},  // 12
  {0, 0, 0, 240, 280, 430, 790},  // 13
  {0, 0, 0, 280, 320, 430, 790},  // 14
  {0, 0, 0, 320, 360, 430, 790},  // 15
  {0, 0, 0, 360, 400, 430, 790},  // 16  
  
  {0, 0,  0, 18,  90, 700, 780},    // RSSI icon 34
  // vfo knob points 35-58
  // starting corner for the knob 530, 230
  // touch point is 20x20 inside 40x40 button

  {0, 0, 0, 215, 245, 585, 615}, //12:00  //35
  {0, 0, 0, 217, 247, 603, 633},          //36
  {0, 0, 0, 224, 254, 620, 650},          //37
  {0, 0, 0, 236, 266, 634, 664},          //38 
  {0, 0, 0, 250, 280, 646, 676},          //39
  {0, 0, 0, 267, 297, 653, 683},          //40 
  {0, 0, 0, 285, 315, 655, 685}, // 3:00  //41
  {0, 0, 0, 303, 333, 653, 683},          //42
  {0, 0, 0, 320, 350, 646, 676},          //43
  {0, 0, 0, 334, 364, 634, 664},          //44
  {0, 0, 0, 346, 376, 620, 650},          //45
  {0, 0, 0, 353, 383, 603, 633},          //46
  {0, 0, 0, 355, 385, 585, 615}, // 6:00  //47
  {0, 0, 0, 353, 383, 567, 597},          //48
  {0, 0, 0, 346, 376, 550, 580},          //49
  {0, 0, 0, 334, 364, 536, 566},          //50 
  {0, 0, 0, 320, 350, 524, 554},          //51
  {0, 0, 0, 303, 333, 517, 547},          //52 
  {0, 0, 0, 285, 315, 515, 545}, // 9:00  //53 
  {0, 0, 0, 267, 297, 517, 547},          //54
  {0, 0, 0, 250, 280, 524, 554},          //55
  {0, 0, 0, 236, 266, 536, 566},          //56 
  {0, 0, 0, 224, 254, 550, 580},          //57
  {0, 0, 0, 217, 247, 567, 597},          //58 
  // for board test
  {0, 0, 0,   0,  50,   0,  80}, // Top L //59
  {0, 0, 0,   0,  50, 720, 800}, // Top R //60
  {0, 0, 0, 430, 480,   0,  80}, // Bot L //61 
  {0, 0, 0, 430, 480, 720, 800}, // Bot R //62
  // day night buttons
  {0, 0, 0, 200, 278, 582, 615}, // day   //63
  {0, 0, 0, 280, 358, 582, 615}, // night //64
  // TGM clear button
  {0, 0, 0, 420, 470,  40, 140}, // TGM graph clear
  
};

/*  not using these yet
  
  // channel selector popup
  {0, 0,   0,  49, 250, 300},    //1
  {0, 0,  50,  99, 250, 300},
  {0, 0, 100, 149, 250, 300},
  {0, 0, 150, 199, 250, 300},
  {0, 0, 200, 249, 250, 300},
  {0, 0, 250, 299, 250, 300},
  {0, 0, 300, 349, 250, 300},
  {0, 0, 350, 399, 250, 300},    //8 
  {0, 0, 400, 449, 250, 300},
  {0, 0, 450, 499, 250, 300},
  {0, 0, 500, 549, 250, 300},
  {0, 0, 550, 599, 250, 300},
  {0, 0, 600, 649, 250, 300},
  {0, 0, 650, 699, 250, 300},
  {0, 0, 700, 749, 250, 300},
  {0, 0, 750, 799, 250, 300},   //16
       
  
  // icons
  {0, 0,  18,  90,  20,  72},    // power icon (screen 5)
  {0, 0,  18,  90,  90, 125},    // monitor
  {0, 0,  18,  90, 132, 204},    // messages
  {0, 0,  18,  90, 204, 276},    // encrypt
  {0, 0,  18,  90, 290, 362},    // tone disable
  {0, 0,  18,  90, 376, 448},    // private/group call
  {0, 0,  18,  90, 440, 516},    // talk_around
  {0, 0,  18,  90, 520, 592},    // Scan
  {0, 0,  18,  90, 620, 692},    // GPS
  {0, 0, 270, 330, 620, 760},  // power text box  
};
*/

// =============================================================================
//
//  Deactivate all touch points
//
//  for easier screen switching, clear all and only set the active ones
//
// History: 09/28/2018 
//
// Status: written, needs revision for use with FT857

void all_touches_off( void )
{
  int i;
  for(i = 0; i < CS800D_num_touch_points; i++) CS800D_touch_point[i].is_active = 0;
}
// =============================================================================
//
//
//  button press handled by a touch point 
//
//  03/08/2019 
//
uint8_t tp_being_handled;

void tp_button_handler( uint8_t tp_num, uint8_t button_num, uint16_t timeout )
{
  if( CS800D_touch_point[tp_num].tp_state  == 0)
  {
    // it's been pressed
    CS800_buttons[button_num].press_flag = 1;
    // set timer to release, short press only      
    short_release_flag = &CS800_buttons[button_num].release_flag;
    short_press_timeout = timeout;          
    CS800D_touch_point[tp_num].tp_state = 1;
  }
  else // unreleased
  {
    if( CS800D_touch_point[ tp_being_handled ].touch == 0)
    {
      // released
      tp_being_handled = 0;
      CS800D_touch_point[tp_num].tp_state = 0;
    }
  }    
}
//
// =============================================================================
//
//  Locate touch points on screen
//   handle action for a valid touch
//
//  For now, directly reading touch array data.
//   Need to modify to read state machine debounced, averaged data
//   After state machine is working.
//
//  sets is_active for a touch point when a touch is made
//  clears is_active when released

// History: 08/24/2018
//          02/18/2019 renamed for CS800D only
//
// Status: written, working 

int active_touch = 0;
int prev_active_touch = 36;  // for the vfo knob

uint8_t dial_pos[7];

int b_test_tp = 0;

void CS800D_process_touches( void )
{
  int i;
  int cur_x;
  int cur_y;
  int touch_point_count = 0;
  // int rot_direction = 0;     // 0 =clockwise 1 =ccw
  b_test_tp = 0;
 
  // turn off touch interrupt while reading data
  nvic_disableIRQ( EXTI9_5_INTR ); 
  // trap touch data for this routine
  cur_x = Panel_Touch[0].x_coord;
  cur_y = Panel_Touch[0].y_coord;
  // clear data so that it doesn't get used over
  Panel_Touch[0].x_coord = 0;
  Panel_Touch[0].y_coord = 0;  
  // enable touch interrupt 
  nvic_enableIRQ( EXTI9_5_INTR );  
  
  if((short_press_timeout == 0) && (short_press_holdoff == 0))
  {
    // which touch?
    if(TS_touch_count > 0)
    {
      for(i = 0; i < CS800D_num_touch_points; i++)
      {
        if( CS800D_touch_point[i].is_active )
        {
          // if it's in the box
          if( (cur_x > CS800D_touch_point[i].xmin) &&
              (cur_x < CS800D_touch_point[i].xmax) &&
              (cur_y > CS800D_touch_point[i].ymin) &&
              (cur_y < CS800D_touch_point[i].ymax) )
          {
            touch_point_count++;
            CS800D_touch_point[i].touch = 1;
            active_touch = i+1;
          }
          else CS800D_touch_point[i].touch = 0;
        }
      }
    }
    
// handle touches that do not require a release
/* vfo knob turned off for now...   
    // vfo knob
    if( active_touch > 35 && active_touch < 60 )
    {
      if( active_touch > prev_active_touch )
      {
        rot_direction = 0; // clockwise
      }
      if( active_touch < prev_active_touch )
      {
        rot_direction = 1; // counter clockwise
      }
      // handle 12:00 wrap
      if( active_touch < prev_active_touch - 22 )
      {
        rot_direction = 0; // clockwise        
      }
      if( active_touch > prev_active_touch + 22 )
      {
        rot_direction = 1; // counter clockwise        
      }
      // now that we have direction, set the same as the right knob
      if( rot_direction == 1 )
      {
        last_key_press = 0x1C;
        key_type = 2; 
      }
      if( rot_direction == 0 ) 
      {
        last_key_press = 0x1C;
        key_type = 3;
      }
      prev_active_touch = active_touch;
      active_touch = 0;
    }
    else prev_active_touch = dial_rotation + 35;
*/
    
// handle touches that require a release
    
    // finish processing the previous touch if no new ones
    if(active_touch == 0 && tp_being_handled > 0 )
                      active_touch = tp_being_handled +1;
    // if( TS_touch_count == 0)  // no new touches, prior active can be handled
    if( TS_touch_count == 0 || tp_being_handled > 0 ) 
    {
      if(active_touch)
      {
        // handle the active touch,
        // active touch point 1 is the first table item touch_point[0]
        // there is no tp zero.
        switch(active_touch)
        {
        case 1:
          {
            // volume up
            volume_updown( 0 );          
            break;
          }
        case 2:
          {
            // volume down
            volume_updown( 1 );  
            break;
          }
        case 3:
          {
            // channel up
            channel_updown( 1 );            
            break;
          }        
        case 4:
          {
            // channel down
            channel_updown( 0 );            
            break;
          }
        case 5:
          {
            if( short_press_timeout == 0 )
            {
              // lastcall popup
              popup_lastcall_list();  // toggles
              short_release_flag = &CS800_buttons[2].release_flag;
              short_press_timeout = 200;                
            }
            break;
          }
        case 6: // zone alias
          {
            if( short_press_timeout == 0 )
            {
              // set up direct entry to favorites
              display_mode = 3;   // no packet displays for now
              all_touches_off();
              // turn off sending buttons to radio
              deactivate_all_keypress();
              active_screen_num = 6;
              // font colors 
              set_font_colors( 3, 14 );   // 
              process_zone_favs( 0 );
              short_release_flag = &CS800_buttons[2].release_flag;
              short_press_timeout = 200;
            }              
            break;
          }          
        case 7: // channel alias
          {
            if( short_press_timeout == 0 )
            {            
              // set up direct entry to favorites
              display_mode = 3;   // no packet displays for now
              all_touches_off();
              // turn off sending buttons to radio
              deactivate_all_keypress();
              active_screen_num = 6;
              // font colors 
              set_font_colors( 3, 14 );   // 
              process_channel_favs( 0 );
              short_release_flag = &CS800_buttons[2].release_flag;
              short_press_timeout = 200;  
            }
            break;
          }
        case 8:
          {
            // P1
            tp_being_handled = 7;
            tp_button_handler( 7, 3, 100 );            
            break;
          } 
        case 9:
          {
            // P2
            tp_being_handled = 8;
            tp_button_handler( 8, 4, 100 );
            break;
          } 
        case 10:
          {
              // P3
            tp_being_handled = 9;
            tp_button_handler( 9, 5, 100 );            
            break;
          }
        case 11:
          {
              // P4
            tp_being_handled = 10;
            tp_button_handler( 10, 6, 100 );            
            break;
          }
        case 12:  // power button icon
          {
            if( short_press_timeout == 0 )
            {
              // display menu for panel controls
              if(display_mode == 0)
              {
                reset_panel_menu();  // reset before starting                 
                display_mode = 1;
                display_screen( 6 , 1);  // for front panel menu
              }
              short_release_flag = &CS800_buttons[2].release_flag;
              short_press_timeout = 300;  //.3 sec
            }
            break;
          }
        case 13:
          {
            // red+
            if(adj_color.red < 255)
            {
              adj_color.red++;
              display_color( color_adj_x, color_adj_y, adj_color);
            }
            break;
          }          
        case 14:
          {
            // red-
            if(adj_color.red > 0)
            {
              adj_color.red--;
              display_color( color_adj_x, color_adj_y, adj_color);
            }
            break;
          }
        case 15:
          {
            // green+
            if(adj_color.green < 255)
            {
              adj_color.green++;
              display_color( color_adj_x, color_adj_y, adj_color);
            }
            break;
          }    
        case 16:
          {
            // green-
            if(adj_color.green > 0)
            {
              adj_color.green--;
              display_color( color_adj_x, color_adj_y, adj_color);
            }
            break;
          }
        case 17:
          {
            // blue+
            if(adj_color.blue < 255)
            {
              adj_color.blue++;
              display_color( color_adj_x, color_adj_y, adj_color);
            }
            break;
          }
        case 18:
          {
            // blue-
            if(adj_color.blue > 0)
            {
              adj_color.blue--;              
              display_color( color_adj_x, color_adj_y, adj_color );
            }
            break;
          }
        case 35:  // RSSI icon
          {
            if( short_press_timeout == 0 )
            {
              // toggle rssi graph on 
              rssi_graph_on ^= 1;
              
              if( rssi_graph_on )  // if turned on
              {
                // clear gps and rssi icon
                clear_rectangle( 615, 10, 180, 80); 
                // draw graph done by call in timer
              }
              else              // else turn off
              {
                // clear graph area
                clear_rectangle( 615, 10, 180, 80);               
                // redraw gps and rssi icons
                draw_gps();
 
                rssi_icon_isactive = 0;  // force redraw
                draw_rssi_icon( 0, 690, 15 );
                // clear the data
                for( i = 0; i < 180; i++) RSSI_history[ i ] = 0;
              }
              short_release_flag = &CS800_buttons[2].release_flag;
              short_press_timeout = 300;                
            }
            break;
          }
         // board tests, all the same 
        case 60:  // top left
        case 61:  // top right
        case 62:  // bottom left
        case 63:  // bottom right         
          {
            // reduce to 1-4
            b_test_tp = active_touch - 59;
            break;
          }
        case 64: // day brightness button
          {
            // set brightness control
            ssd1963_set_brightness( day_bright_val );
            short_press_timeout = 300;  //.3 sec
            break;
          }
        case 65: // night brightness
          {
            // set brightness control
            ssd1963_set_brightness( night_bright_val );
            short_press_timeout = 300;  //.3 sec
            break;
          }
        case 66: // TGM clear button
          {
            // clear table
            TGM_clear_table();
            // redisplay graph
            TGM_display_graph();
            break;
          }
          
        // no handling of icon touches yet 
        default: break;
        }        
        // handle all favorite touch points  19-34
        // active touch is index + 1
        if( (active_touch > 18) && (active_touch < 35) ) 
        {
          // make touched favorite the current item
          // if there is data ( can't make a blank fav active)
          if( get_menu_stack() == CHANFAV )
          {
            if( favorite_channel[active_touch - 19].channel_num > 0 &&
                favorite_channel[active_touch - 19].channel_num < 17 )
            {
              cf_num = active_touch - 19;
              // set a select keypress
              last_key_press = 0x18;         
            }
          }
          if( get_menu_stack() == ZONEFAV )
          {
            if( favorite_zone[active_touch - 19].zone_num > 0 &&
                favorite_zone[active_touch - 19].zone_num < 250 )
            {
              zf_num = active_touch - 19;
              // set a select keypress
              last_key_press = 0x18;            
            }
          }
        }
        active_touch = 0;
      }    
    }
    if ( TS_touch_count == 0 ) 
    {
      // set all to released
      for( i = 0; i< CS800D_num_touch_points; i++)
      CS800D_touch_point[ i ].touch = 0;    
    }
    
  }
}

// =============================================================================
// for FT857

uint32_t FT857D_num_touch_points = 13; 

const uint16_t FT857D_touch_point[13][4] = {       // Ymin, Ymax,  Xmin, Xmax,

// top row   
10,  60,    8, 108,  // 0  left arrow 
10,  60,  122, 222,  // 1  right arrow
10,  60,  236, 336,  // 2  DSP key
10,  60,  350, 450,  // 3  down arrow
10,  60,  464, 564,  // 4  up arrow
10,  60,  578, 678,  // 5  V/M key
10,  60,  692, 792,  // 6  lock key
// left side 
100, 150,   8, 108,  // 7  Home
190, 240,   8, 108,  // 8  Clar
280, 330,   8, 108,  // 9  Func
// bottom 
365, 415, 200, 300,  // 10 A
365, 415, 385, 485,  // 11 B  
365, 415, 558, 658,  // 12 C
};  

const uint8_t FT857D_key_codes [15][4] = {
0x7F, 0x8F, 0x18, 0x00,   // 0  left arrow 
0xBF, 0x8F, 0x18, 0x00,   // 1  right arrow 
0xFE, 0x8F, 0x18, 0x00,   // 2  DSP key 
0xDF, 0x8F, 0x18, 0x00,   // 3  down arrow 
0xEF, 0x8F, 0x18, 0x00,   // 4  up arrow 
0xFB, 0x8F, 0x18, 0x00,   // 5  V/M key 
0xFD, 0x8F, 0x18, 0x00,   // 6  lock key 

0xFF, 0x87, 0x18, 0x00,   // 7  Home 
0xFF, 0x8F, 0x10, 0x00,   // 8  Clar 
0xF7, 0x8F, 0x18, 0x00,   // 9  Func 


0xFF, 0x8B, 0x18, 0x00,   // 10 A 
0xFF, 0x8D, 0x18, 0x00,   // 11 B 
0xFF, 0x8E, 0x18, 0x00,   // 12 C 
0xFF, 0x8F, 0x08, 0x00,   // 13 Select key
0xFF, 0x8F, 0x18, 0x00,   // 14 Release any key
};

uint8_t FT857D_major_state[13];    // Major_state
uint8_t FT857D_minor_state[13];    // Minor state

//
// =============================================================================
// This should be called from a 10 millisecond interrupt
//
//  Locate touch points on screen
//   handle action for a valid touch
//
//  For now, directly reading touch array data.
//   Need to modify to read state machine debounced, averaged data
//   After state machine is working.
//
//  sets is_active for a touch point when a touch is made
//  clears is_active when released

// History: 02/18/2019 revised from CS800d for FT857D
//
// Status: written, tested, working
//

void FT857D_process_touches( void )
{
  int tp;
  int cur_x;
  int cur_y; 
//
// Handle mechanical switches
//
//   FT857D_mechanical_switches();   // done in timer
//
// trap touch data for this routine in case interrupt routine has new data
// before this routine has processed the data. Clear the data so it will not
// be used again. 
//
// If this routine has a lower priority than the program that captures the data
// then the interrupt should be disabled for the next four instructions.
//
  cur_y = Panel_Touch[0].x_coord;
  cur_x = Panel_Touch[0].y_coord;
  Panel_Touch[0].x_coord = 0;
  Panel_Touch[0].y_coord = 0;  

  for (tp = 0; tp < 13; tp++)
  {
    FT857D_touch_state_program(cur_y,cur_x, tp);
  }
  
}
// =============================================================================
//
//  Log touch point events
//
void FT857_log_touch_event( uint8_t tp )
{
  switch(tp)
  {
  case 3: log_event( "DSP" ); break;
  case 5: log_event( "V/M" ); break;
  case 6: log_event( "Lock" ); break;
  case 7: log_event( "Home" ); break;
  case 8: log_event( "Clar" ); break;
  case 9: log_event( "Func" ); break;
  case 10: log_event( "Button A" ); break;
  case 11: log_event( "Button B" ); break;
  case 12: log_event( "Button C" ); break;
  default: break;
  }
}

// =============================================================================
//
// State program for touch screen
//
void  FT857D_touch_state_program(uint32_t cur_y, uint32_t cur_x, uint32_t tp)                               
{
  // State 0 is waiting for the first match
  if (FT857D_major_state[tp] == 0)
  {
    if((cur_x > FT857D_touch_point[tp][2]) &&
       (cur_x < FT857D_touch_point[tp][3]) &&
       (cur_y > FT857D_touch_point[tp][0]) &&
       (cur_y < FT857D_touch_point[tp][1]))
    {
      FT857D_major_state[tp] = 1;
      FT857D_minor_state[tp] = 0;
    }
    return;
  }

  // State 1 is waiting for five contigous matches. If we get five contigous
  // matches, then we advance to the next state to process the key press
  if (FT857D_major_state[tp] == 1)
  {
    if((cur_x > FT857D_touch_point[tp][2]) &&
       (cur_x < FT857D_touch_point[tp][3]) &&
       (cur_y > FT857D_touch_point[tp][0]) &&
       (cur_y < FT857D_touch_point[tp][1]))
    {
      FT857D_minor_state[tp]++;
      if (FT857D_minor_state[tp] == 4)
      {
        FT857D_major_state[tp]++;
      }
    }
    else
    {
      if ((cur_y + cur_x > 0))    // check for valid point
      {
        FT857D_major_state[tp] = 0;
      }
    }
    return;
  }
  // State 2 is waiting for the UART to be available to accept Tx data.
  // If ready to transmit, start the transmission.
  if (FT857D_major_state[tp] == 2)
  {
    if ( FT857D_uart1_isready() )
    {
      // Setup transmitter buffer and start by outputing 0xA5
      FT857D_tx_buffer[0] = 0x06;
      FT857D_tx_buffer[1] = 0x91;
      FT857D_tx_buffer[2] = FT857D_key_codes [tp][0];
      FT857D_tx_buffer[3] = FT857D_key_codes [tp][1];
      FT857D_tx_buffer[4] = FT857D_key_codes [tp][2];
      FT857D_tx_buffer[5] = FT857D_key_codes [tp][3];
      FT857D_tx_buffer[6] = 0x91 + FT857D_tx_buffer[2] + FT857D_tx_buffer[3] +
                                  FT857D_tx_buffer[4] + FT857D_tx_buffer[5];
      uart_FT857D_send_0xA5();
      FT857D_major_state[tp]++;
      FT857D_minor_state[tp] = 0;
      // event log
      FT857_log_touch_event( tp );      
      
    }
    return;
  }

  // State 3 is waiting for five contigous mismatches 
  if (FT857D_major_state[tp] == 3)
  { 
    if((cur_x > FT857D_touch_point[tp][2]) && (cur_x < FT857D_touch_point[tp][3]) 
    && (cur_y > FT857D_touch_point[tp][0]) && (cur_y < FT857D_touch_point[tp][1]))
    {
      FT857D_minor_state[tp] = 0;      
    }
    else
    {
      FT857D_minor_state[tp]++;
      if (FT857D_minor_state[tp] == 5)
      {
        FT857D_major_state[tp]++;
      }
    }
    return;
  }

  // State 4 is waiting for the UART to be available to accept Tx data.
  // If ready to transmit, start the transmission.
  if (FT857D_major_state[tp] == 4)
  {   
    if ( FT857D_uart1_isready() )
    {

      // Setup transmitter buffer and start by
      // outputing 0xA5 0xFF, 0x8F, 0x18, 0x00
      process_release_packet();

      // go to to state 0 to process next key press.
      FT857D_major_state[tp] = 0;
    }
    return;    
  }

  // If we got here there was a problem in the firmware.
  FT857D_major_state[tp] = 0;
  error_log( 3002 );  // log a bad state error
  
}





