//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 27, 2018         	Design Documents                         uart1.c
//
//                    UART1 handling routines
//
// Programmer
//  Jerry Wanger, Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
// 
//
// History
//  03/27/2018 copied from CS7000, STM32F7 for revision to STM32F405
//  04/04/2018 revised to eliminate use of register structures
//  01/15/2019 combined Uart1, Uart2, and uart_revised into this.
//  05/31/2019 made conditional compile for FT857D
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  USART1_Handler                    ISR, which handler used depends on radio
//     -- for CS800D --
//  uart1_CS800D_initialize           Initialize UART1 for CS800D
//  USART1_CS800D_Handler             ISR handler for CS800D
//  uart1_CS800D_receive_interruptB   RX portion of ISR
//  uart1_CS800D_transmit_interrupt   TX portion of ISR
//  uart1_tx_coordinate               handle transmit scheduleing 
//  set_UART1TXRX_to_GPIO             allow GPIO control of pins at startup
//  set_UART1TXRX_to_UART             set GPIO for control by UART1
//
//     -- for FT857D --
//  uart1_FT857D_initialize           Initialize UART1 for FT857D
//  USART1_FT857D_Handler             ISR handler for FT857D
//  uart1_FT857D_transmit_interrupt   TX portion of ISR
//  uart1_FT857D_receive_interrupt    RX portion of ISR
//  uart1_FT857D_rx_state_00          RX state     
//  uart1_FT857D_rx_state_01          RX state
//  uart1_FT857D_rx_state_02          RX state
//  uart1_FT857D_send_0xA5            Sends A5 on UART1
//  uart_FT857D_send_0xA5             Sends A5 on UART1 or UART2
//  FT857D_auto_initialization        radio startup detect and send packets
//
// Status: written, not used
//  uart1_CS800D_receive_interrupt    RX portion of ISR
//  uart1_FT857D_receive_sniffing     debugging, packet sniffing
//==============================================================================
#include <stdint.h>
#include <stdio.h>
#include "CSI_UART.h"
#include "CSI_TIMERS.h"
#include "CSI_GPIO.h"
#include "CSI_REGISTERS.h"
#include "CSI_interrupt.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_SCREENS.h"
#include "CSI_FT857D.h"
#include "CSI_FT857_SCREENS.h"
#include "CSI_RADIO.h"

// *****************************************************************************
// global pointers for USART1 and RCC
// *****************************************************************************
uint32_t volatile *USART1_SR   = (uint32_t *) USART1_SR_ADDRESS;
uint32_t volatile *USART1_DR   = (uint32_t *) USART1_DR_ADDRESS; 
uint32_t volatile *USART1_CR1  = (uint32_t *) USART1_CR1_ADDRESS;    
uint32_t volatile *USART1_CR2  = (uint32_t *) USART1_CR2_ADDRESS; 
uint32_t volatile *USART1_CR3  = (uint32_t *) USART1_CR3_ADDRESS;
uint32_t volatile *USART1_BRR  = (uint32_t *) USART1_BRR_ADDRESS;

uint32_t volatile *RCC_APB2ENR = (uint32_t *) RCC_APB2ENR_ADDRESS;

// *****************************************************************************
// programs local to this module
// *****************************************************************************
void uart1_CS800D_initialize(void);
void uart1_FT857D_initialize(void);

void USART1_Handler(void);
void USART1_FT857D_Handler(void);
void USART1_CS800D_Handler(void);

void uart1_CS800D_receive_interrupt(void);
void uart1_CS800D_transmit_interrupt(void);

#ifdef radio_FT857D
void uart1_FT857D_receive_interrupt(void);
void uart1_FT857D_transmit_interrupt(void);
void uart1_FT857D_rx_state_00(void);
void uart1_FT857D_rx_state_01(void);
void uart1_FT857D_rx_state_02(void);
void uart1_FT857D_send_0xA5(void);
void uart_FT857D_send_0xA5(void);
#endif /* FT857D */

void FT857D_auto_initialization(void);

void uart1_tx_coordinate(void);
void set_UART1TXRX_to_GPIO (void);
void set_UART1TXRX_to_UART (void);
uint8_t get_UART1RX(void);
// *****************************************************************************
// variables defined in other modules
// *****************************************************************************
extern uint8_t current_radio_type;
extern uint32_t FT857D_uart_tx_watch_dog_timer; 
extern uint32_t FT857D_uart_rx_watch_dog_timer; 
// *****************************************************************************
// variales defined in this module
// *****************************************************************************
//
// FT857D variables
//
uint32_t FT857D_uart1_rx_data_ack_character;
uint32_t FT857D_uart1_rx_state;
uint32_t FT857D_uart1_tx_flag = 1;
uint8_t  FT857D_uart1_rx_data;
uint8_t  FT857D_uart1_tx_character_count;
uint8_t  FT857D_uart1_rx_character_count;
uint8_t  FT857D_uart1_rx_checksum;
uint32_t TC_flag;
uint32_t FT857D_auto_init_state;
uint32_t FT857D_auto_init_trigger;
//
// These variables are shared with the FT857D UART2 programs
//
uint8_t  FT857D_tx_buffer_index;
uint8_t  FT857D_tx_buffer[256];
uint8_t  FT857D_rx_buffer_index;
uint8_t  FT857D_rx_buffer[256];
uint32_t FT857D_rx_fifo_output_pointer;
uint32_t FT857D_rx_fifo_input_pointer;
uint8_t  FT857D_rx_fifo[1024];

uint32_t uart1_rx_state;
uint32_t uart1_rx_data;
uint32_t uart1_rx_character_count;
uint32_t uart1_rx_buffer_index;
uint32_t uart1_rx_watch_dog_timer;
uint32_t uart1_rx_watch_dog_flag;
uint32_t uart1_rx_flag;

uint8_t  uart1_tx_character_count;
uint32_t uart1_tx_buffer_index;
uint32_t uart1_tx_flag;

uint8_t uart1_tx_buffer[UART_DATA_BUF_SIZE];
uint8_t uart1_rx_buffer[UART_DATA_BUF_SIZE];

// =============================================================================
//                                                              uart1_initialize
// Initialize USART 1 for use by all radios
//

//  History: 09/24/2019 made general routine for all radios

void uart1_initialize( uint8_t radio_type )
{
  // turn on clock for USART1, RCC_APB2ENR
  *RCC_APB2ENR |=  RCC_APB2ENR_UART1EN;
  
  // Set up CR1 and CR2 but disable UART for now
  *USART1_CR1 = 0x00000024;
  *USART1_CR2 = 0x00000000;
  // set baud rate divider
  *USART1_BRR = get_radio_baud( radio_type );  
  *USART1_SR =  0x00C00000;  // Set SR back to factory default.  

  // Enable the interrupt controller for USART1
  nvic_enableIRQ(37);

  // Enable the UART
  *USART1_CR1 = 0x00002024; 

  // enable idle interrupt
  *USART1_CR1 = *USART1_CR1 | IDLEIE;  
}
/*
// =============================================================================
//                                                       uart1_CS800D_initialize
// Initialize USART 1 for use with CS800 radios
//
void uart1_CS800D_initialize(void)
{
  // turn on clock for USART1, RCC_APB2ENR
  *RCC_APB2ENR |=  RCC_APB2ENR_UART1EN;
  
  // Set up CR1 and CR2 but disable UART for now
  *USART1_CR1 = 0x00000024;
  *USART1_CR2 = 0x00000000;

  // baud rate 57.6K firmware version 20.42 and up 
  // 84 Mhz / (16 * 57,600 ) = 91.1458  91.125 is close
  // set baud rate register to 91.125
  // 0x5B for 91 mantissa 2/16 = 0.125 for fraction
  *USART1_BRR = 0x000005B2;  // Based on a 84 MHz clock
  *USART1_SR =  0x00C00000;  // Set SR back to factory default.  

  // Enable the interrupt controller for USART1
  nvic_enableIRQ(37);

  // Enable the UART
  *USART1_CR1 = 0x00002024; 

  // enable idle interrupt
  *USART1_CR1 = *USART1_CR1 | IDLEIE;  
}
// =============================================================================
//                                                       uart1_FT857D_initialize
// Initialize USART 1 for use with FT857D radios
//
void uart1_FT857D_initialize(void)
{
  // turn on clock for USART1, RCC_APB2ENR
  *RCC_APB2ENR |=  RCC_APB2ENR_UART1EN;

  // Set up CR1 and CR2 but disable UART for now
  *USART1_CR1 = 0x00000024;
  *USART1_CR2 = 0x00000000;

  // baud rate 62.5K firmware version 20.42 and up
  // 84 Mhz/ (16 * 62500) = 84 = 0x54
  // 42 Mhz/ (16 * 62500) = 42 = 0x2A
  // 0x54 for mantissa and 0 for fraction
  *USART1_BRR = 0x00000540;  // Based on a 84 MHz clock
  *USART1_SR =  0x00C00000;  // Set SR back to factory default.  

  // Enable the interrupt controller for USART1
  nvic_enableIRQ(37);
 
  // Enable the UART
  *USART1_CR1 = 0x0000202C;
}
*/
// =============================================================================
// February 23, 2019		Design Documents                  USART1_Handler
//
// This program handles the interrupt for the USART1. This program should 
// override the weak interrupt in the startup program. This part of the program
// just determines which interrupt handler program should be used. At this time
// the choice is between a CS800D or a FT857D
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  current_radio_type determines which interrupt handler should be used. Zero
//  is for the CS800D and 1 is for the FT857D
//
// Exit Conditions
//  This program calls the approriate routine
//
// External functions called
//  void USART1_CS800D_Handler(void)
//  void USART1_FT857D_Handler(void)
//
// External variables used
//  current_radio_type
//
#pragma call_graph_root = "interrupt"
void USART1_Handler(void)
{
  if (current_radio_type)
  {
    #ifdef radio_FT857D    
    USART1_FT857D_Handler();
    #endif /* FT857D */
  }
  else
  {
    USART1_CS800D_Handler();
  }
}
// =============================================================================
// April 20, 2018		Design Documents           USART1_CS800D_Handler
//
// This program handles the interrupt for the USART1. This program should 
// override the weak interrupt in the startup program.
// Read switch on rotary encoder 1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If there is a Receiver interrupt, a Transmit interrupt, or a Transmit
//  Complete interrupt, this program will handle it.
//
// External functions called
//  void uart1_receive_interrupt(void)
//  void uart1_transmit_interrupt(void)
//
// External variables used
//  uart1_tx_flag
//
// Internal variables used
//  uint32_t volatile *USART1_CR1
//  uint32_t volatile *USART1_SR
//  uint32_t status
//
// Constants
//  defined in register_definitions.h
//    USART1_CR1_ADDRESS
//    USART1_SR_ADDRESS
//    BIT05_MASK
//    BIT06_MASK
//    BIT07_MASK
//
// History
//    05/01/2019  added handling of interrupt flags ORE and IDLE
//                to eliminate overrun errors.
//
//    08/09/2019 added clearing of TC flag at end of tx 
//
// Status
//  written, working
// 

uint32_t uart_counter = 0;
// debugging
// char dump_byte[4];

// debugging
// uint8_t in_process_flag = 0;

#pragma call_graph_root = "interrupt"
void USART1_CS800D_Handler(void)
{
  uint32_t status;
 
  uart_counter++;
  // read status
  status = *USART1_SR;
  
  if(status & IDLE )
  {
    // idle behaves the same as an rxne, but it's not data
    // read data to prevent overrun
    uart1_rx_data = *USART1_DR;
    // read status again
    status = *USART1_SR;
    // continue on to other status conditions
  }
  
  // for debugging wd timeouts, check for overrun
  if(status & ORE )
  {
    if( status & RXNE )  // data is still available to be read
    { 
      // read data
      uart1_rx_data = *USART1_DR;
      // read status again
      status = *USART1_SR;      
    }
    error_log( 1701 );
  }

  // Determine the source of the interrupt. If receive interrupt, process
  if (status & RXNE)
  {
    // uart1_CS800D_receive_interrupt();      // Jerry's original
    uart1_CS800D_receive_interruptB();     // dsg revised, working
  }
  
  // Determine the source of the interrupt. If transmit interrupt process
  if( status & TXE )
  {
    uart1_CS800D_transmit_interrupt();
  } 

  // If transmission complete interrupt, turn off the TCIE interrupt and clear
  // the uart1_tx_flag to indicate another packet can be sent.
  if (status & TC)
  {
    *USART1_CR1 = *USART1_CR1 & (~TCIE);
    uart1_tx_flag = 0;
    *USART1_SR = *USART1_SR & (~TC); 
    // log_event( "pkt sent");  // for debug
  }
}
// =============================================================================
// February 23, 2019		Design Documents           USART1_FT857D_Handler
//
// This program handles the interrupt for the USART1 for the CS800D. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  If there is a Receiver interrupt, a Transmit interrupt, or a Transmit
//  Complete interrupt, this program will handle it.
//
// External functions called
//  void uart1_FT857D_receive_interrupt(void)
//  void uart1_FT857D_transmit_interrupt(void)
//
// External variables used
//  uart1_tx_flag
//
// Internal variables used
//  uint32_t status
//

// =========================
#ifdef radio_FT857D

void USART1_FT857D_Handler(void) 
{
uint32_t status;

  // Determine the source of the interrupt. If receive interrupt, process
  status = *USART1_SR;

  if (status & ORE) 
  {
    if( status & RXNE )  // data is still available to be read
    { 
      // read data
      uart1_rx_data = *USART1_DR;
      // read status again
      status = *USART1_SR;      
    }
    error_log( 1701 );
  }
  if (status & RXNE)
  {
      uart1_FT857D_receive_interrupt();
      //uart1_FT857D_receive_sniffing();
  }  

  // Determine the source of the interrupt. If transmit interrupt process
  if (status & TXE)
  {
    uart1_FT857D_transmit_interrupt();
  }
  // idle line at startup
  if (status & IDLE)
  {
    // read data to prevent overrun (idle line should be FF) 
    uart1_rx_data = *USART1_DR;
    // set TX to allow restart
    FT857D_uart_tx_watch_dog_timer = 0;
    FT857D_uart1_tx_flag = 1;
    FT857D_uart1_rx_data_ack_character = 0;    
  } 

  // If transmission complete interrupt, turn off the TCIE interrupt. Because
  // the transmission is now complete, set the uart2_rx_data_ack_character flag to
  // 3 and set the watchdog timer.  We now have to wait for another 0x06 from the
  // radio. If we don't get the acknowledgement within the watchdog time, we have
  // to resend the data.
  if ((status & TC) && TC_flag)
  { 
      *USART1_CR1 = *USART1_CR1 & (~TCIE);
      TC_flag = 0;
      *USART1_SR = *USART1_SR & (~TC);
      FT857D_uart1_rx_data_ack_character = 3;
      FT857D_uart_tx_watch_dog_timer = 100;    
  } 
}
// ===============================
#endif /* FT857D */

//
// *****************************************************************************
//
// Feb 23, 2018			Design Documents  uart1_CS800D_receive_interrupt
//
// This program reads UART1 and puts the data in a buffer. The data only goes in
// a buffer if it meets the proper structure for a message. The structure of the
// buffer is as follows:
//   
// 0xB1 0x00 NN DD...DD K1 K2 0xB4
//
// 0xB1 is the start of the message
// 0x00 is the second character for start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message, NN, or end of message
// K1   is the MSB of checksum
// K2   is the LSB of checksum
// 0xB4 is the end of the message
//
// This program is called when there is a receive interrupt.
//
// This program sets a counter and a flag which is used by an external program
// to act as a watchdog timer in case the program gets passed state 0 but does
// not finish in a reasonable amount of time.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Exit Conditions
//  uart1_rx_buffer[256] is the receive buffer
//  uart1_rx_length is the number of characters in the buffer
//  uart1_rx_flag indicates there is data in the buffer
//
// External functions called
//
// External variables used
//  uart1_rx_state
//  uart1_rx_data
//  uart1_rx_watch_dog_timer
//  uart1_rx_watch_dog_flag
//  uart1_rx_character_count
//  uart1_rx_error_diagnostic
//  uart1_rx_buffer_index
//  uart1_rx_buffer[256]
//
// Internal variables used
//  uint32_t volatile *USART1_DR   
//
// Constants
//  defined in register_definitions.h
//    USART1_DR_ADDRESS
//
// History
//     08/16/2018 added checksum
//
// Status
//  written, debugging watchdog timeouts. Not in use, revision written
//
uint16_t uart1_checksum = 0;
uint8_t  uart1_good_data = 0;

void uart1_CS800D_receive_interrupt(void)
{
//
// If we got here we got a receiver interrupt. Read the data register.
//
  uart1_rx_data = *USART1_DR;
//
// If the uart1_rx_watch_dog_flag has a value of 1 and uart1_rx_watch_dog_timer
// is zero, then we had a watchdog event. It means it was not able to get a
// complete packet within 100 milliseconds. Nornmal time is 54 milliseconds 
// worst case
//
  if (uart1_rx_watch_dog_flag & (!uart1_rx_watch_dog_timer))
  {
    uart1_rx_state = 0;
    error_log(1703);
  }
//
// *****************************************************************************
// State 0 is waiting for the start of message. Stay in this state until we
// get a 0xB1 character.
// *****************************************************************************
//
  if (uart1_rx_state == 0) 
  {
    if (uart1_rx_data == 0xB1)
    {
      uart1_rx_buffer[0] = uart1_rx_data;
      uart1_rx_watch_dog_timer = 100;
      uart1_rx_watch_dog_flag = 1;
      uart1_checksum = 0;
      uart1_good_data = 0;
      uart1_rx_state++;
    }
    return;
  }
//
// *****************************************************************************
// State 1 is waiting for a 0x00. If the character is not 00, we set the
// the error message to 1. I am not sure what this character is actually 
// there for but it is part of the protocol from the radio.
// *****************************************************************************
//
  if (uart1_rx_state == 1)
  {
    if (uart1_rx_data != 0x00)
    {
      uart1_rx_state = 0;
    }
    else 
    {
      uart1_rx_buffer[1] = uart1_rx_data;
      uart1_rx_state++;
    }
    return;
  }
//
// *****************************************************************************
// State 2 is waiting for the number of characters in the message
// *****************************************************************************
//
  if (uart1_rx_state == 2)
  {
    uart1_rx_character_count = uart1_rx_data + 3;
    uart1_rx_buffer[2] = uart1_rx_data;
    uart1_rx_buffer_index = 3; 
    uart1_rx_state++;
    return;    
  }
//
// *****************************************************************************
// State 3 is waiting for the message. When the number of characters received
// is equal to the character count, advance to the next state. Note that
// we do not currently do a check sum at this point. If the next character is
// a 0xB4, then the rest of the data is probably OK.
// *****************************************************************************
//
  if (uart1_rx_state == 3)
  {
    if(uart1_rx_buffer_index < uart1_rx_character_count - 2)
                 uart1_checksum += uart1_rx_data;
    uart1_rx_buffer[uart1_rx_buffer_index++] = uart1_rx_data;

    if (uart1_rx_buffer_index == uart1_rx_character_count)
    {
      uart1_rx_state++;
    }
    return;
  }
//
// *****************************************************************************
// State 4 is waiting for the end of message. If the character is not the end
// of the message, assume it is and just give show the last character is a
// 0xB5 instead of the normal 0xB4
// *****************************************************************************
//
  if (uart1_rx_state == 4)
  {
    if (uart1_rx_data != 0xB4)
    {
      uart1_rx_buffer[uart1_rx_buffer_index] = 0xB5;
    }
    else
    {
      uart1_rx_buffer[uart1_rx_buffer_index] = uart1_rx_data;
    }
    uart1_rx_flag = 1;
    uart1_rx_watch_dog_flag = 0;
    if(uart1_checksum == uart1_rx_buffer[uart1_rx_character_count - 1] +
                         (uart1_rx_buffer[uart1_rx_character_count - 2] << 8))
              uart1_good_data = 1;
    else uart1_good_data = 0;
                           
    uart1_rx_state = 0;
    return;
  }
//
// If we got here then there is an error in the program. The uart1_rx_state is
// greater than 4. Set the state back to state 0
//
  error_log( 3002 );
  uart1_rx_state = 0;
}

// =============================================================================
//
//   revised receive
//
// 04/22/2019
// revised with a switch instead of if statements and multiple returns
// 05/01/2019
// added setting watchdog flag to 0 each time rx_state is reset to 0
// added error logs

// debugging data out of packet
uint8_t bad_data_buffer[256];
uint8_t bad_data_index = 0;

int uart1_rx_buffer_max = 0;

void uart1_CS800D_receive_interruptB(void)
{
  // We have a receiver interrupt. Read the data register.
  uart1_rx_data = *USART1_DR;
  // If the uart1_rx_watch_dog_flag has a value of 1 and uart1_rx_watch_dog_timer
  // is zero, then we had a watchdog event. It means it was not able to get a
  // complete packet within 100 milliseconds. Nornmal time is 54 milliseconds 
  // worst case
  if (uart1_rx_watch_dog_flag & (!uart1_rx_watch_dog_timer))
  {
    uart1_rx_state = 0;
    uart1_rx_watch_dog_flag = 0;    
    // for debugging
    error_log( 1703 );
  }

  switch( uart1_rx_state )
  {
  // State 0 is waiting for the start of message. Stay in this state until we
  // get a 0xB1 character.
  case 0:
    {    
      if( uart1_rx_data == 0xB1)
      {
        uart1_rx_buffer[0] = uart1_rx_data;
        // uart1_rx_watch_dog_timer = 200;   // tested longer
        uart1_rx_watch_dog_timer = 100;
        uart1_rx_watch_dog_flag = 1;
        uart1_checksum = 0;
        uart1_good_data = 0;
        uart1_rx_state++;
        // a good start so reset bad data 
        if( bad_data_index > 0 )
        {
          // break here to view data
          bad_data_index = 0;
        }
      }
      else 
      {
        bad_data_buffer[bad_data_index++] = uart1_rx_data;        
        // error_log( 1704 );
      }
      break;
    }
  // State 1 is waiting for a 0x00. If the character is not 00, we set the
  // the error message to 1. I am not sure what this character is actually 
  // there for but it is part of the protocol from the radio.    
  case 1:
    {
      if (uart1_rx_data != 0x00)
      {
        uart1_rx_state = 0;
        uart1_rx_watch_dog_flag = 0;
      }
      else 
      {
        uart1_rx_buffer[1] = uart1_rx_data;
        uart1_rx_state++;
      }
      break;
    }
  // State 2 is waiting for the number of characters in the message
  case 2:
  {
    uart1_rx_character_count = uart1_rx_data + 3;
    uart1_rx_buffer[2] = uart1_rx_data;
    uart1_rx_buffer_index = 3; 
    uart1_rx_state++;
    break;    
  }

  // State 3 is waiting for the message. When the number of characters received
  // is equal to the character count, advance to the next state. Checksum. 
  case 3:
  {
    if(uart1_rx_buffer_index < uart1_rx_character_count - 2)
                 uart1_checksum += uart1_rx_data;
    uart1_rx_buffer[uart1_rx_buffer_index++] = uart1_rx_data;

    if (uart1_rx_buffer_index == uart1_rx_character_count)
    {
      uart1_rx_state++;
    }
    break;
  }
  // State 4 is waiting for the end of message. If the character is not the end
  // of the message, assume it is and just give show the last character is a
  // 0xB5 instead of the normal 0xB4
  case 4:
  {
    if (uart1_rx_data != 0xB4)
    {
      uart1_rx_buffer[uart1_rx_buffer_index] = 0xB5;
    }
    else
    {
      uart1_rx_buffer[uart1_rx_buffer_index] = uart1_rx_data;
    }
    uart1_rx_flag = 1;
    uart1_rx_watch_dog_flag = 0;
    if(uart1_checksum == uart1_rx_buffer[uart1_rx_character_count - 1] +
                         (uart1_rx_buffer[uart1_rx_character_count - 2] << 8))
              uart1_good_data = 1;
    else uart1_good_data = 0;
                           
    uart1_rx_state = 0;
    break;
  }
  // If we got here then there is an error in the program.
  // Set the state back to state 0
  default:
    {
    uart1_rx_state = 0;
    uart1_rx_watch_dog_flag = 0;    
    error_log( 3002 );
    break;
    }
  }  // end of switch
}
// =============================================================================
//
//   Receive to rx buffer only for packet sniffing
//    Stores one full buffer of all data received
//    Only stores one 0x06 for watchdog, skips the rest
//    No packet handling done, mostly just for startup analysis
//
uint8_t ps_buffer_index = 0;

void uart1_FT857D_receive_sniffing(void)
{
  static uint8_t prev_data = 0;
  // Read the data register.
  uart1_rx_data = *USART1_DR;
  // only let the first 06 through, eliminate lots of repeats
  if( !((uart1_rx_data == 0x06 ) && (prev_data == 0x06 )) )
  {
    // store in the buffer
    uart1_rx_buffer[ps_buffer_index++] = uart1_rx_data;
  }
  prev_data = uart1_rx_data; 
  // buffer is 256 so index of uint8 will wrap
  // stop at a full buffer by rolling back one
  if( ps_buffer_index == 255 ) ps_buffer_index = 254;
  
}

// 
//
// *****************************************************************************
//
// Feb 23, 2018		Design Documents         uart1_CS800D_transmit_interrupt
//
// This program writes data from a buffer to the UART1. The structure of the
// buffer is as follows:
//   
// 0xB1 0x00 NN DD...DD K1 K2 0xB4
//
// 0xB1 is the start of the message
// 0x00 is the second character for start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message, NN, or end of message
// K1   is the MSB of checksum
// K2   is the LSB of checksum
// 0xB4 is the end of the message
//
// This program is called when there is a transmit interrupt.
//
// This program sets a flag when all the data in the buffer is transmitted
//
// The programn works as follows:
//   When the character count is greater than 1, the data is outputed which will
//   clear the interrupt.
//
//   When the character count is equal to 1, the TXEIE is disabled and the TCIE
//   is enabled. At the end of this routine the character count will be zero.
//
//   When get a TC interrupt, turn off the TCIE interrupt and set the 
//   uart1_tx_flag to indicate another transmission can be started. This is done
//   in the calling program called USART1_Handler 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  uart1_tx_buffer[256] is the transmit buffer
//  uart1_tx_character_count is the number of characters in the buffer
//  uart1_tx_buffer_index is the index into the tx buffer
//
// Exit Conditions
//  uart1_tx_flag indicates all the data is transmitted
//
// External functions called
//
// External variables used
//  uart1_tx_character_count
//  uart1_tx_buffer[256]
//
// Internal variables used  
//  uint32_t volatile *USART1_DR
//  uint32_t volatile *USART1_CR1
//
// Constants
//  defined in register_definitions.h
//    USART1_DR_ADDRESS; 
//    USART1_CR1_ADDRESS
//    BIT06_MASK
//    BIT07_MASK
//
// History 
//
// Status
//  written
//
void uart1_CS800D_transmit_interrupt(void)
{
//
// If buffer empty, just exit from routine. It means we came here from TC
// interrupt
//
  if (uart1_tx_character_count)
  {

    // Turn off buffer empty interrupt and turn on buffer complete interrupt
    if (uart1_tx_character_count == 1)
    {
      *USART1_CR1 = *USART1_CR1 & (~TXEIE); 
      *USART1_CR1 = *USART1_CR1 | TCIE;
    }

    // Output data in buffer and decrement character count
    *USART1_DR = uart1_tx_buffer[uart1_tx_buffer_index++];
    uart1_tx_character_count--;
  }
}
// =============================================================================
//
//  Function to check rx state
//   added to keep rx state variables local
//
uint8_t FT857D_uart1_isready( void )
{
  uint8_t ready = 0;
  if( ( FT857D_uart1_rx_state == 0 ) && ( FT857D_uart1_tx_flag == 1 )) ready = 1;
  return ready;
}

//
// *****************************************************************************
// December 31, 2018		Design Documents  uart1_FT857D_receive_interrupt
//
// This program reads data from the UART and puts it in a buffer. The structure
// of the incoming data is as follows:
//   
// 0xA5 NN DD...DD CS
//
// 0xA5 is the start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message and NN.
// CS   is the checksum
//
// When the A5 is received, the panel sends back an 0x06 right away. After the 
// check sum is received, the panel sends back an 0x06 right away.
//
// This program puts data first into a 256 byte buffer. If the checksum is good
// at the end, it transfers the data to a 1024 byte fifo. The purpose is because
// the radio sends out data faster than the panel could put it on the display.
// However, after it sends a few packets, it then waits a significant amount of
// time for the panel to put the data on the display.
// *****************************************************************************

#ifdef radio_FT857D

void uart1_FT857D_receive_interrupt(void)
{
  // We have a receiver interrupt. Read the data register
  FT857D_uart1_rx_data = *USART1_DR;
  FT857D_auto_init_timer = 1500; // 1.5 seconds timeout

 switch(FT857D_uart1_rx_state)
 {
    case 0:
      uart1_FT857D_rx_state_00();
      break;
    case 1:
      uart1_FT857D_rx_state_01();
      break; 
    case 2:
      uart1_FT857D_rx_state_02();
      break;
    default:
      FT857D_uart1_rx_state = 0;
      error_log( 3002 );
      break;
  }
}
#endif /* FT857D */
//
// *****************************************************************************
// December 31, 2018		Design Documents        uart1_FT857D_rx_state_00
//
// State 0 is waiting for either a periodic handshake or the start of a packet.
// If its a periodic handshake, the radio sends a 0x90 and the panel responds
// with a 0x06. If its a packet, the radio sends a 0xA5 and the panel responds
// with a 0x06 and then goes to the next state to receive the rest of the packet
// This state is also wating for a flag to indicate that anther program just
// sent a command to the radio and is expecting a handshake back from the radio.
// History
//  07/09/2019 added holdoff for timer (FT857D_uart_tx_watch_dog_timer)
//             to prevent radio rebooting after reply to packet start

// *****************************************************************************
void uart1_FT857D_rx_state_00(void)
{
  // if periodic handshake, send 0x06 and return.
  if (FT857D_uart1_rx_data == 0x90)
  {
    *USART1_DR = 0x06;
    // log_event( "handshake" );    
    return;
  }

  // If first part of a packet, send 0x06, set flag to indicate trying to receive
  // a packet and advance to the next state and return
  if (FT857D_uart1_rx_data == 0xA5)
  {
    if( !FT857D_uart_tx_watch_dog_timer )
    {
      *USART1_DR = 0x06;
      FT857D_uart1_rx_state++;
      //log_event( "A5 rx start" ); 
    }
    return;
  }

  // If it got an 0x06 it is because the panel originated a packet and it is
  // waiting for an ecknowledgement from the radio. 
  if (FT857D_uart1_rx_data == 0x06)
  {

  // If uart2_rx_data_ack_character is 1, then its the begining of the buffer 
  // being sent. Setup the transmission to send the rest of the packet. Because
  // the 0xA5 has already been sent, the buffer empty flag should be enabled so
  // we should immediately go to the transmitter interrupt upon exit from this
  // routine.We disable the watchdog timer because we are not waiting for any
  // outside devices to send an acknowledgement.
  if (FT857D_uart1_rx_data_ack_character == 1)
    {
      FT857D_uart1_tx_character_count = FT857D_tx_buffer[0]+1; // get char count 
      FT857D_tx_buffer_index = 0;               // point to char count 
      *USART1_CR1 = *USART1_CR1 | TXEIE;        // enable buffer empty int 
      FT857D_uart1_rx_data_ack_character = 2;   // check for error
      FT857D_uart_tx_watch_dog_timer = 0;       // disable watch dog
      return;
    }

    // If we get an 0x06 at this time, there is an error. We need to send the 
    // complete packet before we can expect an 0x06 character. If this happens we
    // should set a flag and wait for the transmission complete flag before we
    // resend the packet.
    if (FT857D_uart1_rx_data_ack_character == 2)
    {
      error_log( 1705 );
      return;
    }

    // At this point we got an acknowledgemet and we can now set the uart2_tx_flag 
    // flag to indicate we can send another packet. Clear out watch dog timer so 
    // it will not interfer with the next packet.
    if (FT857D_uart1_rx_data_ack_character == 3)
    {
      FT857D_uart_tx_watch_dog_timer = 0;
      FT857D_uart1_tx_flag = 1;
      FT857D_uart1_rx_data_ack_character = 0;    
    }
    return;
  }

  // If we get a 0x15 at this point, it is because the radio got the wrong
  // checksum. Make believe it went through and clear out the approriate 
  // variables.
  if (FT857D_uart1_rx_data == 0x15) 
  {
    FT857D_uart_tx_watch_dog_timer = 0;
    FT857D_uart1_tx_flag = 1;
    FT857D_uart1_rx_data_ack_character = 0; 
    error_log( 3400 );
    return;
  }
  // If its any other character just ignore and return
  // log anything other than 0, or 0xFF
  if( (FT857D_uart1_rx_data != 0xFF) && (FT857D_uart1_rx_data != 0) )
     error_log( 1705 );
}
//
// *****************************************************************************
// December 31, 2018		Design Documents        uart1_FT857D_rx_state_01
//
// State 1 is waiting for the character count of the packet. The first character
// that comes in after the A5 is the number of characters in the rest of the 
// packet including the checksum
// *****************************************************************************
void uart1_FT857D_rx_state_01(void)
{
   FT857D_rx_buffer[0] = 0xA5;
   FT857D_rx_buffer[1] = FT857D_uart1_rx_data;
   FT857D_uart1_rx_character_count = FT857D_uart1_rx_data;
   FT857D_uart1_rx_checksum = 0;
   FT857D_rx_buffer_index = 2;   
   FT857D_uart1_rx_state++;
}
//
// *****************************************************************************
// December 31, 2018		Design Documents        uart1_FT857D_rx_state_02
//
// State 2 is waiting for the rest of the packet. When it gets the checksum and
// its correct it sends back an 0x06. If the checksum is wrong it sends back an
// 0x07 (Need to confirm about the 0x07)
// *****************************************************************************
void uart1_FT857D_rx_state_02(void)
{
  uint32_t i;
  
  FT857D_rx_buffer[FT857D_rx_buffer_index++] = FT857D_uart1_rx_data;  
  if (--FT857D_uart1_rx_character_count)
  {
    FT857D_uart1_rx_checksum = FT857D_uart1_rx_checksum + FT857D_uart1_rx_data;
    return;
  }
  else
  {
    FT857D_uart1_rx_state = 0;

    // If data is good, transfer to fifo. If data is bad, ignore data. 
    if (FT857D_uart1_rx_checksum == FT857D_uart1_rx_data)
    {
      *USART1_DR = 0x06;         // acknowledge good data
      
      // log_event( "rx state 2 good" );
      
      for (i = 0; i < FT857D_rx_buffer[1]+2; i++)
      {
        FT857D_rx_fifo[FT857D_rx_fifo_input_pointer++] =  FT857D_rx_buffer[i];
        if (FT857D_rx_fifo_input_pointer > 1023) FT857D_rx_fifo_input_pointer = 0;
      }
      FT857D_uart_rx_watch_dog_timer = 100;  // 10 ms timeout
    }
    else error_log( 3400 );
  }
}
//
// *****************************************************************************
// December 31, 2018	    Design Documents     uart1_FT857D_transmit_interrupt
//
// This program writes data from a buffer to the UART2. The structure of the
// buffer is as follows:
//   
// 0xA5 NN DD...DD CS
//
// 0xA5 is the start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message and NN.
// CS   is the checksum
//
// When the A5 is sent, the radio sends back an 0x06 right away. After the 
// check sum is sent, the radio sends back an 0x06 right away.
//
// Another program sends out the A5 and sets up this interrupt routine.
//
// This program is called when there is a transmit interrupt.
//
// This program sets a flag when all the data in the buffer is transmitted
//
// The programn works as follows:
//   When the character count is greater than 1, the data is outputed which will
//   clear the interrupt.
//
//   When the character count is equal to 1, the TXEIE is disabled and the TCIE
//   is enabled. At the end of this routine the character count will be zero.
//
//   When get a TC interrupt, turn off the TCIE interrupt and set the 
//   uart1_tx_flag to indicate another transmission can be started. This is done
//   in the calling program called USART2_Handler 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  uart2_tx_buffer[256] is the transmit buffer
//  uart2_tx_character_count is the number of characters in the buffer
//  uart2_tx_buffer_index is the index into the tx buffer
//  uart2_tx_mode is 0 for a single character transmit and 1 otherwise
//
// Exit Conditions
//  uart2_tx_flag indicates all the data is transmitted
//
// External functions called
//
// External variables used
//  uart2_tx_character_count
//  uart2_tx_mode
//  uart2_tx_buffer[256]
//
// Internal variables used  
//  uint32_t volatile *USART1_DR
//  uint32_t volatile *USART1_CR1
//
// Constants
//  defined in register_definitions.h
//    USART2_DR_ADDRESS; 
//    USART2_CR1_ADDRESS
//    BIT06_MASK
//    BIT07_MASK
//
// History 
//
// Status
//  written
// *****************************************************************************
void uart1_FT857D_transmit_interrupt(void)
{
  // Process multi-byte transmissions. If buffer empty, just exit from routine. 
  // It means we came here from TC interrupt
  if (FT857D_uart1_tx_character_count)
  {
    // Turn off buffer empty interrupt and turn on buffer complete interrupt. Set
    // the TC_flag so the interrupt program knows it should use it.
    if (FT857D_uart1_tx_character_count == 1)
    {
      *USART1_CR1 = *USART1_CR1 & (~TXEIE); // buffer interrupt      
      *USART1_CR1 = *USART1_CR1 | TCIE;
      TC_flag = 1;      
      FT857D_uart_tx_watch_dog_timer = 100;
      //log_event( "tx done" );      
    }

    // Output data in buffer and decrement character count
    *USART1_DR = FT857D_tx_buffer[FT857D_tx_buffer_index++];    
    FT857D_uart1_tx_character_count--;
  }
}
// *****************************************************************************
// setup transmitter and send first character             uart1_FT857D_send_0xA5
void  uart1_FT857D_send_0xA5(void)
{
  // Reset a bunch of variables to allow everything to work.
  FT857D_uart1_rx_data_ack_character = 1;
  FT857D_uart1_tx_flag = 0;
  FT857D_uart1_rx_state = 0;
  *USART1_DR = 0xA5;
  FT857D_uart_tx_watch_dog_timer = 100;
}
// *****************************************************************************
// setup appropriate data to uart                          uart_FT857D_send_0xA5
void  uart_FT857D_send_0xA5(void)
{
  #ifdef radio_FT857D
  if (current_radio_type == 0)
  {
    uart2_FT857D_send_0xA5();
  }
  #endif /* FT857D */  
  if (current_radio_type == 1)
  {
    uart1_FT857D_send_0xA5();
  }
  #ifdef radio_FT857D
  if (current_radio_type == 2)
  {
    uart2_FT857D_send_0xA5();
  }               
  #endif /* FT857D */  
  
}

// ****************************************************************************
// April 23, 2018		Design Documents             uart1_tx_coordinate
//
// This program takes two sources of data for transmitting back to the main
// radio and coordinates them so there is no conflict.
//
// Source 1 is used for data from the microphone keys, ptt, and various switches
// on the keypad. 
//   key_buffer_flag = 1 means there is data to be sent to radio body
//   key_buffer[] contains the data
//   length of buffer is key_buffer[2] + 3
//
// Source 2 is used for acknowledgements from radio
//   ack_buffer_flag indicates there is data to be sent
//   ack_buffer[] contains the data
//   length of buffer is ack_buffer[2] + 3
//
// If UART is busy sending data, then  uart1_tx_flag = 1
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  If the UART1 is not transmitting and there is a source of data to be sent,
//  then this program sets up the transmitter 
//
// External variables used
//  uart1_tx_flag
//  ack_buffer_flag
//  ack_buffer_delay_flag
//  acknowledgement_delay
//  uart1_tx_packet_length
//  ack_buffer[32]
//  uart1_tx_buffer[256]
//  key_buffer[11]
//  key_buffer_flag
//
// Internal variables used
//  uint32_t volatile *USART1_DR
//  uint32_t uart1_tx_packet_length
//  uint32_t i
//
// Constants   
//
// History
//       11/06/2018  revised, combined transmit start sections 
//
// Status
//  written
//
//

void uart1_tx_coordinate(void)
{
  uint32_t uart1_tx_packet_length;
  static uint8_t ack_buffer_delay_flag = 0;
  uint32_t i;
  // If the flag is true, then the radio is currently using the UART.   
  if (uart1_tx_flag) return;
  // return if no flag
  if(!ack_buffer_flag && !key_buffer_flag) return;

  // If we got to this point, the transmitter buffer is empty. If the 
  // ack_buffer_flag is set, then there is new data to transmit and we have to 
  // set up the buffer. However, before the buffer is set up, we have to delay 
  // about 15 milliseconds to be compatible with the existing radio.
  // modified to 3 msec for new firmware
  if (ack_buffer_flag)
  {
    if (!ack_buffer_delay_flag)
    {
      acknowledgement_delay = 1;
      ack_buffer_delay_flag = 1;
    }
    if (acknowledgement_delay) return;
    ack_buffer_delay_flag = 0;
    // Transfer the data from the ack_buffer to the uart1_tx_buffer. Clear the
    // ack_buffer_flag to indicate the ack_buffer is now available to get more
    // data.
    uart1_tx_packet_length = ack_buffer[2] + 4;
    for (i = 0; i < uart1_tx_packet_length; i++)
    {
      uart1_tx_buffer[i] = ack_buffer[i];
    }
    ack_buffer_flag = 0;
  }
  // If we got here and the key_buffer_flag is non zero, then there is a key that
  // has to be sent back to the radio.
  else if (key_buffer_flag)
  {
    // Transfer the data from the key_buffer to the uart1_tx_buffer. Clear the
    // key_buffer_flag to indicate the key_buffer is now available to get more
    // data.
    uart1_tx_packet_length = key_buffer[2] + 4;
    for (i = 0; i < uart1_tx_packet_length; i++)
    {
      uart1_tx_buffer[i] = key_buffer[i];
    }
    key_buffer_flag = 0;
  }
  // start sending
  uart1_tx_flag = 1;
  uart1_tx_character_count = --uart1_tx_packet_length;
  uart1_tx_buffer_index = 1;
  // Turn on the UART, output the first character to UART1 and enable the
  // interrupt for the UART1.
  *USART1_CR1 = *USART1_CR1 | TE;
  *USART1_DR = uart1_tx_buffer[0];
  *USART1_CR1 = *USART1_CR1 | (TE | TXEIE);
}

// =============================================================================
//
//  Set Uart1 lines tx and RX to GPIO
//
//  for use during radio handshake at startup
//
//  UART1 TX is on GPIO B6
//  UART1 RX is on GPIO A10
//
//  History
//    10/23/2018 written 


void set_UART1TXRX_to_GPIO ( void )
{
  // clear ALT mode A10
  //ALTLO 700 -> 000
  *GPIO_A_ALTHI &= ~0x00000F00;
  // B6 ALTHI 7000000 -> 0000000
  *GPIO_B_ALTLO &= ~0x0F000000;  
  
  // clear mode bits for input
  *GPIO_A_MODER &= ~0x00300000;
  // set mode bits to 01 for output  
  *GPIO_A_MODER |= ~0x00100000;
  
  // clear before setting
  *GPIO_B_MODER &= ~0x00003000;
  // set mode bits to 01 for output
  *GPIO_B_MODER |=  0x00001000;    
}

// =============================================================================
//
//  Set Uart1 lines tx and rx to use the uart
//
//  History
//    10/23/2018 written
void set_UART1TXRX_to_UART ( void )
{
  // set ALT mode A10 to 7
  //ALTLO 0000-> 700
  *GPIO_A_ALTHI &= ~0x00000F00;   
  *GPIO_A_ALTHI |=  0x00000700;   
    
  // B6 ALTLO 0000000 -> 7000000
  *GPIO_B_ALTLO &= ~0x0F000000;   
  *GPIO_B_ALTLO |=  0x07000000;   
  
  // set mode bits for ALT
  // clear first
  *GPIO_A_MODER &= ~0x00300000;  
  // set
  *GPIO_A_MODER |=  0x00200000;   
  
  // clear before setting
  *GPIO_B_MODER &= ~0x00003000;
  // set mode bits to 02 for ALT
  *GPIO_B_MODER |=  0x00002000;     
}

// =============================================================================
//
//  get UART1 RX line
//    for response to radio handshake at startup
//
//  History
//    10/23/2018 written
uint8_t get_UART1RX( void )
{
  if(*GPIO_A_IDR & BIT10_MASK ) return 1;  
  else  return 0;  
}
//
//==============================================================================
// March 7, 2019                                      FT857D_auto_initialization
//
// This program must be called from the 10 millisecond interrupt program. This
// program monitors the FT857D_auto_init_timer variable which gets reset to 1.5
// second everytime a character is received. If the character is not received
// within that time period, it advances to the next state and waits for the 
// radio to generate a character. When it does it then starts the initialization
// sequence.
//
//   This will run with the FT857 connected to either UART1 or UART2   
//

extern uint32_t FT857D_startup_timer;
uint8_t FT857D_volume_rampup = 30;

void FT857D_auto_initialization(void)
{
  // If we got here we need to initialize the radio. We are waiting for a receive
  // character from the radio and when we receive something we advance to the
  // next state.
  switch( FT857D_auto_init_state )
  {
  case 0:
  {
    if (FT857D_auto_init_trigger)
    {    
      FT857D_auto_init_trigger = 0;
      if (FT857D_uart1_tx_flag)
      {
        if( FT857D_rfsql_mode == 1 ) 
        {        
          FT857D_tx_buffer[0] = 0x03;    // packet 1
          FT857D_tx_buffer[1] = 0x96;    
          FT857D_tx_buffer[2] = 0x02;
          FT857D_tx_buffer[3] = 0x98;
          uart_FT857D_send_0xA5();
        }
      }
      FT857D_auto_init_state++;
    }
    break;
  }
  case 1:
  {
    if (FT857D_uart1_tx_flag)
    {
      FT857D_tx_buffer[0] = 0x03;    // PACKET 2    
      FT857D_tx_buffer[1] = 0x99;
      FT857D_tx_buffer[2] = 0x01;
      FT857D_tx_buffer[3] = 0x9A;
      uart_FT857D_send_0xA5();  
      FT857D_auto_init_state++; 
    }
    break;
  }
  case 2:
  {
    if (FT857D_uart1_tx_flag)
    {
      // button packet
      FT857D_tx_buffer[0] = 0x06;    // packet 3
      FT857D_tx_buffer[1] = 0x9A;
      FT857D_tx_buffer[2] = 0xFF;
      FT857D_tx_buffer[3] = 0x8F;
      FT857D_tx_buffer[4] = 0x18;
      FT857D_tx_buffer[5] = 0x00;
      FT857D_tx_buffer[6] = 0x40; 
      uart_FT857D_send_0xA5();
      FT857D_auto_init_state++;
    }
    break;
  }
  case 3:
  {
    if (FT857D_uart1_tx_flag)
    {
      // button packet 2
      FT857D_tx_buffer[0] = 0x06;    // packet 4
      FT857D_tx_buffer[1] = 0x91;
      FT857D_tx_buffer[2] = 0xFF;
      FT857D_tx_buffer[3] = 0x8F;
      FT857D_tx_buffer[4] = 0x18;
      FT857D_tx_buffer[5] = 0x00;
      FT857D_tx_buffer[6] = 0x37; 
      uart_FT857D_send_0xA5();
      FT857D_auto_init_state++; 
    }
    break;
  }
  case 4:
  {
    if (FT857D_uart1_tx_flag)
    {
      // volume knob packet
      FT857D_tx_buffer[0] = 0x03;    // PACKET 5    
      FT857D_tx_buffer[1] = 0x97;
      // use volume ramp      
      if( FT857_volume_value > FT857D_volume_rampup )
      {
        FT857D_tx_buffer[2] = FT857D_volume_rampup;
        if( FT857D_volume_rampup < 205 ) FT857D_volume_rampup += 50;
      }
      else FT857D_tx_buffer[2] = FT857_volume_value;
      
      FT857D_tx_buffer[3] = 0x97 + FT857D_tx_buffer[2]; 
      uart_FT857D_send_0xA5();
      FT857D_auto_init_state++; 
    }
    break;
  }

  case 5:
  {
    if (FT857D_uart1_tx_flag && ( FT857D_rfsql_mode > 0) )
    {
      if( FT857D_rfsql_mode == 1 ) 
      {
        // knob packet for RF gain mode
        FT857D_tx_buffer[0] = 0x03;    // PACKET 6    
        FT857D_tx_buffer[1] = 0x96;
        FT857D_tx_buffer[2] = 0x02;    // need to ramp?
        FT857D_tx_buffer[3] = 0x98;
      }
      if( FT857D_rfsql_mode == 2 ) 
      {
        // squelch knob packet
        FT857D_tx_buffer[0] = 0x03;       
        FT857D_tx_buffer[1] = 0x95;
        FT857D_tx_buffer[2] = 0x00;    
        FT857D_tx_buffer[3] = 0x95;
      }
      
      uart_FT857D_send_0xA5();    
      FT857D_auto_init_state++;
    }
    break;
  }
  case 6:
  {
    if (FT857D_uart1_tx_flag)
    {
      FT857D_tx_buffer[0] = 0x03;    // PACKET 7    
      FT857D_tx_buffer[1] = 0x98;
      FT857D_tx_buffer[2] = 0x0C;
      FT857D_tx_buffer[3] = 0xA4;      
      uart_FT857D_send_0xA5();     
      FT857D_auto_init_state++;
  
    }
    break;
  }
  case 7:
  {
    if (FT857D_uart1_tx_flag)
    {
      FT857D_tx_buffer[0] = 0x03;    
      FT857D_tx_buffer[1] = 0x97;      
      // ramping volume
      if( FT857_volume_value > FT857D_volume_rampup )
      {
        FT857D_tx_buffer[2] = FT857D_volume_rampup;
        if( FT857D_volume_rampup < 205 ) FT857D_volume_rampup += 50;
      }
      else
      {
        FT857D_tx_buffer[2] = FT857_volume_value;
        FT857D_auto_init_state++;   // proceed to next state when done
      }
      
      FT857D_tx_buffer[3] = 0x97 + FT857D_tx_buffer[2]; 
      uart_FT857D_send_0xA5();
      FT857D_startup_timer = 3;  // timer sets squelch at 2
    }
    break;
  }
  case 8:
  {
    // If we got to this state the radio is on. Watch for it to go off
    if (!FT857D_auto_init_timer)
    {
      FT857D_auto_init_state = 0;
      FT857_clear_squelch_graph();  
      FT857_clear_radio_screen();      
    }
    break;
  }
  default:
    {
      // Should never get here
      FT857D_auto_init_state = 0;
      error_log( 3001 );
      break;
    }
  }
}
