//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Jan 15, 2019         	Design Documents                         uart2.c
//
//                    UART2 handling routines
//
// Programmer
//  Jerry Wanger, Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
// 
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
// USART2_Handler                USART 2 Interrupt Handler
// uart2_receive_interrupt       uart2 receive interrupt routine
// uart2_rx_state_00             uart2 waiting for first character of packet
// uart2_rx_state_01             uart2 waiting for character count of packet
// uart2_rx_state_02             uart2 waiting for rest of characters
// uart2_transmit_interrupt      uart2 transmit interrupt routine
// uart2_send_uart2_tx_data      uart2 transmit a single byte without interrupt
// uart2_FT857D_send_0xA5        send 0xA5 to radio
// uart2_FT857D_initialize       initialize uart2 for FT857
//==============================================================================
#include <stdint.h>
#include "CSI_UART.h"
#include "CSI_REGISTERS.h"
#include "CSI_interrupt.h"
#include "CSI_SWITCHES.h"
#include "CSI_ERROR_LOG.h"

// make global pointers for USART2
uint32_t volatile *USART2_SR  = (uint32_t *) USART2_SR_ADDRESS;
uint32_t volatile *USART2_DR  = (uint32_t *) USART2_DR_ADDRESS;
uint32_t volatile *USART2_CR1 = (uint32_t *) USART2_CR1_ADDRESS;
uint32_t volatile *USART2_CR2 = (uint32_t *) USART2_CR2_ADDRESS;
uint32_t volatile *USART2_CR3 = (uint32_t *) USART2_CR3_ADDRESS;
uint32_t volatile *USART2_BRR = (uint32_t *) USART2_BRR_ADDRESS;

// Local only programs
void uart2_receive_interrupt(void);
void uart2_rx_state_00(void);
void uart2_rx_state_01(void);
void uart2_rx_state_02(void);
void uart2_transmit_interrupt(void);
void uart2_enable_tx_te(void);
void send_uart2_tx_data(uint8_t);

// Programs that might be called by an external program
void uart2_FT857D_send_0xA5(void);
void uart2_initialize(void);


uint32_t uart2_rx_state;
uint32_t uart2_rx_data;
uint32_t uart2_rx_character_count;
uint32_t uart2_rx_data_ack_character;
uint32_t uart2_rx_flag;

uint8_t  uart2_tx_character_count;

uint32_t uart2_tx_flag;
uint32_t uart2_tx_mode;
uint32_t uart2_tx_transmission_error;

uint8_t uart2_tx_checksum = 0;
uint8_t uart2_rx_checksum = 0;

uint32_t uart2_status;

extern uint32_t FT857D_uart_tx_watch_dog_timer; 
extern uint32_t FT857D_uart_rx_watch_dog_timer;
// *****************************************************************************
// *****************************************************************************
//          These routines are for the FT857D radios and use UART 2
// *****************************************************************************
// *****************************************************************************
// ****************************************************************************
// January 15, 2019		Design Documents                  USART2_Handler
//
// This program handles the interrupt for the USART2. This program should 
// override the weak interrupt in the startup program.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If there is a Receiver interrupt, a Transmit interrupt, or a Transmit
//  Complete interrupt, this program will handle it.
//
// External functions called
//  void uart2_receive_interrupt(void)
//  void uart2_transmit_interrupt(void)
//
// External variables used
//  uint32_t volatile *USART2_CR1
//  uint32_t volatile *USART2_SR
//  uart2_tx_flag
//
// Internal variables used
//  uint32_t status
//
// Constants
//  defined in register_definitions.h
//    USART2_CR1_ADDRESS
//    USART2_SR_ADDRESS
//    BIT05_MASK
//    BIT06_MASK
//    BIT07_MASK
//
// History
//
// Status
//  written, working
//
#pragma call_graph_root = "interrupt"
void USART2_Handler(void)
{
  
//
// Determine the source of the interrupt. If receive interrupt, process
//
  uart2_status = *USART2_SR;
  if (uart2_status & RXNE)
  {
    uart2_receive_interrupt();
  }
//
// Determine the source of the interrupt. If transmit interrupt process
//
  if (uart2_status & TXE)
  {
    uart2_transmit_interrupt();
  } 
//
// If transmission complete interrupt, turn off the TCIE interrupt. Because
// the transmission is now complete, set the uart2_rx_data_ack_character flag to
// 3 and set the watchdog timer.  We now have to wait for another 0x06 from the
// radio. If we don't get the acknowledgement within the watchdog time, we have
// to resend the data.
//
  if ((uart2_status & TC) && TC_flag)
  {
    *USART2_CR1 = *USART2_CR1 & (~TCIE);
    TC_flag = 0;
    *USART2_SR = *USART2_SR & (~TC);     
    uart2_rx_data_ack_character = 3;
    FT857D_uart_tx_watch_dog_timer = 100; 
  } 
}
//
// *****************************************************************************
// December 31, 2018		Design Documents         uart2_receive_interrupt
//
// This program reads data from the UART and puts it in a buffer. The structure
// of the incoming data is as follows:
//   
// 0xA5 NN DD...DD CS
//
// 0xA5 is the start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message and NN.
// CS   is the checksum
//
// When the A5 is received, the panel sends back an 0x06 right away. After the 
// check sum is received, the panel sends back an 0x06 right away.
//
// This program puts data first into a 256 byte buffer. If the checksum is good
// at the end, it transfers the data to a 1024 byte fifo. The purpose is because
// the radio sends out data faster than the panel could put it on the display.
// However, after it sends a few packets, it then waits a significant amount of
// time for the panel to put the data on the display.
// *****************************************************************************
extern uint32_t FT857D_auto_init_timer;

void uart2_receive_interrupt(void)
{
  // A receiver interrupt. Read the data register and set the watchdog timer.
  uart2_rx_data = *USART2_DR;
  FT857D_auto_init_timer = 1500; // 1.5 seconds timeout
  FT857D_uart_rx_watch_dog_timer = 10;

 switch(uart2_rx_state)
 {
    case 0:
      uart2_rx_state_00();
      break;
    case 1:
      uart2_rx_state_01();
      break; 
    case 2:
      uart2_rx_state_02();
      break;
    default:
      uart2_rx_state = 0;
      error_log( 3002 );      
      break;
  }
} 
//
// *****************************************************************************
// December 31, 2018		Design Documents               uart2_rx_state_00
//
// State 0 is waiting for either a periodic handshake or the start of a packet.
// If its a periodic handshake, the radio sends a 0x90 and the panel responds
// with a 0x06. If its a packet, the radio sends a 0xA5 and the panel responds
// with a 0x06 and then goes to the next state to receive the rest of the packet
// This state is also wating for a flag to indicate that anther program just
// sent a command to the radio and is expecting a handshake back from the radio.
// *****************************************************************************
void uart2_rx_state_00(void)
{
  // if periodic handshake, send 0x06 and return.
  if (uart2_rx_data == 0x90)
  {
    *USART2_DR = 0x06;
    return;
  }

  // If first part of a packet, send 0x06, set flag to indicate trying to
  // receive a packet and advance to the next state and return
  if (uart2_rx_data == 0xA5)
  {
    *USART2_DR = 0x06;
    uart2_rx_state++;
    return;
  }

  // If it got an 0x06 it is because the panel originated a packet and it is
  // waiting for an ecknowledgement from the radio. 
  if (uart2_rx_data == 0x06)
  {

  // If uart2_rx_data_ack_character is 1, then its the begining of the buffer 
  // being sent. Setup the transmission so send the rest of the packet. Because
  // the 0xA5 has already been sent, the buffer empty flag should be enabled so
  // we should immediately go to the transmitter interrupt upon exit from this
  // routine.We disable the watchdog timer because we are not waiting for any
  // outside devices to send an acknowledgement.
    if (uart2_rx_data_ack_character == 1)
    {
      uart2_tx_character_count = FT857D_tx_buffer[0]+1; // char count first char     
      FT857D_tx_buffer_index = 0;                 // point to char count
      *USART2_CR1 = *USART2_CR1 | TXEIE;          // enable buffer empty int
      uart2_rx_data_ack_character = 2;            // check for error
      FT857D_uart_tx_watch_dog_timer = 0;         // disable w
      return;
    }

    // If we get an 0x06 at this time, there is an error. We need to send the 
    // complete packet before we can expect an 0x06 character. If this happens we
    // should set a flag and wait for the transmission complete flag before we
    // resend the packet.
    if (uart2_rx_data_ack_character == 2)
    {
      uart2_tx_transmission_error = 1;
      return;
    }

  // At this point we got an acknowledgemet and we can now set the uart2_tx_flag 
  // flag to indicate we can send another packet. Clear out watch dog timer so 
  // it will not interfer with the next packet.
    if (uart2_rx_data_ack_character == 3)
    {
      FT857D_uart_tx_watch_dog_timer = 0;
      uart2_tx_flag = 1;
      uart2_rx_data_ack_character = 0;
    }
    return;
  }

  // If we get a 0x15 at this point, it is because the radio got the wrong
  // checksum. Make believe it went through and clear out the approriate 
  // variables. 
  if (uart2_rx_data == 0x15) 
  {
    FT857D_uart_tx_watch_dog_timer = 0;
    uart2_tx_flag = 1;
    uart2_rx_data_ack_character = 0;   
    error_log( 3400 );
    return;
  }

  // If its any other character just ignore and return
  // log anything other than 0, or 0xFF
  if( (uart2_rx_data != 0xFF) && (uart2_rx_data != 0) ) error_log( 1705 );
}
//
// *****************************************************************************
// December 31, 2018		Design Documents               uart2_rx_state_01
//
// State 1 is waiting for the character count of the packet. The first character
// that comes in after the A5 is the number of characters in the rest of the 
// packet including the checksum
// *****************************************************************************
void uart2_rx_state_01(void)
{
   FT857D_rx_buffer[0] = 0xA5;
   FT857D_rx_buffer[1] = uart2_rx_data;
   uart2_rx_character_count = uart2_rx_data;
   uart2_rx_checksum = 0;
   FT857D_rx_buffer_index = 2;   
   uart2_rx_state++;
}
//
//
// *****************************************************************************
// December 31, 2018		Design Documents               uart2_rx_state_02
//
// State 2 is waiting for the rest of the packet. When it gets the checksum and
// its correct it sends back an 0x06. If the checksum is wrong it sends back an
// 0x07 (Need to confirm about the 0x07)
// *****************************************************************************
void uart2_rx_state_02(void)
{
uint32_t i;

  FT857D_rx_buffer[FT857D_rx_buffer_index++] = uart2_rx_data;  
  if (--uart2_rx_character_count)
  {
    uart2_rx_checksum = uart2_rx_checksum + uart2_rx_data;
    return;
  }
  else
  {
    uart2_rx_state = 0;

    // If data is good, transfer to fifo. If data is bad, ignore data. 
    if (uart2_rx_checksum == uart2_rx_data)
    {
      *USART2_DR = 0x06;         // acknowledge good data
      for (i = 0; i < FT857D_rx_buffer[1]+2; i++)
      {
        FT857D_rx_fifo[FT857D_rx_fifo_input_pointer++] =  FT857D_rx_buffer[i];
        if (FT857D_rx_fifo_input_pointer > 1023) FT857D_rx_fifo_input_pointer = 0;
      }
      FT857D_uart_rx_watch_dog_timer = 100;  // 10 ms timeout   
    }
    else error_log( 3400 );
  }
}
//
// *****************************************************************************
// December 31, 2018		Design Documents        uart2_transmit_interrupt
//
// This program writes data from a buffer to the UART2. The structure of the
// buffer is as follows:
//   
// 0xA5 NN DD...DD CS
//
// 0xA5 is the start of the message
// NN   is the number of characters in the message including checksum but not
//      including start of message and NN.
// CS   is the checksum
//
// When the A5 is sent, the radio sends back an 0x06 right away. After the 
// check sum is sent, the radio sends back an 0x06 right away.
//
// Another program sends out the A5 and sets up this interrupt routine.
//
// This program is called when there is a transmit interrupt.
//
// This program sets a flag when all the data in the buffer is transmitted
//
// The programn works as follows:
//   When the character count is greater than 1, the data is outputed which will
//   clear the interrupt.
//
//   When the character count is equal to 1, the TXEIE is disabled and the TCIE
//   is enabled. At the end of this routine the character count will be zero.
//
//   When get a TC interrupt, turn off the TCIE interrupt and set the 
//   uart2_tx_flag to indicate another transmission can be started. This is done
//   in the calling program called USART2_Handler 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  uart2_tx_buffer[256] is the transmit buffer
//  uart2_tx_character_count is the number of characters in the buffer
//  uart2_tx_buffer_index is the index into the tx buffer
//  uart2_tx_mode is 0 for a single character transmit and 1 otherwise
//
// Exit Conditions
//  uart2_tx_flag indicates all the data is transmitted
//
// External functions called
//
// External variables used
//  uart2_tx_character_count
//  uart2_tx_mode
//  uart2_tx_buffer[256]
//
// Internal variables used  
//  uint32_t volatile *USART1_DR
//  uint32_t volatile *USART1_CR1
//
// Constants
//  defined in register_definitions.h
//    USART2_DR_ADDRESS; 
//    USART2_CR1_ADDRESS
//    BIT06_MASK
//    BIT07_MASK
//
// History 
//
// Status
//  written
// *****************************************************************************
void uart2_transmit_interrupt(void)
{

  // Process multi-byte transmissions. If buffer empty, just exit from routine. 
  // It means we came here from TC interrupt
  if (uart2_tx_character_count)
  {

    // Turn off buffer empty interrupt and turn on buffer complete interrupt.
    // Set the TC_flag so the interrupt program knows it should use it.
    if (uart2_tx_character_count == 1)
    {
      *USART2_CR1 = *USART2_CR1 & (~TXEIE); // buffer interrupt
      *USART2_CR1 = *USART2_CR1 | TCIE;
      TC_flag = 1;      
      FT857D_uart_tx_watch_dog_timer = 100;
    }

    // Output data in buffer and decrement character count
    *USART2_DR = FT857D_tx_buffer[FT857D_tx_buffer_index++];
    uart2_tx_character_count--;
  }
}
// *****************************************************************************
// December 31, 2018		Design Documents                 send_uart2_0xA5
//
// This routine sets up uart2 and sends the first character of the packet.
// It is assumed that when this program is called, the uart2_tx_buffer[] is
// already set up. That means the first position is the packet length, the
// rest of the characters contain the data and the last character is the
// checksum.
//
// uart2_tx_transmission_error when 1 indicates that an acknowledgement
// character has been received before the completee packet was sent.
//
// uart2_rx_data_ack_character controls the state machine for handling 
// acknowledgement characters within the receive interrupt routine.
//
// uart2_tx_flag when 1, indicates the packet has been successfully sent.
//
// uart2_rx_state controls the state machine for the receiver interrupt. When
// transmitting packets, the receiver interrupt must be in state 0.
//
// uart2_tx_watch_dog_timer is used to trap errors. When the timer gets to a 
// value of 1, then there was an error and the packet must be resent. If the
// value of the watchdog timer is zero, then the watchdog function will be
// disabled. A value of 100 indicates there is a 10 ms wait for the timeout.
//
// uart2_enable_tx_te(void) removed to match uart1 behavior
/*
// void uart2_enable_tx_te(void) routine is used for enabling the UART2 for
// transmitting. It does not enable any interrupts.
*/
//
// *****************************************************************************
//
void uart2_FT857D_send_0xA5(void)
{

  // Reset a bunch of variables to allow everything to work.
  uart2_tx_transmission_error = 0; 
  uart2_rx_data_ack_character = 1;
  uart2_tx_flag = 0;
  uart2_rx_state = 0;
  *USART2_DR = 0xA5;
  FT857D_uart_tx_watch_dog_timer = 100;
}
// =============================================================================
//                                                       uart2_FT857D_initialize
// Initialize USART 2 for use with FT857D radios
//
void uart2_FT857D_initialize(void)
{
  uint32_t volatile *RCC_APB1ENR = (uint32_t *) RCC_APB1ENR_ADDRESS;

  // Enable clock for USART 2. Enable bit 17 from RCC_APB1ENR register.
  *RCC_APB1ENR = *RCC_APB1ENR | RCC_APB1ENR_UART2EN; // Turn on clock for USART2

  // Set up CR1 and CR2 but disable UART for now
  *USART2_CR1 = 0x00000024;
  *USART2_CR2 = 0x00000000;

  // baud rate 62.5K firmware version 20.42 and up
  // 84 Mhz/ (16 * 62500) = 84 = 0x54
  // 42 Mhz/ (16 * 62500) = 42 = 0x2A
  // 0x54 for mantissa and 0 for fraction
  *USART2_BRR = 0x000002A0;  // Based on a 42 MHz clock
  *USART2_SR =  0x00C00000;  // Set SR back to factory default.

  // Enable the interrupt controller for UART2
  nvic_enableIRQ(38);

  // Enable the UART
  *USART2_CR1 = 0x0000202C;  
}
// *****************************************************************************
/* not in use, left for debugging
// =============================================================================
// The program below is used to test the transmitter and output packets to the
// display to determine what it does.
//
// This program is used to output data to the transmitter from an array
//
// First byte is character count. The program generates the 0xA5 code
//

uint8_t xmt_packets[][30] = 
{ //      1    2     3     4     5     6     7     8     9     10    11    12    13    14    15    16    17    18    19    20    21    22    23     24    25    26    27    28    29
  0x04, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 1  clear display
  0x17, 0x41, 0x00, 0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 2 Line 0
  0x18, 0x41, 0x01, 0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 3 Line 1
  0x10, 0x41, 0x02, 0x01, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 4 Line 2
  0x07, 0x41, 0x03, 0x00, 0x41, 0x42, 0x43, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 5 Line 3
  0x1A, 0x41, 0x04, 0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x00, 0x00, 0x00, 0x00,  // 6 Line 4
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 8
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 9
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 10 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 11 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 12
};

uint8_t binary_to_hex[16] = 
{
  '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
};

uint8_t  karray;
uint8_t  kitem; 
uint16_t FT857D_output_character(uint8_t character, uint8_t line, uint8_t column);

void transmit_packets(void);
void calculate_checksum(void);

void transmit_packets(void)
{
uint8_t i;
uint8_t j;
uint8_t count;
//
// Put all characters on all five lines.
//
  calculate_checksum();
  *USART2_SR = 0;

  for (i = 0; i < 6; i++)
  {
//
// Transfer packet data to uart2_tx_buffer[];
//
    count = xmt_packets[i][0] + 1;
    for (j = 0; j < count; j++)
    {
      FT857D_tx_buffer[j] = xmt_packets[i][j];
    }
//
// Start sending the packet and wait for it to be finished
//
    uart2_FT857D_send_0xA5();
    while (uart2_tx_flag == 0) {}  // Wait till packet is complete
  }

//
// Get current array and item and display it on the BFD
//
  karray = 0x50;
  kitem = 0;  
loop:  
  
  FT857D_output_character(binary_to_hex[karray >> 4], 2, 1);
  FT857D_output_character(binary_to_hex[karray & 0x0F], 2, 2);
  FT857D_output_character(binary_to_hex[kitem >> 4], 2, 4);
  FT857D_output_character(binary_to_hex[kitem & 0x0F], 2, 5);  
  
//
// If left rotary switch turn left decrement karray
//
  if((last_key_press == 0x1C) && (key_type == 2))
  {
    kitem--;
    last_key_press = 0;
    goto loop;
  }
  if((last_key_press == 0x1C) && (key_type == 3))
  {
    kitem++;
    last_key_press = 0;
    goto loop;
  }
  if(last_key_press == 0x19)
  {
    karray++;
    last_key_press = 0;
    goto loop;
  }
  if(last_key_press == 0x1A) 
  {
    karray--; 
    last_key_press = 0;
    goto loop;
  }
  
  if((last_key_press == 0x15))
  {
    FT857D_tx_buffer[0] = 0x04;
    FT857D_tx_buffer[1] = 0x40;
    FT857D_tx_buffer[2] = karray;
    FT857D_tx_buffer[3] = kitem;
    FT857D_tx_buffer[4] = 0x40 + karray + kitem;
//
// Start sending the packet and wait for it to be finished
//
    uart2_FT857D_send_0xA5();
    while (uart2_tx_flag == 0) {}  // Wait till packet is complete
  }
  last_key_press = 0;
  

  goto loop;
}

//
// calculate check sum for xmt_packets array. First byte is 0xA5, second byte is
// character count. Character count is 10 or less
//
uint8_t checksum;
uint8_t ki;
uint8_t kj;
uint8_t kpacket_length;
void calculate_checksum(void)
{
  for (ki = 0; ki < 11; ki++)
  {
    kpacket_length = xmt_packets[ki][0];
    checksum = 0;
    for (kj = 0; kj < kpacket_length-1; kj++)
    {
      checksum = checksum + xmt_packets[ki][kj+1];
    }
    xmt_packets[ki][kj+1] = checksum;
  }
}
*/