//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Jul 11, 2018		        Design Documents             CSI_Bootload_Jump.c 
// 
//                     Jump to STM system bootloader 
//                       for large display project
//
// based on code at:
// https://stm32f4-discovery.net/2017/04/tutorial-jump-system-memory-software-stm32/
// rewritten for STM32F405 and no HAL
//
// The STM32F405 is supplied with a built in bootloader in static ram.
//  The bootloader is used to load a program into the flash memory of the CPU.
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  07/11/2018 
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  Bootloader_Jump               sets up cpu and jumps to system bootloader 
//
//==============================================================================
#include <stdint.h>
#include <intrinsics.h>
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"

//==============================================================================
//
//  Jump to system memory boot from user application
//
// This function is used at startup to jump to the built in bootloader.
//
// test with DFUse in PC connected to USB-2 connector.
// Hold p1 and p4 at startup. 

void Bootloader_Jump(void)
{
  void (*sysmem_bootjump)(void);
    
  uint32_t volatile *RCC_CR         = (uint32_t *) RCC_CR_ADDRESS;
  uint32_t volatile *RCC_CFGR       = (uint32_t *) RCC_CFGR_ADDRESS;
  uint32_t volatile *RCC_PLLCFGR    = (uint32_t *) RCC_PLLCFGR_ADDRESS;
  uint32_t volatile *RCC_CIR        = (uint32_t *) RCC_CIR_ADDRESS;
  uint32_t volatile *FLASH_ACR      = (uint32_t *) FLASH_ACR_ADDRESS;  
  uint32_t volatile *CPACR          = (uint32_t *) CPACR_ADDRESS;

  // turn off clocks used to check gpio pins
  uint32_t volatile *PWR_CR         = (uint32_t *) PWR_CR_ADDRESS;
  uint32_t volatile *RCC_AHB1ENR    = (uint32_t *) RCC_AHB1ENR_ADDRESS;
  uint32_t volatile *RCC_APB1ENR    = (uint32_t *) RCC_APB1ENR_ADDRESS;
  
  *PWR_CR      = 0;  // PWR Power control register
  *RCC_AHB1ENR = 0;  // RCC AHB1 peripheral clock enable register
  *RCC_APB1ENR = 0;  // RCC APB1 peripheral clock enable register
  
  // Set system memory address. 
  // from AN2606, STM32F405 system memory is at 0x1FFF 0000   
  volatile uint32_t sysmem_addr = 0x1FFF0000;
  
  *CPACR |= ((3UL << 10*2)|(3UL << 11*2));  // set CP10 and CP11 Full Access 
    
  // Disable RCC, set it to default (after reset) settings
  // Internal clock, no PLL, etc.
  *RCC_CR =      0x00000081; // HSI on, system will set bit1 when HSI is ready
  *RCC_CFGR =    0;
  *RCC_PLLCFGR = 0x24003010;
  *RCC_CIR =     0;
  
  *FLASH_ACR =   0;  // set flash access to default
   
  // Disable systick timer and reset it to default values
  uint32_t volatile *STK_CTRL = (uint32_t *) STK_CTRL_ADDRESS;
  uint32_t volatile *STK_LOAD = (uint32_t *) STK_LOAD_ADDRESS;
  uint32_t volatile *STK_VAL  = (uint32_t *) STK_VAL_ADDRESS;
  uint32_t volatile *SHPR3    = (uint32_t *) SHPR3_ADDRESS;   
    
  *STK_CTRL  = 0;
  *STK_LOAD  = 0;
  *STK_VAL   = 0;
  *SHPR3     = 0;
 
  //  Disable all interrupts
  uint32_t volatile *NVIC_ICER0 = (uint32_t *) NVIC_ICER0_ADDRESS;
  uint32_t volatile *NVIC_ICER1 = (uint32_t *) NVIC_ICER1_ADDRESS;
  uint32_t volatile *NVIC_ICER2 = (uint32_t *) NVIC_ICER2_ADDRESS;  
  
  // clear enabled interrupts by setting to 1
  *NVIC_ICER0 = 0xFFFFFFFF;
  *NVIC_ICER1 = 0xFFFFFFFF;  
  *NVIC_ICER2 = 0xFFFFFFFF;
  // clear pending interrupts
  uint32_t volatile *NVIC_ICPR0 = (uint32_t *) NVIC_ICPR0_ADDRESS;
  uint32_t volatile *NVIC_ICPR1 = (uint32_t *) NVIC_ICPR1_ADDRESS;
  uint32_t volatile *NVIC_ICPR2 = (uint32_t *) NVIC_ICPR2_ADDRESS;  
  // clear pending interrupts by setting to 1
  *NVIC_ICPR0 = 0xFFFFFFFF;
  *NVIC_ICPR1 = 0xFFFFFFFF;  
  *NVIC_ICPR2 = 0xFFFFFFFF;  
  
  //  Remap system memory to address 0x0000 0000 in address space
  uint32_t volatile *SYSCFG_MEMRMP = (uint32_t *) SYSCFG_MEMRMP_ADDRESS; 
  *SYSCFG_MEMRMP  = 0x01;
  
  //  Set jump memory location for system memory
  //   Use address with 4 bytes offset which specifies jump location
  //   where bootloader program starts
  sysmem_bootjump = (void (*)(void)) (*((uint32_t *)(sysmem_addr + 4)));
    
  // Set main stack pointer.
  //  This must be done last for the local variables in this function
  //  to have the correct values. After this, the stack pointer points to a
  //  a different memory location.
  //  Set direct address location which specifies stack pointer in SRAM location
  __set_MSP(*(uint32_t *)sysmem_addr);
  
  //  !!test asm to substitute for IAR intrinsic function __set_MSP
  // test worked. reverted to use intrinsic for release
  // needs operands and clobbers to be correct  
  //asm("LDR R1,[SP, #+0]");
  //asm("LDR R1,[R1, #+0]");
  //asm("MSR MSP,R1");  

  // Call the function to jump to the system bootloader
  sysmem_bootjump();
    
  // test with DFUse in PC connected to USB-2 connector
}
