//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 7,2018             Design Documents                         CSI_eeprom.c
//
//			routines for EEPROM
//                   large panel display project
//
//           for 24LC1025 EEPROM.
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//  
// History
//  06/7/2018 moved from CSI_i2c.c
//
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  EEPROM_MemWrite              Write one byte data to EEPROM
//  EEPROM_busypoll              EEProm polling after write 
//  EEPROM_MultipleWrite         Write multiple data to EEPROM
//  EEPROM_MemRead               Read multple data from EEPROM
//  EEPROM_Mem_fill              Write multiple bytes of the same data to EEPRM
//  EEPROM_waitfor_I2C           Wait for EEPROM to complete a write
//  save_radio_type              Writes radio type to EEPROM
//  load_radio_type              Reads radio type to EEPROM
//
// Old returns left to prevent compile errors
//
//  eeprom_write
//  eeprom_write16
//  eeprom_read
//  eeprom_read16
//
// Status: for debugging only
//
//  I2C_statlog                  logs I2C status for debugging
//  EEPROM_TestWrite             Test write to EEPROM without ISR
//  EEPROM_TestRead              Test read of EEPROM without ISR

//==============================================================================
//
#include "CSI_eeprom.h"
#include "CSI_i2c.h"
#include "CSI_interrupt.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_TIMERS.h"

// =============================================================================
//
// Writes the EEPROM device with a single byte
//   a three byte write to a slave that has a word sized memory address.
//   address is sent as two bytes, high first then low
//
//   for 24LC1025 EEPROM. Two separate 512 K blocks 
//   which block selected is controlled by the "block select bit" bit 3
//
//   just for I2C 2 because there's only 1 buffer (for now)
//
//  Single byte EEPROM writes are very bad. If lots of data is written,
//  the eeprom does a page write at the end of each write instruction.
//  If single bytes are written, each byte must wait, and the eeprom
//  must do many page writes. The page write has a long delay.
//  Multiple byte writes should be combined and one page write done
//  after all the bytes have been transmitted.
//
// History
//    07/05/2018  added busypoll to end, must wait after the write
//
// Status: written, tested, working
//  
//
void EEPROM_MemWrite( uint8_t block, uint8_t AddrHigh,
                     uint8_t AddrLow,uint8_t MemData)
{
  uint8_t control_byte = I2C_eprom;
  // set block bit for access to second block if needed  bit 3
  if(block) control_byte |= block << 3; 
  TxRxDevAddress = control_byte;
  // put data in the dataout buffer
  I2C_put_dataout( AddrHigh );  
  I2C_put_dataout( AddrLow );
  I2C_put_dataout( MemData );

  I2C_rx_count_to_read = 0;
  // start tx, 4 bytes control, 2 address, 1 data
  i2c_start_tx( 2, control_byte, 4 );
  
  // wait for I2C to finish
  while(!I2C_2_txrx_complete);
  //do polling to wait for EEProm write to complete
  EEPROM_busypoll();  
  
}

// ===============================
// for debugging, log of I2C_2 status 1

int statlog[100];
int statindx = 0;

void I2C_statlog( void )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
  
  statlog[statindx++] = *SR1;
  if(statindx > 99) statindx = 0;
}
// =============================================================================
//   DEBUG TESTING
//  test with interrupts turned off
//   Initialize enables the interrupts, this turns them off 
//
// Writes the EEPROM slave device with a single byte
//
// written, tested
// 
// =============================================================================
//
//  EEPROM Write a single byte
//
// Status
//    Written, tested, working


void EEPROM_TestWrite( uint8_t block, uint8_t AddrHigh,
                      uint8_t AddrLow,uint8_t MemData)
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
    // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint16_t stat1; 
  uint16_t stat2;
  // make 8 bit pointer to the Data register
  uint8_t volatile *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);
  uint8_t control_byte = I2C_eprom;
  
  // disble all interrupts in the module
  I2C_All_Interrupts( 2, 0 );
  
  // turn off interrupts in the interrupt controller
  nvic_disableIRQ( I2C2_EV_INTR );  // events
  nvic_disableIRQ( I2C2_ER_INTR );  // errors    
  
  // set block bit for access to second block if needed  bit 3
  if(block) control_byte |= block << 3;
  // clear LSB to indicate transmission
  // put data in the dataout buffer
  // I2C_put_dataout( control_byte );
  I2C_put_dataout( AddrHigh );  
  I2C_put_dataout( AddrLow );
  I2C_put_dataout( MemData );
  
  I2C_tx_count = 3;
  I2C_txrx = 1;

  // set start condition
  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) );

  // write control byte  byte 1
  stat1 = *SR1;   // need to read sr1 then write dr to clear sb
  *DR = control_byte;

  // TxE will not be set after the control byte  
  // wait for ADDR
  do  stat1 = *SR1; while( !(stat1 & I2C_ADDR) );
 
  // I2C_statlog();  //0

  stat1 = *SR1;
  stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
  
  if(stat2);   // to eliminate compiler warning
  
  // send memory address  byte 2
  // *DR = AddrHigh;
  *DR = I2C_2_data_out[I2C_dbufout_tail++];
  if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;   
 
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );   
    // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  // I2C_statlog();  //1

  //*DR = AddrLow;   // byte 3
  *DR = I2C_2_data_out[I2C_dbufout_tail++];
  if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;
  
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );   
    // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  // I2C_statlog();  //2
  
  // send data
  // *DR = MemData;   // byte 4
  *DR = I2C_2_data_out[I2C_dbufout_tail++];
  if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;
  
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );   
    // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  
  // delay before setting stop
  delay(90);
  // I2C_statlog();  //3
  // stop needs to be set before end of transmission of last byte
  I2C_set_stop( 2 );

  delay(20);
  // AF will not be set yet
  
  // begin polling
  // need first command to set AF to 1
  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) );
  // send control byte
  *DR = control_byte;
  do stat1 = *SR1; while( !(stat1 & I2C_AF));
  
  // poll until AF bit is clear
  while( stat1 & I2C_AF )   
  {
    I2C_set_start( 2 );
    // wait until start bit is set
    do stat1 = *SR1; while( !(stat1 & I2C_START) );  
    // send control byte
    *DR = control_byte;

    stat1 = *SR1;
    if(stat1 & I2C_ADDR)
    {
      stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
      *SR1 &= ~I2C_AF;  // clear AF if good address, break
      break;
    }
  } 
  
  I2C_set_stop( 2 );
  
  // I2C_statlog(); //6
  
  statlog[statindx++] = 0xFF;
  if(statindx > 99) statindx = 0;

}
//
// =============================================================================
//
//  EEPROM polling after a write to wait for eeprom to finish
//    not using the ISR
//
//    needs a timeout and error loop
//
// History
//  06/06/2018
//  07/03/2018  modified from working version in board test, needs testing
// Status: written, testing, debug needed, not working correctly, leaves I2C
//         or eeprom in non-functional state at end of this   

void EEPROM_busypoll( void )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
    // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint16_t stat1; 
  uint16_t stat2;
  
  // make 8 bit pointer to the Data register
  uint8_t volatile *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);
  
  uint8_t control_byte = I2C_eprom;
  
  nvic_disableIRQ( I2C2_EV_INTR );
  // disble all interrupts in the module
  I2C_All_Interrupts( 2, 0 );
  
  // just to be sure, clear LSB to indicate transmission
  control_byte &= 0xFE;
  // need a command to set AF to 1
  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) );
  // send control byte
  *DR = control_byte;
  //
  // 100 Khz 9 bits 90 usec
  // wait 90 usec, should get AF 
  delay(90);
  stat1 = *SR1;
  // DEBUG here!!! needed a breakpoint here, then it ran 
  // delay or wait for something other than AF 1
  //
  // do stat1 = *SR1; while( !(stat1 & I2C_AF));
  
  // poll until AF bit is clear
  while( stat1 & I2C_AF )   
  {
    *SR1 &= ~I2C_AF;  // clear AF
    I2C_set_start( 2 );
    // wait until start bit is set
    do stat1 = *SR1; while( !(stat1 & I2C_START) );  
    // send control byte
    *DR = control_byte;
    // wait 90 usec
    delay(90);
   
    // no txe after the control byte, wait for addr or af
    stat1 = *SR1;
    if(stat1 & I2C_ADDR)
    {
      stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
      *SR1 &= ~I2C_AF;  // clear AF if good address, break
      break;
    }
  }
  I2C_set_stop( 2 );
  // sometimes ends with ADDR set, read both stats to clear
  stat1 = *SR1;
  stat2 = *SR2;  
  if(stat2);   // to eliminate compiler warning
  
  // clear any current interrupts
  *SR1 = 0;
  
  // reenable all interrupts in the module
  I2C_All_Interrupts( 2, 1 );

  nvic_enableIRQ( I2C2_EV_INTR );

  
}

// =============================================================================
//   Debuggng test
//
//   READ
//
// read the EEPROM device one byte, using module without interrupts
//

void EEPROM_TestRead( uint8_t block, uint8_t AddrHigh, uint8_t AddrLow )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
  // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint8_t volatile *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);
  uint16_t stat1;
  uint16_t stat2;
  
  uint8_t control_byte = I2C_eprom;

    // disble all interrupts in the module
    I2C_All_Interrupts( 2, 0 );
    
    // turn off interrupts in the interrupt controller
    nvic_disableIRQ( I2C2_EV_INTR );  // events
    nvic_disableIRQ( I2C2_ER_INTR );  // errors    
  
  // set block bit for access to second block if needed  bit 3
  if(block) control_byte |= block << 3;
   // clear LSB to indicate transmission
  control_byte &= 0xFE; 
  // put data in the dataout buffer
  I2C_put_dataout( AddrHigh );  
  I2C_put_dataout( AddrLow );

  I2C_tx_count = 2;
  I2C_txrx = 1;

  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) ); 
  // write control byte  byte 1
  stat1 = *SR1;   // need to read sr1 then write dr to clear sb
  *DR = control_byte;

  // TxE will not be set after the control byte  
  // wait for ADDR
  do  stat1 = *SR1; while( !(stat1 & I2C_ADDR) );
  // I2C_statlog(); 

  stat1 = *SR1;
  stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
  if(stat2);   // to eliminate compiler warning
  
  // send memory address  byte 2
  // *DR = AddrHigh;
  *DR = I2C_2_data_out[I2C_dbufout_tail++];
  if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;   
 
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );   
    // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  // I2C_statlog(); 

  //*DR = AddrLow;   // byte 3
  *DR = I2C_2_data_out[I2C_dbufout_tail++];
  if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;
  
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );   
    // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  // I2C_statlog();  
 
  // send new control byte for reading
  // set LSB to indicate receive
  control_byte |= 0x0001;
 
  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) ); 
  // Set to NOACK  
  I2C_set_ack( 2, 0 );

  // write control byte  
  *DR = control_byte;
  
  // switch to receiving
  I2C_rx_count = 1;
  I2C_txrx = 0;
  
  // TxE will not be set after the control byte  
  // wait for ADDR
  do  stat1 = *SR1; while( !(stat1 & I2C_ADDR) );
  stat1 = *SR1;
  stat2 = *SR2;   // read SR2 after a good address, to clear ADDR  
    // Set to NOACK  
  I2C_set_ack( 2, 0 );
 
  // read status register
  // I2C_statlog();  
  
  // wait for RxNE 
  do stat1 = *SR1; while( !(stat1 & I2C_RXNE) );
  
  // read data
  I2C_2_data_in[I2C_dbufin_head++] = I2C_read_byte( 2 );
  
  // send stop
  I2C_set_stop( 2 );

  statlog[statindx++] = *SR1;
  if(statindx > 99) statindx = 0;
  
  statlog[statindx++] = 0xFF;
  if(statindx > 99) statindx = 0;

}

// =============================================================================
//
// Writes the eeprom device with multiple bytes
//   write from the data_out buffer to eeprom. It has a word sized memory 
//   address is sent as two separate bytes, high first then low
//
//   The maximum bytes to write 128 is the size of the page buffer in the EPROM
//
//   Data in the data_out buffer must be loaded with the destination address
//   in the first two bytes.
//   The count includes the two address bytes and the control byte.
//
//   for 24LC1025 EPROM. Two separate 512 K blocks 
//   which block selected is controlled by the "block select bit" bit 3 
//
//   just for I2C 2 because there's only 1 buffer (for now)
//
// Status: written, tested, working

void EEPROM_MultipleWrite( uint8_t block, uint16_t num_bytes)
{
  uint8_t mem_pagepos;
  
  uint8_t control_byte = I2C_eprom;
  
  /*
  uint16_t mem_address;
  uint16_t mem_page;

  uint8_t num_towrite;
  
  uint8_t startAddrH;
  uint8_t start_AddrL;
    
  // handle multiple page writes
 
  mem_address = ( AddrHigh << 8 ) + AddrLow; 
  mem_page = mem_address >> 7;
  mem_pagepos = AddrLow & 0x7F;   // clear high bit

  // set starting memory address
  startAddrH = AddrHigh;
  startAddrL = AddrLow;
  // loop
  while (num_bytes > 0)
  {
    // does numbytes + pagepos extend past end of page?
    if((mem_pagepos + num_bytes) > 128)
    {
      // if yes write to end of page 128 - pagepos bytes
      
       // make new starting memory address for next page
       // set pagepos to 0, the beginning of the next page
       // reduce numbytes by num written      
    }
    else
    {
    // if no write numbytes
      // keep address unchanged
      // reduce num_bytes
      // break when done
    }
  }  // end loop
  
  num_towrite = 128 - mem_pagepos;
  */
  // starting address is stored in the outdata buffer high byte first
  // we only need the low byte to determine position in the page
  mem_pagepos = I2C_2_data_out[I2C_dbufout_tail + 1] & 0x7F; // clear high bit
  
  // subtract 3 for the control byte, and 2 address bytes
  if((mem_pagepos + num_bytes - 3) > 128)  
  {
    // limit the write to not go past the end of the page
    num_bytes = 128 - mem_pagepos;
    error_log( 2202);
  }
  //todo: break up large write into multiple page writes

  // set block bit for access to second block if needed  bit 3
  if(block) control_byte |= block << 3; 
  
  TxRxDevAddress = control_byte;
  I2C_rx_count_to_read = 0;  
  
  // start tx, I2C 2, device, number of bytes
  i2c_start_tx( 2, control_byte, num_bytes );
}
// =============================================================================
//
// Random access reads of the EEPROM
//   address is sent as two separate bytes, high first then low
//   then the read command is sent
//   data will be placed in the datain buffer until the slave stops sending

//   for 24LC1025 EPROM. Two separate 512 K blocks 
//   which block selected is controlled by the "block select bit" bit 3 
//
//   just for I2C 2 because there's only 1 buffer (for now)
//
void EEPROM_MemRead( uint8_t block, uint8_t AddrHigh,
                      uint8_t AddrLow,uint16_t count_to_read)
{
  uint8_t control_byte = I2C_eprom;
  
  // set block bit for access to second block if needed,  bit 3
  if(block) control_byte |= block << 3; 
  // put address data in the dataout buffer
  I2C_put_dataout( AddrHigh );  
  I2C_put_dataout( AddrLow );
  
  // set count so that transmitter knows a read follows and not to send a stop
  I2C_rx_count_to_read = count_to_read;
  // send three bytes, the memory address + control byte 
  // start tx, I2C 2, device, 3 bytes
  I2C_rx_count = 0;
  i2c_start_tx( 2, control_byte, 3 );
  
  // receiving is done in the ISR

}
// =============================================================================
//
//  EEPROM fill memory with a byte
//
//  writes a number of the same byte up to 128 bytes
//  erasing is not required for EEPROM,
//  but this is useful for restoring to the unwritten state FF,
//  use: EEPROM_Mem_fill( 0, 0, 0, 128, 0xFF) 


void EEPROM_Mem_fill( uint8_t block, uint8_t AddrHigh,
                      uint8_t AddrLow, uint8_t num_bytes, uint8_t fillbyte )
{
  int i;
  // load transmit buffer, address
  I2C_put_dataout( AddrHigh );  
  I2C_put_dataout( AddrLow );  
  // load transmit buffer, data
  for( i = 0; i < num_bytes; i++) I2C_put_dataout( fillbyte );
  // send the data
  EEPROM_MultipleWrite( block, num_bytes);
  EEPROM_waitfor_I2C();
}
// =============================================================================
//
//  EEPROM wait for I2C to finish
//  reset I2C and EEPROM if it times out
//
// The reset was a fix until the I2C was debugged.
//  If the I2C is working correctly, it doesn't timeout
//
//  Leaving it in for error handle

//  11/30/2018 modifed from similar wait with the touch screen
//  12/20/2018 increased timeout to 30 for larger write of favorites data

void EEPROM_waitfor_I2C( void )
{
   int EEP_waitcount = 50;
   unsigned long msec;
   msec = millisec;
   while(!I2C_2_txrx_complete)
   {
    if(millisec != msec) 
    {
      msec++;
      if(--EEP_waitcount == 0)
      {
         error_log( 2203 );
         log_event( "EEPROM-I2C reset" );
         // re-initialize I2C 
         i2c_2_initialize( I2C_eprom ); 
         break;
      }
    }
  }
}
// =============================================================================
//
//   Save radio type
//   
//    for the first use CF has not been written to the config flag
//    so the radio type set in main will be written and the flag set
//    during initialize()
//    All radio changes after first use will have the flag set and
//    whichever radio type set.

//   more radio info will probably be stored here

//
// History 02/19/2019 written
//         07/22/2019 added panel config flag

// eeprom address for radio type
// 0x2500
#define RADIO_INFO_HI  0x26
#define RADIO_INFO_LO  0x00

void save_radio_type( uint8_t r_type )
{
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout( RADIO_INFO_HI );  
  I2C_put_dataout( RADIO_INFO_LO );

  I2C_put_dataout( r_type );
  I2C_put_dataout( 0xCF );     

  // write to EEPROM  3+2 4 bytes
  EEPROM_MultipleWrite( 0, 5);
  EEPROM_waitfor_I2C();
}
// =============================================================================
//
//   Load radio type from eeprom
//
// History 02/19/2019 written
//         07/22/2019 added panel config flag


uint8_t panel_config_flag = 0;  

uint8_t load_radio_type( void )
{
  uint8_t r_type;
  // read 
  EEPROM_MemRead( 0, RADIO_INFO_HI,RADIO_INFO_LO, 2);
  // wait for I2C to finish
  EEPROM_waitfor_I2C();  
  r_type = I2C_get_datain();
  // read config flag
  panel_config_flag = I2C_get_datain();
  
  return r_type;
}

                  
// =============================================================================
// old routines (not working, left to prevent compile errors)
// These are called in CSI_programming.c which has not been updated or tested.
// =============
void eeprom_write(uint32_t address, uint8_t data)
{
}
// =============
void eeprom_write16(uint32_t address, uint16_t data)
{
uint8_t i;
  i = data;
  eeprom_write(address++,i);
  i = (data >> 8);
  eeprom_write(address,i);
}
// =============
uint8_t eeprom_read(uint32_t address)
{
  
  return 5;
}
// =============
uint16_t eeprom_read16(uint32_t address)
{
uint8_t i;
uint8_t j;
  i = eeprom_read(address++);
  j = eeprom_read(address);
  return (j*256) + i;
}





/***************************** End Of File ***********************************/
