//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 22,2018             Design Documents                            CSI_i2c.c
//
//			routines for I2C 
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// Many of these functions will work for any I2C module by sending
// the module number.
// There are some functions that are dedicated to I2C 2 only.
// for the large display project only I2C 2 is used.
//
// I2C transmit sequence:
// the default mode, and resting state is slave, with no clock being sent
// set the start condition bit in CR1 
// when data is written to the DR the module switches to master mode
// and starts sending  
  
// 7 bit addressing mode:
// LSB of the slave address indicates tx or rx
// make the LSB 0 for transmit, 1 for receive 
// set the start condition bit in CR1 
// write slave address to the data register, the master starts sending data 
// when the start sequence is done the start bit is set
// the master begins sending the address data  
// and the SB event interrupt is set
// read SR1 and write the data1 while the address is being sent  
// when the address has been sent the ADDR event interrupt is set
// read SR1 SR2 and write the data2 while the data1 is being sent
// when the data1 has been sent TXE = 1 event interrupt is set
// the data2 is already in the DR so sending begins
// write data3 to the DR
// when done, set the stop condition bit in CR1   
//  
// History
//  03/22/2018 copied from CS7000 for OMAP L-138 (Kirisun radio)
//             revising for STM32F405
//  03/27/2018 done writing
//  06/06/2018 finished rewriting , EEPROM routines moved to CSI_eeprom.c
//  09/04/2018 Bug fix of ISR, no longer hangs
//  02/14/2019 Bug fix of ISR for touch panel, resets less often
//  04/12/2019 Bug fix of ISR changed timing of delay at the end of transmission
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  i2c_2_initialize                 Initialize I2C
//  i2c_start_tx                     begin a transmission
//  I2C2_EV_Handler                  I2C event handler ISR
//  I2C2_ER_Handler                  I2C error handler ISR
//  dbufin_readynum                  gets number of bytes ready in the buffer
//  I2C_put_dataout                  puts data in the tranmit buffer
//  I2C_get_datain                   gets data from the receive buffer
//  I2C_Event_Interrupts             turn on or off event interrupts
//  I2C_Error_Interrupts             turn on or off error interrupts
//  I2C_Buffer_Interrupts            turn on or off buffer interrupts
//  I2C_All_Interrupts               turn on or off all I2C interrupts
//  I2C_Busy                         Check iof device is busy
//  I2C_SetFreq                      set I2C input clock frequency (APB1)
//  I2C_Enable                       turn on I2C module
//  I2C_Disable                      turn off I2C module
//  I2C_SetClock                     set I2C clock rate
//  I2C_SetRiseT                     set I2C rise time
//  I2C_Get_status1                  get I2C satus1 as 16 bit
//  I2C_send_byte                    Send single byte to the data register
//  I2C_read_byte                    Read single byte from the data register
//  I2C_set_start                    sets I2C start bit
//  I2C_set_stop                     sets I2C stop bit
//  I2C_set_ack                      sets I2C acknowledge bit

// Status: written, unused
//
//  I2C_SlaveRegWrite                write to an I2C register for commands
//  I2C_SWReset                      I2C software reset
//  I2C_Get_STAT1                    get I2C satus1 as a register type
//  I2C_Get_STAT2                    get I2C satus2 as a register type
//  I2C_Get_status2                  get I2C satus2 as 16 bit
//  I2C_clear_start                  clears I2C start condition
//  I2C_set_slave_ad                 sets slave address
//  I2C_read_device_id               I2C read device id 
//  I2C_TS_test                      I2C test to find touch screen address 
//
//==============================================================================
//
#include "CSI_i2c.h"
#include "CSI_GPIO.h"
#include "CSI_interrupt.h"
#include "CSI_CLOCKS.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_TIMERS.h"
#include "CSI_REGISTERS.h"
#include "CSI_TouchPanel.h"

int i2c2_device_active = 0;

uint8_t I2C_2_data_out[I2C_DATA_BUF_SIZE];
int I2C_dbufout_head = 0;
volatile int I2C_dbufout_tail = 0;  // volatile because ISR sends data out
volatile int I2C_tx_count = 0;      // number of bytes to be transmitted

uint8_t I2C_2_data_in[I2C_DATA_BUF_SIZE];
volatile int I2C_dbufin_head = 0;  // volatile because ISR can receive data
int I2C_dbufin_tail = 0;
volatile int I2C_rx_count = 0;   // number that has been read by I2C but not
                                 // read from data_in buffer
int volatile I2C_rx_count_to_read = 0;  // number of bytes to be received 

int I2C_txrx;  // corresponds to TRA bit in CR2, 0 receive, 1 transmit

// base address array for selecting base address from I2C number
// there is no I2C_0 but it is clearer to see I2CBase[1]
// and have it mean the base for I2C_1 than using I2CBase[0]
uint32_t *I2CBase[4] = { (uint32_t *) 0,
                         (uint32_t *) I2C1_Base,
                         (uint32_t *) I2C2_Base,
                         (uint32_t *) I2C3_Base};

//
//==============================================================================
//
//  Initialize I2C for the slaves on I2C 2, of the Large Display project
//
// device_address is for which slave is being initialized
// initializations may be differeent for each slave so the
// routine does one or the other depending

// slave: 24LC1025 EPROM
// slave: Touch Panel

// I2C spec has 3 defined speeds 100 Khz Standard
//                               400 Khz Fast               
//                               3.4 Mhz High-speed 
// Status: 
//    written, working


void i2c_2_initialize( uint8_t device_address )
{
  // disable all interrupts in the module
  I2C_All_Interrupts( 2, 0 );
  // turn off interrupts in the interrupt controller
  nvic_disableIRQ( I2C2_EV_INTR );  // events
  nvic_disableIRQ( I2C2_ER_INTR );  // errors     
 
  // these don't change with the slave
  // pg. 847 reference manual
  // changes to the clock setting must be done when the
  // module is not enabled, disable 
  I2C_Disable( 2 );

  // restart buffers
  I2C_dbufout_head = 0;
  I2C_dbufout_tail = 0;
  I2C_dbufin_head = 0;
  I2C_dbufin_tail = 0;    
  
  // program peripheral input clock
  // I2C2 is in APB1 = 42 Mhz
  I2C_SetFreq( 2, 42);
  // set device address 
  I2C_set_slave_ad( 2, I2C_Module2 );     

  // the rest may change, put in the setup per device
  // the initializations for each slave may be different!
  switch(device_address)
  {
  case I2C_eprom: // set up I2c for the EPROM
    {
      // turn off TouchPanel, no touches while using eeprom
      TouchPanel_disable();
      // configure clock control regs 42 Mhz = 23.8 nano seconds
      // 100 Khz clock = 5000 nanosec, divider should be 210, verified   
      I2C_SetClock( 2, 200000);
      // configure rise time reg
      I2C_SetRiseT( 2, 43 ); 

      i2c2_device_active = I2C_eprom;
      break;
    }
  case I2C_touch: // set up I2c for the touch panel
    {
      // configure clock control regs 42 Mhz = 23.8 nano seconds
      // 100 Khz clock = 5000 nanosec, divider should be 210, verified 
      I2C_SetClock( 2, 200000);
      // configure rise time reg
      I2C_SetRiseT( 2, 43 );    
      i2c2_device_active = I2C_touch;
      break;
    }
  default:
    {
      error_log( 3001 );
      break;  // bad device address, should not occur
    }
  }
  // enable the module 
  I2C_Enable( 2 );
  
  // enable all interrupts in the module
  I2C_All_Interrupts( 2, 1 );
  // turn on interrupts in the interrupt controller
  nvic_enableIRQ( I2C2_EV_INTR );  // events
  nvic_enableIRQ( I2C2_ER_INTR );  // errors    
}
// =============================================================================
//
//  Begin a data transmission
//   The ISR will continue until done
//   the send_count is the number of data to be sent including the
//   control byte command.
//
// device_address
// Eprom block 0 write  0xA0
// Eprom block 1 write  0xA8
// 
// Touch Panel read  0x29
// Touch Panel write 0x28         

// Status: written, working 

void i2c_start_tx( uint8_t I2Cn, uint8_t control_byte, int send_count )
{
  // make 16bit pointer to status 1 for I2Cn
  // uint16_t volatile *SR1 = (uint16_t *)
  //                           ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_SR1);
  // uint16_t stat1; 
  I2C_tx_count = send_count;
  I2C_txrx = 1;
  I2C_2_txrx_complete = 0;
  // just to be sure, clear LSB to indicate transmission
  control_byte &= 0xFE;
  TxRxDevAddress = control_byte;
  // set start bit
  I2C_set_start( I2Cn );
  // setting start will activate the ISR SB interrupt
  // the interrupt will put the control byte into the data register
}

/* not being used, may be needed for touch screen
// =============================================================================
//
//  Begin receiving 
//   The ISR will continue until done
//
// device_address
// Eprom block 0 read  0xA1
// Eprom block 1 read  0xA9
// Touch Panel read  0x29
// Touch Panel write 0x28  

   rx is started by using tx start,
   Put a number in bytes to read.
   with bytes to read, the ISR switches to read after tx start.



void i2c_start_rx( uint8_t I2Cn, uint8_t control_byte )
{
   // module address was set in init
   // Reference manual pg 851

   // set LSB to indicate reception
   control_byte |= 0x01;
   // set acknowledge bit
   I2C_set_ack( I2Cn, 1 );
 
   I2C_txrx = 1;  // for transmitting control and address
   I2C_tx_count = 1;
   
   // set start bit
   I2C_set_start( I2Cn );   
   // send slave device address and control bits
   I2C_send_byte( I2Cn, control_byte );   
   I2C_rx_count = 0;
   I2C_txrx = 0;
}
*/

uint8_t TxRxDevAddress;

// =============================================================================
//
// ISR handler for i2c event interrupts
//
// handle events
// SB       start bit sent    
// ADDR     address sent
// ADD10    10bit header sent  (not used)
// STOPF    stop received
// BTF      data byte transfer finished
// RxNE     receive buffer not empty
// TxE      transmit buffer empty
//
//  overrides the __weak declaration in cstartupCSI
//
//  This is for I2C 2 only
//
//  see ref man pg. 852-3
//
// todo: if it is needed, put in a cpu receiving as a slave section
//       if(stat1 & 0x0010); // STOPF stop detected (slave)
//
// History 02/14/2018 moved rx counter up, placed noack immediately after read
//         04/12/2019 changed timing of delay at the end of transmission
//
// Status: written, working for EEPROM,
//                  working with occasional resets for Touch panel

volatile uint8_t I2C_2_txrx_complete = 0;

void I2C2_EV_Handler( void )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
  // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint8_t *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);
  //  pointer to control register
  //uint16_t volatile *CR1 = (uint16_t *)
  //                          ((uint32_t) I2CBase[2] + (uint32_t) I2C_CR1);
  
  uint16_t stat1;
  uint16_t stat2;
  // uint8_t temp;
  
  // disable event interrupts
  nvic_disableIRQ( I2C2_EV_INTR );
  I2C_All_Interrupts( 2, 0 );  
  // get status
  stat1 = *SR1;
  
  if(I2C_txrx == 1 )
  { // ==========================================
    // master transmitting
  
    // SB start
    if(stat1 & I2C_START) 
    { 
       // start is done. 
       // be sure control byte is set for write, clear low bit
       TxRxDevAddress &= 0xFE;
       // write control byte to DR
       *DR = TxRxDevAddress;
       I2C_tx_count--;   
    }
    // ADDR Address sent
    if(stat1 & I2C_ADDR) 
    { 
      // control byte is done. send first data
      // SR1 has been read, read SR2
       stat2 = *SR2;
       // if there is additional data send it
       if(I2C_tx_count > 0)
       {
         *DR = I2C_2_data_out[I2C_dbufout_tail++];
         if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;
         I2C_tx_count--;        
       }
    }
    // TxE will be set after second data and thereafter      
    if(stat1 & I2C_TXE) // TxE Data register empty
    { 
      // send any additional data
       if(I2C_tx_count > 0)
       {
         *DR = I2C_2_data_out[I2C_dbufout_tail++];
         if(I2C_dbufout_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufout_tail = 0;
         I2C_tx_count--;
       }
       else // tx_count = 0, no additional data stop transmitting,
       {
          // if the write was not an address for a following read
          if(I2C_rx_count_to_read == 0)
          {
             // set the stop condition, the write is finished  
             I2C_set_stop( 2 );
             // need to wait until the last bits of data have been sent.
             // TXE occurs when the data is transferred to the shift register
             // Stop needs to be set before the end of the byte.
             // When stop is set, BTF and TXE are cleared so can't be used to
             // set the txrx complete flag
             // need a delay to allow the transfer to finish.
             // 50, 60, 70, 80 work, 40 failed
             // 200 Khz clock, 5 usec per bit, 8 bits plus ack, 45 usec
             delay(50);  
             I2C_2_txrx_complete = 1;
          }
          else  // there is data to be read
          {
            I2C_txrx = 0;  // set to receive
            I2C_set_start( 2 ); // new start command 
          }  
       }
    }
    if(stat1 & I2C_BTF)  // byte transfer finished
    {
      // if the data sent was the last, I2C_tx_count == 0
      if( I2C_tx_count == 0 )
      {
        // if the write was not an address for a following read
        // set the stop condition
        if(I2C_rx_count_to_read == 0)
        {
          // no delay after BTF
           I2C_set_stop( 2 );
           I2C_2_txrx_complete = 1;
        }
        else  // there is data to be read
        {
          I2C_txrx = 0;  // set to receive
          I2C_set_start( 2 ); // new start command
        }
      }      
    }
  }

  // ==========================================
  else     // master receiving  
  {
    // SB start
    if(stat1 & I2C_START) 
    { 
       // set to ack
       //if( I2C_rx_count_to_read > 1) I2C_set_ack( 2, 1 ); 
       //else I2C_set_ack( 2, 0 );
       I2C_set_ack( 2, 1 ); 
       // send control for read to DR
       TxRxDevAddress |= 0x01;
       *DR = TxRxDevAddress;
    }
    // ADDR Address sent
    if(stat1 & I2C_ADDR) 
    { 
      // control byte is done read data
      // SR1 has been read, read SR2 to clear addr
      stat2 = *SR2;
      if(stat2);  // to eliminate compiler warning
      // although the manual says noack must be set before clearing
      // ADDR by reading sr1 & sr2. moving this up caused read to hang
      //  ack was set at start clear if this is the last byte
      //if( I2C_rx_count_to_read < 2) I2C_set_ack( 2, 0 );
    }

    // debugging: moved counter up, placed noack immediately after read
    
    // debugging: rxe OR'd with BTF didn't work,
    //            no timeouts, but data wrong, missing bytes
    
    // RxNE Data register not empty
    if( stat1 & I2C_RXNE ) 
    {
      I2C_rx_count_to_read--;
      // read, store in buffer  
      I2C_2_data_in[I2C_dbufin_head++] = *DR;
      // if there is only one left to read, set to NOACK
      if( I2C_rx_count_to_read == 1) I2C_set_ack( 2, 0 );      
      
      if(I2C_dbufin_head > I2C_DATA_BUF_SIZE -1) I2C_dbufin_head = 0;
      I2C_rx_count++;
      // check for buffer wrap error
      if( I2C_dbufin_head == I2C_dbufin_tail) error_log( 3021 );
      // may need to increase buffer size 
    }
    // if done reading
    if(I2C_rx_count_to_read == 0)   
    {
       // set the stop bit
       I2C_set_stop( 2 );
       I2C_2_txrx_complete = 1;
    }
    else  // debugging, removing this causes timeout errors. why?
    {
      I2C_2_txrx_complete = 0;      
    }
  }
   // renable event interrupts
  I2C_All_Interrupts( 2, 1 ); 
  nvic_enableIRQ( I2C2_EV_INTR );
 
}

// =============================================================================
//
// ISR handler for i2c error interrupts
//
//  overrides the __weak declaration in cstartupCSI
//
// handle errors
// BERR     bus error
// ARLO     arbitration loss
// AF       acknowledge failure
// OVR      overrun underrun
// PECERR   PEC error
// TIMEOUT  timeout/ Tlow error
// SMBALERT SMBus alert
//
//  STATUS: written, working, No real error handling here yet,
//          an error is reported to the log, and the error is cleared.
//
void I2C2_ER_Handler( void )
{
  uint32_t stat1;
  // make 16 bit pointer to status 1 for I2C 2
  uint16_t *SR1 = (uint16_t *)
                  ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);  
  
  // disable error interrupts
  nvic_disableIRQ( I2C2_ER_INTR );  
  // get status, use 16bit local to read 
  stat1 = I2C_Get_status1( 2 );
 
  // multiple error bits may be set, check each
  
  if(stat1 & 0x0100) // BERR  Bus Error
  {
    // clear error
    *SR1 &= ~0x0100;
    error_log( 1402 );
  }
  if(stat1 & 0x0200) // ARLO Arbitration Lost
  {
    // clear error
    *SR1 &= ~0x0200;
    error_log( 1403 );
  }
  if(stat1 & I2C_AF) // AF Acknowledge Failure
  {
    // clear error
    *SR1 &= ~I2C_AF;    
    error_log( 1404 );
  }
  if(stat1 & 0x0800) // OVR Underrun / Overrun 
  {
    // clear error
    *SR1 &= ~0x0800;  
    error_log( 1405 );
  }
  if(stat1 & 0x1000) // PECERR Packet error checking Error
  {
    // clear error
    *SR1 &= ~0x1000; 
    error_log( 1406 );
  }
  if(stat1 & 0x4000) // TIMEOUT       
  {
    // clear error
    *SR1 &= ~0x4000;     
    error_log( 1407 );
  }
  if(stat1 & 0x8000) // SMBALERT
  {
    // clear error
    *SR1 &= ~0x8000;     
    error_log( 1408 );
  }
  // renable error interrupts
  nvic_enableIRQ( I2C2_ER_INTR );    

}
// =============================================================================
//
// Writes a slave register with the given data value
//   a two byte write to a slave that has control registers
//
//   device address is probably different for control registers and data
//   just for I2C 2 because there's only 1 buffer (for now)
//
// Status: written, untested, not used with the EEPROM,
//     but maybe it will be used with the touch screen
// 
void I2C_SlaveRegWrite( uint8_t control_byte, uint8_t regAddr, uint8_t regData)
{
  TxRxDevAddress = control_byte;
  // put data in the dataout buffer
  I2C_put_dataout( regAddr );  
  I2C_put_dataout( regData );  
  // start tx, I2C 2, device, 2 bytes
  i2c_start_tx( 2, control_byte, 2 );
}

// =============================================================================
//   get count of data in I2C data_in buffer
//
// History 11/29/2018
//
// Status: written to fix no data received at startup, but
//  a touch panel interrupt is sent.

// if count is zero, no data

int dbufin_readynum ( void )
{
  int count;
  count = I2C_dbufin_head - I2C_dbufin_tail;
  if (count < 0 ) count += I2C_DATA_BUF_SIZE;
  return count;
}

// =============================================================================
//
//  Puts data into dataout buffer
//   updates buffer index
//   just for I2C 2 because there's only 1 buffer (for now)
//
// Status: written, working
//
void I2C_put_dataout( uint8_t Data )
{
  // put data in the data_out buffer  
  I2C_2_data_out[I2C_dbufout_head++] = Data;
  if(I2C_dbufout_head > I2C_DATA_BUF_SIZE -1) I2C_dbufout_head = 0;
  if( I2C_dbufout_head == I2C_dbufout_tail) error_log( 3021 );
}
// =============================================================================
//
//  Gets data from the data_in buffer
//   updates buffer index
//   just for I2C 2 because there's only 1 buffer (for now)
//
// Status: written, working
//
uint8_t I2C_get_datain( void )
{
  uint8_t gdata;
  // if it's empty
  if( I2C_dbufin_tail == I2C_dbufin_head )
  {
     error_log( 3022 );
     return 0;
  }
  else
  // get data in the buffer  
  {
    gdata = I2C_2_data_in[I2C_dbufin_tail++];
    if(I2C_dbufin_tail > I2C_DATA_BUF_SIZE -1) I2C_dbufin_tail = 0;

    return gdata;
  }
}
// ============================================================================
//
//  Enable or disable Event Interrupts
//
//  This enables the interrupts in the I2C module, It does not activate
//  the NVIC and turn on the interrupt 
//
// call with the number of I2C unit 
// and on = 1 off = 0
// example: I2C_Event_Interrupts( 2, 1 ); turn on event interrupts for I2C 2
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working
//

void I2C_Event_Interrupts(uint8_t I2Cn, int onoff)
{
  // make control 2 register pointer for I2Cn
  I2C_CR2_reg volatile *CR2 = (I2C_CR2_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t)I2C_CR2);
  CR2->ITEVTEN = onoff;
}
// ============================================================================
//
//  Enable or disable Error Interrupts
//
//  This enables the interrupts in the I2C module, It does not activate
//  the NVIC or turn on the interrupt. 
//
// call with the number of I2C unit 
// and on = 1 off = 0
// example: I2C_Error_Interrupts( 2, 1 ); turn on error interrupts for I2C 2
//
// Status: written, working
//

void I2C_Error_Interrupts(uint8_t I2Cn, int onoff)
{
  // make control 2 register pointer for I2Cn
  I2C_CR2_reg volatile *CR2 = (I2C_CR2_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR2);
  CR2->ITERREN = onoff;
    
}
// ============================================================================
//
//  Enable or disable Buffer Interrupts
//
//  This enables the interrupts in the I2C module, It does not activate
//  the NVIC and turn on the interrupt 
//
// call with the number of I2C unit 
// and on = 1 off = 0
// example: I2C_Buffer_Interrupts( 2, 1 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working
//
void I2C_Buffer_Interrupts(uint8_t I2Cn, int onoff)
{
    // make control 2 register pointer for I2Cn
    I2C_CR2_reg volatile *CR2 = (I2C_CR2_reg *)
                                ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR2);
    CR2->ITBUFEN = onoff;
}
// ============================================================================
//
//  Enable or disable all I2C Interrupts
//
//  This enables the interrupts in the I2C module, It does not activate
//  the NVIC and turn on the interrupt 
//
// call with the number of I2C unit 
// and on = 1 off =0
// example: I2C_Event_Interrupts( 2, 1 );
//
// Status: written, working
//
void I2C_All_Interrupts(uint8_t I2Cn, int onoff)
{
  // make control 2 register pointer for I2Cn
  I2C_CR2_reg volatile *CR2 = (I2C_CR2_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR2);
  // make a copy of the register
  I2C_CR2_reg regtemp;
  regtemp = *CR2;
  // change the bits
  regtemp.ITEVTEN = onoff; 
  regtemp.ITERREN = onoff;
  regtemp.ITBUFEN = onoff;
  // write back to the register
  *CR2 = regtemp;
}
/*
// ============================================================================
//
//  I2C Master Bus Busy 
//
// call with the number for the I2C unit 
// example: bbusy = I2C_Busy( 2 );
//
// Status: written, working, not being used
//
unsigned int I2C_Busy(uint8_t I2Cn)
{
    // make status 2 register pointer for I2Cn
    I2C_SR2_reg volatile *SR2 = (I2C_SR2_reg *)
                                ((uint32_t)I2CBase[I2Cn] + (uint32_t)I2C_SR2);
    return ( SR2->BUSY );
}
*/
// ============================================================================
//
//  Set I2C clock
//
// call with the number for the I2C unit 
// and the frequency in Mhz of the APB clock that supplies the I2C module
// frequency is 2 Mhz min to 50 Mhz max or APB1
// example: I2C_SetFreq( 2 , 42 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, not being used
//
void I2C_SetFreq(uint8_t I2Cn, uint8_t freq)
{
  // make control 2 register pointer for I2Cn
  I2C_CR2_reg volatile *CR2 = (I2C_CR2_reg *)
                              ((uint32_t)I2CBase[I2Cn] +(uint32_t)I2C_CR2);
  CR2->FREQ = freq;
}
// ============================================================================
//
//  I2C Enable 
//
// call with the number for the I2C unit 
// example: I2C_Enable( 2 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working
//
void I2C_Enable(uint8_t I2Cn )
{
  // make control 1 register pointer for I2Cn
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t)I2C_CR1);
  CR1->PEN = 1;
}
// ============================================================================
//
//  I2C Disable 
//
// call with the number for the I2C unit 
// example: I2C_Disable( 2 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working
//
void I2C_Disable(uint8_t I2Cn )
{
  // make control 1 register pointer for I2Cn
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t)I2CBase[I2Cn] + (uint32_t)I2C_CR1);
  CR1->PEN = 0;
}
// ============================================================================
//
//  I2C set clock rate
//
// call with the number for the I2C unit 
// and the clock frequency desired
// divider number set is min 4, max 4095 12bits
// and the divider is on time, half the total cycle time
// example: I2C_SetClock( 2 , 100000 );  sets IC2 2 to 100 Khz
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working
//
void I2C_SetClock(uint8_t I2Cn, int clkfreq)
{
  int divider;
  divider = CLCK_APB1 / (2 * clkfreq ); 
  // make CCR register pointer for I2Cn
  I2C_CCR_reg volatile *CCR = (I2C_CCR_reg *)
                               ((uint32_t) I2CBase[I2Cn] + (uint32_t)I2C_CCR);
  CCR->CDIV = divider;
}
// ============================================================================
//
//  I2C set max rise time
//
// call with the number for the I2C unit 
// ref manual pg. 873
// rise time max for standard I2C is 1000 nsec
// Tpclk = 1 / 21 Mhz  47.6 nsec. 1000/47.6 = 21 add 1 = 22 
// example: I2C_SetRiseT( 2 , 22);  
//
// Status: written, working
//
void I2C_SetRiseT(uint8_t I2Cn, int trise )
{
  // Trise register only has one value in it, no need for a structure
  // make standard 32 bit pointer to TRISE for I2Cn
  uint32_t volatile *TRISE = (uint32_t *)
                             ((uint32_t)I2CBase[I2Cn] + (uint32_t)I2C_TRISE);
  // write to the register
  *TRISE = trise; 
}
// ============================================================================
//
//  I2C software reset
//
// use this to break out of a hung condition, 
// all initializations will have to be redone 
// call with the number for the I2C unit 
// example: I2C_SWReset( 2, 1 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, untested

void I2C_SWReset(uint8_t I2Cn, int onoff  )
{
  // make control 1 register pointer for I2Cn
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t)I2CBase[I2Cn] + (uint32_t)I2C_CR1);
  CR1->SWRST = onoff;
}
// ============================================================================
//
//  I2C get status register 1 - into a register structure
//
// read the status register
//  returns a register type that has status register 1 data in the
//  register structre 
//
// call with the number for the I2C unit 
// example: I2C_Get_STAT1( 2 );
//
// History
//    02/13/2019 changed to a direct return of the register.
//
// Status: written, working

I2C_SR1_reg I2C_Get_STAT1(uint8_t I2Cn )
{
  // make register pointer to status 1 for I2Cn
  I2C_SR1_reg volatile *SR1 = (I2C_SR1_reg *)
                              ((uint32_t)I2CBase[I2Cn] + (uint32_t)I2C_SR1);
  /*
  // make a copy of register 1
  I2C_SR1_reg regtemp;
  regtemp = *SR1;
  return regtemp;
  */
  return *SR1;
}
// ============================================================================
//
//  I2C get status register 1 - into a 16 bit word
//
// read the status register
//  returns the 16 bit integer status register 1 
//
// call with the number for the I2C unit 
// example: I2C_Get_status1( 2 );
//
// Status: written, working

uint16_t I2C_Get_status1(uint8_t I2Cn )
{
  // make 16bit pointer to status 1 for I2Cn
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_SR1);
  return *SR1;
}
// ============================================================================
//
//  I2C get status register 2
//
// read the statu register
//  returns a register type that has status register 2 data
//
// call with the number for the I2C unit 
// example: I2C_Get_STAT2( 2  );
//
// Status: written, not used

I2C_SR2_reg I2C_Get_STAT2(uint8_t I2Cn )
{
  // make register pointer to status 2 for I2Cn
  I2C_SR2_reg volatile *SR2 = (I2C_SR2_reg *) 
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_SR2);
  return *SR2;
}
// ============================================================================
//
//  I2C get status register 2
//
// read the status register
//  returns the 16 bit integer status register 2
//
// call with the number for the I2C unit 
// example: I2C_Get_status2( 2 );
//
// Status: written, working

uint16_t I2C_Get_status2(uint8_t I2Cn )
{
  // make 16bit pointer to status 2 for I2Cn
  uint16_t volatile *SR2 = (uint16_t *)
                           ((uint32_t) I2CBase[I2Cn] + (uint32_t)I2C_SR2);
  return *SR2;
}
/* not being used
// ============================================================================
//
//  I2C write data
//
// single byte write to the data register
//
// call with the number for the I2C unit and the byte to send 
// example: I2C_send( 2, data );
//
// Status: written, working, not being used

void I2C_send_byte(uint8_t I2Cn, uint8_t wdata)
{
    // make register pointer to data register
    uint8_t volatile *DR = (uint8_t *)
                           ((uint32_t) I2CBase[I2Cn] + (uint32_t)I2C_DR);
    *DR = wdata;
}

// ============================================================================
//
//  I2C read data
//
// single byte read from the data register
//
// call with the number for the I2C unit 
// example: data = I2C_read_byte( 2 );
//
// Status: written, working, not being used

uint8_t I2C_read_byte(uint8_t I2Cn )
{
    // make register pointer to data register
    uint8_t volatile *DR = (uint8_t *)
                           ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_DR);
    return *DR;
}
*/
// ============================================================================
//
//  I2C set start condition
//
// call with the number for the I2C unit 
// example: I2C_set_start( 2 );  
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working

void I2C_set_start(uint8_t I2Cn )
{
  // make register pointer to control register
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR1);
  CR1->START = 1;
}
// ============================================================================
//
//  I2C clear start condition
//
// call with the number for the I2C unit 
// example: I2C_start( 2 );
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working, not being used

void I2C_clear_start(uint8_t I2Cn )
{
  // make register pointer to control register
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR1);
  CR1->START = 0;
}

// ============================================================================
//
//  I2C set stop condition
//
// call with the number for the I2C unit 
// example: I2C_stop( 2 ); 
//
// History
//    02/13/2019 changed to a direct write to the register.
//
// Status: written, working

void I2C_set_stop(uint8_t I2Cn )
{
  // make register pointer to control register
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR1);
  // change to a direct write
  CR1->STOP = 1;
}

// ============================================================================
//
//  I2C set 7 bit slave address for the module
//
// call with the number for the I2C unit
// and 7 bit device address address with LSB, 0 for 7 bit 
// This is the slave address for the module if another master on
// the bus were to send. If the module is the only master this is unused
// example: I2C_set_address( 2, 0x01 );
//
// Status: written, untested, cpu is master, probably not going to use this

void I2C_set_slave_ad(uint8_t I2Cn, uint8_t address )
{
  // make register pointer to OAR register
  uint16_t volatile *OAR1 = (uint16_t *)
                            ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_OAR1);
  // make a copy of the register
  uint16_t regtemp;
  regtemp = *OAR1;
  // mask off existing bits and set new
  regtemp = (regtemp & 0xFF01) | (address << 1) ;
  // write back to the register    
  *OAR1 = regtemp;
}
// ============================================================================
//
//  I2C set / clear acknowledge bit
//
// call with the number for the I2C unit, and 0 or 1
// example: I2C_set_ack( 2, 1 );
//
// Status: written, working
//
// History
//    02/13/2019 changed to a direct write to the register.


void I2C_set_ack(uint8_t I2Cn, uint8_t onoff )
{
  // make register pointer to control register
  I2C_CR1_reg volatile *CR1 = (I2C_CR1_reg *)
                              ((uint32_t) I2CBase[I2Cn] + (uint32_t) I2C_CR1);
  // direct write
  CR1->ACK = onoff;

}

// ============================================================================
//
// read device id
//
//    This is from the I2C standard. It is an optional part of the standard.
//    see pg 21 of I2C standard UM10204.
//  should receive 24 bits, 3 bytes
//     12 bits mfr
//      9 bits part id
//      3 bits revision
//
//    The Eprom did not respond, it was not mentioned in the datasheet.
//   
// Status: written, working but failed for EEPROM, no acknowledge, no TxE

uint8_t device_ID[3];

void I2C_read_device_id(uint8_t device_address )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
  // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint16_t stat1; 
  uint16_t stat2;
  
  uint8_t volatile *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);

  // disble all interrupts in the module
  I2C_All_Interrupts( 2, 0 );
  
  // turn off interrupts in the interrupt controller
  nvic_disableIRQ( I2C2_EV_INTR );  // events
  nvic_disableIRQ( I2C2_ER_INTR );  // errors    
  
  // set start bit 
  I2C_set_start( 2 );

  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) );

  // send I2C device id address command set to write 1111 1000
  stat1 = *SR1;   
  *DR = 0xF8;

  // TxE will not be set after the control byte  
  // wait for ADDR
  //do  stat1 = *SR1; while( !(stat1 & I2C_ADDR) );
  // wait for AF
  do stat1 = *SR1; while( !(stat1 & I2C_AF));
  *SR1 &= ~I2C_AF;  // clear AF 
  stat1 = *SR1;
  stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
  if(stat2);  // to eliminate compiler warning  
  // send I2C device address 
  *DR = device_address;
 
  // wait until TXE bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_TXE) );
  // wait for BTF
  do stat1 = *SR1; while( !(stat1 & I2C_BTF) );
  
  // send a restart
  I2C_set_start( 2 );
  // wait until start bit is set
  do stat1 = *SR1; while( !(stat1 & I2C_START) );
 
  // send I2C device id address command set to read 1111 1001
  *DR = 0xF9;
  
  // wait for ADDR
  // do stat1 = *SR1; while( !(stat1 & I2C_ADDR) );
  
  stat1 = *SR1;
  stat2 = *SR2;   // read SR2 after a good address, to clear ADDR

  // Set to ACK  
  I2C_set_ack( 2, 1 );
  
  // wait for RxNE 
  do stat1 = *SR1; while( !(stat1 & I2C_RXNE) ); 
  // read ID
  device_ID[0] = I2C_read_byte( 2 );
  // wait for RxNE 
  do stat1 = *SR1; while( !(stat1 & I2C_RXNE) ); 
  // read ID
  device_ID[1] = I2C_read_byte( 2 );
    // Set to NOACK  
  I2C_set_ack( 2, 1 );
  
  // wait for RxNE 
  do stat1 = *SR1; while( !(stat1 & I2C_RXNE) ); 
  // read ID
  device_ID[2] = I2C_read_byte( 2 );

  I2C_set_stop( 2 );

}
// =============================================================================
//
//  test I2C for device address of touch screen
//
// found A0 A1 A8 A9 all eeprom
//       28 29 touch screen
//   no response on F8 so the chips do not implement I2C device ID
//
uint8_t active_dev[10];

uint8_t I2C_TS_test( void )
{
  // make 16bit pointer to status 1 for I2C 2
  uint16_t volatile *SR1 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR1);
  // make 16bit pointer to status 2 for I2C 2
  uint16_t volatile *SR2 = (uint16_t *)
                            ((uint32_t) I2CBase[2] + (uint32_t) I2C_SR2);
  uint16_t stat1; 
  uint16_t stat2;
  
  uint8_t volatile *DR = (uint8_t *)
                     ((uint32_t) I2CBase[2] + (uint32_t)I2C_DR);

  // disble all interrupts in the module
  I2C_All_Interrupts( 2, 0 );
  
  *GPIO_F_BSRR = BIT02_ON;
  
  uint8_t control_byte = 0;
  int i;
  int devcount = 0;
  int timeout = 2000;
  
  for(i = 0;i < 256; i++)
  {
    timeout = 2000;
    control_byte = i;
    // set start bit 
    I2C_set_start( 2 );
    // wait until start bit is set
    do stat1 = *SR1; while( !(stat1 & I2C_START) );
    stat1 = *SR1;   
    *DR = control_byte;
    // wait for ADDR
    stat1 = *SR1;
    while( !(stat1 & I2C_ADDR) )
    {
      if( !timeout-- ) break;
      stat1 = *SR1;
      if(stat1 & I2C_ADDR)
      {
         stat2 = *SR2;   // read SR2 after a good address, to clear ADDR
         if(stat2);  // to eliminate compiler warning      
         active_dev[devcount++] = control_byte;
      }
    }
    // send stop
    I2C_set_stop( 2 );
    delay(90);
  }
  return devcount;
}

/***************************** End Of File ***********************************/
