//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Dec 20, 2016             Design Documents                           CSI_i2c.h
//
//			header file for I2C 
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  12/20/2016 started, originaly written for OMAP L-138
//  03/22/2018 revising for use with STM32F405
//
// Status
//  writing revision
//
//******************************************************************************

#ifndef CSI_I2C_H_
#define CSI_I2C_H_

#include <stdint.h>

extern int i2c2_device_active;

// I2C addresses are 7 bit with LSB indicating Tx or RX
// I2C device addresses for Large Display
#define	I2C_eprom		0xA0    // 24LC1025 EPROM (160)
// Eprom start bit, 1010 control code,
// block bit 0 or 1 512K bits per block, 64K bytes 
// two bits address, hard wired on board, 00
// 1 bit read/write 1 = read 0 = write
// b10100000  block 0 write  0xA0
// b10110000  block 1 write  0XB0
// b10100001  block 0 read   0xA1
// b10110001  block 1 read   0xB1

// I2C device addresses for touch panel
#define	I2C_touch		0x28   // touch panel (40) 
// I2C device address for I2C module 2 in the CPU
// 3 I2C modules in the CPU each could have a different address
// ony used if module is used as a slave. 
#define	I2C_Module2        0x02

// status register bits
#define  I2C_START  0x0001
#define  I2C_ADDR   0x0002
#define  I2C_BTF    0x0004
#define  I2C_RXNE   0x0040
#define  I2C_TXE    0x0080
#define  I2C_AF     0x0400
// for control reg 1
#define  I2C_POS    0x0800

// create structures for the commonly used registers
//  the same structures can be used to make structure variables for each I2C
//  16 bit registers

// Control register 1
typedef struct
{
  unsigned int PEN	        :1;  // peripheral enable (PE) was used in a uart define
  unsigned int SMBUS            :1;  // SMBus mode
  unsigned int res1		:1;  // reserved 0
  unsigned int SMBTYPE  	:1;  // SMBus type
  unsigned int ENARP		:1;  // ARP enable
  unsigned int ENPEC		:1;  // PEC enable
  unsigned int ENGC		:1;  // General call enable
  unsigned int NOSTRETCH	:1;  // clock stretching disable
  unsigned int START		:1;  // Start generation
  unsigned int STOP		:1;  // Stop generation
  unsigned int ACK		:1;  // acknowledge enable
  unsigned int POS		:1;  // PEC position
  unsigned int PEC		:1;  // packet error checking
  unsigned int ALERT		:1;  // SMBus alert
  unsigned int res2		:1;  // reserved 0  
  unsigned int SWRST		:1;  // software reset
} I2C_CR1_reg;

// Control register 2
typedef struct
{
  unsigned int FREQ	        :6;  // clock frequency 2-50 Mhz
  unsigned int res1             :2;  // reserved 0
  unsigned int ITERREN		:1;  // error interrupt enable
  unsigned int ITEVTEN		:1;  // event interrupt enable
  unsigned int ITBUFEN		:1;  // buffer interrupt enable
  unsigned int DMAEN		:1;  // DMA requests enable
  unsigned int LAST             :1;  // DMA last transfer
  unsigned int res2             :3;  // reserrved 0
} I2C_CR2_reg;

// status register 1
typedef struct
{
  unsigned int SB	        :1;  // start bit
  unsigned int ADDR             :1;  // address sent, matched
  unsigned int BTF		:1;  // byte transfer finished
  unsigned int ADD10		:1;  // 10 bit header sent
  unsigned int STOPF		:1;  // stop detected
  unsigned int res1		:1;  // reserved, should be 0
  unsigned int RxNE  	        :1;  // Data register not empty RX
  unsigned int TxE		:1;  // Data register empty TX
  unsigned int BERR		:1;  // Bus error
  unsigned int ARLO		:1;  // arbitration lost
  unsigned int AF		:1;  // acknowledge fail
  unsigned int OVR		:1;  // overrun underrun
  unsigned int PECERR		:1;  // PEC error in reception
  unsigned int res2		:1;  // reserved, should be 0
  unsigned int TIMEOUT          :1;  // timeout error
  unsigned int SMBALERT         :1;  // SMBus alert
} I2C_SR1_reg;

// status register 2
typedef struct
{
  unsigned int MSL	        :1;  // master slave
  unsigned int BUSY             :1;  // bus busy
  unsigned int TRA		:1;  // transmitter receiver
  unsigned int res1		:1;  // reserved, should be 0
  unsigned int GENCALL		:1;  // general call address
  unsigned int SMBDEFAULT	:1;  // SMB default device address
  unsigned int SMBHOST 	        :1;  // SMB host 
  unsigned int DUALF		:1;  // dual flag
  unsigned int PEC		:8;  // packet error check
} I2C_SR2_reg;

// clock control register
typedef struct
{
  unsigned int CDIV	        :12; // clock rate divider
  unsigned int res1		:2;  // reserved, should be 0
  unsigned int DUTY		:1;  // duty cycle
  unsigned int FS       	:1;  // fast or slow
} I2C_CCR_reg;

// control byte address and r/w bit
extern uint8_t TxRxDevAddress;
extern volatile uint8_t I2C_2_txrx_complete;
// number of bytes to be received 
extern volatile int I2C_rx_count_to_read;
extern uint32_t *I2CBase[4];

// function prototypes
void i2c_2_initialize( uint8_t device_address );

void I2C2_EV_Handler( void );
void I2C2_ER_Handler( void );

void I2C_Event_Interrupts( uint8_t, int );
void I2C_Error_Interrupts( uint8_t, int );
void I2C_Buffer_Interrupts( uint8_t, int );
void I2C_All_Interrupts(uint8_t, int );
unsigned int I2C_Busy( uint8_t );
void I2C_SetFreq( uint8_t, uint8_t );
void I2C_Enable( uint8_t );
void I2C_Disable( uint8_t );
void I2C_SetClock( uint8_t, int );
void I2C_SetRiseT( uint8_t, int );
void I2C_SWReset( uint8_t, int );
uint32_t I2C_Get_STATUS( uint8_t );
I2C_SR1_reg I2C_Get_STAT1( uint8_t );
uint16_t I2C_Get_status1( uint8_t );
I2C_SR2_reg I2C_Get_STAT2( uint8_t );
uint16_t I2C_Get_status2( uint8_t );
uint8_t I2C_read_byte( uint8_t );
void I2C_send_byte( uint8_t, uint8_t );
void I2C_set_start( uint8_t );
void I2C_clear_start(uint8_t I2Cn );
void I2C_set_stop( uint8_t );
void i2c_start_tx( uint8_t, uint8_t, int );
void i2c_start_rx( uint8_t I2Cn, uint8_t device_address );
void I2C_set_slave_ad( uint8_t, uint8_t );
void I2C_set_ack( uint8_t, uint8_t );
void I2C_SlaveRegWrite( uint8_t, uint8_t, uint8_t);

// debugging
void I2C_SlaveTestWrite( uint8_t control_byte, uint8_t block,
                       uint8_t AddrHigh, uint8_t AddrLow,uint8_t MemData);
void I2C_SlaveTestRead( uint8_t control_byte, uint8_t block,
                       uint8_t AddrHigh, uint8_t AddrLow );
// void I2C_read_device_id( uint8_t device_address );
uint8_t I2C_TS_test( void );

// for eprom
void I2C_SlaveMemRead( uint8_t control_byte, uint8_t block, uint8_t AddrHigh,
                       uint8_t AddrLow, uint16_t count_to_read);
void I2C_SlaveMemWrite( uint8_t control_byte, uint8_t block,
                       uint8_t AddrHigh, uint8_t AddrLow,uint8_t MemData);
void I2C_MultipleWrite( uint8_t control_byte, uint8_t block,
                        uint16_t num_bytes);
void I2C_2_EEPROM_busypoll( uint8_t control_byte );

// input and output buffers
// buffer size for one full page write (128) plus 2 starting address bytes 
#define I2C_DATA_BUF_SIZE 130
extern uint8_t I2C_2_data_out[I2C_DATA_BUF_SIZE];
extern int I2C_dbufout_head;
extern volatile int I2C_dbufout_tail;
extern volatile int I2C_tx_count; 

extern uint8_t I2C_2_data_in[I2C_DATA_BUF_SIZE];
extern volatile int I2C_dbufin_head; 
extern int I2C_dbufin_tail;
extern volatile int I2C_rx_count; 

extern int I2C_txrx;

uint8_t I2C_get_datain( void );
void I2C_put_dataout( uint8_t );
int dbufin_readynum ( void );

// base addresses for I2C modules
#define I2C1_Base       0x40005400      // I2C 1
#define I2C2_Base       0x40005800      // I2C 2
#define I2C3_Base       0x40005C00      // I2C 3

// I2C register offsets

#define I2C_CR1 	0x00000000	// control register 1
#define I2C_CR2	        0x00000004	// control register 2
#define I2C_OAR1	0x00000008	// own address 1
#define I2C_OAR2	0x0000000C	// own address 2
#define I2C_DR  	0x00000010	// data register
#define I2C_SR1 	0x00000014	// status register 1
#define I2C_SR2 	0x00000018	// status register 2
#define I2C_CCR 	0x0000001C	// clock control
#define I2C_TRISE	0x00000020	// maximum rise time
#define I2C_FLTR	0x00000024	// Filter

#endif /* CSI_I2C_H_ */
