//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 20, 2018 		Design Documents      	            initialize.c
//
//                      Initializes Large Display project
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  03/20/2018 started
//  01/15/2019 combined with FT857 start
//  05/31/2019 made conditional compile for FT857D
//  07/22/2019 added radio type config flag   
//
// Status
//==============================================================================
// NAME OF ROUTINE                      Function
//
// initialize                           Initialize BFD
//
//==============================================================================
#include "CSI_initialize.h"
#include <string.h>
#include "CSI_INITIALIZE_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_GPIO.h"
#include "CSI_i2c.h"
#include "CSI_spi.h"
#include "CSI_ExtFlash.h"
#include "CSI_UART.h"
#include "CSI_eeprom.h"
#include "CSI_interrupt.h"
#include "CSI_COLOR.h"
#include "CSI_timer_two.h"
#include "CSI_TIMERS.h"
#include "CSI_TouchPanel.h"
#include "CSI_secure.h"
#include "CSI_SWITCHES.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_FEATURES.h"
#include "CSI_SCREENS.h"
#include "CSI_RADIO.h"
#include "CSI_SSD1963.h"
#include "CSI_FT857D.h"

extern uint8_t startup_mode;
uint8_t FT857D_radio_active = 0;

extern void init_systick(void);
// =============================================================================
//
//  Initializations after startup, called in Main before everything else
//
void initialize( void )
{
  uint8_t sw_held = 0;
  uint8_t restore = 0;
  uint8_t e_radio_type = 0;
  uint8_t start_radio_type = 0;
  unsigned int start_time;
  
  // set CPU clocks done in startupSTM32F4xx.c
  // set all registers for startup
  //initialize_registers();
  initialize_registers_fast();  // working
  
  // setup SYStick
  init_systick();
  // setup i2c to get radio type
  // I2C - EEPROM
  // eprom will be read at startup then i2c will switch to touch panel. 
  i2c_2_initialize( I2C_eprom );
  start_radio_type = current_radio_type; // as set by main
  // read eeprom to determine what radio the panel is set for
  e_radio_type = load_radio_type();
  // if a bad read from eeprom set to what main has
  if( e_radio_type > 3 )
  {
    e_radio_type = start_radio_type;   // set to what main had      
  }
  // reading eeprom also gets the config flag
  if( panel_config_flag == 0xCF )
  {
    current_radio_type = e_radio_type;
  }
  else
  {
    // eeprom has never been written, or eeprom error
    // write current type and flag to eeprom
    save_radio_type( current_radio_type );    
  }
 
  // These are the radio types. 
  // 0:  CS800D or CS800 only attached to primary port
  // 1:  FT857D only attached to primary port
  // 2:  FT857D attached to UART2 port 
  switch(current_radio_type) {
    case 0:
      {
        FT857D_radio_active = 0;
        break;
      }
    #ifdef radio_FT857D  
    case 1:
    case 2:
      FT857D_radio_active = 1;
      load_FT857_info();
      active_screen_num = 1;    // set screen to prevent drawing until ready
      FT857D_startup_timer = 50;
      break;
    #endif /* FT857D */    
    default: break;
  }
 
  // If current radio type is 0, then only the CS800D/CS800 exists and we do not
  // have to do anything special. If the current radio type is 1 or 2, then the
  // FT857 exists.
#ifdef radio_FT857D      
  if (current_radio_type == 2)
  {
    uart2_FT857D_initialize();
  }
#endif /* FT857D */   
  //
  // ***************************************************************************
  // CS800D and CS800 radio start-up. For this start-up, P1 through P2 held down
  // when power is first applied has the following meaning:
  //   P1 - Firmware programming of the radio
  //   P2 - RCDB (reset parameters)
  //   P3 - reload the default settings in the CS-BFD
  // The only time we do not execute this routine if the FT857D is attached.
  // ***************************************************************************
  if( current_radio_type == 0 )
  {
    // read switches 3 times, 1 msec debounce
    // use 2 bits per switch
    // 1/18/2019 switch read routines changed to: 0 is unpressed 1 is pressed
    int i; 
    for(i = 0; i < 3; i++)
    {
      // P1
      if(read_switch_s3()) sw_held += 0x01;  //3
      //P2
      if(read_switch_s4()) sw_held += 0x04;  //12
      //P3
      if(read_switch_s5()) sw_held += 0x10;  //48
      
      // wait between reading but not at the end
      if(i < 2) msec_delay(1);
    }  
    // handshake for radio before starting 
    switch(sw_held)
    {
    case 3:  // p1 held
      {
        // set both uart lines to output
        set_UART1TXRX_to_GPIO();     
        
        // set both TX and RX low.
        *GPIO_B_BSRR = 0x00400000;
        *GPIO_A_BSRR = 0x04000000;
        startup_mode = 1;
        break;
      }
    case 12:   // p2 held  
      {
         // switch to uart
        set_UART1TXRX_to_UART(); 
        // initialize uart 
        uart1_initialize( CS800D );
    
        // fixed delay 0.5 sec
        msec_delay( 480 );
        RCDB_start();
        startup_mode = 2;
        // prevent keys from being sent
        deactivate_all_keypress();      
        break;
      }
    case 48:   // p3 held
      {  
        // set to restore default settings
        restore = 1;
        // no break here, falls through to normal start
      }
    default: // nothing pressed, anything else, normal start 
      {
        start_time = millisec;
        // wait for low pulse from radio
        while(get_UART1RX() == 1)
        {
          // if we don't get one proceed anyway
          if(millisec > start_time + 3000) break;
        }
        // switch to uart
        set_UART1TXRX_to_UART(); 
        // initialize uart 
        uart1_initialize( CS800D );      
      
        startup_mode = 0;      
        break;
      }
    }
  }
#ifdef radio_FT857D
  // ***************************************************************************
  // FT857D radio start-up. For this start-up, current_radio_type is defined as:
  //  1 - uses primary port (USART 1)
  //  2 - uses secondary port (USART 2)
  // We already initialized USART2 at the begining of the initialized routine
  // ***************************************************************************
  if( FT857D_radio_active == 1) // FT857D
  {
    if (current_radio_type == 1)
    {
      set_UART1TXRX_to_UART(); 
      uart1_initialize( FT857D );
    }
    /*  debug, remove, for later with both radios connected
    //
    // If we got here and current_radio_type is 2, then we also have to initialize
    // the CS-BFD for the CS800D and CS800
    //

    if (current_radio_type == 2)
    {
      // set_UART1TXRX_to_UART(); 
      // uart1_CS800D_initialize();      
      startup_mode = 0;         
    }
    */
  }
 
#endif /* FT857D */   
  // ========================================
  // setup software interrupts
  SWI_config( 5 );  // use EXTI line 5 for 1 msec timer
  SWI_config( 6 );  // use EXTI line 6 for 10 msec timer
  // configure timer 2 for 10 millisecond
  init_10milli_timer();
  // turn on timer 2
  timer_10milli_enable( 1 ); 
  
  // for debugging: fill color data with FF to simulate a blank chip
  // erase_colorset();
  // ============================
  // load colors from eeprom
  load_colorset();    
  // if the palette is bad,
  if(color_check( current_palette ) == 0 )
  {
    load_palette( (RGB_color*) default_palette );
    restore_default_colors(); // palette loaded, need color settings.
  }
  // restore button was held
  // get default settings
  if(restore == 1 )
  {
    // set up standard colors
    load_palette( (RGB_color*) default_palette );
    restore_default_colors();
    if( current_radio_type == 0 ) // CS800D
    {
      // clear and save all favorites
      clear_all_favorites( favorite_zone );
      save_all_favorites( favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO );
      clear_all_favorites( favorite_channel );
      save_all_favorites( favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO );   
    }
  }
  if( current_radio_type == 0 ) // CS800D
  {
    // load stored function buttons
    FB_button_read();
    // also store to current
    memcpy( &current_key_functions, &stored_key_functions, FB_num_keys);
    get_button_assign();

    // read favorite zones
    read_favorites( favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO );
    // channels 
    read_favorites( favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO );  
    // check favorites, if the data has never been written, write blank data
    if(favorite_zone[0].name[0] == 0xFF)
    {
      clear_all_favorites( favorite_zone );
      save_all_favorites( favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO );
      clear_all_favorites( favorite_channel );
      save_all_favorites( favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO );
    }
  }
  // ===========================
  // LCD controller
  ssd1963_cg_initialize();
  
  // SPI 1 external Flash
  spi1_init();
  // reset flash in case it was hung
  flash_reset();
  
  // read eeropm and flash, needs to be after spi and I2c
  // check for stored chip id's
  
  // TODO: What should happen if it doesn't match??
  check_securepage2();  
  
  // init timer 3 for mc59  
  init_10micro_timer(); 
  
  // after reading EEPROM switch to touch panel
  // setup for touch panel:
  i2c_2_initialize( I2C_touch );
  // initialize touch panel
  TouchPanel_init();

}