//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
///
// Mar 22, 2018			Design Documents                 CSI_interrupt.c 
// 
//       Interrupt configutration, enable set and clear routines
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  03/22/2018 revised from CS7000 for STM32F7
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  NVIC_enableIRQ              Enables a specific NVIC interrupt
//  NVIC_disableIRQ             Disables a specific NVIC interrupt
//  SWI_config                  Configure a softare interrupt line
//  trigger_enable              Set a trigger direction for EXTI line
//  EXTI9_5_Handler             ISR for EXTI lines 5-9
//  EXTI15_10_Handler           ISR for EXTI lines 10-15
//  EXTI2_Handler               ISR for EXTI 2 GPIO pin for touch screen
//  set_interrupt_priority      sets priority for an interrupt

// Status: written, not used
//
//  enable_interrupts           Enables interrupts (inline ASM)  
//  disable_interrupts          Disables interrupts (inline ASM)
//  SWIE_activate               Activate a software interrupt or event
//  SWE_config                  Configure a softare event line
//  trigger_disable             Clear trigger setting for EXTI line
//
//******************************************************************************
#include "CSI_interrupt.h"
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include <stdint.h>
#include "CSI_ERROR_LOG.h"
#include "CSI_i2c.h"
#include "CSI_TouchPanel.h"

// Software interrupts and event lines
// 0-22 
// see Reference manual pg. 372-375 pg.381
// EXTI lines 0,1,2,3,4 have dedicated interrupt numbers
// and are the lines used for GPIO generated interrupts 
//                                                     interrupt 
// EXTI line  0 is on                                      6
// EXTI line  1 is on                                      7
// EXTI line  2 is on                                      8
// EXTI line  3 is on                                      9
// EXTI line  4 is on                                     10 
// EXTI lines 5-9 are on                                  23
// EXTI lines 10-15 are on                                40
// EXTI line  16 is on PVD output                          1 
// EXTI line  17 is on RTC alarm event                    41       
// EXTI line  18 is on USB OTG FS wakeup event            42 
// EXTI line  19 is on Ethernet wakeup event              61
// EXTI line  20 is on USB OTG HS wakeup event            76
// EXTI line  21 is on PVD RTC tamper and timestamp event  2
// EXTI line  22 is on PVD RTC wakeup event                3

uint8_t exti_Line[23] = { 6, 7, 8, 9, 10, 23, 23, 23, 23, 23,
  40, 40, 40, 40, 40, 40, 1, 41, 42, 61, 76, 2, 3};

//==============================================================================
//
// enable interrupts
//  assembler single op code to enable interrupts
//
inline void enable_interrupts(void)
{
 asm volatile ("cpsie i");
}

//
// disable interrupts
//  assembler single op code to disable interrupts
//
inline void disable_interrupts(void)
{
 asm volatile ("cpsid i");
 
}
//==============================================================================
//
// Turn on a specific interrupt using the 
//  NVIC Interrupt Set Enable Register (ISER)

inline void nvic_enableIRQ(uint8_t IRQn)
{
  uint8_t irq_reg;
  
  irq_reg = IRQn >> 5;    // 32 irq per register
  // calculate register address starting at base ISER0
  // each register is 4 bytes so add 4 x register number to base address
  uint32_t volatile *NVIC_ISERx = (uint32_t *) (NVIC_ISER0_ADDRESS + (irq_reg << 2));
  // set the bit
  *NVIC_ISERx = (uint32_t) (1 << (IRQn & 0x1F)); // & 0x1F removes register part
}
//==============================================================================
//
// Turn off a specific interrupt using the
//  NVIC Interrupt Clear Enable Register (ICER)

inline void nvic_disableIRQ(uint8_t IRQn)
{
  uint8_t irq_reg;
  
  irq_reg = IRQn >> 5;    // 32 irq per register
  // calculate register address starting at base ICER0
  // each register is 4 bytes so add 4 to register address per register
  uint32_t volatile *NVIC_ICERx = (uint32_t *) NVIC_ICER0_ADDRESS + (irq_reg << 2);
  // set the bit
  *NVIC_ICERx = (uint32_t) (1 << (IRQn & 0x1F));
}
//==============================================================================
//
//  configure a software interrupt
//    on a line 0 - 22

void SWI_config( uint8_t linenum )
{
  // for an interrupt configure mask bits EXTI_IMR 
  uint32_t volatile *EXTI_IMR = (uint32_t *) EXTI_IMR_ADDRESS;
  *EXTI_IMR |= 1 << linenum;
  // enable the interrupt in the controller
  nvic_enableIRQ( exti_Line[linenum] );  
}
//==============================================================================
//
//  configure a software event
//    on a line 0 - 22
//
void SWE_config( uint8_t linenum )
{
  // for an event configure mask bits EXTI_EMR
  uint32_t volatile *EXTI_EMR = (uint32_t *) EXTI_EMR_ADDRESS;
  *EXTI_EMR |= 1 << linenum;
  // enable the interrupt in the controller
  nvic_enableIRQ( exti_Line[linenum] );  
}
//==============================================================================
// 
//  Activate a software interrupt or event 
//  Interrupts/Events are 0 to 22
//
void SWIE_activate ( uint8_t linenum )
{
  uint32_t volatile *EXTI_SWIER = (uint32_t *) EXTI_SWIER_ADDRESS;
  *EXTI_SWIER |= 1 << linenum;
}
//==============================================================================
//
//  set trigger direction for EXTI 
//    on a line 0 - 22,  risefall = 0 for rise 1 for fall
//
//    trigger on a rise or trirgger on a fall.
//    possibly trigger on both. Call twice or modify routine.

void trigger_enable( uint8_t linenum, uint8_t risefall )
{
  if(risefall)
  {
    // for an falling trigger 
    uint32_t volatile *EXTI_FTSR = (uint32_t *) EXTI_FTSR_ADDRESS;
    *EXTI_FTSR |= 1 << linenum;
  }
  else  // rising
  {
    uint32_t volatile *EXTI_RTSR = (uint32_t *) EXTI_RTSR_ADDRESS;
    *EXTI_RTSR |= 1 << linenum;    
  }
}
//==============================================================================
//
//   clear trigger direction for EXTI 
//    on a line 0 - 22,  risefall = 0 for rise 1 for fall 

void trigger_disable( uint8_t linenum, uint8_t risefall )
{
  if(risefall)
  {
    // for an falling trigger 
    uint32_t volatile *EXTI_FTSR = (uint32_t *) EXTI_FTSR_ADDRESS;
    *EXTI_FTSR &= ~(1 << linenum);
  }
  else  // rising
  {
    uint32_t volatile *EXTI_RTSR = (uint32_t *) EXTI_RTSR_ADDRESS;
    *EXTI_RTSR &= ~(1 << linenum);    
  }
}
//==============================================================================// 
//  ISR for interrupts or events on lines  5-9
//    overrides __weak definition in cstartup
//
// Status
//   Written, testing, working.  Touch Screen interrupt hangs

// debugging:
// this interrupt has higher priority than the I2C interrupt.
// waiting here for the i2c to get data from the touch panel isn't working.
// Somewhere it said that the touch data is not ready the instant the interrupt
// is generated. Users must wait for the data buffer ready bit to be set.
//
//  07/03/2018   Moved priority of this down below I2C2
//  03/01/2019   Changed to read additional touches if count is > 1
//               The first touch gets read when status is read

// The touch panel sends interrupts at startup with no touches.
// Then it times out and stops sending them.
// The touch data buffer is not ready, so polling the status register for
// the data buffer ready flag doesn't succeed, wastes time.

uint8_t volatile TS_touch_ready;
int TS_waitcount = 0;
extern unsigned long millisec;

void EXTI9_5_Handler( void )
{

  // int msec;
  // disable 
  nvic_disableIRQ( EXTI9_5_INTR );
  
  // read Pending register to determine which line(s) were triggered
  uint32_t volatile *EXTI_PR = (uint32_t *) EXTI_PR_ADDRESS;
  uint32_t pending = *EXTI_PR;
  
  // handle any lines triggered
  // line 5
  if(pending & BIT05_MASK )
  {
    // line 5 is 1 msec interrupt from systick
    // ???
    // process_keypad();
    // clear the triggered line, writing a 1 clears it
    *EXTI_PR |= BIT05_MASK; 
  }
  // ========================================
  // line 6
  // Touch screen handler
  // F 6 is Touch screen interrupt
  if(pending & BIT06_MASK )
  {
    // skip touches that occur while using the eeprom
    if( i2c2_device_active == I2C_touch)
    {
      // === test, only start if txrx in complete
      if( I2C_2_txrx_complete == 1 )
      {
        // debug problem:
        // using read multi instead of read register
        // not sure why reading the single register routine fails.
        TouchPanel_read_multi( GT911_Status, 8 );
        //TS_status = TouchPanel_read_register( GT911_Status );

        // need to wait for communication to complete
        // program is done after sending start, but ISR continues
        //log_event( "get TS status" );

        if( TouchPanel_waitfor_I2C_RX() == 1 )
        {
          // get data of first touch and status from rx buffer
          TS_status = TouchPanel_read_touch_buffer();
          
          // read all touch data into the touch array
          TouchPanel_get_touch_data( TS_status ); 
          // write to clear data buffer reg
          TouchPanel_write ( GT911_Status, 0 );  
          
          //log_event( "TS clear buffer" );
          TouchPanel_waitfor_I2C_TX();
          //while(!I2C_2_txrx_complete);
        }
        else
        {
          // read did not complete set status to 0, no touches 
           TS_status = 0;
        }
      }
    }
    // clear the triggered line
    *EXTI_PR |= BIT06_MASK;
  }
  // End of Touch panel handler
  //  ====================================
  // line 7
  if(pending & BIT07_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT07_MASK;
  }
  // line 8
  if(pending & BIT08_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT08_MASK;
  }
  // line 9
  if(pending & BIT09_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT09_MASK;
  }
    // enable 
  nvic_enableIRQ( EXTI9_5_INTR );
}
//==============================================================================
// 
//  ISR for interrupts or events on lines  10 - 15
//    overrides __weak definition in startupCSI
//
void EXTI15_10_Handler( void )
{
  // read Pending register to determine which line(s) were triggered
  uint32_t volatile *EXTI_PR = (uint32_t *) EXTI_PR_ADDRESS;
  uint32_t pending = *EXTI_PR;
  
  // line 10
  if(pending & BIT10_MASK )
  {
    // handle it...
    // clear the triggered line, writing a 1 clears it
    *EXTI_PR |= BIT10_MASK;
  }
  // line 11
  if(pending & BIT11_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT11_MASK;
  }
  // line 12
  if(pending & BIT12_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT12_MASK;
  }
  // line 13
  if(pending & BIT13_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT13_MASK;
  }    
  // line 14
  if(pending & BIT14_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT14_MASK;
  }
  // line 15
  if(pending & BIT15_MASK )
  {
    // handle it...
    // clear the triggered line
    *EXTI_PR |= BIT15_MASK;
  }    
}
//==============================================================================
// 
//  ISR for EXTI 2 interrupt
//    GPIO mapped F6 for touch panel
//    overrides __weak definition in startupCSI
//
// not is use, touch was not on exti2

void EXTI2_Handler( void )
{
    uint32_t volatile *EXTI_PR = (uint32_t *) EXTI_PR_ADDRESS;
  // disable 
    nvic_disableIRQ( EXTI2_INTR );
  // check to see which line triggerd the interrupt
  // read pending register EXTI_PR lines 0 to 22 
  // (not needed there's only one)
  
  // Handle touch panel interrupt
  // if I2C not set for touchpanel (not needed I2C is good for both)
  //if( i2c0_device_ready != I2C_touch) i2c_2_initialize( I2C_touch );

  // get info from touch panel
  // more to write...
  // read_touch1 puts data for first touch point in the dataout reg
  // need to handle:
  //   debounce?
  //   more than one touch point
  //   touch start and end.
  //   double tap
  //   drag and drop
  //
  //  TouchPanel_read_touch1();
  // clear pending (by writing a 1)
  *EXTI_PR |= (1 << 6);
  
  // re-enable
  nvic_enableIRQ( EXTI2_INTR ); 
 
}
// =============================================================================
//
//  Set priority for an interrupt
//
//  an IPR register controls 4 interrupt priorities
//  priority registers are byte adressable
//  8 bits per interrupt, but only the high 4 bits are used, 16 levels
//  the lower the value the higher the priority 0-255
//  all the interrupts have default priorities
//  see the table in the refernce manual pg 1745
 
void set_interrupt_priority( uint8_t int_num, uint8_t priority)
{
  // make pointer to 8 bit portion of 32 bit register  (4 interrupt per 32 bit reg)
  uint8_t volatile *NVIC_IPRx = (uint8_t *) NVIC_IPR0_ADDRESS + int_num;
    
  *NVIC_IPRx = priority & 0xF0; // set only the high bits
  
}



