// =============================================================================
//
//      system_stm32f4xxx.c 
//      Modified for Large Panel Display
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
//  The original STM system file was for the STM evaluation board.
//  This has been edited to work with the large panel display.
//  Original code has been commented out to show changes. 
//  Startup_clocks() and SetSysClock() are merged into SystemInit.
//
//  SetSysClock() section contains three while wait loops that have no timeout.
//  These could cause a program hang, but they are in the original file.
//  They have not caused program hangs, yet.
// 
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  SysTick_Handler                 ISR system tick handler
//  SystemInit                      initializes the CPU system

// Status: written, not tested, not used
//
// SystemInit_ExtMemCtl             initialize external memory (for reference) 

//==============================================================================
#include <stdint.h>
#include "CSI_REGISTERS.h"
#include "CSI_TIMERS.h"
#include "CSI_CORE_REGISTERS.h"
#include "CSI_SCREENS.h"
#include "CSI_FEATURES.h"
#include "CSI_PROCESS_CS800D.h"

/**
  ******************************************************************************
  * @file    system_stm32f4xx.c
  * @author  MCD Application Team
  * @version V1.0.2
  * @date    05-March-2012
  * @brief   CMSIS Cortex-M4 Device Peripheral Access Layer System Source File.
  *          This file contains the system clock configuration for STM32F4xx devices,
  *          and is generated by the clock configuration tool
  *          stm32f4xx_Clock_Configuration_V1.0.1.xls
  *             
  * 1.  This file provides two functions and one global variable to be called from 
  *     user application:
  *      - SystemInit(): Setups the system clock (System clock source, PLL Multiplier
  *                      and Divider factors, AHB/APBx prescalers and Flash settings),
  *                      depending on the configuration made in the clock xls tool. 
  *                      This function is called at startup just after reset and 
  *                      before branch to main program. This call is made inside
  *                      the "startup_stm32f4xx.s" file.
  *
  *      - SystemCoreClock variable: Contains the core clock (HCLK), it can be used
  *                                  by the user application to setup the SysTick 
  *                                  timer or configure other parameters.
  *                                     
  *      - SystemCoreClockUpdate(): Updates the variable SystemCoreClock and must
  *                                 be called whenever the core clock is changed
  *                                 during program execution.
  *
  * 2. After each device reset the HSI (16 MHz) is used as system clock source.
  *    Then SystemInit() function is called, in "startup_stm32f4xx.s" file, to
  *    configure the system clock before to branch to main program.
  *
  * 3. If the system clock source selected by user fails to startup, the SystemInit()
  *    function will do nothing and HSI still used as system clock source. User can 
  *    add some code to deal with this issue inside the SetSysClock() function.
  *
  * 4. The default value of HSE crystal is set to 25MHz, refer to "HSE_VALUE" define
  *    in "stm32f4xx.h" file. When HSE is used as system clock source, directly or
  *    through PLL, and you are using different crystal you have to adapt the HSE
  *    value to your own configuration.
  *
  * 5. This file configures the system clock as follows:
  *=============================================================================
  *=============================================================================
  *        Supported STM32F4xx device revision    | Rev A
  *-----------------------------------------------------------------------------
  *        System Clock source                    | PLL (HSE)
  *-----------------------------------------------------------------------------
  *        SYSCLK(Hz)                             | 168000000
  *-----------------------------------------------------------------------------
  *        HCLK(Hz)                               | 168000000
  *-----------------------------------------------------------------------------
  *        AHB Prescaler                          | 1
  *-----------------------------------------------------------------------------
  *        APB1 Prescaler                         | 4
  *-----------------------------------------------------------------------------
  *        APB2 Prescaler                         | 2
  *-----------------------------------------------------------------------------
  *        HSE Frequency(Hz)                      | 25000000
  *-----------------------------------------------------------------------------
  *        PLL_M                                  | 25
  *-----------------------------------------------------------------------------
  *        PLL_N                                  | 336
  *-----------------------------------------------------------------------------
  *        PLL_P                                  | 2
  *-----------------------------------------------------------------------------
  *        PLL_Q                                  | 7
  *-----------------------------------------------------------------------------
  *        PLLI2S_N                               | NA
  *-----------------------------------------------------------------------------
  *        PLLI2S_R                               | NA
  *-----------------------------------------------------------------------------
  *        I2S input clock                        | NA
  *-----------------------------------------------------------------------------
  *        VDD(V)                                 | 3.3
  *-----------------------------------------------------------------------------
  *        Main regulator output voltage          | Scale1 mode
  *-----------------------------------------------------------------------------
  *        Flash Latency(WS)                      | 5
  *-----------------------------------------------------------------------------
  *        Prefetch Buffer                        | OFF
  *-----------------------------------------------------------------------------
  *        Instruction cache                      | ON
  *-----------------------------------------------------------------------------
  *        Data cache                             | ON
  *-----------------------------------------------------------------------------
  *        Require 48MHz for USB OTG FS,          | Enabled
  *        SDIO and RNG clock                     |
  *-----------------------------------------------------------------------------
  *=============================================================================
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/** @addtogroup CMSIS
  * @{
  */

/** @addtogroup stm32f4xx_system
  * @{
  */  
  
/** @addtogroup STM32F4xx_System_Private_Includes
  * @{
  */

/* #include "stm32f4xx.h"  */

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_TypesDefinitions
  * @{
  */

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_Defines
  * @{
  */

/************************* Miscellaneous Configuration ************************/
/*!< Uncomment the following line if you need to use external SRAM mounted
     on STM324xG_EVAL board as data memory  */
/* #define DATA_IN_ExtSRAM */

/*!< Uncomment the following line if you need to relocate your vector Table in
     Internal SRAM. */
//#define VECT_TAB_SRAM
#define SRAM_BASE  0x20000000
//#define FLASH_BASE  0x08000000
#define VECT_TAB_OFFSET  0x00 /*!< Vector Table base offset field. 
                                   This value must be a multiple of 0x200. */
/******************************************************************************/

/************************* PLL Parameters *************************************/
// settings for the demo board

/* PLL_VCO = (HSE_VALUE or HSI_VALUE / PLL_M) * PLL_N */
// #define PLL_M      25
// #define PLL_N      336

/* SYSCLK = PLL_VCO / PLL_P */
//#define PLL_P      2

/* USB OTG FS, SDIO and RNG Clock =  PLL_VCO / PLLQ */
//#define PLL_Q      7

/******************************************************************************/

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_Macros
  * @{
  */

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_Variables
  * @{
  */

  uint32_t SystemCoreClock = 168000000;

  // __I uint8_t AHBPrescTable[16] = {0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 6, 7, 8, 9};

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_FunctionPrototypes
  * @{
  */

// static void SetSysClock(void);
//#ifdef DATA_IN_ExtSRAM
  //static void SystemInit_ExtMemCtl(void); 
//#endif /* DATA_IN_ExtSRAM */

/**
  * @}
  */

/** @addtogroup STM32F4xx_System_Private_Functions
  * @{
  */

/**
  * @brief  Setup the microcontroller system
  *         Initialize the Embedded Flash Interface, the PLL and update the 
  *         SystemFrequency variable.
  * @param  None
  * @retval None
  */
   
//extern void startup_clocks(void);  // not used

#define RCC_CR_HSEON        0x00010000    // bit 16
#define RCC_CR_HSERDY       0x00020000    // bit 17
#define RCC_APB1ENR_PWREN   0x10000000    // bit 28
#define PWR_CR_VOS          0x00004000    // bit 14
#define RCC_CR_PLLON        0x01000000    // bit 24
#define RCC_CR_PLLRDY       0x02000000    // bit 25

#define HSE_STARTUP_TIMEOUT 100000
__no_init uint32_t HSEStatus;  // no init so that variable is not zeroed

void SystemInit(void)
{
  // code for startup clocks placed here  
  // startup_clocks();
  /* FPU settings ------------------------------------------------------------*/
  //  __IOM uint32_t CPACR;  /*!< Offset: 0x088 (R/W)  Coprocessor Access Control Register */
  //  #if (__FPU_PRESENT == 1) && (__FPU_USED == 1)
  //    SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 Full Access */
  //  #endif
  uint32_t volatile *CPACR = (uint32_t *) CPACR_ADDRESS;
  *CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 Full Access */  
  
  // use direct registers instead of structures
  uint32_t volatile *RCC_CR         = (uint32_t *) RCC_CR_ADDRESS;
  uint32_t volatile *RCC_CFGR       = (uint32_t *) RCC_CFGR_ADDRESS;
  uint32_t volatile *RCC_PLLCFGR    = (uint32_t *) RCC_PLLCFGR_ADDRESS;
  uint32_t volatile *RCC_CIR        = (uint32_t *) RCC_CIR_ADDRESS;
  uint32_t volatile *RCC_APB1ENR    = (uint32_t *) RCC_APB1ENR_ADDRESS;
  uint32_t volatile *PWR_CR         = (uint32_t *) PWR_CR_ADDRESS;
  uint32_t volatile *FLASH_ACR      = (uint32_t *) FLASH_ACR_ADDRESS;
  
  /* Reset the RCC clock configuration to the default reset state ------------*/
  /* Set HSION bit */  
    *RCC_CR |= (uint32_t)0x00000001;
    //  RCC->CR |= (uint32_t)0x00000001;

  /* Reset CFGR register */
    *RCC_CFGR = 0x00000000;
   //  RCC->CFGR = 0x00000000;

  /* Reset HSEON, CSSON and PLLON bits */
    *RCC_CR &= (uint32_t)0xFEF6FFFF;
   //  RCC->CR &= (uint32_t)0xFEF6FFFF;

  /* Reset PLLCFGR register */
    *RCC_PLLCFGR = 0x24003010; 
   //  RCC->PLLCFGR = 0x24003010;

  /* Reset HSEBYP bit */
    *RCC_CR &= (uint32_t)0xFFFBFFFF;    // bit 18 cleared
//  RCC->CR &= (uint32_t)0xFFFBFFFF;

  /* Disable all interrupts */
    *RCC_CIR = 0x00000000;
//  RCC->CIR = 0x00000000;

//#ifdef DATA_IN_ExtSRAM
  //SystemInit_ExtMemCtl(); 
//#endif /* DATA_IN_ExtSRAM */
         
  /* Configure the System clock source, PLL Multiplier and Divider factors, 
     AHB/APBx prescalers and Flash settings ----------------------------------*/
//  setsysclock code placed here
//  SetSysClock(); 
// ====================================== 
  uint32_t StartUpCounter = 0;    
    
/* Enable HSE */
  *RCC_CR |= (uint32_t)RCC_CR_HSEON;
 
  /* Wait till HSE is ready and if Time out is reached exit */
  do
  {
    HSEStatus = *RCC_CR & RCC_CR_HSERDY;
    StartUpCounter++;
  } while((HSEStatus == 0) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

  if ((*RCC_CR & RCC_CR_HSERDY) != 0)  // RESET
  {
    HSEStatus = (uint32_t)0x01;
  }
  else
  {
    HSEStatus = (uint32_t)0x00;
  }

  if (HSEStatus == (uint32_t)0x01)
  {
    /* Select regulator voltage output Scale 1 mode, System frequency up to 168 MHz */
    
    *RCC_APB1ENR |= RCC_APB1ENR_PWREN;
    *PWR_CR |= PWR_CR_VOS;
    //*PWR_CR = PWR_CR_VOS; // tested dicect set. default is 0x4000 had 0xC000

    /* HCLK = SYSCLK / 1*/
    // RCC->CFGR |= RCC_CFGR_HPRE_DIV1;
    // #define  RCC_CFGR_HPRE_DIV1   ((uint32_t)0x00000000)   /*!< SYSCLK not divided */
    // RCC_CFGR was cleared to 0 above so no need to clear bits before settting
    // *RCC_CFGR |= (uint32_t) 0; // OR 0 doesn't do anything
    
    // RCC_CFGR is 0x00000000 on reset, no need to clear bits
    // #define  RCC_CFGR_PPRE2_DIV2  ((uint32_t)0x00008000)   /*!< HCLK divided by 2 */
    /* PCLK2 = HCLK / 2*/
    //RCC->CFGR |= RCC_CFGR_PPRE2_DIV2;  
    *RCC_CFGR |= (uint32_t)0x00008000;  // 0x100 <<13 divide by 2 84 Mhz high speed APB2

    // #define  RCC_CFGR_PPRE1_DIV4  ((uint32_t)0x00001400)   /*!< HCLK divided by 4 */
    /* PCLK1 = HCLK / 4*/
    //RCC->CFGR |= RCC_CFGR_PPRE1_DIV4; 
    *RCC_CFGR |= (uint32_t)0x00001400;  // 0x101 <<10  divide by 2 42Mhz low speed APB1
    
    // for testing clock out MO2 on pin C9
    // set clock out divider before turning on PLL
    // set to 111, divide by 5  0x38 (all bits 1 so no need to clear any)
     // *RCC_CFGR |= (uint32_t)0x38000000;  // 168/5 = 33.6 Mhz 
 
    // ============================================
    // for 8 Mhz external, prototype board
    // configure PLL
    // VCO = 8 Mhz * N/M = 336 Mhz ( max 432 Mhz ) OK
    // PLL = VCO/P = 168 Mhz  (max 168 Mhz ) OK
    // USB OTG FS = VCO/Q = 48 Mhz (required for USB) OK
    // default 0x24003010
    // Q = 7   0x07000000 bits 24-27
    // N = 168 0x00002A00 bits 6-14 (0xA8 << 6)
    // M = 4   0x00000004 bits 0-5  default sets bit4, clear
    // P = 2   0x00000000 bits 16-17 for 00 P = 2
    //         0x27002A04
    // ============================================
    // for 12 Mhz external, production board
    // configure PLL
    // VCO = 12 Mhz * N/M = 336 Mhz ( max 432 Mhz ) OK
    // PLL = VCO/P = 168 Mhz  (max 168 Mhz ) OK
    // USB OTG FS = VCO/Q = 48 Mhz (required for USB) OK
    // default 0x24003010
    // Q = 7   0x07000000 bits 24-27
    // N = 168 0x00002A00 bits 6-14 (0xA8 << 6)
    // M = 6   0x00000006 bits 0-5  default sets bit4,  clear
    // P = 2   0x00000000 bits 16-17 for 00 P = 2
    //         0x27002A06

    //
    //  need to set HSE as source bit 22 of cfgr is PLLsrce  0 HSI, 1 HSE    
    // set to use HSE for PLL 
    *RCC_PLLCFGR |= (uint32_t)0x00400000; 
    // then set dividers
    // *RCC_PLLCFGR = 0x27402A04; // 8Mhz working after setting flash latency
    *RCC_PLLCFGR = 0x27402A06; // for 12 Mhz external clock
    
    /* Configure the main PLL */
    //RCC->PLLCFGR = PLL_M | (PLL_N << 6) | (((PLL_P >> 1) -1) << 16) |
    //                (RCC_PLLCFGR_PLLSRC_HSE) | (PLL_Q << 24);

    /* Enable the main PLL */
    // RCC->CR |= RCC_CR_PLLON; 
    *RCC_CR |= RCC_CR_PLLON;    // bit 24
    
    /* Wait till the main PLL is ready */
    // dead wait loop here, should have a timeout and error
    while((*RCC_CR & RCC_CR_PLLRDY) == 0) ;
   
    /* Configure Flash prefetch, Instruction cache, Data cache and wait state */
    // FLASH->ACR = FLASH_ACR_ICEN |FLASH_ACR_DCEN |FLASH_ACR_LATENCY_5WS;
    // ICEN = bit09  DCEN = bit10  latency 5ws = 101 bits 0-2
    // flash interface register 0x40023C00 ACR is base + 0
    // FLASH_ACR default =0x00000000
    *FLASH_ACR |= 0x00000605;

    /* Select the main PLL as system clock source */
    //RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));  //  // 0x00000003 
    //RCC->CFGR |= RCC_CFGR_SW_PLL;  // 0x00000002 PLL
    *RCC_CFGR &= 0xFFFFFFFC; 
    *RCC_CFGR |= 0x00000002; 
    
    /* Wait till the main PLL is used as system clock source */
    // #define  RCC_CFGR_SWS       ((uint32_t)0x0000000C)        /*!< SWS[1:0] bits (System Clock Switch Status) */
    // #define  RCC_CFGR_SWS_PLL   ((uint32_t)0x00000008)        /*!< PLL used as system clock */
    // dead wait loop here, should have a timeout and error    
    while ((*RCC_CFGR & 0x0000000C ) != 0x00000008);

  }
  else
  { /* If HSE fails to start-up, the application will have wrong clock
         configuration. User can add here some code to deal with this error */
    // this section may not be working corretly
    
    // bit 22 of cfgr is PLLsrce  0 HSI, 1 HSE
    // set to use HSI for PLL 
    *RCC_PLLCFGR &= ~0x00400000;
    // change for HSI and increase M divider to 8
    *RCC_PLLCFGR = 0x27002A08;
    /* Enable the main PLL */
    // RCC->CR |= RCC_CR_PLLON; 
    *RCC_CR |= RCC_CR_PLLON;    // bit 24
    
    /* Wait till the main PLL is ready */
    // dead wait loop here, should have a timeout and error
    while((*RCC_CR & RCC_CR_PLLRDY) == 0);
   
    /* Configure Flash prefetch, Instruction cache, Data cache and wait state */
    // FLASH->ACR = FLASH_ACR_ICEN |FLASH_ACR_DCEN |FLASH_ACR_LATENCY_5WS;

    /* Select the main PLL as system clock source */
    //RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));  //  // 0x00000003 
    //RCC->CFGR |= RCC_CFGR_SW_PLL;  // 0x00000002 PLL
    *RCC_CFGR &= 0xFFFFFFFC; 
    *RCC_CFGR |= 0x00000002; 
    
    /* Wait till the main PLL is used as system clock source */
    // #define  RCC_CFGR_SWS       ((uint32_t)0x0000000C)        /*!< SWS[1:0] bits (System Clock Switch Status) */
    // #define  RCC_CFGR_SWS_PLL   ((uint32_t)0x00000008)        /*!< PLL used as system clock */
    // dead wait loop here, should have a timeout and error    
    while ((*RCC_CFGR & 0x0000000C ) != 0x00000008);

    //===============    
  }
    /* Configure the Vector Table location add offset address ------------------*/
   //uint32_t volatile *VTOR = (uint32_t *) VTOR_ADDRESS;
    
   //#ifdef VECT_TAB_SRAM
   //*VTOR = SRAM_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal SRAM */
   //#else
   //*VTOR = FLASH_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal FLASH */
   //#endif

}
// =============================================================================
//
//
// clean version
/*

void SystemInit(void)
{
  uint32_t volatile *CPACR = (uint32_t *) CPACR_ADDRESS;
  // *CPACR |= ((3UL << 10*2)|(3UL << 11*2));  // set CP10 and CP11 Full Access 
  
  uint32_t volatile *RCC_CR         = (uint32_t *) RCC_CR_ADDRESS;
  uint32_t volatile *RCC_CFGR       = (uint32_t *) RCC_CFGR_ADDRESS;
  uint32_t volatile *RCC_PLLCFGR    = (uint32_t *) RCC_PLLCFGR_ADDRESS;
  uint32_t volatile *RCC_CIR        = (uint32_t *) RCC_CIR_ADDRESS;
  uint32_t volatile *RCC_APB1ENR    = (uint32_t *) RCC_APB1ENR_ADDRESS;
  uint32_t volatile *PWR_CR         = (uint32_t *) PWR_CR_ADDRESS;
  uint32_t volatile *FLASH_ACR      = (uint32_t *) FLASH_ACR_ADDRESS;
  
  // Reset registers
  *RCC_CR |= (uint32_t)0x00000001;
  *RCC_CFGR = 0x00000000;
  *RCC_CR &= (uint32_t)0xFEF6FFFF;
  *RCC_PLLCFGR = 0x24003010; 
  *RCC_CR &= (uint32_t)0xFFFBFFFF;    // bit 18 cleared
  // Disable all interrupts 
  *RCC_CIR = 0x00000000;
         
  // Configure the System clock source
  
  uint32_t StartUpCounter = 0;    
  // Enable HSE 
  *RCC_CR |= (uint32_t)RCC_CR_HSEON;
 
  // Wait till HSE is ready, if time out is reached exit 
  do
  {
    HSEStatus = *RCC_CR & RCC_CR_HSERDY;
    StartUpCounter++;
  } while((HSEStatus == 0) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

  if ((*RCC_CR & RCC_CR_HSERDY) != 0)  // RESET
  {
    HSEStatus = (uint32_t)0x01;
  }
  else
  {
    HSEStatus = (uint32_t)0x00;
  }

  if (HSEStatus == (uint32_t)0x01)
  {
    // Select regulator voltage output Scale 1 mode,
    // System frequency up to 168 MHz
    *RCC_APB1ENR |= RCC_APB1ENR_PWREN;
    *PWR_CR |= PWR_CR_VOS;
    *RCC_CFGR |= (uint32_t)0x00008000;  // 0x100 <<13 divide by 2 84 Mhz high speed APB2
    *RCC_CFGR |= (uint32_t)0x00001400;  // 0x101 <<10  divide by 2 42Mhz low speed APB1
    // configure PLL
    // set to use HSE for PLL 
    *RCC_PLLCFGR |= (uint32_t)0x00400000; 
    
    // ============================================
    // for 12 Mhz external, production board
    // VCO = 12 Mhz * N/M = 336 Mhz ( max 432 Mhz ) OK
    // PLL = VCO/P = 168 Mhz  (max 168 Mhz ) OK
    // USB OTG FS = VCO/Q = 48 Mhz (required for USB) OK
    // default 0x24003010
    // Q = 7   0x07000000 bits 24-27
    // N = 168 0x00002A00 bits 6-14 (0xA8 << 6)
    // M = 6   0x00000006 bits 0-5  default sets bit4,  clear
    // P = 2   0x00000000 bits 16-17 for 00 P = 2
    //         0x27002A06
    // then set dividers
    *RCC_PLLCFGR = 0x27402A06; 
    // Enable the main PLL 
    *RCC_CR |= RCC_CR_PLLON;    // bit 24
    
    // Wait till the main PLL is ready 
    // dead wait loop here, should have a timeout and error
    while((*RCC_CR & RCC_CR_PLLRDY) == 0) ;
   
    // Configure Flash prefetch, Instruction cache, Data cache and wait state 
    // FLASH_ACR default =0x00000000
    *FLASH_ACR |= 0x00000605;

    // Select the main PLL as system clock source 
    *RCC_CFGR &= 0xFFFFFFFC; 
    *RCC_CFGR |= 0x00000002; 
    
    // Wait till the main PLL is used as system clock source
    // dead wait loop here, should have a timeout and error    
    while ((*RCC_CFGR & 0x0000000C ) != 0x00000008);

  }
  else
  { // If HSE fails to start-up, the application will have wrong clock
    //     configuration. User can add here some code to deal with this error 
    // this section may not be working corretly
    
    // bit 22 of cfgr is PLLsrce  0 HSI, 1 HSE
    // set to use HSI for PLL 
    *RCC_PLLCFGR &= ~0x00400000;
    // change for HSI and increase M divider to 8
    *RCC_PLLCFGR = 0x27002A08;
    // Enable the main PLL 
    *RCC_CR |= RCC_CR_PLLON;    // bit 24
    
    // Wait till the main PLL is ready 
    // dead wait loop here, should have a timeout and error
    while((*RCC_CR & RCC_CR_PLLRDY) == 0);
   
    // Configure Flash prefetch, Instruction cache, Data cache and wait state 
    *FLASH_ACR |= 0x00000605;

    // Select the main PLL as system clock source 
    *RCC_CFGR &= 0xFFFFFFFC; 
    *RCC_CFGR |= 0x00000002; 
    
    // Wait till the main PLL is used as system clock source 
    // dead wait loop here, should have a timeout and error    
    while ((*RCC_CFGR & 0x0000000C ) != 0x00000008);
  }
}
*/

// =============================================================================
/**
  * @brief  Setup the external memory controller. Called in startup_stm32f4xx.s 
  *          before jump to __main
  * @param  None
  * @retval None
  */ 
#ifdef DATA_IN_ExtSRAM
/**
  * @brief  Setup the external memory controller.
  *         Called in startup_stm32f4xx.s before jump to main.
  *         This function configures the external SRAM mounted on STM324xG_EVAL board
  *         This SRAM will be used as program data memory (including heap and stack).
  * @param  None
  * @retval None
  */
//==============================================================================
// this is an evaluation board program.
// It's not being used. Useful for reference

void SystemInit_ExtMemCtl(void)
{
/*-- GPIOs Configuration -----------------------------------------------------*/
/*
 +-------------------+--------------------+------------------+------------------+
 +                       SRAM pins assignment                                   +
 +-------------------+--------------------+------------------+------------------+
 | PD0  <-> FSMC_D2  | PE0  <-> FSMC_NBL0 | PF0  <-> FSMC_A0 | PG0 <-> FSMC_A10 | 
 | PD1  <-> FSMC_D3  | PE1  <-> FSMC_NBL1 | PF1  <-> FSMC_A1 | PG1 <-> FSMC_A11 | 
 | PD4  <-> FSMC_NOE | PE3  <-> FSMC_A19  | PF2  <-> FSMC_A2 | PG2 <-> FSMC_A12 | 
 | PD5  <-> FSMC_NWE | PE4  <-> FSMC_A20  | PF3  <-> FSMC_A3 | PG3 <-> FSMC_A13 | 
 | PD8  <-> FSMC_D13 | PE7  <-> FSMC_D4   | PF4  <-> FSMC_A4 | PG4 <-> FSMC_A14 | 
 | PD9  <-> FSMC_D14 | PE8  <-> FSMC_D5   | PF5  <-> FSMC_A5 | PG5 <-> FSMC_A15 | 
 | PD10 <-> FSMC_D15 | PE9  <-> FSMC_D6   | PF12 <-> FSMC_A6 | PG9 <-> FSMC_NE2 | 
 | PD11 <-> FSMC_A16 | PE10 <-> FSMC_D7   | PF13 <-> FSMC_A7 |------------------+
 | PD12 <-> FSMC_A17 | PE11 <-> FSMC_D8   | PF14 <-> FSMC_A8 | 
 | PD13 <-> FSMC_A18 | PE12 <-> FSMC_D9   | PF15 <-> FSMC_A9 | 
 | PD14 <-> FSMC_D0  | PE13 <-> FSMC_D10  |------------------+
 | PD15 <-> FSMC_D1  | PE14 <-> FSMC_D11  |
 |                   | PE15 <-> FSMC_D12  |
 +-------------------+--------------------+
*/
   /* Enable GPIOD, GPIOE, GPIOF and GPIOG interface clock */
  RCC->AHB1ENR   = 0x00000078;
  
  /* Connect PDx pins to FSMC Alternate function */
  GPIOD->AFR[0]  = 0x00cc00cc;
  GPIOD->AFR[1]  = 0xcc0ccccc;
  /* Configure PDx pins in Alternate function mode */  
  GPIOD->MODER   = 0xaaaa0a0a;
  /* Configure PDx pins speed to 100 MHz */  
  GPIOD->OSPEEDR = 0xffff0f0f;
  /* Configure PDx pins Output type to push-pull */  
  GPIOD->OTYPER  = 0x00000000;
  /* No pull-up, pull-down for PDx pins */ 
  GPIOD->PUPDR   = 0x00000000;

  /* Connect PEx pins to FSMC Alternate function */
  GPIOE->AFR[0]  = 0xc00cc0cc;
  GPIOE->AFR[1]  = 0xcccccccc;
  /* Configure PEx pins in Alternate function mode */ 
  GPIOE->MODER   = 0xaaaa828a;
  /* Configure PEx pins speed to 100 MHz */ 
  GPIOE->OSPEEDR = 0xffffc3cf;
  /* Configure PEx pins Output type to push-pull */  
  GPIOE->OTYPER  = 0x00000000;
  /* No pull-up, pull-down for PEx pins */ 
  GPIOE->PUPDR   = 0x00000000;

  /* Connect PFx pins to FSMC Alternate function */
  GPIOF->AFR[0]  = 0x00cccccc;
  GPIOF->AFR[1]  = 0xcccc0000;
  /* Configure PFx pins in Alternate function mode */   
  GPIOF->MODER   = 0xaa000aaa;
  /* Configure PFx pins speed to 100 MHz */ 
  GPIOF->OSPEEDR = 0xff000fff;
  /* Configure PFx pins Output type to push-pull */  
  GPIOF->OTYPER  = 0x00000000;
  /* No pull-up, pull-down for PFx pins */ 
  GPIOF->PUPDR   = 0x00000000;

  /* Connect PGx pins to FSMC Alternate function */
  GPIOG->AFR[0]  = 0x00cccccc;
  GPIOG->AFR[1]  = 0x000000c0;
  /* Configure PGx pins in Alternate function mode */ 
  GPIOG->MODER   = 0x00080aaa;
  /* Configure PGx pins speed to 100 MHz */ 
  GPIOG->OSPEEDR = 0x000c0fff;
  /* Configure PGx pins Output type to push-pull */  
  GPIOG->OTYPER  = 0x00000000;
  /* No pull-up, pull-down for PGx pins */ 
  GPIOG->PUPDR   = 0x00000000;
  
/*-- FSMC Configuration ------------------------------------------------------*/
  /* Enable the FSMC interface clock */
  RCC->AHB3ENR         = 0x00000001;

  /* Configure and enable Bank1_SRAM2 */
  FSMC_Bank1->BTCR[2]  = 0x00001015;
  FSMC_Bank1->BTCR[3]  = 0x00010603;
  FSMC_Bank1E->BWTR[2] = 0x0fffffff;
/*
  Bank1_SRAM2 is configured as follow:

  p.FSMC_AddressSetupTime = 3;
  p.FSMC_AddressHoldTime = 0;
  p.FSMC_DataSetupTime = 6;
  p.FSMC_BusTurnAroundDuration = 1;
  p.FSMC_CLKDivision = 0;
  p.FSMC_DataLatency = 0;
  p.FSMC_AccessMode = FSMC_AccessMode_A;

  FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM2;
  FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
  FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_PSRAM;
  FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
  FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
  FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
  FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
  FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;
  FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &p;
  FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &p;
*/
  
}
#endif /* DATA_IN_ExtSRAM */


/**
  * @}
  */

/**
  * @}
  */
  
/**
  * @}
  */    
/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
